@@ -6,6 +6,13 @@ if test -z $AUTORECONF; then
         exit 1
 fi
 
+PKGCONFIG=`which pkg-config`
+if test -z $PKGCONFIG; then
+        echo "*** No pkg-config found, please install it ***"
+        # warn without exiting, since pkg-config is only needed
+        # by configure, not autogen.sh
+fi
+
 LIBTOOLIZE=`which libtoolize`
 if test -z $LIBTOOLIZE; then
         echo "*** No libtool found, please install it ***"
