@@ -15,5 +15,5 @@ OPTIONS="--disable-oss
         --disable-ffmpeg
         --disable-v4l2"
 # TODO: autotools should be doing this
-cd "`dirname $BASH_SOURCE`"/pjproject-2.0.1
-./configure $OPTIONS && make dep && make -j1 && echo "pjsip successfully compiled"
+cd "`dirname $BASH_SOURCE`"/pjproject-2.1.0
+CFLAGS=-g ./configure $OPTIONS && make dep && make -j1 && echo "pjsip successfully compiled"
