@@ -1,5 +1,5 @@
-/* $Id: resolver.c 3553 2011-05-05 06:14:19Z nanang $ */
-/*
+/* $Id: resolver.c 4333 2013-01-23 09:53:39Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjlib-util/resolver.h>
 #include <pjlib-util/errno.h>
@@ -37,7 +37,7 @@
 #define THIS_FILE	    "resolver.c"
 
 
-/* Check that maximum DNS nameservers is not too large.
+/* Check that maximum DNS nameservers is not too large. 
  * This has got todo with the datatype to index the nameserver in the query.
  */
 #if PJ_DNS_RESOLVER_MAX_NS > 256
@@ -72,7 +72,7 @@ static const char *state_names[3] =
 };
 
 
-/*
+/* 
  * Each nameserver entry.
  * A name server is identified by its socket address (IP and port).
  * Each NS will have a flag to indicate whether it's properly functioning.
@@ -84,7 +84,7 @@ struct nameserver
     enum ns_state   state;		/**< Nameserver state.		    */
     pj_time_val	    state_expiry;	/**< Time set next state.	    */
     pj_time_val	    rt_delay;		/**< Response time.		    */
-
+    
 
     /* For calculating rt_delay: */
     pj_uint16_t	    q_id;		/**< Query ID.			    */
@@ -92,7 +92,7 @@ struct nameserver
 };
 
 
-/* Child query list head
+/* Child query list head 
  * See comments on pj_dns_async_query below.
  */
 struct query_head
@@ -109,11 +109,11 @@ struct res_key
 };
 
 
-/*
+/* 
  * This represents each asynchronous query entry.
- * This entry will be put in two hash tables, the first one keyed on the DNS
+ * This entry will be put in two hash tables, the first one keyed on the DNS 
  * transaction ID to match response with the query, and the second one keyed
- * on "res_key" structure above to match a new request against outstanding
+ * on "res_key" structure above to match a new request against outstanding 
  * requests.
  *
  * An asynchronous entry may have child entries; child entries are subsequent
@@ -180,7 +180,8 @@ struct pj_dns_resolver
     unsigned char	 udp_rx_pkt[UDPSZ];/**< UDP receive buffer.	    */
     unsigned char	 udp_tx_pkt[UDPSZ];/**< UDP receive buffer.	    */
     pj_ssize_t		 udp_len;	/**< Length of received packet.	    */
-    pj_ioqueue_op_key_t	 udp_op_key;	/**< UDP read operation key.	    */
+    pj_ioqueue_op_key_t	 udp_op_rx_key;	/**< UDP read operation key.	    */
+    pj_ioqueue_op_key_t	 udp_op_tx_key;	/**< UDP write operation key.	    */
     pj_sockaddr_in	 udp_src_addr;	/**< Source address of packet	    */
     int			 udp_addr_len;	/**< Source address length.	    */
 
@@ -209,8 +210,8 @@ struct pj_dns_resolver
 
 
 /* Callback from ioqueue when packet is received */
-static void on_read_complete(pj_ioqueue_key_t *key,
-                             pj_ioqueue_op_key_t *op_key,
+static void on_read_complete(pj_ioqueue_key_t *key, 
+                             pj_ioqueue_op_key_t *op_key, 
                              pj_ssize_t bytes_read);
 
 /* Callback to be called when query has timed out */
@@ -223,6 +224,63 @@ static pj_status_t select_nameservers(pj_dns_resolver *resolver,
 				      unsigned servers[]);
 
 
+/* Close UDP socket */
+static void close_sock(pj_dns_resolver *resv)
+{
+    /* Close existing socket */
+    if (resv->udp_key != NULL) {
+	pj_ioqueue_unregister(resv->udp_key);
+	resv->udp_key = NULL;
+	resv->udp_sock = PJ_INVALID_SOCKET;
+    } else if (resv->udp_sock != PJ_INVALID_SOCKET) {
+	pj_sock_close(resv->udp_sock);
+	resv->udp_sock = PJ_INVALID_SOCKET;
+    }
+}
+
+
+/* Initialize UDP socket */
+static pj_status_t init_sock(pj_dns_resolver *resv)
+{
+    pj_ioqueue_callback socket_cb;
+    pj_status_t status;
+
+    /* Create the UDP socket */
+    status = pj_sock_socket(pj_AF_INET(), pj_SOCK_DGRAM(), 0, &resv->udp_sock);
+    if (status != PJ_SUCCESS)
+	return status;
+
+    /* Bind to any address/port */
+    status = pj_sock_bind_in(resv->udp_sock, 0, 0);
+    if (status != PJ_SUCCESS)
+	return status;
+
+    /* Register to ioqueue */
+    pj_bzero(&socket_cb, sizeof(socket_cb));
+    socket_cb.on_read_complete = &on_read_complete;
+    status = pj_ioqueue_register_sock(resv->pool, resv->ioqueue,
+				      resv->udp_sock, resv, &socket_cb,
+				      &resv->udp_key);
+    if (status != PJ_SUCCESS)
+	return status;
+
+    pj_ioqueue_op_key_init(&resv->udp_op_rx_key, sizeof(resv->udp_op_rx_key));
+    pj_ioqueue_op_key_init(&resv->udp_op_tx_key, sizeof(resv->udp_op_tx_key));
+
+    /* Start asynchronous read to the UDP socket */
+    resv->udp_len = sizeof(resv->udp_rx_pkt);
+    resv->udp_addr_len = sizeof(resv->udp_src_addr);
+    status = pj_ioqueue_recvfrom(resv->udp_key, &resv->udp_op_rx_key,
+				 resv->udp_rx_pkt, &resv->udp_len,
+				 PJ_IOQUEUE_ALWAYS_ASYNC,
+				 &resv->udp_src_addr, &resv->udp_addr_len);
+    if (status != PJ_EPENDING)
+	return status;
+
+    return PJ_SUCCESS;
+}
+
+
 /* Initialize DNS settings with default values */
 PJ_DEF(void) pj_dns_settings_default(pj_dns_settings *s)
 {
@@ -247,7 +305,6 @@ PJ_DEF(pj_status_t) pj_dns_resolver_create( pj_pool_factory *pf,
 {
     pj_pool_t *pool;
     pj_dns_resolver *resv;
-    pj_ioqueue_callback socket_cb;
     pj_status_t status;
 
     /* Sanity check */
@@ -266,7 +323,7 @@ PJ_DEF(pj_status_t) pj_dns_resolver_create( pj_pool_factory *pf,
     resv->pool = pool;
     resv->udp_sock = PJ_INVALID_SOCKET;
     pj_strdup2_with_null(pool, &resv->name, name);
-
+    
     /* Create the mutex */
     status = pj_mutex_create_recursive(pool, name, &resv->mutex);
     if (status != PJ_SUCCESS)
@@ -302,37 +359,11 @@ PJ_DEF(pj_status_t) pj_dns_resolver_create( pj_pool_factory *pf,
     resv->hquerybyres = pj_hash_create(pool, Q_HASH_TABLE_SIZE);
     pj_list_init(&resv->query_free_nodes);
 
-    /* Create the UDP socket */
-    status = pj_sock_socket(pj_AF_INET(), pj_SOCK_DGRAM(), 0, &resv->udp_sock);
+    /* Initialize the UDP socket */
+    status = init_sock(resv);
     if (status != PJ_SUCCESS)
 	goto on_error;
 
-    /* Bind to any address/port */
-    status = pj_sock_bind_in(resv->udp_sock, 0, 0);
-    if (status != PJ_SUCCESS)
-	goto on_error;
-
-    /* Register to ioqueue */
-    pj_bzero(&socket_cb, sizeof(socket_cb));
-    socket_cb.on_read_complete = &on_read_complete;
-    status = pj_ioqueue_register_sock(pool, resv->ioqueue, resv->udp_sock,
-				      resv, &socket_cb, &resv->udp_key);
-    if (status != PJ_SUCCESS)
-	goto on_error;
-
-    pj_ioqueue_op_key_init(&resv->udp_op_key, sizeof(resv->udp_op_key));
-
-    /* Start asynchronous read to the UDP socket */
-    resv->udp_len = sizeof(resv->udp_rx_pkt);
-    resv->udp_addr_len = sizeof(resv->udp_src_addr);
-    status = pj_ioqueue_recvfrom(resv->udp_key, &resv->udp_op_key,
-				 resv->udp_rx_pkt, &resv->udp_len,
-				 PJ_IOQUEUE_ALWAYS_ASYNC,
-				 &resv->udp_src_addr, &resv->udp_addr_len);
-    if (status != PJ_EPENDING)
-	goto on_error;
-
-
     /* Looks like everything is okay */
     *p_resolver = resv;
     return PJ_SUCCESS;
@@ -359,7 +390,7 @@ PJ_DEF(pj_status_t) pj_dns_resolver_destroy( pj_dns_resolver *resolver,
 	it = pj_hash_first(resolver->hquerybyid, &it_buf);
 	while (it) {
 	    pj_dns_async_query *q = (pj_dns_async_query *)
-				    pj_hash_this(resolver->hquerybyid, it);
+	    			    pj_hash_this(resolver->hquerybyid, it);
 	    pj_dns_async_query *cq;
 	    if (q->cb)
 		(*q->cb)(q->user_data, PJ_ECANCELLED, NULL);
@@ -380,7 +411,7 @@ PJ_DEF(pj_status_t) pj_dns_resolver_destroy( pj_dns_resolver *resolver,
 	struct cached_res *cache;
 
 	cache = (struct cached_res*) pj_hash_this(resolver->hrescache, it);
-	pj_hash_set(NULL, resolver->hrescache, &cache->key,
+	pj_hash_set(NULL, resolver->hrescache, &cache->key, 
 		    sizeof(cache->key), 0, NULL);
 	pj_pool_release(cache->pool);
 
@@ -392,14 +423,7 @@ PJ_DEF(pj_status_t) pj_dns_resolver_destroy( pj_dns_resolver *resolver,
 	resolver->timer = NULL;
     }
 
-    if (resolver->udp_key != NULL) {
-	pj_ioqueue_unregister(resolver->udp_key);
-	resolver->udp_key = NULL;
-	resolver->udp_sock = PJ_INVALID_SOCKET;
-    } else if (resolver->udp_sock != PJ_INVALID_SOCKET) {
-	pj_sock_close(resolver->udp_sock);
-	resolver->udp_sock = PJ_INVALID_SOCKET;
-    }
+    close_sock(resolver);
 
     if (resolver->own_ioqueue && resolver->ioqueue) {
 	pj_ioqueue_destroy(resolver->ioqueue);
@@ -422,7 +446,7 @@ PJ_DEF(pj_status_t) pj_dns_resolver_destroy( pj_dns_resolver *resolver,
 
 
 /*
- * Configure name servers for the DNS resolver.
+ * Configure name servers for the DNS resolver. 
  */
 PJ_DEF(pj_status_t) pj_dns_resolver_set_ns( pj_dns_resolver *resolver,
 					    unsigned count,
@@ -449,7 +473,7 @@ PJ_DEF(pj_status_t) pj_dns_resolver_set_ns( pj_dns_resolver *resolver,
     for (i=0; i<count; ++i) {
 	struct nameserver *ns = &resolver->ns[i];
 
-	status = pj_sockaddr_in_init(&ns->addr, &servers[i],
+	status = pj_sockaddr_in_init(&ns->addr, &servers[i], 
 				     (pj_uint16_t)(ports ? ports[i] : PORT));
 	if (status != PJ_SUCCESS) {
 	    pj_mutex_unlock(resolver->mutex);
@@ -460,7 +484,7 @@ PJ_DEF(pj_status_t) pj_dns_resolver_set_ns( pj_dns_resolver *resolver,
 	ns->state_expiry = now;
 	ns->rt_delay.sec = 10;
     }
-
+    
     resolver->ns_count = count;
 
     pj_mutex_unlock(resolver->mutex);
@@ -500,7 +524,7 @@ PJ_DEF(pj_status_t) pj_dns_resolver_get_settings( pj_dns_resolver *resolver,
 
 
 /*
- * Poll for events from the resolver.
+ * Poll for events from the resolver. 
  */
 PJ_DEF(void) pj_dns_resolver_handle_events(pj_dns_resolver *resolver,
 					   const pj_time_val *timeout)
@@ -515,7 +539,7 @@ PJ_DEF(void) pj_dns_resolver_handle_events(pj_dns_resolver *resolver,
 }
 
 
-/* Get one query node from the free node, if any, or allocate
+/* Get one query node from the free node, if any, or allocate 
  * a new one.
  */
 static pj_dns_async_query *alloc_qnode(pj_dns_resolver *resolver,
@@ -561,15 +585,6 @@ static pj_status_t transmit_query(pj_dns_resolver *resolver,
     pj_time_val delay;
     pj_status_t status;
 
-    /* Create DNS query packet */
-    pkt_size = sizeof(resolver->udp_tx_pkt);
-    name = pj_str(q->key.name);
-    status = pj_dns_make_query(resolver->udp_tx_pkt, &pkt_size,
-			       q->id, q->key.qtype, &name);
-    if (status != PJ_SUCCESS) {
-	return status;
-    }
-
     /* Select which nameserver(s) to send requests to. */
     server_cnt = PJ_ARRAY_SIZE(servers);
     status = select_nameservers(resolver, &server_cnt, servers);
@@ -595,6 +610,28 @@ static pj_status_t transmit_query(pj_dns_resolver *resolver,
 	return status;
     }
 
+    /* Check if the socket is available for sending */
+    if (pj_ioqueue_is_pending(resolver->udp_key, &resolver->udp_op_tx_key)) {
+	++q->transmit_cnt;
+	PJ_LOG(4,(resolver->name.ptr,
+		  "Socket busy in transmitting DNS %s query for %s%s",
+		  pj_dns_get_type_name(q->key.qtype),
+		  q->key.name,
+		  (q->transmit_cnt < resolver->settings.qretr_count?
+		   ", will try again later":"")));
+	return PJ_SUCCESS;
+    }
+
+    /* Create DNS query packet */
+    pkt_size = sizeof(resolver->udp_tx_pkt);
+    name = pj_str(q->key.name);
+    status = pj_dns_make_query(resolver->udp_tx_pkt, &pkt_size,
+			       q->id, q->key.qtype, &name);
+    if (status != PJ_SUCCESS) {
+	pj_timer_heap_cancel(resolver->timer, &q->timer_entry);
+	return status;
+    }
+
     /* Get current time. */
     pj_gettimeofday(&now);
 
@@ -603,16 +640,19 @@ static pj_status_t transmit_query(pj_dns_resolver *resolver,
 	pj_ssize_t sent  = (pj_ssize_t) pkt_size;
 	struct nameserver *ns = &resolver->ns[servers[i]];
 
-	pj_sock_sendto(resolver->udp_sock, resolver->udp_tx_pkt, &sent, 0,
-		       &resolver->ns[servers[i]].addr, sizeof(pj_sockaddr_in));
+	status = pj_ioqueue_sendto(resolver->udp_key,
+				   &resolver->udp_op_tx_key,
+				   resolver->udp_tx_pkt, &sent, 0,
+				   &resolver->ns[servers[i]].addr,
+				   sizeof(pj_sockaddr_in));
 
-	PJ_LOG(4,(resolver->name.ptr,
+	PJ_PERROR(4,(resolver->name.ptr, status,
 		  "%s %d bytes to NS %d (%s:%d): DNS %s query for %s",
 		  (q->transmit_cnt==0? "Transmitting":"Re-transmitting"),
-		  (int)sent, servers[i],
-		  pj_inet_ntoa(ns->addr.sin_addr),
+		  (int)pkt_size, servers[i],
+		  pj_inet_ntoa(ns->addr.sin_addr), 
 		  (int)pj_ntohs(ns->addr.sin_port),
-		  pj_dns_get_type_name(q->key.qtype),
+		  pj_dns_get_type_name(q->key.qtype), 
 		  q->key.name));
 
 	if (ns->q_id == 0) {
@@ -715,8 +755,8 @@ PJ_DEF(pj_status_t) pj_dns_resolver_start_query( pj_dns_resolver *resolver,
      * and the cached entry has not expired.
      */
     hval = 0;
-    cache = (struct cached_res *) pj_hash_get(resolver->hrescache, &key,
-					      sizeof(key), &hval);
+    cache = (struct cached_res *) pj_hash_get(resolver->hrescache, &key, 
+    					      sizeof(key), &hval);
     if (cache) {
 	/* We've found a cached entry. */
 
@@ -724,7 +764,7 @@ PJ_DEF(pj_status_t) pj_dns_resolver_start_query( pj_dns_resolver *resolver,
 	if (PJ_TIME_VAL_GT(cache->expiry_time, now)) {
 
 	    /* Log */
-	    PJ_LOG(5,(resolver->name.ptr,
+	    PJ_LOG(5,(resolver->name.ptr, 
 		      "Picked up DNS %s record for %.*s from cache, ttl=%d",
 		      pj_dns_get_type_name(type),
 		      (int)name->slen, name->ptr,
@@ -761,8 +801,8 @@ PJ_DEF(pj_status_t) pj_dns_resolver_start_query( pj_dns_resolver *resolver,
     }
 
     /* Next, check if we have pending query on the same resource */
-    q = (pj_dns_async_query *) pj_hash_get(resolver->hquerybyres, &key,
-					   sizeof(key), NULL);
+    q = (pj_dns_async_query *) pj_hash_get(resolver->hquerybyres, &key, 
+    					   sizeof(key), NULL);
     if (q) {
 	/* Yes, there's another pending query to the same key.
 	 * Just create a new child query and add this query to
@@ -778,7 +818,7 @@ PJ_DEF(pj_status_t) pj_dns_resolver_start_query( pj_dns_resolver *resolver,
 	 */
 	status = PJ_SUCCESS;
 	goto on_return;
-    }
+    } 
 
     /* There's no pending query to the same key, initiate a new one. */
     q = alloc_qnode(resolver, options, user_data, cb);
@@ -798,7 +838,7 @@ PJ_DEF(pj_status_t) pj_dns_resolver_start_query( pj_dns_resolver *resolver,
     }
 
     /* Add query entry to the hash tables */
-    pj_hash_set_np(resolver->hquerybyid, &q->id, sizeof(q->id),
+    pj_hash_set_np(resolver->hquerybyid, &q->id, sizeof(q->id), 
 		   0, q->hbufid, q);
     pj_hash_set_np(resolver->hquerybyres, &q->key, sizeof(q->key),
 		   0, q->hbufkey, q);
@@ -835,8 +875,8 @@ PJ_DEF(pj_status_t) pj_dns_resolver_cancel_query(pj_dns_async_query *query,
 }
 
 
-/*
- * DNS response containing A packet.
+/* 
+ * DNS response containing A packet. 
  */
 PJ_DEF(pj_status_t) pj_dns_parse_a_response(const pj_dns_parsed_packet *pkt,
 					    pj_dns_a_record *rec)
@@ -980,11 +1020,11 @@ static void set_nameserver_state(pj_dns_resolver *resolver,
  * sent the request to is as follows:
  *  - select the first nameserver that is known to be good for the
  *    last PJ_DNS_RESOLVER_GOOD_NS_TTL interval.
- *  - for all NSes, if last_known_good >= PJ_DNS_RESOLVER_GOOD_NS_TTL,
+ *  - for all NSes, if last_known_good >= PJ_DNS_RESOLVER_GOOD_NS_TTL, 
  *    include the NS to re-check again that the server is still good,
  *    unless the NS is known to be bad in the last PJ_DNS_RESOLVER_BAD_NS_TTL
  *    interval.
- *  - for all NSes, if last_known_bad >= PJ_DNS_RESOLVER_BAD_NS_TTL,
+ *  - for all NSes, if last_known_bad >= PJ_DNS_RESOLVER_BAD_NS_TTL, 
  *    also include the NS to re-check again that the server is still bad.
  */
 static pj_status_t select_nameservers(pj_dns_resolver *resolver,
@@ -1071,7 +1111,7 @@ static void report_nameserver_status(pj_dns_resolver *resolver,
 
     if (!pkt || rcode == PJ_DNS_RCODE_SERVFAIL ||
 	        rcode == PJ_DNS_RCODE_REFUSED ||
-	        rcode == PJ_DNS_RCODE_NOTAUTH)
+	        rcode == PJ_DNS_RCODE_NOTAUTH) 
     {
 	is_good = PJ_FALSE;
     } else {
@@ -1097,7 +1137,7 @@ static void report_nameserver_status(pj_dns_resolver *resolver,
 		ns->rt_delay = rt;
 		ns->q_id = 0;
 	    }
-	    set_nameserver_state(resolver, i,
+	    set_nameserver_state(resolver, i, 
 				 (is_good ? STATE_ACTIVE : STATE_BAD), &now);
 	    break;
 	}
@@ -1117,7 +1157,7 @@ static void update_res_cache(pj_dns_resolver *resolver,
 
     /* If status is unsuccessful, clear the same entry from the cache */
     if (status != PJ_SUCCESS) {
-	cache = (struct cached_res *) pj_hash_get(resolver->hrescache, key,
+	cache = (struct cached_res *) pj_hash_get(resolver->hrescache, key, 
 						  sizeof(*key), &hval);
 	if (cache)
 	    free_entry(resolver, cache);
@@ -1129,7 +1169,7 @@ static void update_res_cache(pj_dns_resolver *resolver,
     if (set_expiry) {
 	if (pkt->hdr.anscount == 0 || status != PJ_SUCCESS) {
 	    /* If we don't have answers for the name, then give a different
-	     * ttl value (note: PJ_DNS_RESOLVER_INVALID_TTL may be zero,
+	     * ttl value (note: PJ_DNS_RESOLVER_INVALID_TTL may be zero, 
 	     * which means that invalid names won't be kept in the cache)
 	     */
 	    ttl = PJ_DNS_RESOLVER_INVALID_TTL;
@@ -1153,7 +1193,7 @@ static void update_res_cache(pj_dns_resolver *resolver,
 
     /* If TTL is zero, clear the same entry in the hash table */
     if (ttl == 0) {
-	cache = (struct cached_res *) pj_hash_get(resolver->hrescache, key,
+	cache = (struct cached_res *) pj_hash_get(resolver->hrescache, key, 
 						  sizeof(*key), &hval);
 	if (cache)
 	    free_entry(resolver, cache);
@@ -1162,8 +1202,8 @@ static void update_res_cache(pj_dns_resolver *resolver,
     }
 
     /* Get a cache response entry */
-    cache = (struct cached_res *) pj_hash_get(resolver->hrescache, key,
-					      sizeof(*key), &hval);
+    cache = (struct cached_res *) pj_hash_get(resolver->hrescache, key, 
+    					      sizeof(*key), &hval);
     if (cache == NULL) {
 	cache = alloc_entry(resolver);
     }
@@ -1175,7 +1215,7 @@ static void update_res_cache(pj_dns_resolver *resolver,
      * the name being requested.
      */
     cache->pkt = NULL;
-    pj_dns_packet_dup(cache->pool, pkt,
+    pj_dns_packet_dup(cache->pool, pkt, 
 		      PJ_DNS_NO_NS | PJ_DNS_NO_AR,
 		      &cache->pkt);
 
@@ -1248,6 +1288,9 @@ static void on_timeout( pj_timer_heap_t *timer_heap,
     pj_hash_set(NULL, resolver->hquerybyid, &q->id, sizeof(q->id), 0, NULL);
     pj_hash_set(NULL, resolver->hquerybyres, &q->key, sizeof(q->key), 0, NULL);
 
+    /* Workaround for deadlock problem in #1565 (similar to #1108) */
+    pj_mutex_unlock(resolver->mutex);
+
     /* Call application callback, if any. */
     if (q->cb)
 	(*q->cb)(q->user_data, PJ_ETIMEDOUT, NULL);
@@ -1260,6 +1303,9 @@ static void on_timeout( pj_timer_heap_t *timer_heap,
 	cq = cq->next;
     }
 
+    /* Workaround for deadlock problem in #1565 (similar to #1108) */
+    pj_mutex_lock(resolver->mutex);
+
     /* Clear data */
     q->timer_entry.id = 0;
     q->user_data = NULL;
@@ -1280,8 +1326,8 @@ static void on_timeout( pj_timer_heap_t *timer_heap,
 
 
 /* Callback from ioqueue when packet is received */
-static void on_read_complete(pj_ioqueue_key_t *key,
-                             pj_ioqueue_op_key_t *op_key,
+static void on_read_complete(pj_ioqueue_key_t *key, 
+                             pj_ioqueue_op_key_t *op_key, 
                              pj_ssize_t bytes_read)
 {
     pj_dns_resolver *resolver;
@@ -1302,8 +1348,8 @@ static void on_read_complete(pj_ioqueue_key_t *key,
 
 	status = -bytes_read;
 	pj_strerror(status, errmsg, sizeof(errmsg));
-	PJ_LOG(4,(resolver->name.ptr,
-		  "DNS resolver read error from %s:%d: %s",
+	PJ_LOG(4,(resolver->name.ptr, 
+		  "DNS resolver read error from %s:%d: %s", 
 		  pj_inet_ntoa(resolver->udp_src_addr.sin_addr),
 		  pj_ntohs(resolver->udp_src_addr.sin_port),
 		  errmsg));
@@ -1311,9 +1357,9 @@ static void on_read_complete(pj_ioqueue_key_t *key,
 	goto read_next_packet;
     }
 
-    PJ_LOG(5,(resolver->name.ptr,
+    PJ_LOG(5,(resolver->name.ptr, 
 	      "Received %d bytes DNS response from %s:%d",
-	      (int)bytes_read,
+	      (int)bytes_read, 
 	      pj_inet_ntoa(resolver->udp_src_addr.sin_addr),
 	      pj_ntohs(resolver->udp_src_addr.sin_port)));
 
@@ -1323,14 +1369,14 @@ static void on_read_complete(pj_ioqueue_key_t *key,
 	goto read_next_packet;
 
     /* Create temporary pool from a fixed buffer */
-    pool = pj_pool_create_on_buf("restmp", resolver->tmp_pool,
+    pool = pj_pool_create_on_buf("restmp", resolver->tmp_pool, 
 				 sizeof(resolver->tmp_pool));
 
     /* Parse DNS response */
     status = -1;
     dns_pkt = NULL;
     PJ_TRY {
-	status = pj_dns_parse_packet(pool, resolver->udp_rx_pkt,
+	status = pj_dns_parse_packet(pool, resolver->udp_rx_pkt, 
 				     (unsigned)bytes_read, &dns_pkt);
     }
     PJ_CATCH_ANY {
@@ -1346,22 +1392,22 @@ static void on_read_complete(pj_ioqueue_key_t *key,
 	char errmsg[PJ_ERR_MSG_SIZE];
 
 	pj_strerror(status, errmsg, sizeof(errmsg));
-	PJ_LOG(3,(resolver->name.ptr,
-		  "Error parsing DNS response from %s:%d: %s",
-		  pj_inet_ntoa(resolver->udp_src_addr.sin_addr),
-		  pj_ntohs(resolver->udp_src_addr.sin_port),
+	PJ_LOG(3,(resolver->name.ptr, 
+		  "Error parsing DNS response from %s:%d: %s", 
+		  pj_inet_ntoa(resolver->udp_src_addr.sin_addr), 
+		  pj_ntohs(resolver->udp_src_addr.sin_port), 
 		  errmsg));
 	goto read_next_packet;
     }
 
     /* Find the query based on the transaction ID */
-    q = (pj_dns_async_query*)
+    q = (pj_dns_async_query*) 
         pj_hash_get(resolver->hquerybyid, &dns_pkt->hdr.id,
 		    sizeof(dns_pkt->hdr.id), NULL);
     if (!q) {
-	PJ_LOG(5,(resolver->name.ptr,
+	PJ_LOG(5,(resolver->name.ptr, 
 		  "DNS response from %s:%d id=%d discarded",
-		  pj_inet_ntoa(resolver->udp_src_addr.sin_addr),
+		  pj_inet_ntoa(resolver->udp_src_addr.sin_addr), 
 		  pj_ntohs(resolver->udp_src_addr.sin_port),
 		  (unsigned)dns_pkt->hdr.id));
 	goto read_next_packet;
@@ -1382,7 +1428,7 @@ static void on_read_complete(pj_ioqueue_key_t *key,
     /* Workaround for deadlock problem in #1108 */
     pj_mutex_unlock(resolver->mutex);
 
-    /* Notify applications first, to allow application to modify the
+    /* Notify applications first, to allow application to modify the 
      * record before it is saved to the hash table.
      */
     if (q->cb)
@@ -1405,7 +1451,7 @@ static void on_read_complete(pj_ioqueue_key_t *key,
 
     /* Save/update response cache. */
     update_res_cache(resolver, &q->key, status, PJ_TRUE, dns_pkt);
-
+    
     /* Recycle query objects, starting with the child queries */
     if (!pj_list_empty(&q->child_head)) {
 	pj_dns_async_query *child_q;
@@ -1427,15 +1473,15 @@ read_next_packet:
     }
     bytes_read = sizeof(resolver->udp_rx_pkt);
     resolver->udp_addr_len = sizeof(resolver->udp_src_addr);
-    status = pj_ioqueue_recvfrom(resolver->udp_key, op_key,
+    status = pj_ioqueue_recvfrom(resolver->udp_key, op_key, 
 				 resolver->udp_rx_pkt,
 				 &bytes_read, PJ_IOQUEUE_ALWAYS_ASYNC,
-				 &resolver->udp_src_addr,
+				 &resolver->udp_src_addr, 
 				 &resolver->udp_addr_len);
     if (status != PJ_EPENDING) {
 	char errmsg[PJ_ERR_MSG_SIZE];
 
-	pj_strerror(status, errmsg, sizeof(errmsg));
+	pj_strerror(status, errmsg, sizeof(errmsg));	
 	PJ_LOG(4,(resolver->name.ptr, "DNS resolver ioqueue read error: %s",
 		  errmsg));
 
@@ -1474,14 +1520,14 @@ PJ_DEF(pj_status_t) pj_dns_resolver_add_entry( pj_dns_resolver *resolver,
     pj_bzero(&key, sizeof(struct res_key));
     if (pkt->hdr.anscount) {
 	/* Make sure name is not too long. */
-	PJ_ASSERT_RETURN(pkt->ans[0].name.slen < PJ_MAX_HOSTNAME,
+	PJ_ASSERT_RETURN(pkt->ans[0].name.slen < PJ_MAX_HOSTNAME, 
 			 PJ_ENAMETOOLONG);
 
 	init_res_key(&key, pkt->ans[0].type, &pkt->ans[0].name);
 
     } else {
 	/* Make sure name is not too long. */
-	PJ_ASSERT_RETURN(pkt->q[0].name.slen < PJ_MAX_HOSTNAME,
+	PJ_ASSERT_RETURN(pkt->q[0].name.slen < PJ_MAX_HOSTNAME, 
 			 PJ_ENAMETOOLONG);
 
 	init_res_key(&key, pkt->q[0].type, &pkt->q[0].name);
@@ -1551,9 +1597,9 @@ PJ_DEF(void) pj_dns_resolver_dump(pj_dns_resolver *resolver,
 	while (it) {
 	    struct cached_res *cache;
 	    cache = (struct cached_res*)pj_hash_this(resolver->hrescache, it);
-	    PJ_LOG(3,(resolver->name.ptr,
+	    PJ_LOG(3,(resolver->name.ptr, 
 		      "   Type %s: %s",
-		      pj_dns_get_type_name(cache->key.qtype),
+		      pj_dns_get_type_name(cache->key.qtype), 
 		      cache->key.name));
 	    it = pj_hash_next(resolver->hrescache, it);
 	}
@@ -1567,9 +1613,9 @@ PJ_DEF(void) pj_dns_resolver_dump(pj_dns_resolver *resolver,
 	while (it) {
 	    struct pj_dns_async_query *q;
 	    q = (pj_dns_async_query*) pj_hash_this(resolver->hquerybyid, it);
-	    PJ_LOG(3,(resolver->name.ptr,
+	    PJ_LOG(3,(resolver->name.ptr, 
 		      "   Type %s: %s",
-		      pj_dns_get_type_name(q->key.qtype),
+		      pj_dns_get_type_name(q->key.qtype), 
 		      q->key.name));
 	    it = pj_hash_next(resolver->hquerybyid, it);
 	}
@@ -1585,3 +1631,4 @@ PJ_DEF(void) pj_dns_resolver_dump(pj_dns_resolver *resolver,
     pj_mutex_unlock(resolver->mutex);
 #endif
 }
+
