@@ -1,6 +1,6 @@
-/* $Id: ioqueue.h 3553 2011-05-05 06:14:19Z nanang $
+/* $Id: ioqueue.h 4359 2013-02-21 11:18:36Z bennylp $
  */
-/*
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #ifndef __PJ_IOQUEUE_H__
 #define __PJ_IOQUEUE_H__
@@ -35,7 +35,7 @@ PJ_BEGIN_DECL
  * @brief Input/Output
  * @ingroup PJ_OS
  *
- * This section contains API building blocks to perform network I/O and
+ * This section contains API building blocks to perform network I/O and 
  * communications. If provides:
  *  - @ref PJ_SOCK
  *\n
@@ -69,13 +69,13 @@ PJ_BEGIN_DECL
  * asynchronous operation and to be notified later when the operation has
  * completed.
  *
- * The I/O Queue can work on both socket and file descriptors. For
+ * The I/O Queue can work on both socket and file descriptors. For 
  * asynchronous file operations however, one must make sure that the correct
  * file I/O back-end is used, because not all file I/O back-end can be
  * used with the ioqueue. Please see \ref PJ_FILE_IO for more details.
  *
  * The framework works natively in platforms where asynchronous operation API
- * exists, such as in Windows NT with IoCompletionPort/IOCP. In other
+ * exists, such as in Windows NT with IoCompletionPort/IOCP. In other 
  * platforms, the I/O queue abstracts the operating system's event poll API
  * to provide semantics similar to IoCompletionPort with minimal penalties
  * (i.e. per ioqueue and per handle mutex protection).
@@ -88,14 +88,14 @@ PJ_BEGIN_DECL
  *    platform.
  *
  * Currently, the I/O Queue is implemented using:
- *  - <tt><b>select()</b></tt>, as the common denominator, but the least
- *    efficient. Also the number of descriptor is limited to
+ *  - <tt><b>select()</b></tt>, as the common denominator, but the least 
+ *    efficient. Also the number of descriptor is limited to 
  *    \c PJ_IOQUEUE_MAX_HANDLES (which by default is 64).
- *  - <tt><b>/dev/epoll</b></tt> on Linux (user mode and kernel mode),
+ *  - <tt><b>/dev/epoll</b></tt> on Linux (user mode and kernel mode), 
  *    a much faster replacement for select() on Linux (and more importantly
  *    doesn't have limitation on number of descriptors).
- *  - <b>I/O Completion ports</b> on Windows NT/2000/XP, which is the most
- *    efficient way to dispatch events in Windows NT based OSes, and most
+ *  - <b>I/O Completion ports</b> on Windows NT/2000/XP, which is the most 
+ *    efficient way to dispatch events in Windows NT based OSes, and most 
  *    importantly, it doesn't have the limit on how many handles to monitor.
  *    And it works with files (not only sockets) as well.
  *
@@ -110,9 +110,9 @@ PJ_BEGIN_DECL
  * These parallel executions are completely safe when the events happen for
  * two different handles.
  *
- * However, with multithreading, care must be taken when multiple events
- * happen on the same handle, or when event is happening on a handle (and
- * the callback is being executed) and application is performing
+ * However, with multithreading, care must be taken when multiple events 
+ * happen on the same handle, or when event is happening on a handle (and 
+ * the callback is being executed) and application is performing 
  * unregistration to the handle at the same time.
  *
  * The treatments of above scenario differ according to the concurrency
@@ -121,17 +121,17 @@ PJ_BEGIN_DECL
  * \subsection pj_ioq_concur_set Concurrency Settings for Handles
  *
  * Concurrency can be set on per handle (key) basis, by using
- * #pj_ioqueue_set_concurrency() function. The default key concurrency value
- * for the handle is inherited from the key concurrency setting of the ioqueue,
+ * #pj_ioqueue_set_concurrency() function. The default key concurrency value 
+ * for the handle is inherited from the key concurrency setting of the ioqueue, 
  * and the key concurrency setting for the ioqueue can be changed by using
- * #pj_ioqueue_set_default_concurrency(). The default key concurrency setting
+ * #pj_ioqueue_set_default_concurrency(). The default key concurrency setting 
  * for ioqueue itself is controlled by compile time setting
  * PJ_IOQUEUE_DEFAULT_ALLOW_CONCURRENCY.
  *
  * Note that this key concurrency setting only controls whether multiple
- * threads are allowed to operate <b>on the same key</b> at the same time.
- * The ioqueue itself always allows multiple threads to enter the ioqeuue at
- * the same time, and also simultaneous callback calls to <b>differrent
+ * threads are allowed to operate <b>on the same key</b> at the same time. 
+ * The ioqueue itself always allows multiple threads to enter the ioqeuue at 
+ * the same time, and also simultaneous callback calls to <b>differrent 
  * keys</b> is always allowed regardless to the key concurrency setting.
  *
  * \subsection pj_ioq_parallel Parallel Callback Executions for the Same Handle
@@ -158,7 +158,7 @@ PJ_BEGIN_DECL
  * promote good performance and scalability for application.
  *
  * However this setting has a major drawback with regard to synchronization,
- * and application MUST carefully follow the following guidelines to ensure
+ * and application MUST carefully follow the following guidelines to ensure 
  * that parallel access to the key does not cause problems:
  *
  *  - Always note that callback may be called simultaneously for the same
@@ -175,7 +175,7 @@ PJ_BEGIN_DECL
  *
  * \subsection pj_ioq_disallow_concur Concurrency is Disabled
  *
- * Alternatively, application may disable key concurrency to make
+ * Alternatively, application may disable key concurrency to make 
  * synchronization easier. As noted above, there are three ways to control
  * key concurrency setting:
  *  - by controlling on per handle/key basis, with #pj_ioqueue_set_concurrency().
@@ -207,7 +207,7 @@ PJ_BEGIN_DECL
  * the pointer supplied when the asynchronous function is called.
  */
 typedef struct pj_ioqueue_op_key_t
-{
+{ 
     void *internal__[32];           /**< Internal I/O Queue data.   */
     void *activesock_data;	    /**< Active socket data.	    */
     void *user_data;                /**< Application data.          */
@@ -225,13 +225,13 @@ typedef struct pj_ioqueue_callback
      *
      * @param key	    The key.
      * @param op_key        Operation key.
-     * @param bytes_read    >= 0 to indicate the amount of data read,
+     * @param bytes_read    >= 0 to indicate the amount of data read, 
      *                      otherwise negative value containing the error
      *                      code. To obtain the pj_status_t error code, use
      *                      (pj_status_t code = -bytes_read).
      */
-    void (*on_read_complete)(pj_ioqueue_key_t *key,
-                             pj_ioqueue_op_key_t *op_key,
+    void (*on_read_complete)(pj_ioqueue_key_t *key, 
+                             pj_ioqueue_op_key_t *op_key, 
                              pj_ssize_t bytes_read);
 
     /**
@@ -240,13 +240,13 @@ typedef struct pj_ioqueue_callback
      *
      * @param key	    The key.
      * @param op_key        Operation key.
-     * @param bytes_sent    >= 0 to indicate the amount of data written,
+     * @param bytes_sent    >= 0 to indicate the amount of data written, 
      *                      otherwise negative value containing the error
      *                      code. To obtain the pj_status_t error code, use
      *                      (pj_status_t code = -bytes_sent).
      */
-    void (*on_write_complete)(pj_ioqueue_key_t *key,
-                              pj_ioqueue_op_key_t *op_key,
+    void (*on_write_complete)(pj_ioqueue_key_t *key, 
+                              pj_ioqueue_op_key_t *op_key, 
                               pj_ssize_t bytes_sent);
 
     /**
@@ -257,9 +257,9 @@ typedef struct pj_ioqueue_callback
      * @param sock          Newly connected socket.
      * @param status	    Zero if the operation completes successfully.
      */
-    void (*on_accept_complete)(pj_ioqueue_key_t *key,
-                               pj_ioqueue_op_key_t *op_key,
-                               pj_sock_t sock,
+    void (*on_accept_complete)(pj_ioqueue_key_t *key, 
+                               pj_ioqueue_op_key_t *op_key, 
+                               pj_sock_t sock, 
                                pj_status_t status);
 
     /**
@@ -268,7 +268,7 @@ typedef struct pj_ioqueue_callback
      * @param key	    The key.
      * @param status	    PJ_SUCCESS if the operation completes successfully.
      */
-    void (*on_connect_complete)(pj_ioqueue_key_t *key,
+    void (*on_connect_complete)(pj_ioqueue_key_t *key, 
                                 pj_status_t status);
 } pj_ioqueue_callback;
 
@@ -320,14 +320,14 @@ PJ_DECL(const char*) pj_ioqueue_name(void);
 /**
  * Create a new I/O Queue framework.
  *
- * @param pool		The pool to allocate the I/O queue structure.
- * @param max_fd	The maximum number of handles to be supported, which
+ * @param pool		The pool to allocate the I/O queue structure. 
+ * @param max_fd	The maximum number of handles to be supported, which 
  *			should not exceed PJ_IOQUEUE_MAX_HANDLES.
  * @param ioqueue	Pointer to hold the newly created I/O Queue.
  *
  * @return		PJ_SUCCESS on success.
  */
-PJ_DECL(pj_status_t) pj_ioqueue_create( pj_pool_t *pool,
+PJ_DECL(pj_status_t) pj_ioqueue_create( pj_pool_t *pool, 
 					pj_size_t max_fd,
 					pj_ioqueue_t **ioqueue);
 
@@ -345,8 +345,8 @@ PJ_DECL(pj_status_t) pj_ioqueue_destroy( pj_ioqueue_t *ioque );
  * be called right after the I/O queue is created, before any handle is
  * registered to the I/O queue.
  *
- * Initially the I/O queue is created with non-recursive mutex protection.
- * Applications can supply alternative lock to be used by calling this
+ * Initially the I/O queue is created with non-recursive mutex protection. 
+ * Applications can supply alternative lock to be used by calling this 
  * function.
  *
  * @param ioque         The ioqueue instance.
@@ -355,13 +355,13 @@ PJ_DECL(pj_status_t) pj_ioqueue_destroy( pj_ioqueue_t *ioque );
  *
  * @return              PJ_SUCCESS or the appropriate error code.
  */
-PJ_DECL(pj_status_t) pj_ioqueue_set_lock( pj_ioqueue_t *ioque,
+PJ_DECL(pj_status_t) pj_ioqueue_set_lock( pj_ioqueue_t *ioque, 
 					  pj_lock_t *lock,
 					  pj_bool_t auto_delete );
 
 /**
  * Set default concurrency policy for this ioqueue. If this function is not
- * called, the default concurrency policy for the ioqueue is controlled by
+ * called, the default concurrency policy for the ioqueue is controlled by 
  * compile time setting PJ_IOQUEUE_DEFAULT_ALLOW_CONCURRENCY.
  *
  * Note that changing the concurrency setting to the ioqueue will only affect
@@ -378,19 +378,19 @@ PJ_DECL(pj_status_t) pj_ioqueue_set_default_concurrency(pj_ioqueue_t *ioqueue,
 							pj_bool_t allow);
 
 /**
- * Register a socket to the I/O queue framework.
+ * Register a socket to the I/O queue framework. 
  * When a socket is registered to the IOQueue, it may be modified to use
- * non-blocking IO. If it is modified, there is no guarantee that this
+ * non-blocking IO. If it is modified, there is no guarantee that this 
  * modification will be restored after the socket is unregistered.
  *
- * @param pool	    To allocate the resource for the specified handle,
- *		    which must be valid until the handle/key is unregistered
+ * @param pool	    To allocate the resource for the specified handle, 
+ *		    which must be valid until the handle/key is unregistered 
  *		    from I/O Queue.
  * @param ioque	    The I/O Queue.
  * @param sock	    The socket.
  * @param user_data User data to be associated with the key, which can be
  *		    retrieved later.
- * @param cb	    Callback to be called when I/O operation completes.
+ * @param cb	    Callback to be called when I/O operation completes. 
  * @param key       Pointer to receive the key to be associated with this
  *                  socket. Subsequent I/O queue operation will need this
  *                  key.
@@ -404,6 +404,19 @@ PJ_DECL(pj_status_t) pj_ioqueue_register_sock( pj_pool_t *pool,
 					       const pj_ioqueue_callback *cb,
                                                pj_ioqueue_key_t **key );
 
+/**
+ * Variant of pj_ioqueue_register_sock() with additional group lock parameter.
+ * If group lock is set for the key, the key will add the reference counter
+ * when the socket is registered and decrease it when it is destroyed.
+ */
+PJ_DECL(pj_status_t) pj_ioqueue_register_sock2(pj_pool_t *pool,
+					       pj_ioqueue_t *ioque,
+					       pj_sock_t sock,
+					       pj_grp_lock_t *grp_lock,
+					       void *user_data,
+					       const pj_ioqueue_callback *cb,
+                                               pj_ioqueue_key_t **key );
+
 /**
  * Unregister from the I/O Queue framework. Caller must make sure that
  * the key doesn't have any pending operations before calling this function,
@@ -411,7 +424,7 @@ PJ_DECL(pj_status_t) pj_ioqueue_register_sock( pj_pool_t *pool,
  * operations except asynchronous connect, and if necessary call
  * #pj_ioqueue_post_completion() to cancel the pending operations.
  *
- * Note that asynchronous connect operation will automatically be
+ * Note that asynchronous connect operation will automatically be 
  * cancelled during the unregistration.
  *
  * Also note that when I/O Completion Port backend is used, application
@@ -433,7 +446,7 @@ PJ_DECL(pj_status_t) pj_ioqueue_unregister( pj_ioqueue_key_t *key );
  *
  * @param key	    The key that was previously obtained from registration.
  *
- * @return          The user data associated with the descriptor, or NULL
+ * @return          The user data associated with the descriptor, or NULL 
  *                  on error or if no data is associated with the key during
  *                  registration.
  */
@@ -479,7 +492,7 @@ PJ_DECL(pj_status_t) pj_ioqueue_set_concurrency(pj_ioqueue_key_t *key,
 						pj_bool_t allow);
 
 /**
- * Acquire the key's mutex. When the key's concurrency is disabled,
+ * Acquire the key's mutex. When the key's concurrency is disabled, 
  * application may call this function to synchronize its operation
  * with the key's callback (i.e. this function will block until the
  * key's callback returns).
@@ -510,8 +523,8 @@ PJ_DECL(void) pj_ioqueue_op_key_init( pj_ioqueue_op_key_t *op_key,
 
 /**
  * Check if operation is pending on the specified operation key.
- * The \c op_key must have been initialized with #pj_ioqueue_op_key_init()
- * or submitted as pending operation before, or otherwise the result
+ * The \c op_key must have been initialized with #pj_ioqueue_op_key_init() 
+ * or submitted as pending operation before, or otherwise the result 
  * is undefined.
  *
  * @param key       The key.
@@ -526,7 +539,7 @@ PJ_DECL(pj_bool_t) pj_ioqueue_is_pending( pj_ioqueue_key_t *key,
 
 /**
  * Post completion status to the specified operation key and call the
- * appropriate callback. When the callback is called, the number of bytes
+ * appropriate callback. When the callback is called, the number of bytes 
  * received in read/write callback or the status in accept/connect callback
  * will be set from the \c bytes_status parameter.
  *
@@ -546,7 +559,7 @@ PJ_DECL(pj_status_t) pj_ioqueue_post_completion( pj_ioqueue_key_t *key,
 
 #if defined(PJ_HAS_TCP) && PJ_HAS_TCP != 0
 /**
- * Instruct I/O Queue to accept incoming connection on the specified
+ * Instruct I/O Queue to accept incoming connection on the specified 
  * listening socket. This function will return immediately (i.e. non-blocking)
  * regardless whether a connection is immediately available. If the function
  * can't complete immediately, the caller will be notified about the incoming
@@ -561,16 +574,16 @@ PJ_DECL(pj_status_t) pj_ioqueue_post_completion( pj_ioqueue_key_t *key,
  *                  called.
  * @param new_sock  Argument which contain pointer to receive the new socket
  *                  for the incoming connection.
- * @param local	    Optional argument which contain pointer to variable to
+ * @param local	    Optional argument which contain pointer to variable to 
  *                  receive local address.
- * @param remote    Optional argument which contain pointer to variable to
+ * @param remote    Optional argument which contain pointer to variable to 
  *                  receive the remote address.
  * @param addrlen   On input, contains the length of the buffer for the
  *		    address, and on output, contains the actual length of the
  *		    address. This argument is optional.
  * @return
- *  - PJ_SUCCESS    When connection is available immediately, and the
- *                  parameters will be updated to contain information about
+ *  - PJ_SUCCESS    When connection is available immediately, and the 
+ *                  parameters will be updated to contain information about 
  *                  the new connection. In this case, a completion callback
  *                  WILL NOT be called.
  *  - PJ_EPENDING   If no connection is available immediately. When a new
@@ -598,7 +611,7 @@ PJ_DECL(pj_status_t) pj_ioqueue_accept( pj_ioqueue_key_t *key,
  * @return
  *  - PJ_SUCCESS    If socket is connected immediately. In this case, the
  *                  completion callback WILL NOT be called.
- *  - PJ_EPENDING   If operation is queued, or
+ *  - PJ_EPENDING   If operation is queued, or 
  *  - non-zero      Indicates the error code.
  */
 PJ_DECL(pj_status_t) pj_ioqueue_connect( pj_ioqueue_key_t *key,
@@ -617,7 +630,7 @@ PJ_DECL(pj_status_t) pj_ioqueue_connect( pj_ioqueue_key_t *key,
  * @param timeout	polling timeout, or NULL if the thread wishes to wait
  *			indefinetely for the event.
  *
- * @return
+ * @return 
  *  - zero if timed out (no event).
  *  - (<0) if error occured during polling. Callback will NOT be called.
  *  - (>1) to indicate numbers of events. Callbacks have been called.
@@ -628,8 +641,8 @@ PJ_DECL(int) pj_ioqueue_poll( pj_ioqueue_t *ioque,
 
 /**
  * Instruct the I/O Queue to read from the specified handle. This function
- * returns immediately (i.e. non-blocking) regardless whether some data has
- * been transfered. If the operation can't complete immediately, caller will
+ * returns immediately (i.e. non-blocking) regardless whether some data has 
+ * been transfered. If the operation can't complete immediately, caller will 
  * be notified about the completion when it calls pj_ioqueue_poll(). If data
  * is immediately available, the function will return PJ_SUCCESS and the
  * callback WILL NOT be called.
@@ -638,7 +651,7 @@ PJ_DECL(int) pj_ioqueue_poll( pj_ioqueue_t *ioque,
  * @param op_key    An operation specific key to be associated with the
  *                  pending operation, so that application can keep track of
  *                  which operation has been completed when the callback is
- *                  called. Caller must make sure that this key remains
+ *                  called. Caller must make sure that this key remains 
  *                  valid until the function completes.
  * @param buffer    The buffer to hold the read data. The caller MUST make sure
  *		    that this buffer remain valid until the framework completes
@@ -698,8 +711,8 @@ PJ_DECL(pj_status_t) pj_ioqueue_recv( pj_ioqueue_key_t *key,
  *                  specified.
  *
  * @return
- *  - PJ_SUCCESS    If immediate data has been received. In this case, the
- *		    callback must have been called before this function
+ *  - PJ_SUCCESS    If immediate data has been received. In this case, the 
+ *		    callback must have been called before this function 
  *		    returns, and no pending operation is scheduled.
  *  - PJ_EPENDING   If the operation has been queued.
  *  - non-zero      The return value indicates the error code.
@@ -714,9 +727,9 @@ PJ_DECL(pj_status_t) pj_ioqueue_recvfrom( pj_ioqueue_key_t *key,
 
 /**
  * Instruct the I/O Queue to write to the handle. This function will return
- * immediately (i.e. non-blocking) regardless whether some data has been
+ * immediately (i.e. non-blocking) regardless whether some data has been 
  * transfered. If the function can't complete immediately, the caller will
- * be notified about the completion when it calls pj_ioqueue_poll(). If
+ * be notified about the completion when it calls pj_ioqueue_poll(). If 
  * operation completes immediately and data has been transfered, the function
  * returns PJ_SUCCESS and the callback will NOT be called.
  *
@@ -725,15 +738,15 @@ PJ_DECL(pj_status_t) pj_ioqueue_recvfrom( pj_ioqueue_key_t *key,
  *                  pending operation, so that application can keep track of
  *                  which operation has been completed when the callback is
  *                  called.
- * @param data	    The data to send. Caller MUST make sure that this buffer
+ * @param data	    The data to send. Caller MUST make sure that this buffer 
  *		    remains valid until the write operation completes.
  * @param length    On input, it specifies the length of data to send. When
  *                  data was sent immediately, this function returns PJ_SUCCESS
  *                  and this parameter contains the length of data sent. If
  *                  data can not be sent immediately, an asynchronous operation
  *                  is scheduled and caller will be notified via callback the
- *                  number of bytes sent. This parameter can point to local
- *                  variable on caller's stack and doesn't have to remain
+ *                  number of bytes sent. This parameter can point to local 
+ *                  variable on caller's stack and doesn't have to remain 
  *                  valid until the operation has completed.
  * @param flags     Send flags. If flags has PJ_IOQUEUE_ALWAYS_ASYNC then
  *		    the function will never return PJ_SUCCESS.
@@ -755,9 +768,9 @@ PJ_DECL(pj_status_t) pj_ioqueue_send( pj_ioqueue_key_t *key,
 
 /**
  * Instruct the I/O Queue to write to the handle. This function will return
- * immediately (i.e. non-blocking) regardless whether some data has been
+ * immediately (i.e. non-blocking) regardless whether some data has been 
  * transfered. If the function can't complete immediately, the caller will
- * be notified about the completion when it calls pj_ioqueue_poll(). If
+ * be notified about the completion when it calls pj_ioqueue_poll(). If 
  * operation completes immediately and data has been transfered, the function
  * returns PJ_SUCCESS and the callback will NOT be called.
  *
@@ -766,15 +779,15 @@ PJ_DECL(pj_status_t) pj_ioqueue_send( pj_ioqueue_key_t *key,
  *                  pending operation, so that application can keep track of
  *                  which operation has been completed when the callback is
  *                  called.
- * @param data	    the data to send. Caller MUST make sure that this buffer
+ * @param data	    the data to send. Caller MUST make sure that this buffer 
  *		    remains valid until the write operation completes.
  * @param length    On input, it specifies the length of data to send. When
  *                  data was sent immediately, this function returns PJ_SUCCESS
  *                  and this parameter contains the length of data sent. If
  *                  data can not be sent immediately, an asynchronous operation
  *                  is scheduled and caller will be notified via callback the
- *                  number of bytes sent. This parameter can point to local
- *                  variable on caller's stack and doesn't have to remain
+ *                  number of bytes sent. This parameter can point to local 
+ *                  variable on caller's stack and doesn't have to remain 
  *                  valid until the operation has completed.
  * @param flags     send flags. If flags has PJ_IOQUEUE_ALWAYS_ASYNC then
  *		    the function will never return PJ_SUCCESS.
@@ -802,3 +815,4 @@ PJ_DECL(pj_status_t) pj_ioqueue_sendto( pj_ioqueue_key_t *key,
 PJ_END_DECL
 
 #endif	/* __PJ_IOQUEUE_H__ */
+
