@@ -1,5 +1,5 @@
-/* $Id: guid.c 3553 2011-05-05 06:14:19Z nanang $ */
-/*
+/* $Id: guid.c 4208 2012-07-18 07:52:33Z ming $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,13 +15,34 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
+#include <pj/ctype.h>
 #include <pj/guid.h>
 #include <pj/pool.h>
 
+PJ_DEF(pj_str_t*) pj_generate_unique_string_lower(pj_str_t *str)
+{
+    int i;
+
+    pj_generate_unique_string(str);
+    for (i = 0; i < str->slen; i++)
+	str->ptr[i] = (char)pj_tolower(str->ptr[i]);
+
+    return str;
+}
+
 PJ_DEF(void) pj_create_unique_string(pj_pool_t *pool, pj_str_t *str)
 {
     str->ptr = (char*)pj_pool_alloc(pool, PJ_GUID_STRING_LENGTH);
     pj_generate_unique_string(str);
 }
+
+PJ_DEF(void) pj_create_unique_string_lower(pj_pool_t *pool, pj_str_t *str)
+{
+    int i;
+
+    pj_create_unique_string(pool, str);
+    for (i = 0; i < str->slen; i++)
+	str->ptr[i] = (char)pj_tolower(str->ptr[i]);
+}
