@@ -1,5 +1,5 @@
-/* $Id: os_core_unix.c 3999 2012-03-30 07:10:13Z bennylp $ */
-/*
+/* $Id: os_core_unix.c 4359 2013-02-21 11:18:36Z bennylp $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 /*
  * Contributors:
@@ -97,7 +97,18 @@ struct pj_sem_t
 #if defined(PJ_HAS_EVENT_OBJ) && PJ_HAS_EVENT_OBJ != 0
 struct pj_event_t
 {
-    char		obj_name[PJ_MAX_OBJ_NAME];
+    enum event_state {
+	EV_STATE_OFF,
+	EV_STATE_SET,
+	EV_STATE_PULSED
+    } state;
+
+    pj_mutex_t		mutex;
+    pthread_cond_t	cond;
+
+    pj_bool_t		auto_reset;
+    unsigned		threads_waiting;
+    unsigned		threads_to_release;
 };
 #endif	/* PJ_HAS_EVENT_OBJ */
 
@@ -153,13 +164,13 @@ PJ_DEF(pj_status_t) pj_init(void)
     /* Init logging */
     pj_log_init();
 
-    /* Initialize exception ID for the pool.
+    /* Initialize exception ID for the pool. 
      * Must do so after critical section is configured.
      */
     rc = pj_exception_id_alloc("PJLIB/No memory", &PJ_NO_MEMORY_EXCEPTION);
     if (rc != PJ_SUCCESS)
         return rc;
-
+    
     /* Init random seed. */
     /* Or probably not. Let application in charge of this */
     /* pj_srand( clock() ); */
@@ -176,7 +187,7 @@ PJ_DEF(pj_status_t) pj_init(void)
 	    return rc;
 	}
     }
-#endif
+#endif   
 
     /* Flag PJLIB as initialized */
     ++initialized;
@@ -371,7 +382,7 @@ PJ_DEF(int) pj_thread_get_prio_max(pj_thread_t *thread)
 /*
  * Get native thread handle
  */
-PJ_DEF(void*) pj_thread_get_os_handle(pj_thread_t *thread)
+PJ_DEF(void*) pj_thread_get_os_handle(pj_thread_t *thread) 
 {
     PJ_ASSERT_RETURN(thread, NULL);
 
@@ -429,12 +440,12 @@ PJ_DEF(pj_status_t) pj_thread_register ( const char *cstr_thread_name,
     thread->signature2 = SIGNATURE2;
 
     if(cstr_thread_name && pj_strlen(&thread_name) < sizeof(thread->obj_name)-1)
-	pj_ansi_snprintf(thread->obj_name, sizeof(thread->obj_name),
+	pj_ansi_snprintf(thread->obj_name, sizeof(thread->obj_name), 
 			 cstr_thread_name, thread->thread);
     else
-	pj_ansi_snprintf(thread->obj_name, sizeof(thread->obj_name),
+	pj_ansi_snprintf(thread->obj_name, sizeof(thread->obj_name), 
 			 "thr%p", (void*)thread->thread);
-
+    
     rc = pj_thread_local_set(thread_tls_id, thread);
     if (rc != PJ_SUCCESS) {
 	pj_bzero(desc, sizeof(struct pj_thread_t));
@@ -521,11 +532,11 @@ static void *thread_main(void *param)
 /*
  * pj_thread_create(...)
  */
-PJ_DEF(pj_status_t) pj_thread_create( pj_pool_t *pool,
+PJ_DEF(pj_status_t) pj_thread_create( pj_pool_t *pool, 
 				      const char *thread_name,
-				      pj_thread_proc *proc,
+				      pj_thread_proc *proc, 
 				      void *arg,
-				      pj_size_t stack_size,
+				      pj_size_t stack_size, 
 				      unsigned flags,
 				      pj_thread_t **ptr_thread)
 {
@@ -543,11 +554,11 @@ PJ_DEF(pj_status_t) pj_thread_create( pj_pool_t *pool,
     /* Create thread record and assign name for the thread */
     rec = (struct pj_thread_t*) pj_pool_zalloc(pool, sizeof(pj_thread_t));
     PJ_ASSERT_RETURN(rec, PJ_ENOMEM);
-
+    
     /* Set name. */
-    if (!thread_name)
+    if (!thread_name) 
 	thread_name = "thr%p";
-
+    
     if (strchr(thread_name, '%')) {
 	pj_ansi_snprintf(rec->obj_name, PJ_MAX_OBJ_NAME, thread_name, rec);
     } else {
@@ -575,7 +586,7 @@ PJ_DEF(pj_status_t) pj_thread_create( pj_pool_t *pool,
     } else {
 	pj_assert(rec->suspended_mutex == NULL);
     }
-
+    
 
     /* Init thread attributes */
     pthread_attr_init(&thread_attr);
@@ -656,7 +667,7 @@ PJ_DEF(pj_thread_t*) pj_thread_this(void)
 {
 #if PJ_HAS_THREADS
     pj_thread_t *rec = (pj_thread_t*)pj_thread_local_get(thread_tls_id);
-
+    
     if (rec == NULL) {
 	pj_assert(!"Calling pjlib from unknown/external thread. You must "
 		   "register external threads with pj_thread_register() "
@@ -694,8 +705,8 @@ PJ_DEF(pj_status_t) pj_thread_join(pj_thread_t *p)
     if (result == 0)
 	return PJ_SUCCESS;
     else {
-	/* Calling pthread_join() on a thread that no longer exists and
-	 * getting back ESRCH isn't an error (in this context).
+	/* Calling pthread_join() on a thread that no longer exists and 
+	 * getting back ESRCH isn't an error (in this context). 
 	 * Thanks Phil Torre <ptorre@zetron.com>.
 	 */
 	return result==ESRCH ? PJ_SUCCESS : PJ_RETURN_OS_ERROR(result);
@@ -750,9 +761,9 @@ PJ_DEF(pj_status_t) pj_thread_sleep(unsigned msec)
 
     /* MacOS X (reported on 10.5) seems to always set errno to ETIMEDOUT.
      * It does so because usleep() is declared to return int, and we're
-     * supposed to check for errno only when usleep() returns non-zero.
+     * supposed to check for errno only when usleep() returns non-zero. 
      * Unfortunately, usleep() is declared to return void in other platforms
-     * so it's not possible to always check for the return value (unless
+     * so it's not possible to always check for the return value (unless 
      * we add a detection routine in autoconf).
      *
      * As a workaround, here we check if ETIMEDOUT is returned and
@@ -820,7 +831,7 @@ PJ_DEF(pj_status_t) pj_thread_get_stack_info( pj_thread_t *thread,
 /*
  * pj_atomic_create()
  */
-PJ_DEF(pj_status_t) pj_atomic_create( pj_pool_t *pool,
+PJ_DEF(pj_status_t) pj_atomic_create( pj_pool_t *pool, 
 				      pj_atomic_value_t initial,
 				      pj_atomic_t **ptr_atomic)
 {
@@ -830,7 +841,7 @@ PJ_DEF(pj_status_t) pj_atomic_create( pj_pool_t *pool,
     atomic_var = PJ_POOL_ZALLOC_T(pool, pj_atomic_t);
 
     PJ_ASSERT_RETURN(atomic_var, PJ_ENOMEM);
-
+    
 #if PJ_HAS_THREADS
     rc = pj_mutex_create(pool, "atm%p", PJ_MUTEX_SIMPLE, &atomic_var->mutex);
     if (rc != PJ_SUCCESS)
@@ -868,7 +879,7 @@ PJ_DEF(void) pj_atomic_set(pj_atomic_t *atomic_var, pj_atomic_value_t value)
     atomic_var->value = value;
 #if PJ_HAS_THREADS
     pj_mutex_unlock( atomic_var->mutex);
-#endif
+#endif 
 }
 
 /*
@@ -877,7 +888,7 @@ PJ_DEF(void) pj_atomic_set(pj_atomic_t *atomic_var, pj_atomic_value_t value)
 PJ_DEF(pj_atomic_value_t) pj_atomic_get(pj_atomic_t *atomic_var)
 {
     pj_atomic_value_t oldval;
-
+    
     PJ_CHECK_STACK();
 
 #if PJ_HAS_THREADS
@@ -947,8 +958,8 @@ PJ_DEF(void) pj_atomic_dec(pj_atomic_t *atomic_var)
 
 /*
  * pj_atomic_add_and_get()
- */
-PJ_DEF(pj_atomic_value_t) pj_atomic_add_and_get( pj_atomic_t *atomic_var,
+ */ 
+PJ_DEF(pj_atomic_value_t) pj_atomic_add_and_get( pj_atomic_t *atomic_var, 
                                                  pj_atomic_value_t value )
 {
     pj_atomic_value_t new_value;
@@ -956,7 +967,7 @@ PJ_DEF(pj_atomic_value_t) pj_atomic_add_and_get( pj_atomic_t *atomic_var,
 #if PJ_HAS_THREADS
     pj_mutex_lock(atomic_var->mutex);
 #endif
-
+    
     atomic_var->value += value;
     new_value = atomic_var->value;
 
@@ -969,8 +980,8 @@ PJ_DEF(pj_atomic_value_t) pj_atomic_add_and_get( pj_atomic_t *atomic_var,
 
 /*
  * pj_atomic_add()
- */
-PJ_DEF(void) pj_atomic_add( pj_atomic_t *atomic_var,
+ */ 
+PJ_DEF(void) pj_atomic_add( pj_atomic_t *atomic_var, 
                             pj_atomic_value_t value )
 {
     pj_atomic_add_and_get(atomic_var, value);
@@ -1000,9 +1011,9 @@ PJ_DEF(pj_status_t) pj_thread_local_alloc(long *p_index)
 	if (tls_flag[i] == 0)
 	    break;
     }
-    if (i == MAX_THREADS)
+    if (i == MAX_THREADS) 
 	return PJ_ETOOMANY;
-
+    
     tls_flag[i] = 1;
     tls[i] = NULL;
 
@@ -1107,7 +1118,7 @@ static pj_status_t init_mutex(pj_mutex_t *mutex, const char *name, int type)
 #elif (defined(PJ_RTEMS) && PJ_RTEMS!=0) || \
        defined(PJ_PTHREAD_MUTEXATTR_T_HAS_RECURSIVE)
 	// Phil Torre <ptorre@zetron.com>:
-	// The RTEMS implementation of POSIX mutexes doesn't include
+	// The RTEMS implementation of POSIX mutexes doesn't include 
 	// pthread_mutexattr_settype(), so what follows is a hack
 	// until I get RTEMS patched to support the set/get functions.
 	//
@@ -1120,7 +1131,7 @@ static pj_status_t init_mutex(pj_mutex_t *mutex, const char *name, int type)
 	rc = pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
 #endif
     }
-
+    
     if (rc != 0) {
 	return PJ_RETURN_OS_ERROR(rc);
     }
@@ -1129,7 +1140,7 @@ static pj_status_t init_mutex(pj_mutex_t *mutex, const char *name, int type)
     if (rc != 0) {
 	return PJ_RETURN_OS_ERROR(rc);
     }
-
+    
     rc = pthread_mutexattr_destroy(&attr);
     if (rc != 0) {
 	pj_status_t status = PJ_RETURN_OS_ERROR(rc);
@@ -1164,8 +1175,8 @@ static pj_status_t init_mutex(pj_mutex_t *mutex, const char *name, int type)
 /*
  * pj_mutex_create()
  */
-PJ_DEF(pj_status_t) pj_mutex_create(pj_pool_t *pool,
-				    const char *name,
+PJ_DEF(pj_status_t) pj_mutex_create(pj_pool_t *pool, 
+				    const char *name, 
 				    int type,
 				    pj_mutex_t **ptr_mutex)
 {
@@ -1180,7 +1191,7 @@ PJ_DEF(pj_status_t) pj_mutex_create(pj_pool_t *pool,
 
     if ((rc=init_mutex(mutex, name, type)) != PJ_SUCCESS)
 	return rc;
-
+    
     *ptr_mutex = mutex;
     return PJ_SUCCESS;
 #else /* PJ_HAS_THREADS */
@@ -1192,7 +1203,7 @@ PJ_DEF(pj_status_t) pj_mutex_create(pj_pool_t *pool,
 /*
  * pj_mutex_create_simple()
  */
-PJ_DEF(pj_status_t) pj_mutex_create_simple( pj_pool_t *pool,
+PJ_DEF(pj_status_t) pj_mutex_create_simple( pj_pool_t *pool, 
                                             const char *name,
 					    pj_mutex_t **mutex )
 {
@@ -1221,11 +1232,11 @@ PJ_DEF(pj_status_t) pj_mutex_lock(pj_mutex_t *mutex)
     PJ_ASSERT_RETURN(mutex, PJ_EINVAL);
 
 #if PJ_DEBUG
-    PJ_LOG(6,(mutex->obj_name, "Mutex: thread %s is waiting (mutex owner=%s)",
+    PJ_LOG(6,(mutex->obj_name, "Mutex: thread %s is waiting (mutex owner=%s)", 
 				pj_thread_this()->obj_name,
 				mutex->owner_name));
 #else
-    PJ_LOG(6,(mutex->obj_name, "Mutex: thread %s is waiting",
+    PJ_LOG(6,(mutex->obj_name, "Mutex: thread %s is waiting", 
 				pj_thread_this()->obj_name));
 #endif
 
@@ -1239,14 +1250,14 @@ PJ_DEF(pj_status_t) pj_mutex_lock(pj_mutex_t *mutex)
 	++mutex->nesting_level;
     }
 
-    PJ_LOG(6,(mutex->obj_name,
-	      (status==0 ?
-		"Mutex acquired by thread %s (level=%d)" :
+    PJ_LOG(6,(mutex->obj_name, 
+	      (status==0 ? 
+		"Mutex acquired by thread %s (level=%d)" : 
 		"Mutex acquisition FAILED by %s (level=%d)"),
 	      pj_thread_this()->obj_name,
 	      mutex->nesting_level));
 #else
-    PJ_LOG(6,(mutex->obj_name,
+    PJ_LOG(6,(mutex->obj_name, 
 	      (status==0 ? "Mutex acquired by thread %s" : "FAILED by %s"),
 	      pj_thread_this()->obj_name));
 #endif
@@ -1279,11 +1290,11 @@ PJ_DEF(pj_status_t) pj_mutex_unlock(pj_mutex_t *mutex)
 	mutex->owner_name[0] = '\0';
     }
 
-    PJ_LOG(6,(mutex->obj_name, "Mutex released by thread %s (level=%d)",
-				pj_thread_this()->obj_name,
+    PJ_LOG(6,(mutex->obj_name, "Mutex released by thread %s (level=%d)", 
+				pj_thread_this()->obj_name, 
 				mutex->nesting_level));
 #else
-    PJ_LOG(6,(mutex->obj_name, "Mutex released by thread %s",
+    PJ_LOG(6,(mutex->obj_name, "Mutex released by thread %s", 
 				pj_thread_this()->obj_name));
 #endif
 
@@ -1310,7 +1321,7 @@ PJ_DEF(pj_status_t) pj_mutex_trylock(pj_mutex_t *mutex)
     PJ_CHECK_STACK();
     PJ_ASSERT_RETURN(mutex, PJ_EINVAL);
 
-    PJ_LOG(6,(mutex->obj_name, "Mutex: thread %s is trying",
+    PJ_LOG(6,(mutex->obj_name, "Mutex: thread %s is trying", 
 				pj_thread_this()->obj_name));
 
     status = pthread_mutex_trylock( &mutex->mutex );
@@ -1321,18 +1332,18 @@ PJ_DEF(pj_status_t) pj_mutex_trylock(pj_mutex_t *mutex)
 	pj_ansi_strcpy(mutex->owner_name, mutex->owner->obj_name);
 	++mutex->nesting_level;
 
-	PJ_LOG(6,(mutex->obj_name, "Mutex acquired by thread %s (level=%d)",
+	PJ_LOG(6,(mutex->obj_name, "Mutex acquired by thread %s (level=%d)", 
 				   pj_thread_this()->obj_name,
 				   mutex->nesting_level));
 #else
-	PJ_LOG(6,(mutex->obj_name, "Mutex acquired by thread %s",
+	PJ_LOG(6,(mutex->obj_name, "Mutex acquired by thread %s", 
 				  pj_thread_this()->obj_name));
 #endif
     } else {
-	PJ_LOG(6,(mutex->obj_name, "Mutex: thread %s's trylock() failed",
+	PJ_LOG(6,(mutex->obj_name, "Mutex: thread %s's trylock() failed", 
 				    pj_thread_this()->obj_name));
     }
-
+    
     if (status==0)
 	return PJ_SUCCESS;
     else
@@ -1414,7 +1425,7 @@ PJ_DEF(pj_status_t) pj_rwmutex_create(pj_pool_t *pool, const char *name,
     pj_status_t status;
 
     PJ_UNUSED_ARG(name);
-
+    
     rwm = PJ_POOL_ALLOC_T(pool, pj_rwmutex_t);
     PJ_ASSERT_RETURN(rwm, PJ_ENOMEM);
 
@@ -1504,9 +1515,9 @@ PJ_DEF(pj_status_t) pj_rwmutex_destroy(pj_rwmutex_t *mutex)
 /*
  * pj_sem_create()
  */
-PJ_DEF(pj_status_t) pj_sem_create( pj_pool_t *pool,
+PJ_DEF(pj_status_t) pj_sem_create( pj_pool_t *pool, 
 				   const char *name,
-				   unsigned initial,
+				   unsigned initial, 
 				   unsigned max,
 				   pj_sem_t **ptr_sem)
 {
@@ -1525,7 +1536,7 @@ PJ_DEF(pj_status_t) pj_sem_create( pj_pool_t *pool,
 	char sem_name[PJ_GUID_MAX_LENGTH+1];
 	pj_str_t nam;
 
-	/* We should use SEM_NAME_LEN, but this doesn't seem to be
+	/* We should use SEM_NAME_LEN, but this doesn't seem to be 
 	 * declared anywhere? The value here is just from trial and error
 	 * to get the longest name supported.
 	 */
@@ -1542,7 +1553,7 @@ PJ_DEF(pj_status_t) pj_sem_create( pj_pool_t *pool,
 	}
 
 	/* Create semaphore */
-	sem->sem = sem_open(sem_name, O_CREAT|O_EXCL, S_IRUSR|S_IWUSR,
+	sem->sem = sem_open(sem_name, O_CREAT|O_EXCL, S_IRUSR|S_IWUSR, 
 			    initial);
 	if (sem->sem == SEM_FAILED)
 	    return PJ_RETURN_OS_ERROR(pj_get_native_os_error());
@@ -1552,10 +1563,10 @@ PJ_DEF(pj_status_t) pj_sem_create( pj_pool_t *pool,
     }
 #else
     sem->sem = PJ_POOL_ALLOC_T(pool, sem_t);
-    if (sem_init( sem->sem, 0, initial) != 0)
+    if (sem_init( sem->sem, 0, initial) != 0) 
 	return PJ_RETURN_OS_ERROR(pj_get_native_os_error());
 #endif
-
+    
     /* Set name. */
     if (!name) {
 	name = "sem%p";
@@ -1588,16 +1599,16 @@ PJ_DEF(pj_status_t) pj_sem_wait(pj_sem_t *sem)
     PJ_CHECK_STACK();
     PJ_ASSERT_RETURN(sem, PJ_EINVAL);
 
-    PJ_LOG(6, (sem->obj_name, "Semaphore: thread %s is waiting",
+    PJ_LOG(6, (sem->obj_name, "Semaphore: thread %s is waiting", 
 			      pj_thread_this()->obj_name));
 
     result = sem_wait( sem->sem );
-
+    
     if (result == 0) {
-	PJ_LOG(6, (sem->obj_name, "Semaphore acquired by thread %s",
+	PJ_LOG(6, (sem->obj_name, "Semaphore acquired by thread %s", 
 				  pj_thread_this()->obj_name));
     } else {
-	PJ_LOG(6, (sem->obj_name, "Semaphore: thread %s FAILED to acquire",
+	PJ_LOG(6, (sem->obj_name, "Semaphore: thread %s FAILED to acquire", 
 				  pj_thread_this()->obj_name));
     }
 
@@ -1623,11 +1634,11 @@ PJ_DEF(pj_status_t) pj_sem_trywait(pj_sem_t *sem)
     PJ_ASSERT_RETURN(sem, PJ_EINVAL);
 
     result = sem_trywait( sem->sem );
-
+    
     if (result == 0) {
-	PJ_LOG(6, (sem->obj_name, "Semaphore acquired by thread %s",
+	PJ_LOG(6, (sem->obj_name, "Semaphore acquired by thread %s", 
 				  pj_thread_this()->obj_name));
-    }
+    } 
     if (result == 0)
 	return PJ_SUCCESS;
     else
@@ -1700,13 +1711,48 @@ PJ_DEF(pj_status_t) pj_event_create(pj_pool_t *pool, const char *name,
 				    pj_bool_t manual_reset, pj_bool_t initial,
 				    pj_event_t **ptr_event)
 {
-    pj_assert(!"Not supported!");
-    PJ_UNUSED_ARG(pool);
-    PJ_UNUSED_ARG(name);
-    PJ_UNUSED_ARG(manual_reset);
-    PJ_UNUSED_ARG(initial);
-    PJ_UNUSED_ARG(ptr_event);
-    return PJ_EINVALIDOP;
+    pj_event_t *event;
+
+    event = PJ_POOL_ALLOC_T(pool, pj_event_t);
+
+    init_mutex(&event->mutex, name, PJ_MUTEX_SIMPLE);
+    pthread_cond_init(&event->cond, 0);
+    event->auto_reset = !manual_reset;
+    event->threads_waiting = 0;
+
+    if (initial) {
+	event->state = EV_STATE_SET;
+	event->threads_to_release = 1;
+    } else {
+	event->state = EV_STATE_OFF;
+	event->threads_to_release = 0;
+    }
+
+    *ptr_event = event;
+    return PJ_SUCCESS;
+}
+
+static void event_on_one_release(pj_event_t *event)
+{
+    if (event->state == EV_STATE_SET) {
+	if (event->auto_reset) {
+	    event->threads_to_release = 0;
+	    event->state = EV_STATE_OFF;
+	} else {
+	    /* Manual reset remains on */
+	}
+    } else {
+	if (event->auto_reset) {
+	    /* Only release one */
+	    event->threads_to_release = 0;
+	    event->state = EV_STATE_OFF;
+	} else {
+	    event->threads_to_release--;
+	    pj_assert(event->threads_to_release >= 0);
+	    if (event->threads_to_release==0)
+		event->state = EV_STATE_OFF;
+	}
+    }
 }
 
 /*
@@ -1714,8 +1760,14 @@ PJ_DEF(pj_status_t) pj_event_create(pj_pool_t *pool, const char *name,
  */
 PJ_DEF(pj_status_t) pj_event_wait(pj_event_t *event)
 {
-    PJ_UNUSED_ARG(event);
-    return PJ_EINVALIDOP;
+    pthread_mutex_lock(&event->mutex.mutex);
+    event->threads_waiting++;
+    while (event->state == EV_STATE_OFF)
+	pthread_cond_wait(&event->cond, &event->mutex.mutex);
+    event->threads_waiting--;
+    event_on_one_release(event);
+    pthread_mutex_unlock(&event->mutex.mutex);
+    return PJ_SUCCESS;
 }
 
 /*
@@ -1723,8 +1775,16 @@ PJ_DEF(pj_status_t) pj_event_wait(pj_event_t *event)
  */
 PJ_DEF(pj_status_t) pj_event_trywait(pj_event_t *event)
 {
-    PJ_UNUSED_ARG(event);
-    return PJ_EINVALIDOP;
+    pj_status_t status;
+
+    pthread_mutex_lock(&event->mutex.mutex);
+    status = event->state != EV_STATE_OFF ? PJ_SUCCESS : -1;
+    if (status==PJ_SUCCESS) {
+	event_on_one_release(event);
+    }
+    pthread_mutex_unlock(&event->mutex.mutex);
+
+    return status;
 }
 
 /*
@@ -1732,8 +1792,15 @@ PJ_DEF(pj_status_t) pj_event_trywait(pj_event_t *event)
  */
 PJ_DEF(pj_status_t) pj_event_set(pj_event_t *event)
 {
-    PJ_UNUSED_ARG(event);
-    return PJ_EINVALIDOP;
+    pthread_mutex_lock(&event->mutex.mutex);
+    event->threads_to_release = 1;
+    event->state = EV_STATE_SET;
+    if (event->auto_reset)
+	pthread_cond_signal(&event->cond);
+    else
+	pthread_cond_broadcast(&event->cond);
+    pthread_mutex_unlock(&event->mutex.mutex);
+    return PJ_SUCCESS;
 }
 
 /*
@@ -1741,8 +1808,18 @@ PJ_DEF(pj_status_t) pj_event_set(pj_event_t *event)
  */
 PJ_DEF(pj_status_t) pj_event_pulse(pj_event_t *event)
 {
-    PJ_UNUSED_ARG(event);
-    return PJ_EINVALIDOP;
+    pthread_mutex_lock(&event->mutex.mutex);
+    if (event->threads_waiting) {
+	event->threads_to_release = event->auto_reset ? 1 :
+					event->threads_waiting;
+	event->state = EV_STATE_PULSED;
+	if (event->threads_to_release==1)
+	    pthread_cond_signal(&event->cond);
+	else
+	    pthread_cond_broadcast(&event->cond);
+    }
+    pthread_mutex_unlock(&event->mutex.mutex);
+    return PJ_SUCCESS;
 }
 
 /*
@@ -1750,8 +1827,11 @@ PJ_DEF(pj_status_t) pj_event_pulse(pj_event_t *event)
  */
 PJ_DEF(pj_status_t) pj_event_reset(pj_event_t *event)
 {
-    PJ_UNUSED_ARG(event);
-    return PJ_EINVALIDOP;
+    pthread_mutex_lock(&event->mutex.mutex);
+    event->state = EV_STATE_OFF;
+    event->threads_to_release = 0;
+    pthread_mutex_unlock(&event->mutex.mutex);
+    return PJ_SUCCESS;
 }
 
 /*
@@ -1759,8 +1839,9 @@ PJ_DEF(pj_status_t) pj_event_reset(pj_event_t *event)
  */
 PJ_DEF(pj_status_t) pj_event_destroy(pj_event_t *event)
 {
-    PJ_UNUSED_ARG(event);
-    return PJ_EINVALIDOP;
+    pj_mutex_destroy(&event->mutex);
+    pthread_cond_destroy(&event->cond);
+    return PJ_SUCCESS;
 }
 
 #endif	/* PJ_HAS_EVENT_OBJ */
