@@ -1,5 +1,5 @@
-/* $Id: sock_bsd.c 4170 2012-06-19 07:40:19Z bennylp $ */
-/*
+/* $Id: sock_bsd.c 4233 2012-08-21 11:16:06Z ming $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pj/sock.h>
 #include <pj/os.h>
@@ -230,7 +230,7 @@ PJ_DEF(char*) pj_inet_ntoa(pj_in_addr inaddr)
 /*
  * This function converts the Internet host address cp from the standard
  * numbers-and-dots notation into binary data and stores it in the structure
- * that inp points to.
+ * that inp points to. 
  */
 PJ_DEF(int) pj_inet_aton(const pj_str_t *cp, struct pj_in_addr *inp)
 {
@@ -272,7 +272,7 @@ PJ_DEF(pj_status_t) pj_inet_pton(int af, const pj_str_t *src, void *dst)
     PJ_ASSERT_RETURN(af==PJ_AF_INET || af==PJ_AF_INET6, PJ_EAFNOTSUP);
     PJ_ASSERT_RETURN(src && src->slen && dst, PJ_EINVAL);
 
-    /* Initialize output with PJ_IN_ADDR_NONE for IPv4 (to be
+    /* Initialize output with PJ_IN_ADDR_NONE for IPv4 (to be 
      * compatible with pj_inet_aton()
      */
     if (af==PJ_AF_INET) {
@@ -317,7 +317,7 @@ PJ_DEF(pj_status_t) pj_inet_pton(int af, const pj_str_t *src, void *dst)
 
 	sock_addr.addr.sa_family = (pj_uint16_t)af;
 	rc = WSAStringToAddress(
-		PJ_STRING_TO_NATIVE(tempaddr,wtempaddr,sizeof(wtempaddr)),
+		PJ_STRING_TO_NATIVE(tempaddr,wtempaddr,sizeof(wtempaddr)), 
 		af, NULL, (LPSOCKADDR)&sock_addr, &addr_len);
 	if (rc != 0) {
 	    /* If you get rc 130022 Invalid argument (WSAEINVAL) with IPv6,
@@ -465,8 +465,8 @@ PJ_DEF(const pj_str_t*) pj_gethostname(void)
 /*
  * Create new socket/endpoint for communication and returns a descriptor.
  */
-PJ_DEF(pj_status_t) pj_sock_socket(int af,
-				   int type,
+PJ_DEF(pj_status_t) pj_sock_socket(int af, 
+				   int type, 
 				   int proto,
 				   pj_sock_t *sock)
 {
@@ -474,14 +474,14 @@ PJ_DEF(pj_status_t) pj_sock_socket(int af,
 
     /* Sanity checks. */
     PJ_ASSERT_RETURN(sock!=NULL, PJ_EINVAL);
-    PJ_ASSERT_RETURN((unsigned)PJ_INVALID_SOCKET==INVALID_SOCKET,
+    PJ_ASSERT_RETURN((unsigned)PJ_INVALID_SOCKET==INVALID_SOCKET, 
                      (*sock=PJ_INVALID_SOCKET, PJ_EINVAL));
 
     *sock = WSASocket(af, type, proto, NULL, 0, WSA_FLAG_OVERLAPPED);
 
-    if (*sock == PJ_INVALID_SOCKET)
+    if (*sock == PJ_INVALID_SOCKET) 
 	return PJ_RETURN_OS_ERROR(pj_get_native_netos_error());
-
+    
 #if PJ_SOCK_DISABLE_WSAECONNRESET && \
     (!defined(PJ_WIN32_WINCE) || PJ_WIN32_WINCE==0)
 
@@ -515,9 +515,9 @@ PJ_DEF(pj_status_t) pj_sock_socket(int af,
 /*
  * Create new socket/endpoint for communication and returns a descriptor.
  */
-PJ_DEF(pj_status_t) pj_sock_socket(int af,
-				   int type,
-				   int proto,
+PJ_DEF(pj_status_t) pj_sock_socket(int af, 
+				   int type, 
+				   int proto, 
 				   pj_sock_t *sock)
 {
 
@@ -525,9 +525,9 @@ PJ_DEF(pj_status_t) pj_sock_socket(int af,
 
     /* Sanity checks. */
     PJ_ASSERT_RETURN(sock!=NULL, PJ_EINVAL);
-    PJ_ASSERT_RETURN(PJ_INVALID_SOCKET==-1,
+    PJ_ASSERT_RETURN(PJ_INVALID_SOCKET==-1, 
                      (*sock=PJ_INVALID_SOCKET, PJ_EINVAL));
-
+    
     *sock = socket(af, type, proto);
     if (*sock == PJ_INVALID_SOCKET)
 	return PJ_RETURN_OS_ERROR(pj_get_native_netos_error());
@@ -540,7 +540,7 @@ PJ_DEF(pj_status_t) pj_sock_socket(int af,
 #if defined(PJ_IPHONE_OS_HAS_MULTITASKING_SUPPORT) && \
     PJ_IPHONE_OS_HAS_MULTITASKING_SUPPORT!=0
 	if (type == pj_SOCK_DGRAM()) {
-	    pj_sock_setsockopt(*sock, pj_SOL_SOCKET(), SO_NOSIGPIPE,
+	    pj_sock_setsockopt(*sock, pj_SOL_SOCKET(), SO_NOSIGPIPE, 
 			       &val, sizeof(val));
 	}
 #endif
@@ -552,7 +552,7 @@ PJ_DEF(pj_status_t) pj_sock_socket(int af,
 /*
  * Bind socket.
  */
-PJ_DEF(pj_status_t) pj_sock_bind( pj_sock_t sock,
+PJ_DEF(pj_status_t) pj_sock_bind( pj_sock_t sock, 
 				  const pj_sockaddr_t *addr,
 				  int len)
 {
@@ -572,7 +572,7 @@ PJ_DEF(pj_status_t) pj_sock_bind( pj_sock_t sock,
 /*
  * Bind socket.
  */
-PJ_DEF(pj_status_t) pj_sock_bind_in( pj_sock_t sock,
+PJ_DEF(pj_status_t) pj_sock_bind_in( pj_sock_t sock, 
 				     pj_uint32_t addr32,
 				     pj_uint16_t port)
 {
@@ -680,15 +680,15 @@ PJ_DEF(pj_status_t) pj_sock_sendto(pj_sock_t sock,
 {
     PJ_CHECK_STACK();
     PJ_ASSERT_RETURN(len, PJ_EINVAL);
-
+    
     CHECK_ADDR_LEN(to, tolen);
 
-    *len = sendto(sock, (const char*)buf, *len, flags,
+    *len = sendto(sock, (const char*)buf, *len, flags, 
 		  (const struct sockaddr*)to, tolen);
 
-    if (*len < 0)
+    if (*len < 0) 
 	return PJ_RETURN_OS_ERROR(pj_get_native_netos_error());
-    else
+    else 
 	return PJ_SUCCESS;
 }
 
@@ -705,7 +705,7 @@ PJ_DEF(pj_status_t) pj_sock_recv(pj_sock_t sock,
 
     *len = recv(sock, (char*)buf, *len, flags);
 
-    if (*len < 0)
+    if (*len < 0) 
 	return PJ_RETURN_OS_ERROR(pj_get_native_netos_error());
     else
 	return PJ_SUCCESS;
@@ -723,15 +723,16 @@ PJ_DEF(pj_status_t) pj_sock_recvfrom(pj_sock_t sock,
 {
     PJ_CHECK_STACK();
     PJ_ASSERT_RETURN(buf && len, PJ_EINVAL);
-    PJ_ASSERT_RETURN(from && fromlen, (*len=-1, PJ_EINVAL));
 
-    *len = recvfrom(sock, (char*)buf, *len, flags,
+    *len = recvfrom(sock, (char*)buf, *len, flags, 
 		    (struct sockaddr*)from, (socklen_t*)fromlen);
 
-    if (*len < 0)
+    if (*len < 0) 
 	return PJ_RETURN_OS_ERROR(pj_get_native_netos_error());
     else {
-	PJ_SOCKADDR_RESET_LEN(from);
+        if (from) {
+            PJ_SOCKADDR_RESET_LEN(from);
+        }
 	return PJ_SUCCESS;
     }
 }
@@ -828,19 +829,21 @@ PJ_DEF(pj_status_t) pj_sock_accept( pj_sock_t serverfd,
 	PJ_SOCKADDR_SET_LEN(addr, *addrlen);
     }
 #endif
-
+    
     *newsock = accept(serverfd, (struct sockaddr*)addr, (socklen_t*)addrlen);
     if (*newsock==PJ_INVALID_SOCKET)
 	return PJ_RETURN_OS_ERROR(pj_get_native_netos_error());
     else {
-
+	
 #if defined(PJ_SOCKADDR_HAS_LEN) && PJ_SOCKADDR_HAS_LEN!=0
 	if (addr) {
 	    PJ_SOCKADDR_RESET_LEN(addr);
 	}
 #endif
-
+	    
 	return PJ_SUCCESS;
     }
 }
 #endif	/* PJ_HAS_TCP */
+
+
