@@ -1,5 +1,5 @@
-/* $Id: ioq_udp.c 3553 2011-05-05 06:14:19Z nanang $ */
-/*
+/* $Id: ioq_udp.c 4238 2012-08-31 06:17:56Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include "test.h"
 
@@ -71,18 +71,18 @@ static pj_ioqueue_op_key_t  *callback_read_op,
                             *callback_write_op,
                             *callback_accept_op;
 
-static void on_ioqueue_read(pj_ioqueue_key_t *key,
+static void on_ioqueue_read(pj_ioqueue_key_t *key, 
                             pj_ioqueue_op_key_t *op_key,
                             pj_ssize_t bytes_read)
 {
     callback_read_key = key;
     callback_read_op = op_key;
     callback_read_size = bytes_read;
-    TRACE__((THIS_FILE, "     callback_read_key = %p, bytes=%d",
+    TRACE__((THIS_FILE, "     callback_read_key = %p, bytes=%d", 
 	     key, bytes_read));
 }
 
-static void on_ioqueue_write(pj_ioqueue_key_t *key,
+static void on_ioqueue_write(pj_ioqueue_key_t *key, 
                              pj_ioqueue_op_key_t *op_key,
                              pj_ssize_t bytes_written)
 {
@@ -91,7 +91,7 @@ static void on_ioqueue_write(pj_ioqueue_key_t *key,
     callback_write_size = bytes_written;
 }
 
-static void on_ioqueue_accept(pj_ioqueue_key_t *key,
+static void on_ioqueue_accept(pj_ioqueue_key_t *key, 
                               pj_ioqueue_op_key_t *op_key,
                               pj_sock_t sock, int status)
 {
@@ -107,7 +107,7 @@ static void on_ioqueue_connect(pj_ioqueue_key_t *key, int status)
     callback_connect_status = status;
 }
 
-static pj_ioqueue_callback test_cb =
+static pj_ioqueue_callback test_cb = 
 {
     &on_ioqueue_read,
     &on_ioqueue_write,
@@ -125,7 +125,7 @@ static pj_ioqueue_callback test_cb =
  * compliance_test()
  * To test that the basic IOQueue functionality works. It will just exchange
  * data between two sockets.
- */
+ */ 
 static int compliance_test(pj_bool_t allow_concur)
 {
     pj_sock_t ssock=-1, csock=-1;
@@ -190,14 +190,14 @@ static int compliance_test(pj_bool_t allow_concur)
     // We put this after inactivity socket, hopefully this can represent the
     // worst waiting time.
     TRACE_("registering first sockets...");
-    rc = pj_ioqueue_register_sock(pool, ioque, ssock, NULL,
+    rc = pj_ioqueue_register_sock(pool, ioque, ssock, NULL, 
 			          &test_cb, &skey);
     if (rc != PJ_SUCCESS) {
 	app_perror("...error(10): ioqueue_register error", rc);
 	status=-25; goto on_error;
     }
     TRACE_("registering second sockets...");
-    rc = pj_ioqueue_register_sock( pool, ioque, csock, NULL,
+    rc = pj_ioqueue_register_sock( pool, ioque, csock, NULL, 
 			           &test_cb, &ckey);
     if (rc != PJ_SUCCESS) {
 	app_perror("...error(11): ioqueue_register error", rc);
@@ -219,10 +219,10 @@ static int compliance_test(pj_bool_t allow_concur)
 	status=-28; goto on_error;
     } else if (rc == PJ_EPENDING) {
 	recv_pending = 1;
-	PJ_LOG(3, (THIS_FILE,
+	PJ_LOG(3, (THIS_FILE, 
 		   "......ok: recvfrom returned pending"));
     } else {
-	PJ_LOG(3, (THIS_FILE,
+	PJ_LOG(3, (THIS_FILE, 
 		   "......error: recvfrom returned immediate ok!"));
 	status=-29; goto on_error;
     }
@@ -238,25 +238,25 @@ static int compliance_test(pj_bool_t allow_concur)
     // Write must return the number of bytes.
     TRACE_("start sendto...");
     bytes = bufsize;
-    rc = pj_ioqueue_sendto(ckey, &write_op, send_buf, &bytes, 0, &dst_addr,
+    rc = pj_ioqueue_sendto(ckey, &write_op, send_buf, &bytes, 0, &dst_addr, 
 			   sizeof(dst_addr));
     if (rc != PJ_SUCCESS && rc != PJ_EPENDING) {
         app_perror("...error: pj_ioqueue_sendto", rc);
 	status=-30; goto on_error;
     } else if (rc == PJ_EPENDING) {
 	send_pending = 1;
-	PJ_LOG(3, (THIS_FILE,
+	PJ_LOG(3, (THIS_FILE, 
 		   "......ok: sendto returned pending"));
     } else {
 	send_pending = 0;
-	PJ_LOG(3, (THIS_FILE,
+	PJ_LOG(3, (THIS_FILE, 
 		   "......ok: sendto returned immediate success"));
     }
 
     // reset callback variables.
     callback_read_size = callback_write_size = 0;
     callback_accept_status = callback_connect_status = -2;
-    callback_read_key = callback_write_key =
+    callback_read_key = callback_write_key = 
         callback_accept_key = callback_connect_key = NULL;
     callback_read_op = callback_write_op = NULL;
 
@@ -267,7 +267,7 @@ static int compliance_test(pj_bool_t allow_concur)
 
 	TRACE_("poll...");
 #ifdef PJ_SYMBIAN
-	rc = pj_symbianos_poll(-1, 5000);
+	rc = pj_symbianos_poll(-1, PJ_TIME_VAL_MSEC(timeout));
 #else
 	rc = pj_ioqueue_poll(ioque, &timeout);
 #endif
@@ -303,7 +303,7 @@ static int compliance_test(pj_bool_t allow_concur)
 
 
 	    recv_pending = 0;
-	}
+	} 
 
         if (callback_write_key != NULL) {
             if (callback_write_size != bufsize) {
@@ -318,22 +318,22 @@ static int compliance_test(pj_bool_t allow_concur)
 
             send_pending = 0;
 	}
-    }
-
+    } 
+    
     // Success
     status = 0;
 
 on_error:
     if (skey)
-	pj_ioqueue_unregister(skey);
+    	pj_ioqueue_unregister(skey);
     else if (ssock != -1)
 	pj_sock_close(ssock);
-
+    
     if (ckey)
-	pj_ioqueue_unregister(ckey);
+    	pj_ioqueue_unregister(ckey);
     else if (csock != -1)
 	pj_sock_close(csock);
-
+    
     if (ioque != NULL)
 	pj_ioqueue_destroy(ioque);
     pj_pool_release(pool);
@@ -342,8 +342,8 @@ on_error:
 }
 
 
-static void on_read_complete(pj_ioqueue_key_t *key,
-                             pj_ioqueue_op_key_t *op_key,
+static void on_read_complete(pj_ioqueue_key_t *key, 
+                             pj_ioqueue_op_key_t *op_key, 
                              pj_ssize_t bytes_read)
 {
     unsigned *p_packet_cnt = (unsigned*) pj_ioqueue_get_user_data(key);
@@ -356,9 +356,9 @@ static void on_read_complete(pj_ioqueue_key_t *key,
 
 /*
  * unregister_test()
- * Check if callback is still called after socket has been unregistered or
+ * Check if callback is still called after socket has been unregistered or 
  * closed.
- */
+ */ 
 static int unregister_test(pj_bool_t allow_concur)
 {
     enum { RPORT = 50000, SPORT = 50001 };
@@ -543,10 +543,10 @@ static int many_handles_test(pj_bool_t allow_concur)
     if (!pool)
 	return PJ_ENOMEM;
 
-    key = (pj_ioqueue_key_t**)
-	  pj_pool_alloc(pool, MAX*sizeof(pj_ioqueue_key_t*));
+    key = (pj_ioqueue_key_t**) 
+    	  pj_pool_alloc(pool, MAX*sizeof(pj_ioqueue_key_t*));
     sock = (pj_sock_t*) pj_pool_alloc(pool, MAX*sizeof(pj_sock_t));
-
+    
     /* Create IOQueue */
     rc = pj_ioqueue_create(pool, MAX, &ioqueue);
     if (rc != PJ_SUCCESS || ioqueue == NULL) {
@@ -565,7 +565,7 @@ static int many_handles_test(pj_bool_t allow_concur)
 	sock[count] = PJ_INVALID_SOCKET;
 	rc = pj_sock_socket(pj_AF_INET(), pj_SOCK_DGRAM(), 0, &sock[count]);
 	if (rc != PJ_SUCCESS || sock[count] == PJ_INVALID_SOCKET) {
-	    PJ_LOG(3,(THIS_FILE, "....unable to create %d-th socket, rc=%d",
+	    PJ_LOG(3,(THIS_FILE, "....unable to create %d-th socket, rc=%d", 
 				 count, rc));
 	    break;
 	}
@@ -573,7 +573,7 @@ static int many_handles_test(pj_bool_t allow_concur)
 	rc = pj_ioqueue_register_sock(pool, ioqueue, sock[count],
 				      NULL, &test_cb, &key[count]);
 	if (rc != PJ_SUCCESS || key[count] == NULL) {
-	    PJ_LOG(3,(THIS_FILE, "....unable to register %d-th socket, rc=%d",
+	    PJ_LOG(3,(THIS_FILE, "....unable to register %d-th socket, rc=%d", 
 				 count, rc));
 	    return -30;
 	}
@@ -581,7 +581,7 @@ static int many_handles_test(pj_bool_t allow_concur)
 
     /* Test complete. */
 
-    /* Now deregister and close all handles. */
+    /* Now deregister and close all handles. */ 
 
     /* NOTE for RTEMS:
      *  It seems that the order of close(sock) is pretty important here.
@@ -606,7 +606,7 @@ static int many_handles_test(pj_bool_t allow_concur)
     if (rc != PJ_SUCCESS) {
 	app_perror("...error in pj_ioqueue_destroy", rc);
     }
-
+    
     pj_pool_release(pool);
 
     PJ_LOG(3,(THIS_FILE,"....many_handles_test() ok"));
@@ -621,7 +621,7 @@ static int many_handles_test(pj_bool_t allow_concur)
 /*
  * Benchmarking IOQueue
  */
-static int bench_test(pj_bool_t allow_concur, int bufsize,
+static int bench_test(pj_bool_t allow_concur, int bufsize, 
 		      int inactive_sock_count)
 {
     pj_sock_t ssock=-1, csock=-1;
@@ -682,7 +682,7 @@ static int bench_test(pj_bool_t allow_concur, int bufsize,
 
     // Allocate inactive sockets, and bind them to some arbitrary address.
     // Then register them to the I/O queue, and start a read operation.
-    inactive_sock = (pj_sock_t*)pj_pool_alloc(pool,
+    inactive_sock = (pj_sock_t*)pj_pool_alloc(pool, 
 				    inactive_sock_count*sizeof(pj_sock_t));
     inactive_read_op = (pj_ioqueue_op_key_t*)pj_pool_alloc(pool,
                               inactive_sock_count*sizeof(pj_ioqueue_op_key_t));
@@ -702,7 +702,7 @@ static int bench_test(pj_bool_t allow_concur, int bufsize,
 	    app_perror("...error: pj_sock_bind()", rc);
 	    goto on_error;
 	}
-	rc = pj_ioqueue_register_sock(pool, ioque, inactive_sock[i],
+	rc = pj_ioqueue_register_sock(pool, ioque, inactive_sock[i], 
 			              NULL, &test_cb, &keys[i]);
 	if (rc != PJ_SUCCESS) {
 	    pj_sock_close(inactive_sock[i]);
@@ -724,14 +724,14 @@ static int bench_test(pj_bool_t allow_concur, int bufsize,
     // Register server and client socket.
     // We put this after inactivity socket, hopefully this can represent the
     // worst waiting time.
-    rc = pj_ioqueue_register_sock(pool, ioque, ssock, NULL,
+    rc = pj_ioqueue_register_sock(pool, ioque, ssock, NULL, 
 			          &test_cb, &skey);
     if (rc != PJ_SUCCESS) {
 	app_perror("...error(2): pj_ioqueue_register_sock()", rc);
 	goto on_error;
     }
 
-    rc = pj_ioqueue_register_sock(pool, ioque, csock, NULL,
+    rc = pj_ioqueue_register_sock(pool, ioque, csock, NULL, 
 			          &test_cb, &ckey);
     if (rc != PJ_SUCCESS) {
 	app_perror("...error(3): pj_ioqueue_register_sock()", rc);
@@ -783,7 +783,7 @@ static int bench_test(pj_bool_t allow_concur, int bufsize,
 	do {
 	    pj_time_val timeout = { 1, 0 };
 #ifdef PJ_SYMBIAN
-	    rc = pj_symbianos_poll(-1, 1000);
+	    rc = pj_symbianos_poll(-1, PJ_TIME_VAL_MSEC(timeout));
 #else
 	    rc = pj_ioqueue_poll(ioque, &timeout);
 #endif
@@ -800,8 +800,8 @@ static int bench_test(pj_bool_t allow_concur, int bufsize,
 	}
 
 	// Compare recv buffer with send buffer.
-	if (callback_read_size != bufsize ||
-	    pj_memcmp(send_buf, recv_buf, bufsize))
+	if (callback_read_size != bufsize || 
+	    pj_memcmp(send_buf, recv_buf, bufsize)) 
 	{
 	    rc = -10;
 	    PJ_LOG(3,(THIS_FILE, "   error: size/buffer mismatch"));
@@ -812,8 +812,9 @@ static int bench_test(pj_bool_t allow_concur, int bufsize,
 	do {
 	    pj_time_val timeout = { 0, 10 };
 #ifdef PJ_SYMBIAN
+	    PJ_UNUSED_ARG(timeout);
 	    rc = pj_symbianos_poll(-1, 100);
-#else
+#else	    
 	    rc = pj_ioqueue_poll(ioque, &timeout);
 #endif
 	} while (rc>0);
@@ -829,11 +830,11 @@ static int bench_test(pj_bool_t allow_concur, int bufsize,
 	tzero.u32.hi = tzero.u32.lo = 0;
 	usec_delay = pj_elapsed_usec( &tzero, &t_elapsed);
 
-	PJ_LOG(3, (THIS_FILE, "...%10d %15d  % 9d",
+	PJ_LOG(3, (THIS_FILE, "...%10d %15d  % 9d", 
 	           bufsize, inactive_sock_count, usec_delay));
 
     } else {
-	PJ_LOG(2, (THIS_FILE, "...ERROR rc=%d (buf:%d, fds:%d)",
+	PJ_LOG(2, (THIS_FILE, "...ERROR rc=%d (buf:%d, fds:%d)", 
 			      rc, bufsize, inactive_sock_count+2));
     }
 
@@ -851,14 +852,14 @@ static int bench_test(pj_bool_t allow_concur, int bufsize,
     return rc;
 
 on_error:
-    PJ_LOG(1,(THIS_FILE, "...ERROR: %s",
+    PJ_LOG(1,(THIS_FILE, "...ERROR: %s", 
 	      pj_strerror(pj_get_netos_error(), errbuf, sizeof(errbuf))));
     if (ssock)
 	pj_sock_close(ssock);
     if (csock)
 	pj_sock_close(csock);
-    for (i=0; i<inactive_sock_count && inactive_sock &&
-	      inactive_sock[i]!=PJ_INVALID_SOCKET; ++i)
+    for (i=0; i<inactive_sock_count && inactive_sock && 
+	      inactive_sock[i]!=PJ_INVALID_SOCKET; ++i) 
     {
 	pj_sock_close(inactive_sock[i]);
     }
@@ -893,7 +894,7 @@ static int udp_ioqueue_test_imp(pj_bool_t allow_concur)
     if ((status=many_handles_test(allow_concur)) != 0) {
 	return status;
     }
-
+    
     //return 0;
 
     PJ_LOG(4, (THIS_FILE, "...benchmarking different buffer size:"));
@@ -915,9 +916,9 @@ static int udp_ioqueue_test_imp(pj_bool_t allow_concur)
     }
 //pass2:
     bufsize = 512;
-    for (sock_count=SOCK_INACTIVE_MIN+2;
-	 sock_count<=SOCK_INACTIVE_MAX+2;
-	 sock_count *= 2)
+    for (sock_count=SOCK_INACTIVE_MIN+2; 
+	 sock_count<=SOCK_INACTIVE_MAX+2; 
+	 sock_count *= 2) 
     {
 	//PJ_LOG(3,(THIS_FILE, "...testing with %d fds", sock_count));
 	if ((status=bench_test(allow_concur, bufsize, sock_count-2)) != 0)
@@ -943,7 +944,9 @@ int udp_ioqueue_test()
 
 #else
 /* To prevent warning about "translation unit is empty"
- * when this test is disabled.
+ * when this test is disabled. 
  */
 int dummy_uiq_udp;
 #endif	/* INCLUDE_UDP_IOQUEUE_TEST */
+
+
