@@ -1,5 +1,5 @@
-/* $Id: symb_vas_dev.cpp 3841 2011-10-24 09:28:13Z ming $ */
-/*
+/* $Id: symb_vas_dev.cpp 4243 2012-08-31 11:42:17Z nanang $ */
+/* 
  * Copyright (C) 2009-2011 Teluu Inc. (http://www.teluu.com)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjmedia-audiodev/audiodev_imp.h>
 #include <pjmedia-audiodev/errno.h>
@@ -40,10 +40,10 @@
 #include <VoIPILBCDecoderIntfc.h>
 #include <VoIPILBCEncoderIntfc.h>
 
-/* AMR helper */
+/* AMR helper */  
 #include <pjmedia-codec/amr_helper.h>
 
-/* Pack/unpack G.729 frame of S60 DSP codec, taken from:
+/* Pack/unpack G.729 frame of S60 DSP codec, taken from:  
  * http://wiki.forum.nokia.com/index.php/TSS000776_-_Payload_conversion_for_G.729_audio_format
  */
 #include "s60_g729_bitstream.h"
@@ -88,7 +88,7 @@ struct vas_stream
 {
     // Base
     pjmedia_aud_stream	 base;			/**< Base class.	*/
-
+    
     // Pool
     pj_pool_t		*pool;			/**< Memory pool.       */
 
@@ -111,7 +111,7 @@ struct vas_stream
     pj_uint16_t		 rec_buf_len;		/**< Record buffer length. */
     void                *strm_data;		/**< Stream data.	*/
 
-    /* Resampling is needed, in case audio device is opened with clock rate
+    /* Resampling is needed, in case audio device is opened with clock rate 
      * other than 8kHz (only for PCM format).
      */
     pjmedia_resample	*play_resample;		/**< Resampler for playback. */
@@ -132,7 +132,7 @@ static pj_status_t factory_init(pjmedia_aud_dev_factory *f);
 static pj_status_t factory_destroy(pjmedia_aud_dev_factory *f);
 static pj_status_t factory_refresh(pjmedia_aud_dev_factory *f);
 static unsigned    factory_get_dev_count(pjmedia_aud_dev_factory *f);
-static pj_status_t factory_get_dev_info(pjmedia_aud_dev_factory *f,
+static pj_status_t factory_get_dev_info(pjmedia_aud_dev_factory *f, 
 					unsigned index,
 					pjmedia_aud_dev_info *info);
 static pj_status_t factory_default_param(pjmedia_aud_dev_factory *f,
@@ -170,7 +170,7 @@ static pjmedia_aud_dev_factory_op factory_op =
     &factory_refresh
 };
 
-static pjmedia_aud_stream_op stream_op =
+static pjmedia_aud_stream_op stream_op = 
 {
     &stream_get_param,
     &stream_get_cap,
@@ -238,17 +238,17 @@ public:
     void Stop();
 
     TInt ActivateSpeaker(TBool active);
-
+    
     TInt SetVolume(TInt vol) { return iVoIPDnlink->SetVolume(vol); }
     TInt GetVolume() { TInt vol;iVoIPDnlink->GetVolume(vol);return vol; }
     TInt GetMaxVolume() { TInt vol;iVoIPDnlink->GetMaxVolume(vol);return vol; }
-
+    
     TInt SetGain(TInt gain) { return iVoIPUplink->SetGain(gain); }
     TInt GetGain() { TInt gain;iVoIPUplink->GetGain(gain);return gain; }
     TInt GetMaxGain() { TInt gain;iVoIPUplink->GetMaxGain(gain);return gain; }
 
     TBool IsStarted();
-
+    
 private:
     CPjAudioEngine(struct vas_stream *parent_strm,
 		   PjAudioCallback rec_cb,
@@ -322,14 +322,14 @@ void CPjAudioEngine::ConstructL()
     User::LeaveIfError(err);
 
     if (parentStrm_->param.dir != PJMEDIA_DIR_CAPTURE) {
-	err = iFactory->CreateDownlinkStream(ver,
+	err = iFactory->CreateDownlinkStream(ver, 
 					     CVoIPUtilityFactory::EVoIPCall,
 					     iVoIPDnlink);
 	User::LeaveIfError(err);
     }
 
     if (parentStrm_->param.dir != PJMEDIA_DIR_PLAYBACK) {
-	err = iFactory->CreateUplinkStream(ver,
+	err = iFactory->CreateUplinkStream(ver, 
 					   CVoIPUtilityFactory::EVoIPCall,
 					   iVoIPUplink);
 	User::LeaveIfError(err);
@@ -359,10 +359,10 @@ CPjAudioEngine::CPjAudioEngine(struct vas_stream *parent_strm,
 CPjAudioEngine::~CPjAudioEngine()
 {
     Stop();
-
+    
     if (iVoIPUplink)
 	iVoIPUplink->Close();
-
+    
     if (iVoIPDnlink)
 	iVoIPDnlink->Close();
 
@@ -371,15 +371,15 @@ CPjAudioEngine::~CPjAudioEngine()
     delete iVoIPDnlink;
     delete iVoIPUplink;
     delete iFactory;
-
+    
     TRACE_((THIS_FILE, "Sound device destroyed"));
 }
 
 TBool CPjAudioEngine::IsStarted()
 {
-    return ((((parentStrm_->param.dir & PJMEDIA_DIR_CAPTURE) == 0) ||
+    return ((((parentStrm_->param.dir & PJMEDIA_DIR_CAPTURE) == 0) || 
 	       up_state_ == STATE_STREAMING) &&
-	    (((parentStrm_->param.dir & PJMEDIA_DIR_PLAYBACK) == 0) ||
+	    (((parentStrm_->param.dir & PJMEDIA_DIR_PLAYBACK) == 0) || 
 	       dn_state_ == STATE_STREAMING));
 }
 
@@ -394,7 +394,7 @@ TInt CPjAudioEngine::InitPlay()
     err = iVoIPDnlink->SetFormat(setting_.format, dec_fmt_if);
     if (err != KErrNone)
 	return err;
-
+    
     err = dec_fmt_if->SetObserver(*this);
     if (err != KErrNone)
 	return err;
@@ -405,7 +405,7 @@ TInt CPjAudioEngine::InitPlay()
 TInt CPjAudioEngine::InitRec()
 {
     TInt err;
-
+    
     pj_assert(iVoIPUplink);
 
     delete enc_fmt_if;
@@ -413,18 +413,18 @@ TInt CPjAudioEngine::InitRec()
     err = iVoIPUplink->SetFormat(setting_.format, enc_fmt_if);
     if (err != KErrNone)
 	return err;
-
+    
     err = enc_fmt_if->SetObserver(*this);
     if (err != KErrNone)
 	return err;
-
+    
     return iVoIPUplink->Open(*this);
 }
 
 TInt CPjAudioEngine::StartPlay()
 {
     TInt err = KErrNone;
-
+    
     pj_assert(iVoIPDnlink);
     pj_assert(dn_state_ == STATE_READY);
 
@@ -438,7 +438,7 @@ TInt CPjAudioEngine::StartPlay()
 				      setting_.mode);
 	}
 	break;
-
+	
     case EILBC:
 	{
 	    CVoIPILBCDecoderIntfc *ilbcdec_if = (CVoIPILBCDecoderIntfc*)
@@ -452,14 +452,14 @@ TInt CPjAudioEngine::StartPlay()
 	/* Ticket #1008: AMR playback issue on few devices, e.g: E72, E52 */
 	err = dec_fmt_if->SetFrameMode(ETrue);
 	break;
-
+	
     default:
 	break;
     }
 
     if (err != KErrNone)
 	goto on_return;
-
+    
     /* Configure audio routing */
     ActivateSpeaker(setting_.loudspk);
 
@@ -480,7 +480,7 @@ on_return:
 TInt CPjAudioEngine::StartRec()
 {
     TInt err = KErrNone;
-
+    
     pj_assert(iVoIPUplink);
     pj_assert(up_state_ == STATE_READY);
 
@@ -503,21 +503,21 @@ TInt CPjAudioEngine::StartRec()
 				      setting_.mode);
 	}
 	break;
-
+	
     case EAMR_NB:
 	err = enc_fmt_if->SetBitRate(setting_.mode);
 	break;
-
+	
     default:
 	break;
     }
-
+    
     if (err != KErrNone)
 	goto on_return;
-
+    
     /* Configure general codec setting */
     enc_fmt_if->SetVAD(setting_.vad);
-
+    
     /* Start recorder */
     err = iVoIPUplink->Start();
 
@@ -535,7 +535,7 @@ on_return:
 TInt CPjAudioEngine::Start()
 {
     TInt err = KErrNone;
-
+    
     if (iVoIPDnlink) {
 	switch(dn_state_) {
 	case STATE_READY:
@@ -551,7 +551,7 @@ TInt CPjAudioEngine::Start()
 	    break;
 	}
     }
-
+    
     if (iVoIPUplink) {
 	switch(up_state_) {
 	case STATE_READY:
@@ -606,14 +606,14 @@ void CPjAudioEngine::Stop()
 TInt CPjAudioEngine::ActivateSpeaker(TBool active)
 {
     TInt err = KErrNotSupported;
-
+    
     if (iVoIPDnlink) {
 	err = iVoIPDnlink->SetAudioDevice(active?
 				    CVoIPAudioDownlinkStream::ELoudSpeaker :
 				    CVoIPAudioDownlinkStream::EHandset);
 	TRACE_((THIS_FILE, "Loudspeaker turned %s", (active? "on":"off")));
     }
-
+    
     return err;
 }
 
@@ -677,7 +677,7 @@ void CPjAudioEngine::Event(const CVoIPAudioUplinkStream& /*aSrc*/,
 
 	    up_state_ = STATE_READY;
 	    TRACE_((THIS_FILE, "Uplink opened"));
-
+	    
 	    if (last_state == STATE_STARTING)
 		StartRec();
 	}
@@ -698,7 +698,7 @@ void CPjAudioEngine::Event(const CVoIPAudioUplinkStream& /*aSrc*/,
 }
 
 // Callback from MVoIPFormatObserver
-void CPjAudioEngine::Event(const CVoIPFormatIntfc& /*aSrc*/,
+void CPjAudioEngine::Event(const CVoIPFormatIntfc& /*aSrc*/, 
 			   TInt aEventType)
 {
     snd_perror("Format event", aEventType);
@@ -719,13 +719,13 @@ static void RecCbPcm2(CVoIPDataBuffer *buf, void *user_data)
 
     /* Get the buffer */
     buf->GetPayloadPtr(buffer);
-
+    
     /* Call parent callback */
     p_buf = (pj_int16_t*) buffer.Ptr();
     buf_len = buffer.Length() >> 1;
     while (buf_len) {
 	unsigned req;
-
+	
 	req = strm->param.samples_per_frame - strm->rec_buf_len;
 	if (req > buf_len)
 	    req = buf_len;
@@ -733,7 +733,7 @@ static void RecCbPcm2(CVoIPDataBuffer *buf, void *user_data)
 	p_buf += req;
 	buf_len -= req;
 	strm->rec_buf_len += req;
-
+	
 	if (strm->rec_buf_len >= strm->param.samples_per_frame) {
 	    pjmedia_frame f;
 
@@ -760,7 +760,7 @@ static void PlayCbPcm2(CVoIPDataBuffer *buf, void *user_data)
     f.size = strm->param.samples_per_frame << 1;
     strm->play_cb(strm->user_data, &f);
     if (f.type != PJMEDIA_FRAME_TYPE_AUDIO) {
-	pjmedia_zero_samples((pj_int16_t*)f.buf,
+	pjmedia_zero_samples((pj_int16_t*)f.buf, 
 			     strm->param.samples_per_frame);
     }
     f.size = strm->param.samples_per_frame << 1;
@@ -782,7 +782,7 @@ static void RecCbPcm(CVoIPDataBuffer *buf, void *user_data)
 
     /* Get the buffer */
     buf->GetPayloadPtr(buffer);
-
+    
     /* Buffer has to contain normal speech. */
     pj_assert(buffer[0] == 1 && buffer[1] == 0);
 
@@ -820,20 +820,20 @@ static void RecCbPcm(CVoIPDataBuffer *buf, void *user_data)
 	samples_processed += samples_to_process;
 
 	/* Buffer is full, time to call parent callback */
-	if (strm->rec_buf_len == strm->param.samples_per_frame /
+	if (strm->rec_buf_len == strm->param.samples_per_frame / 
 				 strm->param.channel_count /
-				 strm->resample_factor)
+				 strm->resample_factor) 
 	{
 	    pjmedia_frame f;
 
 	    /* Need to resample clock rate? */
 	    if (strm->rec_resample) {
 		unsigned resampled = 0;
-
+		
 		while (resampled < strm->rec_buf_len) {
-		    pjmedia_resample_run(strm->rec_resample,
+		    pjmedia_resample_run(strm->rec_resample, 
 				&strm->rec_buf[resampled],
-				strm->pcm_buf +
+				strm->pcm_buf + 
 				resampled * strm->resample_factor);
 		    resampled += 80;
 		}
@@ -885,13 +885,13 @@ static void PlayCbPcm(CVoIPDataBuffer *buf, void *user_data)
      * encode the PCM samples into G.711 and put it into VAS buffer.
      */
     unsigned samples_processed = 0;
-
+    
     while (samples_processed < g711_frame_len) {
 	/* Need more samples to play, time to call parent callback */
 	if (strm->play_buf_len == 0) {
 	    pjmedia_frame f;
 	    unsigned samples_got;
-
+	    
 	    f.size = strm->param.samples_per_frame << 1;
 	    if (strm->play_resample || strm->param.channel_count != 1)
 		f.buf = strm->pcm_buf;
@@ -901,11 +901,11 @@ static void PlayCbPcm(CVoIPDataBuffer *buf, void *user_data)
 	    /* Call parent callback */
 	    strm->play_cb(strm->user_data, &f);
 	    if (f.type != PJMEDIA_FRAME_TYPE_AUDIO) {
-		pjmedia_zero_samples((pj_int16_t*)f.buf,
+		pjmedia_zero_samples((pj_int16_t*)f.buf, 
 				     strm->param.samples_per_frame);
 	    }
-
-	    samples_got = strm->param.samples_per_frame /
+	    
+	    samples_got = strm->param.samples_per_frame / 
 			  strm->param.channel_count /
 			  strm->resample_factor;
 
@@ -922,17 +922,17 @@ static void PlayCbPcm(CVoIPDataBuffer *buf, void *user_data)
 	    /* Need to resample clock rate? */
 	    if (strm->play_resample) {
 		unsigned resampled = 0;
-
-		while (resampled < samples_got)
+		
+		while (resampled < samples_got) 
 		{
-		    pjmedia_resample_run(strm->play_resample,
-				strm->pcm_buf +
+		    pjmedia_resample_run(strm->play_resample, 
+				strm->pcm_buf + 
 				resampled * strm->resample_factor,
 				&strm->play_buf[resampled]);
 		    resampled += 80;
 		}
 	    }
-
+	    
 	    strm->play_buf_len = samples_got;
 	    strm->play_buf_start = 0;
 	}
@@ -965,13 +965,13 @@ static void RecCb(CVoIPDataBuffer *buf, void *user_data)
 
     /* Get the buffer */
     buf->GetPayloadPtr(buffer);
-
+    
     switch(strm->param.ext_fmt.id) {
     case PJMEDIA_FORMAT_AMR:
 	{
 	    const pj_uint8_t *p = (const pj_uint8_t*)buffer.Ptr() + 1;
 	    unsigned len = buffer.Length() - 1;
-
+	    
 	    pjmedia_frame_ext_append_subframe(frame, p, len << 3, 160);
 	    if (frame->samples_cnt == strm->param.samples_per_frame) {
 		frame->base.type = PJMEDIA_FRAME_TYPE_EXTENDED;
@@ -981,7 +981,7 @@ static void RecCb(CVoIPDataBuffer *buf, void *user_data)
 	    }
 	}
 	break;
-
+	
     case PJMEDIA_FORMAT_G729:
 	{
 	    /* Check if we got a normal or SID frame. */
@@ -989,19 +989,19 @@ static void RecCb(CVoIPDataBuffer *buf, void *user_data)
 		enum { NORMAL_LEN = 22, SID_LEN = 8 };
 		TBitStream *bitstream = (TBitStream*)strm->strm_data;
 		unsigned src_len = buffer.Length()- 2;
-
+		
 		pj_assert(src_len == NORMAL_LEN || src_len == SID_LEN);
-
+		
 		const TDesC8& p = bitstream->CompressG729Frame(
-					    buffer.Right(src_len),
+					    buffer.Right(src_len), 
 					    src_len == SID_LEN);
-
-		pjmedia_frame_ext_append_subframe(frame, p.Ptr(),
+		
+		pjmedia_frame_ext_append_subframe(frame, p.Ptr(), 
 						  p.Length() << 3, 80);
 	    } else { /* We got null frame. */
 		pjmedia_frame_ext_append_subframe(frame, NULL, 0, 80);
 	    }
-
+	    
 	    if (frame->samples_cnt == strm->param.samples_per_frame) {
 		frame->base.type = PJMEDIA_FRAME_TYPE_EXTENDED;
 		strm->rec_cb(strm->user_data, (pjmedia_frame*)frame);
@@ -1014,20 +1014,21 @@ static void RecCb(CVoIPDataBuffer *buf, void *user_data)
     case PJMEDIA_FORMAT_ILBC:
 	{
 	    unsigned samples_got;
-
-	    samples_got = strm->param.ext_fmt.bitrate == 15200? 160 : 240;
-
+	    
+	    samples_got =
+	        strm->param.ext_fmt.det.aud.avg_bps == 15200? 160 : 240;
+	    
 	    /* Check if we got a normal or SID frame. */
 	    if (buffer[0] != 0) {
 		const pj_uint8_t *p = (const pj_uint8_t*)buffer.Ptr() + 2;
 		unsigned len = buffer.Length() - 2;
-
+		
 		pjmedia_frame_ext_append_subframe(frame, p, len << 3,
 						  samples_got);
 	    } else { /* We got null frame. */
 		pjmedia_frame_ext_append_subframe(frame, NULL, 0, samples_got);
 	    }
-
+	    
 	    if (frame->samples_cnt == strm->param.samples_per_frame) {
 		frame->base.type = PJMEDIA_FRAME_TYPE_EXTENDED;
 		strm->rec_cb(strm->user_data, (pjmedia_frame*)frame);
@@ -1036,16 +1037,16 @@ static void RecCb(CVoIPDataBuffer *buf, void *user_data)
 	    }
 	}
 	break;
-
+	
     case PJMEDIA_FORMAT_PCMU:
     case PJMEDIA_FORMAT_PCMA:
 	{
 	    unsigned samples_processed = 0;
-
+	    
 	    /* Make sure it is normal frame. */
 	    pj_assert(buffer[0] == 1 && buffer[1] == 0);
 
-	    /* Detect the recorder G.711 frame size, player frame size will
+	    /* Detect the recorder G.711 frame size, player frame size will 
 	     * follow this recorder frame size.
 	     */
 	    if (vas_g711_frame_len == 0) {
@@ -1053,22 +1054,22 @@ static void RecCb(CVoIPDataBuffer *buf, void *user_data)
 		TRACE_((THIS_FILE, "Detected VAS G.711 frame size = %u samples",
 			vas_g711_frame_len));
 	    }
-
-	    /* Convert VAS buffer format into pjmedia_frame_ext. Whenever
-	     * samples count in the frame is equal to stream's samples per
+	    
+	    /* Convert VAS buffer format into pjmedia_frame_ext. Whenever 
+	     * samples count in the frame is equal to stream's samples per 
 	     * frame, call parent stream callback.
 	     */
 	    while (samples_processed < vas_g711_frame_len) {
 		unsigned tmp;
 		const pj_uint8_t *pb = (const pj_uint8_t*)buffer.Ptr() +
 				       2 + samples_processed;
-
+    
 		tmp = PJ_MIN(strm->param.samples_per_frame - frame->samples_cnt,
 			     vas_g711_frame_len - samples_processed);
-
+		
 		pjmedia_frame_ext_append_subframe(frame, pb, tmp << 3, tmp);
 		samples_processed += tmp;
-
+    
 		if (frame->samples_cnt == strm->param.samples_per_frame) {
 		    frame->base.type = PJMEDIA_FRAME_TYPE_EXTENDED;
 		    strm->rec_cb(strm->user_data, (pjmedia_frame*)frame);
@@ -1078,7 +1079,7 @@ static void RecCb(CVoIPDataBuffer *buf, void *user_data)
 	    }
 	}
 	break;
-
+	
     default:
 	break;
     }
@@ -1106,16 +1107,16 @@ static void PlayCb(CVoIPDataBuffer *buf, void *user_data)
 			  frame->base.type==PJMEDIA_FRAME_TYPE_NONE);
 	    }
 
-	    if (frame->base.type == PJMEDIA_FRAME_TYPE_EXTENDED) {
+	    if (frame->base.type == PJMEDIA_FRAME_TYPE_EXTENDED) { 
 		pjmedia_frame_ext_subframe *sf;
 		unsigned samples_cnt;
-
+		
 		sf = pjmedia_frame_ext_get_subframe(frame, 0);
 		samples_cnt = frame->samples_cnt / frame->subframe_cnt;
-
+		
 		if (sf->data && sf->bitlen) {
 		    /* AMR header for VAS is one byte, the format (may be!):
-		     * 0xxxxy00, where xxxx:frame type, y:not sure.
+		     * 0xxxxy00, where xxxx:frame type, y:not sure. 
 		     */
 		    unsigned len = (sf->bitlen+7)>>3;
 		    enum {SID_FT = 8 };
@@ -1123,10 +1124,10 @@ static void PlayCb(CVoIPDataBuffer *buf, void *user_data)
 
 		    if (len >= pjmedia_codec_amrnb_framelen[0])
 			ft = pjmedia_codec_amr_get_mode2(PJ_TRUE, len);
-
+		    
 		    amr_header |= ft << 3;
 		    buffer.Append(amr_header);
-
+		    
 		    buffer.Append((TUint8*)sf->data, len);
 		} else {
 		    enum {NO_DATA_FT = 15 };
@@ -1136,19 +1137,19 @@ static void PlayCb(CVoIPDataBuffer *buf, void *user_data)
 		}
 
 		pjmedia_frame_ext_pop_subframes(frame, 1);
-
+	    
 	    } else { /* PJMEDIA_FRAME_TYPE_NONE */
 		enum {NO_DATA_FT = 15 };
 		pj_uint8_t amr_header = 4 | (NO_DATA_FT << 3);
 
 		buffer.Append(amr_header);
-
+		
 		frame->samples_cnt = 0;
 		frame->subframe_cnt = 0;
 	    }
 	}
 	break;
-
+	
     case PJMEDIA_FORMAT_G729:
 	{
 	    if (frame->samples_cnt == 0) {
@@ -1158,20 +1159,20 @@ static void PlayCb(CVoIPDataBuffer *buf, void *user_data)
 			  frame->base.type==PJMEDIA_FRAME_TYPE_NONE);
 	    }
 
-	    if (frame->base.type == PJMEDIA_FRAME_TYPE_EXTENDED) {
+	    if (frame->base.type == PJMEDIA_FRAME_TYPE_EXTENDED) { 
 		pjmedia_frame_ext_subframe *sf;
 		unsigned samples_cnt;
-
+		
 		sf = pjmedia_frame_ext_get_subframe(frame, 0);
 		samples_cnt = frame->samples_cnt / frame->subframe_cnt;
-
+		
 		if (sf->data && sf->bitlen) {
 		    enum { NORMAL_LEN = 10, SID_LEN = 2 };
 		    pj_bool_t sid_frame = ((sf->bitlen >> 3) == SID_LEN);
 		    TBitStream *bitstream = (TBitStream*)strm->strm_data;
 		    const TPtrC8 src(sf->data, sf->bitlen>>3);
 		    const TDesC8 &dst = bitstream->ExpandG729Frame(src,
-								   sid_frame);
+								   sid_frame); 
 		    if (sid_frame) {
 			buffer.Append(2);
 			buffer.Append(0);
@@ -1188,16 +1189,16 @@ static void PlayCb(CVoIPDataBuffer *buf, void *user_data)
 		}
 
 		pjmedia_frame_ext_pop_subframes(frame, 1);
-
+	    
 	    } else { /* PJMEDIA_FRAME_TYPE_NONE */
 		buffer.Append(2);
 		buffer.Append(0);
-
+		
 		buffer.AppendFill(0, 22);
 	    }
 	}
 	break;
-
+	
     case PJMEDIA_FORMAT_ILBC:
 	{
 	    if (frame->samples_cnt == 0) {
@@ -1207,63 +1208,65 @@ static void PlayCb(CVoIPDataBuffer *buf, void *user_data)
 			  frame->base.type==PJMEDIA_FRAME_TYPE_NONE);
 	    }
 
-	    if (frame->base.type == PJMEDIA_FRAME_TYPE_EXTENDED) {
+	    if (frame->base.type == PJMEDIA_FRAME_TYPE_EXTENDED) { 
 		pjmedia_frame_ext_subframe *sf;
 		unsigned samples_cnt;
-
+		
 		sf = pjmedia_frame_ext_get_subframe(frame, 0);
 		samples_cnt = frame->samples_cnt / frame->subframe_cnt;
-
-		pj_assert((strm->param.ext_fmt.bitrate == 15200 &&
+		
+		pj_assert((strm->param.ext_fmt.det.aud.avg_bps == 15200 && 
 			   samples_cnt == 160) ||
-			  (strm->param.ext_fmt.bitrate != 15200 &&
+			  (strm->param.ext_fmt.det.aud.avg_bps != 15200 &&
 			   samples_cnt == 240));
-
+		
 		if (sf->data && sf->bitlen) {
 		    buffer.Append(1);
 		    buffer.Append(0);
 		    buffer.Append((TUint8*)sf->data, sf->bitlen>>3);
 		} else {
 		    unsigned frame_len;
-
+		    
 		    buffer.Append(1);
 		    buffer.Append(0);
-
+		    
 		    /* VAS iLBC frame is 20ms or 30ms */
-		    frame_len = strm->param.ext_fmt.bitrate == 15200? 38 : 50;
+		    frame_len =
+		        strm->param.ext_fmt.det.aud.avg_bps == 15200? 38 : 50;
 		    buffer.AppendFill(0, frame_len);
 		}
 
 		pjmedia_frame_ext_pop_subframes(frame, 1);
-
+	    
 	    } else { /* PJMEDIA_FRAME_TYPE_NONE */
-
+		
 		unsigned frame_len;
-
+		
 		buffer.Append(1);
 		buffer.Append(0);
-
+		
 		/* VAS iLBC frame is 20ms or 30ms */
-		frame_len = strm->param.ext_fmt.bitrate == 15200? 38 : 50;
+		frame_len =
+		    strm->param.ext_fmt.det.aud.avg_bps == 15200? 38 : 50;
 		buffer.AppendFill(0, frame_len);
 
 	    }
 	}
 	break;
-
+	
     case PJMEDIA_FORMAT_PCMU:
     case PJMEDIA_FORMAT_PCMA:
 	{
 	    unsigned samples_ready = 0;
 	    unsigned samples_req = vas_g711_frame_len;
-
+	    
 	    /* Assume frame size is 10ms if frame size hasn't been known. */
 	    if (samples_req == 0)
 		samples_req = 80;
-
+	    
 	    buffer.Append(1);
 	    buffer.Append(0);
-
+	    
 	    /* Call parent stream callback to get samples to play. */
 	    while (samples_ready < samples_req) {
 		if (frame->samples_cnt == 0) {
@@ -1272,11 +1275,11 @@ static void PlayCb(CVoIPDataBuffer *buf, void *user_data)
 		    pj_assert(frame->base.type==PJMEDIA_FRAME_TYPE_EXTENDED ||
 			      frame->base.type==PJMEDIA_FRAME_TYPE_NONE);
 		}
-
-		if (frame->base.type == PJMEDIA_FRAME_TYPE_EXTENDED) {
+    
+		if (frame->base.type == PJMEDIA_FRAME_TYPE_EXTENDED) { 
 		    pjmedia_frame_ext_subframe *sf;
 		    unsigned samples_cnt;
-
+		    
 		    sf = pjmedia_frame_ext_get_subframe(frame, 0);
 		    samples_cnt = frame->samples_cnt / frame->subframe_cnt;
 		    if (sf->data && sf->bitlen) {
@@ -1288,12 +1291,12 @@ static void PlayCb(CVoIPDataBuffer *buf, void *user_data)
 			buffer.AppendFill(silc, samples_cnt);
 		    }
 		    samples_ready += samples_cnt;
-
+		    
 		    pjmedia_frame_ext_pop_subframes(frame, 1);
-
+		
 		} else { /* PJMEDIA_FRAME_TYPE_NONE */
 		    pj_uint8_t silc;
-
+		    
 		    silc = (strm->param.ext_fmt.id==PJMEDIA_FORMAT_PCMU)?
 			    pjmedia_linear2ulaw(0) : pjmedia_linear2alaw(0);
 		    buffer.AppendFill(silc, samples_req - samples_ready);
@@ -1305,7 +1308,7 @@ static void PlayCb(CVoIPDataBuffer *buf, void *user_data)
 	    }
 	}
 	break;
-
+	
     default:
 	break;
     }
@@ -1363,7 +1366,7 @@ static pj_status_t factory_init(pjmedia_aud_dev_factory *f)
 			PJMEDIA_AUD_DEV_CAP_OUTPUT_ROUTE |
 			PJMEDIA_AUD_DEV_CAP_VAD |
 			PJMEDIA_AUD_DEV_CAP_CNG;
-    af->dev_info.routes = PJMEDIA_AUD_DEV_ROUTE_EARPIECE |
+    af->dev_info.routes = PJMEDIA_AUD_DEV_ROUTE_EARPIECE | 
 			  PJMEDIA_AUD_DEV_ROUTE_LOUDSPEAKER;
     af->dev_info.input_count = 1;
     af->dev_info.output_count = 1;
@@ -1374,21 +1377,21 @@ static pj_status_t factory_init(pjmedia_aud_dev_factory *f)
     if (err != KErrNone)
 	goto on_error;
 
-    /* On VAS 2.0, uplink & downlink stream should be instantiated before
+    /* On VAS 2.0, uplink & downlink stream should be instantiated before 
      * querying formats.
      */
-    err = vas_factory_->CreateUplinkStream(vas_version,
+    err = vas_factory_->CreateUplinkStream(vas_version, 
 				          CVoIPUtilityFactory::EVoIPCall,
 				          vas_uplink);
     if (err != KErrNone)
 	goto on_error;
-
-    err = vas_factory_->CreateDownlinkStream(vas_version,
+    
+    err = vas_factory_->CreateDownlinkStream(vas_version, 
 				            CVoIPUtilityFactory::EVoIPCall,
 				            vas_dnlink);
     if (err != KErrNone)
 	goto on_error;
-
+    
     uplink_formats.Reset();
     err = vas_factory_->GetSupportedUplinkFormats(uplink_formats);
     if (err != KErrNone)
@@ -1406,60 +1409,67 @@ static pj_status_t factory_init(pjmedia_aud_dev_factory *f)
     vas_dnlink = NULL;
     delete vas_factory_;
     vas_factory_ = NULL;
-
+    
     for (TInt i = 0; i < dnlink_formats.Count(); i++) {
+	pjmedia_format ext_fmt;
+	
 	/* Format must be supported by both downlink & uplink. */
 	if (uplink_formats.Find(dnlink_formats[i]) == KErrNotFound)
 	    continue;
-
+	
 	switch (dnlink_formats[i]) {
 	case EAMR_NB:
-	    af->dev_info.ext_fmt[ext_fmt_cnt].id = PJMEDIA_FORMAT_AMR;
-	    af->dev_info.ext_fmt[ext_fmt_cnt].bitrate = 7400;
-	    af->dev_info.ext_fmt[ext_fmt_cnt].vad = PJ_TRUE;
+	    pjmedia_format_init_audio(&ext_fmt, PJMEDIA_FORMAT_AMR,
+				      8000, 1, 16, 20, 7400, 12200);
+	    af->dev_info.ext_fmt[ext_fmt_cnt] = ext_fmt;
+	    //af->dev_info.ext_fmt[ext_fmt_cnt].vad = PJ_TRUE;
 	    break;
 
 	case EG729:
-	    af->dev_info.ext_fmt[ext_fmt_cnt].id = PJMEDIA_FORMAT_G729;
-	    af->dev_info.ext_fmt[ext_fmt_cnt].bitrate = 8000;
-	    af->dev_info.ext_fmt[ext_fmt_cnt].vad = PJ_FALSE;
+	    pjmedia_format_init_audio(&ext_fmt, PJMEDIA_FORMAT_G729,
+				      8000, 1, 16, 20, 8000, 8000);
+	    af->dev_info.ext_fmt[ext_fmt_cnt] = ext_fmt;
+	    //af->dev_info.ext_fmt[ext_fmt_cnt].vad = PJ_FALSE;
 	    break;
 
 	case EILBC:
-	    af->dev_info.ext_fmt[ext_fmt_cnt].id = PJMEDIA_FORMAT_ILBC;
-	    af->dev_info.ext_fmt[ext_fmt_cnt].bitrate = 13333;
-	    af->dev_info.ext_fmt[ext_fmt_cnt].vad = PJ_TRUE;
+	    pjmedia_format_init_audio(&ext_fmt, PJMEDIA_FORMAT_ILBC,
+				      8000, 1, 16, 30, 13333, 15200);
+	    af->dev_info.ext_fmt[ext_fmt_cnt] = ext_fmt;
+	    //af->dev_info.ext_fmt[ext_fmt_cnt].vad = PJ_TRUE;
 	    break;
 
 	case EG711:
 #if PJMEDIA_AUDIO_DEV_SYMB_VAS_VERSION==2
 	case EG711_10MS:
 #endif
-	    af->dev_info.ext_fmt[ext_fmt_cnt].id = PJMEDIA_FORMAT_PCMU;
-	    af->dev_info.ext_fmt[ext_fmt_cnt].bitrate = 64000;
-	    af->dev_info.ext_fmt[ext_fmt_cnt].vad = PJ_FALSE;
+	    pjmedia_format_init_audio(&ext_fmt, PJMEDIA_FORMAT_PCMU,
+				      8000, 1, 16, 20, 64000, 64000);
+	    af->dev_info.ext_fmt[ext_fmt_cnt] = ext_fmt;
+	    //af->dev_info.ext_fmt[ext_fmt_cnt].vad = PJ_FALSE;
 	    ++ext_fmt_cnt;
-	    af->dev_info.ext_fmt[ext_fmt_cnt].id = PJMEDIA_FORMAT_PCMA;
-	    af->dev_info.ext_fmt[ext_fmt_cnt].bitrate = 64000;
-	    af->dev_info.ext_fmt[ext_fmt_cnt].vad = PJ_FALSE;
+	    pjmedia_format_init_audio(&ext_fmt, PJMEDIA_FORMAT_PCMA,
+				      8000, 1, 16, 20, 64000, 64000);
+	    af->dev_info.ext_fmt[ext_fmt_cnt] = ext_fmt;
+	    //af->dev_info.ext_fmt[ext_fmt_cnt].vad = PJ_FALSE;
 	    break;
-
+	
 	default:
 	    continue;
 	}
-
+	
 	++ext_fmt_cnt;
     }
-
+    
     af->dev_info.ext_fmt_cnt = ext_fmt_cnt;
 
     uplink_formats.Close();
     dnlink_formats.Close();
-
+    
     PJ_LOG(3, (THIS_FILE, "VAS initialized"));
 
     return PJ_SUCCESS;
-
+    
 on_error:
     return PJ_RETURN_OS_ERROR(err);
 }
@@ -1474,7 +1484,7 @@ static pj_status_t factory_destroy(pjmedia_aud_dev_factory *f)
     pj_pool_release(pool);
 
     PJ_LOG(3, (THIS_FILE, "VAS destroyed"));
-
+    
     return PJ_SUCCESS;
 }
 
@@ -1493,7 +1503,7 @@ static unsigned factory_get_dev_count(pjmedia_aud_dev_factory *f)
 }
 
 /* API: get device info */
-static pj_status_t factory_get_dev_info(pjmedia_aud_dev_factory *f,
+static pj_status_t factory_get_dev_info(pjmedia_aud_dev_factory *f, 
 					unsigned index,
 					pjmedia_aud_dev_info *info)
 {
@@ -1550,19 +1560,19 @@ static pj_status_t factory_create_stream(pjmedia_aud_dev_factory *f,
     PJ_ASSERT_RETURN(param->bits_per_sample == BITS_PER_SAMPLE, PJ_EINVAL);
 
     /* Supported clock rates:
-     * - for non-PCM format: 8kHz
-     * - for PCM format: 8kHz and 16kHz
+     * - for non-PCM format: 8kHz  
+     * - for PCM format: 8kHz and 16kHz  
      */
     PJ_ASSERT_RETURN(param->clock_rate == 8000 ||
-		     (param->clock_rate == 16000 &&
+		     (param->clock_rate == 16000 && 
 		      param->ext_fmt.id == PJMEDIA_FORMAT_L16),
 		     PJ_EINVAL);
 
     /* Supported channels number:
      * - for non-PCM format: mono
-     * - for PCM format: mono and stereo
+     * - for PCM format: mono and stereo  
      */
-    PJ_ASSERT_RETURN(param->channel_count == 1 ||
+    PJ_ASSERT_RETURN(param->channel_count == 1 || 
 		     (param->channel_count == 2 &&
 		      param->ext_fmt.id == PJMEDIA_FORMAT_L16),
 		     PJ_EINVAL);
@@ -1577,11 +1587,11 @@ static pj_status_t factory_create_stream(pjmedia_aud_dev_factory *f,
 
     if (strm->param.flags & PJMEDIA_AUD_DEV_CAP_EXT_FORMAT == 0)
 	strm->param.ext_fmt.id = PJMEDIA_FORMAT_L16;
-
+	
     /* Set audio engine fourcc. */
     switch(strm->param.ext_fmt.id) {
     case PJMEDIA_FORMAT_L16:
-#ifdef USE_NATIVE_PCM
+#ifdef USE_NATIVE_PCM	
 	vas_setting.format = EPCM16;
 #else
 	vas_setting.format = EG711;
@@ -1608,16 +1618,16 @@ static pj_status_t factory_create_stream(pjmedia_aud_dev_factory *f,
     /* Set audio engine mode. */
     if (strm->param.ext_fmt.id == PJMEDIA_FORMAT_L16)
     {
-#ifdef USE_NATIVE_PCM
+#ifdef USE_NATIVE_PCM	
 	vas_setting.mode = 0;
 #else
 	vas_setting.mode = CVoIPFormatIntfc::EG711uLaw;
 #endif
-    }
+    } 
     else if (strm->param.ext_fmt.id == PJMEDIA_FORMAT_AMR)
     {
-	vas_setting.mode = strm->param.ext_fmt.bitrate;
-    }
+	vas_setting.mode = strm->param.ext_fmt.det.aud.avg_bps;
+    } 
     else if (strm->param.ext_fmt.id == PJMEDIA_FORMAT_PCMU)
     {
 	vas_setting.mode = CVoIPFormatIntfc::EG711uLaw;
@@ -1628,7 +1638,7 @@ static pj_status_t factory_create_stream(pjmedia_aud_dev_factory *f,
     }
     else if (strm->param.ext_fmt.id == PJMEDIA_FORMAT_ILBC)
     {
-	if (strm->param.ext_fmt.bitrate == 15200)
+	if (strm->param.ext_fmt.det.aud.avg_bps == 15200)
 	    vas_setting.mode = CVoIPFormatIntfc::EiLBC20mSecFrame;
 	else
 	    vas_setting.mode = CVoIPFormatIntfc::EiLBC30mSecFrame;
@@ -1636,7 +1646,7 @@ static pj_status_t factory_create_stream(pjmedia_aud_dev_factory *f,
 	vas_setting.mode = 0;
     }
 
-    /* Disable VAD on L16, G711, iLBC, and also G729 (G729's SID
+    /* Disable VAD on L16, G711, iLBC, and also G729 (G729's SID 
      * potentially cause noise?).
      */
     if (strm->param.ext_fmt.id == PJMEDIA_FORMAT_PCMU ||
@@ -1647,13 +1657,15 @@ static pj_status_t factory_create_stream(pjmedia_aud_dev_factory *f,
     {
 	vas_setting.vad = EFalse;
     } else {
-	vas_setting.vad = strm->param.ext_fmt.vad;
+	vas_setting.vad = (strm->param.flags & PJMEDIA_AUD_DEV_CAP_VAD) &&
+			  strm->param.vad_enabled;
     }
-
+    
     /* Set other audio engine attributes. */
-    vas_setting.plc = strm->param.plc_enabled;
+    vas_setting.plc = (strm->param.flags & PJMEDIA_AUD_DEV_CAP_PLC) &&
+		      strm->param.plc_enabled;
     vas_setting.cng = vas_setting.vad;
-    vas_setting.loudspk =
+    vas_setting.loudspk = 
 		strm->param.output_route==PJMEDIA_AUD_DEV_ROUTE_LOUDSPEAKER;
 
     /* Set audio engine callbacks. */
@@ -1677,8 +1689,8 @@ static pj_status_t factory_create_stream(pjmedia_aud_dev_factory *f,
 
     /* play_buf size is samples per frame scaled in to 8kHz mono. */
     strm->play_buf = (pj_int16_t*)pj_pool_zalloc(
-					pool,
-					(strm->param.samples_per_frame /
+					pool, 
+					(strm->param.samples_per_frame / 
 					strm->resample_factor /
 					strm->param.channel_count) << 1);
     strm->play_buf_len = 0;
@@ -1686,28 +1698,28 @@ static pj_status_t factory_create_stream(pjmedia_aud_dev_factory *f,
 
     /* rec_buf size is samples per frame scaled in to 8kHz mono. */
     strm->rec_buf  = (pj_int16_t*)pj_pool_zalloc(
-					pool,
-					(strm->param.samples_per_frame /
+					pool, 
+					(strm->param.samples_per_frame / 
 					strm->resample_factor /
 					strm->param.channel_count) << 1);
     strm->rec_buf_len = 0;
 
     if (strm->param.ext_fmt.id == PJMEDIA_FORMAT_G729) {
 	TBitStream *g729_bitstream = new TBitStream;
-
+	
 	PJ_ASSERT_RETURN(g729_bitstream, PJ_ENOMEM);
 	strm->strm_data = (void*)g729_bitstream;
     }
-
+	
     /* Init resampler when format is PCM and clock rate is not 8kHz */
-    if (strm->param.clock_rate != 8000 &&
+    if (strm->param.clock_rate != 8000 && 
 	strm->param.ext_fmt.id == PJMEDIA_FORMAT_L16)
     {
 	pj_status_t status;
-
+	
 	if (strm->param.dir & PJMEDIA_DIR_CAPTURE) {
 	    /* Create resample for recorder */
-	    status = pjmedia_resample_create( pool, PJ_TRUE, PJ_FALSE, 1,
+	    status = pjmedia_resample_create( pool, PJ_TRUE, PJ_FALSE, 1, 
 					      8000,
 					      strm->param.clock_rate,
 					      80,
@@ -1715,10 +1727,10 @@ static pj_status_t factory_create_stream(pjmedia_aud_dev_factory *f,
 	    if (status != PJ_SUCCESS)
 		return status;
 	}
-
+    
 	if (strm->param.dir & PJMEDIA_DIR_PLAYBACK) {
 	    /* Create resample for player */
-	    status = pjmedia_resample_create( pool, PJ_TRUE, PJ_FALSE, 1,
+	    status = pjmedia_resample_create( pool, PJ_TRUE, PJ_FALSE, 1, 
 					      strm->param.clock_rate,
 					      8000,
 					      80 * strm->resample_factor,
@@ -1730,19 +1742,19 @@ static pj_status_t factory_create_stream(pjmedia_aud_dev_factory *f,
 
     /* Create PCM buffer, when the clock rate is not 8kHz or not mono */
     if (strm->param.ext_fmt.id == PJMEDIA_FORMAT_L16 &&
-	(strm->resample_factor > 1 || strm->param.channel_count != 1))
+	(strm->resample_factor > 1 || strm->param.channel_count != 1)) 
     {
-	strm->pcm_buf = (pj_int16_t*)pj_pool_zalloc(pool,
+	strm->pcm_buf = (pj_int16_t*)pj_pool_zalloc(pool, 
 					strm->param.samples_per_frame << 1);
     }
 
-
+    
     /* Create the audio engine. */
     TRAPD(err, strm->engine = CPjAudioEngine::NewL(strm,
 						   vas_rec_cb, vas_play_cb,
 						   strm, vas_setting));
     if (err != KErrNone) {
-	pj_pool_release(pool);
+    	pj_pool_release(pool);
 	return PJ_RETURN_OS_ERROR(err);
     }
 
@@ -1769,7 +1781,7 @@ static pj_status_t stream_get_param(pjmedia_aud_stream *s,
     {
 	pi->flags |= PJMEDIA_AUD_DEV_CAP_OUTPUT_VOLUME_SETTING;
     }
-
+    
     return PJ_SUCCESS;
 }
 
@@ -1784,24 +1796,24 @@ static pj_status_t stream_get_cap(pjmedia_aud_stream *s,
     PJ_ASSERT_RETURN(s && pval, PJ_EINVAL);
 
     switch (cap) {
-    case PJMEDIA_AUD_DEV_CAP_OUTPUT_ROUTE:
+    case PJMEDIA_AUD_DEV_CAP_OUTPUT_ROUTE: 
 	if (strm->param.dir & PJMEDIA_DIR_PLAYBACK) {
 	    *(pjmedia_aud_dev_route*)pval = strm->param.output_route;
 	    status = PJ_SUCCESS;
 	}
 	break;
-
-    /* There is a case that GetMaxGain() stucks, e.g: in N95. */
+    
+    /* There is a case that GetMaxGain() stucks, e.g: in N95. */ 
     /*
     case PJMEDIA_AUD_DEV_CAP_INPUT_VOLUME_SETTING:
 	if (strm->param.dir & PJMEDIA_DIR_CAPTURE) {
 	    PJ_ASSERT_RETURN(strm->engine, PJ_EINVAL);
-
+	    
 	    TInt max_gain = strm->engine->GetMaxGain();
 	    TInt gain = strm->engine->GetGain();
-
+	    
 	    if (max_gain > 0 && gain >= 0) {
-		*(unsigned*)pval = gain * 100 / max_gain;
+		*(unsigned*)pval = gain * 100 / max_gain; 
 		status = PJ_SUCCESS;
 	    } else {
 		status = PJMEDIA_EAUD_NOTREADY;
@@ -1813,12 +1825,12 @@ static pj_status_t stream_get_cap(pjmedia_aud_stream *s,
     case PJMEDIA_AUD_DEV_CAP_OUTPUT_VOLUME_SETTING:
 	if (strm->param.dir & PJMEDIA_DIR_PLAYBACK) {
 	    PJ_ASSERT_RETURN(strm->engine, PJ_EINVAL);
-
+	    
 	    TInt max_vol = strm->engine->GetMaxVolume();
 	    TInt vol = strm->engine->GetVolume();
-
+	    
 	    if (max_vol > 0 && vol >= 0) {
-		*(unsigned*)pval = vol * 100 / max_vol;
+		*(unsigned*)pval = vol * 100 / max_vol; 
 		status = PJ_SUCCESS;
 	    } else {
 		status = PJMEDIA_EAUD_NOTREADY;
@@ -1828,7 +1840,7 @@ static pj_status_t stream_get_cap(pjmedia_aud_stream *s,
     default:
 	break;
     }
-
+    
     return status;
 }
 
@@ -1843,13 +1855,13 @@ static pj_status_t stream_set_cap(pjmedia_aud_stream *s,
     PJ_ASSERT_RETURN(s && pval, PJ_EINVAL);
 
     switch (cap) {
-    case PJMEDIA_AUD_DEV_CAP_OUTPUT_ROUTE:
+    case PJMEDIA_AUD_DEV_CAP_OUTPUT_ROUTE: 
 	if (strm->param.dir & PJMEDIA_DIR_PLAYBACK) {
 	    pjmedia_aud_dev_route r = *(const pjmedia_aud_dev_route*)pval;
 	    TInt err;
 
 	    PJ_ASSERT_RETURN(strm->engine, PJ_EINVAL);
-
+	    
 	    switch (r) {
 	    case PJMEDIA_AUD_DEV_ROUTE_DEFAULT:
 	    case PJMEDIA_AUD_DEV_ROUTE_EARPIECE:
@@ -1865,20 +1877,20 @@ static pj_status_t stream_set_cap(pjmedia_aud_stream *s,
 		break;
 	    }
 	    if (status == PJ_SUCCESS)
-		strm->param.output_route = r;
+		strm->param.output_route = r; 
 	}
 	break;
 
-    /* There is a case that GetMaxGain() stucks, e.g: in N95. */
+    /* There is a case that GetMaxGain() stucks, e.g: in N95. */ 
     /*
     case PJMEDIA_AUD_DEV_CAP_INPUT_VOLUME_SETTING:
 	if (strm->param.dir & PJMEDIA_DIR_CAPTURE) {
 	    PJ_ASSERT_RETURN(strm->engine, PJ_EINVAL);
-
+	    
 	    TInt max_gain = strm->engine->GetMaxGain();
 	    if (max_gain > 0) {
 		TInt gain, err;
-
+		
 		gain = *(unsigned*)pval * max_gain / 100;
 		err = strm->engine->SetGain(gain);
 		status = (err==KErrNone)? PJ_SUCCESS:PJ_RETURN_OS_ERROR(err);
@@ -1894,11 +1906,11 @@ static pj_status_t stream_set_cap(pjmedia_aud_stream *s,
     case PJMEDIA_AUD_DEV_CAP_OUTPUT_VOLUME_SETTING:
 	if (strm->param.dir & PJMEDIA_DIR_PLAYBACK) {
 	    PJ_ASSERT_RETURN(strm->engine, PJ_EINVAL);
-
+	    
 	    TInt max_vol = strm->engine->GetMaxVolume();
 	    if (max_vol > 0) {
 		TInt vol, err;
-
+		
 		vol = *(unsigned*)pval * max_vol / 100;
 		err = strm->engine->SetVolume(vol);
 		status = (err==KErrNone)? PJ_SUCCESS:PJ_RETURN_OS_ERROR(err);
@@ -1912,7 +1924,7 @@ static pj_status_t stream_set_cap(pjmedia_aud_stream *s,
     default:
 	break;
     }
-
+    
     return status;
 }
 
@@ -1927,26 +1939,26 @@ static pj_status_t stream_start(pjmedia_aud_stream *strm)
 	enum { VAS_WAIT_START = 2000 }; /* in msecs */
 	TTime start, now;
 	TInt err = stream->engine->Start();
+	
+    	if (err != KErrNone)
+    	    return PJ_RETURN_OS_ERROR(err);
 
-	if (err != KErrNone)
-	    return PJ_RETURN_OS_ERROR(err);
-
-	/* Perform synchronous start, timeout after VAS_WAIT_START ms */
+    	/* Perform synchronous start, timeout after VAS_WAIT_START ms */
 	start.UniversalTime();
 	do {
-	    pj_symbianos_poll(-1, 100);
-	    now.UniversalTime();
-	} while (!stream->engine->IsStarted() &&
+    	    pj_symbianos_poll(-1, 100);
+    	    now.UniversalTime();
+    	} while (!stream->engine->IsStarted() &&
 		 (now.MicroSecondsFrom(start) < VAS_WAIT_START * 1000));
-
+	
 	if (stream->engine->IsStarted()) {
-
+	    
 	    /* Apply output volume setting if specified */
-	    if (stream->param.flags &
-		PJMEDIA_AUD_DEV_CAP_OUTPUT_VOLUME_SETTING)
+	    if (stream->param.flags & 
+		PJMEDIA_AUD_DEV_CAP_OUTPUT_VOLUME_SETTING) 
 	    {
 		stream_set_cap(strm,
-			       PJMEDIA_AUD_DEV_CAP_OUTPUT_VOLUME_SETTING,
+			       PJMEDIA_AUD_DEV_CAP_OUTPUT_VOLUME_SETTING, 
 			       &stream->param.output_vol);
 	    }
 
@@ -1954,7 +1966,7 @@ static pj_status_t stream_start(pjmedia_aud_stream *strm)
 	} else {
 	    return PJ_ETIMEDOUT;
 	}
-    }
+    }    
 
     return PJ_EINVALIDOP;
 }
@@ -1967,7 +1979,7 @@ static pj_status_t stream_stop(pjmedia_aud_stream *strm)
     PJ_ASSERT_RETURN(stream, PJ_EINVAL);
 
     if (stream->engine) {
-	stream->engine->Stop();
+    	stream->engine->Stop();
     }
 
     return PJ_SUCCESS;
@@ -1995,11 +2007,12 @@ static pj_status_t stream_destroy(pjmedia_aud_stream *strm)
     pj_pool_t *pool;
     pool = stream->pool;
     if (pool) {
-	stream->pool = NULL;
-	pj_pool_release(pool);
+    	stream->pool = NULL;
+    	pj_pool_release(pool);
     }
 
     return PJ_SUCCESS;
 }
 
 #endif // PJMEDIA_AUDIO_DEV_HAS_SYMB_VAS
+
