@@ -1,5 +1,5 @@
-/* $Id: ffmpeg_vid_codecs.c 4089 2012-04-26 07:27:06Z nanang $ */
-/*
+/* $Id: ffmpeg_vid_codecs.c 4311 2012-12-20 06:45:09Z nanang $ */
+/* 
  * Copyright (C) 2010-2011 Teluu Inc. (http://www.teluu.com)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjmedia-codec/ffmpeg_vid_codecs.h>
 #include <pjmedia-codec/h263_packetizer.h>
@@ -31,7 +31,7 @@
 
 
 /*
- * Only build this file if PJMEDIA_HAS_FFMPEG_VID_CODEC != 0 and
+ * Only build this file if PJMEDIA_HAS_FFMPEG_VID_CODEC != 0 and 
  * PJMEDIA_HAS_VIDEO != 0
  */
 #if defined(PJMEDIA_HAS_FFMPEG_VID_CODEC) && \
@@ -41,7 +41,7 @@
 #define THIS_FILE   "ffmpeg_vid_codecs.c"
 
 #define LIBAVCODEC_VER_AT_LEAST(major,minor)  (LIBAVCODEC_VERSION_MAJOR > major || \
-					       (LIBAVCODEC_VERSION_MAJOR == major && \
+     					       (LIBAVCODEC_VERSION_MAJOR == major && \
 					        LIBAVCODEC_VERSION_MINOR >= minor))
 
 #include "../pjmedia/ffmpeg_util.h"
@@ -62,11 +62,16 @@
 #endif
 
 #if LIBAVCODEC_VER_AT_LEAST(53,61)
-/* Not sure when AVCodec::encode2 is introduced. It appears in
- * libavcodec 53.61 where some codecs actually still use AVCodec::encode
- * (e.g: H263, H264).
- */
-#  define AVCODEC_HAS_ENCODE(c)		(c->encode || c->encode2)
+#  if LIBAVCODEC_VER_AT_LEAST(54,59)
+   /* Not sure when AVCodec::encode is obsoleted/removed. */
+#      define AVCODEC_HAS_ENCODE(c)	(c->encode2)
+#  else
+   /* Not sure when AVCodec::encode2 is introduced. It appears in 
+    * libavcodec 53.61 where some codecs actually still use AVCodec::encode
+    * (e.g: H263, H264).
+    */
+#      define AVCODEC_HAS_ENCODE(c)	(c->encode || c->encode2)
+#  endif
 #  define AV_OPT_SET(obj,name,val,opt)	(av_opt_set(obj,name,val,opt)==0)
 #  define AV_OPT_SET_INT(obj,name,val)	(av_opt_set_int(obj,name,val,0)==0)
 #else
@@ -78,27 +83,27 @@
 
 
 /* Prototypes for FFMPEG codecs factory */
-static pj_status_t ffmpeg_test_alloc( pjmedia_vid_codec_factory *factory,
+static pj_status_t ffmpeg_test_alloc( pjmedia_vid_codec_factory *factory, 
 				      const pjmedia_vid_codec_info *id );
-static pj_status_t ffmpeg_default_attr( pjmedia_vid_codec_factory *factory,
-				        const pjmedia_vid_codec_info *info,
+static pj_status_t ffmpeg_default_attr( pjmedia_vid_codec_factory *factory, 
+				        const pjmedia_vid_codec_info *info, 
 				        pjmedia_vid_codec_param *attr );
-static pj_status_t ffmpeg_enum_codecs( pjmedia_vid_codec_factory *factory,
-				       unsigned *count,
+static pj_status_t ffmpeg_enum_codecs( pjmedia_vid_codec_factory *factory, 
+				       unsigned *count, 
 				       pjmedia_vid_codec_info codecs[]);
-static pj_status_t ffmpeg_alloc_codec( pjmedia_vid_codec_factory *factory,
-				       const pjmedia_vid_codec_info *info,
+static pj_status_t ffmpeg_alloc_codec( pjmedia_vid_codec_factory *factory, 
+				       const pjmedia_vid_codec_info *info, 
 				       pjmedia_vid_codec **p_codec);
-static pj_status_t ffmpeg_dealloc_codec( pjmedia_vid_codec_factory *factory,
+static pj_status_t ffmpeg_dealloc_codec( pjmedia_vid_codec_factory *factory, 
 				         pjmedia_vid_codec *codec );
 
 /* Prototypes for FFMPEG codecs implementation. */
-static pj_status_t  ffmpeg_codec_init( pjmedia_vid_codec *codec,
+static pj_status_t  ffmpeg_codec_init( pjmedia_vid_codec *codec, 
 				       pj_pool_t *pool );
-static pj_status_t  ffmpeg_codec_open( pjmedia_vid_codec *codec,
+static pj_status_t  ffmpeg_codec_open( pjmedia_vid_codec *codec, 
 				       pjmedia_vid_codec_param *attr );
 static pj_status_t  ffmpeg_codec_close( pjmedia_vid_codec *codec );
-static pj_status_t  ffmpeg_codec_modify(pjmedia_vid_codec *codec,
+static pj_status_t  ffmpeg_codec_modify(pjmedia_vid_codec *codec, 
 				        const pjmedia_vid_codec_param *attr );
 static pj_status_t  ffmpeg_codec_get_param(pjmedia_vid_codec *codec,
 					   pjmedia_vid_codec_param *param);
@@ -119,7 +124,7 @@ static pj_status_t ffmpeg_codec_decode( pjmedia_vid_codec *codec,
 					pjmedia_frame *output);
 
 /* Definition for FFMPEG codecs operations. */
-static pjmedia_vid_codec_op ffmpeg_op =
+static pjmedia_vid_codec_op ffmpeg_op = 
 {
     &ffmpeg_codec_init,
     &ffmpeg_codec_open,
@@ -178,7 +183,7 @@ typedef struct ffmpeg_private
     unsigned     		     enc_processed;
     void			    *dec_buf;
     unsigned			     dec_buf_size;
-    pj_timestamp		     last_dec_keyframe_ts;
+    pj_timestamp		     last_dec_keyframe_ts; 
 
     /* The ffmpeg codec states. */
     AVCodec			    *enc;
@@ -190,10 +195,10 @@ typedef struct ffmpeg_private
      * may be needed for post-decoding.
      */
     enum PixelFormat		     expected_dec_fmt;
-						/**< Expected output format of
+						/**< Expected output format of 
 						     ffmpeg decoder	    */
 
-    void			    *data;	/**< Codec specific data    */
+    void			    *data;	/**< Codec specific data    */		    
 } ffmpeg_private;
 
 
@@ -287,7 +292,7 @@ static ffmpeg_codec_desc codec_desc[] =
 	&h264_packetize, &h264_unpacketize, &h264_preopen, &h264_postopen,
 	&pjmedia_vid_codec_h264_match_sdp,
 	/* Leading space for better compatibility (strange indeed!) */
-	{2, { {{"profile-level-id",16},    {"42e01e",6}},
+	{2, { {{"profile-level-id",16},    {"42e01e",6}}, 
 	      {{" packetization-mode",19},  {"1",1}}, } },
     },
 #endif
@@ -298,7 +303,7 @@ static ffmpeg_codec_desc codec_desc[] =
 	PJMEDIA_FORMAT_H263,
 	{352, 288},	{15, 1},	256000, 256000,
 	&h263_packetize, &h263_unpacketize, &h263_preopen, NULL, NULL,
-	{2, { {{"CIF",3},   {"1",1}},
+	{2, { {{"CIF",3},   {"1",1}}, 
 	      {{"QCIF",4},  {"1",1}}, } },
     },
 #endif
@@ -380,7 +385,7 @@ static pj_status_t h264_preopen(ffmpeg_private *ff)
 	AVCodecContext *ctx = ff->enc_ctx;
 	const char *profile = NULL;
 
-	vfd = pjmedia_format_get_video_format_detail(&ff->param.enc_fmt,
+	vfd = pjmedia_format_get_video_format_detail(&ff->param.enc_fmt, 
 						     PJ_TRUE);
 
 	/* Override generic params after applying SDP fmtp */
@@ -515,7 +520,7 @@ static pj_status_t h263_preopen(ffmpeg_private *ff)
 	pjmedia_video_format_detail *vfd;
 	AVCodecContext *ctx = ff->enc_ctx;
 
-	vfd = pjmedia_format_get_video_format_detail(&ff->param.enc_fmt,
+	vfd = pjmedia_format_get_video_format_detail(&ff->param.enc_fmt, 
 						     PJ_TRUE);
 
 	/* Override generic params after applying SDP fmtp */
@@ -609,7 +614,7 @@ PJ_DEF(pj_status_t) pjmedia_codec_ffmpeg_vid_init(pjmedia_vid_codec_mgr *mgr,
 	return PJ_ENOMEM;
 
     /* Create mutex. */
-    status = pj_mutex_create_simple(pool, "ffmpeg codec factory",
+    status = pj_mutex_create_simple(pool, "ffmpeg codec factory", 
 				    &ffmpeg_factory.mutex);
     if (status != PJ_SUCCESS)
 	goto on_error;
@@ -617,7 +622,7 @@ PJ_DEF(pj_status_t) pjmedia_codec_ffmpeg_vid_init(pjmedia_vid_codec_mgr *mgr,
     pjmedia_ffmpeg_add_ref();
 #if !LIBAVCODEC_VER_AT_LEAST(53,20)
     /* avcodec_init() dissappeared between version 53.20 and 54.15, not sure
-     * exactly when
+     * exactly when 
      */
     avcodec_init();
 #endif
@@ -628,7 +633,7 @@ PJ_DEF(pj_status_t) pjmedia_codec_ffmpeg_vid_init(pjmedia_vid_codec_mgr *mgr,
         ffmpeg_codec_desc *desc;
 	pjmedia_format_id fmt_id;
 	int codec_info_idx;
-
+        
 #if LIBAVCODEC_VERSION_MAJOR <= 52
 #   define AVMEDIA_TYPE_VIDEO	CODEC_TYPE_VIDEO
 #endif
@@ -647,7 +652,7 @@ PJ_DEF(pj_status_t) pjmedia_codec_ffmpeg_vid_init(pjmedia_vid_codec_mgr *mgr,
 	    continue;
 
 	codec_info_idx = find_codec_idx_by_fmt_id(fmt_id);
-	/* Skip if codec is unwanted by this wrapper (not listed in
+	/* Skip if codec is unwanted by this wrapper (not listed in 
 	 * the codec info array)
 	 */
 	if (codec_info_idx < 0)
@@ -683,7 +688,7 @@ PJ_DEF(pj_status_t) pjmedia_codec_ffmpeg_vid_init(pjmedia_vid_codec_mgr *mgr,
 			       "format %d", *p));
 		    continue;
 		}
-
+		
 		//raw_fmt[raw_fmt_cnt++] = fmt_id;
 		/* Disable some formats due to H.264 error:
 		 * x264 [error]: baseline profile doesn't support 4:4:4
@@ -711,14 +716,14 @@ PJ_DEF(pj_status_t) pjmedia_codec_ffmpeg_vid_init(pjmedia_vid_codec_mgr *mgr,
 	    }
 
 	    desc->info.dec_fmt_id_cnt = raw_fmt_cnt;
-	    pj_memcpy(desc->info.dec_fmt_id, raw_fmt,
+	    pj_memcpy(desc->info.dec_fmt_id, raw_fmt, 
 		      sizeof(raw_fmt[0])*raw_fmt_cnt);
 	}
 
 	/* Get supported framerates */
 	if (c->supported_framerates) {
 	    const AVRational *fr = c->supported_framerates;
-	    while ((fr->num != 0 || fr->den != 0) &&
+	    while ((fr->num != 0 || fr->den != 0) && 
 		   desc->info.fps_cnt < PJMEDIA_VID_CODEC_MAX_FPS_CNT)
 	    {
 		desc->info.fps[desc->info.fps_cnt].num = fr->num;
@@ -733,7 +738,7 @@ PJ_DEF(pj_status_t) pjmedia_codec_ffmpeg_vid_init(pjmedia_vid_codec_mgr *mgr,
             desc->info.dir |= PJMEDIA_DIR_ENCODING;
             desc->enc = c;
         }
-
+	
 	/* Get ffmpeg decoder instance */
         if (AVCODEC_HAS_DECODE(c) && !desc->dec) {
             desc->info.dir |= PJMEDIA_DIR_DECODING;
@@ -780,12 +785,12 @@ PJ_DEF(pj_status_t) pjmedia_codec_ffmpeg_vid_init(pjmedia_vid_codec_mgr *mgr,
 	    /* Copy description from base codec */
 	    if (!desc->info.dec_fmt_id_cnt) {
 		desc->info.dec_fmt_id_cnt = base_desc->info.dec_fmt_id_cnt;
-		pj_memcpy(desc->info.dec_fmt_id, base_desc->info.dec_fmt_id,
+		pj_memcpy(desc->info.dec_fmt_id, base_desc->info.dec_fmt_id, 
 			  sizeof(pjmedia_format_id)*desc->info.dec_fmt_id_cnt);
 	    }
 	    if (!desc->info.fps_cnt) {
 		desc->info.fps_cnt = base_desc->info.fps_cnt;
-		pj_memcpy(desc->info.fps, base_desc->info.fps,
+		pj_memcpy(desc->info.fps, base_desc->info.fps, 
 			  sizeof(desc->info.fps[0])*desc->info.fps_cnt);
 	    }
 	    if (!desc->info.clock_rate) {
@@ -841,7 +846,7 @@ PJ_DEF(pj_status_t) pjmedia_codec_ffmpeg_vid_init(pjmedia_vid_codec_mgr *mgr,
     }
 
     /* Register codec factory to codec manager. */
-    status = pjmedia_vid_codec_mgr_register_factory(mgr,
+    status = pjmedia_vid_codec_mgr_register_factory(mgr, 
 						    &ffmpeg_factory.base);
     if (status != PJ_SUCCESS)
 	goto on_error;
@@ -887,10 +892,10 @@ PJ_DEF(pj_status_t) pjmedia_codec_ffmpeg_vid_deinit(void)
 }
 
 
-/*
- * Check if factory can allocate the specified codec.
+/* 
+ * Check if factory can allocate the specified codec. 
  */
-static pj_status_t ffmpeg_test_alloc( pjmedia_vid_codec_factory *factory,
+static pj_status_t ffmpeg_test_alloc( pjmedia_vid_codec_factory *factory, 
 				      const pjmedia_vid_codec_info *info )
 {
     const ffmpeg_codec_desc *desc;
@@ -909,8 +914,8 @@ static pj_status_t ffmpeg_test_alloc( pjmedia_vid_codec_factory *factory,
 /*
  * Generate default attribute.
  */
-static pj_status_t ffmpeg_default_attr( pjmedia_vid_codec_factory *factory,
-				        const pjmedia_vid_codec_info *info,
+static pj_status_t ffmpeg_default_attr( pjmedia_vid_codec_factory *factory, 
+				        const pjmedia_vid_codec_info *info, 
 				        pjmedia_vid_codec_param *attr )
 {
     const ffmpeg_codec_desc *desc;
@@ -970,7 +975,7 @@ static pj_status_t ffmpeg_default_attr( pjmedia_vid_codec_factory *factory,
  * Enum codecs supported by this factory.
  */
 static pj_status_t ffmpeg_enum_codecs( pjmedia_vid_codec_factory *factory,
-				       unsigned *count,
+				       unsigned *count, 
 				       pjmedia_vid_codec_info codecs[])
 {
     unsigned i, max_cnt;
@@ -983,7 +988,7 @@ static pj_status_t ffmpeg_enum_codecs( pjmedia_vid_codec_factory *factory,
 
     for (i=0; i<max_cnt; ++i) {
 	if (codec_desc[i].enabled) {
-	    pj_memcpy(&codecs[*count], &codec_desc[i].info,
+	    pj_memcpy(&codecs[*count], &codec_desc[i].info, 
 		      sizeof(pjmedia_vid_codec_info));
 	    (*count)++;
 	}
@@ -995,7 +1000,7 @@ static pj_status_t ffmpeg_enum_codecs( pjmedia_vid_codec_factory *factory,
 /*
  * Allocate a new codec instance.
  */
-static pj_status_t ffmpeg_alloc_codec( pjmedia_vid_codec_factory *factory,
+static pj_status_t ffmpeg_alloc_codec( pjmedia_vid_codec_factory *factory, 
 				       const pjmedia_vid_codec_info *info,
 				       pjmedia_vid_codec **p_codec)
 {
@@ -1045,7 +1050,7 @@ on_error:
 /*
  * Free codec.
  */
-static pj_status_t ffmpeg_dealloc_codec( pjmedia_vid_codec_factory *factory,
+static pj_status_t ffmpeg_dealloc_codec( pjmedia_vid_codec_factory *factory, 
 				         pjmedia_vid_codec *codec )
 {
     ffmpeg_private *ff;
@@ -1066,7 +1071,7 @@ static pj_status_t ffmpeg_dealloc_codec( pjmedia_vid_codec_factory *factory,
 /*
  * Init codec.
  */
-static pj_status_t ffmpeg_codec_init( pjmedia_vid_codec *codec,
+static pj_status_t ffmpeg_codec_init( pjmedia_vid_codec *codec, 
 				      pj_pool_t *pool )
 {
     PJ_UNUSED_ARG(codec);
@@ -1102,7 +1107,7 @@ static pj_status_t open_ffmpeg_codec(ffmpeg_private *ff,
     ff->expected_dec_fmt = pix_fmt;
 
     /* Get video format detail for shortcut access to encoded format */
-    vfd = pjmedia_format_get_video_format_detail(&ff->param.enc_fmt,
+    vfd = pjmedia_format_get_video_format_detail(&ff->param.enc_fmt, 
 						 PJ_TRUE);
 
     /* Allocate ffmpeg codec context */
@@ -1230,7 +1235,7 @@ on_error:
 /*
  * Open codec.
  */
-static pj_status_t ffmpeg_codec_open( pjmedia_vid_codec *codec,
+static pj_status_t ffmpeg_codec_open( pjmedia_vid_codec *codec, 
 				      pjmedia_vid_codec_param *attr )
 {
     ffmpeg_private *ff;
@@ -1334,7 +1339,7 @@ static pj_status_t ffmpeg_codec_close( pjmedia_vid_codec *codec )
 /*
  * Modify codec settings.
  */
-static pj_status_t  ffmpeg_codec_modify( pjmedia_vid_codec *codec,
+static pj_status_t  ffmpeg_codec_modify( pjmedia_vid_codec *codec, 
 				         const pjmedia_vid_codec_param *attr)
 {
     ffmpeg_private *ff = (ffmpeg_private*)codec->codec_data;
@@ -1389,7 +1394,7 @@ static pj_status_t  ffmpeg_unpacketize(pjmedia_vid_codec *codec,
         return (*ff->desc->unpacketize)(ff, payload, payload_len,
                                         bits, bits_len, bits_pos);
     }
-
+    
     return PJ_ENOTSUP;
 }
 
@@ -1431,7 +1436,7 @@ static pj_status_t ffmpeg_codec_encode_whole(pjmedia_vid_codec *codec,
     avframe.pts = av_rescale_q(input->timestamp.u64, src_timebase,
 			       ff->enc_ctx->time_base);
     */
-
+    
     for (i[0] = 0; i[0] < ff->enc_vfi->plane_cnt; ++i[0]) {
         avframe.data[i[0]] = p;
         avframe.linesize[i[0]] = ff->enc_vafp.strides[i[0]];
@@ -1501,7 +1506,7 @@ static pj_status_t ffmpeg_codec_encode_begin(pjmedia_vid_codec *codec,
 	if (status != PJ_SUCCESS)
 	    return status;
 
-	ff->enc_buf_is_keyframe = (whole_frm.bit_info &
+	ff->enc_buf_is_keyframe = (whole_frm.bit_info & 
 				   PJMEDIA_VID_FRM_KEYFRAME);
 	ff->enc_frame_len = (unsigned)whole_frm.size;
 	ff->enc_processed = 0;
@@ -1586,7 +1591,7 @@ static pj_status_t check_decode_result(pjmedia_vid_codec *codec,
 	pj_status_t status;
 
 	/* Get current raw format id from ffmpeg decoder context */
-	status = PixelFormat_to_pjmedia_format_id(ff->dec_ctx->pix_fmt,
+	status = PixelFormat_to_pjmedia_format_id(ff->dec_ctx->pix_fmt, 
 						  &new_fmt_id);
 	if (status != PJ_SUCCESS)
 	    return status;
@@ -1696,7 +1701,7 @@ static pj_status_t ffmpeg_codec_decode_whole(pjmedia_vid_codec *codec,
 #endif
 
 #if LIBAVCODEC_VER_AT_LEAST(52,72)
-    err = avcodec_decode_video2(ff->dec_ctx, &avframe,
+    err = avcodec_decode_video2(ff->dec_ctx, &avframe, 
                                 &got_picture, &avpacket);
 #else
     err = avcodec_decode_video(ff->dec_ctx, &avframe,
@@ -1741,7 +1746,7 @@ static pj_status_t ffmpeg_codec_decode_whole(pjmedia_vid_codec *codec,
 	    if (avframe.linesize[i]!=vafp->strides[i]) {
 		/* Padding exists, copy line by line */
 		pj_uint8_t *q_end;
-
+                    
 		q_end = q+vafp->plane_bytes[i];
 		while(q < q_end) {
 		    pj_memcpy(q, p, vafp->strides[i]);
@@ -1761,7 +1766,7 @@ static pj_status_t ffmpeg_codec_decode_whole(pjmedia_vid_codec *codec,
 	output->type = PJMEDIA_FRAME_TYPE_NONE;
 	output->size = 0;
     }
-
+    
     return PJ_SUCCESS;
 }
 
@@ -1815,3 +1820,4 @@ static pj_status_t ffmpeg_codec_decode( pjmedia_vid_codec *codec,
 #endif
 
 #endif	/* PJMEDIA_HAS_FFMPEG_VID_CODEC */
+
