@@ -1,5 +1,5 @@
-/* $Id: endpoint.c 3999 2012-03-30 07:10:13Z bennylp $ */
-/*
+/* $Id: endpoint.c 4240 2012-08-31 09:03:36Z ming $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjmedia/endpoint.h>
 #include <pjmedia/errno.h>
@@ -46,9 +46,13 @@ static const pj_str_t STR_SENDRECV = { "sendrecv", 8 };
 
 
 /* Config to control rtpmap inclusion for static payload types */
-pj_bool_t pjmedia_add_rtpmap_for_static_pt =
+pj_bool_t pjmedia_add_rtpmap_for_static_pt = 
 	    PJMEDIA_ADD_RTPMAP_FOR_STATIC_PT;
 
+/* Config to control use of RFC3890 TIAS */
+pj_bool_t pjmedia_add_bandwidth_tias_in_sdp =
+            PJMEDIA_ADD_BANDWIDTH_TIAS_IN_SDP;
+
 
 
 /* Worker thread proc. */
@@ -146,7 +150,7 @@ PJ_DEF(pj_status_t) pjmedia_endpt_create(pj_pool_factory *pf,
 
     /* Create ioqueue if none is specified. */
     if (endpt->ioqueue == NULL) {
-
+	
 	endpt->own_ioqueue = PJ_TRUE;
 
 	status = pj_ioqueue_create( endpt->pool, PJ_IOQUEUE_MAX_HANDLES,
@@ -155,7 +159,7 @@ PJ_DEF(pj_status_t) pjmedia_endpt_create(pj_pool_factory *pf,
 	    goto on_error;
 
 	if (worker_cnt == 0) {
-	    PJ_LOG(4,(THIS_FILE, "Warning: no worker thread is created in"
+	    PJ_LOG(4,(THIS_FILE, "Warning: no worker thread is created in"  
 				 "media endpoint for internal ioqueue"));
 	}
     }
@@ -296,9 +300,9 @@ PJ_DEF(unsigned) pjmedia_endpt_get_thread_count(pjmedia_endpt *endpt)
 }
 
 /**
- * Get a reference to one of the worker threads of the media endpoint
+ * Get a reference to one of the worker threads of the media endpoint 
  */
-PJ_DEF(pj_thread_t*) pjmedia_endpt_get_thread(pjmedia_endpt *endpt,
+PJ_DEF(pj_thread_t*) pjmedia_endpt_get_thread(pjmedia_endpt *endpt, 
 					      unsigned index)
 {
     PJ_ASSERT_RETURN(endpt, NULL);
@@ -475,22 +479,22 @@ PJ_DEF(pj_status_t) pjmedia_endpt_create_audio_sdp(pjmedia_endpt *endpt,
 	    pjmedia_codec_fmtp *dec_fmtp = &codec_param.setting.dec_fmtp;
 
 	    /* Print codec PT */
-	    buf_len += pj_ansi_snprintf(buf,
-					MAX_FMTP_STR_LEN - buf_len,
-					"%d",
+	    buf_len += pj_ansi_snprintf(buf, 
+					MAX_FMTP_STR_LEN - buf_len, 
+					"%d", 
 					codec_info->pt);
 
 	    for (i = 0; i < dec_fmtp->cnt; ++i) {
 		unsigned test_len = 2;
 
 		/* Check if buf still available */
-		test_len = dec_fmtp->param[i].val.slen +
+		test_len = dec_fmtp->param[i].val.slen + 
 			   dec_fmtp->param[i].name.slen;
 		if (test_len + buf_len >= MAX_FMTP_STR_LEN)
 		    return PJ_ETOOBIG;
 
 		/* Print delimiter */
-		buf_len += pj_ansi_snprintf(&buf[buf_len],
+		buf_len += pj_ansi_snprintf(&buf[buf_len], 
 					    MAX_FMTP_STR_LEN - buf_len,
 					    (i == 0?" ":";"));
 
@@ -505,9 +509,9 @@ PJ_DEF(pj_status_t) pjmedia_endpt_create_audio_sdp(pjmedia_endpt *endpt,
 					    (int)dec_fmtp->param[i].val.slen,
 					    dec_fmtp->param[i].val.ptr);
 		else
-		    buf_len += pj_ansi_snprintf(&buf[buf_len],
+		    buf_len += pj_ansi_snprintf(&buf[buf_len], 
 					    MAX_FMTP_STR_LEN - buf_len,
-					    "%.*s",
+					    "%.*s", 
 					    (int)dec_fmtp->param[i].val.slen,
 					    dec_fmtp->param[i].val.ptr);
 	    }
@@ -551,10 +555,10 @@ PJ_DEF(pj_status_t) pjmedia_endpt_create_audio_sdp(pjmedia_endpt *endpt,
     /* Put bandwidth info in media level using bandwidth modifier "TIAS"
      * (RFC3890).
      */
-    if (max_bitrate) {
+    if (max_bitrate && pjmedia_add_bandwidth_tias_in_sdp) {
 	const pj_str_t STR_BANDW_MODIFIER = { "TIAS", 4 };
 	pjmedia_sdp_bandw *b;
-
+	    
 	b = PJ_POOL_ALLOC_T(pool, pjmedia_sdp_bandw);
 	b->modifier = STR_BANDW_MODIFIER;
 	b->value = max_bitrate;
@@ -599,7 +603,7 @@ PJ_DEF(pj_status_t) pjmedia_endpt_create_video_sdp(pjmedia_endpt *endpt,
 	return status;
 
     cnt = PJ_ARRAY_SIZE(codec_info);
-    status = pjmedia_vid_codec_mgr_enum_codecs(NULL, &cnt,
+    status = pjmedia_vid_codec_mgr_enum_codecs(NULL, &cnt, 
 					       codec_info, codec_prio);
 
     /* Check that there are not too many codecs */
@@ -661,22 +665,22 @@ PJ_DEF(pj_status_t) pjmedia_endpt_create_video_sdp(pjmedia_endpt *endpt,
 	    pjmedia_codec_fmtp *dec_fmtp = &codec_param.dec_fmtp;
 
 	    /* Print codec PT */
-	    buf_len += pj_ansi_snprintf(buf,
-					MAX_FMTP_STR_LEN - buf_len,
-					"%d",
+	    buf_len += pj_ansi_snprintf(buf, 
+					MAX_FMTP_STR_LEN - buf_len, 
+					"%d", 
 					codec_info[i].pt);
 
 	    for (j = 0; j < dec_fmtp->cnt; ++j) {
 		unsigned test_len = 2;
 
 		/* Check if buf still available */
-		test_len = dec_fmtp->param[j].val.slen +
+		test_len = dec_fmtp->param[j].val.slen + 
 			   dec_fmtp->param[j].name.slen;
 		if (test_len + buf_len >= MAX_FMTP_STR_LEN)
 		    return PJ_ETOOBIG;
 
 		/* Print delimiter */
-		buf_len += pj_ansi_snprintf(&buf[buf_len],
+		buf_len += pj_ansi_snprintf(&buf[buf_len], 
 					    MAX_FMTP_STR_LEN - buf_len,
 					    (j == 0?" ":";"));
 
@@ -691,9 +695,9 @@ PJ_DEF(pj_status_t) pjmedia_endpt_create_video_sdp(pjmedia_endpt *endpt,
 					    (int)dec_fmtp->param[j].val.slen,
 					    dec_fmtp->param[j].val.ptr);
 		else
-		    buf_len += pj_ansi_snprintf(&buf[buf_len],
+		    buf_len += pj_ansi_snprintf(&buf[buf_len], 
 					    MAX_FMTP_STR_LEN - buf_len,
-					    "%.*s",
+					    "%.*s", 
 					    (int)dec_fmtp->param[j].val.slen,
 					    dec_fmtp->param[j].val.ptr);
 	    }
@@ -704,7 +708,7 @@ PJ_DEF(pj_status_t) pjmedia_endpt_create_video_sdp(pjmedia_endpt *endpt,
 	    attr->value = pj_strdup3(pool, buf);
 	    m->attr[m->attr_count++] = attr;
 	}
-
+    
 	/* Find maximum bitrate in this media */
 	vfd = pjmedia_format_get_video_format_detail(&codec_param.enc_fmt,
 						     PJ_TRUE);
@@ -715,10 +719,10 @@ PJ_DEF(pj_status_t) pjmedia_endpt_create_video_sdp(pjmedia_endpt *endpt,
     /* Put bandwidth info in media level using bandwidth modifier "TIAS"
      * (RFC3890).
      */
-    if (max_bitrate) {
+    if (max_bitrate && pjmedia_add_bandwidth_tias_in_sdp) {
 	const pj_str_t STR_BANDW_MODIFIER = { "TIAS", 4 };
 	pjmedia_sdp_bandw *b;
-
+	    
 	b = PJ_POOL_ALLOC_T(pool, pjmedia_sdp_bandw);
 	b->modifier = STR_BANDW_MODIFIER;
 	b->value = max_bitrate;
@@ -756,19 +760,19 @@ PJ_DEF(pj_status_t) pjmedia_endpt_create_base_sdp( pjmedia_endpt *endpt,
     sdp->origin.net_type = STR_IN;
 
     if (origin->addr.sa_family == pj_AF_INET()) {
-	sdp->origin.addr_type = STR_IP4;
-	pj_strdup2(pool, &sdp->origin.addr,
-		   pj_inet_ntoa(origin->ipv4.sin_addr));
+ 	sdp->origin.addr_type = STR_IP4;
+ 	pj_strdup2(pool, &sdp->origin.addr,
+ 		   pj_inet_ntoa(origin->ipv4.sin_addr));
     } else if (origin->addr.sa_family == pj_AF_INET6()) {
-	char tmp_addr[PJ_INET6_ADDRSTRLEN];
+ 	char tmp_addr[PJ_INET6_ADDRSTRLEN];
 
-	sdp->origin.addr_type = STR_IP6;
-	pj_strdup2(pool, &sdp->origin.addr,
-		   pj_sockaddr_print(origin, tmp_addr, sizeof(tmp_addr), 0));
+ 	sdp->origin.addr_type = STR_IP6;
+ 	pj_strdup2(pool, &sdp->origin.addr,
+ 		   pj_sockaddr_print(origin, tmp_addr, sizeof(tmp_addr), 0));
 
     } else {
-	pj_assert(!"Invalid address family");
-	return PJ_EAFNOTSUP;
+ 	pj_assert(!"Invalid address family");
+ 	return PJ_EAFNOTSUP;
     }
 
     if (sess_name)
@@ -848,11 +852,11 @@ static const char *good_number(char *buf, pj_int32_t val)
     if (val < 1000) {
 	pj_ansi_sprintf(buf, "%d", val);
     } else if (val < 1000000) {
-	pj_ansi_sprintf(buf, "%d.%dK",
+	pj_ansi_sprintf(buf, "%d.%dK", 
 			val / 1000,
 			(val % 1000) / 100);
     } else {
-	pj_ansi_sprintf(buf, "%d.%02dM",
+	pj_ansi_sprintf(buf, "%d.%02dM", 
 			val / 1000000,
 			(val % 1000000) / 10000);
     }
@@ -872,7 +876,7 @@ PJ_DEF(pj_status_t) pjmedia_endpt_dump(pjmedia_endpt *endpt)
     PJ_LOG(3,(THIS_FILE, "Dumping PJMEDIA capabilities:"));
 
     count = PJ_ARRAY_SIZE(codec_info);
-    if (pjmedia_codec_mgr_enum_codecs(&endpt->codec_mgr,
+    if (pjmedia_codec_mgr_enum_codecs(&endpt->codec_mgr, 
 				      &count, codec_info, prio) != PJ_SUCCESS)
     {
 	PJ_LOG(3,(THIS_FILE, " -error: failed to enum codecs"));
@@ -901,14 +905,14 @@ PJ_DEF(pj_status_t) pjmedia_endpt_dump(pjmedia_endpt *endpt)
 	    pj_bzero(&param, sizeof(pjmedia_codec_param));
 	}
 
-	PJ_LOG(3,(THIS_FILE,
+	PJ_LOG(3,(THIS_FILE, 
 		  "   %s codec #%2d: pt=%d (%.*s @%dKHz/%d, %sbps, %dms%s%s%s%s%s)",
 		  type, i, codec_info[i].pt,
 		  (int)codec_info[i].encoding_name.slen,
 		  codec_info[i].encoding_name.ptr,
 		  codec_info[i].clock_rate/1000,
 		  codec_info[i].channel_cnt,
-		  good_number(bps, param.info.avg_bps),
+		  good_number(bps, param.info.avg_bps), 
 		  param.info.frm_ptime * param.setting.frm_per_pkt,
 		  (param.setting.vad ? " vad" : ""),
 		  (param.setting.cng ? " cng" : ""),
