@@ -1,5 +1,5 @@
-/* $Id: sdp.c 3945 2012-01-27 09:12:59Z nanang $ */
-/*
+/* $Id: sdp.c 4367 2013-02-21 20:49:19Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjmedia/sdp.h>
 #include <pjmedia/errno.h>
@@ -42,7 +42,7 @@ enum {
 #define THIS_FILE	"sdp.c"
 
 typedef struct parse_context
-{
+{ 
     pj_status_t last_error;
 } parse_context;
 
@@ -85,7 +85,7 @@ static void init_sdp_parser(void)
 	pj_leave_critical_section();
 	return;
     }
-
+    
     pj_cis_buf_init(&cis_buf);
 
     pj_cis_init(&cis_buf, &cs_token);
@@ -121,11 +121,11 @@ PJ_DEF(pjmedia_sdp_attr*) pjmedia_sdp_attr_create( pj_pool_t *pool,
     return attr;
 }
 
-PJ_DEF(pjmedia_sdp_attr*) pjmedia_sdp_attr_clone(pj_pool_t *pool,
+PJ_DEF(pjmedia_sdp_attr*) pjmedia_sdp_attr_clone(pj_pool_t *pool, 
 						 const pjmedia_sdp_attr *rhs)
 {
     pjmedia_sdp_attr *attr;
-
+    
     PJ_ASSERT_RETURN(pool && rhs, NULL);
 
     attr = PJ_POOL_ALLOC_T(pool, pjmedia_sdp_attr);
@@ -136,7 +136,7 @@ PJ_DEF(pjmedia_sdp_attr*) pjmedia_sdp_attr_clone(pj_pool_t *pool,
     return attr;
 }
 
-PJ_DEF(pjmedia_sdp_attr*) pjmedia_sdp_attr_find (unsigned count,
+PJ_DEF(pjmedia_sdp_attr*) pjmedia_sdp_attr_find (unsigned count, 
 				       pjmedia_sdp_attr *const attr_array[],
 				       const pj_str_t *name,
 				       const pj_str_t *c_fmt)
@@ -155,14 +155,14 @@ PJ_DEF(pjmedia_sdp_attr*) pjmedia_sdp_attr_find (unsigned count,
 		if (pt == c_pt) {
 		    return (pjmedia_sdp_attr*)a;
 		}
-	    } else
+	    } else 
 		return (pjmedia_sdp_attr*)a;
 	}
     }
     return NULL;
 }
 
-PJ_DEF(pjmedia_sdp_attr*) pjmedia_sdp_attr_find2(unsigned count,
+PJ_DEF(pjmedia_sdp_attr*) pjmedia_sdp_attr_find2(unsigned count, 
 				       pjmedia_sdp_attr *const attr_array[],
 				       const char *c_name,
 				       const pj_str_t *c_fmt)
@@ -211,7 +211,7 @@ PJ_DEF(unsigned) pjmedia_sdp_attr_remove_all(unsigned *count,
 	    ++removed;
 	} else {
 	    ++i;
-	}
+	}   
     }
 
     return removed;
@@ -433,18 +433,18 @@ PJ_DEF(pjmedia_sdp_attr*) pjmedia_sdp_attr_create_rtcp(pj_pool_t *pool,
     attr->name = pj_str("rtcp");
     attr->value.ptr = (char*) pj_pool_alloc(pool, ATTR_LEN);
     if (a->addr.sa_family == pj_AF_INET()) {
-	attr->value.slen =
+	attr->value.slen = 
 	    pj_ansi_snprintf(attr->value.ptr, ATTR_LEN,
 			    "%u IN IP4 %s",
 			    pj_ntohs(a->ipv4.sin_port),
 			    pj_inet_ntoa(a->ipv4.sin_addr));
     } else if (a->addr.sa_family == pj_AF_INET6()) {
 	char tmp_addr[PJ_INET6_ADDRSTRLEN];
-	attr->value.slen =
+	attr->value.slen = 
 	    pj_ansi_snprintf(attr->value.ptr, ATTR_LEN,
 			    "%u IN IP6 %s",
 			    pj_sockaddr_get_port(a),
-			    pj_sockaddr_print(a, tmp_addr,
+			    pj_sockaddr_print(a, tmp_addr, 
 					      sizeof(tmp_addr), 0));
 
     } else {
@@ -492,7 +492,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_rtpmap_to_attr(pj_pool_t *pool,
     attr->name.slen = 6;
 
     /* Format: ":pt enc_name/clock_rate[/param]" */
-    len = pj_ansi_snprintf(tempbuf, sizeof(tempbuf),
+    len = pj_ansi_snprintf(tempbuf, sizeof(tempbuf), 
 			   "%.*s %.*s/%u%s%.*s",
 			   (int)rtpmap->pt.slen,
 			   rtpmap->pt.ptr,
@@ -533,7 +533,7 @@ static int print_connection_info( pjmedia_sdp_conn *c, char *buf, int len)
 }
 
 
-PJ_DEF(pjmedia_sdp_conn*) pjmedia_sdp_conn_clone (pj_pool_t *pool,
+PJ_DEF(pjmedia_sdp_conn*) pjmedia_sdp_conn_clone (pj_pool_t *pool, 
 						  const pjmedia_sdp_conn *rhs)
 {
     pjmedia_sdp_conn *c = PJ_POOL_ALLOC_T(pool, pjmedia_sdp_conn);
@@ -547,7 +547,7 @@ PJ_DEF(pjmedia_sdp_conn*) pjmedia_sdp_conn_clone (pj_pool_t *pool,
 }
 
 PJ_DEF(pjmedia_sdp_bandw*)
-pjmedia_sdp_bandw_clone (pj_pool_t *pool,
+pjmedia_sdp_bandw_clone (pj_pool_t *pool, 
 			 const pjmedia_sdp_bandw *rhs)
 {
     pjmedia_sdp_bandw *b = PJ_POOL_ALLOC_T(pool, pjmedia_sdp_bandw);
@@ -579,7 +579,7 @@ static pj_ssize_t print_bandw(const pjmedia_sdp_bandw *bandw,
     return p-buf;
 }
 
-static pj_ssize_t print_attr(const pjmedia_sdp_attr *attr,
+static pj_ssize_t print_attr(const pjmedia_sdp_attr *attr, 
 			     char *buf, pj_size_t len)
 {
     char *p = buf;
@@ -591,7 +591,7 @@ static pj_ssize_t print_attr(const pjmedia_sdp_attr *attr,
     *p++ = '=';
     pj_memcpy(p, attr->name.ptr, attr->name.slen);
     p += attr->name.slen;
-
+    
 
     if (attr->value.slen) {
 	*p++ = ':';
@@ -646,7 +646,7 @@ static int print_media_desc( pjmedia_sdp_media *m, char *buf, int len)
 	}
 	p += printed;
     }
-
+    
     /* print optional bandwidth info. */
     for (i=0; i<m->bandw_count; ++i) {
 	printed = print_bandw(m->bandw[i], p, end-p);
@@ -669,7 +669,7 @@ static int print_media_desc( pjmedia_sdp_media *m, char *buf, int len)
 }
 
 PJ_DEF(pjmedia_sdp_media*) pjmedia_sdp_media_clone(
-						 pj_pool_t *pool,
+						 pj_pool_t *pool, 
 						 const pjmedia_sdp_media *rhs)
 {
     unsigned int i;
@@ -744,12 +744,12 @@ PJ_DEF(unsigned) pjmedia_sdp_media_remove_all_attr(pjmedia_sdp_media *m,
 }
 
 PJ_DEF(pj_status_t) pjmedia_sdp_media_remove_attr(pjmedia_sdp_media *m,
-						  pjmedia_sdp_attr *attr)
+			      			  pjmedia_sdp_attr *attr)
 {
     return pjmedia_sdp_attr_remove(&m->attr_count, m->attr, attr);
 }
 
-static int print_session(const pjmedia_sdp_session *ses,
+static int print_session(const pjmedia_sdp_session *ses, 
 			 char *buf, pj_ssize_t len)
 {
     char *p = buf;
@@ -758,7 +758,7 @@ static int print_session(const pjmedia_sdp_session *ses,
     int printed;
 
     /* Check length for v= and o= lines. */
-    if (len < 5+
+    if (len < 5+ 
 	      2+ses->origin.user.slen+18+
 	      ses->origin.net_type.slen+ses->origin.addr.slen + 2)
     {
@@ -1078,7 +1078,7 @@ static void parse_media(pj_scanner *scanner, pjmedia_sdp_media *med,
 	if (med->desc.fmt_count < PJMEDIA_MAX_SDP_FMT)
 	    med->desc.fmt[med->desc.fmt_count++] = fmt;
 	else
-	    PJ_PERROR(2,(THIS_FILE, PJ_ETOOMANY,
+	    PJ_PERROR(2,(THIS_FILE, PJ_ETOOMANY, 
 		         "Error adding SDP media format %.*s, "
 			 "format is ignored",
 			 (int)fmt.slen, fmt.ptr));
@@ -1112,12 +1112,12 @@ static pjmedia_sdp_attr *parse_attr( pj_pool_t *pool, pj_scanner *scanner,
 
     /* skip a= */
     pj_scan_advance_n(scanner, 2, SKIP_WS);
-
+    
     /* get attr name. */
     pj_scan_get(scanner, &cs_token, &attr->name);
 
-    if (*scanner->curptr && *scanner->curptr != '\r' &&
-	*scanner->curptr != '\n')
+    if (*scanner->curptr && *scanner->curptr != '\r' && 
+	*scanner->curptr != '\n') 
     {
 	/* skip ':' if present. */
 	if (*scanner->curptr == ':')
@@ -1156,13 +1156,13 @@ static void apply_media_direction(pjmedia_sdp_session *sdp)
     const pj_str_t recvonly = { "recvonly", 8 };
     const pj_str_t sendrecv = { "sendrecv", 8 };
 
-    /* Find direction attribute in session, don't need to find default
+    /* Find direction attribute in session, don't need to find default 
      * direction "sendrecv".
      */
     for (i = 0; i < sdp->attr_count && !dir_attr; ++i) {
 	if (!pj_strcmp(&sdp->attr[i]->name, &sendonly) ||
 	    !pj_strcmp(&sdp->attr[i]->name, &recvonly) ||
-	    !pj_strcmp(&sdp->attr[i]->name, &inactive))
+	    !pj_strcmp(&sdp->attr[i]->name, &inactive)) 
 	{
 	    dir_attr = sdp->attr[i];
 	}
@@ -1186,7 +1186,7 @@ static void apply_media_direction(pjmedia_sdp_session *sdp)
 		if (!pj_strcmp(&m->attr[j]->name, &sendrecv) ||
 		    !pj_strcmp(&m->attr[j]->name, &sendonly) ||
 		    !pj_strcmp(&m->attr[j]->name, &recvonly) ||
-		    !pj_strcmp(&m->attr[j]->name, &inactive))
+		    !pj_strcmp(&m->attr[j]->name, &inactive)) 
 		{
 		    break;
 		}
@@ -1204,7 +1204,7 @@ static void apply_media_direction(pjmedia_sdp_session *sdp)
  * Parse SDP message.
  */
 PJ_DEF(pj_status_t) pjmedia_sdp_parse( pj_pool_t *pool,
-				       char *buf, pj_size_t len,
+				       char *buf, pj_size_t len, 
 				       pjmedia_sdp_session **p_sdp)
 {
     pj_scanner scanner;
@@ -1310,7 +1310,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_parse( pj_pool_t *pool,
 
     }
     PJ_CATCH_ANY {
-
+	
 	char errmsg[PJ_ERR_MSG_SIZE];
 	pj_strerror(ctx.last_error, errmsg, sizeof(errmsg));
 
@@ -1336,7 +1336,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_parse( pj_pool_t *pool,
 /*
  * Print SDP description.
  */
-PJ_DEF(int) pjmedia_sdp_print( const pjmedia_sdp_session *desc,
+PJ_DEF(int) pjmedia_sdp_print( const pjmedia_sdp_session *desc, 
 			       char *buf, pj_size_t size)
 {
     return print_session(desc, buf, size);
@@ -1347,7 +1347,7 @@ PJ_DEF(int) pjmedia_sdp_print( const pjmedia_sdp_session *desc,
  * Clone session
  */
 PJ_DEF(pjmedia_sdp_session*) pjmedia_sdp_session_clone( pj_pool_t *pool,
-						const pjmedia_sdp_session *rhs)
+			   			const pjmedia_sdp_session *rhs)
 {
     pjmedia_sdp_session *sess;
     unsigned i;
@@ -1412,7 +1412,7 @@ static pj_status_t validate_sdp_conn(const pjmedia_sdp_conn *c)
     CHECK( c, PJ_EINVAL);
     CHECK( pj_strcmp2(&c->net_type, "IN")==0, PJMEDIA_SDP_EINCONN);
     CHECK( pj_strcmp2(&c->addr_type, "IP4")==0 ||
-	   pj_strcmp2(&c->addr_type, "IP6")==0,
+	   pj_strcmp2(&c->addr_type, "IP6")==0, 
 	   PJMEDIA_SDP_EINCONN);
     CHECK( c->addr.slen != 0, PJMEDIA_SDP_EINCONN);
 
@@ -1422,6 +1422,14 @@ static pj_status_t validate_sdp_conn(const pjmedia_sdp_conn *c)
 
 /* Validate SDP session descriptor. */
 PJ_DEF(pj_status_t) pjmedia_sdp_validate(const pjmedia_sdp_session *sdp)
+{
+    return pjmedia_sdp_validate2(sdp, PJ_TRUE);
+}
+
+
+/* Validate SDP session descriptor. */
+PJ_DEF(pj_status_t) pjmedia_sdp_validate2(const pjmedia_sdp_session *sdp,
+					  pj_bool_t strict)
 {
     unsigned i;
     const pj_str_t STR_RTPMAP = { "rtpmap", 6 };
@@ -1430,10 +1438,10 @@ PJ_DEF(pj_status_t) pjmedia_sdp_validate(const pjmedia_sdp_session *sdp)
 
     /* Validate origin line. */
     CHECK( sdp->origin.user.slen != 0, PJMEDIA_SDP_EINORIGIN);
-    CHECK( pj_strcmp2(&sdp->origin.net_type, "IN")==0,
+    CHECK( pj_strcmp2(&sdp->origin.net_type, "IN")==0, 
 	   PJMEDIA_SDP_EINORIGIN);
     CHECK( pj_strcmp2(&sdp->origin.addr_type, "IP4")==0 ||
-	   pj_strcmp2(&sdp->origin.addr_type, "IP6")==0,
+	   pj_strcmp2(&sdp->origin.addr_type, "IP6")==0, 
 	   PJMEDIA_SDP_EINORIGIN);
     CHECK( sdp->origin.addr.slen != 0, PJMEDIA_SDP_EINORIGIN);
 
@@ -1471,7 +1479,8 @@ PJ_DEF(pj_status_t) pjmedia_sdp_validate(const pjmedia_sdp_session *sdp)
 	 */
 	if (m->conn == NULL) {
 	    if (sdp->conn == NULL)
-		return PJMEDIA_SDP_EMISSINGCONN;
+		if (strict || m->desc.port != 0)
+		    return PJMEDIA_SDP_EMISSINGCONN;
 	}
 
 	/* Verify payload type. */
@@ -1483,7 +1492,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_validate(const pjmedia_sdp_session *sdp)
 	    if (pj_isdigit(*m->desc.fmt[j].ptr)) {
 		unsigned pt = pj_strtoul(&m->desc.fmt[j]);
 
-		/* Payload type is between 0 and 127.
+		/* Payload type is between 0 and 127. 
 		 */
 		CHECK( pt <= 127, PJMEDIA_SDP_EINPT);
 
@@ -1493,7 +1502,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_validate(const pjmedia_sdp_session *sdp)
 		if (m->desc.port != 0 && pt >= 96) {
 		    const pjmedia_sdp_attr *a;
 
-		    a = pjmedia_sdp_media_find_attr(m, &STR_RTPMAP,
+		    a = pjmedia_sdp_media_find_attr(m, &STR_RTPMAP, 
 						    &m->desc.fmt[j]);
 		    CHECK( a != NULL, PJMEDIA_SDP_EMISSINGRTPMAP);
 		}
@@ -1505,6 +1514,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_validate(const pjmedia_sdp_session *sdp)
     return PJ_SUCCESS;
 }
 
+
 PJ_DEF(pj_status_t) pjmedia_sdp_transport_cmp( const pj_str_t *t1,
 					       const pj_str_t *t2)
 {
