@@ -1,5 +1,5 @@
-/* $Id: sdp_neg.c 3980 2012-03-20 09:23:20Z ming $ */
-/*
+/* $Id: sdp_neg.c 4367 2013-02-21 20:49:19Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjmedia/sdp_neg.h>
 #include <pjmedia/sdp.h>
@@ -43,7 +43,7 @@ struct pjmedia_sdp_neg
 			*neg_remote_sdp;    /**< Temporary remote SDP.	     */
 };
 
-static const char *state_str[] =
+static const char *state_str[] = 
 {
     "STATE_NULL",
     "STATE_LOCAL_OFFER",
@@ -63,7 +63,7 @@ struct fmt_match_cb_t
 static unsigned fmt_match_cb_cnt;
 
 /* The registered customized SDP format negotiation callbacks */
-static struct fmt_match_cb_t
+static struct fmt_match_cb_t 
 	      fmt_match_cb[PJMEDIA_SDP_NEG_MAX_CUSTOM_FMT_NEG_CB];
 
 /* Redefining a very long identifier name, just for convenience */
@@ -138,7 +138,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_create_w_remote_offer(pj_pool_t *pool,
     *p_neg = NULL;
 
     /* Validate remote offer and initial answer */
-    status = pjmedia_sdp_validate(remote);
+    status = pjmedia_sdp_validate2(remote, PJ_FALSE);
     if (status != PJ_SUCCESS)
 	return status;
 
@@ -150,7 +150,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_create_w_remote_offer(pj_pool_t *pool,
     neg->neg_remote_sdp = pjmedia_sdp_session_clone(pool, remote);
 
     if (initial) {
-	PJ_ASSERT_RETURN((status=pjmedia_sdp_validate(initial))==PJ_SUCCESS,
+	PJ_ASSERT_RETURN((status=pjmedia_sdp_validate(initial))==PJ_SUCCESS, 
 			 status);
 
 	neg->initial_sdp = pjmedia_sdp_session_clone(pool, initial);
@@ -159,7 +159,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_create_w_remote_offer(pj_pool_t *pool,
 	neg->state = PJMEDIA_SDP_NEG_STATE_WAIT_NEGO;
 
     } else {
-
+	
 	neg->state = PJMEDIA_SDP_NEG_STATE_REMOTE_OFFER;
 
     }
@@ -286,7 +286,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_modify_local_offer( pj_pool_t *pool,
     PJ_ASSERT_RETURN(pool && neg && local, PJ_EINVAL);
 
     /* Can only do this in STATE_DONE. */
-    PJ_ASSERT_RETURN(neg->state == PJMEDIA_SDP_NEG_STATE_DONE,
+    PJ_ASSERT_RETURN(neg->state == PJMEDIA_SDP_NEG_STATE_DONE, 
 		     PJMEDIA_SDPNEG_EINSTATE);
 
     /* Validate the new offer */
@@ -315,9 +315,9 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_modify_local_offer( pj_pool_t *pool,
     pj_strdup(pool, &new_offer->origin.addr, &old_offer->origin.addr);
 
     /* Generating the new offer, in the case media lines doesn't match the
-     * active SDP (e.g. current/active SDP's have m=audio and m=video lines,
-     * and the new offer only has m=audio line), the negotiator will fix
-     * the new offer by reordering and adding the missing media line with
+     * active SDP (e.g. current/active SDP's have m=audio and m=video lines, 
+     * and the new offer only has m=audio line), the negotiator will fix 
+     * the new offer by reordering and adding the missing media line with 
      * port number set to zero.
      */
     for (oi = 0; oi < old_offer->media_count; ++oi) {
@@ -331,8 +331,8 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_modify_local_offer( pj_pool_t *pool,
 	    nm = new_offer->media[ni];
 	    if (pj_strcmp(&nm->desc.media, &om->desc.media) == 0) {
 		if (ni != oi) {
-		    /* The same media found but the position unmatched to the
-		     * old offer, so let's put this media in the right place,
+		    /* The same media found but the position unmatched to the 
+		     * old offer, so let's put this media in the right place, 
 		     * and keep the order of the rest.
 		     */
 		    pj_array_insert(new_offer->media,		 /* array    */
@@ -374,7 +374,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_send_local_offer( pj_pool_t *pool,
 
     /* Can only do this in STATE_DONE or STATE_LOCAL_OFFER. */
     PJ_ASSERT_RETURN(neg->state == PJMEDIA_SDP_NEG_STATE_DONE ||
-		     neg->state == PJMEDIA_SDP_NEG_STATE_LOCAL_OFFER,
+		     neg->state == PJMEDIA_SDP_NEG_STATE_LOCAL_OFFER, 
 		     PJMEDIA_SDPNEG_EINSTATE);
 
     if (neg->state == PJMEDIA_SDP_NEG_STATE_DONE) {
@@ -382,7 +382,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_send_local_offer( pj_pool_t *pool,
 	PJ_ASSERT_RETURN(neg->active_local_sdp, PJMEDIA_SDPNEG_ENOACTIVE);
 
 	neg->state = PJMEDIA_SDP_NEG_STATE_LOCAL_OFFER;
-	neg->neg_local_sdp = pjmedia_sdp_session_clone(pool,
+	neg->neg_local_sdp = pjmedia_sdp_session_clone(pool, 
 						       neg->active_local_sdp);
 	*offer = neg->active_local_sdp;
 
@@ -393,7 +393,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_send_local_offer( pj_pool_t *pool,
 	*offer = neg->neg_local_sdp;
     }
 
-
+    
     return PJ_SUCCESS;
 }
 
@@ -409,14 +409,14 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_set_remote_answer( pj_pool_t *pool,
      * If we haven't provided local offer, then rx_remote_offer() should
      * be called instead of this function.
      */
-    PJ_ASSERT_RETURN(neg->state == PJMEDIA_SDP_NEG_STATE_LOCAL_OFFER,
+    PJ_ASSERT_RETURN(neg->state == PJMEDIA_SDP_NEG_STATE_LOCAL_OFFER, 
 		     PJMEDIA_SDPNEG_EINSTATE);
 
     /* We're ready to negotiate. */
     neg->state = PJMEDIA_SDP_NEG_STATE_WAIT_NEGO;
     neg->has_remote_answer = PJ_TRUE;
     neg->neg_remote_sdp = pjmedia_sdp_session_clone(pool, remote);
-
+ 
     return PJ_SUCCESS;
 }
 
@@ -431,7 +431,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_set_remote_offer( pj_pool_t *pool,
      * If we already provide local offer, then rx_remote_answer() should
      * be called instead of this function.
      */
-    PJ_ASSERT_RETURN(neg->state == PJMEDIA_SDP_NEG_STATE_DONE,
+    PJ_ASSERT_RETURN(neg->state == PJMEDIA_SDP_NEG_STATE_DONE, 
 		     PJMEDIA_SDPNEG_EINSTATE);
 
     /* State now is STATE_REMOTE_OFFER. */
@@ -452,7 +452,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_set_local_answer( pj_pool_t *pool,
      * If we already provide local offer, then rx_remote_answer() should
      * be called instead of this function.
      */
-    PJ_ASSERT_RETURN(neg->state == PJMEDIA_SDP_NEG_STATE_REMOTE_OFFER,
+    PJ_ASSERT_RETURN(neg->state == PJMEDIA_SDP_NEG_STATE_REMOTE_OFFER, 
 		     PJMEDIA_SDPNEG_EINSTATE);
 
     /* State now is STATE_WAIT_NEGO. */
@@ -463,7 +463,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_set_local_answer( pj_pool_t *pool,
 	    /* I don't think there is anything in RFC 3264 that mandates
 	     * answerer to place the same origin (and increment version)
 	     * in the answer, but probably it won't hurt either.
-	     * Note that the version will be incremented in
+	     * Note that the version will be incremented in 
 	     * pjmedia_sdp_neg_negotiate()
 	     */
 	    neg->neg_local_sdp->origin.id = neg->initial_sdp->origin.id;
@@ -547,7 +547,7 @@ static void update_media_direction(pj_pool_t *pool,
 	 * it is currently "sendrecv". Otherwise if local is NOT "sendonly",
 	 * then set local direction to "inactive"
 	 */
-
+    
 	switch (old_dir) {
 	case PJMEDIA_DIR_ENCODING_DECODING:
 	    new_dir = PJMEDIA_DIR_ENCODING;
@@ -561,8 +561,8 @@ static void update_media_direction(pj_pool_t *pool,
 	}
 
     } else {
-	/* Remote indicates "sendrecv" capability. No change to local
-	 * direction
+	/* Remote indicates "sendrecv" capability. No change to local 
+	 * direction 
 	 */
     }
 
@@ -585,7 +585,7 @@ static void update_media_direction(pj_pool_t *pool,
 	    /* sendrecv */
 	    break;
 	}
-
+	
 	if (a) {
 	    pjmedia_sdp_media_add_attr(local, a);
 	}
@@ -613,11 +613,11 @@ static pj_status_t process_m_answer( pj_pool_t *pool,
 
     /* Check that transport in the answer match our offer. */
 
-    /* At this point, transport type must be compatible,
+    /* At this point, transport type must be compatible, 
      * the transport instance will do more validation later.
      */
-    if (pjmedia_sdp_transport_cmp(&answer->desc.transport,
-				  &offer->desc.transport)
+    if (pjmedia_sdp_transport_cmp(&answer->desc.transport, 
+				  &offer->desc.transport) 
 	!= PJ_SUCCESS)
     {
 	return PJMEDIA_SDPNEG_EINVANSTP;
@@ -626,8 +626,8 @@ static pj_status_t process_m_answer( pj_pool_t *pool,
 
     /* Check if remote has rejected our offer */
     if (answer->desc.port == 0) {
-
-	/* Remote has rejected our offer.
+	
+	/* Remote has rejected our offer. 
 	 * Deactivate our media too.
 	 */
 	pjmedia_sdp_media_deactivate(pool, offer);
@@ -646,9 +646,9 @@ static pj_status_t process_m_answer( pj_pool_t *pool,
 
     /* Process direction attributes */
     update_media_direction(pool, answer, offer);
-
-    /* If asymetric media is allowed, then just check that remote answer has
-     * codecs that are within the offer.
+ 
+    /* If asymetric media is allowed, then just check that remote answer has 
+     * codecs that are within the offer. 
      *
      * Otherwise if asymetric media is not allowed, then we will choose only
      * one codec in our initial offer to match the answer.
@@ -685,7 +685,7 @@ static pj_status_t process_m_answer( pj_pool_t *pool,
 	    unsigned pt;
 	    pj_uint32_t j;
 	    pj_str_t *fmt = &offer->desc.fmt[i];
-
+	    
 
 	    /* Find matching answer */
 	    pt = pj_strtoul(fmt);
@@ -711,18 +711,18 @@ static pj_status_t process_m_answer( pj_pool_t *pool,
 		}
 		pjmedia_sdp_attr_get_rtpmap(a, &or_);
 
-		/* Find paylaod in answer SDP with matching
+		/* Find paylaod in answer SDP with matching 
 		 * encoding name and clock rate.
 		 */
 		for (j=0; j<answer->desc.fmt_count; ++j) {
-		    a = pjmedia_sdp_media_find_attr2(answer, "rtpmap",
+		    a = pjmedia_sdp_media_find_attr2(answer, "rtpmap", 
 						     &answer->desc.fmt[j]);
 		    if (a) {
 			pjmedia_sdp_rtpmap ar;
 			pjmedia_sdp_attr_get_rtpmap(a, &ar);
 
 			/* See if encoding name, clock rate, and channel
-			 * count match
+			 * count match 
 			 */
 			if (!pj_stricmp(&or_.enc_name, &ar.enc_name) &&
 			    or_.clock_rate == ar.clock_rate &&
@@ -778,7 +778,7 @@ static pj_status_t process_m_answer( pj_pool_t *pool,
 	 * - Resort offer formats so the order match to the answer.
 	 * - Remove answer formats that unmatches to the offer.
 	 */
-
+	
 	/* Resort offer formats */
 	for (i=0; i<offer->desc.fmt_count; ++i) {
 	    unsigned j;
@@ -797,7 +797,7 @@ static pj_status_t process_m_answer( pj_pool_t *pool,
 	    unsigned del_cnt = 0;
 	    for (i=0; i<answer->desc.fmt_count;) {
 		/* The offer is ordered now, also the offer_fmt_prior */
-		if (i >= offer->desc.fmt_count ||
+		if (i >= offer->desc.fmt_count || 
 		    offer_fmt_prior[i]-del_cnt != i)
 		{
 		    pj_str_t *fmt = &answer->desc.fmt[i];
@@ -814,7 +814,7 @@ static pj_status_t process_m_answer( pj_pool_t *pool,
 			pjmedia_sdp_media_remove_attr(answer, a);
 
 		    /* Remove this format from answer's array */
-		    pj_array_erase(answer->desc.fmt,
+		    pj_array_erase(answer->desc.fmt, 
 				   sizeof(answer->desc.fmt[0]),
 				   answer->desc.fmt_count, i);
 		    --answer->desc.fmt_count;
@@ -886,7 +886,7 @@ static pj_status_t process_answer(pj_pool_t *pool,
 	else if (status == PJMEDIA_SDPNEG_EANSNOMEDIA) {
 	    pjmedia_sdp_media_deactivate(pool, offer->media[omi]);
 	    pjmedia_sdp_media_deactivate(pool, answer->media[ami]);
-	}
+	} 
 	/* Return the error code, for other errors. */
 	else if (status != PJ_SUCCESS) {
 	    return status;
@@ -1026,13 +1026,13 @@ static pj_status_t match_offer(pj_pool_t *pool,
 	master = preanswer;
 	slave  = offer;
     }
-
-    /* With the addition of telephone-event and dodgy MS RTC SDP,
+    
+    /* With the addition of telephone-event and dodgy MS RTC SDP, 
      * the answer generation algorithm looks really shitty...
      */
     for (i=0; i<master->desc.fmt_count; ++i) {
 	unsigned j;
-
+	
 	if (pj_isdigit(*master->desc.fmt[i].ptr)) {
 	    /* This is normal/standard payload type, where it's identified
 	     * by payload number.
@@ -1040,7 +1040,7 @@ static pj_status_t match_offer(pj_pool_t *pool,
 	    unsigned pt;
 
 	    pt = pj_strtoul(&master->desc.fmt[i]);
-
+	    
 	    if (pt < 96) {
 		/* For static payload type, it's enough to compare just
 		 * the payload number.
@@ -1048,8 +1048,8 @@ static pj_status_t match_offer(pj_pool_t *pool,
 
 		master_has_codec = 1;
 
-		/* We just need to select one codec.
-		 * Continue if we have selected matching codec for previous
+		/* We just need to select one codec. 
+		 * Continue if we have selected matching codec for previous 
 		 * payload.
 		 */
 		if (found_matching_codec)
@@ -1077,7 +1077,7 @@ static pj_status_t match_offer(pj_pool_t *pool,
 		pj_bool_t is_codec;
 
 		/* Get the rtpmap for the payload type in the master. */
-		a = pjmedia_sdp_media_find_attr2(master, "rtpmap",
+		a = pjmedia_sdp_media_find_attr2(master, "rtpmap", 
 						 &master->desc.fmt[i]);
 		if (!a) {
 		    pj_assert(!"Bug! Offer should have been validated");
@@ -1096,27 +1096,27 @@ static pj_status_t match_offer(pj_pool_t *pool,
 			continue;
 		    is_codec = 1;
 		}
-
-		/* Find paylaod in our initial SDP with matching
+		
+		/* Find paylaod in our initial SDP with matching 
 		 * encoding name and clock rate.
 		 */
 		for (j=0; j<slave->desc.fmt_count; ++j) {
-		    a = pjmedia_sdp_media_find_attr2(slave, "rtpmap",
+		    a = pjmedia_sdp_media_find_attr2(slave, "rtpmap", 
 						     &slave->desc.fmt[j]);
 		    if (a) {
 			pjmedia_sdp_rtpmap lr;
 			pjmedia_sdp_attr_get_rtpmap(a, &lr);
 
 			/* See if encoding name, clock rate, and
-			 * channel count  match
+			 * channel count  match 
 			 */
 			if (!pj_stricmp(&or_.enc_name, &lr.enc_name) &&
 			    or_.clock_rate == lr.clock_rate &&
 			    (pj_stricmp(&or_.param, &lr.param)==0 ||
-			     (lr.param.slen==0 && or_.param.slen==1 &&
-						 *or_.param.ptr=='1') ||
-			     (or_.param.slen==0 && lr.param.slen==1 &&
-						  *lr.param.ptr=='1')))
+			     (lr.param.slen==0 && or_.param.slen==1 && 
+						 *or_.param.ptr=='1') || 
+			     (or_.param.slen==0 && lr.param.slen==1 && 
+						  *lr.param.ptr=='1'))) 
 			{
 			    /* Match! */
 			    if (is_codec) {
@@ -1142,12 +1142,12 @@ static pj_status_t match_offer(pj_pool_t *pool,
 				found_matching_telephone_event = 1;
 			    }
 
-			    pt_offer[pt_answer_count] =
+			    pt_offer[pt_answer_count] = 
 						prefer_remote_codec_order?
 						offer->desc.fmt[i]:
 						offer->desc.fmt[j];
-			    pt_answer[pt_answer_count++] =
-						prefer_remote_codec_order?
+			    pt_answer[pt_answer_count++] = 
+						prefer_remote_codec_order? 
 						preanswer->desc.fmt[j]:
 						preanswer->desc.fmt[i];
 			    break;
@@ -1174,7 +1174,7 @@ static pj_status_t match_offer(pj_pool_t *pool,
 		    pt_offer[pt_answer_count] = prefer_remote_codec_order?
 						offer->desc.fmt[i]:
 						offer->desc.fmt[j];
-		    pt_answer[pt_answer_count++] = prefer_remote_codec_order?
+		    pt_answer[pt_answer_count++] = prefer_remote_codec_order? 
 						   preanswer->desc.fmt[j]:
 						   preanswer->desc.fmt[i];
 		    break;
@@ -1214,20 +1214,20 @@ static pj_status_t match_offer(pj_pool_t *pool,
 	pj_assert(j != answer->desc.fmt_count);
 	str_swap(&answer->desc.fmt[i], &answer->desc.fmt[j]);
     }
-
+    
     /* Remove unwanted local formats. */
     for (i=pt_answer_count; i<answer->desc.fmt_count; ++i) {
 	pjmedia_sdp_attr *a;
 
 	/* Remove rtpmap for this format */
-	a = pjmedia_sdp_media_find_attr2(answer, "rtpmap",
+	a = pjmedia_sdp_media_find_attr2(answer, "rtpmap", 
 					 &answer->desc.fmt[i]);
 	if (a) {
 	    pjmedia_sdp_media_remove_attr(answer, a);
 	}
 
 	/* Remove fmtp for this format */
-	a = pjmedia_sdp_media_find_attr2(answer, "fmtp",
+	a = pjmedia_sdp_media_find_attr2(answer, "fmtp", 
 					 &answer->desc.fmt[i]);
 	if (a) {
 	    pjmedia_sdp_media_remove_attr(answer, a);
@@ -1260,7 +1260,7 @@ static pj_status_t create_answer( pj_pool_t *pool,
     char media_used[PJMEDIA_MAX_SDP_MEDIA];
     unsigned i;
 
-    /* Validate remote offer.
+    /* Validate remote offer. 
      * This should have been validated before.
      */
     PJ_ASSERT_RETURN((status=pjmedia_sdp_validate(offer))==PJ_SUCCESS, status);
@@ -1299,7 +1299,7 @@ static pj_status_t create_answer( pj_pool_t *pool,
                 pj_status_t status2;
 
 		/* See if it has matching codec. */
-		status2 = match_offer(pool, prefer_remote_codec_order,
+		status2 = match_offer(pool, prefer_remote_codec_order, 
 				      om, im, initial, &am);
 		if (status2 == PJ_SUCCESS) {
 		    /* Mark media as used. */
@@ -1371,7 +1371,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_negotiate( pj_pool_t *pool,
     PJ_ASSERT_RETURN(pool && neg, PJ_EINVAL);
 
     /* Must be in STATE_WAIT_NEGO state. */
-    PJ_ASSERT_RETURN(neg->state == PJMEDIA_SDP_NEG_STATE_WAIT_NEGO,
+    PJ_ASSERT_RETURN(neg->state == PJMEDIA_SDP_NEG_STATE_WAIT_NEGO, 
 		     PJMEDIA_SDPNEG_EINSTATE);
 
     /* Must have remote offer. */
@@ -1389,7 +1389,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_negotiate( pj_pool_t *pool,
     } else {
 	pjmedia_sdp_session *answer = NULL;
 
-	status = create_answer(pool, neg->prefer_remote_codec_order,
+	status = create_answer(pool, neg->prefer_remote_codec_order, 
 			       neg->neg_local_sdp, neg->neg_remote_sdp,
 			       &answer);
 	if (status == PJ_SUCCESS) {
@@ -1463,7 +1463,7 @@ PJ_DECL(pj_status_t) pjmedia_sdp_neg_register_fmt_match_cb(
     }
 
     /* Unregistration */
-
+    
     if (cb == NULL) {
 	if (i == fmt_match_cb_cnt)
 	    return PJ_ENOTFOUND;
@@ -1520,7 +1520,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_fmt_match(pj_pool_t *pool,
     }
 
     /* Get the format rtpmap from the offer. */
-    attr = pjmedia_sdp_media_find_attr2(offer, "rtpmap",
+    attr = pjmedia_sdp_media_find_attr2(offer, "rtpmap", 
 					&offer->desc.fmt[o_fmt_idx]);
     if (!attr) {
 	pj_assert(!"Bug! Offer haven't been validated");
@@ -1529,7 +1529,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_fmt_match(pj_pool_t *pool,
     pjmedia_sdp_attr_get_rtpmap(attr, &o_rtpmap);
 
     /* Get the format rtpmap from the answer. */
-    attr = pjmedia_sdp_media_find_attr2(answer, "rtpmap",
+    attr = pjmedia_sdp_media_find_attr2(answer, "rtpmap", 
 					&answer->desc.fmt[a_fmt_idx]);
     if (!attr) {
 	pj_assert(!"Bug! Answer haven't been validated");
@@ -1546,3 +1546,4 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_fmt_match(pj_pool_t *pool,
     return custom_fmt_match(pool, &o_rtpmap.enc_name,
 			    offer, o_fmt_idx, answer, a_fmt_idx, option);
 }
+
