@@ -1,5 +1,5 @@
-/* $Id: transport_ice.c 3999 2012-03-30 07:10:13Z bennylp $ */
-/*
+/* $Id: transport_ice.c 4350 2013-02-15 03:57:31Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjmedia/transport_ice.h>
 #include <pjnath/errno.h>
@@ -139,17 +139,17 @@ static pj_status_t transport_destroy  (pjmedia_transport *tp);
 /*
  * And these are ICE callbacks.
  */
-static void ice_on_rx_data(pj_ice_strans *ice_st,
-			   unsigned comp_id,
+static void ice_on_rx_data(pj_ice_strans *ice_st, 
+			   unsigned comp_id, 
 			   void *pkt, pj_size_t size,
 			   const pj_sockaddr_t *src_addr,
 			   unsigned src_addr_len);
-static void ice_on_ice_complete(pj_ice_strans *ice_st,
+static void ice_on_ice_complete(pj_ice_strans *ice_st, 
 				pj_ice_strans_op op,
 			        pj_status_t status);
 
 
-static pjmedia_transport_op transport_ice_op =
+static pjmedia_transport_op transport_ice_op = 
 {
     &transport_get_info,
     &transport_attach,
@@ -191,7 +191,7 @@ PJ_DEF(pj_status_t) pjmedia_ice_create(pjmedia_endpt *endpt,
 				       unsigned comp_cnt,
 				       const pj_ice_strans_cfg *cfg,
 				       const pjmedia_ice_cb *cb,
-				       pjmedia_transport **p_tp)
+	    			       pjmedia_transport **p_tp)
 {
     return pjmedia_ice_create2(endpt, name, comp_cnt, cfg, cb, 0, p_tp);
 }
@@ -205,7 +205,7 @@ PJ_DEF(pj_status_t) pjmedia_ice_create2(pjmedia_endpt *endpt,
 				        const pj_ice_strans_cfg *cfg,
 				        const pjmedia_ice_cb *cb,
 					unsigned options,
-				        pjmedia_transport **p_tp)
+	    			        pjmedia_transport **p_tp)
 {
     return pjmedia_ice_create3(endpt, name, comp_cnt, cfg, cb,
                                options, NULL, p_tp);
@@ -221,7 +221,7 @@ PJ_DEF(pj_status_t) pjmedia_ice_create3(pjmedia_endpt *endpt,
 				        const pjmedia_ice_cb *cb,
 					unsigned options,
 					void *user_data,
-				        pjmedia_transport **p_tp)
+	    			        pjmedia_transport **p_tp)
 {
     pj_pool_t *pool;
     pj_ice_strans_cb ice_st_cb;
@@ -259,7 +259,7 @@ PJ_DEF(pj_status_t) pjmedia_ice_create3(pjmedia_endpt *endpt,
     ice_st_cb.on_rx_data = &ice_on_rx_data;
 
     /* Create ICE */
-    status = pj_ice_strans_create(name, cfg, comp_cnt, tp_ice,
+    status = pj_ice_strans_create(name, cfg, comp_cnt, tp_ice, 
 				  &ice_st_cb, &tp_ice->ice_st);
     if (status != PJ_SUCCESS) {
 	pj_pool_release(pool);
@@ -278,14 +278,16 @@ static void set_no_ice(struct transport_ice *tp_ice, const char *reason,
     if (err != PJ_SUCCESS) {
 	char errmsg[PJ_ERR_MSG_SIZE];
 	pj_strerror(err, errmsg, sizeof(errmsg));
-	PJ_LOG(4,(tp_ice->base.name,
+	PJ_LOG(4,(tp_ice->base.name, 
 		  "Stopping ICE, reason=%s:%s", reason, errmsg));
     } else {
-	PJ_LOG(4,(tp_ice->base.name,
+	PJ_LOG(4,(tp_ice->base.name, 
 		  "Stopping ICE, reason=%s", reason));
     }
 
-    pj_ice_strans_stop_ice(tp_ice->ice_st);
+    if (tp_ice->ice_st) {
+	pj_ice_strans_stop_ice(tp_ice->ice_st);
+    }
 
     tp_ice->use_ice = PJ_FALSE;
 }
@@ -304,7 +306,7 @@ static int print_sdp_cand_attr(char *buffer, int max_len,
 			    cand->foundation.ptr,
 			    (unsigned)cand->comp_id,
 			    cand->prio,
-			    pj_sockaddr_print(&cand->addr, ipaddr,
+			    pj_sockaddr_print(&cand->addr, ipaddr, 
 					      sizeof(ipaddr), 0),
 			    (unsigned)pj_sockaddr_get_port(&cand->addr));
     if (len < 1 || len >= max_len)
@@ -389,7 +391,7 @@ static pj_status_t encode_session_in_sdp(struct transport_ice *tp_ice,
 					 unsigned comp_cnt,
 					 pj_bool_t restart_session)
 {
-    enum {
+    enum { 
 	ATTR_BUF_LEN = 160,	/* Max len of a=candidate attr */
 	RATTR_BUF_LEN= 160	/* Max len of a=remote-candidates attr */
     };
@@ -410,7 +412,7 @@ static pj_status_t encode_session_in_sdp(struct transport_ice *tp_ice,
      *
      * 9.1.2.2: Existing Media Streams with ICE Completed
      *   The agent MUST include a candidate attributes for candidates
-     *   matching the default destination for each component of the
+     *   matching the default destination for each component of the 
      *   media stream, and MUST NOT include any other candidates.
      *
      * When ICE has not completed, we shall include all candidates.
@@ -435,7 +437,7 @@ static pj_status_t encode_session_in_sdp(struct transport_ice *tp_ice,
 	pjmedia_sdp_attr_add(&m->attr_count, m->attr, attr);
 
 	/* Encode ice-pwd attribute */
-	attr = pjmedia_sdp_attr_create(sdp_pool, STR_ICE_PWD.ptr,
+	attr = pjmedia_sdp_attr_create(sdp_pool, STR_ICE_PWD.ptr, 
 				       &local_pwd);
 	pjmedia_sdp_attr_add(&m->attr_count, m->attr, attr);
 
@@ -445,7 +447,7 @@ static pj_status_t encode_session_in_sdp(struct transport_ice *tp_ice,
 	rem_cand.slen = 0;
 
 	/* 9.1.2.2: Existing Media Streams with ICE Completed
-	 *   The default destination for media (i.e., the values of
+	 *   The default destination for media (i.e., the values of 
 	 *   the IP addresses and ports in the m and c line used for
 	 *   that media stream) MUST be the local candidate from the
 	 *   highest priority nominated pair in the valid list for each
@@ -462,30 +464,30 @@ static pj_status_t encode_session_in_sdp(struct transport_ice *tp_ice,
 	if (conn == NULL)
 	    conn = sdp_local->conn;
 
-	conn->addr.ptr = (char*) pj_pool_alloc(sdp_pool,
+	conn->addr.ptr = (char*) pj_pool_alloc(sdp_pool, 
 					       PJ_INET6_ADDRSTRLEN);
-	pj_sockaddr_print(&check->lcand->addr, conn->addr.ptr,
+	pj_sockaddr_print(&check->lcand->addr, conn->addr.ptr, 
 			  PJ_INET6_ADDRSTRLEN, 0);
 	conn->addr.slen = pj_ansi_strlen(conn->addr.ptr);
 	m->desc.port = pj_sockaddr_get_port(&check->lcand->addr);
 
 	/* Override address RTCP attribute if it's present */
 	if (comp_cnt == 2 &&
-	    (check = pj_ice_strans_get_valid_pair(tp_ice->ice_st,
+	    (check = pj_ice_strans_get_valid_pair(tp_ice->ice_st, 
 						  COMP_RTCP)) != NULL &&
-	    (a_rtcp = pjmedia_sdp_attr_find(m->attr_count, m->attr,
-					    &STR_RTCP, 0)) != NULL)
+	    (a_rtcp = pjmedia_sdp_attr_find(m->attr_count, m->attr, 
+					    &STR_RTCP, 0)) != NULL) 
 	{
 	    pjmedia_sdp_attr_remove(&m->attr_count, m->attr, a_rtcp);
 
-	    a_rtcp = pjmedia_sdp_attr_create_rtcp(sdp_pool,
+	    a_rtcp = pjmedia_sdp_attr_create_rtcp(sdp_pool, 
 						  &check->lcand->addr);
 	    if (a_rtcp)
 		pjmedia_sdp_attr_add(&m->attr_count, m->attr, a_rtcp);
 	}
 
-	/* Encode only candidates matching the default destination
-	 * for each component
+	/* Encode only candidates matching the default destination 
+	 * for each component 
 	 */
 	for (comp=0; comp < comp_cnt; ++comp) {
 	    int len;
@@ -498,7 +500,7 @@ static pj_status_t encode_session_in_sdp(struct transport_ice *tp_ice,
 
 	    /* Print and add local candidate in the pair */
 	    value.ptr = attr_buf;
-	    value.slen = print_sdp_cand_attr(attr_buf, ATTR_BUF_LEN,
+	    value.slen = print_sdp_cand_attr(attr_buf, ATTR_BUF_LEN, 
 					     check->lcand);
 	    if (value.slen < 0) {
 		pj_assert(!"Not enough attr_buf to print candidate");
@@ -510,17 +512,17 @@ static pj_status_t encode_session_in_sdp(struct transport_ice *tp_ice,
 	    pjmedia_sdp_attr_add(&m->attr_count, m->attr, attr);
 
 	    /* Append to a=remote-candidates attribute */
-	    if (pj_ice_strans_get_role(tp_ice->ice_st) ==
-				    PJ_ICE_SESS_ROLE_CONTROLLING)
+	    if (pj_ice_strans_get_role(tp_ice->ice_st) == 
+				    PJ_ICE_SESS_ROLE_CONTROLLING) 
 	    {
 		char rem_addr[PJ_INET6_ADDRSTRLEN];
 
-		pj_sockaddr_print(&check->rcand->addr, rem_addr,
+		pj_sockaddr_print(&check->rcand->addr, rem_addr, 
 				  sizeof(rem_addr), 0);
 		len = pj_ansi_snprintf(
 			   rem_cand.ptr + rem_cand.slen,
 			   RATTR_BUF_LEN - rem_cand.slen,
-			   "%s%u %s %u",
+			   "%s%u %s %u", 
 			   (rem_cand.slen==0? "" : " "),
 			   comp+1, rem_addr,
 			   pj_sockaddr_get_port(&check->rcand->addr)
@@ -537,16 +539,16 @@ static pj_status_t encode_session_in_sdp(struct transport_ice *tp_ice,
 
 	/* 9.1.2.2: Existing Media Streams with ICE Completed
 	 *   In addition, if the agent is controlling, it MUST include
-	 *   the a=remote-candidates attribute for each media stream
+	 *   the a=remote-candidates attribute for each media stream 
 	 *   whose check list is in the Completed state.  The attribute
-	 *   contains the remote candidates from the highest priority
+	 *   contains the remote candidates from the highest priority 
 	 *   nominated pair in the valid list for each component of that
 	 *   media stream.
 	 */
-	if (pj_ice_strans_get_role(tp_ice->ice_st) ==
-				    PJ_ICE_SESS_ROLE_CONTROLLING)
+	if (pj_ice_strans_get_role(tp_ice->ice_st) == 
+				    PJ_ICE_SESS_ROLE_CONTROLLING) 
 	{
-	    attr = pjmedia_sdp_attr_create(sdp_pool, STR_REM_CAND.ptr,
+	    attr = pjmedia_sdp_attr_create(sdp_pool, STR_REM_CAND.ptr, 
 					   &rem_cand);
 	    pjmedia_sdp_attr_add(&m->attr_count, m->attr, attr);
 	}
@@ -567,7 +569,7 @@ static pj_status_t encode_session_in_sdp(struct transport_ice *tp_ice,
 					   &local_ufrag);
 	    pjmedia_sdp_attr_add(&m->attr_count, m->attr, attr);
 
-	    attr = pjmedia_sdp_attr_create(sdp_pool, STR_ICE_PWD.ptr,
+	    attr = pjmedia_sdp_attr_create(sdp_pool, STR_ICE_PWD.ptr, 
 					   &local_pwd);
 	    pjmedia_sdp_attr_add(&m->attr_count, m->attr, attr);
 
@@ -603,7 +605,7 @@ static pj_status_t encode_session_in_sdp(struct transport_ice *tp_ice,
 	    for (i=0; i<cand_cnt; ++i) {
 		pj_str_t value;
 
-		value.slen = print_sdp_cand_attr(attr_buf, ATTR_BUF_LEN,
+		value.slen = print_sdp_cand_attr(attr_buf, ATTR_BUF_LEN, 
 						 &cand[i]);
 		if (value.slen < 0) {
 		    pj_assert(!"Not enough attr_buf to print candidate");
@@ -611,7 +613,7 @@ static pj_status_t encode_session_in_sdp(struct transport_ice *tp_ice,
 		}
 
 		value.ptr = attr_buf;
-		attr = pjmedia_sdp_attr_create(sdp_pool,
+		attr = pjmedia_sdp_attr_create(sdp_pool, 
 					       STR_CANDIDATE.ptr,
 					       &value);
 		pjmedia_sdp_attr_add(&m->attr_count, m->attr, attr);
@@ -641,7 +643,7 @@ static pj_status_t encode_session_in_sdp(struct transport_ice *tp_ice,
             m->bandw_count++;
         }
     }
-
+    
 
     return PJ_SUCCESS;
 }
@@ -687,7 +689,7 @@ static pj_status_t parse_cand(const char *obj_name,
 	goto on_return;
     }
     if (pj_ansi_stricmp(token, "UDP") != 0) {
-	TRACE__((obj_name,
+	TRACE__((obj_name, 
 		 "Expecting ICE UDP transport only in candidate"));
 	goto on_return;
     }
@@ -756,7 +758,7 @@ static pj_status_t parse_cand(const char *obj_name,
 	cand->type = PJ_ICE_CAND_TYPE_PRFLX;
 
     } else {
-	PJ_LOG(5,(obj_name, "Invalid ICE candidate type %s in candidate",
+	PJ_LOG(5,(obj_name, "Invalid ICE candidate type %s in candidate", 
 		  token));
 	goto on_return;
     }
@@ -777,7 +779,7 @@ static pj_status_t create_initial_offer(struct transport_ice *tp_ice,
     pj_status_t status;
 
     /* Encode ICE in SDP */
-    status = encode_session_in_sdp(tp_ice, sdp_pool, loc_sdp, media_index,
+    status = encode_session_in_sdp(tp_ice, sdp_pool, loc_sdp, media_index, 
 				   tp_ice->comp_cnt, PJ_FALSE);
     if (status != PJ_SUCCESS) {
 	set_no_ice(tp_ice, "Error encoding SDP answer", status);
@@ -815,8 +817,8 @@ static pj_status_t verify_ice_sdp(struct transport_ice *tp_ice,
 	return PJ_SUCCESS;
     }
 
-    /* Verify that default target for each component matches one of the
-     * candidate for the component. Otherwise stop ICE with ICE ice_mismatch
+    /* Verify that default target for each component matches one of the 
+     * candidate for the component. Otherwise stop ICE with ICE ice_mismatch 
      * error.
      */
 
@@ -828,9 +830,9 @@ static pj_status_t verify_ice_sdp(struct transport_ice *tp_ice,
 	return PJMEDIA_SDP_EMISSINGCONN;
 
     /* Verify address family matches */
-    if ((tp_ice->af==pj_AF_INET() &&
+    if ((tp_ice->af==pj_AF_INET() && 
 	 pj_strcmp(&rem_conn->addr_type, &STR_IP4)!=0) ||
-	(tp_ice->af==pj_AF_INET6() &&
+	(tp_ice->af==pj_AF_INET6() && 
 	 pj_strcmp(&rem_conn->addr_type, &STR_IP6)!=0))
     {
 	return PJMEDIA_SDP_ETPORTNOTEQUAL;
@@ -848,7 +850,7 @@ static pj_status_t verify_ice_sdp(struct transport_ice *tp_ice,
 	/* Get default RTCP candidate from a=rtcp line, if present, otherwise
 	 * calculate default RTCP candidate from default RTP target.
 	 */
-	attr = pjmedia_sdp_attr_find(rem_m->attr_count, rem_m->attr,
+	attr = pjmedia_sdp_attr_find(rem_m->attr_count, rem_m->attr, 
 				     &STR_RTCP, NULL);
 	has_rtcp = (attr != NULL);
 
@@ -860,12 +862,12 @@ static pj_status_t verify_ice_sdp(struct transport_ice *tp_ice,
 		/* Error parsing a=rtcp attribute */
 		return status;
 	    }
-
+    	
 	    if (rtcp_attr.addr.slen) {
 		/* Verify address family matches */
-		if ((tp_ice->af==pj_AF_INET() &&
+		if ((tp_ice->af==pj_AF_INET() && 
 		     pj_strcmp(&rtcp_attr.addr_type, &STR_IP4)!=0) ||
-		    (tp_ice->af==pj_AF_INET6() &&
+		    (tp_ice->af==pj_AF_INET6() && 
 		     pj_strcmp(&rtcp_attr.addr_type, &STR_IP6)!=0))
 		{
 		    return PJMEDIA_SDP_ETPORTNOTEQUAL;
@@ -880,8 +882,8 @@ static pj_status_t verify_ice_sdp(struct transport_ice *tp_ice,
 		}
 	    } else {
 		/* Assign RTCP address */
-		status = pj_sockaddr_init(tp_ice->af, &rtcp_addr,
-					  NULL,
+		status = pj_sockaddr_init(tp_ice->af, &rtcp_addr, 
+					  NULL, 
 					  (pj_uint16_t)rtcp_attr.port);
 		if (status != PJ_SUCCESS) {
 		    return PJMEDIA_SDP_EINRTCP;
@@ -890,14 +892,14 @@ static pj_status_t verify_ice_sdp(struct transport_ice *tp_ice,
 	    }
 	} else {
 	    unsigned rtcp_port;
-
+    	
 	    rtcp_port = pj_sockaddr_get_port(&rem_conn_addr) + 1;
 	    pj_sockaddr_cp(&rtcp_addr, &rem_conn_addr);
 	    pj_sockaddr_set_port(&rtcp_addr, (pj_uint16_t)rtcp_port);
 	}
     }
 
-    /* Find the default addresses in a=candidate attributes.
+    /* Find the default addresses in a=candidate attributes. 
      */
     for (i=0; i<rem_m->attr_count; ++i) {
 	pj_ice_sess_cand cand;
@@ -905,23 +907,23 @@ static pj_status_t verify_ice_sdp(struct transport_ice *tp_ice,
 	if (pj_strcmp(&rem_m->attr[i]->name, &STR_CANDIDATE)!=0)
 	    continue;
 
-	status = parse_cand(tp_ice->base.name, tmp_pool,
+	status = parse_cand(tp_ice->base.name, tmp_pool, 
 			    &rem_m->attr[i]->value, &cand);
 	if (status != PJ_SUCCESS) {
-	    PJ_LOG(4,(tp_ice->base.name,
+	    PJ_LOG(4,(tp_ice->base.name, 
 		      "Error in parsing SDP candidate attribute '%.*s', "
 		      "candidate is ignored",
-		      (int)rem_m->attr[i]->value.slen,
+		      (int)rem_m->attr[i]->value.slen, 
 		      rem_m->attr[i]->value.ptr));
 	    continue;
 	}
 
 	if (!comp1_found && cand.comp_id==COMP_RTP &&
-	    pj_sockaddr_cmp(&rem_conn_addr, &cand.addr)==0)
+	    pj_sockaddr_cmp(&rem_conn_addr, &cand.addr)==0) 
 	{
 	    comp1_found = PJ_TRUE;
 	} else if (!comp2_found && cand.comp_id==COMP_RTCP &&
-		    pj_sockaddr_cmp(&rtcp_addr, &cand.addr)==0)
+		    pj_sockaddr_cmp(&rtcp_addr, &cand.addr)==0) 
 	{
 	    comp2_found = PJ_TRUE;
 	}
@@ -949,7 +951,7 @@ static pj_status_t verify_ice_sdp(struct transport_ice *tp_ice,
     /* Detect remote restarting session */
     if (pj_ice_strans_has_sess(tp_ice->ice_st) &&
 	(pj_ice_strans_sess_is_running(tp_ice->ice_st) ||
-	 pj_ice_strans_sess_is_complete(tp_ice->ice_st)))
+	 pj_ice_strans_sess_is_complete(tp_ice->ice_st))) 
     {
 	pj_str_t rem_run_ufrag, rem_run_pwd;
 	pj_ice_strans_get_ufrag_pwd(tp_ice->ice_st, NULL, NULL,
@@ -980,12 +982,12 @@ static pj_status_t verify_ice_sdp(struct transport_ice *tp_ice,
 	}
     }
 
-    PJ_LOG(4,(tp_ice->base.name,
+    PJ_LOG(4,(tp_ice->base.name, 
 	      "Processing SDP: support ICE=%u, common comp_cnt=%u, "
 	      "ice_mismatch=%u, ice_restart=%u, local_role=%s",
-	      (sdp_state->match_comp_cnt != 0),
-	      sdp_state->match_comp_cnt,
-	      sdp_state->ice_mismatch,
+	      (sdp_state->match_comp_cnt != 0), 
+	      sdp_state->match_comp_cnt, 
+	      sdp_state->ice_mismatch, 
 	      sdp_state->ice_restart,
 	      pj_ice_sess_role_name(sdp_state->local_role)));
 
@@ -1010,8 +1012,8 @@ static pj_status_t create_initial_answer(struct transport_ice *tp_ice,
     }
 
     /* Verify the offer */
-    status = verify_ice_sdp(tp_ice, sdp_pool, rem_sdp, media_index,
-			    PJ_ICE_SESS_ROLE_CONTROLLED,
+    status = verify_ice_sdp(tp_ice, sdp_pool, rem_sdp, media_index, 
+			    PJ_ICE_SESS_ROLE_CONTROLLED, 
 			    &tp_ice->rem_offer_state);
     if (status != PJ_SUCCESS) {
 	set_no_ice(tp_ice, "Invalid SDP offer", status);
@@ -1032,7 +1034,7 @@ static pj_status_t create_initial_answer(struct transport_ice *tp_ice,
     }
 
     /* Encode ICE in SDP */
-    status = encode_session_in_sdp(tp_ice, sdp_pool, loc_sdp, media_index,
+    status = encode_session_in_sdp(tp_ice, sdp_pool, loc_sdp, media_index, 
 				   tp_ice->rem_offer_state.match_comp_cnt,
 				   PJ_FALSE);
     if (status != PJ_SUCCESS) {
@@ -1073,8 +1075,8 @@ static pj_status_t create_subsequent_answer(struct transport_ice *tp_ice,
     pj_status_t status;
 
     /* We have a session */
-    status = verify_ice_sdp(tp_ice, sdp_pool, rem_sdp, media_index,
-			    PJ_ICE_SESS_ROLE_CONTROLLED,
+    status = verify_ice_sdp(tp_ice, sdp_pool, rem_sdp, media_index, 
+			    PJ_ICE_SESS_ROLE_CONTROLLED, 
 			    &tp_ice->rem_offer_state);
     if (status != PJ_SUCCESS) {
 	/* Something wrong with the offer */
@@ -1122,7 +1124,7 @@ static pj_status_t create_subsequent_answer(struct transport_ice *tp_ice,
 	/* Looks like now remote is offering ICE, so we need to create
 	 * ICE session now.
 	 */
-	status = pj_ice_strans_init_ice(tp_ice->ice_st,
+	status = pj_ice_strans_init_ice(tp_ice->ice_st, 
 					PJ_ICE_SESS_ROLE_CONTROLLED,
 					NULL, NULL);
 	if (status != PJ_SUCCESS) {
@@ -1167,7 +1169,7 @@ static pj_status_t transport_media_create(pjmedia_transport *tp,
     /* Init ICE, the initial role is set now based on availability of
      * rem_sdp, but it will be checked again later.
      */
-    ice_role = (rem_sdp==NULL ? PJ_ICE_SESS_ROLE_CONTROLLING :
+    ice_role = (rem_sdp==NULL ? PJ_ICE_SESS_ROLE_CONTROLLING : 
 				PJ_ICE_SESS_ROLE_CONTROLLED);
     status = pj_ice_strans_init_ice(tp_ice->ice_st, ice_role, NULL, NULL);
 
@@ -1205,7 +1207,7 @@ static pj_status_t transport_encode_sdp(pjmedia_transport *tp,
 
     if (tp_ice->initial_sdp) {
 	if (rem_sdp) {
-	    status = create_initial_answer(tp_ice, sdp_pool, sdp_local,
+	    status = create_initial_answer(tp_ice, sdp_pool, sdp_local, 
 					   rem_sdp, media_index);
 	} else {
 	    status = create_initial_offer(tp_ice, sdp_pool, sdp_local,
@@ -1248,7 +1250,7 @@ static pj_status_t start_ice(struct transport_ice *tp_ice,
 
     /* Allocate candidate array */
     cand = (pj_ice_sess_cand*)
-	   pj_pool_calloc(tmp_pool, PJ_ICE_MAX_CAND,
+	   pj_pool_calloc(tmp_pool, PJ_ICE_MAX_CAND, 
 			  sizeof(pj_ice_sess_cand));
 
     /* Get all candidates in the media */
@@ -1262,10 +1264,10 @@ static pj_status_t start_ice(struct transport_ice *tp_ice,
 	    continue;
 
 	/* Parse candidate */
-	status = parse_cand(tp_ice->base.name, tmp_pool, &attr->value,
+	status = parse_cand(tp_ice->base.name, tmp_pool, &attr->value, 
 			    &cand[cand_cnt]);
 	if (status != PJ_SUCCESS) {
-	    PJ_LOG(4,(tp_ice->base.name,
+	    PJ_LOG(4,(tp_ice->base.name, 
 		      "Error in parsing SDP candidate attribute '%.*s', "
 		      "candidate is ignored",
 		      (int)attr->value.slen, attr->value.ptr));
@@ -1276,7 +1278,7 @@ static pj_status_t start_ice(struct transport_ice *tp_ice,
     }
 
     /* Start ICE */
-    return pj_ice_strans_start_ice(tp_ice->ice_st, &ufrag_attr->value,
+    return pj_ice_strans_start_ice(tp_ice->ice_st, &ufrag_attr->value, 
 				   &pwd_attr->value, cand_cnt, cand);
 }
 
@@ -1331,7 +1333,7 @@ static pj_status_t transport_media_start(pjmedia_transport *tp,
 	struct sdp_state answer_state;
 
 	/* Verify the answer */
-	status = verify_ice_sdp(tp_ice, tmp_pool, rem_sdp, media_index,
+	status = verify_ice_sdp(tp_ice, tmp_pool, rem_sdp, media_index, 
 				PJ_ICE_SESS_ROLE_CONTROLLING, &answer_state);
 	if (status != PJ_SUCCESS) {
 	    /* Something wrong in the SDP answer */
@@ -1342,25 +1344,25 @@ static pj_status_t transport_media_start(pjmedia_transport *tp,
 	/* Does it have ICE? */
 	if (answer_state.match_comp_cnt == 0) {
 	    /* Remote doesn't support ICE */
-	    set_no_ice(tp_ice, "Remote answer doesn't support ICE",
+	    set_no_ice(tp_ice, "Remote answer doesn't support ICE", 
 		       PJ_SUCCESS);
 	    return PJ_SUCCESS;
 	}
 
 	/* Check if remote has reported ice-mismatch */
-	if (pjmedia_sdp_attr_find(rem_m->attr_count, rem_m->attr,
+	if (pjmedia_sdp_attr_find(rem_m->attr_count, rem_m->attr, 
 				  &STR_ICE_MISMATCH, NULL) != NULL)
 	{
 	    /* Remote has reported ice-mismatch */
-	    set_no_ice(tp_ice,
-		       "Remote answer contains 'ice-mismatch' attribute",
+	    set_no_ice(tp_ice, 
+		       "Remote answer contains 'ice-mismatch' attribute", 
 		       PJ_SUCCESS);
 	    return PJ_SUCCESS;
 	}
 
 	/* Check if remote has indicated a restart */
 	if (answer_state.ice_restart) {
-	    PJ_LOG(2,(tp_ice->base.name,
+	    PJ_LOG(2,(tp_ice->base.name, 
 		      "Warning: remote has signalled ICE restart in SDP "
 		      "answer which is disallowed. Remote ICE negotiation"
 		      " may fail."));
@@ -1370,10 +1372,10 @@ static pj_status_t transport_media_start(pjmedia_transport *tp,
 	if (answer_state.ice_mismatch) {
 	    /* This happens either when a B2BUA modified remote answer but
 	     * strangely didn't modify our offer, or remote is not capable
-	     * of detecting mismatch in our offer (it didn't put
+	     * of detecting mismatch in our offer (it didn't put 
 	     * 'ice-mismatch' attribute in the answer).
 	     */
-	    PJ_LOG(2,(tp_ice->base.name,
+	    PJ_LOG(2,(tp_ice->base.name, 
 		      "Warning: remote answer mismatch, but it does not "
 		      "reject our offer with 'ice-mismatch'. ICE negotiation "
 		      "may fail"));
@@ -1410,7 +1412,7 @@ static pj_status_t transport_media_start(pjmedia_transport *tp,
 
 	/* Check for ICE ice_mismatch condition in the offer */
 	if (tp_ice->rem_offer_state.ice_mismatch) {
-	    set_no_ice(tp_ice, "Remote offer mismatch: ",
+	    set_no_ice(tp_ice, "Remote offer mismatch: ", 
 		       PJNATH_EICEMISMATCH);
 	    return PJ_SUCCESS;
 	}
@@ -1427,7 +1429,7 @@ static pj_status_t transport_media_start(pjmedia_transport *tp,
 	}
 
 	/* Either remote has requested ICE restart or this is our
-	 * first answer.
+	 * first answer. 
 	 */
 
 	/* Stop ICE */
@@ -1437,27 +1439,27 @@ static pj_status_t transport_media_start(pjmedia_transport *tp,
 	    /* We have put new ICE ufrag and pwd in the answer. Now
 	     * create a new ICE session with that ufrag/pwd pair.
 	     */
-	    get_ice_attr(sdp_local, sdp_local->media[media_index],
+	    get_ice_attr(sdp_local, sdp_local->media[media_index], 
 			 &ufrag_attr, &pwd_attr);
-	    status = pj_ice_strans_init_ice(tp_ice->ice_st,
+	    status = pj_ice_strans_init_ice(tp_ice->ice_st, 
 					    tp_ice->rem_offer_state.local_role,
-					    &ufrag_attr->value,
+					    &ufrag_attr->value, 
 					    &pwd_attr->value);
 	    if (status != PJ_SUCCESS) {
-		PJ_LOG(1,(tp_ice->base.name,
+		PJ_LOG(1,(tp_ice->base.name, 
 			  "ICE re-initialization failed (status=%d)!",
 			  status));
 		return status;
 	    }
 	}
 
-	/* Ticket #977: Update role if turns out we're supposed to be the
-	 * Controlling agent (e.g. when talking to ice-lite peer).
+	/* Ticket #977: Update role if turns out we're supposed to be the 
+	 * Controlling agent (e.g. when talking to ice-lite peer). 
 	 */
 	if (tp_ice->rem_offer_state.local_role==PJ_ICE_SESS_ROLE_CONTROLLING &&
-	    pj_ice_strans_has_sess(tp_ice->ice_st))
+	    pj_ice_strans_has_sess(tp_ice->ice_st)) 
 	{
-	    pj_ice_strans_change_role(tp_ice->ice_st,
+	    pj_ice_strans_change_role(tp_ice->ice_st, 
 				      PJ_ICE_SESS_ROLE_CONTROLLING);
 	}
 
@@ -1468,7 +1470,7 @@ static pj_status_t transport_media_start(pjmedia_transport *tp,
     /* Now start ICE */
     status = start_ice(tp_ice, tmp_pool, rem_sdp, media_index);
     if (status != PJ_SUCCESS) {
-	PJ_LOG(1,(tp_ice->base.name,
+	PJ_LOG(1,(tp_ice->base.name, 
 		  "ICE restart failed (status=%d)!",
 		  status));
 	return status;
@@ -1484,7 +1486,7 @@ static pj_status_t transport_media_start(pjmedia_transport *tp,
 static pj_status_t transport_media_stop(pjmedia_transport *tp)
 {
     struct transport_ice *tp_ice = (struct transport_ice*)tp;
-
+    
     set_no_ice(tp_ice, "media stop requested", PJ_SUCCESS);
 
     return PJ_SUCCESS;
@@ -1517,7 +1519,7 @@ static pj_status_t transport_get_info(pjmedia_transport *tp,
 	pj_sockaddr_cp(&info->sock_info.rtcp_addr_name, &cand.addr);
     }
 
-    /* Set remote address originating RTP & RTCP if this transport has
+    /* Set remote address originating RTP & RTCP if this transport has 
      * ICE activated or received any packets.
      */
     if (tp_ice->use_ice || tp_ice->rtp_src_cnt) {
@@ -1541,13 +1543,15 @@ static pj_status_t transport_get_info(pjmedia_transport *tp,
 	ii = (pjmedia_ice_transport_info*) tsi->buffer;
 	pj_bzero(ii, sizeof(*ii));
 
+	ii->active = tp_ice->use_ice;
+
 	if (pj_ice_strans_has_sess(tp_ice->ice_st))
 	    ii->role = pj_ice_strans_get_role(tp_ice->ice_st);
 	else
 	    ii->role = PJ_ICE_SESS_ROLE_UNKNOWN;
 	ii->sess_state = pj_ice_strans_get_state(tp_ice->ice_st);
 	ii->comp_cnt = pj_ice_strans_get_running_comp_cnt(tp_ice->ice_st);
-
+	
 	for (i=1; i<=ii->comp_cnt && i<=PJ_ARRAY_SIZE(ii->comp); ++i) {
 	    const pj_ice_sess_check *chk;
 
@@ -1623,14 +1627,14 @@ static pj_status_t transport_send_rtp(pjmedia_transport *tp,
     /* Simulate packet lost on TX direction */
     if (tp_ice->tx_drop_pct) {
 	if ((pj_rand() % 100) <= (int)tp_ice->tx_drop_pct) {
-	    PJ_LOG(5,(tp_ice->base.name,
+	    PJ_LOG(5,(tp_ice->base.name, 
 		      "TX RTP packet dropped because of pkt lost "
 		      "simulation"));
 	    return PJ_SUCCESS;
 	}
     }
 
-    return pj_ice_strans_sendto(tp_ice->ice_st, 1,
+    return pj_ice_strans_sendto(tp_ice->ice_st, 1, 
 			        pkt, size, &tp_ice->remote_rtp,
 				tp_ice->addr_len);
 }
@@ -1655,7 +1659,7 @@ static pj_status_t transport_send_rtcp2(pjmedia_transport *tp,
 	    addr = &tp_ice->remote_rtcp;
 	    addr_len = pj_sockaddr_get_len(addr);
 	}
-	return pj_ice_strans_sendto(tp_ice->ice_st, 2, pkt, size,
+	return pj_ice_strans_sendto(tp_ice->ice_st, 2, pkt, size, 
 				    addr, addr_len);
     } else {
 	return PJ_SUCCESS;
@@ -1663,7 +1667,7 @@ static pj_status_t transport_send_rtcp2(pjmedia_transport *tp,
 }
 
 
-static void ice_on_rx_data(pj_ice_strans *ice_st, unsigned comp_id,
+static void ice_on_rx_data(pj_ice_strans *ice_st, unsigned comp_id, 
 			   void *pkt, pj_size_t size,
 			   const pj_sockaddr_t *src_addr,
 			   unsigned src_addr_len)
@@ -1678,15 +1682,15 @@ static void ice_on_rx_data(pj_ice_strans *ice_st, unsigned comp_id,
 	/* Simulate packet lost on RX direction */
 	if (tp_ice->rx_drop_pct) {
 	    if ((pj_rand() % 100) <= (int)tp_ice->rx_drop_pct) {
-		PJ_LOG(5,(tp_ice->base.name,
+		PJ_LOG(5,(tp_ice->base.name, 
 			  "RX RTP packet dropped because of pkt lost "
 			  "simulation"));
 		return;
 	    }
 	}
 
-	/* See if source address of RTP packet is different than the
-	 * configured address, and switch RTP remote address to
+	/* See if source address of RTP packet is different than the 
+	 * configured address, and switch RTP remote address to 
 	 * source packet address after several consecutive packets
 	 * have been received.
 	 */
@@ -1735,7 +1739,7 @@ static void ice_on_rx_data(pj_ice_strans *ice_st, unsigned comp_id,
 		    if (!pj_sockaddr_has_addr(&tp_ice->rtcp_src_addr)) {
 			pj_uint16_t port;
 
-			pj_sockaddr_cp(&tp_ice->remote_rtcp,
+			pj_sockaddr_cp(&tp_ice->remote_rtcp, 
 				       &tp_ice->remote_rtp);
 
 			port = (pj_uint16_t)
@@ -1745,7 +1749,7 @@ static void ice_on_rx_data(pj_ice_strans *ice_st, unsigned comp_id,
 			PJ_LOG(4,(tp_ice->base.name,
 				  "Remote RTCP address switched to predicted "
 				  "address %s",
-				  pj_sockaddr_print(&tp_ice->remote_rtcp,
+				  pj_sockaddr_print(&tp_ice->remote_rtcp, 
 						    addr_text,
 						    sizeof(addr_text), 3)));
 		    }
@@ -1797,7 +1801,7 @@ static void ice_on_rx_data(pj_ice_strans *ice_st, unsigned comp_id,
 }
 
 
-static void ice_on_ice_complete(pj_ice_strans *ice_st,
+static void ice_on_ice_complete(pj_ice_strans *ice_st, 
 				pj_ice_strans_op op,
 			        pj_status_t result)
 {
@@ -1850,3 +1854,4 @@ static pj_status_t transport_destroy(pjmedia_transport *tp)
 
     return PJ_SUCCESS;
 }
+
