@@ -1,4 +1,4 @@
-/* $Id: vid_port.c 4168 2012-06-18 05:59:08Z ming $ */
+/* $Id: vid_port.c 4290 2012-11-01 03:06:33Z ming $ */
 /*
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  *
@@ -262,7 +262,7 @@ PJ_DEF(pj_status_t) pjmedia_vid_port_create( pj_pool_t *pool,
     ptime_usec = PJMEDIA_PTIME(&vfd->fps);
     pjmedia_clock_src_init(&vp->clocksrc, PJMEDIA_TYPE_VIDEO,
                            prm->vidparam.clock_rate, ptime_usec);
-    vp->sync_clocksrc.max_sync_ticks =
+    vp->sync_clocksrc.max_sync_ticks = 
         PJMEDIA_CLOCK_SYNC_MAX_RESYNC_DURATION *
         1000 / vp->clocksrc.ptime_usec;
 
@@ -309,7 +309,7 @@ PJ_DEF(pj_status_t) pjmedia_vid_port_create( pj_pool_t *pool,
          * may run at a different rate.
 	 */
 	need_frame_buf = PJ_TRUE;
-
+            
         param.usec_interval = PJMEDIA_PTIME(&vfd->fps);
         param.clock_rate = prm->vidparam.clock_rate;
         status = pjmedia_clock_create2(pool, &param,
@@ -332,9 +332,7 @@ PJ_DEF(pj_status_t) pjmedia_vid_port_create( pj_pool_t *pool,
 	                        PJMEDIA_SIG_VID_PORT,
 			        prm->vidparam.dir, &prm->vidparam.fmt);
 
-	if (vp->stream_role == ROLE_ACTIVE) {
-	    need_frame_buf = PJ_TRUE;
-	}
+        need_frame_buf = PJ_TRUE;
     }
 
     if (need_frame_buf) {
@@ -567,7 +565,7 @@ static pj_status_t vidstream_event_cb(pjmedia_event *event,
                                       void *user_data)
 {
     pjmedia_vid_port *vp = (pjmedia_vid_port*)user_data;
-
+    
     /* Just republish the event to our client */
     return pjmedia_event_publish(NULL, vp, event, 0);
 }
@@ -581,15 +579,15 @@ static pj_status_t client_port_event_cb(pjmedia_event *event,
         const pjmedia_video_format_detail *vfd;
         pjmedia_vid_dev_param vid_param;
         pj_status_t status;
-
+        
 	pjmedia_vid_port_stop(vp);
-
+        
         /* Retrieve the video format detail */
         vfd = pjmedia_format_get_video_format_detail(
                   &event->data.fmt_changed.new_fmt, PJ_TRUE);
         if (!vfd || !vfd->fps.num || !vfd->fps.denum)
             return PJMEDIA_EVID_BADFORMAT;
-
+        
 	/* Change the destination format to the new format */
 	pjmedia_format_copy(&vp->conv.conv_param.src,
 			    &event->data.fmt_changed.new_fmt);
@@ -622,10 +620,10 @@ static pj_status_t client_port_event_cb(pjmedia_event *event,
                 return status;
             }
         }
-
+        
         if (vp->stream_role == ROLE_PASSIVE) {
             pjmedia_clock_param clock_param;
-
+            
             /**
              * Initially, frm_buf was allocated the biggest
              * supported size, so we do not need to re-allocate
@@ -636,10 +634,10 @@ static pj_status_t client_port_event_cb(pjmedia_event *event,
             clock_param.clock_rate = vid_param.clock_rate;
             pjmedia_clock_modify(vp->clock, &clock_param);
         }
-
+        
 	pjmedia_vid_port_start(vp);
     }
-
+    
     /* Republish the event, post the event to the event manager
      * to avoid deadlock if vidport is trying to stop the clock.
      */
@@ -654,12 +652,14 @@ static pj_status_t convert_frame(pjmedia_vid_port *vp,
     pj_status_t status = PJ_SUCCESS;
 
     if (vp->conv.conv) {
-	dst_frame->buf  = vp->conv.conv_buf;
-	dst_frame->size = vp->conv.conv_buf_size;
+        if (!dst_frame->buf || dst_frame->size < vp->conv.conv_buf_size) {
+            dst_frame->buf  = vp->conv.conv_buf;
+	    dst_frame->size = vp->conv.conv_buf_size;
+        }
 	status = pjmedia_converter_convert(vp->conv.conv,
 					   src_frame, dst_frame);
     }
-
+    
     return status;
 }
 
@@ -684,7 +684,7 @@ static pj_status_t get_frame_from_buffer(pjmedia_vid_port *vp,
     else
         pjmedia_frame_copy(frame, vp->frm_buf);
     pj_mutex_unlock(vp->frm_mutex);
-
+    
     return status;
 }
 
@@ -747,7 +747,7 @@ static void dec_clock_cb(const pj_timestamp *ts, void *user_data)
     status = vidstream_render_cb(vp->strm, vp, &frame);
     if (status != PJ_SUCCESS)
         return;
-
+    
     if (frame.size > 0)
 	status = pjmedia_vid_dev_stream_put_frame(vp->strm, &frame);
 }
@@ -779,7 +779,7 @@ static pj_status_t vidstream_render_cb(pjmedia_vid_dev_stream *stream,
 {
     pjmedia_vid_port *vp = (pjmedia_vid_port*)user_data;
     pj_status_t status = PJ_SUCCESS;
-
+    
     pj_bzero(frame, sizeof(pjmedia_frame));
     if (vp->role==ROLE_ACTIVE) {
         unsigned frame_ts = vp->clocksrc.clock_rate / 1000 *
@@ -787,28 +787,28 @@ static pj_status_t vidstream_render_cb(pjmedia_vid_dev_stream *stream,
 
         if (!vp->client_port)
             return status;
-
+        
         if (vp->sync_clocksrc.sync_clocksrc) {
             pjmedia_clock_src *src = vp->sync_clocksrc.sync_clocksrc;
             pj_int32_t diff;
             unsigned nsync_frame;
-
+            
             /* Synchronization */
             /* Calculate the time difference (in ms) with the sync source */
             diff = pjmedia_clock_src_get_time_msec(&vp->clocksrc) -
                    pjmedia_clock_src_get_time_msec(src) -
                    vp->sync_clocksrc.sync_delta;
-
+            
             /* Check whether sync source made a large jump */
             if (diff < 0 && -diff > PJMEDIA_CLOCK_SYNC_MAX_SYNC_MSEC) {
                 pjmedia_clock_src_update(&vp->clocksrc, NULL);
-                vp->sync_clocksrc.sync_delta =
+                vp->sync_clocksrc.sync_delta = 
                     pjmedia_clock_src_get_time_msec(src) -
                     pjmedia_clock_src_get_time_msec(&vp->clocksrc);
                 vp->sync_clocksrc.nsync_frame = 0;
                 return status;
             }
-
+            
             /* Calculate the difference (in frames) with the sync source */
             nsync_frame = abs(diff) * 1000 / vp->clocksrc.ptime_usec;
             if (nsync_frame == 0) {
@@ -816,7 +816,7 @@ static pj_status_t vidstream_render_cb(pjmedia_vid_dev_stream *stream,
                 vp->sync_clocksrc.nsync_frame = 0;
             } else {
                 pj_int32_t init_sync_frame = nsync_frame;
-
+                
                 /* Check whether it's a new sync or whether we need to reset
                  * the sync
                  */
@@ -829,17 +829,17 @@ static pj_status_t vidstream_render_cb(pjmedia_vid_dev_stream *stream,
                 } else {
                     init_sync_frame = vp->sync_clocksrc.nsync_frame;
                 }
-
+                
                 if (diff >= 0) {
                     unsigned skip_mod;
-
+                    
                     /* We are too fast */
                     if (vp->sync_clocksrc.max_sync_ticks > 0) {
-                        skip_mod = init_sync_frame /
+                        skip_mod = init_sync_frame / 
                         vp->sync_clocksrc.max_sync_ticks + 2;
                     } else
                         skip_mod = init_sync_frame + 2;
-
+                    
                     PJ_LOG(5, (THIS_FILE, "synchronization: early by %d ms",
                                diff));
                     /* We'll play a frame every skip_mod-th tick instead of
@@ -851,7 +851,7 @@ static pj_status_t vidstream_render_cb(pjmedia_vid_dev_stream *stream,
                     }
                 } else {
                     unsigned i, ndrop = init_sync_frame;
-
+                    
                     /* We are too late, drop the frame */
                     if (vp->sync_clocksrc.max_sync_ticks > 0) {
                         ndrop /= vp->sync_clocksrc.max_sync_ticks;
@@ -859,13 +859,13 @@ static pj_status_t vidstream_render_cb(pjmedia_vid_dev_stream *stream,
                     }
                     PJ_LOG(5, (THIS_FILE, "synchronization: late, "
                                "dropping %d frame(s)", ndrop));
-
+                    
                     if (ndrop >= nsync_frame) {
                         vp->sync_clocksrc.nsync_frame = 0;
                         ndrop = nsync_frame;
                     } else
                         vp->sync_clocksrc.nsync_progress += ndrop;
-
+                    
                     for (i = 0; i < ndrop; i++) {
                         vp->frm_buf->size = vp->frm_buf_size;
                         status = pjmedia_port_get_frame(vp->client_port,
@@ -874,14 +874,14 @@ static pj_status_t vidstream_render_cb(pjmedia_vid_dev_stream *stream,
                             pjmedia_clock_src_update(&vp->clocksrc, NULL);
                             return status;
                         }
-
+                        
                         pj_add_timestamp32(&vp->clocksrc.timestamp,
                                            frame_ts);
                     }
                 }
             }
         }
-
+        
         vp->frm_buf->size = vp->frm_buf_size;
         status = pjmedia_port_get_frame(vp->client_port, vp->frm_buf);
         if (status != PJ_SUCCESS) {
@@ -921,7 +921,8 @@ static pj_status_t vid_pasv_port_put_frame(struct pjmedia_port *this_port,
          */
         pj_status_t status;
         pjmedia_frame frame_;
-
+        
+        pj_bzero(&frame_, sizeof(frame_));
         status = convert_frame(vp, frame, &frame_);
         if (status != PJ_SUCCESS)
             return status;
