@@ -1,5 +1,5 @@
-/* $Id: stun_sock_test.c 3553 2011-05-05 06:14:19Z nanang $ */
-/*
+/* $Id: stun_sock_test.c 4360 2013-02-21 11:26:35Z bennylp $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include "test.h"
 
@@ -67,9 +67,9 @@ static pj_bool_t srv_on_data_recvfrom(pj_activesock_t *asock,
 	pj_stun_msg *req_msg, *res_msg;
 
 	pool = pj_pool_create(mem, "stunsrv", 512, 512, NULL);
-
+    
 	/* Parse request */
-	status = pj_stun_msg_decode(pool, (pj_uint8_t*)data, size,
+	status = pj_stun_msg_decode(pool, (pj_uint8_t*)data, size, 
 				    PJ_STUN_IS_DATAGRAM | PJ_STUN_CHECK_PACKET,
 				    &req_msg, NULL, NULL);
 	if (status != PJ_SUCCESS) {
@@ -98,13 +98,13 @@ static pj_bool_t srv_on_data_recvfrom(pj_activesock_t *asock,
 	    pj_sockaddr_in addr;
 
 	    pj_sockaddr_in_init(&addr, &srv->ip_to_send, srv->port_to_send);
-	    pj_stun_msg_add_sockaddr_attr(pool, res_msg,
+	    pj_stun_msg_add_sockaddr_attr(pool, res_msg, 
 					  PJ_STUN_ATTR_XOR_MAPPED_ADDR,
 					  PJ_TRUE, &addr, sizeof(addr));
 	}
 
 	/* Encode */
-	status = pj_stun_msg_encode(res_msg, (pj_uint8_t*)data, 100, 0,
+	status = pj_stun_msg_encode(res_msg, (pj_uint8_t*)data, 100, 0, 
 				    NULL, &size);
 	if (status != PJ_SUCCESS) {
 	    app_perror("   pj_stun_msg_encode()", status);
@@ -114,7 +114,7 @@ static pj_bool_t srv_on_data_recvfrom(pj_activesock_t *asock,
 
 	/* Send back */
 	sent = size;
-	pj_activesock_sendto(asock, &srv->send_key, data, &sent, 0,
+	pj_activesock_sendto(asock, &srv->send_key, data, &sent, 0, 
 			     src_addr, addr_len);
 
 	pj_pool_release(pool);
@@ -122,7 +122,7 @@ static pj_bool_t srv_on_data_recvfrom(pj_activesock_t *asock,
     } else if (srv->flag & ECHO) {
 	/* Send back */
 	sent = size;
-	pj_activesock_sendto(asock, &srv->send_key, data, &sent, 0,
+	pj_activesock_sendto(asock, &srv->send_key, data, &sent, 0, 
 			     src_addr, addr_len);
 
     }
@@ -151,7 +151,7 @@ static pj_status_t create_server(pj_pool_t *pool,
     pj_bzero(&activesock_cb, sizeof(activesock_cb));
     activesock_cb.on_data_recvfrom = &srv_on_data_recvfrom;
     status = pj_activesock_create_udp(pool, &srv->addr, NULL, ioqueue,
-				      &activesock_cb, srv, &srv->asock,
+				      &activesock_cb, srv, &srv->asock, 
 				      &srv->addr);
     if (status != PJ_SUCCESS)
 	return status;
@@ -189,7 +189,7 @@ struct stun_client
     unsigned		 on_rx_data_cnt;
 };
 
-static pj_bool_t stun_sock_on_status(pj_stun_sock *stun_sock,
+static pj_bool_t stun_sock_on_status(pj_stun_sock *stun_sock, 
 				     pj_stun_sock_op op,
 				     pj_status_t status)
 {
@@ -298,7 +298,7 @@ static int timeout_test(pj_stun_config *cfg, pj_bool_t destroy_on_err)
     struct stun_client *client;
     pj_str_t srv_addr;
     pj_time_val timeout, t;
-    int ret = 0;
+    int i, ret = 0;
     pj_status_t status;
 
     PJ_LOG(3,(THIS_FILE, "  timeout test [%d]", destroy_on_err));
@@ -314,7 +314,7 @@ static int timeout_test(pj_stun_config *cfg, pj_bool_t destroy_on_err)
     }
 
     srv_addr = pj_str("127.0.0.1");
-    status = pj_stun_sock_start(client->sock, &srv_addr,
+    status = pj_stun_sock_start(client->sock, &srv_addr, 
 				pj_ntohs(srv->addr.ipv4.sin_port), NULL);
     if (status != PJ_SUCCESS) {
 	destroy_server(srv);
@@ -359,6 +359,8 @@ static int timeout_test(pj_stun_config *cfg, pj_bool_t destroy_on_err)
 on_return:
     destroy_server(srv);
     destroy_client(client);
+    for (i=0; i<7; ++i)
+	handle_events(cfg, 50);
     return ret;
 }
 
@@ -373,7 +375,7 @@ static int missing_attr_test(pj_stun_config *cfg, pj_bool_t destroy_on_err)
     struct stun_client *client;
     pj_str_t srv_addr;
     pj_time_val timeout, t;
-    int ret = 0;
+    int i, ret = 0;
     pj_status_t status;
 
     PJ_LOG(3,(THIS_FILE, "  missing attribute test [%d]", destroy_on_err));
@@ -389,7 +391,7 @@ static int missing_attr_test(pj_stun_config *cfg, pj_bool_t destroy_on_err)
     }
 
     srv_addr = pj_str("127.0.0.1");
-    status = pj_stun_sock_start(client->sock, &srv_addr,
+    status = pj_stun_sock_start(client->sock, &srv_addr, 
 				pj_ntohs(srv->addr.ipv4.sin_port), NULL);
     if (status != PJ_SUCCESS) {
 	destroy_server(srv);
@@ -426,6 +428,8 @@ static int missing_attr_test(pj_stun_config *cfg, pj_bool_t destroy_on_err)
 on_return:
     destroy_server(srv);
     destroy_client(client);
+    for (i=0; i<7; ++i)
+	handle_events(cfg, 50);
     return ret;
 }
 
@@ -440,7 +444,7 @@ static int keep_alive_test(pj_stun_config *cfg)
     pj_stun_sock_info info;
     pj_str_t srv_addr;
     pj_time_val timeout, t;
-    int ret = 0;
+    int i, ret = 0;
     pj_status_t status;
 
     PJ_LOG(3,(THIS_FILE, "  normal operation"));
@@ -460,7 +464,7 @@ static int keep_alive_test(pj_stun_config *cfg)
      */
     PJ_LOG(3,(THIS_FILE, "    initial Binding request"));
     srv_addr = pj_str("127.0.0.1");
-    status = pj_stun_sock_start(client->sock, &srv_addr,
+    status = pj_stun_sock_start(client->sock, &srv_addr, 
 				pj_ntohs(srv->addr.ipv4.sin_port), NULL);
     if (status != PJ_SUCCESS) {
 	destroy_server(srv);
@@ -554,7 +558,7 @@ static int keep_alive_test(pj_stun_config *cfg)
 	PJ_LOG(3,(THIS_FILE, "     sending to %s", pj_sockaddr_print(&info.srv_addr, txt, sizeof(txt), 3)));
     }
     status = pj_stun_sock_sendto(client->sock, NULL, &ret, sizeof(ret),
-				 0, &info.srv_addr,
+				 0, &info.srv_addr, 
 				 pj_sockaddr_get_len(&info.srv_addr));
     if (status != PJ_SUCCESS && status != PJ_EPENDING) {
 	app_perror("    error: server sending data", status);
@@ -749,7 +753,7 @@ static int keep_alive_test(pj_stun_config *cfg)
      * Part 5: Failed keep-alive
      */
     PJ_LOG(3,(THIS_FILE, "    failed keep-alive scenario"));
-
+    
     /* Change server operation mode to respond without attribute */
     srv->flag = RESPOND_STUN;
 
@@ -791,6 +795,8 @@ static int keep_alive_test(pj_stun_config *cfg)
 on_return:
     destroy_server(srv);
     destroy_client(client);
+    for (i=0; i<7; ++i)
+	handle_events(cfg, 50);
     return ret;
 }
 
@@ -828,7 +834,7 @@ int stun_sock_test(void)
 	ret = -8;
 	goto on_return;
     }
-
+    
     pj_stun_config_init(&stun_cfg, mem, 0, ioqueue, timer_heap);
 
     DO_TEST(timeout_test(&stun_cfg, PJ_FALSE));
@@ -845,3 +851,5 @@ on_return:
     if (pool) pj_pool_release(pool);
     return ret;
 }
+
+
