@@ -1,5 +1,5 @@
-/* $Id: ice_strans.c 4133 2012-05-21 14:00:17Z bennylp $ */
-/*
+/* $Id: ice_strans.c 4412 2013-03-05 03:12:32Z riza $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjnath/ice_strans.h>
 #include <pjnath/errno.h>
@@ -31,8 +31,9 @@
 #include <pj/string.h>
 #include <pj/compat/socket.h>
 
+#define ENABLE_TRACE 0
 
-#if 0
+#if defined(ENABLE_TRACE) && (ENABLE_TRACE != 0)
 #  define TRACE_PKT(expr)	    PJ_LOG(5,expr)
 #else
 #  define TRACE_PKT(expr)
@@ -64,7 +65,7 @@ enum tp_type
 
 
 /* The candidate type preference when STUN candidate is used */
-static pj_uint8_t srflx_pref_table[4] =
+static pj_uint8_t srflx_pref_table[4] = 
 {
 #if PJNATH_ICE_PRIO_STD
     100,    /**< PJ_ICE_HOST_PREF	    */
@@ -83,14 +84,14 @@ static pj_uint8_t srflx_pref_table[4] =
 
 /* ICE callbacks */
 static void	   on_ice_complete(pj_ice_sess *ice, pj_status_t status);
-static pj_status_t ice_tx_pkt(pj_ice_sess *ice,
+static pj_status_t ice_tx_pkt(pj_ice_sess *ice, 
 			      unsigned comp_id,
 			      unsigned transport_id,
 			      const void *pkt, pj_size_t size,
 			      const pj_sockaddr_t *dst_addr,
 			      unsigned dst_addr_len);
-static void	   ice_rx_data(pj_ice_sess *ice,
-			       unsigned comp_id,
+static void	   ice_rx_data(pj_ice_sess *ice, 
+			       unsigned comp_id, 
 			       unsigned transport_id,
 			       void *pkt, pj_size_t size,
 			       const pj_sockaddr_t *src_addr,
@@ -109,7 +110,7 @@ static pj_bool_t stun_on_data_sent(pj_stun_sock *stun_sock,
 				   pj_ioqueue_op_key_t *send_key,
 				   pj_ssize_t sent);
 /* Notification when the status of the STUN transport has changed. */
-static pj_bool_t stun_on_status(pj_stun_sock *stun_sock,
+static pj_bool_t stun_on_status(pj_stun_sock *stun_sock, 
 				pj_stun_sock_op op,
 				pj_status_t status);
 
@@ -126,18 +127,16 @@ static void turn_on_state(pj_turn_sock *turn_sock, pj_turn_state_t old_state,
 
 
 /* Forward decls */
+static void ice_st_on_destroy(void *obj);
 static void destroy_ice_st(pj_ice_strans *ice_st);
 #define ice_st_perror(ice_st,msg,rc) pjnath_perror(ice_st->obj_name,msg,rc)
 static void sess_init_update(pj_ice_strans *ice_st);
 
-static void sess_add_ref(pj_ice_strans *ice_st);
-static pj_bool_t sess_dec_ref(pj_ice_strans *ice_st);
-
 /**
  * This structure describes an ICE stream transport component. A component
  * in ICE stream transport typically corresponds to a single socket created
  * for this component, and bound to a specific transport address. This
- * component may have multiple alias addresses, for example one alias
+ * component may have multiple alias addresses, for example one alias 
  * address for each interfaces in multi-homed host, another for server
  * reflexive alias, and another for relayed alias. For each transport
  * address alias, an ICE stream transport candidate (#pj_ice_sess_cand) will
@@ -172,7 +171,7 @@ struct pj_ice_strans
     void		    *user_data;	/**< Application data.		*/
     pj_ice_strans_cfg	     cfg;	/**< Configuration.		*/
     pj_ice_strans_cb	     cb;	/**< Application callback.	*/
-    pj_lock_t		    *init_lock; /**< Initialization mutex.	*/
+    pj_grp_lock_t	    *grp_lock;  /**< Group lock.		*/
 
     pj_ice_strans_state	     state;	/**< Session state.		*/
     pj_ice_sess		    *ice;	/**< ICE session.		*/
@@ -183,7 +182,6 @@ struct pj_ice_strans
 
     pj_timer_entry	     ka_timer;	/**< STUN keep-alive timer.	*/
 
-    pj_atomic_t		    *busy_cnt;	/**< To prevent destroy		*/
     pj_bool_t		     destroy_req;/**< Destroy has been called?	*/
     pj_bool_t		     cb_called;	/**< Init error callback called?*/
 };
@@ -373,10 +371,10 @@ static pj_status_t create_comp(pj_ice_strans *ice_st, unsigned comp_id)
 	stun_sock_cb.on_rx_data = &stun_on_rx_data;
 	stun_sock_cb.on_status = &stun_on_status;
 	stun_sock_cb.on_data_sent = &stun_on_data_sent;
-
+	
 	/* Override component specific QoS settings, if any */
 	if (ice_st->cfg.comp[comp_id-1].qos_type) {
-	    ice_st->cfg.stun.cfg.qos_type =
+	    ice_st->cfg.stun.cfg.qos_type = 
 		ice_st->cfg.comp[comp_id-1].qos_type;
 	}
 	if (ice_st->cfg.comp[comp_id-1].qos_params.flags) {
@@ -393,8 +391,8 @@ static pj_status_t create_comp(pj_ice_strans *ice_st, unsigned comp_id)
 	if (status != PJ_SUCCESS)
 	    return status;
 
-	/* Start STUN Binding resolution and add srflx candidate
-	 * only if server is set
+	/* Start STUN Binding resolution and add srflx candidate 
+	 * only if server is set 
 	 */
 	if (ice_st->cfg.stun.server.slen) {
 	    pj_stun_sock_info stun_sock_info;
@@ -402,16 +400,16 @@ static pj_status_t create_comp(pj_ice_strans *ice_st, unsigned comp_id)
 	    /* Add pending job */
 	    ///sess_add_ref(ice_st);
 
-	    PJ_LOG(4,(ice_st->obj_name,
+	    PJ_LOG(4,(ice_st->obj_name, 
 		      "Comp %d: srflx candidate starts Binding discovery",
 		      comp_id));
 
 	    pj_log_push_indent();
 
 	    /* Start Binding resolution */
-	    status = pj_stun_sock_start(comp->stun_sock,
+	    status = pj_stun_sock_start(comp->stun_sock, 
 					&ice_st->cfg.stun.server,
-					ice_st->cfg.stun.port,
+					ice_st->cfg.stun.port, 
 					ice_st->cfg.resolver);
 	    if (status != PJ_SUCCESS) {
 		///sess_dec_ref(ice_st);
@@ -457,8 +455,8 @@ static pj_status_t create_comp(pj_ice_strans *ice_st, unsigned comp_id)
 	    if (status != PJ_SUCCESS)
 		return status;
 
-	    for (i=0; i<stun_sock_info.alias_cnt &&
-		      i<ice_st->cfg.stun.max_host_cands; ++i)
+	    for (i=0; i<stun_sock_info.alias_cnt && 
+		      i<ice_st->cfg.stun.max_host_cands; ++i) 
 	    {
 		char addrinfo[PJ_INET6_ADDRSTRLEN+10];
 		const pj_sockaddr *addr = &stun_sock_info.aliases[i];
@@ -469,8 +467,8 @@ static pj_status_t create_comp(pj_ice_strans *ice_st, unsigned comp_id)
 		    break;
 		}
 
-		/* Ignore loopback addresses unless cfg->stun.loop_addr
-		 * is set
+		/* Ignore loopback addresses unless cfg->stun.loop_addr 
+		 * is set 
 		 */
 		if ((pj_ntohl(addr->ipv4.sin_addr.s_addr)>>24)==127) {
 		    if (ice_st->cfg.stun.loop_addr==PJ_FALSE)
@@ -490,7 +488,7 @@ static pj_status_t create_comp(pj_ice_strans *ice_st, unsigned comp_id)
 		pj_ice_calc_foundation(ice_st->pool, &cand->foundation,
 				       cand->type, &cand->base_addr);
 
-		PJ_LOG(4,(ice_st->obj_name,
+		PJ_LOG(4,(ice_st->obj_name, 
 			  "Comp %d: host candidate %s added",
 			  comp_id, pj_sockaddr_print(&cand->addr, addrinfo,
 						     sizeof(addrinfo), 3)));
@@ -503,12 +501,19 @@ static pj_status_t create_comp(pj_ice_strans *ice_st, unsigned comp_id)
 	add_update_turn(ice_st, comp);
     }
 
+    /* It's possible that we end up without any candidates */
+    if (comp->cand_cnt == 0) {
+	PJ_LOG(4,(ice_st->obj_name,
+		  "Error: no candidate is created due to settings"));
+	return PJ_EINVAL;
+    }
+
     return PJ_SUCCESS;
 }
 
 
-/*
- * Create ICE stream transport
+/* 
+ * Create ICE stream transport 
  */
 PJ_DEF(pj_status_t) pj_ice_strans_create( const char *name,
 					  const pj_ice_strans_cfg *cfg,
@@ -539,44 +544,43 @@ PJ_DEF(pj_status_t) pj_ice_strans_create( const char *name,
     ice_st->obj_name = pool->obj_name;
     ice_st->user_data = user_data;
 
-    PJ_LOG(4,(ice_st->obj_name,
+    PJ_LOG(4,(ice_st->obj_name, 
 	      "Creating ICE stream transport with %d component(s)",
 	      comp_cnt));
     pj_log_push_indent();
 
-    pj_ice_strans_cfg_copy(pool, &ice_st->cfg, cfg);
-    pj_memcpy(&ice_st->cb, cb, sizeof(*cb));
-
-    status = pj_atomic_create(pool, 0, &ice_st->busy_cnt);
+    status = pj_grp_lock_create(pool, NULL, &ice_st->grp_lock);
     if (status != PJ_SUCCESS) {
-	destroy_ice_st(ice_st);
-	return status;
-    }
-
-    status = pj_lock_create_recursive_mutex(pool, ice_st->obj_name,
-					    &ice_st->init_lock);
-    if (status != PJ_SUCCESS) {
-	destroy_ice_st(ice_st);
+	pj_pool_release(pool);
 	pj_log_pop_indent();
 	return status;
     }
 
+    pj_grp_lock_add_ref(ice_st->grp_lock);
+    pj_grp_lock_add_handler(ice_st->grp_lock, pool, ice_st,
+			    &ice_st_on_destroy);
+
+    pj_ice_strans_cfg_copy(pool, &ice_st->cfg, cfg);
+    ice_st->cfg.stun.cfg.grp_lock = ice_st->grp_lock;
+    ice_st->cfg.turn.cfg.grp_lock = ice_st->grp_lock;
+    pj_memcpy(&ice_st->cb, cb, sizeof(*cb));
+
     ice_st->comp_cnt = comp_cnt;
-    ice_st->comp = (pj_ice_strans_comp**)
+    ice_st->comp = (pj_ice_strans_comp**) 
 		   pj_pool_calloc(pool, comp_cnt, sizeof(pj_ice_strans_comp*));
 
     /* Move state to candidate gathering */
     ice_st->state = PJ_ICE_STRANS_STATE_INIT;
 
-    /* Acquire initialization mutex to prevent callback to be
+    /* Acquire initialization mutex to prevent callback to be 
      * called before we finish initialization.
      */
-    pj_lock_acquire(ice_st->init_lock);
+    pj_grp_lock_acquire(ice_st->grp_lock);
 
     for (i=0; i<comp_cnt; ++i) {
 	status = create_comp(ice_st, i+1);
 	if (status != PJ_SUCCESS) {
-	    pj_lock_release(ice_st->init_lock);
+	    pj_grp_lock_release(ice_st->grp_lock);
 	    destroy_ice_st(ice_st);
 	    pj_log_pop_indent();
 	    return status;
@@ -584,9 +588,9 @@ PJ_DEF(pj_status_t) pj_ice_strans_create( const char *name,
     }
 
     /* Done with initialization */
-    pj_lock_release(ice_st->init_lock);
+    pj_grp_lock_release(ice_st->grp_lock);
 
-    PJ_LOG(4,(ice_st->obj_name, "ICE stream transport created"));
+    PJ_LOG(4,(ice_st->obj_name, "ICE stream transport %p created", ice_st));
 
     *p_ice_st = ice_st;
 
@@ -598,14 +602,35 @@ PJ_DEF(pj_status_t) pj_ice_strans_create( const char *name,
     return PJ_SUCCESS;
 }
 
+/* REALLY destroy ICE */
+static void ice_st_on_destroy(void *obj)
+{
+    pj_ice_strans *ice_st = (pj_ice_strans*)obj;
+
+    PJ_LOG(4,(ice_st->obj_name, "ICE stream transport %p destroyed", obj));
+
+    /* Done */
+    pj_pool_release(ice_st->pool);
+}
+
 /* Destroy ICE */
 static void destroy_ice_st(pj_ice_strans *ice_st)
 {
     unsigned i;
 
-    PJ_LOG(5,(ice_st->obj_name, "ICE stream transport destroying.."));
+    PJ_LOG(5,(ice_st->obj_name, "ICE stream transport %p destroy request..",
+	      ice_st));
     pj_log_push_indent();
 
+    pj_grp_lock_acquire(ice_st->grp_lock);
+
+    if (ice_st->destroy_req) {
+	pj_grp_lock_release(ice_st->grp_lock);
+	return;
+    }
+
+    ice_st->destroy_req = PJ_TRUE;
+
     /* Destroy ICE if we have ICE */
     if (ice_st->ice) {
 	pj_ice_sess_destroy(ice_st->ice);
@@ -616,38 +641,19 @@ static void destroy_ice_st(pj_ice_strans *ice_st)
     for (i=0; i<ice_st->comp_cnt; ++i) {
 	if (ice_st->comp[i]) {
 	    if (ice_st->comp[i]->stun_sock) {
-		pj_stun_sock_set_user_data(ice_st->comp[i]->stun_sock, NULL);
 		pj_stun_sock_destroy(ice_st->comp[i]->stun_sock);
 		ice_st->comp[i]->stun_sock = NULL;
 	    }
 	    if (ice_st->comp[i]->turn_sock) {
-		pj_turn_sock_set_user_data(ice_st->comp[i]->turn_sock, NULL);
 		pj_turn_sock_destroy(ice_st->comp[i]->turn_sock);
 		ice_st->comp[i]->turn_sock = NULL;
 	    }
 	}
     }
-    ice_st->comp_cnt = 0;
 
-    /* Destroy mutex */
-    if (ice_st->init_lock) {
-	pj_lock_acquire(ice_st->init_lock);
-	pj_lock_release(ice_st->init_lock);
-	pj_lock_destroy(ice_st->init_lock);
-	ice_st->init_lock = NULL;
-    }
+    pj_grp_lock_dec_ref(ice_st->grp_lock);
+    pj_grp_lock_release(ice_st->grp_lock);
 
-    /* Destroy reference counter */
-    if (ice_st->busy_cnt) {
-	pj_assert(pj_atomic_get(ice_st->busy_cnt)==0);
-	pj_atomic_destroy(ice_st->busy_cnt);
-	ice_st->busy_cnt = NULL;
-    }
-
-    PJ_LOG(4,(ice_st->obj_name, "ICE stream transport destroyed"));
-
-    /* Done */
-    pj_pool_release(ice_st->pool);
     pj_log_pop_indent();
 }
 
@@ -723,7 +729,7 @@ static void sess_init_update(pj_ice_strans *ice_st)
     ice_st->cb_called = PJ_TRUE;
     ice_st->state = PJ_ICE_STRANS_STATE_READY;
     if (ice_st->cb.on_ice_complete)
-	(*ice_st->cb.on_ice_complete)(ice_st, PJ_ICE_STRANS_OP_INIT,
+	(*ice_st->cb.on_ice_complete)(ice_st, PJ_ICE_STRANS_OP_INIT, 
 				      PJ_SUCCESS);
 }
 
@@ -732,44 +738,11 @@ static void sess_init_update(pj_ice_strans *ice_st)
  */
 PJ_DEF(pj_status_t) pj_ice_strans_destroy(pj_ice_strans *ice_st)
 {
-    PJ_ASSERT_RETURN(ice_st, PJ_EINVAL);
-
-    ice_st->destroy_req = PJ_TRUE;
-    if (pj_atomic_get(ice_st->busy_cnt) > 0) {
-	PJ_LOG(5,(ice_st->obj_name,
-		  "ICE strans object is busy, will destroy later"));
-	return PJ_EPENDING;
-    }
-
     destroy_ice_st(ice_st);
     return PJ_SUCCESS;
 }
 
 
-/*
- * Increment busy counter.
- */
-static void sess_add_ref(pj_ice_strans *ice_st)
-{
-    pj_atomic_inc(ice_st->busy_cnt);
-}
-
-/*
- * Decrement busy counter. If the counter has reached zero and destroy
- * has been requested, destroy the object and return FALSE.
- */
-static pj_bool_t sess_dec_ref(pj_ice_strans *ice_st)
-{
-    int count = pj_atomic_dec_and_get(ice_st->busy_cnt);
-    pj_assert(count >= 0);
-    if (count==0 && ice_st->destroy_req) {
-	pj_ice_strans_destroy(ice_st);
-	return PJ_FALSE;
-    } else {
-	return PJ_TRUE;
-    }
-}
-
 /*
  * Get user data
  */
@@ -792,7 +765,7 @@ PJ_DEF(pj_status_t) pj_ice_strans_get_options( pj_ice_strans *ice_st,
 }
 
 /*
- * Specify various options for this ICE stream transport.
+ * Specify various options for this ICE stream transport. 
  */
 PJ_DEF(pj_status_t) pj_ice_strans_set_options(pj_ice_strans *ice_st,
 					      const pj_ice_sess_options *opt)
@@ -832,8 +805,10 @@ PJ_DEF(pj_status_t) pj_ice_strans_init_ice(pj_ice_strans *ice_st,
 
     /* Create! */
     status = pj_ice_sess_create(&ice_st->cfg.stun_cfg, ice_st->obj_name, role,
-			        ice_st->comp_cnt, &ice_cb,
-			        local_ufrag, local_passwd, &ice_st->ice);
+			        ice_st->comp_cnt, &ice_cb, 
+			        local_ufrag, local_passwd,
+			        ice_st->grp_lock,
+			        &ice_st->ice);
     if (status != PJ_SUCCESS)
 	return status;
 
@@ -848,7 +823,7 @@ PJ_DEF(pj_status_t) pj_ice_strans_init_ice(pj_ice_strans *ice_st,
      * checked first.
      */
     if (ice_st->comp[0]->default_cand >= 0 &&
-	ice_st->comp[0]->cand_list[ice_st->comp[0]->default_cand].type
+	ice_st->comp[0]->cand_list[ice_st->comp[0]->default_cand].type 
 	    == PJ_ICE_CAND_TYPE_SRFLX)
     {
 	pj_ice_sess_set_prefs(ice_st->ice, srflx_pref_table);
@@ -861,7 +836,7 @@ PJ_DEF(pj_status_t) pj_ice_strans_init_ice(pj_ice_strans *ice_st,
 
 	/* Re-enable logging for Send/Data indications */
 	if (comp->turn_sock) {
-	    PJ_LOG(5,(ice_st->obj_name,
+	    PJ_LOG(5,(ice_st->obj_name, 
 		      "Disabling STUN Indication logging for "
 		      "component %d", i+1));
 	    pj_turn_sock_set_log(comp->turn_sock, 0xFFFF);
@@ -874,7 +849,7 @@ PJ_DEF(pj_status_t) pj_ice_strans_init_ice(pj_ice_strans *ice_st,
 
 	    /* Skip if candidate is not ready */
 	    if (cand->status != PJ_SUCCESS) {
-		PJ_LOG(5,(ice_st->obj_name,
+		PJ_LOG(5,(ice_st->obj_name, 
 			  "Candidate %d of comp %d is not added (pending)",
 			  j, i));
 		continue;
@@ -884,10 +859,10 @@ PJ_DEF(pj_status_t) pj_ice_strans_init_ice(pj_ice_strans *ice_st,
 	    pj_assert(pj_sockaddr_has_addr(&cand->addr));
 
 	    /* Add the candidate */
-	    status = pj_ice_sess_add_cand(ice_st->ice, comp->comp_id,
-					  cand->transport_id, cand->type,
-					  cand->local_pref,
-					  &cand->foundation, &cand->addr,
+	    status = pj_ice_sess_add_cand(ice_st->ice, comp->comp_id, 
+					  cand->transport_id, cand->type, 
+					  cand->local_pref, 
+					  &cand->foundation, &cand->addr, 
 					  &cand->base_addr,  &cand->rel_addr,
 					  pj_sockaddr_get_len(&cand->addr),
 					  (unsigned*)&ice_cand_id);
@@ -907,7 +882,7 @@ on_error:
 }
 
 /*
- * Check if the ICE stream transport has the ICE session created.
+ * Check if the ICE stream transport has the ICE session created. 
  */
 PJ_DEF(pj_bool_t) pj_ice_strans_has_sess(pj_ice_strans *ice_st)
 {
@@ -980,7 +955,7 @@ PJ_DEF(unsigned) pj_ice_strans_get_cands_count(pj_ice_strans *ice_st,
 {
     unsigned i, cnt;
 
-    PJ_ASSERT_RETURN(ice_st && ice_st->ice && comp_id &&
+    PJ_ASSERT_RETURN(ice_st && ice_st->ice && comp_id && 
 		     comp_id <= ice_st->comp_cnt, 0);
 
     cnt = 0;
@@ -1003,7 +978,7 @@ PJ_DEF(pj_status_t) pj_ice_strans_enum_cands(pj_ice_strans *ice_st,
 {
     unsigned i, cnt;
 
-    PJ_ASSERT_RETURN(ice_st && ice_st->ice && comp_id &&
+    PJ_ASSERT_RETURN(ice_st && ice_st->ice && comp_id && 
 		     comp_id <= ice_st->comp_cnt && count && cand, PJ_EINVAL);
 
     cnt = 0;
@@ -1037,7 +1012,7 @@ PJ_DEF(pj_status_t) pj_ice_strans_get_def_cand( pj_ice_strans *ice_st,
     } else {
 	pj_ice_strans_comp *comp = ice_st->comp[comp_id - 1];
 	pj_assert(comp->default_cand>=0 && comp->default_cand<comp->cand_cnt);
-	pj_memcpy(cand, &comp->cand_list[comp->default_cand],
+	pj_memcpy(cand, &comp->cand_list[comp->default_cand], 
 		  sizeof(pj_ice_sess_cand));
     }
     return PJ_SUCCESS;
@@ -1103,7 +1078,7 @@ PJ_DEF(pj_status_t) pj_ice_strans_start_ice( pj_ice_strans *ice_st,
 	    }
 
 	    if (count) {
-		status = pj_turn_sock_set_perm(comp->turn_sock, count,
+		status = pj_turn_sock_set_perm(comp->turn_sock, count, 
 					       addrs, 0);
 		if (status != PJ_SUCCESS) {
 		    pj_ice_strans_stop_ice(ice_st);
@@ -1127,16 +1102,16 @@ PJ_DEF(pj_status_t) pj_ice_strans_start_ice( pj_ice_strans *ice_st,
 /*
  * Get valid pair.
  */
-PJ_DEF(const pj_ice_sess_check*)
+PJ_DEF(const pj_ice_sess_check*) 
 pj_ice_strans_get_valid_pair(const pj_ice_strans *ice_st,
 			     unsigned comp_id)
 {
     PJ_ASSERT_RETURN(ice_st && comp_id && comp_id <= ice_st->comp_cnt,
 		     NULL);
-
+    
     if (ice_st->ice == NULL)
 	return NULL;
-
+    
     return ice_st->ice->comp[comp_id-1].valid_check;
 }
 
@@ -1145,6 +1120,8 @@ pj_ice_strans_get_valid_pair(const pj_ice_strans *ice_st,
  */
 PJ_DEF(pj_status_t) pj_ice_strans_stop_ice(pj_ice_strans *ice_st)
 {
+    PJ_ASSERT_RETURN(ice_st, PJ_EINVAL);
+
     if (ice_st->ice) {
 	pj_ice_sess_destroy(ice_st->ice);
 	ice_st->ice = NULL;
@@ -1213,7 +1190,7 @@ PJ_DEF(pj_status_t) pj_ice_strans_sendto( pj_ice_strans *ice_st,
 
 	    if (!comp->turn_log_off) {
 		/* Disable logging for Send/Data indications */
-		PJ_LOG(5,(ice_st->obj_name,
+		PJ_LOG(5,(ice_st->obj_name, 
 			  "Disabling STUN Indication logging for "
 			  "component %d", comp->comp_id));
 		pj_turn_sock_set_log(comp->turn_sock, msg_disable_ind);
@@ -1222,13 +1199,13 @@ PJ_DEF(pj_status_t) pj_ice_strans_sendto( pj_ice_strans *ice_st,
 
 	    status = pj_turn_sock_sendto(comp->turn_sock, (const pj_uint8_t*)data, data_len,
 					 dst_addr, dst_addr_len);
-	    return (status==PJ_SUCCESS||status==PJ_EPENDING) ?
+	    return (status==PJ_SUCCESS||status==PJ_EPENDING) ? 
 		    PJ_SUCCESS : status;
 	} else {
 	    pkt_size = data_len;
-	    status = pj_stun_sock_sendto(comp->stun_sock, NULL, data,
+	    status = pj_stun_sock_sendto(comp->stun_sock, NULL, data, 
 					 data_len, 0, dst_addr, dst_addr_len);
-	    return (status==PJ_SUCCESS||status==PJ_EPENDING) ?
+	    return (status==PJ_SUCCESS||status==PJ_EPENDING) ? 
 		    PJ_SUCCESS : status;
 	}
 
@@ -1246,7 +1223,7 @@ static void on_ice_complete(pj_ice_sess *ice, pj_status_t status)
     pj_time_val t;
     unsigned msec;
 
-    sess_add_ref(ice_st);
+    pj_grp_lock_add_ref(ice_st->grp_lock);
 
     pj_gettimeofday(&t);
     PJ_TIME_VAL_SUB(t, ice_st->start_time);
@@ -1256,8 +1233,8 @@ static void on_ice_complete(pj_ice_sess *ice, pj_status_t status)
 	if (status != PJ_SUCCESS) {
 	    char errmsg[PJ_ERR_MSG_SIZE];
 	    pj_strerror(status, errmsg, sizeof(errmsg));
-	    PJ_LOG(4,(ice_st->obj_name,
-		      "ICE negotiation failed after %ds:%03d: %s",
+	    PJ_LOG(4,(ice_st->obj_name, 
+		      "ICE negotiation failed after %ds:%03d: %s", 
 		      msec/1000, msec%1000, errmsg));
 	} else {
 	    unsigned i;
@@ -1267,7 +1244,7 @@ static void on_ice_complete(pj_ice_sess *ice, pj_status_t status)
 				    PJ_STUN_SESS_LOG_RX_IND)
 	    };
 
-	    PJ_LOG(4,(ice_st->obj_name,
+	    PJ_LOG(4,(ice_st->obj_name, 
 		      "ICE negotiation success after %ds:%03d",
 		      msec/1000, msec%1000));
 
@@ -1279,9 +1256,9 @@ static void on_ice_complete(pj_ice_sess *ice, pj_status_t status)
 		    char lip[PJ_INET6_ADDRSTRLEN+10];
 		    char rip[PJ_INET6_ADDRSTRLEN+10];
 
-		    pj_sockaddr_print(&check->lcand->addr, lip,
+		    pj_sockaddr_print(&check->lcand->addr, lip, 
 				      sizeof(lip), 3);
-		    pj_sockaddr_print(&check->rcand->addr, rip,
+		    pj_sockaddr_print(&check->rcand->addr, rip, 
 				      sizeof(rip), 3);
 
 		    if (check->lcand->transport_id == TP_TURN) {
@@ -1294,7 +1271,7 @@ static void on_ice_complete(pj_ice_sess *ice, pj_status_t status)
 					sizeof(check->rcand->addr));
 
 			/* Disable logging for Send/Data indications */
-			PJ_LOG(5,(ice_st->obj_name,
+			PJ_LOG(5,(ice_st->obj_name, 
 				  "Disabling STUN Indication logging for "
 				  "component %d", i+1));
 			pj_turn_sock_set_log(ice_st->comp[i]->turn_sock,
@@ -1305,14 +1282,14 @@ static void on_ice_complete(pj_ice_sess *ice, pj_status_t status)
 		    PJ_LOG(4,(ice_st->obj_name, " Comp %d: "
 			      "sending from %s candidate %s to "
 			      "%s candidate %s",
-			      i+1,
+			      i+1, 
 			      pj_ice_get_cand_type_name(check->lcand->type),
 			      lip,
 			      pj_ice_get_cand_type_name(check->rcand->type),
 			      rip));
-
+			      
 		} else {
-		    PJ_LOG(4,(ice_st->obj_name,
+		    PJ_LOG(4,(ice_st->obj_name, 
 			      "Comp %d: disabled", i+1));
 		}
 	    }
@@ -1322,20 +1299,20 @@ static void on_ice_complete(pj_ice_sess *ice, pj_status_t status)
 					       PJ_ICE_STRANS_STATE_FAILED;
 
 	pj_log_push_indent();
-	(*ice_st->cb.on_ice_complete)(ice_st, PJ_ICE_STRANS_OP_NEGOTIATION,
+	(*ice_st->cb.on_ice_complete)(ice_st, PJ_ICE_STRANS_OP_NEGOTIATION, 
 				      status);
 	pj_log_pop_indent();
-
+	
     }
 
-    sess_dec_ref(ice_st);
+    pj_grp_lock_dec_ref(ice_st->grp_lock);
 }
 
 /*
  * Callback called by ICE session when it wants to send outgoing packet.
  */
-static pj_status_t ice_tx_pkt(pj_ice_sess *ice,
-			      unsigned comp_id,
+static pj_status_t ice_tx_pkt(pj_ice_sess *ice, 
+			      unsigned comp_id, 
 			      unsigned transport_id,
 			      const void *pkt, pj_size_t size,
 			      const pj_sockaddr_t *dst_addr,
@@ -1344,43 +1321,46 @@ static pj_status_t ice_tx_pkt(pj_ice_sess *ice,
     pj_ice_strans *ice_st = (pj_ice_strans*)ice->user_data;
     pj_ice_strans_comp *comp;
     pj_status_t status;
+#if defined(ENABLE_TRACE) && (ENABLE_TRACE != 0)
+    char daddr[PJ_INET6_ADDRSTRLEN];
+#endif
 
     PJ_ASSERT_RETURN(comp_id && comp_id <= ice_st->comp_cnt, PJ_EINVAL);
 
     comp = ice_st->comp[comp_id-1];
 
-    TRACE_PKT((comp->ice_st->obj_name,
-	      "Component %d TX packet to %s:%d with transport %d",
-	      comp_id,
-	      pj_inet_ntoa(((pj_sockaddr_in*)dst_addr)->sin_addr),
-	      (int)pj_ntohs(((pj_sockaddr_in*)dst_addr)->sin_port),
-	      transport_id));
+    TRACE_PKT((comp->ice_st->obj_name, 
+	       "Component %d TX packet to %s:%d with transport %d",
+	       comp_id, 
+	       pj_sockaddr_print(dst_addr, daddr, sizeof(addr), 0),
+	       pj_sockaddr_get_port(dst_addr),
+	       transport_id));
 
     if (transport_id == TP_TURN) {
 	if (comp->turn_sock) {
-	    status = pj_turn_sock_sendto(comp->turn_sock,
+	    status = pj_turn_sock_sendto(comp->turn_sock, 
 					 (const pj_uint8_t*)pkt, size,
 					 dst_addr, dst_addr_len);
 	} else {
 	    status = PJ_EINVALIDOP;
 	}
     } else if (transport_id == TP_STUN) {
-	status = pj_stun_sock_sendto(comp->stun_sock, NULL,
+	status = pj_stun_sock_sendto(comp->stun_sock, NULL, 
 				     pkt, size, 0,
 				     dst_addr, dst_addr_len);
     } else {
 	pj_assert(!"Invalid transport ID");
 	status = PJ_EINVALIDOP;
     }
-
+    
     return (status==PJ_SUCCESS||status==PJ_EPENDING) ? PJ_SUCCESS : status;
 }
 
 /*
  * Callback called by ICE session when it receives application data.
  */
-static void ice_rx_data(pj_ice_sess *ice,
-		        unsigned comp_id,
+static void ice_rx_data(pj_ice_sess *ice, 
+		        unsigned comp_id, 
 			unsigned transport_id,
 		        void *pkt, pj_size_t size,
 		        const pj_sockaddr_t *src_addr,
@@ -1391,13 +1371,13 @@ static void ice_rx_data(pj_ice_sess *ice,
     PJ_UNUSED_ARG(transport_id);
 
     if (ice_st->cb.on_rx_data) {
-	(*ice_st->cb.on_rx_data)(ice_st, comp_id, pkt, size,
+	(*ice_st->cb.on_rx_data)(ice_st, comp_id, pkt, size, 
 				 src_addr, src_addr_len);
     }
 }
 
 /* Notification when incoming packet has been received from
- * the STUN socket.
+ * the STUN socket. 
  */
 static pj_bool_t stun_on_rx_data(pj_stun_sock *stun_sock,
 				 void *pkt,
@@ -1417,7 +1397,7 @@ static pj_bool_t stun_on_rx_data(pj_stun_sock *stun_sock,
 
     ice_st = comp->ice_st;
 
-    sess_add_ref(ice_st);
+    pj_grp_lock_add_ref(ice_st->grp_lock);
 
     if (ice_st->ice == NULL) {
 	/* The ICE session is gone, but we're still receiving packets.
@@ -1425,28 +1405,28 @@ static pj_bool_t stun_on_rx_data(pj_stun_sock *stun_sock,
 	 * report this to application.
 	 */
 	if (ice_st->cb.on_rx_data) {
-	    (*ice_st->cb.on_rx_data)(ice_st, comp->comp_id, pkt, pkt_len,
+	    (*ice_st->cb.on_rx_data)(ice_st, comp->comp_id, pkt, pkt_len, 
 				     src_addr, addr_len);
 	}
 
     } else {
 
 	/* Hand over the packet to ICE session */
-	status = pj_ice_sess_on_rx_pkt(comp->ice_st->ice, comp->comp_id,
+	status = pj_ice_sess_on_rx_pkt(comp->ice_st->ice, comp->comp_id, 
 				       TP_STUN, pkt, pkt_len,
 				       src_addr, addr_len);
 
 	if (status != PJ_SUCCESS) {
-	    ice_st_perror(comp->ice_st, "Error processing packet",
+	    ice_st_perror(comp->ice_st, "Error processing packet", 
 			  status);
 	}
     }
 
-    return sess_dec_ref(ice_st);
+    return pj_grp_lock_dec_ref(ice_st->grp_lock) ? PJ_FALSE : PJ_TRUE;
 }
 
 /* Notifification when asynchronous send operation to the STUN socket
- * has completed.
+ * has completed. 
  */
 static pj_bool_t stun_on_data_sent(pj_stun_sock *stun_sock,
 				   pj_ioqueue_op_key_t *send_key,
@@ -1459,7 +1439,7 @@ static pj_bool_t stun_on_data_sent(pj_stun_sock *stun_sock,
 }
 
 /* Notification when the status of the STUN transport has changed. */
-static pj_bool_t stun_on_status(pj_stun_sock *stun_sock,
+static pj_bool_t stun_on_status(pj_stun_sock *stun_sock, 
 				pj_stun_sock_op op,
 				pj_status_t status)
 {
@@ -1473,10 +1453,10 @@ static pj_bool_t stun_on_status(pj_stun_sock *stun_sock,
     comp = (pj_ice_strans_comp*) pj_stun_sock_get_user_data(stun_sock);
     ice_st = comp->ice_st;
 
-    sess_add_ref(ice_st);
+    pj_grp_lock_add_ref(ice_st->grp_lock);
 
     /* Wait until initialization completes */
-    pj_lock_acquire(ice_st->init_lock);
+    pj_grp_lock_acquire(ice_st->grp_lock);
 
     /* Find the srflx cancidate */
     for (i=0; i<comp->cand_cnt; ++i) {
@@ -1486,14 +1466,14 @@ static pj_bool_t stun_on_status(pj_stun_sock *stun_sock,
 	}
     }
 
-    pj_lock_release(ice_st->init_lock);
+    pj_grp_lock_release(ice_st->grp_lock);
 
     /* It is possible that we don't have srflx candidate even though this
      * callback is called. This could happen when we cancel adding srflx
      * candidate due to initialization error.
      */
     if (cand == NULL) {
-	return sess_dec_ref(ice_st);
+	return pj_grp_lock_dec_ref(ice_st->grp_lock) ? PJ_FALSE : PJ_TRUE;
     }
 
     switch (op) {
@@ -1546,7 +1526,7 @@ static pj_bool_t stun_on_status(pj_stun_sock *stun_sock,
 		    if (comp->default_cand > idx) {
 			--comp->default_cand;
 		    } else if (comp->default_cand == idx) {
-			comp->default_cand = !idx;
+			comp->default_cand = 0;
 		    }
 
 		    /* Remove srflx candidate */
@@ -1559,11 +1539,11 @@ static pj_bool_t stun_on_status(pj_stun_sock *stun_sock,
 		    cand->status = PJ_SUCCESS;
 		}
 
-		PJ_LOG(4,(comp->ice_st->obj_name,
+		PJ_LOG(4,(comp->ice_st->obj_name, 
 			  "Comp %d: %s, "
 			  "srflx address is %s",
-			  comp->comp_id, op_name,
-			  pj_sockaddr_print(&info.mapped_addr, ipaddr,
+			  comp->comp_id, op_name, 
+			  pj_sockaddr_print(&info.mapped_addr, ipaddr, 
 					     sizeof(ipaddr), 3)));
 
 		sess_init_update(ice_st);
@@ -1574,7 +1554,7 @@ static pj_bool_t stun_on_status(pj_stun_sock *stun_sock,
 	    /* May not have cand, e.g. when error during init */
 	    if (cand)
 		cand->status = status;
-	    if (!ice_st->cfg.stun.ignore_stun_error) {
+	    if (!ice_st->cfg.stun.ignore_stun_error || comp->cand_cnt==1) {
 		sess_fail(ice_st, PJ_ICE_STRANS_OP_INIT,
 			  "STUN binding request failed", status);
 	    } else {
@@ -1609,7 +1589,7 @@ static pj_bool_t stun_on_status(pj_stun_sock *stun_sock,
 	break;
     }
 
-    return sess_dec_ref(ice_st);
+    return pj_grp_lock_dec_ref(ice_st->grp_lock)? PJ_FALSE : PJ_TRUE;
 }
 
 /* Callback when TURN socket has received a packet */
@@ -1628,7 +1608,7 @@ static void turn_on_rx_data(pj_turn_sock *turn_sock,
 	return;
     }
 
-    sess_add_ref(comp->ice_st);
+    pj_grp_lock_add_ref(comp->ice_st->grp_lock);
 
     if (comp->ice_st->ice == NULL) {
 	/* The ICE session is gone, but we're still receiving packets.
@@ -1649,13 +1629,13 @@ static void turn_on_rx_data(pj_turn_sock *turn_sock,
 				       peer_addr, addr_len);
 
 	if (status != PJ_SUCCESS) {
-	    ice_st_perror(comp->ice_st,
-			  "Error processing packet from TURN relay",
+	    ice_st_perror(comp->ice_st, 
+			  "Error processing packet from TURN relay", 
 			  status);
 	}
     }
 
-    sess_dec_ref(comp->ice_st);
+    pj_grp_lock_dec_ref(comp->ice_st->grp_lock);
 }
 
 
@@ -1677,7 +1657,7 @@ static void turn_on_state(pj_turn_sock *turn_sock, pj_turn_state_t old_state,
 	      pj_turn_state_name(old_state), pj_turn_state_name(new_state)));
     pj_log_push_indent();
 
-    sess_add_ref(comp->ice_st);
+    pj_grp_lock_add_ref(comp->ice_st->grp_lock);
 
     if (new_state == PJ_TURN_STATE_READY) {
 	pj_turn_session_info rel_info;
@@ -1691,7 +1671,7 @@ static void turn_on_state(pj_turn_sock *turn_sock, pj_turn_state_t old_state,
 	pj_turn_sock_get_info(turn_sock, &rel_info);
 
 	/* Wait until initialization completes */
-	pj_lock_acquire(comp->ice_st->init_lock);
+	pj_grp_lock_acquire(comp->ice_st->grp_lock);
 
 	/* Find relayed candidate in the component */
 	for (i=0; i<comp->cand_cnt; ++i) {
@@ -1702,24 +1682,24 @@ static void turn_on_state(pj_turn_sock *turn_sock, pj_turn_state_t old_state,
 	}
 	pj_assert(cand != NULL);
 
-	pj_lock_release(comp->ice_st->init_lock);
+	pj_grp_lock_release(comp->ice_st->grp_lock);
 
 	/* Update candidate */
 	pj_sockaddr_cp(&cand->addr, &rel_info.relay_addr);
 	pj_sockaddr_cp(&cand->base_addr, &rel_info.relay_addr);
 	pj_sockaddr_cp(&cand->rel_addr, &rel_info.mapped_addr);
-	pj_ice_calc_foundation(comp->ice_st->pool, &cand->foundation,
-			       PJ_ICE_CAND_TYPE_RELAYED,
+	pj_ice_calc_foundation(comp->ice_st->pool, &cand->foundation, 
+			       PJ_ICE_CAND_TYPE_RELAYED, 
 			       &rel_info.relay_addr);
 	cand->status = PJ_SUCCESS;
 
 	/* Set default candidate to relay */
 	comp->default_cand = cand - comp->cand_list;
 
-	PJ_LOG(4,(comp->ice_st->obj_name,
+	PJ_LOG(4,(comp->ice_st->obj_name, 
 		  "Comp %d: TURN allocation complete, relay address is %s",
-		  comp->comp_id,
-		  pj_sockaddr_print(&rel_info.relay_addr, ipaddr,
+		  comp->comp_id, 
+		  pj_sockaddr_print(&rel_info.relay_addr, ipaddr, 
 				     sizeof(ipaddr), 3)));
 
 	sess_init_update(comp->ice_st);
@@ -1735,22 +1715,28 @@ static void turn_on_state(pj_turn_sock *turn_sock, pj_turn_state_t old_state,
 	pj_turn_sock_set_user_data(turn_sock, NULL);
 	comp->turn_sock = NULL;
 
-	/* Set session to fail if we're still initializing */
-	if (comp->ice_st->state < PJ_ICE_STRANS_STATE_READY) {
-	    sess_fail(comp->ice_st, PJ_ICE_STRANS_OP_INIT,
-		      "TURN allocation failed", info.last_status);
-	} else if (comp->turn_err_cnt > 1) {
-	    sess_fail(comp->ice_st, PJ_ICE_STRANS_OP_KEEP_ALIVE,
-		      "TURN refresh failed", info.last_status);
-	} else {
-	    PJ_PERROR(4,(comp->ice_st->obj_name, info.last_status,
-		      "Comp %d: TURN allocation failed, retrying",
-		      comp->comp_id));
-	    add_update_turn(comp->ice_st, comp);
+	/* Set session to fail on error. last_status PJ_SUCCESS means normal
+	 * deallocation, which should not trigger sess_fail as it may have
+	 * been initiated by ICE destroy
+	 */
+	if (info.last_status != PJ_SUCCESS) {
+	    if (comp->ice_st->state < PJ_ICE_STRANS_STATE_READY) {
+		sess_fail(comp->ice_st, PJ_ICE_STRANS_OP_INIT,
+			  "TURN allocation failed", info.last_status);
+	    } else if (comp->turn_err_cnt > 1) {
+		sess_fail(comp->ice_st, PJ_ICE_STRANS_OP_KEEP_ALIVE,
+			  "TURN refresh failed", info.last_status);
+	    } else {
+		PJ_PERROR(4,(comp->ice_st->obj_name, info.last_status,
+			  "Comp %d: TURN allocation failed, retrying",
+			  comp->comp_id));
+		add_update_turn(comp->ice_st, comp);
+	    }
 	}
     }
 
-    sess_dec_ref(comp->ice_st);
+    pj_grp_lock_dec_ref(comp->ice_st->grp_lock);
 
     pj_log_pop_indent();
 }
+
