@@ -1,5 +1,5 @@
-/* $Id: turn_session.c 3844 2011-10-24 15:03:43Z bennylp $ */
-/*
+/* $Id: turn_session.c 4368 2013-02-21 21:53:28Z bennylp $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjnath/turn_session.h>
 #include <pjnath/errno.h>
@@ -36,7 +36,7 @@
 #define PJ_TURN_CHANNEL_HTABLE_SIZE 8
 #define PJ_TURN_PERM_HTABLE_SIZE    8
 
-static const char *state_names[] =
+static const char *state_names[] = 
 {
     "Null",
     "Resolving",
@@ -96,8 +96,8 @@ struct perm_t
     /* The permission expiration */
     pj_time_val	    expiry;
 
-    /* Arbitrary/random pointer value (token) to map this perm with the
-     * request to create it. It is used to invalidate this perm when the
+    /* Arbitrary/random pointer value (token) to map this perm with the 
+     * request to create it. It is used to invalidate this perm when the 
      * request fails.
      */
     void	   *req_token;
@@ -112,8 +112,9 @@ struct pj_turn_session
     pj_turn_session_cb	 cb;
     void		*user_data;
     pj_stun_config	 stun_cfg;
+    pj_bool_t		 is_destroying;
 
-    pj_lock_t		*lock;
+    pj_grp_lock_t	*grp_lock;
     int			 busy;
 
     pj_turn_state_t	 state;
@@ -161,6 +162,7 @@ struct pj_turn_session
  */
 static void sess_shutdown(pj_turn_session *sess,
 			  pj_status_t status);
+static void turn_sess_on_destroy(void *comp);
 static void do_destroy(pj_turn_session *sess);
 static void send_refresh(pj_turn_session *sess, int lifetime);
 static pj_status_t stun_on_send_msg(pj_stun_session *sess,
@@ -213,7 +215,7 @@ PJ_DEF(void) pj_turn_alloc_param_default(pj_turn_alloc_param *prm)
 /*
  * Duplicate pj_turn_alloc_param.
  */
-PJ_DEF(void) pj_turn_alloc_param_copy( pj_pool_t *pool,
+PJ_DEF(void) pj_turn_alloc_param_copy( pj_pool_t *pool, 
 				       pj_turn_alloc_param *dst,
 				       const pj_turn_alloc_param *src)
 {
@@ -236,6 +238,7 @@ PJ_DEF(pj_status_t) pj_turn_session_create( const pj_stun_config *cfg,
 					    const char *name,
 					    int af,
 					    pj_turn_tp_type conn_type,
+					    pj_grp_lock_t *grp_lock,
 					    const pj_turn_session_cb *cb,
 					    unsigned options,
 					    void *user_data,
@@ -244,7 +247,6 @@ PJ_DEF(pj_status_t) pj_turn_session_create( const pj_stun_config *cfg,
     pj_pool_t *pool;
     pj_turn_session *sess;
     pj_stun_session_cb stun_cb;
-    pj_lock_t *null_lock;
     pj_status_t status;
 
     PJ_ASSERT_RETURN(cfg && cfg->pf && cb && p_sess, PJ_EINVAL);
@@ -281,13 +283,20 @@ PJ_DEF(pj_status_t) pj_turn_session_create( const pj_stun_config *cfg,
     sess->perm_table = pj_hash_create(pool, PJ_TURN_PERM_HTABLE_SIZE);
 
     /* Session lock */
-    status = pj_lock_create_recursive_mutex(pool, sess->obj_name,
-					    &sess->lock);
-    if (status != PJ_SUCCESS) {
-	do_destroy(sess);
-	return status;
+    if (grp_lock) {
+	sess->grp_lock = grp_lock;
+    } else {
+	status = pj_grp_lock_create(pool, NULL, &sess->grp_lock);
+	if (status != PJ_SUCCESS) {
+	    pj_pool_release(pool);
+	    return status;
+	}
     }
 
+    pj_grp_lock_add_ref(sess->grp_lock);
+    pj_grp_lock_add_handler(sess->grp_lock, pool, sess,
+                            &turn_sess_on_destroy);
+
     /* Timer */
     pj_timer_entry_init(&sess->timer, TIMER_NONE, sess, &on_timer_event);
 
@@ -297,7 +306,7 @@ PJ_DEF(pj_status_t) pj_turn_session_create( const pj_stun_config *cfg,
     stun_cb.on_request_complete = &stun_on_request_complete;
     stun_cb.on_rx_indication = &stun_on_rx_indication;
     status = pj_stun_session_create(&sess->stun_cfg, sess->obj_name, &stun_cb,
-				    PJ_FALSE, &sess->stun);
+				    PJ_FALSE, sess->grp_lock, &sess->stun);
     if (status != PJ_SUCCESS) {
 	do_destroy(sess);
 	return status;
@@ -306,16 +315,6 @@ PJ_DEF(pj_status_t) pj_turn_session_create( const pj_stun_config *cfg,
     /* Attach ourself to STUN session */
     pj_stun_session_set_user_data(sess->stun, sess);
 
-    /* Replace mutex in STUN session with a NULL mutex, since access to
-     * STUN session is serialized.
-     */
-    status = pj_lock_create_null_mutex(pool, name, &null_lock);
-    if (status != PJ_SUCCESS) {
-	do_destroy(sess);
-	return status;
-    }
-    pj_stun_session_set_lock(sess->stun, null_lock, PJ_TRUE);
-
     /* Done */
 
     PJ_LOG(4,(sess->obj_name, "TURN client session created"));
@@ -325,32 +324,9 @@ PJ_DEF(pj_status_t) pj_turn_session_create( const pj_stun_config *cfg,
 }
 
 
-/* Destroy */
-static void do_destroy(pj_turn_session *sess)
+static void turn_sess_on_destroy(void *comp)
 {
-    /* Lock session */
-    if (sess->lock) {
-	pj_lock_acquire(sess->lock);
-    }
-
-    /* Cancel pending timer, if any */
-    if (sess->timer.id != TIMER_NONE) {
-	pj_timer_heap_cancel(sess->timer_heap, &sess->timer);
-	sess->timer.id = TIMER_NONE;
-    }
-
-    /* Destroy STUN session */
-    if (sess->stun) {
-	pj_stun_session_destroy(sess->stun);
-	sess->stun = NULL;
-    }
-
-    /* Destroy lock */
-    if (sess->lock) {
-	pj_lock_release(sess->lock);
-	pj_lock_destroy(sess->lock);
-	sess->lock = NULL;
-    }
+    pj_turn_session *sess = (pj_turn_session*) comp;
 
     /* Destroy pool */
     if (sess->pool) {
@@ -363,6 +339,26 @@ static void do_destroy(pj_turn_session *sess)
     }
 }
 
+/* Destroy */
+static void do_destroy(pj_turn_session *sess)
+{
+    PJ_LOG(4,(sess->obj_name, "TURN session destroy request, ref_cnt=%d",
+	      pj_grp_lock_get_ref(sess->grp_lock)));
+
+    pj_grp_lock_acquire(sess->grp_lock);
+    if (sess->is_destroying) {
+	pj_grp_lock_release(sess->grp_lock);
+	return;
+    }
+
+    sess->is_destroying = PJ_TRUE;
+    pj_timer_heap_cancel_if_active(sess->timer_heap, &sess->timer, TIMER_NONE);
+    pj_stun_session_destroy(sess->stun);
+
+    pj_grp_lock_dec_ref(sess->grp_lock);
+    pj_grp_lock_release(sess->grp_lock);
+}
+
 
 /* Set session state */
 static void set_state(pj_turn_session *sess, enum pj_turn_state_t state)
@@ -421,7 +417,10 @@ static void sess_shutdown(pj_turn_session *sess,
 	/* This may recursively call this function again with
 	 * state==PJ_TURN_STATE_DEALLOCATED.
 	 */
+	/* No need to deallocate as we're already deallocating!
+	 * See https://trac.pjsip.org/repos/ticket/1551
 	send_refresh(sess, 0);
+	*/
 	break;
     case PJ_TURN_STATE_DEALLOCATED:
     case PJ_TURN_STATE_DESTROYING:
@@ -434,13 +433,11 @@ static void sess_shutdown(pj_turn_session *sess,
 
 	set_state(sess, PJ_TURN_STATE_DESTROYING);
 
-	if (sess->timer.id != TIMER_NONE) {
-	    pj_timer_heap_cancel(sess->timer_heap, &sess->timer);
-	    sess->timer.id = TIMER_NONE;
-	}
-
-	sess->timer.id = TIMER_DESTROY;
-	pj_timer_heap_schedule(sess->timer_heap, &sess->timer, &delay);
+	pj_timer_heap_cancel_if_active(sess->timer_heap, &sess->timer,
+	                               TIMER_NONE);
+	pj_timer_heap_schedule_w_grp_lock(sess->timer_heap, &sess->timer,
+	                                  &delay, TIMER_DESTROY,
+	                                  sess->grp_lock);
     }
 }
 
@@ -452,11 +449,11 @@ PJ_DEF(pj_status_t) pj_turn_session_shutdown(pj_turn_session *sess)
 {
     PJ_ASSERT_RETURN(sess, PJ_EINVAL);
 
-    pj_lock_acquire(sess->lock);
+    pj_grp_lock_acquire(sess->grp_lock);
 
     sess_shutdown(sess, PJ_SUCCESS);
 
-    pj_lock_release(sess->lock);
+    pj_grp_lock_release(sess->grp_lock);
 
     return PJ_SUCCESS;
 }
@@ -500,9 +497,9 @@ PJ_DEF(pj_status_t) pj_turn_session_get_info( pj_turn_session *sess,
     else
 	pj_bzero(&info->server, sizeof(info->server));
 
-    pj_memcpy(&info->mapped_addr, &sess->mapped_addr,
+    pj_memcpy(&info->mapped_addr, &sess->mapped_addr, 
 	      sizeof(sess->mapped_addr));
-    pj_memcpy(&info->relay_addr, &sess->relay_addr,
+    pj_memcpy(&info->relay_addr, &sess->relay_addr, 
 	      sizeof(sess->relay_addr));
 
     return PJ_SUCCESS;
@@ -550,9 +547,9 @@ PJ_DEF(pj_status_t) pj_turn_session_set_software_name( pj_turn_session *sess,
 {
     pj_status_t status;
 
-    pj_lock_acquire(sess->lock);
+    pj_grp_lock_acquire(sess->grp_lock);
     status = pj_stun_session_set_software_name(sess->stun, sw);
-    pj_lock_release(sess->lock);
+    pj_grp_lock_release(sess->grp_lock);
 
     return status;
 }
@@ -573,11 +570,11 @@ PJ_DEF(pj_status_t) pj_turn_session_set_server( pj_turn_session *sess,
     PJ_ASSERT_RETURN(sess && domain, PJ_EINVAL);
     PJ_ASSERT_RETURN(sess->state == PJ_TURN_STATE_NULL, PJ_EINVALIDOP);
 
-    pj_lock_acquire(sess->lock);
+    pj_grp_lock_acquire(sess->grp_lock);
 
     /* See if "domain" contains just IP address */
     tmp_addr.addr.sa_family = sess->af;
-    status = pj_inet_pton(sess->af, domain,
+    status = pj_inet_pton(sess->af, domain, 
 			  pj_sockaddr_get_addr(&tmp_addr));
     is_ip_addr = (status == PJ_SUCCESS);
 
@@ -620,8 +617,8 @@ PJ_DEF(pj_status_t) pj_turn_session_set_server( pj_turn_session *sess,
 	    goto on_return;
 	}
 
-	status = pj_dns_srv_resolve(domain, &res_name, default_port,
-				    sess->pool, resolver, opt, sess,
+	status = pj_dns_srv_resolve(domain, &res_name, default_port, 
+				    sess->pool, resolver, opt, sess, 
 				    &dns_srv_resolver_cb, &sess->dns_async);
 	if (status != PJ_SUCCESS) {
 	    set_state(sess, PJ_TURN_STATE_NULL);
@@ -659,7 +656,7 @@ PJ_DEF(pj_status_t) pj_turn_session_set_server( pj_turn_session *sess,
 
 	sess->srv_addr_cnt = (pj_uint16_t)cnt;
 	sess->srv_addr_list = (pj_sockaddr*)
-		              pj_pool_calloc(sess->pool, cnt,
+		              pj_pool_calloc(sess->pool, cnt, 
 					     sizeof(pj_sockaddr));
 	for (i=0; i<cnt; ++i) {
 	    pj_sockaddr *addr = &sess->srv_addr_list[i];
@@ -673,7 +670,7 @@ PJ_DEF(pj_status_t) pj_turn_session_set_server( pj_turn_session *sess,
     }
 
 on_return:
-    pj_lock_release(sess->lock);
+    pj_grp_lock_release(sess->grp_lock);
     return status;
 }
 
@@ -687,11 +684,11 @@ PJ_DEF(pj_status_t) pj_turn_session_set_credential(pj_turn_session *sess,
     PJ_ASSERT_RETURN(sess && cred, PJ_EINVAL);
     PJ_ASSERT_RETURN(sess->stun, PJ_EINVALIDOP);
 
-    pj_lock_acquire(sess->lock);
+    pj_grp_lock_acquire(sess->grp_lock);
 
     pj_stun_session_set_credential(sess->stun, PJ_STUN_AUTH_LONG_TERM, cred);
 
-    pj_lock_release(sess->lock);
+    pj_grp_lock_release(sess->grp_lock);
 
     return PJ_SUCCESS;
 }
@@ -708,13 +705,13 @@ PJ_DEF(pj_status_t) pj_turn_session_alloc(pj_turn_session *sess,
     pj_status_t status;
 
     PJ_ASSERT_RETURN(sess, PJ_EINVAL);
-    PJ_ASSERT_RETURN(sess->state>PJ_TURN_STATE_NULL &&
-		     sess->state<=PJ_TURN_STATE_RESOLVED,
+    PJ_ASSERT_RETURN(sess->state>PJ_TURN_STATE_NULL && 
+		     sess->state<=PJ_TURN_STATE_RESOLVED, 
 		     PJ_EINVALIDOP);
 
-    pj_lock_acquire(sess->lock);
+    pj_grp_lock_acquire(sess->grp_lock);
 
-    if (param && param != &sess->alloc_param)
+    if (param && param != &sess->alloc_param) 
 	pj_turn_alloc_param_copy(sess->pool, &sess->alloc_param, param);
 
     if (sess->state < PJ_TURN_STATE_RESOLVED) {
@@ -723,25 +720,25 @@ PJ_DEF(pj_status_t) pj_turn_session_alloc(pj_turn_session *sess,
 	PJ_LOG(4,(sess->obj_name, "Pending ALLOCATE in state %s",
 		  state_names[sess->state]));
 
-	pj_lock_release(sess->lock);
+	pj_grp_lock_release(sess->grp_lock);
 	return PJ_SUCCESS;
 
     }
 
     /* Ready to allocate */
     pj_assert(sess->state == PJ_TURN_STATE_RESOLVED);
-
+    
     /* Create a bare request */
     status = pj_stun_session_create_req(sess->stun, PJ_STUN_ALLOCATE_REQUEST,
 					PJ_STUN_MAGIC, NULL, &tdata);
     if (status != PJ_SUCCESS) {
-	pj_lock_release(sess->lock);
+	pj_grp_lock_release(sess->grp_lock);
 	return status;
     }
 
     /* MUST include REQUESTED-TRANSPORT attribute */
     pj_stun_msg_add_uint_attr(tdata->pool, tdata->msg,
-			      PJ_STUN_ATTR_REQ_TRANSPORT,
+			      PJ_STUN_ATTR_REQ_TRANSPORT, 
 			      PJ_STUN_SET_RT_PROTO(PJ_TURN_TP_UDP));
 
     /* Include BANDWIDTH if requested */
@@ -764,9 +761,9 @@ PJ_DEF(pj_status_t) pj_turn_session_alloc(pj_turn_session *sess,
     /* Send request */
     set_state(sess, PJ_TURN_STATE_ALLOCATING);
     retransmit = (sess->conn_type == PJ_TURN_TP_UDP);
-    status = pj_stun_session_send_msg(sess->stun, NULL, PJ_FALSE,
+    status = pj_stun_session_send_msg(sess->stun, NULL, PJ_FALSE, 
 				      retransmit, sess->srv_addr,
-				      pj_sockaddr_get_len(sess->srv_addr),
+				      pj_sockaddr_get_len(sess->srv_addr), 
 				      tdata);
     if (status != PJ_SUCCESS) {
 	/* Set state back to RESOLVED. We don't want to destroy session now,
@@ -775,7 +772,7 @@ PJ_DEF(pj_status_t) pj_turn_session_alloc(pj_turn_session *sess,
 	set_state(sess, PJ_TURN_STATE_RESOLVED);
     }
 
-    pj_lock_release(sess->lock);
+    pj_grp_lock_release(sess->grp_lock);
     return status;
 }
 
@@ -796,14 +793,14 @@ PJ_DEF(pj_status_t) pj_turn_session_set_perm( pj_turn_session *sess,
 
     PJ_ASSERT_RETURN(sess && addr_cnt && addr, PJ_EINVAL);
 
-    pj_lock_acquire(sess->lock);
+    pj_grp_lock_acquire(sess->grp_lock);
 
     /* Create a bare CreatePermission request */
-    status = pj_stun_session_create_req(sess->stun,
+    status = pj_stun_session_create_req(sess->stun, 
 					PJ_STUN_CREATE_PERM_REQUEST,
 					PJ_STUN_MAGIC, NULL, &tdata);
     if (status != PJ_SUCCESS) {
-	pj_lock_release(sess->lock);
+	pj_grp_lock_release(sess->grp_lock);
 	return status;
     }
 
@@ -843,10 +840,10 @@ PJ_DEF(pj_status_t) pj_turn_session_set_perm( pj_turn_session *sess,
     pj_assert(attr_added != 0);
 
     /* Send the request */
-    status = pj_stun_session_send_msg(sess->stun, req_token, PJ_FALSE,
+    status = pj_stun_session_send_msg(sess->stun, req_token, PJ_FALSE, 
 				      (sess->conn_type==PJ_TURN_TP_UDP),
 				      sess->srv_addr,
-				      pj_sockaddr_get_len(sess->srv_addr),
+				      pj_sockaddr_get_len(sess->srv_addr), 
 				      tdata);
     if (status != PJ_SUCCESS) {
 	/* tdata is already destroyed */
@@ -854,7 +851,7 @@ PJ_DEF(pj_status_t) pj_turn_session_set_perm( pj_turn_session *sess,
 	goto on_error;
     }
 
-    pj_lock_release(sess->lock);
+    pj_grp_lock_release(sess->grp_lock);
     return PJ_SUCCESS;
 
 on_error:
@@ -871,7 +868,7 @@ on_error:
 	if (perm->req_token == req_token)
 	    invalidate_perm(sess, perm);
     }
-    pj_lock_release(sess->lock);
+    pj_grp_lock_release(sess->grp_lock);
     return status;
 }
 
@@ -902,10 +899,10 @@ static void send_refresh(pj_turn_session *sess, int lifetime)
 	set_state(sess, PJ_TURN_STATE_DEALLOCATING);
     }
 
-    status = pj_stun_session_send_msg(sess->stun, NULL, PJ_FALSE,
+    status = pj_stun_session_send_msg(sess->stun, NULL, PJ_FALSE, 
 				      (sess->conn_type==PJ_TURN_TP_UDP),
 				      sess->srv_addr,
-				      pj_sockaddr_get_len(sess->srv_addr),
+				      pj_sockaddr_get_len(sess->srv_addr), 
 				      tdata);
     if (status != PJ_SUCCESS)
 	goto on_error;
@@ -933,7 +930,7 @@ PJ_DEF(pj_status_t) pj_turn_session_sendto( pj_turn_session *sess,
     struct perm_t *perm;
     pj_status_t status;
 
-    PJ_ASSERT_RETURN(sess && pkt && pkt_len && addr && addr_len,
+    PJ_ASSERT_RETURN(sess && pkt && pkt_len && addr && addr_len, 
 		     PJ_EINVAL);
 
     /* Return error if we're not ready */
@@ -942,7 +939,7 @@ PJ_DEF(pj_status_t) pj_turn_session_sendto( pj_turn_session *sess,
     }
 
     /* Lock session now */
-    pj_lock_acquire(sess->lock);
+    pj_grp_lock_acquire(sess->grp_lock);
 
     /* Lookup permission first */
     perm = lookup_perm(sess, addr, pj_sockaddr_get_len(addr), PJ_FALSE);
@@ -950,27 +947,27 @@ PJ_DEF(pj_status_t) pj_turn_session_sendto( pj_turn_session *sess,
 	/* Permission doesn't exist, install it first */
 	char ipstr[PJ_INET6_ADDRSTRLEN+2];
 
-	PJ_LOG(4,(sess->obj_name,
+	PJ_LOG(4,(sess->obj_name, 
 		  "sendto(): IP %s has no permission, requesting it first..",
 		  pj_sockaddr_print(addr, ipstr, sizeof(ipstr), 2)));
 
-	status = pj_turn_session_set_perm(sess, 1, (const pj_sockaddr*)addr,
+	status = pj_turn_session_set_perm(sess, 1, (const pj_sockaddr*)addr, 
 					  0);
 	if (status != PJ_SUCCESS) {
-	    pj_lock_release(sess->lock);
+	    pj_grp_lock_release(sess->grp_lock);
 	    return status;
 	}
     }
 
     /* See if the peer is bound to a channel number */
-    ch = lookup_ch_by_addr(sess, addr, pj_sockaddr_get_len(addr),
+    ch = lookup_ch_by_addr(sess, addr, pj_sockaddr_get_len(addr), 
 			   PJ_FALSE, PJ_FALSE);
     if (ch && ch->num != PJ_TURN_INVALID_CHANNEL && ch->bound) {
 	unsigned total_len;
 
 	/* Peer is assigned a channel number, we can use ChannelData */
 	pj_turn_channel_data *cd = (pj_turn_channel_data*)sess->tx_pkt;
-
+	
 	pj_assert(sizeof(*cd)==4);
 
 	/* Calculate total length, including paddings */
@@ -1002,7 +999,7 @@ PJ_DEF(pj_status_t) pj_turn_session_sendto( pj_turn_session *sess,
 
 	/* Create blank SEND-INDICATION */
 	status = pj_stun_msg_init(&send_ind, PJ_STUN_SEND_INDICATION,
-				  PJ_STUN_MAGIC,
+				  PJ_STUN_MAGIC, 
 				  (const pj_uint8_t*)sess->send_ind_tsx_id);
 	if (status != PJ_SUCCESS)
 	    goto on_return;
@@ -1019,7 +1016,7 @@ PJ_DEF(pj_status_t) pj_turn_session_sendto( pj_turn_session *sess,
 	pj_stun_msg_add_attr(&send_ind, (pj_stun_attr_hdr*)&data_attr);
 
 	/* Encode the message */
-	status = pj_stun_msg_encode(&send_ind, sess->tx_pkt,
+	status = pj_stun_msg_encode(&send_ind, sess->tx_pkt, 
 				    sizeof(sess->tx_pkt), 0,
 				    NULL, &send_ind_len);
 	if (status != PJ_SUCCESS)
@@ -1032,7 +1029,7 @@ PJ_DEF(pj_status_t) pj_turn_session_sendto( pj_turn_session *sess,
     }
 
 on_return:
-    pj_lock_release(sess->lock);
+    pj_grp_lock_release(sess->grp_lock);
     return status;
 }
 
@@ -1052,10 +1049,10 @@ PJ_DEF(pj_status_t) pj_turn_session_bind_channel(pj_turn_session *sess,
     PJ_ASSERT_RETURN(sess && peer_adr && addr_len, PJ_EINVAL);
     PJ_ASSERT_RETURN(sess->state == PJ_TURN_STATE_READY, PJ_EINVALIDOP);
 
-    pj_lock_acquire(sess->lock);
+    pj_grp_lock_acquire(sess->grp_lock);
 
     /* Create blank ChannelBind request */
-    status = pj_stun_session_create_req(sess->stun,
+    status = pj_stun_session_create_req(sess->stun, 
 					PJ_STUN_CHANNEL_BIND_REQUEST,
 					PJ_STUN_MAGIC, NULL, &tdata);
     if (status != PJ_SUCCESS)
@@ -1070,7 +1067,7 @@ PJ_DEF(pj_status_t) pj_turn_session_bind_channel(pj_turn_session *sess,
 	/* Channel is already bound. This is a refresh request. */
 	ch_num = ch->num;
     } else {
-	PJ_ASSERT_ON_FAIL(sess->next_ch <= PJ_TURN_CHANNEL_MAX,
+	PJ_ASSERT_ON_FAIL(sess->next_ch <= PJ_TURN_CHANNEL_MAX, 
 			    {status=PJ_ETOOMANY; goto on_return;});
 	ch->num = ch_num = sess->next_ch++;
     }
@@ -1085,17 +1082,17 @@ PJ_DEF(pj_status_t) pj_turn_session_bind_channel(pj_turn_session *sess,
 				  PJ_STUN_ATTR_XOR_PEER_ADDR, PJ_TRUE,
 				  peer_adr, addr_len);
 
-    /* Send the request, associate peer data structure with tdata
+    /* Send the request, associate peer data structure with tdata 
      * for future reference when we receive the ChannelBind response.
      */
-    status = pj_stun_session_send_msg(sess->stun, ch, PJ_FALSE,
+    status = pj_stun_session_send_msg(sess->stun, ch, PJ_FALSE, 
 				      (sess->conn_type==PJ_TURN_TP_UDP),
 				      sess->srv_addr,
 				      pj_sockaddr_get_len(sess->srv_addr),
 				      tdata);
 
 on_return:
-    pj_lock_release(sess->lock);
+    pj_grp_lock_release(sess->grp_lock);
     return status;
 }
 
@@ -1118,7 +1115,7 @@ PJ_DEF(pj_status_t) pj_turn_session_on_rx_pkt(pj_turn_session *sess,
      */
 
     /* Start locking the session */
-    pj_lock_acquire(sess->lock);
+    pj_grp_lock_acquire(sess->grp_lock);
 
     is_datagram = (sess->conn_type==PJ_TURN_TP_UDP);
 
@@ -1181,7 +1178,7 @@ PJ_DEF(pj_status_t) pj_turn_session_on_rx_pkt(pj_turn_session *sess,
 
 	/* Notify application */
 	if (sess->cb.on_rx_data) {
-	    (*sess->cb.on_rx_data)(sess, ((pj_uint8_t*)pkt)+sizeof(cd),
+	    (*sess->cb.on_rx_data)(sess, ((pj_uint8_t*)pkt)+sizeof(cd), 
 				   cd.length, &ch->addr,
 				   pj_sockaddr_get_len(&ch->addr));
 	}
@@ -1190,7 +1187,7 @@ PJ_DEF(pj_status_t) pj_turn_session_on_rx_pkt(pj_turn_session *sess,
     }
 
 on_return:
-    pj_lock_release(sess->lock);
+    pj_grp_lock_release(sess->grp_lock);
     return status;
 }
 
@@ -1210,7 +1207,7 @@ static pj_status_t stun_on_send_msg(pj_stun_session *stun,
     PJ_UNUSED_ARG(token);
 
     sess = (pj_turn_session*) pj_stun_session_get_user_data(stun);
-    return (*sess->cb.on_send_pkt)(sess, (const pj_uint8_t*)pkt, pkt_size,
+    return (*sess->cb.on_send_pkt)(sess, (const pj_uint8_t*)pkt, pkt_size, 
 				   dst_addr, addr_len);
 }
 
@@ -1219,7 +1216,7 @@ static pj_status_t stun_on_send_msg(pj_stun_session *stun,
  * Handle failed ALLOCATE or REFRESH request. This may switch to alternate
  * server if we have one.
  */
-static void on_session_fail( pj_turn_session *sess,
+static void on_session_fail( pj_turn_session *sess, 
 			     enum pj_stun_method_e method,
 			     pj_status_t status,
 			     const pj_str_t *reason)
@@ -1240,12 +1237,12 @@ static void on_session_fail( pj_turn_session *sess,
 		  pj_stun_get_method_name(method),
 		  (int)reason->slen, reason->ptr));
 
-	/* If this is ALLOCATE response and we don't have more server
+	/* If this is ALLOCATE response and we don't have more server 
 	 * addresses to try, notify application and destroy the TURN
 	 * session.
 	 */
 	if (method==PJ_STUN_ALLOCATE_METHOD &&
-	    sess->srv_addr == &sess->srv_addr_list[sess->srv_addr_cnt-1])
+	    sess->srv_addr == &sess->srv_addr_list[sess->srv_addr_cnt-1]) 
 	{
 
 	    set_state(sess, PJ_TURN_STATE_DEALLOCATED);
@@ -1276,7 +1273,7 @@ static void on_session_fail( pj_turn_session *sess,
 /*
  * Handle successful response to ALLOCATE or REFRESH request.
  */
-static void on_allocate_success(pj_turn_session *sess,
+static void on_allocate_success(pj_turn_session *sess, 
 				enum pj_stun_method_e method,
 				const pj_stun_msg *msg)
 {
@@ -1350,10 +1347,10 @@ static void on_allocate_success(pj_turn_session *sess,
 				    "RELAY-ADDRESS attribute"));
 	return;
     }
-
+    
     /* Save relayed address */
     if (raddr_attr) {
-	/* If we already have relay address, check if the relay address
+	/* If we already have relay address, check if the relay address 
 	 * in the response matches our relay address.
 	 */
 	if (pj_sockaddr_has_addr(&sess->relay_addr)) {
@@ -1365,7 +1362,7 @@ static void on_allocate_success(pj_turn_session *sess,
 	    }
 	} else {
 	    /* Otherwise save the relayed address */
-	    pj_memcpy(&sess->relay_addr, &raddr_attr->sockaddr,
+	    pj_memcpy(&sess->relay_addr, &raddr_attr->sockaddr, 
 		      sizeof(pj_sockaddr));
 	}
     }
@@ -1382,20 +1379,22 @@ static void on_allocate_success(pj_turn_session *sess,
 
     /* Cancel existing keep-alive timer, if any */
     pj_assert(sess->timer.id != TIMER_DESTROY);
-
-    if (sess->timer.id != TIMER_NONE) {
-	pj_timer_heap_cancel(sess->timer_heap, &sess->timer);
-	sess->timer.id = TIMER_NONE;
+    if (sess->timer.id == TIMER_KEEP_ALIVE) {
+	pj_timer_heap_cancel_if_active(sess->timer_heap, &sess->timer,
+				       TIMER_NONE);
     }
 
     /* Start keep-alive timer once allocation succeeds */
-    timeout.sec = sess->ka_interval;
-    timeout.msec = 0;
+    if (sess->state < PJ_TURN_STATE_DEALLOCATING) {
+	timeout.sec = sess->ka_interval;
+	timeout.msec = 0;
 
-    sess->timer.id = TIMER_KEEP_ALIVE;
-    pj_timer_heap_schedule(sess->timer_heap, &sess->timer, &timeout);
+	pj_timer_heap_schedule_w_grp_lock(sess->timer_heap, &sess->timer,
+					  &timeout, TIMER_KEEP_ALIVE,
+					  sess->grp_lock);
 
-    set_state(sess, PJ_TURN_STATE_READY);
+	set_state(sess, PJ_TURN_STATE_READY);
+    }
 }
 
 /*
@@ -1410,8 +1409,8 @@ static void stun_on_request_complete(pj_stun_session *stun,
 				     unsigned src_addr_len)
 {
     pj_turn_session *sess;
-    enum pj_stun_method_e method = (enum pj_stun_method_e)
-				   PJ_STUN_GET_METHOD(tdata->msg->hdr.type);
+    enum pj_stun_method_e method = (enum pj_stun_method_e) 
+			      	   PJ_STUN_GET_METHOD(tdata->msg->hdr.type);
 
     PJ_UNUSED_ARG(src_addr);
     PJ_UNUSED_ARG(src_addr_len);
@@ -1431,8 +1430,8 @@ static void stun_on_request_complete(pj_stun_session *stun,
 	}
 
 	/* Handle ALLOCATE response */
-	if (status==PJ_SUCCESS &&
-	    PJ_STUN_IS_SUCCESS_RESPONSE(response->hdr.type))
+	if (status==PJ_SUCCESS && 
+	    PJ_STUN_IS_SUCCESS_RESPONSE(response->hdr.type)) 
 	{
 
 	    /* Successful Allocate response */
@@ -1460,8 +1459,8 @@ static void stun_on_request_complete(pj_stun_session *stun,
 
     } else if (method == PJ_STUN_REFRESH_METHOD) {
 	/* Handle Refresh response */
-	if (status==PJ_SUCCESS &&
-	    PJ_STUN_IS_SUCCESS_RESPONSE(response->hdr.type))
+	if (status==PJ_SUCCESS && 
+	    PJ_STUN_IS_SUCCESS_RESPONSE(response->hdr.type)) 
 	{
 	    /* Success, schedule next refresh. */
 	    on_allocate_success(sess, method, response);
@@ -1489,8 +1488,8 @@ static void stun_on_request_complete(pj_stun_session *stun,
 
     } else if (method == PJ_STUN_CHANNEL_BIND_METHOD) {
 	/* Handle ChannelBind response */
-	if (status==PJ_SUCCESS &&
-	    PJ_STUN_IS_SUCCESS_RESPONSE(response->hdr.type))
+	if (status==PJ_SUCCESS && 
+	    PJ_STUN_IS_SUCCESS_RESPONSE(response->hdr.type)) 
 	{
 	    /* Successful ChannelBind response */
 	    struct ch_t *ch = (struct ch_t*)token;
@@ -1539,8 +1538,8 @@ static void stun_on_request_complete(pj_stun_session *stun,
 
     } else if (method == PJ_STUN_CREATE_PERM_METHOD) {
 	/* Handle CreatePermission response */
-	if (status==PJ_SUCCESS &&
-	    PJ_STUN_IS_SUCCESS_RESPONSE(response->hdr.type))
+	if (status==PJ_SUCCESS && 
+	    PJ_STUN_IS_SUCCESS_RESPONSE(response->hdr.type)) 
 	{
 	    /* No special handling when the request is successful. */
 	} else {
@@ -1559,7 +1558,7 @@ static void stun_on_request_complete(pj_stun_session *stun,
 		const pj_stun_errcode_attr *eattr;
 
 		eattr = (const pj_stun_errcode_attr*)
-			pj_stun_msg_find_attr(response,
+			pj_stun_msg_find_attr(response, 
 					      PJ_STUN_ATTR_ERROR_CODE, 0);
 		if (eattr) {
 		    err_code = eattr->err_code;
@@ -1580,9 +1579,9 @@ static void stun_on_request_complete(pj_stun_session *stun,
 		it = pj_hash_next(sess->perm_table, it);
 
 		if (perm->req_token == token) {
-		    PJ_LOG(1,(sess->obj_name,
+		    PJ_LOG(1,(sess->obj_name, 
 			      "CreatePermission failed for IP %s: %d/%.*s",
-			      pj_sockaddr_print(&perm->addr, ipstr,
+			      pj_sockaddr_print(&perm->addr, ipstr, 
 						sizeof(ipstr), 2),
 			      err_code, (int)reason.slen, reason.ptr));
 
@@ -1655,14 +1654,14 @@ static pj_status_t stun_on_rx_indication(pj_stun_session *stun,
 
     /* Must have both XOR-PEER-ADDRESS and DATA attributes */
     if (!peer_attr || !data_attr) {
-	PJ_LOG(4,(sess->obj_name,
+	PJ_LOG(4,(sess->obj_name, 
 		  "Received Data indication with missing attributes"));
 	return PJ_EINVALIDOP;
     }
 
     /* Notify application */
     if (sess->cb.on_rx_data) {
-	(*sess->cb.on_rx_data)(sess, data_attr->data, data_attr->length,
+	(*sess->cb.on_rx_data)(sess, data_attr->data, data_attr->length, 
 			       &peer_attr->sockaddr,
 			       pj_sockaddr_get_len(&peer_attr->sockaddr));
     }
@@ -1701,15 +1700,15 @@ static void dns_srv_resolver_cb(void *user_data,
 
     /* Allocate server entries */
     sess->srv_addr_list = (pj_sockaddr*)
-		           pj_pool_calloc(sess->pool, tot_cnt,
+		           pj_pool_calloc(sess->pool, tot_cnt, 
 					  sizeof(pj_sockaddr));
 
     /* Copy results to server entries */
     for (i=0, cnt=0; i<rec->count && cnt<PJ_TURN_MAX_DNS_SRV_CNT; ++i) {
 	unsigned j;
 
-	for (j=0; j<rec->entry[i].server.addr_count &&
-		  cnt<PJ_TURN_MAX_DNS_SRV_CNT; ++j)
+	for (j=0; j<rec->entry[i].server.addr_count && 
+		  cnt<PJ_TURN_MAX_DNS_SRV_CNT; ++j) 
 	{
 	    pj_sockaddr_in *addr = &sess->srv_addr_list[cnt].ipv4;
 
@@ -1747,7 +1746,7 @@ static struct ch_t *lookup_ch_by_addr(pj_turn_session *sess,
     pj_uint32_t hval = 0;
     struct ch_t *ch;
 
-    ch = (struct ch_t*)
+    ch = (struct ch_t*) 
 	 pj_hash_get(sess->ch_table, addr, addr_len, &hval);
     if (ch == NULL && update) {
 	ch = PJ_POOL_ZALLOC_T(sess->pool, struct ch_t);
@@ -1768,7 +1767,7 @@ static struct ch_t *lookup_ch_by_addr(pj_turn_session *sess,
 	    /* Register by channel number */
 	    pj_assert(ch->num != PJ_TURN_INVALID_CHANNEL && ch->bound);
 
-	    if (pj_hash_get(sess->ch_table, &ch->num,
+	    if (pj_hash_get(sess->ch_table, &ch->num, 
 			    sizeof(ch->num), &hval)==0) {
 		pj_hash_set(sess->pool, sess->ch_table, &ch->num,
 			    sizeof(ch->num), hval, ch);
@@ -1795,7 +1794,7 @@ static struct ch_t *lookup_ch_by_addr(pj_turn_session *sess,
 static struct ch_t *lookup_ch_by_chnum(pj_turn_session *sess,
 					 pj_uint16_t chnum)
 {
-    return (struct ch_t*) pj_hash_get(sess->ch_table, &chnum,
+    return (struct ch_t*) pj_hash_get(sess->ch_table, &chnum, 
 				      sizeof(chnum), NULL);
 }
 
@@ -1820,7 +1819,7 @@ static struct perm_t *lookup_perm(pj_turn_session *sess,
     }
 
     /* lookup and create if it doesn't exist and wanted */
-    perm = (struct perm_t*)
+    perm = (struct perm_t*) 
 	   pj_hash_get(sess->perm_table, addr, addr_len, &hval);
     if (perm == NULL && update) {
 	perm = PJ_POOL_ZALLOC_T(sess->pool, struct perm_t);
@@ -1853,7 +1852,7 @@ static void invalidate_perm(pj_turn_session *sess,
 /*
  * Scan permission's hash table to refresh the permission.
  */
-static unsigned refresh_permissions(pj_turn_session *sess,
+static unsigned refresh_permissions(pj_turn_session *sess, 
 				    const pj_time_val *now)
 {
     pj_stun_tx_data *tdata = NULL;
@@ -1875,11 +1874,11 @@ static unsigned refresh_permissions(pj_turn_session *sess,
 		if (tdata == NULL) {
 		    /* Create a bare CreatePermission request */
 		    status = pj_stun_session_create_req(
-					sess->stun,
+					sess->stun, 
 					PJ_STUN_CREATE_PERM_REQUEST,
 					PJ_STUN_MAGIC, NULL, &tdata);
 		    if (status != PJ_SUCCESS) {
-			PJ_LOG(1,(sess->obj_name,
+			PJ_LOG(1,(sess->obj_name, 
 				 "Error creating CreatePermission request: %d",
 				 status));
 			return 0;
@@ -1892,7 +1891,7 @@ static unsigned refresh_permissions(pj_turn_session *sess,
 		}
 
 		status = pj_stun_msg_add_sockaddr_attr(
-					tdata->pool,
+					tdata->pool, 
 					tdata->msg,
 					PJ_STUN_ATTR_XOR_PEER_ADDR,
 					PJ_TRUE,
@@ -1918,13 +1917,13 @@ static unsigned refresh_permissions(pj_turn_session *sess,
     }
 
     if (tdata) {
-	status = pj_stun_session_send_msg(sess->stun, req_token, PJ_FALSE,
+	status = pj_stun_session_send_msg(sess->stun, req_token, PJ_FALSE, 
 					  (sess->conn_type==PJ_TURN_TP_UDP),
 					  sess->srv_addr,
-					  pj_sockaddr_get_len(sess->srv_addr),
+					  pj_sockaddr_get_len(sess->srv_addr), 
 					  tdata);
 	if (status != PJ_SUCCESS) {
-	    PJ_LOG(1,(sess->obj_name,
+	    PJ_LOG(1,(sess->obj_name, 
 		      "Error sending CreatePermission request: %d",
 		      status));
 	    count = 0;
@@ -1945,17 +1944,22 @@ static void on_timer_event(pj_timer_heap_t *th, pj_timer_entry *e)
 
     PJ_UNUSED_ARG(th);
 
-    pj_lock_acquire(sess->lock);
+    pj_grp_lock_acquire(sess->grp_lock);
 
     eid = (enum timer_id_t) e->id;
     e->id = TIMER_NONE;
-
+    
     if (eid == TIMER_KEEP_ALIVE) {
 	pj_time_val now;
 	pj_hash_iterator_t itbuf, *it;
 	pj_bool_t resched = PJ_TRUE;
 	pj_bool_t pkt_sent = PJ_FALSE;
 
+	if (sess->state >= PJ_TURN_STATE_DEALLOCATING) {
+	    /* Ignore if we're deallocating */
+	    goto on_return;
+	}
+
 	pj_gettimeofday(&now);
 
 	/* Refresh allocation if it's time to do so */
@@ -1973,11 +1977,11 @@ static void on_timer_event(pj_timer_heap_t *th, pj_timer_entry *e)
 	/* Scan hash table to refresh bound channels */
 	it = pj_hash_first(sess->ch_table, &itbuf);
 	while (it) {
-	    struct ch_t *ch = (struct ch_t*)
+	    struct ch_t *ch = (struct ch_t*) 
 			      pj_hash_this(sess->ch_table, it);
 	    if (ch->bound && PJ_TIME_VAL_LTE(ch->expiry, now)) {
 
-		/* Send ChannelBind to refresh channel binding and
+		/* Send ChannelBind to refresh channel binding and 
 		 * permission.
 		 */
 		pj_turn_session_bind_channel(sess, &ch->addr,
@@ -2000,7 +2004,7 @@ static void on_timer_event(pj_timer_heap_t *th, pj_timer_entry *e)
 	    pj_status_t rc;
 
 	    /* Create blank SEND-INDICATION */
-	    rc = pj_stun_session_create_ind(sess->stun,
+	    rc = pj_stun_session_create_ind(sess->stun, 
 					    PJ_STUN_SEND_INDICATION, &tdata);
 	    if (rc == PJ_SUCCESS) {
 		/* Add DATA attribute with zero length */
@@ -2008,7 +2012,7 @@ static void on_timer_event(pj_timer_heap_t *th, pj_timer_entry *e)
 					    PJ_STUN_ATTR_DATA, NULL, 0);
 
 		/* Send the indication */
-		pj_stun_session_send_msg(sess->stun, NULL, PJ_FALSE,
+		pj_stun_session_send_msg(sess->stun, NULL, PJ_FALSE, 
 					 PJ_FALSE, sess->srv_addr,
 					 pj_sockaddr_get_len(sess->srv_addr),
 					 tdata);
@@ -2022,18 +2026,19 @@ static void on_timer_event(pj_timer_heap_t *th, pj_timer_entry *e)
 	    delay.sec = sess->ka_interval;
 	    delay.msec = 0;
 
-	    sess->timer.id = TIMER_KEEP_ALIVE;
-	    pj_timer_heap_schedule(sess->timer_heap, &sess->timer, &delay);
+	    pj_timer_heap_schedule_w_grp_lock(sess->timer_heap, &sess->timer,
+	                                      &delay, TIMER_KEEP_ALIVE,
+	                                      sess->grp_lock);
 	}
 
-	pj_lock_release(sess->lock);
-
     } else if (eid == TIMER_DESTROY) {
 	/* Time to destroy */
-	pj_lock_release(sess->lock);
 	do_destroy(sess);
     } else {
 	pj_assert(!"Unknown timer event");
-	pj_lock_release(sess->lock);
     }
+
+on_return:
+    pj_grp_lock_release(sess->grp_lock);
 }
+
