@@ -1,5 +1,5 @@
-/* $Id: icedemo.c 3841 2011-10-24 09:28:13Z ming $ */
-/*
+/* $Id: icedemo.c 4217 2012-07-27 17:24:12Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -93,7 +93,7 @@ static void err_exit(const char *title, pj_status_t status)
 
     if (icedemo.icest)
 	pj_ice_strans_destroy(icedemo.icest);
-
+    
     pj_thread_sleep(500);
 
     icedemo.thread_quit_flag = PJ_TRUE;
@@ -151,13 +151,13 @@ static pj_status_t handle_events(unsigned max_msec, unsigned *p_count)
     pj_assert(timeout.sec >= 0 && timeout.msec >= 0);
     if (timeout.msec >= 1000) timeout.msec = 999;
 
-    /* compare the value with the timeout to wait from timer, and use the
-     * minimum value.
+    /* compare the value with the timeout to wait from timer, and use the 
+     * minimum value. 
     */
     if (PJ_TIME_VAL_GT(timeout, max_timeout))
 	timeout = max_timeout;
 
-    /* Poll ioqueue.
+    /* Poll ioqueue. 
      * Repeat polling the ioqueue while we have immediate events, because
      * timer heap may process more than one events, so if we only process
      * one network events at a time (such as when IOCP backend is used),
@@ -213,7 +213,7 @@ static int icedemo_worker_thread(void *unused)
  * as STUN connectivity checks or TURN signaling).
  */
 static void cb_on_rx_data(pj_ice_strans *ice_st,
-			  unsigned comp_id,
+			  unsigned comp_id, 
 			  void *pkt, pj_size_t size,
 			  const pj_sockaddr_t *src_addr,
 			  unsigned src_addr_len)
@@ -238,11 +238,11 @@ static void cb_on_rx_data(pj_ice_strans *ice_st,
  * This is the callback that is registered to the ICE stream transport to
  * receive notification about ICE state progression.
  */
-static void cb_on_ice_complete(pj_ice_strans *ice_st,
+static void cb_on_ice_complete(pj_ice_strans *ice_st, 
 			       pj_ice_strans_op op,
 			       pj_status_t status)
 {
-    const char *opname =
+    const char *opname = 
 	(op==PJ_ICE_STRANS_OP_INIT? "initialization" :
 	    (op==PJ_ICE_STRANS_OP_NEGOTIATION ? "negotiation" : "unknown_op"));
 
@@ -270,7 +270,7 @@ static void log_func(int level, const char *data, int len)
 
 /*
  * This is the main application initialization function. It is called
- * once (and only once) during application initialization sequence by
+ * once (and only once) during application initialization sequence by 
  * main().
  */
 static pj_status_t icedemo_init(void)
@@ -296,18 +296,18 @@ static pj_status_t icedemo_init(void)
     icedemo.ice_cfg.stun_cfg.pf = &icedemo.cp.factory;
 
     /* Create application memory pool */
-    icedemo.pool = pj_pool_create(&icedemo.cp.factory, "icedemo",
+    icedemo.pool = pj_pool_create(&icedemo.cp.factory, "icedemo", 
 				  512, 512, NULL);
 
     /* Create timer heap for timer stuff */
-    CHECK( pj_timer_heap_create(icedemo.pool, 100,
+    CHECK( pj_timer_heap_create(icedemo.pool, 100, 
 				&icedemo.ice_cfg.stun_cfg.timer_heap) );
 
     /* and create ioqueue for network I/O stuff */
-    CHECK( pj_ioqueue_create(icedemo.pool, 16,
+    CHECK( pj_ioqueue_create(icedemo.pool, 16, 
 			     &icedemo.ice_cfg.stun_cfg.ioqueue) );
 
-    /* something must poll the timer heap and ioqueue,
+    /* something must poll the timer heap and ioqueue, 
      * unless we're on Symbian where the timer heap and ioqueue run
      * on themselves.
      */
@@ -318,14 +318,14 @@ static pj_status_t icedemo_init(void)
 
     /* Create DNS resolver if nameserver is set */
     if (icedemo.opt.ns.slen) {
-	CHECK( pj_dns_resolver_create(&icedemo.cp.factory,
-				      "resolver",
-				      0,
+	CHECK( pj_dns_resolver_create(&icedemo.cp.factory, 
+				      "resolver", 
+				      0, 
 				      icedemo.ice_cfg.stun_cfg.timer_heap,
-				      icedemo.ice_cfg.stun_cfg.ioqueue,
+				      icedemo.ice_cfg.stun_cfg.ioqueue, 
 				      &icedemo.ice_cfg.resolver) );
 
-	CHECK( pj_dns_resolver_set_ns(icedemo.ice_cfg.resolver, 1,
+	CHECK( pj_dns_resolver_set_ns(icedemo.ice_cfg.resolver, 1, 
 				      &icedemo.opt.ns, NULL) );
     }
 
@@ -463,8 +463,8 @@ static void icedemo_destroy_instance(void)
  */
 static void icedemo_init_session(unsigned rolechar)
 {
-    pj_ice_sess_role role = (pj_tolower((pj_uint8_t)rolechar)=='o' ?
-				PJ_ICE_SESS_ROLE_CONTROLLING :
+    pj_ice_sess_role role = (pj_tolower((pj_uint8_t)rolechar)=='o' ? 
+				PJ_ICE_SESS_ROLE_CONTROLLING : 
 				PJ_ICE_SESS_ROLE_CONTROLLED);
     pj_status_t status;
 
@@ -534,7 +534,7 @@ static int print_cand(char buffer[], unsigned maxlen,
 	  cand->foundation.ptr,
 	  (unsigned)cand->comp_id,
 	  cand->prio,
-	  pj_sockaddr_print(&cand->addr, ipaddr,
+	  pj_sockaddr_print(&cand->addr, ipaddr, 
 			    sizeof(ipaddr), 0),
 	  (unsigned)pj_sockaddr_get_port(&cand->addr));
 
@@ -550,7 +550,7 @@ static int print_cand(char buffer[], unsigned maxlen,
     return p-buffer;
 }
 
-/*
+/* 
  * Encode ICE information in SDP.
  */
 static int encode_session(char buffer[], unsigned maxlen)
@@ -562,7 +562,7 @@ static int encode_session(char buffer[], unsigned maxlen)
     pj_status_t status;
 
     /* Write "dummy" SDP v=, o=, s=, and t= lines */
-    PRINT("v=0\no=- 3414953978 3414953978 IN IP4 localhost\ns=ice\nt=0 0\n",
+    PRINT("v=0\no=- 3414953978 3414953978 IN IP4 localhost\ns=ice\nt=0 0\n", 
 	  0, 0, 0, 0, 0, 0);
 
     /* Get ufrag and pwd from current session */
@@ -574,7 +574,7 @@ static int encode_session(char buffer[], unsigned maxlen)
 	   (int)local_ufrag.slen,
 	   local_ufrag.ptr,
 	   (int)local_pwd.slen,
-	   local_pwd.ptr,
+	   local_pwd.ptr, 
 	   0, 0);
 
     /* Write each component */
@@ -614,6 +614,7 @@ static int encode_session(char buffer[], unsigned maxlen)
 	}
 
 	/* Enumerate all candidates for this component */
+	cand_cnt = PJ_ARRAY_SIZE(cand);
 	status = pj_ice_strans_enum_cands(icedemo.icest, comp+1,
 					  &cand_cnt, cand);
 	if (status != PJ_SUCCESS)
@@ -668,7 +669,7 @@ static void icedemo_show_ice(void)
 	return;
     }
 
-    printf("Negotiated comp_cnt: %d\n",
+    printf("Negotiated comp_cnt: %d\n", 
 	   pj_ice_strans_get_running_comp_cnt(icedemo.icest));
     printf("Role               : %s\n",
 	   pj_ice_strans_get_role(icedemo.icest)==PJ_ICE_SESS_ROLE_CONTROLLED ?
@@ -708,7 +709,7 @@ static void icedemo_show_ice(void)
 
 
 /*
- * Input and parse SDP from the remote (containing remote's ICE information)
+ * Input and parse SDP from the remote (containing remote's ICE information) 
  * and save it to global variables.
  */
 static void icedemo_input_remote(void)
@@ -769,14 +770,14 @@ static void icedemo_input_remote(void)
 		}
 
 		comp0_port = atoi(portstr);
-
+		
 	    }
 	    break;
 	case 'c':
 	    {
 		int cnt;
 		char c[32], net[32], ip[80];
-
+		
 		cnt = sscanf(line+2, "%s %s %s", c, net, ip);
 		if (cnt != 3) {
 		    PJ_LOG(1,(THIS_FILE, "Error parsing connection line"));
@@ -846,7 +847,7 @@ static void icedemo_input_remote(void)
 
 		    cand = &icedemo.rem.cand[icedemo.rem.cand_cnt];
 		    pj_bzero(cand, sizeof(*cand));
-
+		    
 		    if (strcmp(type, "host")==0)
 			cand->type = PJ_ICE_CAND_TYPE_HOST;
 		    else if (strcmp(type, "srflx")==0)
@@ -854,7 +855,7 @@ static void icedemo_input_remote(void)
 		    else if (strcmp(type, "relay")==0)
 			cand->type = PJ_ICE_CAND_TYPE_RELAYED;
 		    else {
-			PJ_LOG(1, (THIS_FILE, "Error: invalid candidate type '%s'",
+			PJ_LOG(1, (THIS_FILE, "Error: invalid candidate type '%s'", 
 				   type));
 			goto on_error;
 		    }
@@ -862,7 +863,7 @@ static void icedemo_input_remote(void)
 		    cand->comp_id = (pj_uint8_t)comp_id;
 		    pj_strdup2(icedemo.pool, &cand->foundation, foundation);
 		    cand->prio = prio;
-
+		    
 		    if (strchr(ipaddr, ':'))
 			af = pj_AF_INET6();
 		    else
@@ -922,7 +923,7 @@ static void icedemo_input_remote(void)
 	pj_sockaddr_set_port(&icedemo.rem.def_addr[0], (pj_uint16_t)comp0_port);
     }
 
-    PJ_LOG(3, (THIS_FILE, "Done, %d remote candidate(s) added",
+    PJ_LOG(3, (THIS_FILE, "Done, %d remote candidate(s) added", 
 	       icedemo.rem.cand_cnt));
     return;
 
@@ -956,7 +957,7 @@ static void icedemo_start_nego(void)
 
     PJ_LOG(3,(THIS_FILE, "Starting ICE negotiation.."));
 
-    status = pj_ice_strans_start_ice(icedemo.icest,
+    status = pj_ice_strans_start_ice(icedemo.icest, 
 				     pj_cstr(&rufrag, icedemo.rem.ufrag),
 				     pj_cstr(&rpwd, icedemo.rem.pwd),
 				     icedemo.rem.cand_cnt,
