@@ -1,5 +1,5 @@
-/* $Id: sip_endpoint.h 4154 2012-06-05 10:41:17Z bennylp $ */
-/*
+/* $Id: sip_endpoint.h 4275 2012-10-04 06:11:58Z bennylp $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #ifndef __PJSIP_SIP_ENDPOINT_H__
 #define __PJSIP_SIP_ENDPOINT_H__
@@ -44,13 +44,13 @@ PJ_BEGIN_DECL
  * SIP Endpoint instance (pjsip_endpoint) can be viewed as the master/owner of
  * all SIP objects in an application. It performs the following roles:
  *  - it manages the allocation/deallocation of memory pools for all objects.
- *  - it manages listeners and transports, and how they are used by
+ *  - it manages listeners and transports, and how they are used by 
  *    transactions.
  *  - it receives incoming messages from transport layer and automatically
  *    dispatches them to the correct transaction (or create a new one).
  *  - it has a single instance of timer management (timer heap).
  *  - it manages modules, which is the primary means of extending the library.
- *  - it provides single polling function for all objects and distributes
+ *  - it provides single polling function for all objects and distributes 
  *    events.
  *  - it automatically handles incoming requests which can not be handled by
  *    existing modules (such as when incoming request has unsupported method).
@@ -71,11 +71,11 @@ typedef void (*pjsip_endpt_exit_callback)(pjsip_endpoint *endpt);
 
 /**
  * Create an instance of SIP endpoint from the specified pool factory.
- * The pool factory reference then will be kept by the endpoint, so that
+ * The pool factory reference then will be kept by the endpoint, so that 
  * future memory allocations by SIP components will be taken from the same
  * pool factory.
  *
- * @param pf	        Pool factory that will be used for the lifetime of
+ * @param pf	        Pool factory that will be used for the lifetime of 
  *                      endpoint.
  * @param name          Optional name to be specified for the endpoint.
  *                      If this parameter is NULL, then the name will use
@@ -110,7 +110,7 @@ PJ_DECL(const pj_str_t*) pjsip_endpt_name(const pjsip_endpoint *endpt);
 /**
  * Poll for events. Application must call this function periodically to ensure
  * that all events from both transports and timer heap are handled in timely
- * manner.  This function, like all other endpoint functions, is thread safe,
+ * manner.  This function, like all other endpoint functions, is thread safe, 
  * and application may have more than one thread concurrently calling this function.
  *
  * @param endpt		The endpoint.
@@ -119,7 +119,7 @@ PJ_DECL(const pj_str_t*) pjsip_endpt_name(const pjsip_endpoint *endpt);
  *
  * @return		PJ_SUCCESS on success.
  */
-PJ_DECL(pj_status_t) pjsip_endpt_handle_events( pjsip_endpoint *endpt,
+PJ_DECL(pj_status_t) pjsip_endpt_handle_events( pjsip_endpoint *endpt, 
 					        const pj_time_val *max_timeout);
 
 
@@ -173,7 +173,7 @@ PJ_DECL(pj_status_t) pjsip_endpt_schedule_timer( pjsip_endpoint *endpt,
  * @param endpt	    The endpoint.
  * @param entry	    The timer entry previously registered.
  */
-PJ_DECL(void) pjsip_endpt_cancel_timer( pjsip_endpoint *endpt,
+PJ_DECL(void) pjsip_endpt_cancel_timer( pjsip_endpoint *endpt, 
 					pj_timer_entry *entry );
 
 /**
@@ -188,8 +188,8 @@ PJ_DECL(pj_timer_heap_t*) pjsip_endpt_get_timer_heap(pjsip_endpoint *endpt);
 
 /**
  * Register new module to the endpoint.
- * The endpoint will then call the load and start function in the module to
- * properly initialize the module, and assign a unique module ID for the
+ * The endpoint will then call the load and start function in the module to 
+ * properly initialize the module, and assign a unique module ID for the 
  * module.
  *
  * @param endpt		The endpoint.
@@ -202,7 +202,7 @@ PJ_DECL(pj_status_t) pjsip_endpt_register_module( pjsip_endpoint *endpt,
 
 /**
  * Unregister a module from the endpoint.
- * The endpoint will then call the stop and unload function in the module to
+ * The endpoint will then call the stop and unload function in the module to 
  * properly shutdown the module.
  *
  * @param endpt		The endpoint.
@@ -213,6 +213,77 @@ PJ_DECL(pj_status_t) pjsip_endpt_register_module( pjsip_endpoint *endpt,
 PJ_DECL(pj_status_t) pjsip_endpt_unregister_module( pjsip_endpoint *endpt,
 						    pjsip_module *module );
 
+/**
+ * This describes additional parameters to pjsip_endpt_process_rx_data()
+ * function. Application MUST call pjsip_process_rdata_param_default() to
+ * initialize this structure.
+ */
+typedef struct pjsip_process_rdata_param
+{
+    /**
+     * Specify the minimum priority number of the modules that are allowed
+     * to process the message. Default is zero to allow all modules to
+     * process the message.
+     */
+    unsigned start_prio;
+
+    /**
+     * Specify the pointer of the module where processing will start.
+     * The default is NULL, meaning processing will start from the start
+     * of the module list.
+     */
+    void *start_mod;
+
+    /**
+     * Set to N, then processing will start at Nth module after start
+     * module (where start module can be an explicit module as specified
+     * by \a start_mod or the start of module list when \a start_mod is
+     * NULL). For example, if set to 1, then processing will start from
+     * the next module after start module. Default is zero.
+     */
+    unsigned idx_after_start;
+
+    /**
+     * Print nothing to log. Default is PJ_FALSE.
+     */
+    pj_bool_t silent;
+
+} pjsip_process_rdata_param;
+
+/**
+ * Initialize with default.
+ *
+ * @param p	The param.
+ */
+PJ_DECL(void) pjsip_process_rdata_param_default(pjsip_process_rdata_param *p);
+
+/**
+ * Manually distribute the specified pjsip_rx_data to registered modules.
+ * Normally application does not need to call this function because received
+ * messages will be given to endpoint automatically by transports.
+ *
+ * Application can use this function when it has postponed the processing of
+ * an incoming message, for example to perform long operations such as
+ * database operation or to consult other servers to decide what to do with
+ * the message. In this case, application clones the original rdata, return
+ * from the callback, and perform the long operation. Upon completing the
+ * long operation, it resumes pjsip's module processing by calling this
+ * function, and then free the cloned rdata.
+ *
+ * @param endpt		The endpoint instance.
+ * @param rdata		The rdata to be distributed.
+ * @param p		Optional pointer to param to specify from which module
+ * 			the processing should start.
+ * @param p_handled	Optional pointer to receive last return value of
+ * 			module's \a on_rx_request() or \a on_rx_response()
+ * 			callback.
+ *
+ * @return		PJ_SUCCESS on success.
+ */
+PJ_DECL(pj_status_t) pjsip_endpt_process_rx_data(pjsip_endpoint *endpt,
+                                                 pjsip_rx_data *rdata,
+                                                 pjsip_process_rdata_param *p,
+                                                 pj_bool_t *p_handled);
 
 /**
  * Create pool from the endpoint. All SIP components should allocate their
@@ -330,7 +401,7 @@ PJ_DECL(pj_status_t) pjsip_endpt_set_resolver(pjsip_endpoint *endpt,
 PJ_DECL(pj_dns_resolver*) pjsip_endpt_get_resolver(pjsip_endpoint *endpt);
 
 /**
- * Asynchronously resolve a SIP target host or domain according to rule
+ * Asynchronously resolve a SIP target host or domain according to rule 
  * specified in RFC 3263 (Locating SIP Servers). When the resolving operation
  * has completed, the callback will be called.
  *
@@ -382,7 +453,7 @@ PJ_DECL(pj_ioqueue_t*) pjsip_endpt_get_ioqueue(pjsip_endpoint *endpt);
  *
  * @return	    PJ_SUCCESS on success, or the appropriate error code.
  */
-PJ_DECL(pj_status_t)
+PJ_DECL(pj_status_t) 
 pjsip_endpt_acquire_transport( pjsip_endpoint *endpt,
 			       pjsip_transport_type_e type,
 			       const pj_sockaddr_t *remote,
@@ -393,8 +464,8 @@ pjsip_endpt_acquire_transport( pjsip_endpoint *endpt,
 
 /**
  * Find a SIP transport suitable for sending SIP message to the specified
- * address by also considering the outgoing SIP message data. If transport
- * selector ("sel") is set, then the function will check if the transport
+ * address by also considering the outgoing SIP message data. If transport 
+ * selector ("sel") is set, then the function will check if the transport 
  * selected is suitable to send requests to the specified address.
  *
  * @see pjsip_tpmgr_acquire_transport
@@ -410,7 +481,7 @@ pjsip_endpt_acquire_transport( pjsip_endpoint *endpt,
  *
  * @return	    PJ_SUCCESS on success, or the appropriate error code.
  */
-PJ_DECL(pj_status_t)
+PJ_DECL(pj_status_t) 
 pjsip_endpt_acquire_transport2(pjsip_endpoint *endpt,
 			       pjsip_transport_type_e type,
 			       const pj_sockaddr_t *remote,
@@ -531,7 +602,7 @@ PJ_DECL(void) pjsip_endpt_dump( pjsip_endpoint *endpt, pj_bool_t detail );
 
 
 /**
- * Register cleanup function to be called by SIP endpoint when
+ * Register cleanup function to be called by SIP endpoint when 
  * #pjsip_endpt_destroy() is called.  Note that application should not
  * use or access any endpoint resource (such as pool, ioqueue, timer heap)
  * from within the callback as such resource may have been released when
@@ -581,3 +652,4 @@ void pjsip_endpt_send_tsx_event( pjsip_endpoint *endpt, pjsip_event *evt );
 PJ_END_DECL
 
 #endif	/* __PJSIP_SIP_ENDPOINT_H__ */
+
