@@ -1,5 +1,5 @@
-/* $Id: sip_transport.h 4173 2012-06-20 10:39:05Z ming $ */
-/*
+/* $Id: sip_transport.h 4275 2012-10-04 06:11:58Z bennylp $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #ifndef __PJSIP_SIP_TRANSPORT_H__
 #define __PJSIP_SIP_TRANSPORT_H__
@@ -71,8 +71,8 @@ enum pjsip_transport_flags_e
 {
     PJSIP_TRANSPORT_RELIABLE	    = 1,    /**< Transport is reliable.	    */
     PJSIP_TRANSPORT_SECURE	    = 2,    /**< Transport is secure.	    */
-    PJSIP_TRANSPORT_DATAGRAM	    = 4     /**< Datagram based transport.
-					         (it's also assumed to be
+    PJSIP_TRANSPORT_DATAGRAM	    = 4     /**< Datagram based transport.  
+					         (it's also assumed to be 
 						 connectionless)	    */
 };
 
@@ -117,11 +117,11 @@ PJ_DECL(pj_status_t) pjsip_transport_register_type(unsigned tp_flag,
  *
  * @param name	    Transport name, such as "TCP", or "UDP".
  *
- * @return	    The transport type, or PJSIP_TRANSPORT_UNSPECIFIED if
- *		    the name is not recognized as the name of supported
+ * @return	    The transport type, or PJSIP_TRANSPORT_UNSPECIFIED if 
+ *		    the name is not recognized as the name of supported 
  *		    transport.
  */
-PJ_DECL(pjsip_transport_type_e)
+PJ_DECL(pjsip_transport_type_e) 
 pjsip_transport_get_type_from_name(const pj_str_t *name);
 
 /**
@@ -131,7 +131,7 @@ pjsip_transport_get_type_from_name(const pj_str_t *name);
  *
  * @return	    Transport type.
  */
-PJ_DECL(pjsip_transport_type_e)
+PJ_DECL(pjsip_transport_type_e) 
 pjsip_transport_get_type_from_flag(unsigned flag);
 
 /**
@@ -161,7 +161,7 @@ pjsip_transport_get_flag_from_type( pjsip_transport_type_e type );
  * @return	    The port number, which is the default SIP port number for
  *		    the specified type.
  */
-PJ_DECL(int)
+PJ_DECL(int) 
 pjsip_transport_get_default_port_for_type(pjsip_transport_type_e type);
 
 /**
@@ -212,7 +212,7 @@ typedef enum pjsip_tpselector_type
  * when sending outgoing requests.
  *
  * Normally transport will be selected automatically according to rules about
- * sending requests. But some applications (such as proxies or B2BUAs) may
+ * sending requests. But some applications (such as proxies or B2BUAs) may 
  * want to explicitly use specific transport to send requests, for example
  * when they want to make sure that outgoing request should go from a specific
  * network interface.
@@ -260,7 +260,7 @@ PJ_DECL(void) pjsip_tpselector_dec_ref(pjsip_tpselector *sel);
  *
  *****************************************************************************/
 
-/**
+/** 
  * A customized ioqueue async operation key which is used by transport
  * to locate rdata when a pending read operation completes.
  */
@@ -285,7 +285,7 @@ struct pjsip_rx_data
      * tp_info is part of rdata that remains static for the duration of the
      * buffer. It is initialized when the buffer was created by transport.
      */
-    struct
+    struct 
     {
 	/** Memory pool for this buffer. */
 	pj_pool_t		*pool;
@@ -350,7 +350,7 @@ struct pjsip_rx_data
 	/** The parsed message, if any. */
 	pjsip_msg		*msg;
 
-	/** Short description about the message.
+	/** Short description about the message. 
 	 *  Application should use #pjsip_rx_data_get_info() instead.
 	 */
 	char			*info;
@@ -386,17 +386,17 @@ struct pjsip_rx_data
 	pjsip_clen_hdr		*clen;
 
 	/** "Require" header containing aggregates of all Require
-	 *  headers found in the message, or NULL.
+	 *  headers found in the message, or NULL. 
 	 */
 	pjsip_require_hdr	*require;
 
 	/** "Supported" header containing aggregates of all Supported
-	 *  headers found in the message, or NULL.
+	 *  headers found in the message, or NULL. 
 	 */
 	pjsip_supported_hdr	*supported;
 
-	/** The list of error generated by the parser when parsing
-	    this message.
+	/** The list of error generated by the parser when parsing 
+	    this message. 
 	 */
 	pjsip_parser_err_report parse_err;
 
@@ -409,8 +409,8 @@ struct pjsip_rx_data
      */
     struct
     {
-	/**
-	 * Data attached by modules to this message.
+	/** 
+	 * Data attached by modules to this message. 
 	 */
 	void	*mod_data[PJSIP_MAX_MODULE];
 
@@ -427,6 +427,42 @@ struct pjsip_rx_data
  */
 PJ_DECL(char*) pjsip_rx_data_get_info(pjsip_rx_data *rdata);
 
+/**
+ * Clone pjsip_rx_data. This will duplicate the contents of
+ * pjsip_rx_data and add reference count to the transport.
+ * Once application has finished using the cloned pjsip_rx_data,
+ * it must release it by calling  #pjsip_rx_data_free_cloned().
+ *
+ * By default (if flags is set to zero), this function copies the
+ * transport pointer in \a tp_info, duplicates the \a pkt_info,
+ * perform deep clone of the \a msg_info parts of the rdata, and
+ * fills the \a endpt_info (i.e. the \a mod_data) with zeros.
+ *
+ * @param src	    The source to be cloned.
+ * @param flags	    Optional flags. Must be zero for now.
+ * @param p_rdata   Pointer to receive the cloned rdata.
+ *
+ * @return	    PJ_SUCCESS on success or the appropriate error.
+ */
+PJ_DECL(pj_status_t) pjsip_rx_data_clone(const pjsip_rx_data *src,
+                                         unsigned flags,
+                                         pjsip_rx_data **p_rdata);
+
+/**
+ * Free cloned pjsip_rx_data. This function must be and must only
+ * be called for a cloned pjsip_rx_data. Specifically, it must NOT
+ * be called for the original pjsip_rx_data that is returned by
+ * transports.
+ *
+ * This function will free the memory used by the pjsip_rx_data and
+ * decrement the transport reference counter.
+ *
+ * @param rdata	    The receive data buffer.
+ *
+ * @return	    PJ_SUCCESS on success or the appropriate error.
+ */
+PJ_DECL(pj_status_t) pjsip_rx_data_free_cloned(pjsip_rx_data *rdata);
+
 
 /*****************************************************************************
  *
@@ -480,13 +516,13 @@ struct pjsip_tx_data
     /** A name to identify this buffer. */
     char		 obj_name[PJ_MAX_OBJ_NAME];
 
-    /** Short information describing this buffer and the message in it.
+    /** Short information describing this buffer and the message in it. 
      *  Application should use #pjsip_tx_data_get_info() instead of
      *  directly accessing this member.
      */
     char		*info;
 
-    /** For response message, this contains the reference to timestamp when
+    /** For response message, this contains the reference to timestamp when 
      *  the original request message was received. The value of this field
      *  is set when application creates response message to a request by
      *  calling #pjsip_endpt_create_response.
@@ -540,21 +576,21 @@ struct pjsip_tx_data
      */
     struct
     {
-	/** Server name.
+	/** Server name. 
 	 */
 	pj_str_t		 name;
 
-	/** Server addresses resolved.
+	/** Server addresses resolved. 
 	 */
 	pjsip_server_addresses   addr;
 
-	/** Current server address being tried.
+	/** Current server address being tried. 
 	 */
 	unsigned cur_addr;
 
     } dest_info;
 
-    /** Transport information, only valid during on_tx_request() and
+    /** Transport information, only valid during on_tx_request() and 
      *  on_tx_response() callback.
      */
     struct
@@ -566,8 +602,8 @@ struct pjsip_tx_data
 	int		     dst_port;	    /**< Destination port.	*/
     } tp_info;
 
-    /**
-     * Transport selector, to specify which transport to be used.
+    /** 
+     * Transport selector, to specify which transport to be used. 
      * The value here must be set with pjsip_tx_data_set_transport(),
      * to allow reference counter to be set properly.
      */
@@ -611,7 +647,7 @@ PJ_DECL(pj_status_t) pjsip_tx_data_create( pjsip_tpmgr *mgr,
 
 /**
  * Add reference counter to the transmit buffer. The reference counter controls
- * the life time of the buffer, ie. when the counter reaches zero, then it
+ * the life time of the buffer, ie. when the counter reaches zero, then it 
  * will be destroyed.
  *
  * @param tdata	    The transmit buffer.
@@ -652,7 +688,7 @@ PJ_DECL(pj_bool_t) pjsip_tx_data_is_valid( pjsip_tx_data *tdata );
 /**
  * Invalidate the print buffer to force message to be re-printed. Call
  * when the message has changed after it has been printed to buffer. The
- * message is printed to buffer normally by transport when it is about to be
+ * message is printed to buffer normally by transport when it is about to be 
  * sent to the wire. Subsequent sending of the message will not cause
  * the message to be re-printed, unless application invalidates the buffer
  * by calling this function.
@@ -721,10 +757,10 @@ typedef struct pjsip_transport_key
 typedef enum pjsip_transport_dir
 {
     PJSIP_TP_DIR_NONE,		    /**< Direction not set, normally used by
-				         connectionless transports such as
+				         connectionless transports such as 
 					 UDP transport.			    */
     PJSIP_TP_DIR_OUTGOING,	    /**< Outgoing connection or client mode,
-				         this is only for connection-oriented
+				         this is only for connection-oriented 
 					 transports.			    */
     PJSIP_TP_DIR_INCOMING,	    /**< Incoming connection or server mode,
 					 this is only for connection-oriented
@@ -760,7 +796,7 @@ struct pjsip_transport
     pjsip_host_port	    local_name;	    /**< Published name (eg. STUN). */
     pjsip_host_port	    remote_name;    /**< Remote address name.	    */
     pjsip_transport_dir	    dir;	    /**< Connection direction.	    */
-
+    
     pjsip_endpoint	   *endpt;	    /**< Endpoint instance.	    */
     pjsip_tpmgr		   *tpmgr;	    /**< Transport manager.	    */
     pj_timer_entry	    idle_timer;	    /**< Timer when ref cnt is zero.*/
@@ -784,16 +820,16 @@ struct pjsip_transport
      *			    be called.
      *
      * @return		    Should return PJ_SUCCESS only if data has been
-     *			    succesfully queued to operating system for
+     *			    succesfully queued to operating system for 
      *			    transmission. Otherwise it may return PJ_EPENDING
      *			    if the underlying transport can not send the
      *			    data immediately and will send it later, which in
-     *			    this case caller doesn't have to do anything
-     *			    except wait the calback to be called, if it
+     *			    this case caller doesn't have to do anything 
+     *			    except wait the calback to be called, if it 
      *			    supplies one.
      *			    Other return values indicate the error code.
      */
-    pj_status_t (*send_msg)(pjsip_transport *transport,
+    pj_status_t (*send_msg)(pjsip_transport *transport, 
 			    pjsip_tx_data *tdata,
 			    const pj_sockaddr_t *rem_addr,
 			    int addr_len,
@@ -870,7 +906,7 @@ PJ_DECL(pj_status_t) pjsip_transport_shutdown(pjsip_transport *tp);
  * @param tp		The transport instance.
  *
  * @return		PJ_SUCCESS on success or the appropriate error code.
- *			Some of possible errors are PJSIP_EBUSY if the
+ *			Some of possible errors are PJSIP_EBUSY if the 
  *			transport's reference counter is not zero.
  */
 PJ_DECL(pj_status_t) pjsip_transport_destroy( pjsip_transport *tp);
@@ -889,7 +925,7 @@ PJ_DECL(pj_status_t) pjsip_transport_add_ref( pjsip_transport *tp );
 /**
  * Decrement reference counter of the specified transport. When an object no
  * longer want to keep the reference to the transport, it must decrement the
- * reference counter. When the reference counter of the transport reaches
+ * reference counter. When the reference counter of the transport reaches 
  * zero, the transport manager will start the idle timer to destroy the
  * transport if no objects acquire the reference counter during the idle
  * interval.
@@ -902,7 +938,7 @@ PJ_DECL(pj_status_t) pjsip_transport_dec_ref( pjsip_transport *tp );
 
 
 /**
- * This function is called by transport instances to report an incoming
+ * This function is called by transport instances to report an incoming 
  * packet to the transport manager. The transport manager then would try to
  * parse all SIP messages in the packet, and for each parsed SIP message, it
  * would report the message to the SIP endpoint (#pjsip_endpoint).
@@ -916,7 +952,7 @@ PJ_DECL(pj_status_t) pjsip_transport_dec_ref( pjsip_transport *tp );
  *			packet. If the transport is datagram oriented, the
  *			value will be equal to the size of the packet. For
  *			stream oriented transport (e.g. TCP, TLS), the value
- *			returned may be less than the packet size, if
+ *			returned may be less than the packet size, if 
  *			partial message is received. The transport then MUST
  *			keep the remainder part and report it again to
  *			this function once more data/packet is received.
@@ -972,7 +1008,7 @@ struct pjsip_tpfactory
 
     /**
      * Create new outbound connection suitable for sending SIP message
-     * to specified remote address by also considering outgoing SIP
+     * to specified remote address by also considering outgoing SIP 
      * message data.
      * Note that the factory is responsible for both creating the
      * transport and registering it to the transport manager.
@@ -1034,7 +1070,7 @@ PJ_DECL(pj_status_t) pjsip_tpmgr_unregister_tpfactory(pjsip_tpmgr *mgr,
  * @param status    Receiption status.
  * @param rd	    Received packet.
  */
-typedef void (*pjsip_rx_callback)(pjsip_endpoint *ep, pj_status_t status,
+typedef void (*pjsip_rx_callback)(pjsip_endpoint *ep, pj_status_t status, 
 				  pjsip_rx_data *rd);
 
 /**
@@ -1069,13 +1105,15 @@ PJ_DECL(pj_status_t) pjsip_tpmgr_create( pj_pool_t *pool,
 
 /**
  * Find out the appropriate local address info (IP address and port) to
- * advertise in Contact header based on the remote address to be
+ * advertise in Contact header based on the remote address to be 
  * contacted. The local address info would be the address name of the
  * transport or listener which will be used to send the request.
  *
  * In this implementation, it will only select the transport based on
  * the transport type in the request.
  *
+ * @see pjsip_tpmgr_find_local_addr2()
+ *
  * @param tpmgr	    The transport manager.
  * @param pool	    Pool to allocate memory for the IP address.
  * @param type	    Destination address to contact.
@@ -1092,6 +1130,76 @@ PJ_DECL(pj_status_t) pjsip_tpmgr_find_local_addr( pjsip_tpmgr *tpmgr,
 						  pj_str_t *ip_addr,
 						  int *port);
 
+/**
+ * Parameter for pjsip_tpmgr_find_local_addr2() function.
+ */
+typedef struct pjsip_tpmgr_fla2_param
+{
+    /**
+     * Specify transport type to use. This must be set.
+     */
+    pjsip_transport_type_e	 tp_type;
+
+    /**
+     * Optional pointer to preferred transport, if any.
+     */
+    const pjsip_tpselector	*tp_sel;
+
+    /**
+     * Destination host, if known. The destination host is needed
+     * if \a local_if field below is set.
+     */
+    pj_str_t			 dst_host;
+
+    /**
+     * Specify if the function should return which local interface
+     * to use for the specified destination in \a dst_host. By definition,
+     * the returned address will always be local interface address.
+     */
+    pj_bool_t			 local_if;
+
+    /**
+     * The returned address.
+     */
+    pj_str_t			 ret_addr;
+
+    /**
+     * The returned port.
+     */
+    pj_uint16_t			 ret_port;
+
+    /**
+     * Returned pointer to the transport. Only set if local_if is set.
+     */
+    const void			*ret_tp;
+
+} pjsip_tpmgr_fla2_param;
+
+/**
+ * Initialize with default values.
+ *
+ * @param prm	    The parameter to be initialized.
+ */
+PJ_DECL(void) pjsip_tpmgr_fla2_param_default(pjsip_tpmgr_fla2_param *prm);
+
+/**
+ * Find out the appropriate local address info (IP address and port) to
+ * advertise in Contact or Via header header based on the remote address
+ * to be contacted. The local address info would be the address name of the
+ * transport or listener which will be used to send the request.
+ *
+ * @see pjsip_tpmgr_find_local_addr()
+ *
+ * @param tpmgr	    The transport manager.
+ * @param pool	    Pool to allocate memory for the IP address.
+ * @param param	    Function input and output parameters.
+ *
+ * @return	    PJ_SUCCESS, or the appropriate error code.
+ */
+PJ_DECL(pj_status_t) pjsip_tpmgr_find_local_addr2(pjsip_tpmgr *tpmgr,
+                                                  pj_pool_t *pool,
+                                                  pjsip_tpmgr_fla2_param *prm);
+
 /**
  * Return number of transports currently registered to the transport
  * manager.
@@ -1156,8 +1264,8 @@ PJ_DECL(pj_status_t) pjsip_tpmgr_acquire_transport(pjsip_tpmgr *mgr,
 						   pjsip_transport **tp);
 
 /**
- * Find suitable transport for sending SIP message to specified remote
- * destination by also considering the outgoing SIP message. If no suitable
+ * Find suitable transport for sending SIP message to specified remote 
+ * destination by also considering the outgoing SIP message. If no suitable 
  * transport is found, a new one will be created.
  *
  * This is an internal function since normally application doesn't have access
@@ -1202,7 +1310,7 @@ typedef void (*pjsip_tp_send_callback)(void *token, pjsip_tx_data *tdata,
 /**
  * This is a low-level function to send a SIP message using the specified
  * transport to the specified destination.
- *
+ * 
  * @param tr	    The SIP transport to be used.
  * @param tdata	    Transmit data buffer containing SIP message.
  * @param addr	    Destination address.
@@ -1212,15 +1320,15 @@ typedef void (*pjsip_tp_send_callback)(void *token, pjsip_tx_data *tdata,
  *		    the completion status of the pending send operation.
  *
  * @return	    If the message has been sent successfully, this function
- *		    will return PJ_SUCCESS and the callback will not be
+ *		    will return PJ_SUCCESS and the callback will not be 
  *		    called. If message cannot be sent immediately, this
  *		    function will return PJ_EPENDING, and application will
  *		    be notified later about the completion via the callback.
  *		    Any statuses other than PJ_SUCCESS or PJ_EPENDING
- *		    indicates immediate failure, and in this case the
+ *		    indicates immediate failure, and in this case the 
  *		    callback will not be called.
  */
-PJ_DECL(pj_status_t) pjsip_transport_send( pjsip_transport *tr,
+PJ_DECL(pj_status_t) pjsip_transport_send( pjsip_transport *tr, 
 					   pjsip_tx_data *tdata,
 					   const pj_sockaddr_t *addr,
 					   int addr_len,
@@ -1252,12 +1360,12 @@ PJ_DECL(pj_status_t) pjsip_transport_send( pjsip_transport *tr,
  *		    the completion status of the pending send operation.
  *
  * @return	    If the message has been sent successfully, this function
- *		    will return PJ_SUCCESS and the callback will not be
+ *		    will return PJ_SUCCESS and the callback will not be 
  *		    called. If message cannot be sent immediately, this
  *		    function will return PJ_EPENDING, and application will
  *		    be notified later about the completion via the callback.
  *		    Any statuses other than PJ_SUCCESS or PJ_EPENDING
- *		    indicates immediate failure, and in this case the
+ *		    indicates immediate failure, and in this case the 
  *		    callback will not be called.
  */
 PJ_DECL(pj_status_t) pjsip_tpmgr_send_raw(pjsip_tpmgr *mgr,
@@ -1281,7 +1389,7 @@ typedef enum pjsip_transport_state
 					 to connection-oriented transports
 					 such as TCP and TLS.		    */
     PJSIP_TP_STATE_DISCONNECTED	    /**< Transport disconnected, applicable
-					 only to connection-oriented
+					 only to connection-oriented 
 					 transports such as TCP and TLS.    */
 } pjsip_transport_state;
 
@@ -1336,12 +1444,12 @@ typedef void (*pjsip_tp_state_callback)(
  *
  * Note that this function will override the existing callback, if any, so
  * application is recommended to keep the old callback and manually forward
- * the notification to the old callback, otherwise other component that
- * concerns about the transport state will no longer receive transport state
+ * the notification to the old callback, otherwise other component that 
+ * concerns about the transport state will no longer receive transport state 
  * events.
- *
+ * 
  * @param mgr	    Transport manager.
- * @param cb	    Callback to be called to notify caller about transport
+ * @param cb	    Callback to be called to notify caller about transport 
  *		    state changing.
  *
  * @return	    PJ_SUCCESS on success, or the appropriate error code.
@@ -1352,7 +1460,7 @@ PJ_DECL(pj_status_t) pjsip_tpmgr_set_state_cb(pjsip_tpmgr *mgr,
 
 /**
  * Get the callback of global transport state notification.
- *
+ * 
  * @param mgr	    Transport manager.
  *
  * @return	    The transport state callback or NULL if it is not set.
@@ -1363,9 +1471,9 @@ PJ_DECL(pjsip_tp_state_callback) pjsip_tpmgr_get_state_cb(
 
 /**
  * Add a listener to the specified transport for transport state notification.
- *
+ * 
  * @param tp	    The transport.
- * @param cb	    Callback to be called to notify listener about transport
+ * @param cb	    Callback to be called to notify listener about transport 
  *		    state changing.
  * @param user_data The user data.
  * @param key	    Output key, used to remove this listener.
@@ -1380,9 +1488,9 @@ PJ_DECL(pj_status_t) pjsip_transport_add_state_listener (
 
 
 /**
- * Remove a listener from the specified transport for transport state
+ * Remove a listener from the specified transport for transport state 
  * notification.
- *
+ * 
  * @param tp	    The transport.
  * @param key	    The listener key.
  * @param user_data The user data, for validation purpose.
@@ -1403,3 +1511,4 @@ PJ_DECL(pj_status_t) pjsip_transport_remove_state_listener (
 PJ_END_DECL
 
 #endif	/* __PJSIP_SIP_TRANSPORT_H__ */
+
