@@ -1,5 +1,5 @@
-/* $Id: sip_util.h 3553 2011-05-05 06:14:19Z nanang $ */
-/*
+/* $Id: sip_util.h 4347 2013-02-13 10:19:25Z nanang $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #ifndef __PJSIP_SIP_MISC_H__
 #define __PJSIP_SIP_MISC_H__
@@ -34,11 +34,11 @@ PJ_BEGIN_DECL
  * This module provides utility functions to manage target set for UAC.
  * The target set is provided as pjsip_target_set structure. Initially,
  * the target set for UAC contains only one target, that is the target of
- * the initial request. When 3xx/redirection class response is received,
+ * the initial request. When 3xx/redirection class response is received, 
  * the UAC can use the functionality of this module to add the URI's listed
- * in the Contact header(s) in the response to the target set, and retry
- * sending the request to the next destination/target. The UAC may retry
- * this sequentially until one of the target answers with succesful/2xx
+ * in the Contact header(s) in the response to the target set, and retry 
+ * sending the request to the next destination/target. The UAC may retry 
+ * this sequentially until one of the target answers with succesful/2xx 
  * response, or one target returns global error/6xx response, or all targets
  * are exhausted.
  *
@@ -47,9 +47,9 @@ PJ_BEGIN_DECL
 
 /**
  * This structure describes a target, which can be chained together to form
- * a target set. Each target contains an URI, priority (as q-value), and
- * the last status code and reason phrase received from the target, if the
- * target has been contacted. If the target has not been contacted, the
+ * a target set. Each target contains an URI, priority (as q-value), and 
+ * the last status code and reason phrase received from the target, if the 
+ * target has been contacted. If the target has not been contacted, the 
  * status code field will be zero.
  */
 typedef struct pjsip_target
@@ -91,6 +91,13 @@ typedef enum pjsip_redirect_op
      */
     PJSIP_REDIRECT_ACCEPT,
 
+    /**
+     * Accept the redirection to the current target and replace the To
+     * header in the INVITE request with the current target. The INVITE
+     * request will be resent to the current target.
+     */
+    PJSIP_REDIRECT_ACCEPT_REPLACE,
+
     /**
      * Defer the redirection decision, for example to request permission
      * from the end user.
@@ -145,7 +152,7 @@ PJ_DECL(pj_status_t) pjsip_target_set_add_uri(pjsip_target_set *tset,
 
 /**
  * Extract URI's in the Contact headers of the specified (response) message
- * and add them to the target set. This function will also check if the
+ * and add them to the target set. This function will also check if the 
  * URI's already exist in the target set before adding them to the list.
  *
  * @param tset	    The target set.
@@ -154,8 +161,8 @@ PJ_DECL(pj_status_t) pjsip_target_set_add_uri(pjsip_target_set *tset,
  *		    scanned and the URI's to be extracted, checked, and
  *		    added to the target set.
  *
- * @return	    PJ_SUCCESS if at least one URI was added to the
- *		    target set, or PJ_EEXISTS if all URI's in the message
+ * @return	    PJ_SUCCESS if at least one URI was added to the 
+ *		    target set, or PJ_EEXISTS if all URI's in the message 
  *		    already exists in the target set or if the message
  *		    doesn't contain usable Contact headers, or other error
  *		    codes.
@@ -177,7 +184,7 @@ PJ_DECL(pj_status_t) pjsip_target_set_add_from_msg(pjsip_target_set *tset,
  *		    been tried or at least one target returns 2xx or 6xx
  *		    response.
  */
-PJ_DECL(pjsip_target*)
+PJ_DECL(pjsip_target*) 
 pjsip_target_set_get_next(const pjsip_target_set *tset);
 
 
@@ -245,12 +252,12 @@ PJ_DECL(pj_status_t) pjsip_target_assign_status(pjsip_target *target,
  * @param to	    URL to put in To header.
  * @param contact   Contact to be put as Contact header value, hence
  *		    the format must follow RFC 3261 Section 20.10:
- *		    When the header field value contains a display
- *		    name, the URI including all URI parameters is
- *		    enclosed in "<" and ">".  If no "<" and ">" are
+ *		    When the header field value contains a display 
+ *		    name, the URI including all URI parameters is 
+ *		    enclosed in "<" and ">".  If no "<" and ">" are 
  *		    present, all parameters after the URI are header
- *		    parameters, not URI parameters.  The display name
- *		    can be tokens, or a quoted string, if a larger
+ *		    parameters, not URI parameters.  The display name 
+ *		    can be tokens, or a quoted string, if a larger 
  *		    character set is desired.
  * @param call_id   Optional Call-ID (put NULL to generate unique Call-ID).
  * @param cseq	    Optional CSeq (put -1 to generate random CSeq).
@@ -259,14 +266,14 @@ PJ_DECL(pj_status_t) pjsip_target_assign_status(pjsip_target *target,
  *
  * @return	    PJ_SUCCESS, or the appropriate error code.
  */
-PJ_DECL(pj_status_t) pjsip_endpt_create_request( pjsip_endpoint *endpt,
+PJ_DECL(pj_status_t) pjsip_endpt_create_request( pjsip_endpoint *endpt, 
 						 const pjsip_method *method,
 						 const pj_str_t *target,
 						 const pj_str_t *from,
-						 const pj_str_t *to,
+						 const pj_str_t *to, 
 						 const pj_str_t *contact,
 						 const pj_str_t *call_id,
-						 int cseq,
+						 int cseq, 
 						 const pj_str_t *text,
 						 pjsip_tx_data **p_tdata);
 
@@ -312,7 +319,7 @@ pjsip_endpt_create_request_from_hdr( pjsip_endpoint *endpt,
 
 /**
  * Construct a minimal response message for the received request. This function
- * will construct all the Via, Record-Route, Call-ID, From, To, CSeq, and
+ * will construct all the Via, Record-Route, Call-ID, From, To, CSeq, and 
  * Call-ID headers from the request.
  *
  * Note: the txdata reference counter is set to ZERO!.
@@ -337,7 +344,7 @@ PJ_DECL(pj_status_t) pjsip_endpt_create_response( pjsip_endpoint *endpt,
  * an ACK request to 3xx-6xx final response.
  * The generation of ACK message for 2xx final response is different than
  * this one.
- *
+ * 
  * @param endpt	    The endpoint.
  * @param tdata	    This contains the original INVITE request
  * @param rdata	    The final response.
@@ -394,7 +401,7 @@ PJ_DECL(pj_status_t) pjsip_get_request_dest(const pjsip_tx_data *tdata,
 
 /**
  * Process route-set found in the request and calculate destination to be
- * used to send the request message, based on the request URI and Route
+ * used to send the request message, based on the request URI and Route 
  * headers in the message. The procedure used here follows the guidelines
  * on sending the request in RFC 3261 chapter 8.1.2.
  *
@@ -443,15 +450,15 @@ typedef struct pjsip_send_state
      */
     void *token;
 
-    /** Endpoint instance.
+    /** Endpoint instance. 
      */
     pjsip_endpoint *endpt;
 
-    /** Transmit data buffer being sent.
+    /** Transmit data buffer being sent. 
      */
     pjsip_tx_data *tdata;
 
-    /** Current transport being used.
+    /** Current transport being used. 
      */
     pjsip_transport *cur_transport;
 
@@ -465,7 +472,7 @@ typedef struct pjsip_send_state
 
 
 /**
- * Declaration for callback function to be specified in
+ * Declaration for callback function to be specified in 
  * #pjsip_endpt_send_request_stateless(), #pjsip_endpt_send_response(), or
  * #pjsip_endpt_send_response2().
  *
@@ -478,11 +485,11 @@ typedef void (*pjsip_send_callback)(pjsip_send_state *st, pj_ssize_t sent,
 				    pj_bool_t *cont);
 
 /**
- * Send outgoing request statelessly The function will take care of which
+ * Send outgoing request statelessly The function will take care of which 
  * destination and transport to use based on the information in the message,
  * taking care of URI in the request line and Route header.
  *
- * This function is different than #pjsip_transport_send() in that this
+ * This function is different than #pjsip_transport_send() in that this 
  * function adds/modify the Via header as necessary.
  *
  * @param endpt	    The endpoint instance.
@@ -494,7 +501,7 @@ typedef void (*pjsip_send_callback)(pjsip_send_state *st, pj_ssize_t sent,
  *
  * @return	    PJ_SUCCESS, or the appropriate error code.
  */
-PJ_DECL(pj_status_t)
+PJ_DECL(pj_status_t) 
 pjsip_endpt_send_request_stateless( pjsip_endpoint *endpt,
 				    pjsip_tx_data *tdata,
 				    void *token,
@@ -520,12 +527,12 @@ pjsip_endpt_send_request_stateless( pjsip_endpoint *endpt,
  *		    the completion status of the pending send operation.
  *
  * @return	    If the message has been sent successfully, this function
- *		    will return PJ_SUCCESS and the callback will not be
+ *		    will return PJ_SUCCESS and the callback will not be 
  *		    called. If message cannot be sent immediately, this
  *		    function will return PJ_EPENDING, and application will
  *		    be notified later about the completion via the callback.
  *		    Any statuses other than PJ_SUCCESS or PJ_EPENDING
- *		    indicates immediate failure, and in this case the
+ *		    indicates immediate failure, and in this case the 
  *		    callback will not be called.
  */
 PJ_DECL(pj_status_t) pjsip_endpt_send_raw(pjsip_endpoint *endpt,
@@ -558,12 +565,12 @@ PJ_DECL(pj_status_t) pjsip_endpt_send_raw(pjsip_endpoint *endpt,
  *		    the completion status of the pending send operation.
  *
  * @return	    If the message has been sent successfully, this function
- *		    will return PJ_SUCCESS and the callback will not be
+ *		    will return PJ_SUCCESS and the callback will not be 
  *		    called. If message cannot be sent immediately, this
  *		    function will return PJ_EPENDING, and application will
  *		    be notified later about the completion via the callback.
  *		    Any statuses other than PJ_SUCCESS or PJ_EPENDING
- *		    indicates immediate failure, and in this case the
+ *		    indicates immediate failure, and in this case the 
  *		    callback will not be called.
  */
 PJ_DECL(pj_status_t) pjsip_endpt_send_raw_to_uri(pjsip_endpoint *endpt,
@@ -617,13 +624,13 @@ PJ_DECL(pj_status_t) pjsip_get_response_addr(pj_pool_t *pool,
 					     pjsip_response_addr *res_addr);
 
 /**
- * Send response in tdata statelessly. The function will take care of which
- * response destination and transport to use based on the information in the
+ * Send response in tdata statelessly. The function will take care of which 
+ * response destination and transport to use based on the information in the 
  * Via header (such as the presence of rport, symmetric transport, etc.).
  *
- * This function will create a new ephemeral transport if no existing
+ * This function will create a new ephemeral transport if no existing 
  * transports can be used to send the message to the destination. The ephemeral
- * transport will be destroyed after some period if it is not used to send any
+ * transport will be destroyed after some period if it is not used to send any 
  * more messages.
  *
  * The behavior of this function complies with section 18.2.2 of RFC 3261
@@ -638,10 +645,10 @@ PJ_DECL(pj_status_t) pjsip_get_response_addr(pj_pool_t *pool,
  * @param cb	    Optional callback to notify the transmission status
  *		    to application, and to inform whether next address or
  *		    transport will be tried.
- *
+ * 
  * @return	    PJ_SUCCESS if response has been successfully created and
- *		    sent to transport layer, or a non-zero error code.
- *		    However, even when it returns PJ_SUCCESS, there is no
+ *		    sent to transport layer, or a non-zero error code. 
+ *		    However, even when it returns PJ_SUCCESS, there is no 
  *		    guarantee that the response has been successfully sent.
  */
 PJ_DECL(pj_status_t) pjsip_endpt_send_response( pjsip_endpoint *endpt,
@@ -661,10 +668,10 @@ PJ_DECL(pj_status_t) pjsip_endpt_send_response( pjsip_endpoint *endpt,
  * @param cb	    Optional callback to notify the transmission status
  *		    to application, and to inform whether next address or
  *		    transport will be tried.
- *
+ * 
  * @return	    PJ_SUCCESS if response has been successfully created and
- *		    sent to transport layer, or a non-zero error code.
- *		    However, even when it returns PJ_SUCCESS, there is no
+ *		    sent to transport layer, or a non-zero error code. 
+ *		    However, even when it returns PJ_SUCCESS, there is no 
  *		    guarantee that the response has been successfully sent.
  */
 PJ_DECL(pj_status_t) pjsip_endpt_send_response2(pjsip_endpoint *endpt,
@@ -694,7 +701,7 @@ PJ_DECL(pj_status_t) pjsip_endpt_respond_stateless(pjsip_endpoint *endpt,
 						   const pj_str_t *st_text,
 						   const pjsip_hdr *hdr_list,
 						   const pjsip_msg_body *body);
-
+						    
 /**
  * @}
  */
@@ -748,10 +755,10 @@ typedef void (*pjsip_endpt_send_callback)(void *token, pjsip_event *e);
  *
  * @param endpt	    The endpoint instance.
  * @param tdata	    The transmit data to be sent.
- * @param timeout   Optional timeout for final response to be received, or -1
+ * @param timeout   Optional timeout for final response to be received, or -1 
  *		    if the transaction should not have a timeout restriction.
  *		    The value is in miliseconds.
- * @param token	    Optional token to be associated with the transaction, and
+ * @param token	    Optional token to be associated with the transaction, and 
  *		    to be passed to the callback.
  * @param cb	    Optional callback to be called when the transaction has
  *		    received a final response. The callback will be called with
@@ -777,10 +784,10 @@ PJ_DECL(pj_status_t) pjsip_endpt_send_request( pjsip_endpoint *endpt,
  */
 
 /**
- * Create new request message to be forwarded upstream to new destination URI
- * in uri. The new request is a full/deep clone of the request received in
- * rdata, unless if other copy mechanism is specified in the options.
- * The branch parameter, if not NULL, will be used as the branch-param in
+ * Create new request message to be forwarded upstream to new destination URI 
+ * in uri. The new request is a full/deep clone of the request received in 
+ * rdata, unless if other copy mechanism is specified in the options. 
+ * The branch parameter, if not NULL, will be used as the branch-param in 
  * the Via header. If it is NULL, then a unique branch parameter will be used.
  *
  * Note: this function DOES NOT perform Route information preprocessing as
@@ -794,7 +801,7 @@ PJ_DECL(pj_status_t) pjsip_endpt_send_request( pjsip_endpoint *endpt,
  * @param branch    Optional branch parameter. Application may specify its
  *		    own branch, for example if it wishes to perform loop
  *		    detection. If the branch parameter is not specified,
- *		    this function will generate its own by calling
+ *		    this function will generate its own by calling 
  *		    #pjsip_calculate_branch_id() function.
  * @param options   Optional option flags when duplicating the message.
  * @param tdata	    The result.
@@ -802,7 +809,7 @@ PJ_DECL(pj_status_t) pjsip_endpt_send_request( pjsip_endpoint *endpt,
  * @return	    PJ_SUCCESS on success.
  */
 PJ_DECL(pj_status_t) pjsip_endpt_create_request_fwd(pjsip_endpoint *endpt,
-						    pjsip_rx_data *rdata,
+						    pjsip_rx_data *rdata, 
 						    const pjsip_uri *uri,
 						    const pj_str_t *branch,
 						    unsigned options,
@@ -811,11 +818,11 @@ PJ_DECL(pj_status_t) pjsip_endpt_create_request_fwd(pjsip_endpoint *endpt,
 
 
 /**
- * Create new response message to be forwarded downstream by the proxy from
- * the response message found in rdata. Note that this function practically
- * will clone the response as is, i.e. without checking the validity of the
- * response or removing top most Via header. This function will perform
- * full/deep clone of the response, unless other copy mechanism is used in
+ * Create new response message to be forwarded downstream by the proxy from 
+ * the response message found in rdata. Note that this function practically 
+ * will clone the response as is, i.e. without checking the validity of the 
+ * response or removing top most Via header. This function will perform 
+ * full/deep clone of the response, unless other copy mechanism is used in 
  * the options.
  *
  * @param endpt	    The endpoint instance.
@@ -826,16 +833,16 @@ PJ_DECL(pj_status_t) pjsip_endpt_create_request_fwd(pjsip_endpoint *endpt,
  * @return	    PJ_SUCCESS on success.
  */
 PJ_DECL(pj_status_t) pjsip_endpt_create_response_fwd( pjsip_endpoint *endpt,
-						      pjsip_rx_data *rdata,
+						      pjsip_rx_data *rdata, 
 						      unsigned options,
 						      pjsip_tx_data **tdata);
 
 
 
 /**
- * Create a globally unique branch parameter based on the information in
+ * Create a globally unique branch parameter based on the information in 
  * the incoming request message, for the purpose of creating a new request
- * for forwarding. This is the default implementation used by
+ * for forwarding. This is the default implementation used by 
  * #pjsip_endpt_create_request_fwd() function if the branch parameter is
  * not specified.
  *
@@ -858,3 +865,4 @@ PJ_DECL(pj_str_t) pjsip_calculate_branch_id( pjsip_rx_data *rdata );
 PJ_END_DECL
 
 #endif	/* __PJSIP_SIP_MISC_H__ */
+
