@@ -1,5 +1,5 @@
-/* $Id: sip_100rel.c 3841 2011-10-24 09:28:13Z ming $ */
-/*
+/* $Id: sip_100rel.c 4208 2012-07-18 07:52:33Z ming $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjsip-ua/sip_100rel.h>
 #include <pjsip/sip_endpoint.h>
@@ -60,7 +60,7 @@ static struct mod_100rel
 {
     pjsip_module	 mod;
     pjsip_endpoint	*endpt;
-} mod_100rel =
+} mod_100rel = 
 {
     {
 	NULL, NULL,			    /* prev, next.		*/
@@ -130,10 +130,10 @@ struct dlg_data
 static pj_status_t mod_100rel_load(pjsip_endpoint *endpt)
 {
     mod_100rel.endpt = endpt;
-    pjsip_endpt_add_capability(endpt, &mod_100rel.mod,
+    pjsip_endpt_add_capability(endpt, &mod_100rel.mod, 
 			       PJSIP_H_ALLOW, NULL,
 			       1, &pjsip_prack_method.name);
-    pjsip_endpt_add_capability(endpt, &mod_100rel.mod,
+    pjsip_endpt_add_capability(endpt, &mod_100rel.mod, 
 			       PJSIP_H_SUPPORTED, NULL,
 			       1, &tag_100rel);
 
@@ -168,7 +168,7 @@ static pjsip_require_hdr *find_req_hdr(pjsip_msg *msg)
 
 
 /*
- * Get PRACK method constant.
+ * Get PRACK method constant. 
  */
 PJ_DEF(const pjsip_method*) pjsip_get_prack_method(void)
 {
@@ -264,7 +264,7 @@ PJ_DEF(pj_status_t) pjsip_100rel_create_prack( pjsip_inv_session *inv,
     rseq_hdr = (pjsip_generic_string_hdr*)
 	       pjsip_msg_find_hdr_by_name(msg, &RSEQ, NULL);
     if (rseq_hdr == NULL) {
-	PJ_LOG(4,(dd->inv->dlg->obj_name,
+	PJ_LOG(4,(dd->inv->dlg->obj_name, 
 		 "Ignoring 100rel response with no RSeq header"));
 	return PJSIP_EMISSINGHDR;
     }
@@ -273,7 +273,7 @@ PJ_DEF(pj_status_t) pjsip_100rel_create_prack( pjsip_inv_session *inv,
     /* Find UAC state for the specified call leg */
     uac_state = dd->uac_state_list;
     while (uac_state) {
-	if (pj_strcmp(&uac_state->tag, to_tag)==0)
+	if (pj_stricmp(&uac_state->tag, to_tag)==0)
 	    break;
 	uac_state = uac_state->next;
     }
@@ -301,7 +301,7 @@ PJ_DEF(pj_status_t) pjsip_100rel_create_prack( pjsip_inv_session *inv,
 
     /* Ignore provisional response with out-of-order RSeq */
     } else if (rseq != uac_state->rseq + 1) {
-	PJ_LOG(4,(dd->inv->dlg->obj_name,
+	PJ_LOG(4,(dd->inv->dlg->obj_name, 
 		 "Ignoring 100rel response because RSeq jump "
 		 "(expecting %u, got %u)",
 		 uac_state->rseq+1, rseq));
@@ -320,7 +320,7 @@ PJ_DEF(pj_status_t) pjsip_100rel_create_prack( pjsip_inv_session *inv,
     /* If this response is a forked response from a different call-leg,
      * update the req URI (https://trac.pjsip.org/repos/ticket/1364)
      */
-    if (pj_strcmp(&uac_state->tag, &dd->inv->dlg->remote.info->tag)) {
+    if (pj_stricmp(&uac_state->tag, &dd->inv->dlg->remote.info->tag)) {
 	const pjsip_contact_hdr *mhdr;
 
 	mhdr = (const pjsip_contact_hdr*)
@@ -363,10 +363,10 @@ PJ_DEF(pj_status_t) pjsip_100rel_send_prack( pjsip_inv_session *inv,
     dlg_data *dd;
 
     dd = (dlg_data*) inv->dlg->mod_data[mod_100rel.mod.id];
-    PJ_ASSERT_ON_FAIL(dd != NULL,
+    PJ_ASSERT_ON_FAIL(dd != NULL, 
     {pjsip_tx_data_dec_ref(tdata); return PJSIP_ENOTINITIALIZED; });
 
-    return pjsip_dlg_send_request(inv->dlg, tdata,
+    return pjsip_dlg_send_request(inv->dlg, tdata, 
 				  mod_100rel.mod.id, (void*) dd);
 
 }
@@ -460,12 +460,12 @@ PJ_DEF(pj_status_t) pjsip_100rel_on_rx_prack( pjsip_inv_session *inv,
 
     dd = (dlg_data*) inv->dlg->mod_data[mod_100rel.mod.id];
     if (dd == NULL) {
-	/* UAC sends us PRACK while we didn't send reliable provisional
-	 * response. Respond with 400 (?)
+	/* UAC sends us PRACK while we didn't send reliable provisional 
+	 * response. Respond with 400 (?) 
 	 */
 	const pj_str_t reason = pj_str("Unexpected PRACK");
 
-	status = pjsip_dlg_create_response(inv->dlg, rdata, 400,
+	status = pjsip_dlg_create_response(inv->dlg, rdata, 400, 
 					   &reason, &tdata);
 	if (status == PJ_SUCCESS) {
 	    status = pjsip_dlg_send_response(inv->dlg, tsx, tdata);
@@ -481,7 +481,7 @@ PJ_DEF(pj_status_t) pjsip_100rel_on_rx_prack( pjsip_inv_session *inv,
 
     /* Ignore if we don't have pending transmission */
     if (dd->uas_state == NULL || pj_list_empty(&dd->uas_state->tx_data_list)) {
-	PJ_LOG(4,(dd->inv->dlg->obj_name,
+	PJ_LOG(4,(dd->inv->dlg->obj_name, 
 		  "PRACK ignored - no pending response"));
 	return PJ_EIGNORED;
     }
@@ -503,7 +503,7 @@ PJ_DEF(pj_status_t) pjsip_100rel_on_rx_prack( pjsip_inv_session *inv,
     if (rseq == dd->uas_state->tx_data_list.next->rseq &&
 	cseq == dd->uas_state->cseq)
     {
-	/*
+	/* 
 	 * Yes this PRACK matches outgoing transmission.
 	 */
 	tx_data_list_t *tl = dd->uas_state->tx_data_list.next;
@@ -530,7 +530,7 @@ PJ_DEF(pj_status_t) pjsip_100rel_on_rx_prack( pjsip_inv_session *inv,
 
     } else {
 	/* No it doesn't match */
-	PJ_LOG(4,(dd->inv->dlg->obj_name,
+	PJ_LOG(4,(dd->inv->dlg->obj_name, 
 		 "Rx PRACK with no matching reliable response"));
 	return PJ_EIGNORED;
     }
@@ -562,7 +562,7 @@ static void on_retransmit(pj_timer_heap_t *timer_heap,
     if (dd->uas_state->retransmit_count >= 7) {
 	/* If a reliable provisional response is retransmitted for
 	   64*T1 seconds  without reception of a corresponding PRACK,
-	   the UAS SHOULD reject the original request with a 5xx
+	   the UAS SHOULD reject the original request with a 5xx 
 	   response.
 	*/
 	pj_str_t reason = pj_str("Reliable response timed out");
@@ -574,7 +574,7 @@ static void on_retransmit(pj_timer_heap_t *timer_heap,
 	/* Send 500 response */
 	status = pjsip_inv_end_session(dd->inv, 500, &reason, &tdata);
 	if (status == PJ_SUCCESS) {
-	    pjsip_dlg_send_response(dd->inv->dlg,
+	    pjsip_dlg_send_response(dd->inv->dlg, 
 				    dd->inv->invite_tsx,
 				    tdata);
 	}
@@ -606,7 +606,7 @@ static void on_retransmit(pj_timer_heap_t *timer_heap,
     /* Schedule next retransmission */
     if (dd->uas_state->retransmit_count < 6) {
 	delay.sec = 0;
-	delay.msec = (1 << dd->uas_state->retransmit_count) *
+	delay.msec = (1 << dd->uas_state->retransmit_count) * 
 		     pjsip_cfg()->tsx.t1;
 	pj_time_val_normalize(&delay);
     } else {
@@ -615,7 +615,7 @@ static void on_retransmit(pj_timer_heap_t *timer_heap,
     }
 
 
-    pjsip_endpt_schedule_timer(dd->inv->dlg->endpt,
+    pjsip_endpt_schedule_timer(dd->inv->dlg->endpt, 
 			       &dd->uas_state->retransmit_timer,
 			       &delay);
 
@@ -642,7 +642,7 @@ static pjsip_tx_data *clone_tdata(dlg_data *dd,
 
     /* Duplicate status line */
     msg->line.status.code = src->msg->line.status.code;
-    pj_strdup(dst->pool, &msg->line.status.reason,
+    pj_strdup(dst->pool, &msg->line.status.reason, 
 	      &src->msg->line.status.reason);
 
     /* Duplicate all headers */
@@ -692,22 +692,22 @@ PJ_DEF(pj_status_t) pjsip_100rel_tx_response(pjsip_inv_session *inv,
     dlg_data *dd;
     pjsip_tx_data *old_tdata;
     pj_status_t status;
-
+    
     PJ_ASSERT_RETURN(tdata->msg->type == PJSIP_RESPONSE_MSG,
 		     PJSIP_ENOTRESPONSEMSG);
-
+    
     status_code = tdata->msg->line.status.code;
-
+    
     /* 100 response doesn't need PRACK */
     if (status_code == 100)
 	return pjsip_dlg_send_response(inv->dlg, inv->invite_tsx, tdata);
-
+    
 
     /* Get the 100rel data attached to this dialog */
     dd = (dlg_data*) inv->dlg->mod_data[mod_100rel.mod.id];
     PJ_ASSERT_RETURN(dd != NULL, PJ_EINVALIDOP);
-
-
+    
+    
     /* Clone tdata.
      * We need to clone tdata because we may need to keep it in our
      * retransmission list, while the original dialog may modify it
@@ -716,122 +716,122 @@ PJ_DEF(pj_status_t) pjsip_100rel_tx_response(pjsip_inv_session *inv,
     old_tdata = tdata;
     tdata = clone_tdata(dd, old_tdata);
     pjsip_tx_data_dec_ref(old_tdata);
-
+    
 
     /* Get CSeq header, and make sure this is INVITE response */
     cseq_hdr = (pjsip_cseq_hdr*)
 	        pjsip_msg_find_hdr(tdata->msg, PJSIP_H_CSEQ, NULL);
     PJ_ASSERT_RETURN(cseq_hdr != NULL, PJ_EBUG);
-    PJ_ASSERT_RETURN(cseq_hdr->method.id == PJSIP_INVITE_METHOD,
+    PJ_ASSERT_RETURN(cseq_hdr->method.id == PJSIP_INVITE_METHOD, 
 	PJ_EINVALIDOP);
-
+    
     /* Remove existing Require header */
     req_hdr = find_req_hdr(tdata->msg);
     if (req_hdr) {
 	pj_list_erase(req_hdr);
     }
-
+    
     /* Remove existing RSeq header */
     rseq_hdr = (pjsip_generic_string_hdr*)
 	pjsip_msg_find_hdr_by_name(tdata->msg, &RSEQ, NULL);
     if (rseq_hdr)
 	pj_list_erase(rseq_hdr);
-
+    
     /* Different treatment for provisional and final response */
     if (status_code/100 == 2) {
-
+	
 	/* RFC 3262 Section 3: UAS Behavior:
-
-	  The UAS MAY send a final response to the initial request
-	  before having received PRACKs for all unacknowledged
-	  reliable provisional responses, unless the final response
-	  is 2xx and any of the unacknowledged reliable provisional
-	  responses contained a session description.  In that case,
-	  it MUST NOT send a final response until those provisional
+    
+	  The UAS MAY send a final response to the initial request 
+	  before having received PRACKs for all unacknowledged 
+	  reliable provisional responses, unless the final response 
+	  is 2xx and any of the unacknowledged reliable provisional 
+	  responses contained a session description.  In that case, 
+	  it MUST NOT send a final response until those provisional 
 	  responses are acknowledged.
 	*/
-
+	
 	if (dd->uas_state && has_sdp(dd)) {
 	    /* Yes we have transmitted 1xx with SDP reliably.
 	     * In this case, must queue the 2xx response.
 	     */
 	    tx_data_list_t *tl;
-
+	    
 	    tl = PJ_POOL_ZALLOC_T(tdata->pool, tx_data_list_t);
 	    tl->tdata = tdata;
 	    tl->rseq = (pj_uint32_t)-1;
 	    pj_list_push_back(&dd->uas_state->tx_data_list, tl);
-
+	    
 	    /* Will send later */
 	    status = PJ_SUCCESS;
-
-	    PJ_LOG(4,(dd->inv->dlg->obj_name,
+	    
+	    PJ_LOG(4,(dd->inv->dlg->obj_name, 
 		      "2xx response will be sent after PRACK"));
-
+	    
 	} else if (dd->uas_state) {
-	    /*
+	    /* 
 	    RFC 3262 Section 3: UAS Behavior:
 
 	    If the UAS does send a final response when reliable
-	    responses are still unacknowledged, it SHOULD NOT
+	    responses are still unacknowledged, it SHOULD NOT 
 	    continue to retransmit the unacknowledged reliable
-	    provisional responses, but it MUST be prepared to
-	    process PRACK requests for those outstanding
+	    provisional responses, but it MUST be prepared to 
+	    process PRACK requests for those outstanding 
 	    responses.
 	    */
-
-	    PJ_LOG(4,(dd->inv->dlg->obj_name,
+	    
+	    PJ_LOG(4,(dd->inv->dlg->obj_name, 
 		      "No SDP sent so far, sending 2xx now"));
-
+	    
 	    /* Cancel the retransmit timer */
 	    if (dd->uas_state->retransmit_timer.id) {
 		pjsip_endpt_cancel_timer(dd->inv->dlg->endpt,
 					 &dd->uas_state->retransmit_timer);
 		dd->uas_state->retransmit_timer.id = PJ_FALSE;
 	    }
-
+	    
 	    /* Clear all pending responses (drop 'em) */
 	    clear_all_responses(dd);
-
+	    
 	    /* And transmit the 2xx response */
-	    status=pjsip_dlg_send_response(inv->dlg,
+	    status=pjsip_dlg_send_response(inv->dlg, 
 					   inv->invite_tsx, tdata);
-
+	    
 	} else {
 	    /* We didn't send any reliable provisional response */
-
+	    
 	    /* Transmit the 2xx response */
-	    status=pjsip_dlg_send_response(inv->dlg,
+	    status=pjsip_dlg_send_response(inv->dlg, 
 					   inv->invite_tsx, tdata);
 	}
-
+	
     } else if (status_code >= 300) {
-
-	/*
+	
+	/* 
 	RFC 3262 Section 3: UAS Behavior:
 
 	If the UAS does send a final response when reliable
-	responses are still unacknowledged, it SHOULD NOT
+	responses are still unacknowledged, it SHOULD NOT 
 	continue to retransmit the unacknowledged reliable
-	provisional responses, but it MUST be prepared to
-	process PRACK requests for those outstanding
+	provisional responses, but it MUST be prepared to 
+	process PRACK requests for those outstanding 
 	responses.
 	*/
-
+	
 	/* Cancel the retransmit timer */
 	if (dd->uas_state && dd->uas_state->retransmit_timer.id) {
 	    pjsip_endpt_cancel_timer(dd->inv->dlg->endpt,
 				     &dd->uas_state->retransmit_timer);
 	    dd->uas_state->retransmit_timer.id = PJ_FALSE;
-
+	    
 	    /* Clear all pending responses (drop 'em) */
 	    clear_all_responses(dd);
 	}
-
+	
 	/* And transmit the 2xx response */
-	status=pjsip_dlg_send_response(inv->dlg,
+	status=pjsip_dlg_send_response(inv->dlg, 
 				       inv->invite_tsx, tdata);
-
+	
     } else {
 	/*
 	 * This is provisional response.
@@ -839,7 +839,7 @@ PJ_DEF(pj_status_t) pjsip_100rel_tx_response(pjsip_inv_session *inv,
 	char rseq_str[32];
 	pj_str_t rseq;
 	tx_data_list_t *tl;
-
+	
 	/* Create UAS state if we don't have one */
 	if (dd->uas_state == NULL) {
 	    dd->uas_state = PJ_POOL_ZALLOC_T(inv->dlg->pool,
@@ -850,54 +850,56 @@ PJ_DEF(pj_status_t) pjsip_100rel_tx_response(pjsip_inv_session *inv,
 	    dd->uas_state->retransmit_timer.user_data = dd;
 	    dd->uas_state->retransmit_timer.cb = &on_retransmit;
 	}
-
+	
 	/* Check that CSeq match */
 	PJ_ASSERT_RETURN(cseq_hdr->cseq == dd->uas_state->cseq,
 			 PJ_EINVALIDOP);
-
+	
 	/* Add Require header */
 	req_hdr = pjsip_require_hdr_create(tdata->pool);
 	req_hdr->count = 1;
 	req_hdr->values[0] = tag_100rel;
 	pjsip_msg_add_hdr(tdata->msg, (pjsip_hdr*)req_hdr);
-
+	
 	/* Add RSeq header */
 	pj_ansi_snprintf(rseq_str, sizeof(rseq_str), "%u",
 			 dd->uas_state->rseq);
 	rseq = pj_str(rseq_str);
-	rseq_hdr = pjsip_generic_string_hdr_create(tdata->pool,
+	rseq_hdr = pjsip_generic_string_hdr_create(tdata->pool, 
 						   &RSEQ, &rseq);
 	pjsip_msg_add_hdr(tdata->msg, (pjsip_hdr*)rseq_hdr);
-
+	
 	/* Create list entry for this response */
 	tl = PJ_POOL_ZALLOC_T(tdata->pool, tx_data_list_t);
 	tl->tdata = tdata;
 	tl->rseq = dd->uas_state->rseq++;
-
+	
 	/* Add to queue if there's pending response, otherwise
 	 * transmit immediately.
 	 */
 	if (!pj_list_empty(&dd->uas_state->tx_data_list)) {
-
+	    
 	    int code = tdata->msg->line.status.code;
-
+	    
 	    /* Will send later */
 	    pj_list_push_back(&dd->uas_state->tx_data_list, tl);
 	    status = PJ_SUCCESS;
-
-	    PJ_LOG(4,(dd->inv->dlg->obj_name,
-		      "Reliable %d response enqueued (%d pending)",
+	    
+	    PJ_LOG(4,(dd->inv->dlg->obj_name, 
+		      "Reliable %d response enqueued (%d pending)", 
 		      code, pj_list_size(&dd->uas_state->tx_data_list)));
-
+	    
 	} else {
 	    pj_list_push_back(&dd->uas_state->tx_data_list, tl);
-
+	    
 	    dd->uas_state->retransmit_count = 0;
 	    on_retransmit(NULL, &dd->uas_state->retransmit_timer);
 	    status = PJ_SUCCESS;
 	}
-
+	
     }
-
+    
     return status;
 }
+
+
