@@ -1,5 +1,5 @@
-/* $Id: sip_reg.c 4173 2012-06-20 10:39:05Z ming $ */
-/*
+/* $Id: sip_reg.c 4319 2013-01-16 10:20:55Z bennylp $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjsip-ua/sip_regc.h>
 #include <pjsip/sip_endpoint.h>
@@ -135,7 +135,7 @@ PJ_DEF(pj_status_t) pjsip_regc_create( pjsip_endpoint *endpt, void *token,
     regc->expires = PJSIP_REGC_EXPIRATION_NOT_SPECIFIED;
     regc->add_xuid_param = pjsip_cfg()->regc.add_xuid_param;
 
-    status = pj_lock_create_recursive_mutex(pool, pool->obj_name,
+    status = pj_lock_create_recursive_mutex(pool, pool->obj_name, 
 					    &regc->lock);
     if (status != PJ_SUCCESS) {
 	pj_pool_release(pool);
@@ -207,7 +207,7 @@ PJ_DEF(pj_status_t) pjsip_regc_get_info( pjsip_regc *regc,
     info->auto_reg = regc->auto_reg;
     info->interval = regc->expires;
     info->transport = regc->last_transport;
-
+    
     if (regc->has_tsx)
 	info->next_reg = 0;
     else if (regc->auto_reg == 0)
@@ -251,14 +251,14 @@ static pj_status_t set_contact( pjsip_regc *regc,
     const pj_str_t CONTACT = { "Contact", 7 };
     pjsip_contact_hdr *h;
     int i;
-
+    
     /* Save existing contact list to removed_contact_hdr_list and
      * clear contact_hdr_list.
      */
-    pj_list_merge_last(&regc->removed_contact_hdr_list,
+    pj_list_merge_last(&regc->removed_contact_hdr_list, 
 		       &regc->contact_hdr_list);
 
-    /* Set the expiration of Contacts in to removed_contact_hdr_list
+    /* Set the expiration of Contacts in to removed_contact_hdr_list 
      * zero.
      */
     h = regc->removed_contact_hdr_list.next;
@@ -276,7 +276,7 @@ static pj_status_t set_contact( pjsip_regc *regc,
 	hdr = (pjsip_contact_hdr*)
               pjsip_parse_hdr(regc->pool, &CONTACT, tmp.ptr, tmp.slen, NULL);
 	if (hdr == NULL) {
-	    PJ_LOG(4,(THIS_FILE, "Invalid Contact: \"%.*s\"",
+	    PJ_LOG(4,(THIS_FILE, "Invalid Contact: \"%.*s\"", 
 		     (int)tmp.slen, tmp.ptr));
 	    return PJSIP_EINVALIDURI;
 	}
@@ -288,7 +288,7 @@ static pj_status_t set_contact( pjsip_regc *regc,
 	while (h != &regc->removed_contact_hdr_list) {
 	    int rc;
 
-	    rc = pjsip_uri_cmp(PJSIP_URI_IN_CONTACT_HDR,
+	    rc = pjsip_uri_cmp(PJSIP_URI_IN_CONTACT_HDR, 
 			       h->uri, hdr->uri);
 	    if (rc == 0) {
 		/* Match */
@@ -300,11 +300,11 @@ static pj_status_t set_contact( pjsip_regc *regc,
 	}
 
 	/* If add_xuid_param option is enabled and Contact URI is sip/sips,
-	 * add xuid parameter to assist matching the Contact URI in the
+	 * add xuid parameter to assist matching the Contact URI in the 
 	 * REGISTER response later.
 	 */
 	if (regc->add_xuid_param && (PJSIP_URI_SCHEME_IS_SIP(hdr->uri) ||
-				     PJSIP_URI_SCHEME_IS_SIPS(hdr->uri)))
+				     PJSIP_URI_SCHEME_IS_SIPS(hdr->uri))) 
 	{
 	    pjsip_param *xuid_param;
 	    pjsip_sip_uri *sip_uri;
@@ -335,8 +335,8 @@ PJ_DEF(pj_status_t) pjsip_regc_init( pjsip_regc *regc,
     pj_str_t tmp;
     pj_status_t status;
 
-    PJ_ASSERT_RETURN(regc && srv_url && from_url && to_url &&
-		     contact_cnt && contact && expires, PJ_EINVAL);
+    PJ_ASSERT_RETURN(regc && srv_url && from_url && to_url && 
+		     expires, PJ_EINVAL);
 
     /* Copy server URL. */
     pj_strdup_with_null(regc->pool, &regc->str_srv_url, srv_url);
@@ -352,10 +352,10 @@ PJ_DEF(pj_status_t) pjsip_regc_init( pjsip_regc *regc,
     pj_strdup_with_null(regc->pool, &regc->from_uri, from_url);
     tmp = regc->from_uri;
     regc->from_hdr = pjsip_from_hdr_create(regc->pool);
-    regc->from_hdr->uri = pjsip_parse_uri(regc->pool, tmp.ptr, tmp.slen,
+    regc->from_hdr->uri = pjsip_parse_uri(regc->pool, tmp.ptr, tmp.slen, 
 					  PJSIP_PARSE_URI_AS_NAMEADDR);
     if (!regc->from_hdr->uri) {
-	PJ_LOG(4,(THIS_FILE, "regc: invalid source URI %.*s",
+	PJ_LOG(4,(THIS_FILE, "regc: invalid source URI %.*s", 
 		  from_url->slen, from_url->ptr));
 	return PJSIP_EINVALIDURI;
     }
@@ -363,7 +363,7 @@ PJ_DEF(pj_status_t) pjsip_regc_init( pjsip_regc *regc,
     /* Set "To" header. */
     pj_strdup_with_null(regc->pool, &tmp, to_url);
     regc->to_hdr = pjsip_to_hdr_create(regc->pool);
-    regc->to_hdr->uri = pjsip_parse_uri(regc->pool, tmp.ptr, tmp.slen,
+    regc->to_hdr->uri = pjsip_parse_uri(regc->pool, tmp.ptr, tmp.slen, 
 					PJSIP_PARSE_URI_AS_NAMEADDR);
     if (!regc->to_hdr->uri) {
 	PJ_LOG(4,(THIS_FILE, "regc: invalid target URI %.*s", to_url->slen, to_url->ptr));
@@ -428,7 +428,7 @@ PJ_DEF(pj_status_t) pjsip_regc_set_route_set( pjsip_regc *regc,
 
 
 /*
- * Bind client registration to a specific transport/listener.
+ * Bind client registration to a specific transport/listener. 
  */
 PJ_DEF(pj_status_t) pjsip_regc_set_transport( pjsip_regc *regc,
 					      const pjsip_tpselector *sel)
@@ -473,7 +473,7 @@ PJ_DEF(pj_status_t) pjsip_regc_add_headers( pjsip_regc *regc,
     return PJ_SUCCESS;
 }
 
-static pj_status_t create_request(pjsip_regc *regc,
+static pj_status_t create_request(pjsip_regc *regc, 
 				  pjsip_tx_data **p_tdata)
 {
     pj_status_t status;
@@ -482,7 +482,7 @@ static pj_status_t create_request(pjsip_regc *regc,
     PJ_ASSERT_RETURN(regc && p_tdata, PJ_EINVAL);
 
     /* Create the request. */
-    status = pjsip_endpt_create_request_from_hdr( regc->endpt,
+    status = pjsip_endpt_create_request_from_hdr( regc->endpt, 
 						  pjsip_get_register_method(),
 						  regc->srv_url,
 						  regc->from_hdr,
@@ -573,7 +573,7 @@ PJ_DEF(pj_status_t) pjsip_regc_register(pjsip_regc *regc, pj_bool_t autoreg,
 			       pjsip_hdr_clone(tdata->pool, hdr));
 	pj_list_erase(hdr);
     }
-
+    
 
     if (regc->expires_hdr)
 	pjsip_msg_add_hdr(msg, (pjsip_hdr*)
@@ -687,7 +687,7 @@ PJ_DEF(pj_status_t) pjsip_regc_unregister_all(pjsip_regc *regc,
     hcontact = pjsip_contact_hdr_create(tdata->pool);
     hcontact->star = 1;
     pjsip_msg_add_hdr(msg, (pjsip_hdr*)hcontact);
-
+    
     /* Add Expires:0 header */
     hdr = (pjsip_hdr*) pjsip_expires_hdr_create(tdata->pool, 0);
     pjsip_msg_add_hdr(msg, hdr);
@@ -728,7 +728,7 @@ PJ_DEF(pj_status_t) pjsip_regc_update_expires(  pjsip_regc *regc,
 }
 
 
-static void call_callback(pjsip_regc *regc, pj_status_t status, int st_code,
+static void call_callback(pjsip_regc *regc, pj_status_t status, int st_code, 
 			  const pj_str_t *reason,
 			  pjsip_rx_data *rdata, pj_int32_t expiration,
 			  int contact_cnt, pjsip_contact_hdr *contact[])
@@ -748,7 +748,7 @@ static void call_callback(pjsip_regc *regc, pj_status_t status, int st_code,
     cbparam.contact_cnt = contact_cnt;
     cbparam.expiration = expiration;
     if (contact_cnt) {
-	pj_memcpy( cbparam.contact, contact,
+	pj_memcpy( cbparam.contact, contact, 
 		   contact_cnt*sizeof(pjsip_contact_hdr*));
     }
 
@@ -761,7 +761,7 @@ static void regc_refresh_timer_cb( pj_timer_heap_t *timer_heap,
     pjsip_regc *regc = (pjsip_regc*) entry->user_data;
     pjsip_tx_data *tdata;
     pj_status_t status;
-
+    
     PJ_UNUSED_ARG(timer_heap);
 
     /* Temporarily increase busy flag to prevent regc from being deleted
@@ -773,8 +773,8 @@ static void regc_refresh_timer_cb( pj_timer_heap_t *timer_heap,
     status = pjsip_regc_register(regc, 1, &tdata);
     if (status == PJ_SUCCESS) {
 	status = pjsip_regc_send(regc, tdata);
-    }
-
+    } 
+    
     if (status != PJ_SUCCESS && regc->cb) {
 	char errmsg[PJ_ERR_MSG_SIZE];
 	pj_str_t reason = pj_strerror(status, errmsg, sizeof(errmsg));
@@ -793,12 +793,12 @@ static void schedule_registration ( pjsip_regc *regc, pj_int32_t expiration )
         pj_time_val delay = { 0, 0};
 
         delay.sec = expiration - regc->delay_before_refresh;
-        if (regc->expires != PJSIP_REGC_EXPIRATION_NOT_SPECIFIED &&
-            delay.sec > (pj_int32_t)regc->expires)
+        if (regc->expires != PJSIP_REGC_EXPIRATION_NOT_SPECIFIED && 
+            delay.sec > (pj_int32_t)regc->expires) 
         {
             delay.sec = regc->expires;
         }
-        if (delay.sec < DELAY_BEFORE_REFRESH)
+        if (delay.sec < DELAY_BEFORE_REFRESH) 
             delay.sec = DELAY_BEFORE_REFRESH;
         regc->timer.cb = &regc_refresh_timer_cb;
         regc->timer.id = REFRESH_TIMER;
@@ -818,8 +818,11 @@ PJ_DEF(pj_status_t) pjsip_regc_set_via_sent_by( pjsip_regc *regc,
 
     if (!via_addr)
         pj_bzero(&regc->via_addr, sizeof(regc->via_addr));
-    else
-        regc->via_addr = *via_addr;
+    else {
+        if (pj_strcmp(&regc->via_addr.host, &via_addr->host))
+            pj_strdup(regc->pool, &regc->via_addr.host, &via_addr->host);
+        regc->via_addr.port = via_addr->port;
+    }
     regc->via_tp = via_tp;
 
     return PJ_SUCCESS;
@@ -865,8 +868,8 @@ static pj_int32_t calculate_response_expiration(const pjsip_regc *regc,
     /* Enumerate all Contact headers in the response */
     *contact_cnt = 0;
     for (hdr=msg->hdr.next; hdr!=&msg->hdr; hdr=hdr->next) {
-	if (hdr->type == PJSIP_H_CONTACT &&
-	    *contact_cnt < max_contact)
+	if (hdr->type == PJSIP_H_CONTACT && 
+	    *contact_cnt < max_contact) 
 	{
 	    contacts[*contact_cnt] = (pjsip_contact_hdr*)hdr;
 	    ++(*contact_cnt);
@@ -890,7 +893,7 @@ static pj_int32_t calculate_response_expiration(const pjsip_regc *regc,
 	    for (i=0; i<*contact_cnt; ++i) {
 		const pjsip_contact_hdr *our_hdr;
 
-		our_hdr = (const pjsip_contact_hdr*)
+		our_hdr = (const pjsip_contact_hdr*) 
 			  regc->contact_hdr_list.next;
 
 		/* Match with our Contact header(s) */
@@ -899,7 +902,7 @@ static pj_int32_t calculate_response_expiration(const pjsip_regc *regc,
 		    const pjsip_uri *uri1, *uri2;
 		    pj_bool_t matched = PJ_FALSE;
 
-		    /* Exclude the display name when comparing the URI
+		    /* Exclude the display name when comparing the URI 
 		     * since server may not return it.
 		     */
 		    uri1 = (const pjsip_uri*)
@@ -920,13 +923,13 @@ static pj_int32_t calculate_response_expiration(const pjsip_regc *regc,
 		     */
 		    if (!matched && regc->add_xuid_param &&
 			(PJSIP_URI_SCHEME_IS_SIP(uri1) ||
-			 PJSIP_URI_SCHEME_IS_SIPS(uri1)) &&
+			 PJSIP_URI_SCHEME_IS_SIPS(uri1)) && 
 			(PJSIP_URI_SCHEME_IS_SIP(uri2) ||
-			 PJSIP_URI_SCHEME_IS_SIPS(uri2)))
+			 PJSIP_URI_SCHEME_IS_SIPS(uri2))) 
 		    {
 			const pjsip_sip_uri *sip_uri1, *sip_uri2;
 			const pjsip_param *p1, *p2;
-
+		
 			sip_uri1 = (const pjsip_sip_uri*)uri1;
 			sip_uri2 = (const pjsip_sip_uri*)uri2;
 
@@ -942,8 +945,8 @@ static pj_int32_t calculate_response_expiration(const pjsip_regc *regc,
 		    if (matched) {
 			has_our_contact = PJ_TRUE;
 
-			if (contacts[i]->expires >= 0 &&
-			    contacts[i]->expires < expiration)
+			if (contacts[i]->expires >= 0 && 
+			    contacts[i]->expires < expiration) 
 			{
 			    /* Get the lowest expiration time. */
 			    expiration = contacts[i]->expires;
@@ -971,7 +974,7 @@ static pj_int32_t calculate_response_expiration(const pjsip_regc *regc,
 		    expiration = regc->expires_hdr->ivalue;
 		} else {
 		    /* We didn't request explicit expiration value,
-		     * and server doesn't specify it either. This
+		     * and server doesn't specify it either. This 
 		     * shouldn't happen unless we have a broken
 		     * registrar.
 		     */
@@ -986,7 +989,7 @@ static pj_int32_t calculate_response_expiration(const pjsip_regc *regc,
 	 * have modified the URI's in the response, which is prohibited).
 	 */
 	if (expiration==NOEXP) {
-	    /* If the number of Contact headers in the response matches
+	    /* If the number of Contact headers in the response matches 
 	     * ours, they're all probably ours. Get the expiration
 	     * from there if this is the case, or from Expires header
 	     * if we don't have exact Contact header count, or
@@ -997,7 +1000,7 @@ static pj_int32_t calculate_response_expiration(const pjsip_regc *regc,
 	    our_contact_cnt = pj_list_size(&regc->contact_hdr_list);
 
 	    if (*contact_cnt == our_contact_cnt && *contact_cnt &&
-		contacts[0]->expires >= 0)
+		contacts[0]->expires >= 0) 
 	    {
 		expiration = contacts[0]->expires;
 	    } else if (expires)
@@ -1057,14 +1060,14 @@ static void regc_tsx_callback(void *token, pjsip_event *event)
 	regc->current_op = REGC_IDLE;
 
 	status = pjsip_auth_clt_reinit_req( &regc->auth_sess,
-					    rdata,
-					    tsx->last_tx,
+					    rdata, 
+					    tsx->last_tx,  
 					    &tdata);
 
 	if (status == PJ_SUCCESS) {
 	    status = pjsip_regc_send(regc, tdata);
 	}
-
+	
 	if (status != PJ_SUCCESS) {
 
 	    /* Only call callback if application is still interested
@@ -1072,10 +1075,10 @@ static void regc_tsx_callback(void *token, pjsip_event *event)
 	     */
 	    if (regc->_delete_flag == 0) {
 		/* Should be safe to release the lock temporarily.
-		 * We do this to avoid deadlock.
+		 * We do this to avoid deadlock. 
 		 */
 		pj_lock_release(regc->lock);
-		call_callback(regc, status, tsx->status_code,
+		call_callback(regc, status, tsx->status_code, 
 			      &rdata->msg_info.msg->line.status.reason,
 			      rdata, -1, 0, NULL);
 		pj_lock_acquire(regc->lock);
@@ -1084,7 +1087,7 @@ static void regc_tsx_callback(void *token, pjsip_event *event)
 
     } else if (regc->_delete_flag) {
 
-	/* User has called pjsip_regc_destroy(), so don't call callback.
+	/* User has called pjsip_regc_destroy(), so don't call callback. 
 	 * This regc will be destroyed later in this function.
 	 */
 
@@ -1187,7 +1190,7 @@ handle_err:
 	    rdata = event->body.tsx_state.src.rdata;
 
 	    /* Calculate expiration */
-	    expiration = calculate_response_expiration(regc, rdata,
+	    expiration = calculate_response_expiration(regc, rdata, 
 						       &contact_cnt,
 						       PJSIP_REGC_MAX_CONTACT,
 						       contact);
@@ -1196,7 +1199,7 @@ handle_err:
             schedule_registration(regc, expiration);
 
 	} else {
-	    rdata = (event->body.tsx_state.type==PJSIP_EVENT_RX_MSG) ?
+	    rdata = (event->body.tsx_state.type==PJSIP_EVENT_RX_MSG) ? 
 			event->body.tsx_state.src.rdata : NULL;
 	}
 
@@ -1209,13 +1212,13 @@ handle_err:
 
 	/* Call callback. */
 	/* Should be safe to release the lock temporarily.
-	 * We do this to avoid deadlock.
+	 * We do this to avoid deadlock. 
 	 */
 	pj_lock_release(regc->lock);
-	call_callback(regc, PJ_SUCCESS, tsx->status_code,
-		      (rdata ? &rdata->msg_info.msg->line.status.reason
+	call_callback(regc, PJ_SUCCESS, tsx->status_code, 
+		      (rdata ? &rdata->msg_info.msg->line.status.reason 
 			: &tsx->status_text),
-		      rdata, expiration,
+		      rdata, expiration, 
 		      contact_cnt, contact);
 	pj_lock_acquire(regc->lock);
     }
@@ -1329,3 +1332,5 @@ PJ_DEF(pj_status_t) pjsip_regc_send(pjsip_regc *regc, pjsip_tx_data *tdata)
 
     return status;
 }
+
+
