@@ -1,5 +1,5 @@
-/* $Id: sip_timer.c 3999 2012-03-30 07:10:13Z bennylp $ */
-/*
+/* $Id: sip_timer.c 4213 2012-07-23 13:31:26Z nanang $ */
+/* 
  * Copyright (C) 2009-2011 Teluu Inc. (http://www.teluu.com)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjsip-ua/sip_timer.h>
 #include <pjsip/print_util.h>
@@ -49,7 +49,7 @@ enum timer_refresher {
 };
 
 /* Structure definition of Session Timers */
-struct pjsip_timer
+struct pjsip_timer 
 {
     pj_bool_t			 active;	/**< Active/inactive flag   */
     pjsip_timer_setting		 setting;	/**< Session Timers setting */
@@ -76,15 +76,15 @@ const pjsip_method pjsip_update_method = { PJSIP_OTHER_METHOD, {"UPDATE", 6}};
 /*
  * Session-Expires header vptr.
  */
-static int se_hdr_print(pjsip_sess_expires_hdr *hdr,
+static int se_hdr_print(pjsip_sess_expires_hdr *hdr, 
 			char *buf, pj_size_t size);
-static pjsip_sess_expires_hdr* se_hdr_clone(pj_pool_t *pool,
+static pjsip_sess_expires_hdr* se_hdr_clone(pj_pool_t *pool, 
 					    const pjsip_sess_expires_hdr *hdr);
-static pjsip_sess_expires_hdr* se_hdr_shallow_clone(
+static pjsip_sess_expires_hdr* se_hdr_shallow_clone( 
 					    pj_pool_t *pool,
 					    const pjsip_sess_expires_hdr* hdr);
 
-static pjsip_hdr_vptr se_hdr_vptr =
+static pjsip_hdr_vptr se_hdr_vptr = 
 {
     (pjsip_hdr_clone_fptr) &se_hdr_clone,
     (pjsip_hdr_clone_fptr) &se_hdr_shallow_clone,
@@ -94,15 +94,15 @@ static pjsip_hdr_vptr se_hdr_vptr =
 /*
  * Min-SE header vptr.
  */
-static int min_se_hdr_print(pjsip_min_se_hdr *hdr,
+static int min_se_hdr_print(pjsip_min_se_hdr *hdr, 
 			    char *buf, pj_size_t size);
-static pjsip_min_se_hdr* min_se_hdr_clone(pj_pool_t *pool,
+static pjsip_min_se_hdr* min_se_hdr_clone(pj_pool_t *pool, 
 					  const pjsip_min_se_hdr *hdr);
-static pjsip_min_se_hdr* min_se_hdr_shallow_clone(
+static pjsip_min_se_hdr* min_se_hdr_shallow_clone( 
 					  pj_pool_t *pool,
 					  const pjsip_min_se_hdr* hdr);
 
-static pjsip_hdr_vptr min_se_hdr_vptr =
+static pjsip_hdr_vptr min_se_hdr_vptr = 
 {
     (pjsip_hdr_clone_fptr) &min_se_hdr_clone,
     (pjsip_hdr_clone_fptr) &min_se_hdr_shallow_clone,
@@ -112,7 +112,7 @@ static pjsip_hdr_vptr min_se_hdr_vptr =
 /*
  * Session-Expires header vptr.
  */
-static int se_hdr_print(pjsip_sess_expires_hdr *hdr,
+static int se_hdr_print(pjsip_sess_expires_hdr *hdr, 
 			char *buf, pj_size_t size)
 {
     char *p = buf;
@@ -146,7 +146,7 @@ static int se_hdr_print(pjsip_sess_expires_hdr *hdr,
 
     /* Print generic params */
     printed = pjsip_param_print_on(&hdr->other_param, p, endbuf-p,
-				   &pc->pjsip_TOKEN_SPEC,
+				   &pc->pjsip_TOKEN_SPEC, 
 				   &pc->pjsip_TOKEN_SPEC, ';');
     if (printed < 0)
 	return printed;
@@ -155,7 +155,7 @@ static int se_hdr_print(pjsip_sess_expires_hdr *hdr,
     return p - buf;
 }
 
-static pjsip_sess_expires_hdr* se_hdr_clone(pj_pool_t *pool,
+static pjsip_sess_expires_hdr* se_hdr_clone(pj_pool_t *pool, 
 					    const pjsip_sess_expires_hdr *hsrc)
 {
     pjsip_sess_expires_hdr *hdr = pjsip_sess_expires_hdr_create(pool);
@@ -165,7 +165,7 @@ static pjsip_sess_expires_hdr* se_hdr_clone(pj_pool_t *pool,
     return hdr;
 }
 
-static pjsip_sess_expires_hdr* se_hdr_shallow_clone(
+static pjsip_sess_expires_hdr* se_hdr_shallow_clone( 
 					    pj_pool_t *pool,
 					    const pjsip_sess_expires_hdr* hsrc)
 {
@@ -178,7 +178,7 @@ static pjsip_sess_expires_hdr* se_hdr_shallow_clone(
 /*
  * Min-SE header vptr.
  */
-static int min_se_hdr_print(pjsip_min_se_hdr *hdr,
+static int min_se_hdr_print(pjsip_min_se_hdr *hdr, 
 			    char *buf, pj_size_t size)
 {
     char *p = buf;
@@ -199,7 +199,7 @@ static int min_se_hdr_print(pjsip_min_se_hdr *hdr,
 
     /* Print generic params */
     printed = pjsip_param_print_on(&hdr->other_param, p, endbuf-p,
-				   &pc->pjsip_TOKEN_SPEC,
+				   &pc->pjsip_TOKEN_SPEC, 
 				   &pc->pjsip_TOKEN_SPEC, ';');
     if (printed < 0)
 	return printed;
@@ -208,7 +208,7 @@ static int min_se_hdr_print(pjsip_min_se_hdr *hdr,
     return p - buf;
 }
 
-static pjsip_min_se_hdr* min_se_hdr_clone(pj_pool_t *pool,
+static pjsip_min_se_hdr* min_se_hdr_clone(pj_pool_t *pool, 
 					  const pjsip_min_se_hdr *hsrc)
 {
     pjsip_min_se_hdr *hdr = pjsip_min_se_hdr_create(pool);
@@ -217,7 +217,7 @@ static pjsip_min_se_hdr* min_se_hdr_clone(pj_pool_t *pool,
     return hdr;
 }
 
-static pjsip_min_se_hdr* min_se_hdr_shallow_clone(
+static pjsip_min_se_hdr* min_se_hdr_shallow_clone( 
 					  pj_pool_t *pool,
 					  const pjsip_min_se_hdr* hsrc)
 {
@@ -319,7 +319,7 @@ static void add_timer_headers(pjsip_inv_session *inv, pjsip_tx_data *tdata,
 }
 
 /* Timer callback. When the timer is fired, it can be time to refresh
- * the session if UA is the refresher, otherwise it is time to end
+ * the session if UA is the refresher, otherwise it is time to end 
  * the session.
  */
 static void timer_cb(pj_timer_heap_t *timer_heap, struct pj_timer_entry *entry)
@@ -385,17 +385,17 @@ static void timer_cb(pj_timer_heap_t *timer_heap, struct pj_timer_entry *entry)
 	    pjsip_msg_body *body;
 	    const pjmedia_sdp_session *offer = NULL;
 
-	    pj_assert(pjmedia_sdp_neg_get_state(inv->neg) ==
+	    pj_assert(pjmedia_sdp_neg_get_state(inv->neg) == 
 		      PJMEDIA_SDP_NEG_STATE_DONE);
 
 	    status = pjsip_inv_invite(inv, &tdata);
 	    if (status == PJ_SUCCESS)
-		status = pjmedia_sdp_neg_send_local_offer(inv->pool_prov,
+		status = pjmedia_sdp_neg_send_local_offer(inv->pool_prov, 
 							  inv->neg, &offer);
 	    if (status == PJ_SUCCESS)
 		status = pjmedia_sdp_neg_get_neg_local(inv->neg, &offer);
 	    if (status == PJ_SUCCESS) {
-		status = pjsip_create_sdp_body(tdata->pool,
+		status = pjsip_create_sdp_body(tdata->pool, 
 					(pjmedia_sdp_session*)offer, &body);
 		tdata->msg->body = body;
 	    }
@@ -407,15 +407,15 @@ static void timer_cb(pj_timer_heap_t *timer_heap, struct pj_timer_entry *entry)
 		   (now.sec-inv->timer->last_refresh.sec),
 		   inv->timer->setting.sess_expires));
     } else {
-
+	
 	pj_time_val now;
 
 	/* Refreshee, terminate the session */
-	status = pjsip_inv_end_session(inv, PJSIP_SC_REQUEST_TIMEOUT,
+	status = pjsip_inv_end_session(inv, PJSIP_SC_REQUEST_TIMEOUT, 
 				       NULL, &tdata);
 
 	pj_gettimeofday(&now);
-	PJ_LOG(3, (inv->pool->obj_name,
+	PJ_LOG(3, (inv->pool->obj_name, 
 		   "No session refresh received after %ds "
 		   "(expiration period=%ds), stopping session now!",
 		   (now.sec-inv->timer->last_refresh.sec),
@@ -461,7 +461,7 @@ static void start_timer(pjsip_inv_session *inv)
 			1,		    /* id */
 			inv,		    /* user data */
 			timer_cb);	    /* callback */
-
+    
     /* Set delay based on role, refresher or refreshee */
     if ((timer->refresher == TR_UAC && inv->timer->role == PJSIP_ROLE_UAC) ||
 	(timer->refresher == TR_UAS && inv->timer->role == PJSIP_ROLE_UAS))
@@ -473,7 +473,7 @@ static void start_timer(pjsip_inv_session *inv)
 	 * is the minimum of 32 seconds and one third of the session interval
 	 * before session expiration.
 	 */
-	delay.sec = timer->setting.sess_expires -
+	delay.sec = timer->setting.sess_expires - 
 		    timer->setting.sess_expires/3;
 	delay.sec = PJ_MAX((long)timer->setting.sess_expires-32, delay.sec);
     }
@@ -503,7 +503,7 @@ static void pjsip_timer_deinit_module(pjsip_endpoint *endpt)
 }
 
 /*
- * Initialize Session Timers support in PJSIP.
+ * Initialize Session Timers support in PJSIP. 
  */
 PJ_DEF(pj_status_t) pjsip_timer_init_module(pjsip_endpoint *endpt)
 {
@@ -515,13 +515,13 @@ PJ_DEF(pj_status_t) pjsip_timer_init_module(pjsip_endpoint *endpt)
 	return PJ_SUCCESS;
 
     /* Register Session-Expires header parser */
-    status = pjsip_register_hdr_parser( STR_SE.ptr, STR_SHORT_SE.ptr,
+    status = pjsip_register_hdr_parser( STR_SE.ptr, STR_SHORT_SE.ptr, 
 				        &parse_hdr_se);
     if (status != PJ_SUCCESS)
 	return status;
 
     /* Register Min-SE header parser */
-    status = pjsip_register_hdr_parser( STR_MIN_SE.ptr, NULL,
+    status = pjsip_register_hdr_parser( STR_MIN_SE.ptr, NULL, 
 				        &parse_hdr_min_se);
     if (status != PJ_SUCCESS)
 	return status;
@@ -534,8 +534,8 @@ PJ_DEF(pj_status_t) pjsip_timer_init_module(pjsip_endpoint *endpt)
 
     /* Register deinit module to be executed when PJLIB shutdown */
     if (pjsip_endpt_atexit(endpt, &pjsip_timer_deinit_module) != PJ_SUCCESS) {
-	/* Failure to register this function may cause this module won't
-	 * work properly when the stack is restarted (without quitting
+	/* Failure to register this function may cause this module won't 
+	 * work properly when the stack is restarted (without quitting 
 	 * application).
 	 */
 	pj_assert(!"Failed to register Session Timer deinit.");
@@ -562,7 +562,7 @@ PJ_DEF(pj_status_t) pjsip_timer_setting_default(pjsip_timer_setting *setting)
 }
 
 /*
- * Initialize Session Timers in an INVITE session.
+ * Initialize Session Timers in an INVITE session. 
  */
 PJ_DEF(pj_status_t) pjsip_timer_init_session(
 					pjsip_inv_session *inv,
@@ -636,7 +636,7 @@ PJ_DEF(pjsip_min_se_hdr*) pjsip_min_se_hdr_create(pj_pool_t *pool)
 }
 
 
-/*
+/* 
  * This function generates headers for Session Timers for intial and
  * refresh INVITE or UPDATE.
  */
@@ -655,14 +655,14 @@ PJ_DEF(pj_status_t) pjsip_timer_update_req(pjsip_inv_session *inv,
     if (inv->timer == NULL)
 	pjsip_timer_init_session(inv, NULL);
 
-    /* If refresher role (i.e: ours or peer) has been set/negotiated,
+    /* If refresher role (i.e: ours or peer) has been set/negotiated, 
      * better to keep it.
      */
     if (inv->timer->refresher != TR_UNKNOWN) {
 	pj_bool_t as_refresher;
 
 	/* Check our refresher role */
-	as_refresher =
+	as_refresher = 
 	    (inv->timer->refresher==TR_UAC && inv->timer->role==PJSIP_ROLE_UAC) ||
 	    (inv->timer->refresher==TR_UAS && inv->timer->role==PJSIP_ROLE_UAS);
 
@@ -679,8 +679,8 @@ PJ_DEF(pj_status_t) pjsip_timer_update_req(pjsip_inv_session *inv,
     return PJ_SUCCESS;
 }
 
-/*
- * This function will handle Session Timers part of INVITE/UPDATE
+/* 
+ * This function will handle Session Timers part of INVITE/UPDATE 
  * responses with code:
  * - 422 (Session Interval Too Small)
  * - 2xx final response
@@ -720,7 +720,7 @@ PJ_DEF(pj_status_t) pjsip_timer_process_resp(pjsip_inv_session *inv,
 	pjsip_via_hdr *via;
 
 	/* Get Min-SE value from response */
-	min_se_hdr = (pjsip_min_se_hdr*)
+	min_se_hdr = (pjsip_min_se_hdr*) 
 		     pjsip_msg_find_hdr_by_name(msg, &STR_MIN_SE, NULL);
 	if (min_se_hdr == NULL) {
 	    /* Response 422 should contain Min-SE header */
@@ -731,7 +731,7 @@ PJ_DEF(pj_status_t) pjsip_timer_process_resp(pjsip_inv_session *inv,
 	pj_assert(inv->timer);
 
 	/* Update Min-SE */
-	inv->timer->setting.min_se = PJ_MAX(min_se_hdr->min_se,
+	inv->timer->setting.min_se = PJ_MAX(min_se_hdr->min_se, 
 					    inv->timer->setting.min_se);
 
 	/* Update Session Timers setting */
@@ -759,7 +759,7 @@ PJ_DEF(pj_status_t) pjsip_timer_process_resp(pjsip_inv_session *inv,
 	pjsip_tx_data_add_ref(tdata);
 
 	/* Update Session Timers headers */
-	hdr = (pjsip_hdr*) pjsip_msg_find_hdr_by_name(tdata->msg,
+	hdr = (pjsip_hdr*) pjsip_msg_find_hdr_by_name(tdata->msg, 
 						      &STR_MIN_SE, NULL);
 	if (hdr != NULL) pj_list_erase(hdr);
 
@@ -781,10 +781,10 @@ PJ_DEF(pj_status_t) pjsip_timer_process_resp(pjsip_inv_session *inv,
 
 	/* Find Session-Expires header */
 	se_hdr = (pjsip_sess_expires_hdr*) pjsip_msg_find_hdr_by_names(
-						msg, &STR_SE,
+						msg, &STR_SE, 
 						&STR_SHORT_SE, NULL);
 	if (se_hdr == NULL) {
-	    /* Remote doesn't support/want Session Timers, check if local
+	    /* Remote doesn't support/want Session Timers, check if local 
 	     * require or force to use Session Timers.
 	     */
 	    if (inv->options & PJSIP_INV_REQUIRE_TIMER) {
@@ -801,7 +801,7 @@ PJ_DEF(pj_status_t) pjsip_timer_process_resp(pjsip_inv_session *inv,
 		return PJ_SUCCESS;
 	    }
 	}
-
+	    
 	/* Make sure Session Timers is initialized */
 	if (inv->timer == NULL)
 	    pjsip_timer_init_session(inv, NULL);
@@ -809,7 +809,7 @@ PJ_DEF(pj_status_t) pjsip_timer_process_resp(pjsip_inv_session *inv,
 	/* Session expiration period specified by remote is lower than our
 	 * Min-SE.
 	 */
-	if (se_hdr &&
+	if (se_hdr && 
 	    se_hdr->sess_expires < inv->timer->setting.min_se)
 	{
 	    /* See ticket #954, instead of returning non-PJ_SUCCESS (which
@@ -817,7 +817,7 @@ PJ_DEF(pj_status_t) pjsip_timer_process_resp(pjsip_inv_session *inv,
 	     * SE and update our local SE, as long as it isn't less than 90s.
 	     */
 	    if (se_hdr->sess_expires >= ABS_MIN_SE) {
-		PJ_LOG(3, (inv->pool->obj_name,
+		PJ_LOG(3, (inv->pool->obj_name, 
 			   "Peer responds with bad Session-Expires, %ds, "
 			   "which is less than Min-SE specified in request, "
 			   "%ds. Well, let's just accept and use it.",
@@ -835,10 +835,10 @@ PJ_DEF(pj_status_t) pjsip_timer_process_resp(pjsip_inv_session *inv,
 	}
 
 	/* Update SE. Session-Expires in response cannot be lower than Min-SE.
-	 * Session-Expires in response can only be equal or lower than in
+	 * Session-Expires in response can only be equal or lower than in 
 	 * request.
 	 */
-	if (se_hdr &&
+	if (se_hdr && 
 	    se_hdr->sess_expires <= inv->timer->setting.sess_expires &&
 	    se_hdr->sess_expires >= inv->timer->setting.min_se)
 	{
@@ -922,12 +922,12 @@ PJ_DEF(pj_status_t) pjsip_timer_process_req(pjsip_inv_session *inv,
     se_hdr = (pjsip_sess_expires_hdr*) pjsip_msg_find_hdr_by_names(
 					    msg, &STR_SE, &STR_SHORT_SE, NULL);
     if (se_hdr == NULL) {
-	/* Remote doesn't support/want Session Timers, check if local
-	 * require or force to use Session Timers. Note that Supported and
-	 * Require headers negotiation should have been verified by invite
+	/* Remote doesn't support/want Session Timers, check if local 
+	 * require or force to use Session Timers. Note that Supported and 
+	 * Require headers negotiation should have been verified by invite 
 	 * session.
 	 */
-	if ((inv->options &
+	if ((inv->options & 
 	    (PJSIP_INV_REQUIRE_TIMER | PJSIP_INV_ALWAYS_USE_TIMER)) == 0)
 	{
 	    /* Session Timers not forced/required */
@@ -941,14 +941,14 @@ PJ_DEF(pj_status_t) pjsip_timer_process_req(pjsip_inv_session *inv,
 	pjsip_timer_init_session(inv, NULL);
 
     /* Find Min-SE header */
-    min_se_hdr = (pjsip_min_se_hdr*) pjsip_msg_find_hdr_by_name(msg,
+    min_se_hdr = (pjsip_min_se_hdr*) pjsip_msg_find_hdr_by_name(msg, 
 							    &STR_MIN_SE, NULL);
     /* Update Min-SE */
     min_se = inv->timer->setting.min_se;
     if (min_se_hdr)
 	min_se = PJ_MAX(min_se_hdr->min_se, min_se);
 
-    /* Validate SE. Session-Expires cannot be lower than Min-SE
+    /* Validate SE. Session-Expires cannot be lower than Min-SE 
      * (or 90 seconds if Min-SE is not set).
      */
     if (se_hdr && se_hdr->sess_expires < min_se) {
@@ -966,7 +966,7 @@ PJ_DEF(pj_status_t) pjsip_timer_process_req(pjsip_inv_session *inv,
 	inv->timer->setting.sess_expires = se_hdr->sess_expires;
     } else if (inv->timer->setting.sess_expires < min_se) {
 	/* There is no SE in the request (remote support Session Timers but
-	 * doesn't want to use it, it just specify Min-SE) and local SE is
+	 * doesn't want to use it, it just specify Min-SE) and local SE is 
 	 * lower than Min-SE specified by remote.
 	 */
 	inv->timer->setting.sess_expires = min_se;
@@ -978,21 +978,21 @@ PJ_DEF(pj_status_t) pjsip_timer_process_req(pjsip_inv_session *inv,
     else if (se_hdr && pj_stricmp(&se_hdr->refresher, &STR_UAS) == 0)
 	inv->timer->refresher = TR_UAS;
     else {
-	/* If refresher role (i.e: ours or peer) has been set/negotiated,
+	/* If refresher role (i.e: ours or peer) has been set/negotiated, 
 	 * better to keep it.
 	 */
 	if (inv->timer->refresher != TR_UNKNOWN) {
 	    pj_bool_t as_refresher;
 
 	    /* Check our refresher role */
-	    as_refresher =
+	    as_refresher = 
 		(inv->timer->refresher==TR_UAC && inv->timer->role==PJSIP_ROLE_UAC) ||
 		(inv->timer->refresher==TR_UAS && inv->timer->role==PJSIP_ROLE_UAS);
 
 	    /* Update refresher role */
 	    inv->timer->refresher = as_refresher? TR_UAS : TR_UAC;
 	} else {
-	    /* If UAC support timer (currently check the existance of
+	    /* If UAC support timer (currently check the existance of 
 	     * Session-Expires header in the request), set UAC as refresher.
 	     */
 	    inv->timer->refresher = se_hdr? TR_UAC : TR_UAS;
@@ -1030,9 +1030,36 @@ PJ_DEF(pj_status_t) pjsip_timer_update_resp(pjsip_inv_session *inv,
 	if (inv->timer && inv->timer->active) {
 	    /* Add Session-Expires header and start the timer */
 	    add_timer_headers(inv, tdata, PJ_TRUE, PJ_FALSE);
+
+	    /* Add 'timer' to Require header (see ticket #1560). */
+	    if (inv->timer->refresher == TR_UAC) {
+		pjsip_require_hdr *req_hdr;
+		pj_bool_t req_hdr_has_timer = PJ_FALSE;
+
+		req_hdr = (pjsip_require_hdr*)
+			   pjsip_msg_find_hdr(tdata->msg, PJSIP_H_REQUIRE,
+					      NULL);
+		if (req_hdr == NULL) {
+		    req_hdr = pjsip_require_hdr_create(tdata->pool);
+		    PJ_ASSERT_RETURN(req_hdr, PJ_ENOMEM);
+		    pjsip_msg_add_hdr(tdata->msg, (pjsip_hdr*)req_hdr);
+		} else {
+		    unsigned i;
+		    for (i = 0; i < req_hdr->count; ++i) {
+			if (pj_stricmp(&req_hdr->values[i], &STR_TIMER)) {
+			    req_hdr_has_timer = PJ_TRUE;
+			    break;
+			}
+		    }
+		}
+		if (!req_hdr_has_timer)
+		    req_hdr->values[req_hdr->count++] = STR_TIMER;
+	    }
+	    
+	    /* Finally, start timer. */
 	    start_timer(inv);
 	}
-    }
+    } 
     else if (msg->line.status.code == PJSIP_SC_SESSION_TIMER_TOO_SMALL)
     {
 	/* Add Min-SE header */
