@@ -1,5 +1,5 @@
-/* $Id: sip_ua_layer.c 3664 2011-07-19 03:42:28Z nanang $ */
-/*
+/* $Id: sip_ua_layer.c 4208 2012-07-18 07:52:33Z ming $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include <pjsip/sip_ua_layer.h>
 #include <pjsip/sip_module.h>
@@ -83,7 +83,7 @@ static struct user_agent
     pjsip_ua_init_param  param;
     struct dlg_set	 free_dlgset_nodes;
 
-} mod_ua =
+} mod_ua = 
 {
   {
     NULL, NULL,		    /* prev, next.			*/
@@ -102,7 +102,7 @@ static struct user_agent
   }
 };
 
-/*
+/* 
  * mod_ua_load()
  *
  * Called when module is being loaded by endpoint.
@@ -167,7 +167,7 @@ static void mod_ua_on_tsx_state( pjsip_transaction *tsx, pjsip_event *e)
 
     /* Get the dialog where this transaction belongs. */
     dlg = (pjsip_dialog*) tsx->mod_data[mod_ua.mod.id];
-
+    
     /* If dialog instance has gone, it could mean that the dialog
      * may has been destroyed.
      */
@@ -251,7 +251,7 @@ PJ_DEF(void) pjsip_ua_create_dlg_set_key( pj_pool_t *pool,
 
     pj_memcpy(set_key->ptr, call_id->ptr, call_id->slen);
     set_key->ptr[call_id->slen] = '$';
-    pj_memcpy(set_key->ptr + call_id->slen + 1,
+    pj_memcpy(set_key->ptr + call_id->slen + 1, 
 	      local_tag->ptr, local_tag->slen);
 }
 */
@@ -302,12 +302,13 @@ PJ_DEF(pj_status_t) pjsip_ua_register_dlg( pjsip_user_agent *ua,
 	struct dlg_set *dlg_set;
 
 	dlg_set = (struct dlg_set*)
-		  pj_hash_get( mod_ua.dlg_table, dlg->local.info->tag.ptr,
-			       dlg->local.info->tag.slen,
-			       &dlg->local.tag_hval);
+		  pj_hash_get_lower( mod_ua.dlg_table,
+                                     dlg->local.info->tag.ptr, 
+			             dlg->local.info->tag.slen,
+			             &dlg->local.tag_hval);
 
 	if (dlg_set) {
-	    /* This is NOT the first dialog in the dialog set.
+	    /* This is NOT the first dialog in the dialog set. 
 	     * Just add this dialog in the list.
 	     */
 	    pj_assert(dlg_set->dlg_list.next != (void*)&dlg_set->dlg_list);
@@ -316,7 +317,7 @@ PJ_DEF(pj_status_t) pjsip_ua_register_dlg( pjsip_user_agent *ua,
 	    dlg->dlg_set = dlg_set;
 
 	} else {
-	    /* This is the first dialog in the dialog set.
+	    /* This is the first dialog in the dialog set. 
 	     * Create the dialog set and add this dialog to it.
 	     */
 	    dlg_set = alloc_dlgset_node();
@@ -326,9 +327,11 @@ PJ_DEF(pj_status_t) pjsip_ua_register_dlg( pjsip_user_agent *ua,
 	    dlg->dlg_set = dlg_set;
 
 	    /* Register the dialog set in the hash table. */
-	    pj_hash_set_np(mod_ua.dlg_table,
-			   dlg->local.info->tag.ptr, dlg->local.info->tag.slen,
-			   dlg->local.tag_hval, dlg_set->ht_entry, dlg_set);
+	    pj_hash_set_np_lower(mod_ua.dlg_table, 
+			         dlg->local.info->tag.ptr,
+                                 dlg->local.info->tag.slen,
+			         dlg->local.tag_hval, dlg_set->ht_entry,
+                                 dlg_set);
 	}
 
     } else {
@@ -341,9 +344,10 @@ PJ_DEF(pj_status_t) pjsip_ua_register_dlg( pjsip_user_agent *ua,
 
 	dlg->dlg_set = dlg_set;
 
-	pj_hash_set_np(mod_ua.dlg_table,
-		       dlg->local.info->tag.ptr, dlg->local.info->tag.slen,
-		       dlg->local.tag_hval, dlg_set->ht_entry, dlg_set);
+	pj_hash_set_np_lower(mod_ua.dlg_table, 
+		             dlg->local.info->tag.ptr,
+                             dlg->local.info->tag.slen,
+		             dlg->local.tag_hval, dlg_set->ht_entry, dlg_set);
     }
 
     /* Unlock user agent. */
@@ -387,8 +391,9 @@ PJ_DEF(pj_status_t) pjsip_ua_unregister_dlg( pjsip_user_agent *ua,
 
     /* If dialog list is empty, remove the dialog set from the hash table. */
     if (pj_list_empty(&dlg_set->dlg_list)) {
-	pj_hash_set(NULL, mod_ua.dlg_table, dlg->local.info->tag.ptr,
-		    dlg->local.info->tag.slen, dlg->local.tag_hval, NULL);
+	pj_hash_set_lower(NULL, mod_ua.dlg_table, dlg->local.info->tag.ptr,
+		          dlg->local.info->tag.slen, dlg->local.tag_hval,
+                          NULL);
 
 	/* Return dlg_set to free nodes. */
 	pj_list_push_back(&mod_ua.free_dlgset_nodes, dlg_set);
@@ -415,7 +420,7 @@ PJ_DEF(pjsip_dialog*) pjsip_tsx_get_dlg( pjsip_transaction *tsx )
 
 /*
  * Retrieve the current number of dialog-set currently registered
- * in the hash table.
+ * in the hash table. 
  */
 PJ_DEF(unsigned) pjsip_ua_get_dlg_set_count(void)
 {
@@ -431,7 +436,7 @@ PJ_DEF(unsigned) pjsip_ua_get_dlg_set_count(void)
 }
 
 
-/*
+/* 
  * Find a dialog.
  */
 PJ_DEF(pjsip_dialog*) pjsip_ua_find_dialog(const pj_str_t *call_id,
@@ -449,8 +454,8 @@ PJ_DEF(pjsip_dialog*) pjsip_ua_find_dialog(const pj_str_t *call_id,
 
     /* Lookup the dialog set. */
     dlg_set = (struct dlg_set*)
-	      pj_hash_get(mod_ua.dlg_table, local_tag->ptr, local_tag->slen,
-			  NULL);
+    	      pj_hash_get_lower(mod_ua.dlg_table, local_tag->ptr,
+                                local_tag->slen, NULL);
     if (dlg_set == NULL) {
 	/* Not found */
 	pj_mutex_unlock(mod_ua.mutex);
@@ -461,8 +466,8 @@ PJ_DEF(pjsip_dialog*) pjsip_ua_find_dialog(const pj_str_t *call_id,
      * remote tag.
      */
     dlg = dlg_set->dlg_list.next;
-    while (dlg != (pjsip_dialog*)&dlg_set->dlg_list) {
-	if (pj_strcmp(&dlg->remote.info->tag, remote_tag) == 0)
+    while (dlg != (pjsip_dialog*)&dlg_set->dlg_list) {	
+	if (pj_stricmp(&dlg->remote.info->tag, remote_tag) == 0)
 	    break;
 	dlg = dlg->next;
     }
@@ -474,7 +479,7 @@ PJ_DEF(pjsip_dialog*) pjsip_ua_find_dialog(const pj_str_t *call_id,
     }
 
     /* Dialog has been found. It SHOULD have the right Call-ID!! */
-    PJ_ASSERT_ON_FAIL(pj_strcmp(&dlg->call_id->id, call_id)==0,
+    PJ_ASSERT_ON_FAIL(pj_strcmp(&dlg->call_id->id, call_id)==0, 
 			{pj_mutex_unlock(mod_ua.mutex); return NULL;});
 
     if (lock_dialog) {
@@ -531,7 +536,7 @@ static struct dlg_set *find_dlg_set_for_msg( pjsip_rx_data *rdata )
 	else
 	    role = PJSIP_ROLE_UAC;
 
-	pjsip_tsx_create_key(rdata->tp_info.pool, &key, role,
+	pjsip_tsx_create_key(rdata->tp_info.pool, &key, role, 
 			     pjsip_get_invite_method(), rdata);
 
 	/* Lookup the INVITE transaction */
@@ -563,7 +568,8 @@ static struct dlg_set *find_dlg_set_for_msg( pjsip_rx_data *rdata )
 
 	/* Lookup the dialog set. */
 	dlg_set = (struct dlg_set*)
-		  pj_hash_get(mod_ua.dlg_table, tag->ptr, tag->slen, NULL);
+		  pj_hash_get_lower(mod_ua.dlg_table, tag->ptr, tag->slen,
+                                    NULL);
 	return dlg_set;
     }
 }
@@ -577,9 +583,9 @@ static pj_bool_t mod_ua_on_rx_request(pjsip_rx_data *rdata)
     pj_status_t status;
 
     /* Optimized path: bail out early if request is not CANCEL and it doesn't
-     * have To tag
+     * have To tag 
      */
-    if (rdata->msg_info.to->tag.slen == 0 &&
+    if (rdata->msg_info.to->tag.slen == 0 && 
 	rdata->msg_info.msg->line.req.method.id != PJSIP_CANCEL_METHOD)
     {
 	return PJ_FALSE;
@@ -605,26 +611,26 @@ retry_on_deadlock:
 	pj_mutex_unlock(mod_ua.mutex);
 
 	if (rdata->msg_info.msg->line.req.method.id != PJSIP_ACK_METHOD) {
-	    PJ_LOG(5,(THIS_FILE,
+	    PJ_LOG(5,(THIS_FILE, 
 		      "Unable to find dialogset for %s, answering with 481",
 		      pjsip_rx_data_get_info(rdata)));
 
 	    /* Respond with 481 . */
-	    pjsip_endpt_respond_stateless( mod_ua.endpt, rdata, 481, NULL,
+	    pjsip_endpt_respond_stateless( mod_ua.endpt, rdata, 481, NULL, 
 					   NULL, NULL );
 	}
 	return PJ_TRUE;
     }
 
     /* Dialog set has been found.
-     * Find the dialog in the dialog set based on the content of the remote
+     * Find the dialog in the dialog set based on the content of the remote 
      * tag.
      */
     from_tag = &rdata->msg_info.from->tag;
     dlg = dlg_set->dlg_list.next;
     while (dlg != (pjsip_dialog*)&dlg_set->dlg_list) {
-
-	if (pj_strcmp(&dlg->remote.info->tag, from_tag) == 0)
+	
+	if (pj_stricmp(&dlg->remote.info->tag, from_tag) == 0)
 	    break;
 
 	dlg = dlg->next;
@@ -634,7 +640,7 @@ retry_on_deadlock:
      *	- UAC sends SUBSCRIBE, then UAS sends NOTIFY before answering
      *    SUBSCRIBE request with 2xx.
      *
-     * In this case, we can accept the request ONLY when the original
+     * In this case, we can accept the request ONLY when the original 
      * dialog still has empty To tag.
      */
     if (dlg == (pjsip_dialog*)&dlg_set->dlg_list) {
@@ -646,15 +652,15 @@ retry_on_deadlock:
 	    pj_mutex_unlock(mod_ua.mutex);
 
 	    if (rdata->msg_info.msg->line.req.method.id != PJSIP_ACK_METHOD) {
-		PJ_LOG(5,(THIS_FILE,
+		PJ_LOG(5,(THIS_FILE, 
 		          "Unable to find dialog for %s, answering with 481",
 		          pjsip_rx_data_get_info(rdata)));
 
 		pjsip_endpt_respond_stateless(mod_ua.endpt, rdata,
-					      PJSIP_SC_CALL_TSX_DOES_NOT_EXIST,
+					      PJSIP_SC_CALL_TSX_DOES_NOT_EXIST, 
 					      NULL, NULL, NULL);
 	    } else {
-		PJ_LOG(5,(THIS_FILE,
+		PJ_LOG(5,(THIS_FILE, 
 		          "Unable to find dialog for %s",
 		          pjsip_rx_data_get_info(rdata)));
 	    }
@@ -671,8 +677,8 @@ retry_on_deadlock:
     PJ_LOG(6,(dlg->obj_name, "UA layer acquiring dialog lock for request"));
     status = pjsip_dlg_try_inc_lock(dlg);
     if (status != PJ_SUCCESS) {
-	/* Failed to acquire dialog mutex immediately, this could be
-	 * because of deadlock. Release UA mutex, yield, and retry
+	/* Failed to acquire dialog mutex immediately, this could be 
+	 * because of deadlock. Release UA mutex, yield, and retry 
 	 * the whole thing once again.
 	 */
 	pj_mutex_unlock(mod_ua.mutex);
@@ -734,7 +740,7 @@ retry_on_deadlock:
 	/* Get the dialog set. */
 	dlg_set = (struct dlg_set*) dlg->dlg_set;
 
-	/* Even if transaction is found and (candidate) dialog has been
+	/* Even if transaction is found and (candidate) dialog has been 
 	 * identified, it's possible that the request has forked.
 	 */
 
@@ -761,17 +767,17 @@ retry_on_deadlock:
 
 	/* Get the dialog set. */
 	dlg_set = (struct dlg_set*)
-		  pj_hash_get(mod_ua.dlg_table,
-			      rdata->msg_info.from->tag.ptr,
-			      rdata->msg_info.from->tag.slen,
-			      NULL);
+		  pj_hash_get_lower(mod_ua.dlg_table, 
+			            rdata->msg_info.from->tag.ptr,
+			            rdata->msg_info.from->tag.slen,
+			            NULL);
 
 	if (!dlg_set) {
 	    /* Unlock dialog hash table. */
 	    pj_mutex_unlock(mod_ua.mutex);
 
 	    /* Strayed 2xx response!! */
-	    PJ_LOG(4,(THIS_FILE,
+	    PJ_LOG(4,(THIS_FILE, 
 		      "Received strayed 2xx response (no dialog is found)"
 		      " from %s:%d: %s",
 		      rdata->pkt_info.src_name, rdata->pkt_info.src_port,
@@ -786,17 +792,17 @@ retry_on_deadlock:
      */
     pj_assert(dlg_set && !pj_list_empty(&dlg_set->dlg_list));
 
-    /* Check for forked response.
+    /* Check for forked response. 
      * Request will fork only for the initial INVITE request.
      */
 
-    //This doesn't work when there is authentication challenge, since
+    //This doesn't work when there is authentication challenge, since 
     //first_cseq evaluation will yield false.
     //if (rdata->msg_info.cseq->method.id == PJSIP_INVITE_METHOD &&
     //	rdata->msg_info.cseq->cseq == dlg_set->dlg_list.next->local.first_cseq)
 
     if (rdata->msg_info.cseq->method.id == PJSIP_INVITE_METHOD) {
-
+	
 	int st_code = rdata->msg_info.msg->line.status.code;
 	pj_str_t *to_tag = &rdata->msg_info.to->tag;
 
@@ -812,7 +818,7 @@ retry_on_deadlock:
 		break;
 
 	    /* Otherwise find the one with matching To tag. */
-	    if (pj_strcmp(to_tag, &dlg->remote.info->tag) == 0)
+	    if (pj_stricmp(to_tag, &dlg->remote.info->tag) == 0)
 		break;
 
 	    dlg = dlg->next;
@@ -823,19 +829,19 @@ retry_on_deadlock:
 	 * provisional response OR a 2xx response.
 	 */
 	if (dlg == (pjsip_dialog*)&dlg_set->dlg_list &&
-	    ((st_code/100==1 && st_code!=100) || st_code/100==2))
+	    ((st_code/100==1 && st_code!=100) || st_code/100==2)) 
 	{
 
-	    PJ_LOG(5,(THIS_FILE,
+	    PJ_LOG(5,(THIS_FILE, 
 		      "Received forked %s for existing dialog %s",
-		      pjsip_rx_data_get_info(rdata),
+		      pjsip_rx_data_get_info(rdata), 
 		      dlg_set->dlg_list.next->obj_name));
 
 	    /* Report to application about forked condition.
 	     * Application can either create a dialog or ignore the response.
 	     */
 	    if (mod_ua.param.on_dlg_forked) {
-		dlg = (*mod_ua.param.on_dlg_forked)(dlg_set->dlg_list.next,
+		dlg = (*mod_ua.param.on_dlg_forked)(dlg_set->dlg_list.next, 
 						    rdata);
 		if (dlg == NULL) {
 		    pj_mutex_unlock(mod_ua.mutex);
@@ -844,7 +850,7 @@ retry_on_deadlock:
 	    } else {
 		dlg = dlg_set->dlg_list.next;
 
-		PJ_LOG(4,(THIS_FILE,
+		PJ_LOG(4,(THIS_FILE, 
 			  "Unhandled forked %s from %s:%d, response will be "
 			  "handed over to the first dialog",
 			  pjsip_rx_data_get_info(rdata),
@@ -915,7 +921,7 @@ static void print_dialog( const char *title,
 	pj_ansi_strcpy(userinfo, "<--uri too long-->");
     else
 	userinfo[len] = '\0';
-
+    
     len = pj_ansi_snprintf(buf, size, "%s[%s]  %s",
 			   title,
 			   (dlg->state==PJSIP_DIALOG_STATE_NULL ? " - " :
@@ -939,7 +945,7 @@ PJ_DEF(void) pjsip_ua_dump(pj_bool_t detail)
 
     pj_mutex_lock(mod_ua.mutex);
 
-    PJ_LOG(3, (THIS_FILE, "Number of dialog sets: %u",
+    PJ_LOG(3, (THIS_FILE, "Number of dialog sets: %u", 
 			  pj_hash_count(mod_ua.dlg_table)));
 
     if (detail && pj_hash_count(mod_ua.dlg_table)) {
@@ -975,3 +981,4 @@ PJ_DEF(void) pjsip_ua_dump(pj_bool_t detail)
     pj_mutex_unlock(mod_ua.mutex);
 #endif
 }
+
