@@ -1,5 +1,5 @@
-/* $Id: uri_test.c 3553 2011-05-05 06:14:19Z nanang $ */
-/*
+/* $Id: uri_test.c 4210 2012-07-19 01:00:07Z ming $ */
+/* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 #include "test.h"
 #include <pjsip.h>
@@ -67,6 +67,7 @@ static pjsip_uri *create_uri14( pj_pool_t *pool );
 static pjsip_uri *create_uri15( pj_pool_t *pool );
 static pjsip_uri *create_uri16( pj_pool_t *pool );
 static pjsip_uri *create_uri17( pj_pool_t *pool );
+static pjsip_uri *create_uri18( pj_pool_t *pool );
 static pjsip_uri *create_uri25( pj_pool_t *pool );
 static pjsip_uri *create_uri26( pj_pool_t *pool );
 static pjsip_uri *create_uri27( pj_pool_t *pool );
@@ -81,6 +82,7 @@ static pjsip_uri *create_uri35( pj_pool_t *pool );
 static pjsip_uri *create_uri36( pj_pool_t *pool );
 static pjsip_uri *create_uri37( pj_pool_t *pool );
 static pjsip_uri *create_uri38( pj_pool_t *pool );
+static pjsip_uri *create_uri39( pj_pool_t *pool );
 static pjsip_uri *create_dummy( pj_pool_t *pool );
 
 #define ERR_NOT_EQUAL	-1001
@@ -93,7 +95,7 @@ struct uri_test
     pjsip_uri	    *(*creator)(pj_pool_t *pool);
     const char	    *printed;
     pj_size_t	     len;
-} uri_test_array[] =
+} uri_test_array[] = 
 {
     {
 	PJ_SUCCESS,
@@ -180,7 +182,7 @@ struct uri_test
     },
     {
 	/* Excercise strange character sets allowed in display, user, password,
-	 * host, and port.
+	 * host, and port. 
 	 */
 	PJ_SUCCESS,
 	"This is -. !% *_+`'~ me <sip:a19A&=+$,;?/%2c:%40a&Zz=+$,@"
@@ -194,7 +196,7 @@ struct uri_test
 	&create_uri15,
     },
     {
-	/* Another excercise to the allowed character sets to the username
+	/* Another excercise to the allowed character sets to the username 
 	 * and password.
 	 */
 	PJ_SUCCESS,
@@ -349,6 +351,12 @@ struct uri_test
 	"\xC0\x81 <sip:localhost>",
 	&create_uri38,
 	"\"\xC0\x81\" <sip:localhost>"
+    },
+    {
+	/* Even number of backslash before end quote in display name. */
+	PJ_SUCCESS,
+	"\"User\\\\\" <sip:localhost>",
+	&create_uri39,
     }
 
 };
@@ -423,7 +431,7 @@ static pjsip_uri *create_uri4(pj_pool_t *pool)
 static pjsip_uri *create_uri5(pj_pool_t *pool)
 {
     /* "sip:localhost;pickup=hurry;user=phone;message=I%20am%20sorry"
-       "?Subject=Hello%20There&Server=SIP%20Server"
+       "?Subject=Hello%20There&Server=SIP%20Server" 
      */
     pjsip_sip_uri *url = pjsip_sip_uri_create(pool, 0);
 
@@ -643,7 +651,7 @@ static pjsip_uri *create_uri29(pj_pool_t *pool)
     pjsip_tel_uri *uri = pjsip_tel_uri_create(pool);
 
     uri->number = pj_str("(44).1234-*#+Deaf");
-    return (pjsip_uri*)uri;
+    return (pjsip_uri*)uri;    
 }
 
 /* "tel:+1;isub=/:@&$,-_.!~*'()[]/:&$aA1%21+=" */
@@ -653,7 +661,7 @@ static pjsip_uri *create_uri30(pj_pool_t *pool)
 
     uri->number = pj_str("+1");
     uri->isub_param = pj_str("/:@&$,-_.!~*'()[]/:&$aA1!+=");
-    return (pjsip_uri*)uri;
+    return (pjsip_uri*)uri;    
 }
 
 /* "tel:+1;ext=+123" */
@@ -663,7 +671,7 @@ static pjsip_uri *create_uri31(pj_pool_t *pool)
 
     uri->number = pj_str("+1");
     uri->ext_param = pj_str("+123");
-    return (pjsip_uri*)uri;
+    return (pjsip_uri*)uri;    
 }
 
 /* "tel:911;phone-context=+1-911" */
@@ -673,7 +681,7 @@ static pjsip_uri *create_uri32(pj_pool_t *pool)
 
     uri->number = pj_str("911");
     uri->context = pj_str("+1-911");
-    return (pjsip_uri*)uri;
+    return (pjsip_uri*)uri;    
 }
 
 /* "tel:911;phone-context=emergency.example.com" */
@@ -683,7 +691,7 @@ static pjsip_uri *create_uri33(pj_pool_t *pool)
 
     uri->number = pj_str("911");
     uri->context = pj_str("EMERGENCY.EXAMPLE.COM");
-    return (pjsip_uri*)uri;
+    return (pjsip_uri*)uri;    
 }
 
 /* "tel:911;p1=p1;p2=p2" */
@@ -693,12 +701,12 @@ static pjsip_uri *create_uri34(pj_pool_t *pool)
     pjsip_param *p;
 
     uri->number = pj_str("911");
-
+    
     p = PJ_POOL_ALLOC_T(pool, pjsip_param);
     p->name = p->value = pj_str("p1");
     pj_list_insert_before(&uri->other_param, p);
 
-    return (pjsip_uri*)uri;
+    return (pjsip_uri*)uri;    
 }
 
 /* "sip:user@[::1];maddr=[::01]" */
@@ -734,7 +742,7 @@ static pjsip_uri *create_uri37( pj_pool_t *pool )
 
     url = pjsip_sip_uri_create(pool, 0);
     url->host = pj_str("localhost");
-
+    
     name->uri = (pjsip_uri*)url;
 
     return (pjsip_uri*)name;
@@ -752,13 +760,27 @@ static pjsip_uri *create_uri38( pj_pool_t *pool )
 
     url = pjsip_sip_uri_create(pool, 0);
     url->host = pj_str("localhost");
-
+    
     name->uri = (pjsip_uri*)url;
 
     return (pjsip_uri*)name;
 
 }
 
+/* "\"User\\\\\" <sip:localhost>" */
+static pjsip_uri *create_uri39(pj_pool_t *pool)
+{
+    pjsip_name_addr *name_addr = pjsip_name_addr_create(pool);
+    pjsip_sip_uri *url;
+
+    url = pjsip_sip_uri_create(pool, 0);
+    name_addr->uri = (pjsip_uri*) url;
+
+    pj_strdup2(pool, &name_addr->display, "User\\\\");
+    pj_strdup2(pool, &url->host, "localhost");
+    return (pjsip_uri*)name_addr;
+}
+
 static pjsip_uri *create_dummy(pj_pool_t *pool)
 {
     PJ_UNUSED_ARG(pool);
@@ -956,8 +978,8 @@ static int uri_benchmark(unsigned *p_parse, unsigned *p_print, unsigned *p_cmp)
         avg_parse = 1;
     avg_parse = 1000000 / avg_parse;
 
-    PJ_LOG(3,(THIS_FILE,
-	      "    %u.%u MB of urls parsed in %d.%03ds (avg=%d urls/sec)",
+    PJ_LOG(3,(THIS_FILE, 
+	      "    %u.%u MB of urls parsed in %d.%03ds (avg=%d urls/sec)", 
 	      (unsigned)(var.parse_len/1000000), (unsigned)kbytes,
 	      elapsed.sec, elapsed.msec,
 	      (unsigned)avg_parse));
@@ -975,8 +997,8 @@ static int uri_benchmark(unsigned *p_parse, unsigned *p_print, unsigned *p_cmp)
         avg_print = 1;
     avg_print = 1000000 / avg_print;
 
-    PJ_LOG(3,(THIS_FILE,
-	      "    %u.%u MB of urls printed in %d.%03ds (avg=%d urls/sec)",
+    PJ_LOG(3,(THIS_FILE, 
+	      "    %u.%u MB of urls printed in %d.%03ds (avg=%d urls/sec)", 
 	      (unsigned)(var.print_len/1000000), (unsigned)kbytes,
 	      elapsed.sec, elapsed.msec,
 	      (unsigned)avg_print));
@@ -994,8 +1016,8 @@ static int uri_benchmark(unsigned *p_parse, unsigned *p_print, unsigned *p_cmp)
         avg_cmp = 1;
     avg_cmp = 1000000 / avg_cmp;
 
-    PJ_LOG(3,(THIS_FILE,
-	      "    %u.%u MB of urls compared in %d.%03ds (avg=%d urls/sec)",
+    PJ_LOG(3,(THIS_FILE, 
+	      "    %u.%u MB of urls compared in %d.%03ds (avg=%d urls/sec)", 
 	      (unsigned)(var.cmp_len/1000000), (unsigned)kbytes,
 	      elapsed.sec, elapsed.msec,
 	      (unsigned)avg_cmp));
@@ -1040,8 +1062,8 @@ int uri_test(void)
     avg_len /= PJ_ARRAY_SIZE(uri_test_array);
 
 
-    /*
-     * Print maximum parse/sec
+    /* 
+     * Print maximum parse/sec 
      */
     for (i=0, max=0; i<COUNT; ++i)
 	if (run[i].parse > max) max = run[i].parse;
@@ -1094,3 +1116,4 @@ int uri_test(void)
 
     return PJ_SUCCESS;
 }
+
