@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Alexandre Bourget <alexandre.bourget@savoirfairelinux.com>
@@ -42,27 +42,31 @@
 
 #include "logger.h"
 #include "manager.h"
-#include "dbus/configurationmanager.h"
 
-const char * const Account::AUDIO_CODECS_KEY =      "audioCodecs";  // 0/9/110/111/112/
-const char * const Account::VIDEO_CODECS_KEY =      "videoCodecs";
-const char * const Account::VIDEO_CODEC_ENABLED =   "enabled";
-const char * const Account::VIDEO_CODEC_NAME =      "name";
-const char * const Account::VIDEO_CODEC_PARAMETERS ="parameters";
-const char * const Account::VIDEO_CODEC_BITRATE =   "bitrate";
-const char * const Account::RINGTONE_PATH_KEY =     "ringtonePath";
-const char * const Account::RINGTONE_ENABLED_KEY =  "ringtoneEnabled";
-const char * const Account::DISPLAY_NAME_KEY =      "displayName";
-const char * const Account::ALIAS_KEY =             "alias";
-const char * const Account::TYPE_KEY =              "type";
-const char * const Account::ID_KEY =                "id";
-const char * const Account::USERNAME_KEY =          "username";
+#include "client/configurationmanager.h"
+
+const char * const Account::AUDIO_CODECS_KEY            = "audioCodecs";  // 0/9/110/111/112/
+const char * const Account::VIDEO_CODECS_KEY            = "videoCodecs";
+const char * const Account::VIDEO_CODEC_ENABLED         = "enabled";
+const char * const Account::VIDEO_CODEC_NAME            = "name";
+const char * const Account::VIDEO_CODEC_PARAMETERS      = "parameters";
+const char * const Account::VIDEO_CODEC_BITRATE         = "bitrate";
+const char * const Account::RINGTONE_PATH_KEY           = "ringtonePath";
+const char * const Account::RINGTONE_ENABLED_KEY        = "ringtoneEnabled";
+const char * const Account::DISPLAY_NAME_KEY            = "displayName";
+const char * const Account::ALIAS_KEY                   = "alias";
+const char * const Account::TYPE_KEY                    = "type";
+const char * const Account::ID_KEY                      = "id";
+const char * const Account::USERNAME_KEY                = "username";
 const char * const Account::AUTHENTICATION_USERNAME_KEY = "authenticationUsername";
-const char * const Account::PASSWORD_KEY =          "password";
-const char * const Account::HOSTNAME_KEY =          "hostname";
-const char * const Account::ACCOUNT_ENABLE_KEY =    "enable";
-const char * const Account::ACCOUNT_AUTOANSWER_KEY =    "autoAnswer";
-const char * const Account::MAILBOX_KEY =           "mailbox";
+const char * const Account::PASSWORD_KEY                = "password";
+const char * const Account::HOSTNAME_KEY                = "hostname";
+const char * const Account::ACCOUNT_ENABLE_KEY          = "enable";
+const char * const Account::ACCOUNT_AUTOANSWER_KEY      = "autoAnswer";
+const char * const Account::MAILBOX_KEY                 = "mailbox";
+const char * const Account::DEFAULT_USER_AGENT          = "SFLphone/" PACKAGE_VERSION;
+const char * const Account::USER_AGENT_KEY              = "useragent";
+const char * const Account::PRESENCE_MODULE_ENABLED_KEY = "presenceModuleEnabled";
 
 using std::map;
 using std::string;
@@ -80,14 +84,19 @@ Account::Account(const string &accountID) :
     , audioCodecList_()
     , videoCodecList_()
     , audioCodecStr_()
-    , ringtonePath_("/usr/share/sflphone/ringtones/konga.ul")
+    , ringtonePath_("")
     , ringtoneEnabled_(true)
     , displayName_("")
-    , userAgent_("SFLphone")
+    , userAgent_(DEFAULT_USER_AGENT)
     , mailBox_()
 {
     // Initialize the codec order, used when creating a new account
     loadDefaultCodecs();
+    #ifdef __ANDROID__
+        ringtonePath_ = "/data/data/org.sflphone/files/ringtones/konga.ul";
+    #else
+        ringtonePath_ = "/usr/share/sflphone/ringtones/konga.ul";
+    #endif
 }
 
 Account::~Account()
@@ -97,9 +106,8 @@ void Account::setRegistrationState(const RegistrationState &state)
 {
     if (state != registrationState_) {
         registrationState_ = state;
-
         // Notify the client
-        ConfigurationManager *c(Manager::instance().getDbusManager()->getConfigurationManager());
+        ConfigurationManager *c(Manager::instance().getClient()->getConfigurationManager());
         c->registrationStateChanged(accountID_, registrationState_);
     }
 }
@@ -157,8 +165,8 @@ namespace {
         }
 
         // check that it's in the list of valid codecs and that it has all the required fields
-        for (vector<map<string, string> >::const_iterator i = defaults.begin(); i != defaults.end(); ++i) {
-            const map<string, string>::const_iterator defaultName = i->find(Account::VIDEO_CODEC_NAME);
+        for (const auto &i : defaults) {
+            const auto defaultName = i.find(Account::VIDEO_CODEC_NAME);
             if (defaultName->second == name->second) {
                 return isFieldValid(codec, Account::VIDEO_CODEC_BITRATE, isPositiveInteger)
                     and isFieldValid(codec, Account::VIDEO_CODEC_ENABLED, isBoolean);
@@ -170,16 +178,15 @@ namespace {
 
     bool isCodecListValid(const vector<map<string, string> > &list)
     {
-        const vector<map<string, string> > defaults(libav_utils::getDefaultCodecs());
+        const auto defaults(libav_utils::getDefaultCodecs());
         if (list.size() != defaults.size()) {
             ERROR("New codec list has a different length than the list of supported codecs");
             return false;
         }
 
         // make sure that all codecs are present
-        for (vector<map<string, string> >::const_iterator i = list.begin();
-             i != list.end(); ++i) {
-            if (not isCodecValid(*i, defaults))
+        for (const auto &i : list) {
+            if (not isCodecValid(i, defaults))
                 return false;
         }
         return true;
@@ -235,8 +242,8 @@ void Account::setActiveAudioCodecs(const vector<string> &list)
 
     // list contains the ordered payload of active codecs picked by the user for this account
     // we used the codec vector to save the order.
-    for (vector<string>::const_iterator iter = list.begin(); iter != list.end(); ++iter) {
-        int payload = std::atoi(iter->c_str());
+    for (const auto &item : list) {
+        int payload = std::atoi(item.c_str());
         audioCodecList_.push_back(payload);
     }
 
@@ -302,3 +309,26 @@ Account::getActiveVideoCodecs() const
     result.erase(std::remove_if(result.begin(), result.end(), is_inactive), result.end());
     return result;
 }
+
+#define find_iter()                             \
+        const auto iter = details.find(key);    \
+        if (iter == details.end()) {            \
+            ERROR("Couldn't find key \"%s\"", key); \
+            return;                             \
+        }
+
+void
+Account::parseString(const std::map<std::string, std::string> &details, const char *key, std::string &s)
+{
+    find_iter();
+    s = iter->second;
+}
+
+void
+Account::parseBool(const std::map<std::string, std::string> &details, const char *key, bool &b)
+{
+    find_iter();
+    b = iter->second == Conf::TRUE_STR;
+}
+
+#undef find_iter
