@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Yan Morin <yan.morin@savoirfairelinux.com>
  *
@@ -64,7 +64,7 @@ class Account : public Serializable {
          */
         virtual ~Account();
 
-        virtual void setAccountDetails(std::map<std::string, std::string> details) = 0;
+        virtual void setAccountDetails(const std::map<std::string, std::string> &details) = 0;
 
         virtual std::map<std::string, std::string> getAccountDetails() const = 0;
 
@@ -205,6 +205,10 @@ class Account : public Serializable {
         void loadDefaultCodecs();
 
     protected:
+
+        static void parseString(const std::map<std::string, std::string> &details, const char *key, std::string &s);
+        static void parseBool(const std::map<std::string, std::string> &details, const char *key, bool &b);
+
         friend class ConfigurationTest;
         // General configuration keys for accounts
         static const char * const AUDIO_CODECS_KEY;
@@ -222,6 +226,9 @@ class Account : public Serializable {
         static const char * const ACCOUNT_ENABLE_KEY;
         static const char * const ACCOUNT_AUTOANSWER_KEY;
         static const char * const MAILBOX_KEY;
+        static const char * const USER_AGENT_KEY;
+        static const char * const DEFAULT_USER_AGENT;
+        static const char * const PRESENCE_MODULE_ENABLED_KEY;
 
         static std::string mapStateNumberToString(RegistrationState state);
 
