@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *
@@ -33,14 +33,13 @@
 #include "audiolayer.h"
 #include "audio/dcblocker.h"
 #include "manager.h"
-#include "scoped_lock.h"
 
-unsigned int AudioLayer::captureGain_ = 100;
-unsigned int AudioLayer::playbackGain_ = 100;
-
-AudioLayer::AudioLayer()
-    : isStarted_(false)
-    , playbackMode_(NONE)
+AudioLayer::AudioLayer(const AudioPreference &pref)
+    : isCaptureMuted_(pref.getCaptureMuted())
+    , isPlaybackMuted_(pref.getPlaybackMuted())
+    , captureGain_(pref.getVolumemic())
+    , playbackGain_(pref.getVolumespkr())
+    , isStarted_(false)
     , urgentRingBuffer_(SIZEBUF, MainBuffer::DEFAULT_ID)
     , sampleRate_(Manager::instance().getMainBuffer().getInternalSamplingRate())
     , mutex_()
@@ -48,45 +47,35 @@ AudioLayer::AudioLayer()
     , converter_(sampleRate_)
     , lastNotificationTime_(0)
 {
-    pthread_mutex_init(&mutex_, NULL);
     urgentRingBuffer_.createReadPointer(MainBuffer::DEFAULT_ID);
 }
 
 AudioLayer::~AudioLayer()
-{
-    pthread_mutex_destroy(&mutex_);
-}
+{}
 
 void AudioLayer::flushMain()
 {
-    sfl::ScopedLock guard(mutex_);
+    std::lock_guard<std::mutex> lock(mutex_);
     // should pass call id
     Manager::instance().getMainBuffer().flushAllBuffers();
 }
 
 void AudioLayer::flushUrgent()
 {
-    sfl::ScopedLock guard(mutex_);
+    std::lock_guard<std::mutex> lock(mutex_);
     urgentRingBuffer_.flushAll();
 }
 
-void AudioLayer::putUrgent(void* buffer, int toCopy)
+void AudioLayer::putUrgent(AudioBuffer& buffer)
 {
-    sfl::ScopedLock guard(mutex_);
-    urgentRingBuffer_.put(buffer, toCopy);
-}
-
-void AudioLayer::applyGain(SFLDataFormat *src , int samples, int gain)
-{
-    if (gain != 100)
-        for (int i = 0 ; i < samples; i++)
-            src[i] = src[i] * gain* 0.01;
+    std::lock_guard<std::mutex> lock(mutex_);
+    urgentRingBuffer_.put(buffer);
 }
 
 // Notify (with a beep) an incoming call when there is already a call in progress
 void AudioLayer::notifyIncomingCall()
 {
-    if (!Manager::instance().incomingCallWaiting())
+    if (!Manager::instance().incomingCallsWaiting())
         return;
 
     time_t now = time(NULL);
@@ -103,11 +92,10 @@ void AudioLayer::notifyIncomingCall()
 
     Tone tone("440/160", getSampleRate());
     unsigned int nbSample = tone.getSize();
-    SFLDataFormat buf[nbSample];
-    tone.getNext(buf, nbSample);
+    AudioBuffer buf(nbSample, 1, 8000);
+    tone.getNext(buf, 1.0);
 
     /* Put the data in the urgent ring buffer */
     flushUrgent();
-    putUrgent(buf, sizeof buf);
+    putUrgent(buf);
 }
-
