@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Yan Morin <yan.morin@savoirfairelinux.com>
  *  Author:  Jerome Oufella <jerome.oufella@savoirfairelinux.com>
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
@@ -34,7 +34,7 @@
 #ifndef AUDIO_LAYER_H_
 #define AUDIO_LAYER_H_
 
-#include <pthread.h>
+#include <mutex>
 #include <sys/time.h>
 #include <vector>
 #include "ringbuffer.h"
@@ -67,16 +67,7 @@ class AudioLayer {
             SFL_PCM_RINGTONE = 0x0024       /** To open the ringtone device only */
         };
 
-        enum PlaybackMode {
-            NONE = 0,
-            TONE,
-            RINGTONE,
-            VOICE,
-            URGENT,
-            ZEROS
-        };
-
-        AudioLayer();
+        AudioLayer(const AudioPreference &);
         virtual ~AudioLayer();
 
         virtual std::vector<std::string> getCaptureDeviceList() const = 0;
@@ -109,21 +100,12 @@ class AudioLayer {
             return isStarted_;
         }
 
-        PlaybackMode getPlaybackMode() const {
-            return playbackMode_;
-        }
-
-        void setPlaybackMode(PlaybackMode mode) {
-            playbackMode_ = mode;
-        }
-
         /**
          * Send a chunk of data to the hardware buffer to start the playback
          * Copy data in the urgent buffer.
          * @param buffer The buffer containing the data to be played ( ringtones )
-         * @param toCopy The size of the buffer
          */
-        void putUrgent(void* buffer, int toCopy);
+        void putUrgent(AudioBuffer& buffer);
 
         /**
          * Flush main buffer
@@ -135,50 +117,55 @@ class AudioLayer {
          */
         void flushUrgent();
 
-        /**
-         * Apply gain to audio frame
-         */
-        static void applyGain(SFLDataFormat *src , int samples, int gain);
-
-        /**
-         * Convert audio amplitude value from linear value to dB
-         */
-        static double amplitudeLinearToDB(double value) {
-            return 20.0 * log10(value);
+        bool isCaptureMuted() const {
+            return isCaptureMuted_;
         }
 
         /**
-         * Convert audio amplitude from dB to Linear value
+         * Mute capture (microphone)
          */
-        static double ampluitudeDBToLinear(double value) {
-            return pow(10.0, value / 20.0);
+        void muteCapture(bool muted) {
+            isCaptureMuted_ = muted;
+        }
+
+        bool isPlaybackMuted() const {
+            return isPlaybackMuted_;
+        }
+
+        /**
+         * Mute playback
+         */
+        void mutePlayback(bool muted) {
+            isPlaybackMuted_ = muted;
         }
 
         /**
          * Set capture stream gain (microphone)
+         * Range should be [-1.0, 1.0]
          */
-        void setCaptureGain(unsigned int gain) {
+        void setCaptureGain(double gain) {
             captureGain_ = gain;
         }
 
         /**
-         * Set capture stream gain (microphone)
+         * Get capture stream gain (microphone)
          */
-       unsigned int getCaptureGain(void) {
+        double getCaptureGain() const {
             return captureGain_;
         }
 
         /**
          * Set playback stream gain (speaker)
+         * Range should be [-1.0, 1.0]
          */
-        void setPlaybackGain(unsigned int gain) {
+        void setPlaybackGain(double gain) {
             playbackGain_ = gain;
         }
 
         /**
          * Get playback stream gain (speaker)
          */
-        unsigned int getPlaybackGain(void) {
+        double getPlaybackGain() const {
             return playbackGain_;
         }
 
@@ -196,26 +183,35 @@ class AudioLayer {
          */
         void notifyIncomingCall();
 
+        virtual void updatePreference(AudioPreference &pref, int index, PCMType type) = 0;
+
+    protected:
+
+        /**
+         * True if capture is not to be used
+         */
+        bool isCaptureMuted_;
+
+        /**
+         * True if playback is not to be used
+         */
+        bool isPlaybackMuted_;
+
         /**
          * Gain applied to mic signal
          */
-        static unsigned int captureGain_;
+        double captureGain_;
 
         /**
          * Gain applied to playback signal
          */
-        static unsigned int playbackGain_;
+        double playbackGain_;
 
-        virtual void updatePreference(AudioPreference &pref, int index, PCMType type) = 0;
-
-    protected:
         /**
          * Whether or not the audio layer stream is started
          */
         bool isStarted_;
 
-        PlaybackMode playbackMode_;
-
         /**
          * Urgent ring buffer used for ringtones
          */
@@ -230,7 +226,7 @@ class AudioLayer {
         /**
          * Lock for the entire audio layer
          */
-        pthread_mutex_t mutex_;
+        std::mutex mutex_;
 
         /**
          * Remove audio offset that can be introduced by certain cheap audio device
@@ -243,6 +239,7 @@ class AudioLayer {
         SamplerateConverter converter_;
 
     private:
+
         /**
          * Time of the last incoming call notification
          */
