@@ -1,6 +1,7 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
+ *  Author: Adrien Beraud <adrien.beraud@wisdomvibes.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,15 +40,15 @@
 using std::ptrdiff_t;
 
 #include <ccrtp/rtp.h>
-#include <tr1/array>
+#include <array>
 #include <list>
-#include <pthread.h>
+#include <mutex>
+#include <atomic>
 
 #include "noncopyable.h"
 #include "audio/codecs/audiocodec.h"
 #include "audio/samplerateconverter.h"
-#include "audio/noisesuppress.h"
-#include "audio/gaincontrol.h"
+#include "audio/dsp.h"
 
 class SIPCall;
 
@@ -88,26 +89,26 @@ class AudioRtpRecord {
 
     private:
         std::vector<AudioCodec*> audioCodecs_;
-        pthread_mutex_t audioCodecMutex_;
+        std::mutex audioCodecMutex_;
         // these will have the same value unless we are sending
         // a different codec than we are receiving (asymmetric RTP)
         int encoderPayloadType_;
         int decoderPayloadType_;
         bool hasDynamicPayloadType_;
-        std::tr1::array<SFLDataFormat, DEC_BUFFER_SIZE> decData_;
-// FIXME: resampledData should be resized as needed
-        std::tr1::array<SFLDataFormat, DEC_BUFFER_SIZE * 4> resampledData_;
-        std::tr1::array<unsigned char, DEC_BUFFER_SIZE> encodedData_;
+        AudioBuffer decData_;
+        AudioBuffer resampledData_;
+        std::array<unsigned char, DEC_BUFFER_SIZE> encodedData_;
         SamplerateConverter *converterEncode_;
         SamplerateConverter *converterDecode_;
         int codecFrameSize_;
+        int codecChannels_;
         int converterSamplingRate_;
         double fadeFactor_;
 
 #if HAVE_SPEEXDSP
-        NoiseSuppress *noiseSuppressEncode_;
-        NoiseSuppress *noiseSuppressDecode_;
-        pthread_mutex_t audioProcessMutex_;
+        DSP *dspEncode_;
+        DSP *dspDecode_;
+        std::mutex audioProcessMutex_;
 #endif
 
         unsigned int dtmfPayloadType_;
@@ -117,13 +118,9 @@ class AudioRtpRecord {
         /**
         * Ramp In audio data to avoid audio click from peer
         */
-        void fadeInDecodedData(size_t size);
+        void fadeInDecodedData();//size_t size);
         NON_COPYABLE(AudioRtpRecord);
-#ifdef CCPP_PREFIX
-        ost::AtomicCounter dead_;
-#else
-        ucommon::atomic::counter dead_;
-#endif
+        std::atomic<bool> dead_;
         size_t currentCodecIndex_;
 };
 
@@ -154,7 +151,11 @@ class AudioRtpRecordHandler {
             return audioRtpRecord_.codecFrameSize_;
         }
 
-        bool getHasDynamicPayload() const {
+        int getCodecChannels() const {
+            return audioRtpRecord_.codecChannels_;
+        }
+
+        bool hasDynamicPayload() const {
             return audioRtpRecord_.hasDynamicPayloadType_;
         }
 
@@ -169,7 +170,7 @@ class AudioRtpRecordHandler {
         void initBuffers();
 
 #if HAVE_SPEEXDSP
-        void initNoiseSuppress();
+        void initDSP();
 #endif
 
         /**
@@ -200,7 +201,6 @@ class AudioRtpRecordHandler {
 
     private:
         const std::string id_;
-        GainControl gainController;
         int warningInterval_;
 };
 }
