@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Pierre-Luc Bacon <pierre-luc.bacon@savoirfairelinux.com>
  *  Author: Alexandre Bourget <alexandre.bourget@savoirfairelinux.com>
  *  Author: Laurielle Lea <laurielle.lea@savoirfairelinux.com>
@@ -71,11 +71,13 @@ void AudioRtpSession::updateSessionMedia(const std::vector<AudioCodec*> &audioCo
     Manager::instance().audioSamplingRateChanged(audioRtpRecord_.codecSampleRate_);
 
 #if HAVE_SPEEXDSP
+
     if (lastSamplingRate != audioRtpRecord_.codecSampleRate_) {
         DEBUG("Update noise suppressor with sampling rate %d and frame size %d",
               getCodecSampleRate(), getCodecFrameSize());
-        initNoiseSuppress();
+        initDSP();
     }
+
 #endif
 }
 
@@ -85,6 +87,7 @@ void AudioRtpSession::setSessionMedia(const std::vector<AudioCodec*> &audioCodec
 
     // G722 requires timestamp to be incremented at 8kHz
     const ost::PayloadType payloadType = getEncoderPayloadType();
+
     if (payloadType == ost::sptG722) {
         const int G722_RTP_TIME_INCREMENT = 160;
         timestampIncrement_ = G722_RTP_TIME_INCREMENT;
@@ -93,9 +96,9 @@ void AudioRtpSession::setSessionMedia(const std::vector<AudioCodec*> &audioCodec
 
     if (payloadType == ost::sptG722) {
         const int G722_RTP_CLOCK_RATE = 8000;
-        queue_.setPayloadFormat(ost::DynamicPayloadFormat( payloadType, G722_RTP_CLOCK_RATE));
+        queue_.setPayloadFormat(ost::DynamicPayloadFormat(payloadType, G722_RTP_CLOCK_RATE));
     } else {
-        if (getHasDynamicPayload())
+        if (hasDynamicPayload())
             queue_.setPayloadFormat(ost::DynamicPayloadFormat(payloadType, getCodecSampleRate()));
         else
             queue_.setPayloadFormat(ost::StaticPayloadFormat(static_cast<ost::StaticPayloadType>(payloadType)));
@@ -114,7 +117,8 @@ void AudioRtpSession::sendDtmfEvent()
     DEBUG("Send RTP Dtmf (%d)", dtmf.payload.event);
 
     const int increment = getIncrementForDTMF();
-    timestamp_ += increment;
+    if (dtmf.newevent)
+        timestamp_ += increment;
 
     // discard equivalent size of audio
     processDataEncode();
@@ -125,7 +129,12 @@ void AudioRtpSession::sendDtmfEvent()
     // Set marker in case this is a new Event
     if (dtmf.newevent)
         queue_.setMark(true);
-    queue_.sendImmediate(timestamp_, (const unsigned char *) (& (dtmf.payload)), sizeof (ost::RTPPacket::RFC2833Payload));
+
+    // Send end packet three times (without changing it). Sequence number is
+    // incremented automatically by ccrtp, which is the correct behaviour.
+    const unsigned repetitions = dtmf.payload.ebit ? 3 : 1;
+    for (unsigned i = 0; i < repetitions; ++i)
+        queue_.sendImmediate(timestamp_, (const unsigned char *)(& (dtmf.payload)), sizeof(ost::RTPPacket::RFC2833Payload));
 
     // This is no longer a new event
     if (dtmf.newevent) {
@@ -134,15 +143,22 @@ void AudioRtpSession::sendDtmfEvent()
     }
 
     // restore the payload to audio
-    const ost::StaticPayloadFormat pf(static_cast<ost::StaticPayloadType>(getEncoderPayloadType()));
-    queue_.setPayloadFormat(pf);
+    if (hasDynamicPayload()) {
+        const ost::DynamicPayloadFormat pf(getEncoderPayloadType(), getCodecSampleRate());
+        queue_.setPayloadFormat(pf);
+    } else {
+        const ost::StaticPayloadFormat pf(static_cast<ost::StaticPayloadType>(getEncoderPayloadType()));
+        queue_.setPayloadFormat(pf);
+    }
 
     // decrease length remaining to process for this event
     dtmf.length -= increment;
     dtmf.payload.duration++;
-    // next packet is going to be the last one
+
+    // next packet is going to be the end packet (transmitted 3 times)
     if ((dtmf.length - increment) < increment)
         dtmf.payload.ebit = true;
+
     if (dtmf.length < increment)
         audioRtpRecord_.dtmfQueue_.pop_front();
 }
@@ -200,7 +216,7 @@ void AudioRtpSession::setDestinationIpAddress()
 
     if (!remote_ip_) {
         WARN("Target IP address (%s) is not correct!",
-              call_.getLocalSDP()->getRemoteIP().data());
+             call_.getLocalSDP()->getRemoteIP().data());
         return;
     }
 
@@ -240,7 +256,7 @@ void AudioRtpSession::prepareRtpReceiveThread(const std::vector<AudioCodec*> &au
     setSessionMedia(audioCodecs);
     initBuffers();
 #if HAVE_SPEEXDSP
-    initNoiseSuppress();
+    initDSP();
 #endif
 
     queue_.enableStack();
@@ -276,6 +292,7 @@ AudioRtpSession::AudioRtpSendThread::AudioRtpSendThread(AudioRtpSession &session
 AudioRtpSession::AudioRtpSendThread::~AudioRtpSendThread()
 {
     running_ = false;
+
     if (thread_)
         pthread_join(thread_, NULL);
 }
