@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Pierre-Luc Bacon <pierre-luc.bacon@savoirfairelinux.com>
  *  Author: Alexandre Bourget <alexandre.bourget@savoirfairelinux.com>
  *  Author: Laurielle Lea <laurielle.lea@savoirfairelinux.com>
@@ -43,7 +43,7 @@ AudioSymmetricRtpSession::AudioSymmetricRtpSession(SIPCall &call) :
     , AudioRtpSession(call, *this)
 {
     DEBUG("Setting new RTP session with destination %s:%d",
-            call_.getLocalIp().c_str(), call_.getLocalAudioPort());
+          call_.getLocalIp().c_str(), call_.getLocalAudioPort());
     audioRtpRecord_.callId_ = call_.getCallId();
 }
 
@@ -60,4 +60,26 @@ void AudioSymmetricRtpSession::startReceiveThread()
 {
     ost::SymmetricRTPSession::start();
 }
+
+// redefined from QueueRTCPManager
+void AudioSymmetricRtpSession::onGotRR(ost::SyncSource& source, ost::RTCPCompoundHandler::RecvReport& RR, uint8 blocks)
+{
+    ost::SymmetricRTPSession::onGotRR(source, RR, blocks);
+    // TODO: do something with this data
+#if 0
+    std::cout << "I got an RR RTCP report from "
+        << std::hex << (int)source.getID() << "@"
+        << std::dec
+        << source.getNetworkAddress() << ":"
+        << source.getControlTransportPort() << std::endl;
+#endif
+}
+
+// redefined from QueueRTCPManager
+void AudioSymmetricRtpSession::onGotSR(ost::SyncSource& source, ost::RTCPCompoundHandler::SendReport& SR, uint8 blocks)
+{
+    ost::SymmetricRTPSession::onGotSR(source, SR, blocks);
+    // TODO: do something with this data
+}
+
 }
