@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *  Author: Pierre-Luc Bacon <pierre-luc.bacon@savoirfairelinux.com>
  *
@@ -39,11 +39,6 @@
 #include "manager.h"
 #include "fileutils.h"
 
-#include <libzrtpcpp/zrtpccrtp.h>
-#include <libzrtpcpp/ZrtpQueue.h>
-#include <libzrtpcpp/ZrtpUserCallback.h>
-#include <ccrtp/rtp.h>
-
 namespace sfl {
 
 AudioZrtpSession::AudioZrtpSession(SIPCall &call, const std::string &zidFilename) :
@@ -72,16 +67,20 @@ void AudioZrtpSession::initializeZid()
         throw ZrtpZidException("zid filename empty");
 
     const std::string cache_home(XDG_CACHE_HOME);
+    std::string zidDirName;
     std::string zidCompleteFilename;
 
     if (not cache_home.empty()) {
-        zidCompleteFilename = cache_home + DIR_SEPARATOR_STR + zidFilename_;
+        zidDirName = cache_home;
     } else {
-        zidCompleteFilename = fileutils::get_home_dir() + DIR_SEPARATOR_STR +
-                              ".cache" + DIR_SEPARATOR_STR + PACKAGE +
-                              DIR_SEPARATOR_STR + zidFilename_;
+        zidDirName = fileutils::get_home_dir() + DIR_SEPARATOR_STR +
+                              ".cache" + DIR_SEPARATOR_STR + PACKAGE;
     }
 
+    zidCompleteFilename = zidDirName + DIR_SEPARATOR_STR + zidFilename_;
+
+    fileutils::check_dir(zidDirName.c_str());
+
     if (initialize(zidCompleteFilename.c_str()) >= 0) {
         setEnableZrtp(true);
         setUserCallback(new ZrtpSessionCallback(call_));
