@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Yan Morin <yan.morin@savoirfairelinux.com>
  *  Author: Laurielle Lea <laurielle.lea@savoirfairelinux.com>
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
@@ -40,6 +40,7 @@
 #include <dlfcn.h>
 #include <algorithm> // for std::find
 #include <dlfcn.h>
+#include <stdexcept>
 
 #include "audiocodec.h"
 #include "audiocodecfactory.h"
@@ -56,10 +57,9 @@ AudioCodecFactory::AudioCodecFactory() :
     if (codecDynamicList.empty())
         ERROR("No codecs available");
     else {
-        for (AudioCodecVector::const_iterator iter = codecDynamicList.begin();
-                iter != codecDynamicList.end() ; ++iter) {
-            codecsMap_[(int)(*iter)->getPayloadType()] = *iter;
-            DEBUG("Loaded codec %s" , (*iter)->getMimeSubtype().c_str());
+        for (const auto &codec: codecDynamicList) {
+            codecsMap_[(int) codec->getPayloadType()] = codec;
+            DEBUG("Loaded codec %s" , codec->getMimeSubtype().c_str());
         }
     }
 }
@@ -68,8 +68,9 @@ void
 AudioCodecFactory::setDefaultOrder()
 {
     defaultCodecList_.clear();
-    for (AudioCodecsMap::const_iterator i = codecsMap_.begin(); i != codecsMap_.end(); ++i)
-        defaultCodecList_.push_back(i->first);
+
+    for (const auto &codec : codecsMap_)
+        defaultCodecList_.push_back(codec.first);
 }
 
 std::string
@@ -87,9 +88,10 @@ std::vector<int32_t>
 AudioCodecFactory::getCodecList() const
 {
     std::vector<int32_t> list;
-    for (AudioCodecsMap::const_iterator iter = codecsMap_.begin(); iter != codecsMap_.end(); ++iter)
-        if (iter->second)
-            list.push_back((int32_t) iter->first);
+
+    for (const auto &codec : codecsMap_)
+        if (codec.second)
+            list.push_back((int32_t) codec.first);
 
     return list;
 }
@@ -100,7 +102,7 @@ AudioCodecFactory::getCodec(int payload) const
     AudioCodecsMap::const_iterator iter = codecsMap_.find(payload);
 
     if (iter != codecsMap_.end())
-        return static_cast<sfl::AudioCodec *>(iter->second);
+        return iter->second;
     else {
         ERROR("Cannot find codec %i", payload);
         return NULL;
@@ -130,6 +132,17 @@ AudioCodecFactory::getSampleRate(int payload) const
         return 0;
 }
 
+unsigned
+AudioCodecFactory::getChannels(int payload) const
+{
+    AudioCodecsMap::const_iterator iter = codecsMap_.find(payload);
+
+    if (iter != codecsMap_.end())
+        return iter->second->getChannels();
+    else
+        return 0;
+}
+
 void
 AudioCodecFactory::saveActiveCodecs(const std::vector<std::string>& list)
 {
@@ -137,8 +150,8 @@ AudioCodecFactory::saveActiveCodecs(const std::vector<std::string>& list)
     // list contains the ordered payload of active codecs picked by the user
     // we used the codec vector to save the order.
 
-    for (std::vector<std::string>::const_iterator iter = list.begin(); iter != list.end(); ++iter) {
-        int payload = std::atoi(iter->c_str());
+    for (const auto &codec : list) {
+        int payload = std::atoi(codec.c_str());
 
         if (isCodecLoaded(payload))
             defaultCodecList_.push_back(static_cast<int>(payload));
@@ -148,9 +161,8 @@ AudioCodecFactory::saveActiveCodecs(const std::vector<std::string>& list)
 
 AudioCodecFactory::~AudioCodecFactory()
 {
-    for (std::vector<AudioCodecHandlePointer>::iterator iter =
-         codecInMemory_.begin(); iter != codecInMemory_.end(); ++iter)
-        unloadCodec(*iter);
+    for (auto &codec : codecInMemory_)
+        unloadCodec(codec);
 }
 
 std::vector<sfl::AudioCodec*>
@@ -168,8 +180,13 @@ AudioCodecFactory::scanCodecDirectory()
 
     const char *progDir = fileutils::get_program_dir();
 
-    if (progDir)
+    if (progDir) {
+#ifdef __ANDROID__
+        dirToScan.push_back(std::string(progDir) + DIR_SEPARATOR_STR + "lib/");
+#else
         dirToScan.push_back(std::string(progDir) + DIR_SEPARATOR_STR + "audio/codecs/");
+#endif
+	}
 
     for (size_t i = 0 ; i < dirToScan.size() ; i++) {
         std::string dirStr = dirToScan[i];
@@ -226,13 +243,20 @@ AudioCodecFactory::loadCodec(const std::string &path)
         return NULL;
     }
 
-    sfl::AudioCodec *a = static_cast<sfl::AudioCodec *>(createCodec());
-    if (a)
-        codecInMemory_.push_back(AudioCodecHandlePointer(a, codecHandle));
-    else
-        dlclose(codecHandle);
+    try {
+        sfl::AudioCodec *a = createCodec();
 
-    return a;
+        if (a)
+            codecInMemory_.push_back(AudioCodecHandlePointer(a, codecHandle));
+        else
+            dlclose(codecHandle);
+
+        return a;
+    } catch (const std::runtime_error &e) {
+        ERROR("%s", e.what());
+        dlclose(codecHandle);
+        return nullptr;
+    }
 }
 
 
@@ -240,6 +264,9 @@ void
 AudioCodecFactory::unloadCodec(AudioCodecHandlePointer &ptr)
 {
     destroy_t *destroyCodec = 0;
+    // flush last error
+    dlerror();
+
     if (ptr.second)
         destroyCodec = (destroy_t*) dlsym(ptr.second, "destroy");
 
@@ -260,23 +287,18 @@ AudioCodecFactory::unloadCodec(AudioCodecHandlePointer &ptr)
 sfl::AudioCodec*
 AudioCodecFactory::instantiateCodec(int payload) const
 {
-    std::vector<AudioCodecHandlePointer>::const_iterator iter;
-
-    sfl::AudioCodec *result = NULL;
-    for (iter = codecInMemory_.begin(); iter != codecInMemory_.end(); ++iter) {
-        if (iter->first->getPayloadType() == payload) {
-            create_t* createCodec = (create_t*) dlsym(iter->second , AUDIO_CODEC_ENTRY_SYMBOL);
-
-            const char *error = dlerror();
-
-            if (error)
-                ERROR("%s", error);
-            else
-                result = static_cast<sfl::AudioCodec *>(createCodec());
+    for (const auto &codec : codecInMemory_) {
+        if (codec.first->getPayloadType() == payload) {
+            try {
+                return codec.first->clone();
+            } catch (const std::runtime_error &e) {
+                ERROR("%s", e.what());
+                return nullptr;
+            }
         }
     }
 
-    return result;
+    return nullptr;
 }
 
 bool
@@ -284,7 +306,6 @@ AudioCodecFactory::seemsValid(const std::string &lib)
 {
     // The name of the shared library seems valid  <==> it looks like libcodec_xxx.so
     // We check this
-
     static const std::string prefix("libcodec_");
     static const std::string suffix(".so");
 
@@ -299,28 +320,30 @@ AudioCodecFactory::seemsValid(const std::string &lib)
         return false;
 
     static const std::string validCodecs[] = {
-    "ulaw",
-    "alaw",
-    "g722",
-    "g729", //G729 have to be loaded first, if it is valid or not is checked later
-    "opus", //Opus have to be loaded first, if it is valid or not is checked later
+        "ulaw",
+        "alaw",
+        "g722",
+        "g729", //G729 have to be loaded first, if it is valid or not is checked later
+        "opus", //Opus have to be loaded first, if it is valid or not is checked later
 #ifdef HAVE_SPEEX_CODEC
-    "speex_nb",
-    "speex_wb",
-    "speex_ub",
+        "speex_nb",
+        "speex_wb",
+        "speex_ub",
 #endif
 
 #ifdef HAVE_GSM_CODEC
-    "gsm",
+        "gsm",
 #endif
 
 #ifdef BUILD_ILBC
-    "ilbc",
+        "ilbc",
 #endif
-    ""};
+        ""
+    };
 
     const std::string name(lib.substr(prefix.length(), len));
     const std::string *end = validCodecs + ARRAYSIZE(validCodecs);
+
     return find(validCodecs, end, name) != end;
 }
 
@@ -333,10 +356,8 @@ AudioCodecFactory::alreadyInCache(const std::string &lib)
 bool
 AudioCodecFactory::isCodecLoaded(int payload) const
 {
-    AudioCodecsMap::const_iterator iter;
-
-    for (iter = codecsMap_.begin(); iter != codecsMap_.end(); ++iter)
-        if (iter->first == payload)
+    for (const auto &codec : codecsMap_)
+        if (codec.first == payload)
             return true;
 
     return false;
@@ -359,6 +380,11 @@ AudioCodecFactory::getCodecSpecifications(const int32_t& payload) const
     // Add the bit rate
     ss << getBitRate(static_cast<int>(payload));
     v.push_back(ss.str());
+    ss.str("");
+
+    // Add the channel number
+    ss << getChannels(static_cast<int>(payload));
+    v.push_back(ss.str());
 
     return v;
 }
