@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author:  Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *
  *  Mostly borrowed from asterisk's sources (Steve Underwood <steveu@coppice.org>)
@@ -41,7 +41,7 @@ class G722 : public sfl::AudioCodec {
 
     public:
         G722() : sfl::AudioCodec(9, "G722", 16000, 320, 1), decode_state_(),
-        encode_state_() {
+            encode_state_() {
             bitrate_ = 64;
             hasDynamicPayload_ = false;
 
@@ -50,19 +50,24 @@ class G722 : public sfl::AudioCodec {
         }
 
     private:
-        int decode(SFLDataFormat *dst, unsigned char *src, size_t buf_size)
+        AudioCodec *
+        clone()
+        {
+            return new G722;
+        }
+
+        int decode(SFLAudioSample *dst, unsigned char *src, size_t buf_size)
         {
             return g722_decode(dst, src, buf_size);
         }
 
-        int encode(unsigned char *dst, SFLDataFormat *src, size_t /*buf_size*/)
+        int encode(unsigned char *dst, SFLAudioSample *src, size_t /*buf_size*/)
         {
             int out = g722_encode(dst, src, frameSize_);
             return out;
         }
 
-        static void g722_state_init(g722_state_t &state)
-        {
+        static void g722_state_init(g722_state_t &state) {
             state.itu_test_mode = false;
 
             // 8 => 64 kbps;  7 => 56 kbps;  6 => 48 kbps
@@ -86,12 +91,12 @@ class G722 : public sfl::AudioCodec {
             state.out_bits = 0;
         }
 
-        SFLDataFormat saturate(int32_t amp)
+        SFLAudioSample saturate(int32_t amp)
         {
-            SFLDataFormat amp16 = 0;
+            SFLAudioSample amp16 = 0;
 
             /* Hopefully this is optimised for the common case - not clipping */
-            amp16 = (SFLDataFormat) amp;
+            amp16 = (SFLAudioSample) amp;
 
             if (amp == amp16)
                 return amp16;
@@ -102,8 +107,7 @@ class G722 : public sfl::AudioCodec {
             return INT16_MIN;
         }
 
-        void block4_encode(int band, int d)
-        {
+        void block4_encode(int band, int d) {
             int wd1 = 0;
             int wd2 = 0;
             int wd3 = 0;
@@ -207,8 +211,7 @@ class G722 : public sfl::AudioCodec {
 
         }
 
-        void block4_decode(int band, int d)
-        {
+        void block4_decode(int band, int d) {
             int wd1 = 0;
             int wd2 = 0;
             int wd3 = 0;
@@ -313,7 +316,7 @@ class G722 : public sfl::AudioCodec {
             decode_state_.band[band].s = saturate(decode_state_.band[band].sp + decode_state_.band[band].sz);
         }
 
-        int g722_decode(SFLDataFormat amp[], const uint8_t g722_data[], int len)
+        int g722_decode(SFLAudioSample amp[], const uint8_t g722_data[], int len)
         {
             static const int wl[8] = {-60, -30, 58, 172, 334, 538, 1198, 3042 };
             static const int rl42[16] = {0, 7, 6, 5, 4, 3, 2, 1, 7, 6, 5, 4, 3,  2, 1, 0 };
@@ -508,11 +511,11 @@ class G722 : public sfl::AudioCodec {
                 }
 
                 if (decode_state_.itu_test_mode) {
-                    amp[outlen++] = (SFLDataFormat)(rlow << 1);
-                    amp[outlen++] = (SFLDataFormat)(rhigh << 1);
+                    amp[outlen++] = (SFLAudioSample)(rlow << 1);
+                    amp[outlen++] = (SFLAudioSample)(rhigh << 1);
                 } else {
                     if (decode_state_.eight_k) {
-                        amp[outlen++] = (SFLDataFormat) rlow;
+                        amp[outlen++] = (SFLAudioSample) (rlow << 1);
                     } else {
                         /* Apply the receive QMF */
                         for (i = 0;  i < 22;  i++)
@@ -531,9 +534,9 @@ class G722 : public sfl::AudioCodec {
                             xout1 += decode_state_.x[2*i + 1]*qmf_coeffs[11 - i];
                         }
 
-                        amp[outlen++] = (SFLDataFormat)(xout1 >> 12);
+                        amp[outlen++] = (SFLAudioSample)(xout1 >> 11);
 
-                        amp[outlen++] = (SFLDataFormat)(xout2 >> 12);
+                        amp[outlen++] = (SFLAudioSample)(xout2 >> 11);
                     }
                 }
             }
@@ -541,7 +544,7 @@ class G722 : public sfl::AudioCodec {
             return outlen;
         }
 
-        int g722_encode(uint8_t g722_data[], const SFLDataFormat amp[], int len)
+        int g722_encode(uint8_t g722_data[], const SFLAudioSample amp[], int len)
         {
             static const int q6[32] = {
                 0,   35,   72,  110,  150,  190,  233,  276,
@@ -625,7 +628,7 @@ class G722 : public sfl::AudioCodec {
                         xhigh = amp[j++] >> 1;
                 } else {
                     if (encode_state_.eight_k) {
-                        xlow = amp[j++];
+                        xlow = amp[j++] >> 1;
                     } else {
                         /* Apply the transmit QMF */
                         /* Shuffle the buffer down */
@@ -646,9 +649,9 @@ class G722 : public sfl::AudioCodec {
                             sumeven += encode_state_.x[2*i + 1]*qmf_coeffs[11 - i];
                         }
 
-                        xlow = (sumeven + sumodd) >> 13;
+                        xlow = (sumeven + sumodd) >> 14;
 
-                        xhigh = (sumeven - sumodd) >> 13;
+                        xhigh = (sumeven - sumodd) >> 14;
                     }
                 }
 
