@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *
@@ -30,6 +30,7 @@
  */
 
 #include "global.h"
+#include "sfl_types.h"
 #include "audiocodec.h"
 #include "noncopyable.h"
 #include "array_size.h"
@@ -45,7 +46,8 @@ public:
             speex_enc_bits_(),
             speex_dec_state_(0),
             speex_enc_state_(0),
-            speex_frame_size_(0) {
+            speex_frame_size_(0),
+            mode_(mode) {
                 assert(payload >= 110 && payload <= 112);
                 assert(110 == PAYLOAD_CODEC_SPEEX_8000 &&
                        111 == PAYLOAD_CODEC_SPEEX_16000 &&
@@ -56,11 +58,11 @@ public:
 
                 // Init the decoder struct
                 speex_bits_init(&speex_dec_bits_);
-                speex_dec_state_ = speex_decoder_init(mode);
+                speex_dec_state_ = speex_decoder_init(mode_);
 
                 // Init the encoder struct
                 speex_bits_init(&speex_enc_bits_);
-                speex_enc_state_ = speex_encoder_init(mode);
+                speex_enc_state_ = speex_encoder_init(mode_);
 
                 speex_encoder_ctl(speex_enc_state_, SPEEX_SET_SAMPLING_RATE, &clockRate_);
                 speex_decoder_ctl(speex_dec_state_, SPEEX_GET_FRAME_SIZE, &speex_frame_size_);
@@ -81,15 +83,21 @@ public:
 
 private:
 
+        AudioCodec *
+        clone()
+        {
+            return new Speex(payload_, clockRate_, frameSize_, bitrate_, hasDynamicPayload_, mode_);
+        }
+
         NON_COPYABLE(Speex);
 
-        virtual int decode(short *dst, unsigned char *src, size_t buf_size) {
+        virtual int decode(SFLAudioSample *dst, unsigned char *src, size_t buf_size) {
             speex_bits_read_from(&speex_dec_bits_, (char*) src, buf_size);
             speex_decode_int(speex_dec_state_, &speex_dec_bits_, dst);
             return frameSize_;
         }
 
-        virtual int encode(unsigned char *dst, short *src, size_t buf_size) {
+        virtual int encode(unsigned char *dst, SFLAudioSample *src, size_t buf_size) {
             speex_bits_reset(&speex_enc_bits_);
             speex_encode_int(speex_enc_state_, src, &speex_enc_bits_);
             return speex_bits_write(&speex_enc_bits_, (char*) dst, buf_size);
@@ -100,4 +108,5 @@ private:
         void *speex_dec_state_;
         void *speex_enc_state_;
         int speex_frame_size_;
+        const SpeexMode *mode_;
 };
