@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -38,20 +38,19 @@ AudioStream::AudioStream(pa_context *c,
                          const char *desc,
                          int type,
                          unsigned samplrate,
-                         const std::string &deviceName)
+                         const PaDeviceInfos* infos)
     : audiostream_(0), mainloop_(m)
 {
-    static const pa_channel_map channel_map = {
-        1,
-        { PA_CHANNEL_POSITION_MONO },
-    };
+    const pa_channel_map channel_map = infos->channel_map;
 
     pa_sample_spec sample_spec = {
         PA_SAMPLE_S16LE, // PA_SAMPLE_FLOAT32LE,
         samplrate,
-        1
+        channel_map.channels
     };
 
+    DEBUG("%s: trying to create stream with device %s (%dHz, %d channels)", desc, infos->name.c_str(), samplrate, channel_map.channels);
+
     assert(pa_sample_spec_valid(&sample_spec));
     assert(pa_channel_map_valid(&channel_map));
 
@@ -69,31 +68,30 @@ AudioStream::AudioStream(pa_context *c,
     attributes.fragsize = pa_usec_to_bytes(80 * PA_USEC_PER_MSEC, &sample_spec);
     attributes.minreq = (uint32_t) -1;
 
-    pa_threaded_mainloop_lock(mainloop_);
-    const pa_stream_flags_t flags = static_cast<pa_stream_flags_t>(PA_STREAM_ADJUST_LATENCY |
-                                                                   PA_STREAM_AUTO_TIMING_UPDATE);
+    {
+        PulseMainLoopLock lock(mainloop_);
+        const pa_stream_flags_t flags = static_cast<pa_stream_flags_t>(PA_STREAM_ADJUST_LATENCY | PA_STREAM_AUTO_TIMING_UPDATE);
 
-    if (type == PLAYBACK_STREAM || type == RINGTONE_STREAM) {
-        pa_stream_connect_playback(audiostream_,
-                                   deviceName.empty() ? NULL : deviceName.c_str(),
-                                   &attributes,
-                                   flags,
-                                   NULL, NULL);
-    } else if (type == CAPTURE_STREAM) {
-        pa_stream_connect_record(audiostream_,
-                                 deviceName.empty() ? NULL : deviceName.c_str(),
-                                 &attributes,
-                                 flags);
+        if (type == PLAYBACK_STREAM || type == RINGTONE_STREAM) {
+            pa_stream_connect_playback(audiostream_,
+                    infos->name.empty() ? NULL : infos->name.c_str(),
+                    &attributes,
+                    flags,
+                    NULL, NULL);
+        } else if (type == CAPTURE_STREAM) {
+            pa_stream_connect_record(audiostream_,
+                    infos->name.empty() ? NULL : infos->name.c_str(),
+                    &attributes,
+                    flags);
+        }
     }
 
-    pa_threaded_mainloop_unlock(mainloop_);
-
     pa_stream_set_state_callback(audiostream_, stream_state_callback, NULL);
 }
 
 AudioStream::~AudioStream()
 {
-    pa_threaded_mainloop_lock(mainloop_);
+    PulseMainLoopLock lock(mainloop_);
 
     pa_stream_disconnect(audiostream_);
 
@@ -104,8 +102,6 @@ AudioStream::~AudioStream()
     pa_stream_set_overflow_callback(audiostream_, NULL, NULL);
 
     pa_stream_unref(audiostream_);
-
-    pa_threaded_mainloop_unlock(mainloop_);
 }
 
 void
