@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -34,6 +34,7 @@
 #include <pulse/pulseaudio.h>
 #include <string>
 #include "noncopyable.h"
+#include "pulselayer.h"
 
 /**
  * This data structure contains the different king of audio streams available
@@ -53,9 +54,9 @@ class AudioStream {
          * @param description
          * @param types
          * @param audio sampling rate
-         * @param device name
+         * @param pointer to pa_source_info or pa_sink_info (depending on type).
          */
-        AudioStream(pa_context *, pa_threaded_mainloop *, const char *, int, unsigned, const std::string&);
+        AudioStream(pa_context *, pa_threaded_mainloop *, const char *, int, unsigned, const PaDeviceInfos*);
 
         ~AudioStream();
 
@@ -67,6 +68,18 @@ class AudioStream {
             return audiostream_;
         }
 
+        const pa_sample_spec *sampleSpec() {
+            return pa_stream_get_sample_spec(audiostream_);
+        }
+
+        inline size_t sampleSize() {
+            return pa_sample_size(sampleSpec());
+        }
+
+        inline uint8_t channels() {
+            return sampleSpec()->channels;
+        }
+
         bool isReady();
 
     private:
