@@ -1,8 +1,9 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *  Author: Андрей Лухнов <aol.nnov@gmail.com>
+ *  Author: Adrien Beraud <adrien.beraud@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,6 +33,7 @@
 
 #include <algorithm> // for std::find
 #include <stdexcept>
+
 #include "audiostream.h"
 #include "pulselayer.h"
 #include "audio/samplerateconverter.h"
@@ -39,6 +41,7 @@
 #include "logger.h"
 #include "manager.h"
 
+#include <unistd.h>
 #include <cstdlib>
 #include <fstream>
 
@@ -66,76 +69,109 @@ void stream_moved_callback(pa_stream *s, void *userdata UNUSED)
 
 } // end anonymous namespace
 
-#ifdef RECTODISK
-std::ofstream outfileResampled ("testMicOuputResampled.raw", std::ifstream::binary);
-std::ofstream outfile("testMicOuput.raw", std::ifstream::binary);
-#endif
+PulseMainLoopLock::PulseMainLoopLock(pa_threaded_mainloop *loop) : loop_(loop), destroyLoop_(false)
+{
+    pa_threaded_mainloop_lock(loop_);
+}
+
+// set this flag if we want the loop to be destroyed once it's unlocked
+void PulseMainLoopLock::destroyLoop()
+{
+    destroyLoop_ = true;
+}
+
+PulseMainLoopLock::~PulseMainLoopLock()
+{
+    pa_threaded_mainloop_unlock(loop_);
+    if (destroyLoop_) {
+        pa_threaded_mainloop_stop(loop_);
+        pa_threaded_mainloop_free(loop_);
+    }
+}
 
 PulseLayer::PulseLayer(AudioPreference &pref)
-    : playback_(0)
+    : AudioLayer(pref)
+    , playback_(0)
     , record_(0)
     , ringtone_(0)
     , sinkList_()
     , sourceList_()
-    , mic_buffer_(0)
-    , mic_buf_size_(0)
+    , micBuffer_(0, 1, 8000)
     , context_(0)
     , mainloop_(pa_threaded_mainloop_new())
     , enumeratingSinks_(false)
     , enumeratingSources_(false)
     , preference_(pref)
 {
-    setenv("PULSE_PROP_media.role", "phone", 1);
+    setCaptureGain(pref.getVolumemic());
+    setPlaybackGain(pref.getVolumespkr());
+    muteCapture(pref.getCaptureMuted());
 
     if (!mainloop_)
         throw std::runtime_error("Couldn't create pulseaudio mainloop");
 
-    context_ = pa_context_new(pa_threaded_mainloop_get_api(mainloop_) , "SFLphone");
+    if (pa_threaded_mainloop_start(mainloop_) < 0) {
+        pa_threaded_mainloop_free(mainloop_);
+        throw std::runtime_error("Failed to start pulseaudio mainloop");
+    }
 
-    if (!context_)
+    PulseMainLoopLock lock(mainloop_);
+
+#if PA_CHECK_VERSION(1, 0, 0)
+    pa_proplist *pl = pa_proplist_new();
+    pa_proplist_sets(pl, PA_PROP_MEDIA_ROLE, "phone");
+
+    context_ = pa_context_new_with_proplist(pa_threaded_mainloop_get_api(mainloop_), "SFLphone", pl);
+
+    if (pl)
+        pa_proplist_free(pl);
+
+#else
+    setenv("PULSE_PROP_media.role", "phone", 1);
+    context_ = pa_context_new(pa_threaded_mainloop_get_api(mainloop_), "SFLphone");
+#endif
+
+    if (!context_) {
+        lock.destroyLoop();
         throw std::runtime_error("Couldn't create pulseaudio context");
+    }
 
     pa_context_set_state_callback(context_, context_state_callback, this);
 
-    if (pa_context_connect(context_, NULL , PA_CONTEXT_NOAUTOSPAWN , NULL) < 0)
+    if (pa_context_connect(context_, nullptr , PA_CONTEXT_NOAUTOSPAWN , nullptr) < 0) {
+        lock.destroyLoop();
         throw std::runtime_error("Could not connect pulseaudio context to the server");
+    }
 
-    pa_threaded_mainloop_lock(mainloop_);
-
-    if (pa_threaded_mainloop_start(mainloop_) < 0)
-        throw std::runtime_error("Failed to start pulseaudio mainloop");
-
-    pa_threaded_mainloop_wait(mainloop_);
-
-    if (pa_context_get_state(context_) != PA_CONTEXT_READY)
-        throw std::runtime_error("Couldn't connect to pulse audio server");
-
-    pa_threaded_mainloop_unlock(mainloop_);
+    // wait until context is ready
+    for (;;) {
+        pa_context_state_t context_state = pa_context_get_state(context_);
+        if (not PA_CONTEXT_IS_GOOD(context_state)) {
+            lock.destroyLoop();
+            throw std::runtime_error("Pulse audio context is bad");
+        }
+        if (context_state == PA_CONTEXT_READY)
+            break;
+        pa_threaded_mainloop_wait(mainloop_);
+    }
 
     isStarted_ = true;
 }
 
 PulseLayer::~PulseLayer()
 {
-#ifdef RECTODISK
-    outfile.close();
-    outfileResampled.close();
-#endif
-
     disconnectAudioStream();
 
-    if (mainloop_)
-        pa_threaded_mainloop_stop(mainloop_);
+    {
+        PulseMainLoopLock lock(mainloop_);
 
-    if (context_) {
+        pa_context_set_state_callback(context_, NULL, NULL);
         pa_context_disconnect(context_);
         pa_context_unref(context_);
     }
 
-    if (mainloop_)
-        pa_threaded_mainloop_free(mainloop_);
-
-    delete [] mic_buffer_;
+    pa_threaded_mainloop_stop(mainloop_);
+    pa_threaded_mainloop_free(mainloop_);
 }
 
 void PulseLayer::context_state_callback(pa_context* c, void *user_data)
@@ -143,7 +179,7 @@ void PulseLayer::context_state_callback(pa_context* c, void *user_data)
     PulseLayer *pulse = static_cast<PulseLayer*>(user_data);
     assert(c and pulse and pulse->mainloop_);
     const pa_subscription_mask_t mask = (pa_subscription_mask_t)
-        (PA_SUBSCRIPTION_MASK_SINK | PA_SUBSCRIPTION_MASK_SOURCE);
+                                        (PA_SUBSCRIPTION_MASK_SINK | PA_SUBSCRIPTION_MASK_SOURCE);
 
     switch (pa_context_get_state(c)) {
         case PA_CONTEXT_CONNECTING:
@@ -155,7 +191,7 @@ void PulseLayer::context_state_callback(pa_context* c, void *user_data)
         case PA_CONTEXT_READY:
             DEBUG("Connection to PulseAudio server established");
             pa_threaded_mainloop_signal(pulse->mainloop_, 0);
-            pa_context_subscribe(c, mask, NULL, pulse);
+            pa_context_subscribe(c, mask, nullptr, pulse);
             pa_context_set_subscribe_callback(c, context_changed_callback, pulse);
             pulse->updateSinkList();
             pulse->updateSourceList();
@@ -168,7 +204,6 @@ void PulseLayer::context_state_callback(pa_context* c, void *user_data)
         default:
             ERROR("%s" , pa_strerror(pa_context_errno(c)));
             pa_threaded_mainloop_signal(pulse->mainloop_, 0);
-            pulse->disconnectAudioStream();
             break;
     }
 }
@@ -179,7 +214,8 @@ void PulseLayer::updateSinkList()
     sinkList_.clear();
     enumeratingSinks_ = true;
     pa_operation *op = pa_context_get_sink_info_list(context_, sink_input_info_callback, this);
-    if (op != NULL)
+
+    if (op != nullptr)
         pa_operation_unref(op);
 }
 
@@ -188,47 +224,72 @@ void PulseLayer::updateSourceList()
     sourceList_.clear();
     enumeratingSources_ = true;
     pa_operation *op = pa_context_get_source_info_list(context_, source_input_info_callback, this);
-    if (op != NULL)
+
+    if (op != nullptr)
         pa_operation_unref(op);
 }
 
-bool PulseLayer::inSinkList(const std::string &deviceName) const
+bool PulseLayer::inSinkList(const std::string &deviceName)
 {
-    const bool found = std::find(sinkList_.begin(), sinkList_.end(), deviceName) != sinkList_.end();
+    const bool found = std::find_if(sinkList_.begin(), sinkList_.end(), PaDeviceInfos::nameComparator(deviceName)) != sinkList_.end();
+
     DEBUG("seeking for %s in sinks. %s found", deviceName.c_str(), found ? "" : "NOT");
     return found;
 }
 
-
-bool PulseLayer::inSourceList(const std::string &deviceName) const
+bool PulseLayer::inSourceList(const std::string &deviceName)
 {
-    const bool found = std::find(sourceList_.begin(), sourceList_.end(), deviceName) != sourceList_.end();
+    const bool found = std::find_if(sourceList_.begin(), sourceList_.end(), PaDeviceInfos::nameComparator(deviceName)) != sourceList_.end();
+
     DEBUG("seeking for %s in sources. %s found", deviceName.c_str(), found ? "" : "NOT");
     return found;
 }
 
 std::vector<std::string> PulseLayer::getCaptureDeviceList() const
 {
-    return sourceList_;
+    const unsigned n = sourceList_.size();
+    std::vector<std::string> names(n);
+
+    for (unsigned i = 0; i < n; i++)
+        names[i] = sourceList_[i].name;
+
+    return names;
 }
 
 std::vector<std::string> PulseLayer::getPlaybackDeviceList() const
 {
-    return sinkList_;
+    const unsigned n = sinkList_.size();
+    std::vector<std::string> names(n);
+
+    for (unsigned i = 0; i < n; i++)
+        names[i] = sinkList_[i].name;
+
+    return names;
 }
 
 int PulseLayer::getAudioDeviceIndex(const std::string& name) const
 {
-    int index = std::distance(sourceList_.begin(), std::find(sourceList_.begin(), sourceList_.end(), name));
+    int index = std::distance(sourceList_.begin(), std::find_if(sourceList_.begin(), sourceList_.end(), PaDeviceInfos::nameComparator(name)));
+
     if (index == std::distance(sourceList_.begin(), sourceList_.end())) {
-        // not found in sources, search in sinks then
-        index = std::distance(sinkList_.begin(), std::find(sinkList_.begin(), sinkList_.end(), name));
+        index = std::distance(sinkList_.begin(), std::find_if(sinkList_.begin(), sinkList_.end(), PaDeviceInfos::nameComparator(name)));
     }
+
     return index;
 }
 
+const PaDeviceInfos* PulseLayer::getDeviceInfos(const std::vector<PaDeviceInfos>& list, const std::string& name) const
+{
+    std::vector<PaDeviceInfos>::const_iterator dev_info = std::find_if(list.begin(), list.end(), PaDeviceInfos::nameComparator(name));
+
+    if (dev_info == list.end()) return nullptr;
+
+    return &(*dev_info);
+}
+
 std::string PulseLayer::getAudioDeviceName(int index, PCMType type) const
 {
+
     switch (type) {
         case SFL_PCM_PLAYBACK:
         case SFL_PCM_RINGTONE:
@@ -236,13 +297,17 @@ std::string PulseLayer::getAudioDeviceName(int index, PCMType type) const
                 ERROR("Index %d out of range", index);
                 return "";
             }
-            return sinkList_[index];
+
+            return sinkList_[index].name;
+
         case SFL_PCM_CAPTURE:
             if (index < 0 or static_cast<size_t>(index) >= sourceList_.size()) {
                 ERROR("Index %d out of range", index);
                 return "";
             }
-            return sourceList_[index];
+
+            return sourceList_[index].name;
+
         default:
             return "";
     }
@@ -258,23 +323,44 @@ void PulseLayer::createStreams(pa_context* c)
     std::string ringtoneDevice(preference_.getPulseDeviceRingtone());
     std::string defaultDevice = "";
 
-    DEBUG("Devices:\n   playback: %s\n   record: %s\n   ringtone: %s",
-           playbackDevice.c_str(), captureDevice.c_str(), ringtoneDevice.c_str());
+    DEBUG("Devices: playback: %s record: %s ringtone: %s",
+          playbackDevice.c_str(), captureDevice.c_str(), ringtoneDevice.c_str());
 
-    playback_ = new AudioStream(c, mainloop_, "SFLphone playback", PLAYBACK_STREAM, sampleRate_,
-                                inSinkList(playbackDevice) ? playbackDevice : defaultDevice);
+    // Create playback stream
+    const PaDeviceInfos* dev_infos = getDeviceInfos(sinkList_, playbackDevice);
+
+    if (dev_infos == nullptr) {
+        dev_infos = &sinkList_[0];
+        DEBUG("Prefered playback device not found in device list, selecting %s instead.", dev_infos->name.c_str());
+    }
+
+    playback_ = new AudioStream(c, mainloop_, "SFLphone playback", PLAYBACK_STREAM, sampleRate_, dev_infos);
 
     pa_stream_set_write_callback(playback_->pulseStream(), playback_callback, this);
     pa_stream_set_moved_callback(playback_->pulseStream(), stream_moved_callback, this);
 
-    record_ = new AudioStream(c, mainloop_, "SFLphone capture", CAPTURE_STREAM, sampleRate_,
-                              inSourceList(captureDevice) ? captureDevice : defaultDevice);
+    // Create capture stream
+    dev_infos = getDeviceInfos(sourceList_, captureDevice);
+
+    if (dev_infos == nullptr) {
+        dev_infos = &sourceList_[0];
+        DEBUG("Prefered capture device not found in device list, selecting %s instead.", dev_infos->name.c_str());
+    }
+
+    record_ = new AudioStream(c, mainloop_, "SFLphone capture", CAPTURE_STREAM, sampleRate_, dev_infos);
 
     pa_stream_set_read_callback(record_->pulseStream() , capture_callback, this);
     pa_stream_set_moved_callback(record_->pulseStream(), stream_moved_callback, this);
 
-    ringtone_ = new AudioStream(c, mainloop_, "SFLphone ringtone", RINGTONE_STREAM, sampleRate_,
-                                inSinkList(ringtoneDevice) ? ringtoneDevice : defaultDevice);
+    // Create ringtone stream
+    dev_infos = getDeviceInfos(sinkList_, ringtoneDevice);
+
+    if (dev_infos == nullptr) {
+        dev_infos = &sinkList_[0];
+        DEBUG("Prefered ringtone device not found in device list, selecting %s instead.", dev_infos->name.c_str());
+    }
+
+    ringtone_ = new AudioStream(c, mainloop_, "SFLphone ringtone", RINGTONE_STREAM, sampleRate_, dev_infos);
 
     pa_stream_set_write_callback(ringtone_->pulseStream(), ringtone_callback, this);
     pa_stream_set_moved_callback(ringtone_->pulseStream(), stream_moved_callback, this);
@@ -286,13 +372,13 @@ void PulseLayer::createStreams(pa_context* c)
 }
 
 namespace {
-    // Delete stream and zero out its pointer
-    void
-    cleanupStream(AudioStream *&stream)
-    {
-        delete stream;
-        stream = 0;
-    }
+// Delete stream and zero out its pointer
+void
+cleanupStream(AudioStream *&stream)
+{
+    delete stream;
+    stream = 0;
+}
 }
 
 
@@ -319,15 +405,15 @@ void PulseLayer::startStream()
 void
 PulseLayer::stopStream()
 {
-    pa_threaded_mainloop_lock(mainloop_);
+    {
+        PulseMainLoopLock lock(mainloop_);
 
-    if (playback_)
-        pa_stream_flush(playback_->pulseStream(), NULL, NULL);
+        if (playback_)
+            pa_stream_flush(playback_->pulseStream(), nullptr, nullptr);
 
-    if (record_)
-        pa_stream_flush(record_->pulseStream(), NULL, NULL);
-
-    pa_threaded_mainloop_unlock(mainloop_);
+        if (record_)
+            pa_stream_flush(record_->pulseStream(), nullptr, nullptr);
+    }
 
     disconnectAudioStream();
 }
@@ -338,6 +424,9 @@ void PulseLayer::writeToSpeaker()
         return;
 
     pa_stream *s = playback_->pulseStream();
+    const pa_sample_spec* sample_spec = pa_stream_get_sample_spec(s);
+    size_t sample_size = pa_frame_size(sample_spec);
+    const unsigned n_channels = sample_spec->channels;
 
     // available bytes to be written in pulseaudio internal buffer
     int ret = pa_stream_writable_size(s);
@@ -349,22 +438,29 @@ void PulseLayer::writeToSpeaker()
         return;
 
     size_t writableBytes = ret;
+    const size_t writableSamples = writableBytes / sample_size;
 
     notifyIncomingCall();
 
-    size_t urgentBytes = urgentRingBuffer_.availableForGet(MainBuffer::DEFAULT_ID);
+    size_t urgentSamples = urgentRingBuffer_.availableForGet(MainBuffer::DEFAULT_ID);
+    size_t urgentBytes = urgentSamples * sample_size;
 
-    if (urgentBytes > writableBytes)
-        urgentBytes = writableBytes;
+    if (urgentSamples > writableSamples) {
+        urgentSamples = writableSamples;
+        urgentBytes = urgentSamples * sample_size;
+    }
+
+    SFLAudioSample *data = 0;
 
-    void *data = 0;
     if (urgentBytes) {
-        pa_stream_begin_write(s, &data, &urgentBytes);
-        urgentRingBuffer_.get(data, urgentBytes, MainBuffer::DEFAULT_ID);
-        applyGain(static_cast<SFLDataFormat *>(data), urgentBytes / sizeof(SFLDataFormat), getPlaybackGain());
-        pa_stream_write(s, data, urgentBytes, NULL, 0, PA_SEEK_RELATIVE);
-        // Consume the regular one as well (same amount of bytes)
-        Manager::instance().getMainBuffer().discard(urgentBytes, MainBuffer::DEFAULT_ID);
+        AudioBuffer linearbuff(urgentSamples, n_channels, 8000);
+        pa_stream_begin_write(s, (void**)&data, &urgentBytes);
+        urgentRingBuffer_.get(linearbuff, MainBuffer::DEFAULT_ID); // retrive only the first sample_spec->channels channels
+        linearbuff.applyGain(isPlaybackMuted_ ? 0.0 : playbackGain_);
+        linearbuff.interleave(data);
+        pa_stream_write(s, data, urgentBytes, nullptr, 0, PA_SEEK_RELATIVE);
+        // Consume the regular one as well (same amount of samples)
+        Manager::instance().getMainBuffer().discard(urgentSamples, MainBuffer::DEFAULT_ID);
         return;
     }
 
@@ -374,10 +470,11 @@ void PulseLayer::writeToSpeaker()
 
     if (toneToPlay) {
         if (playback_->isReady()) {
-            pa_stream_begin_write(s, &data, &writableBytes);
-            toneToPlay->getNext((SFLDataFormat*)data, writableBytes / sizeof(SFLDataFormat), 100);
-            applyGain(static_cast<SFLDataFormat *>(data), writableBytes / sizeof(SFLDataFormat), getPlaybackGain());
-            pa_stream_write(s, data, writableBytes, NULL, 0, PA_SEEK_RELATIVE);
+            pa_stream_begin_write(s, (void**)&data, &writableBytes);
+            AudioBuffer linearbuff(writableSamples, n_channels, 8000);
+            toneToPlay->getNext(linearbuff, playbackGain_); // retrive only n_channels
+            linearbuff.interleave(data);
+            pa_stream_write(s, data, writableBytes, nullptr, 0, PA_SEEK_RELATIVE);
         }
 
         return;
@@ -385,18 +482,15 @@ void PulseLayer::writeToSpeaker()
 
     flushUrgent(); // flush remaining samples in _urgentRingBuffer
 
-    size_t availSamples = Manager::instance().getMainBuffer().availableForGet(MainBuffer::DEFAULT_ID) / sizeof(SFLDataFormat);
+    size_t availSamples = Manager::instance().getMainBuffer().availableForGet(MainBuffer::DEFAULT_ID);
 
     if (availSamples == 0) {
-        pa_stream_begin_write(s, &data, &writableBytes);
+        pa_stream_begin_write(s, (void**)&data, &writableBytes);
         memset(data, 0, writableBytes);
-        pa_stream_write(s, data, writableBytes, NULL, 0, PA_SEEK_RELATIVE);
+        pa_stream_write(s, data, writableBytes, nullptr, 0, PA_SEEK_RELATIVE);
         return;
     }
 
-    // how many samples we can write to the output
-    size_t writableSamples = writableBytes / sizeof(SFLDataFormat);
-
     // how many samples we want to read from the buffer
     size_t readableSamples = writableSamples;
 
@@ -404,30 +498,31 @@ void PulseLayer::writeToSpeaker()
 
     unsigned int mainBufferSampleRate = Manager::instance().getMainBuffer().getInternalSamplingRate();
     bool resample = sampleRate_ != mainBufferSampleRate;
+
     if (resample) {
         resampleFactor = (double) sampleRate_ / mainBufferSampleRate;
         readableSamples = (double) readableSamples / resampleFactor;
     }
 
-    if (readableSamples > availSamples)
-        readableSamples = availSamples;
+    readableSamples = std::min(readableSamples, availSamples);
+    size_t nResampled = (double) readableSamples * resampleFactor;
+    size_t resampledBytes =  nResampled * sample_size;
 
-    size_t readableBytes = readableSamples * sizeof(SFLDataFormat);
-    pa_stream_begin_write(s, &data, &readableBytes);
-    Manager::instance().getMainBuffer().getData(data, readableBytes, MainBuffer::DEFAULT_ID);
+    pa_stream_begin_write(s, (void**)&data, &resampledBytes);
+
+    AudioBuffer linearbuff(readableSamples, n_channels, 8000);
+    Manager::instance().getMainBuffer().getData(linearbuff, MainBuffer::DEFAULT_ID);
 
     if (resample) {
-        const size_t nResampled = (double) readableSamples * resampleFactor;
-        size_t resampledBytes =  nResampled * sizeof(SFLDataFormat);
-        SFLDataFormat* rsmpl_out = (SFLDataFormat*) pa_xmalloc(resampledBytes);
-        converter_.resample((SFLDataFormat*)data, rsmpl_out, nResampled,
-                             mainBufferSampleRate, sampleRate_, readableSamples);
-        applyGain(rsmpl_out, nResampled, getPlaybackGain());
-        pa_stream_write(s, rsmpl_out, resampledBytes, NULL, 0, PA_SEEK_RELATIVE);
-        pa_xfree(rsmpl_out);
+        AudioBuffer rsmpl_out(nResampled, 1, sampleRate_);
+        converter_.resample(linearbuff, rsmpl_out);
+        rsmpl_out.applyGain(isPlaybackMuted_ ? 0.0 : playbackGain_);
+        rsmpl_out.interleave(data);
+        pa_stream_write(s, data, resampledBytes, nullptr, 0, PA_SEEK_RELATIVE);
     } else {
-        applyGain(static_cast<SFLDataFormat *>(data), readableSamples, getPlaybackGain());
-        pa_stream_write(s, data, readableBytes, NULL, 0, PA_SEEK_RELATIVE);
+        linearbuff.applyGain(isPlaybackMuted_ ? 0.0 : playbackGain_);
+        linearbuff.interleave(data);
+        pa_stream_write(s, data, resampledBytes, nullptr, 0, PA_SEEK_RELATIVE);
     }
 }
 
@@ -436,41 +531,38 @@ void PulseLayer::readFromMic()
     if (!record_ or !record_->isReady())
         return;
 
-    const char *data = NULL;
+    const char *data = nullptr;
     size_t bytes;
 
+    const size_t sample_size = record_->sampleSize();
+    const uint8_t channels = record_->channels();
+
     if (pa_stream_peek(record_->pulseStream() , (const void**) &data , &bytes) < 0 or !data)
         return;
 
+    assert(channels);
+    assert(sample_size);
+    const size_t samples = bytes / sample_size / channels;
+
+    AudioBuffer in(samples, channels, sampleRate_);
+    in.deinterleave((SFLAudioSample*)data, samples, channels);
+
     unsigned int mainBufferSampleRate = Manager::instance().getMainBuffer().getInternalSamplingRate();
     bool resample = sampleRate_ != mainBufferSampleRate;
 
+    in.applyGain(isCaptureMuted_ ? 0.0 : captureGain_);
+
+    AudioBuffer * out = &in;
+
     if (resample) {
-        double resampleFactor = (double) sampleRate_ / mainBufferSampleRate;
-        bytes = (double) bytes * resampleFactor;
+        micBuffer_.setSampleRate(mainBufferSampleRate);
+        converter_.resample(in, micBuffer_);
+        out = &micBuffer_;
     }
 
-    size_t samples = bytes / sizeof(SFLDataFormat);
-
-    if (bytes > mic_buf_size_) {
-        mic_buf_size_ = bytes;
-        delete [] mic_buffer_;
-        mic_buffer_ = new SFLDataFormat[samples];
-    }
-
-#ifdef RECTODISK
-    outfile.write((const char *)data, bytes);
-#endif
-    if (resample) {
-        converter_.resample((SFLDataFormat*)data, mic_buffer_, samples, mainBufferSampleRate, sampleRate_, samples);
-    }
-
-    dcblocker_.process(mic_buffer_, (SFLDataFormat*)data, samples);
-    applyGain(mic_buffer_, bytes / sizeof(SFLDataFormat), getCaptureGain());
-    Manager::instance().getMainBuffer().putData(mic_buffer_, bytes, MainBuffer::DEFAULT_ID);
-#ifdef RECTODISK
-    outfileResampled.write((const char *)mic_buffer_, bytes);
-#endif
+    dcblocker_.process(*out);
+    out->applyGain(isPlaybackMuted_ ? 0.0 : playbackGain_);
+    Manager::instance().getMainBuffer().putData(*out, MainBuffer::DEFAULT_ID);
 
     if (pa_stream_drop(record_->pulseStream()) < 0)
         ERROR("Capture stream drop failed: %s" , pa_strerror(pa_context_errno(context_)));
@@ -483,6 +575,7 @@ void PulseLayer::ringtoneToSpeaker()
         return;
 
     pa_stream *s = ringtone_->pulseStream();
+    size_t sample_size = ringtone_->sampleSize();
 
     int writable = pa_stream_writable_size(s);
 
@@ -499,13 +592,15 @@ void PulseLayer::ringtoneToSpeaker()
     AudioLoop *fileToPlay = Manager::instance().getTelephoneFile();
 
     if (fileToPlay) {
-        fileToPlay->getNext((SFLDataFormat *) data, bytes / sizeof(SFLDataFormat), 100);
-        applyGain(static_cast<SFLDataFormat *>(data), bytes / sizeof(SFLDataFormat), getPlaybackGain());
-    }
-    else
+        const unsigned samples = (bytes / sample_size) / ringtone_->channels();
+        AudioBuffer tmp(samples, ringtone_->channels(), 8000);
+        fileToPlay->getNext(tmp, playbackGain_);
+        tmp.interleave((SFLAudioSample*) data);
+    } else {
         memset(data, 0, bytes);
+    }
 
-    pa_stream_write(s, data, bytes, NULL, 0, PA_SEEK_RELATIVE);
+    pa_stream_write(s, data, bytes, nullptr, 0, PA_SEEK_RELATIVE);
 }
 
 void
@@ -514,8 +609,9 @@ PulseLayer::context_changed_callback(pa_context* c,
                                      uint32_t idx UNUSED, void *userdata)
 {
     PulseLayer *context = static_cast<PulseLayer*>(userdata);
+
     switch (type & PA_SUBSCRIPTION_EVENT_FACILITY_MASK) {
-        pa_operation *op;
+            pa_operation *op;
 
         case PA_SUBSCRIPTION_EVENT_SINK:
             switch (type & PA_SUBSCRIPTION_EVENT_TYPE_MASK) {
@@ -524,11 +620,14 @@ PulseLayer::context_changed_callback(pa_context* c,
                     DEBUG("Updating sink list");
                     context->sinkList_.clear();
                     op = pa_context_get_sink_info_list(c, sink_input_info_callback, userdata);
-                    if (op != NULL)
+
+                    if (op != nullptr)
                         pa_operation_unref(op);
+
                 default:
                     break;
             }
+
             break;
 
         case PA_SUBSCRIPTION_EVENT_SOURCE:
@@ -538,12 +637,16 @@ PulseLayer::context_changed_callback(pa_context* c,
                     DEBUG("Updating source list");
                     context->sourceList_.clear();
                     op = pa_context_get_source_info_list(c, source_input_info_callback, userdata);
-                    if (op != NULL)
+
+                    if (op != nullptr)
                         pa_operation_unref(op);
+
                 default:
                     break;
             }
+
             break;
+
         default:
             DEBUG("Unhandled event type 0x%x", type);
             break;
@@ -562,32 +665,34 @@ void PulseLayer::source_input_info_callback(pa_context *c UNUSED, const pa_sourc
     }
 
     DEBUG("Source %u\n"
-           "    Name: %s\n"
-           "    Driver: %s\n"
-           "    Description: %s\n"
-           "    Sample Specification: %s\n"
-           "    Channel Map: %s\n"
-           "    Owner Module: %u\n"
-           "    Volume: %s\n"
-           "    Monitor if Sink: %u\n"
-           "    Latency: %0.0f usec\n"
-           "    Flags: %s%s%s\n",
-           i->index,
-           i->name,
-           i->driver,
-           i->description,
-           pa_sample_spec_snprint(s, sizeof(s), &i->sample_spec),
-           pa_channel_map_snprint(cm, sizeof(cm), &i->channel_map),
-           i->owner_module,
-           i->mute ? "muted" : pa_cvolume_snprint(cv, sizeof(cv), &i->volume),
-           i->monitor_of_sink,
-           (double) i->latency,
-           i->flags & PA_SOURCE_HW_VOLUME_CTRL ? "HW_VOLUME_CTRL " : "",
-           i->flags & PA_SOURCE_LATENCY ? "LATENCY " : "",
-           i->flags & PA_SOURCE_HARDWARE ? "HARDWARE" : "");
+          "    Name: %s\n"
+          "    Driver: %s\n"
+          "    Description: %s\n"
+          "    Sample Specification: %s\n"
+          "    Channel Map: %s\n"
+          "    Owner Module: %u\n"
+          "    Volume: %s\n"
+          "    Monitor if Sink: %u\n"
+          "    Latency: %0.0f usec\n"
+          "    Flags: %s%s%s\n",
+          i->index,
+          i->name,
+          i->driver,
+          i->description,
+          pa_sample_spec_snprint(s, sizeof(s), &i->sample_spec),
+          pa_channel_map_snprint(cm, sizeof(cm), &i->channel_map),
+          i->owner_module,
+          i->mute ? "muted" : pa_cvolume_snprint(cv, sizeof(cv), &i->volume),
+          i->monitor_of_sink,
+          (double) i->latency,
+          i->flags & PA_SOURCE_HW_VOLUME_CTRL ? "HW_VOLUME_CTRL " : "",
+          i->flags & PA_SOURCE_LATENCY ? "LATENCY " : "",
+          i->flags & PA_SOURCE_HARDWARE ? "HARDWARE" : "");
 
-    if (not context->inSourceList(i->name))
-        context->sourceList_.push_back(i->name);
+    if (not context->inSourceList(i->name)) {
+        PaDeviceInfos ep_infos(i->index, i->name, i->sample_spec, i->channel_map);
+        context->sourceList_.push_back(ep_infos);
+    }
 }
 
 void PulseLayer::sink_input_info_callback(pa_context *c UNUSED, const pa_sink_info *i, int eol, void *userdata)
@@ -625,26 +730,32 @@ void PulseLayer::sink_input_info_callback(pa_context *c UNUSED, const pa_sink_in
           i->flags & PA_SINK_LATENCY ? "LATENCY " : "",
           i->flags & PA_SINK_HARDWARE ? "HARDWARE" : "");
 
-    if (not context->inSinkList(i->name))
-        context->sinkList_.push_back(i->name);
+    if (not context->inSinkList(i->name)) {
+        PaDeviceInfos ep_infos(i->index, i->name, i->sample_spec, i->channel_map);
+        context->sinkList_.push_back(ep_infos);
+    }
 }
 
 void PulseLayer::updatePreference(AudioPreference &preference, int index, PCMType type)
 {
     const std::string devName(getAudioDeviceName(index, type));
+
     switch (type) {
         case SFL_PCM_PLAYBACK:
             DEBUG("setting %s for playback", devName.c_str());
             preference.setPulseDevicePlayback(devName);
             break;
+
         case SFL_PCM_CAPTURE:
             DEBUG("setting %s for capture", devName.c_str());
             preference.setPulseDeviceRecord(devName);
             break;
+
         case SFL_PCM_RINGTONE:
             DEBUG("setting %s for ringer", devName.c_str());
             preference.setPulseDeviceRingtone(devName);
             break;
+
         default:
             break;
     }
