@@ -1,8 +1,9 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *  Author: Андрей Лухнов <aol.nnov@gmail.com>
+ *  Author: Adrien Beraud <adrien.beraud@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,10 +40,50 @@
 #include <pulse/stream.h>
 #include "audio/audiolayer.h"
 #include "noncopyable.h"
+#include "logger.h"
 
 class AudioPreference;
 class AudioStream;
 
+/**
+ * Convenience structure to hold PulseAudio device propreties such as supported channel number etc.
+ */
+typedef struct PaDeviceInfos {
+        uint32_t index;
+        std::string name;
+        pa_sample_spec sample_spec;
+        pa_channel_map channel_map;
+
+        PaDeviceInfos(unsigned idx, const char* ep_name, const pa_sample_spec &samp_spec, const pa_channel_map &chan_map)
+            : index(idx), name(ep_name), sample_spec(samp_spec), channel_map(chan_map) {}
+        virtual ~PaDeviceInfos() {}
+
+        /**
+         * Unary function to search for a device by name in a list using std functions.
+         */
+        class nameComparator {
+            public:
+                explicit nameComparator(const std::string &ref) : baseline(ref) {}
+                bool operator()(const PaDeviceInfos &arg) {
+                    return arg.name == baseline;
+                }
+            private:
+                const std::string &baseline;
+        };
+} PaDeviceInfos;
+
+class PulseMainLoopLock {
+    public:
+        explicit PulseMainLoopLock(pa_threaded_mainloop *loop);
+        void destroyLoop();
+        ~PulseMainLoopLock();
+
+    private:
+        NON_COPYABLE(PulseMainLoopLock);
+        pa_threaded_mainloop *loop_;
+        bool destroyLoop_;
+};
+
 class PulseLayer : public AudioLayer {
     public:
         PulseLayer(AudioPreference &pref);
@@ -60,13 +101,14 @@ class PulseLayer : public AudioLayer {
 
         void updateSourceList();
 
-        bool inSinkList(const std::string &deviceName) const;
+        bool inSinkList(const std::string &deviceName);
 
-        bool inSourceList(const std::string &deviceName) const;
+        bool inSourceList(const std::string &deviceName);
 
         virtual std::vector<std::string> getCaptureDeviceList() const;
         virtual std::vector<std::string> getPlaybackDeviceList() const;
         int getAudioDeviceIndex(const std::string& name) const;
+
         std::string getAudioDeviceName(int index, PCMType type) const;
 
         virtual void startStream();
@@ -120,20 +162,24 @@ class PulseLayer : public AudioLayer {
         AudioStream* ringtone_;
 
         /**
-         * Contain the list of playback devices
+         * Contains the list of playback devices
          */
-        std::vector<std::string> sinkList_;
+        std::vector<PaDeviceInfos> sinkList_;
 
         /**
-         * Contain the list of capture devices
+         * Contains the list of capture devices
          */
-        std::vector<std::string> sourceList_;
+        std::vector<PaDeviceInfos> sourceList_;
+
+        /**
+         * Returns a pointer to the PaEndpointInfos with the given name in sourceList_, or nullptr if not found.
+         */
+        const PaDeviceInfos* getDeviceInfos(const std::vector<PaDeviceInfos>&, const std::string& name) const;
 
         /*
          * Buffers used to avoid doing malloc/free in the audio thread
          */
-        SFLDataFormat *mic_buffer_;
-        size_t mic_buf_size_;
+        AudioBuffer micBuffer_;
 
         /** PulseAudio context and asynchronous loop */
         pa_context* context_;
