@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  * Author: Yan Morin <yan.morin@savoirfairelinux.com>
  * Author: Laurielle Lea <laurielle.lea@savoirfairelinux.com>
  *
@@ -69,7 +69,7 @@ class DTMFGenerator {
         /** State of the DTMF generator */
         struct DTMFState {
             unsigned int offset;   /** Offset in the sample currently being played */
-            SFLDataFormat* sample;         /** Currently generated code */
+            SFLAudioSample* sample;         /** Currently generated code */
         };
 
         /** State of the DTMF generator */
@@ -79,7 +79,7 @@ class DTMFGenerator {
         static const DTMFTone tones_[NUM_TONES];
 
         /** Generated samples for each tone */
-        SFLDataFormat* toneBuffers_[NUM_TONES];
+        SFLAudioSample* toneBuffers_[NUM_TONES];
 
         /** Sampling rate of generated dtmf */
         int sampleRate_;
@@ -101,26 +101,26 @@ class DTMFGenerator {
 
         /*
          * Get n samples of the signal of code code
-         * @param buffer a SFLDataFormat vector
+         * @param buffer a SFLAudioSample vector
          * @param code   dtmf code to get sound
          */
-        void getSamples(std::vector<SFLDataFormat> &buffer, unsigned char code);
+        void getSamples(std::vector<SFLAudioSample> &buffer, unsigned char code);
 
         /*
          * Get next n samples (continues where previous call to
          * genSample or genNextSamples stopped
-         * @param buffer a SFLDataFormat vector
+         * @param buffer a SFLAudioSample vector
          */
-        void getNextSamples(std::vector<SFLDataFormat> &buffer);
+        void getNextSamples(std::vector<SFLAudioSample> &buffer);
 
     private:
 
         /**
          * Fill tone buffer for a given index of the array of tones.
          * @param index of the tone in the array tones_
-         * @return SFLDataFormat* The generated data
+         * @return SFLAudioSample* The generated data
          */
-        SFLDataFormat* fillToneBuffer(int index);
+        SFLAudioSample* fillToneBuffer(int index);
 };
 
 #endif // DTMFGENERATOR_H
