@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Yan Morin <yan.morin@savoirfairelinux.com>
  *  Author : Laurielle Lea <laurielle.lea@savoirfairelinux.com>
  *
@@ -32,7 +32,6 @@
 #include "manager.h"
 #include "audio/mainbuffer.h"
 #include "history/historyitem.h"
-#include "scoped_lock.h"
 
 Call::Call(const std::string& id, Call::CallType type, const std::string &accountID)
     : callMutex_()
@@ -51,40 +50,37 @@ Call::Call(const std::string& id, Call::CallType type, const std::string &accoun
     , timestamp_start_(0)
     , timestamp_stop_(0)
 {
-    pthread_mutex_init(&callMutex_, NULL);
     time(&timestamp_start_);
 }
 
 Call::~Call()
-{
-    pthread_mutex_destroy(&callMutex_);
-}
+{}
 
 void
 Call::setConnectionState(ConnectionState state)
 {
-    sfl::ScopedLock m(callMutex_);
+    std::lock_guard<std::mutex> lock(callMutex_);
     connectionState_ = state;
 }
 
 Call::ConnectionState
 Call::getConnectionState()
 {
-    sfl::ScopedLock m(callMutex_);
+    std::lock_guard<std::mutex> lock(callMutex_);
     return connectionState_;
 }
 
 void
 Call::setState(CallState state)
 {
-    sfl::ScopedLock m(callMutex_);
+    std::lock_guard<std::mutex> lock(callMutex_);
     callState_ = state;
 }
 
 Call::CallState
 Call::getState()
 {
-    sfl::ScopedLock m(callMutex_);
+    std::lock_guard<std::mutex> lock(callMutex_);
     return callState_;
 }
 
@@ -129,34 +125,32 @@ Call::getStateStr()
 std::string
 Call::getLocalIp()
 {
-    sfl::ScopedLock m(callMutex_);
+    std::lock_guard<std::mutex> lock(callMutex_);
     return localIPAddress_;
 }
 
 unsigned int
 Call::getLocalAudioPort()
 {
-    sfl::ScopedLock m(callMutex_);
+    std::lock_guard<std::mutex> lock(callMutex_);
     return localAudioPort_;
 }
 
 unsigned int
 Call::getLocalVideoPort()
 {
-    sfl::ScopedLock m(callMutex_);
+    std::lock_guard<std::mutex> lock(callMutex_);
     return localVideoPort_;
 }
 
 bool
-Call::setRecording()
+Call::toggleRecording()
 {
-    bool recordStatus = Recordable::recAudio_.isRecording();
-
-    Recordable::recAudio_.setRecording();
+    const bool startRecording = Recordable::recAudio_.toggleRecording();
     MainBuffer &mbuffer = Manager::instance().getMainBuffer();
     std::string process_id = Recordable::recorder_.getRecorderID();
 
-    if (!recordStatus) {
+    if (startRecording) {
         mbuffer.bindHalfDuplexOut(process_id, id_);
         mbuffer.bindHalfDuplexOut(process_id, MainBuffer::DEFAULT_ID);
 
@@ -166,9 +160,7 @@ Call::setRecording()
         mbuffer.unBindHalfDuplexOut(process_id, MainBuffer::DEFAULT_ID);
     }
 
-    Manager::instance().getMainBuffer().dumpInfo();
-
-    return recordStatus;
+    return startRecording;
 }
 
 void Call::time_stop()
@@ -212,10 +204,20 @@ std::map<std::string, std::string> Call::createHistoryEntry() const
     result[HistoryItem::RECORDING_PATH_KEY] = recAudio_.fileExists() ? getFilename() : "";
     result[HistoryItem::TIMESTAMP_START_KEY] = timestamp_to_string(timestamp_start_);
     result[HistoryItem::TIMESTAMP_STOP_KEY] = timestamp_to_string(timestamp_stop_);
-    if (connectionState_ == RINGING)
+
+    // FIXME: state will no longer exist, it will be split into
+    // a boolean field called "missed" and a direction field "incoming" or "outgoing"
+    if (connectionState_ == RINGING) {
         result[HistoryItem::STATE_KEY] = HistoryItem::MISSED_STRING;
-    else
+        result[HistoryItem::MISSED_KEY] = "true";
+    } else {
         result[HistoryItem::STATE_KEY] = getTypeStr();
+        result[HistoryItem::MISSED_KEY] = "false";
+    }
+
+    // now "missed" and direction are independent
+    result[HistoryItem::DIRECTION_KEY] = getTypeStr();
+
     return result;
 }
 
@@ -239,11 +241,12 @@ std::map<std::string, std::string>
 Call::getNullDetails()
 {
     std::map<std::string, std::string> details;
-    details["ACCOUNTID"] = "";
+    details["CALL_TYPE"] = "0";
     details["PEER_NUMBER"] = "Unknown";
-    details["PEER_NAME"] = "Unknown";
     details["DISPLAY_NAME"] = "Unknown";
     details["CALL_STATE"] = "UNKNOWN";
-    details["CALL_TYPE"] = "0";
+    details["CONF_ID"] = "";
+    details["TIMESTAMP_START"] = "";
+    details["ACCOUNTID"] = "";
     return details;
 }
