@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Yan Morin <yan.morin@savoirfairelinux.com>
  *  Author : Laurielle Lea <laurielle.lea@savoirfairelinux.com>
@@ -34,7 +34,7 @@
 
 #include <sstream>
 #include <map>
-#include <pthread.h>
+#include <mutex>
 #include "audio/recordable.h"
 
 /*
@@ -228,12 +228,12 @@ class Call : public Recordable {
         virtual std::map<std::string, std::string>
         createHistoryEntry() const;
 
-        virtual bool setRecording();
+        virtual bool toggleRecording();
 
     private:
         std::string getTypeStr() const;
         /** Protect every attribute that can be changed by two threads */
-        pthread_mutex_t callMutex_;
+        std::mutex callMutex_;
 
         // Informations about call socket / audio
 
