@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Alexandre Bourget <alexandre.bourget@savoirfairelinux.com>
  *  Author: Yan Morin <yan.morin@savoirfairelinux.com>
  *
@@ -91,8 +91,8 @@ ConfigTree::getSections() const
 {
     std::list<std::string> sections;
 
-    for (SectionMap::const_iterator iter = sections_.begin(); iter != sections_.end(); ++iter)
-        sections.push_back(iter->first);
+    for (const auto &item : sections_)
+        sections.push_back(item.first);
 
     return sections;
 }
