@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Alexandre Bourget <alexandre.bourget@savoirfairelinux.com>
  *  Author: Yan Morin <yan.morin@savoirfairelinux.com>
  *
@@ -44,6 +44,7 @@
 
 namespace Conf {
 static const char * const TRUE_STR = "true";
+static const char * const FALSE_STR = "false";
 
 class ConfigTreeItem;
 typedef std::map<std::string, ConfigTreeItem> ItemMap;
@@ -149,10 +150,6 @@ class ConfigTree {
         friend class ConfigTreeIterator;
 
         NON_COPYABLE(ConfigTree);
-    public:
-        ConfigTreeIterator createIterator() {
-            return ConfigTreeIterator(this);
-        }
 };
 
 class ConfigTreeItem {
