@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -52,8 +52,8 @@ YamlNode *YamlDocument::popNode()
 
 void YamlDocument::deleteChildNodes()
 {
-    for (Sequence::iterator it = doc_.begin(); it != doc_.end(); ++it) {
-        YamlNode *yamlNode = static_cast<YamlNode *>(*it);
+    for (auto &it : doc_) {
+        YamlNode *yamlNode = static_cast<YamlNode *>(it);
 
         yamlNode->deleteChildNodes();
         delete yamlNode;
@@ -109,6 +109,17 @@ void MappingNode::getValue(const std::string &key, int *i) const
     *i = std::atoi(node->getValue().c_str());
 }
 
+void MappingNode::getValue(const std::string &key, double *d) const
+{
+    ScalarNode *node = static_cast<ScalarNode*>(getValue(key));
+    if (!node) {
+        ERROR("node %s not found", key.c_str());
+        return;
+    }
+
+    *d = std::atof(node->getValue().c_str());
+}
+
 void MappingNode::getValue(const std::string &key, std::string *v) const
 {
     ScalarNode *node = static_cast<ScalarNode*>(getValue(key));
@@ -124,8 +135,8 @@ void MappingNode::getValue(const std::string &key, std::string *v) const
 
 void MappingNode::deleteChildNodes()
 {
-    for (YamlNodeMap::iterator it = map_.begin(); it != map_.end(); ++it) {
-        YamlNode *yamlNode = static_cast<YamlNode *>(it->second);
+    for (auto &it : map_) {
+        YamlNode *yamlNode = static_cast<YamlNode *>(it.second);
 
         if (!yamlNode)
             continue;
@@ -144,8 +155,8 @@ void SequenceNode::addNode(YamlNode *node)
 
 void SequenceNode::deleteChildNodes()
 {
-    for (Sequence::iterator it = seq_.begin(); it != seq_.end(); ++it) {
-        YamlNode *yamlNode = static_cast<YamlNode *>(*it);
+    for (auto &it : seq_) {
+        YamlNode *yamlNode = static_cast<YamlNode *>(it);
 
         yamlNode->deleteChildNodes();
         delete yamlNode;
