@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -31,30 +31,33 @@
 #include "eventthread.h"
 #include "voiplink.h"
 
-EventThread::EventThread(VoIPLink *link) : link_(link), thread_(0)
+#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 8)
+#define YIELD std::this_thread::yield
+#else
+#define YIELD pthread_yield
+#endif
+
+EventThread::EventThread(VoIPLink *link) : link_(link), thread_()
 {}
 
 EventThread::~EventThread()
 {
-    if (thread_)
-        pthread_join(thread_, NULL);
+    join();
+}
+
+void EventThread::join()
+{
+    if (thread_.joinable())
+        thread_.join();
 }
 
 void EventThread::start()
 {
-    pthread_create(&thread_, NULL, &runCallback, this);
-}
-
-void *
-EventThread::runCallback(void *data)
-{
-    EventThread *context = static_cast<EventThread*>(data);
-    context->run();
-    return NULL;
+    thread_ = std::thread(&EventThread::run, this);
 }
 
 void EventThread::run()
 {
     while (link_->getEvent())
-        pthread_yield();
+        YIELD();
 }
