@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Yan Morin <yan.morin@savoirfairelinux.com>
  *  Author: Laurielle Lea <laurielle.lea@savoirfairelinux.com>
@@ -33,8 +33,11 @@
 #ifndef __GLOBAL_H__
 #define __GLOBAL_H__
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <cstdio>
-#include <libintl.h>
 #include <locale.h>
 #include <string>
 #include <cstdlib>
@@ -42,8 +45,6 @@
 #include <map>
 #include <vector>
 
-#include "config.h"
-
 const char * const ZRTP_ZID_FILENAME = "sfl.zid";
 
 #define ALSA_DFT_CARD_ID 0          /** Index of the default soundcard */
