@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
@@ -43,7 +43,11 @@ const char * const HistoryItem::CONFID_KEY =            "confid";
 const char * const HistoryItem::DISPLAY_NAME_KEY =      "display_name";
 const char * const HistoryItem::PEER_NUMBER_KEY =       "peer_number";
 const char * const HistoryItem::RECORDING_PATH_KEY =    "recordfile";
+// FIXME: Deprecated
 const char * const HistoryItem::STATE_KEY =             "state";
+// New version:
+const char * const HistoryItem::MISSED_KEY =            "missed";
+const char * const HistoryItem::DIRECTION_KEY =         "direction";
 const char * const HistoryItem::TIMESTAMP_START_KEY =   "timestamp_start";
 const char * const HistoryItem::TIMESTAMP_STOP_KEY =    "timestamp_stop";
 const char * const HistoryItem::AUDIO_CODEC_KEY =       "audio_codec";
@@ -103,13 +107,12 @@ bool HistoryItem::hasPeerNumber() const
 void HistoryItem::print(std::ostream &o) const
 {
     // every entry starts with "[" + random integer = "]"
-    for (map<string, string>::const_iterator iter = entryMap_.begin();
-         iter != entryMap_.end(); ++iter) {
+    for (const auto &item : entryMap_) {
         // if the file does not exist anymore, we do not save it
-        if (iter->first == RECORDING_PATH_KEY and not file_exists(iter->second))
-            o << iter->first << "=" << "" << std::endl;
+        if (item.first == RECORDING_PATH_KEY and not file_exists(item.second))
+            o << item.first << "=" << "" << std::endl;
         else
-            o << iter->first << "=" << iter->second << std::endl;
+            o << item.first << "=" << item.second << std::endl;
     }
 }
 
