@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Alexamdre Savard <alexandre.savard@savoirfairelinux.com>
@@ -51,6 +51,8 @@ class HistoryItem {
         static const char * const AUDIO_CODEC_KEY;
         static const char * const VIDEO_CODEC_KEY;
         static const char * const STATE_KEY;
+        static const char * const MISSED_KEY;
+        static const char * const DIRECTION_KEY;
 
         static const char * const MISSED_STRING;
         static const char * const INCOMING_STRING;
