@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Alexandre Bourget <alexandre.bourget@savoirfairelinux.com>
@@ -77,6 +77,9 @@ void IAXAccount::serialize(Conf::YamlEmitter &emitter)
     accountmap.setKeyValue(DISPLAY_NAME_KEY, &displayName);
     accountmap.setKeyValue(AUDIO_CODECS_KEY, &codecs);
 
+    Conf::ScalarNode userAgent(userAgent_);
+    accountmap.setKeyValue(USER_AGENT_KEY, &userAgent);
+
     try {
         emitter.serializeAccount(&accountmap);
     } catch (const Conf::YamlEmitterException &e) {
@@ -97,26 +100,27 @@ void IAXAccount::unserialize(const Conf::YamlNode &map)
     // Update codec list which one is used for SDP offer
     setActiveAudioCodecs(split_string(audioCodecStr_));
     map.getValue(DISPLAY_NAME_KEY, &displayName_);
+
+    map.getValue(USER_AGENT_KEY, &userAgent_);
 }
 
-void IAXAccount::setAccountDetails(std::map<std::string, std::string> details)
+void IAXAccount::setAccountDetails(const std::map<std::string, std::string> &details)
 {
     // Account setting common to SIP and IAX
-    alias_ = details[CONFIG_ACCOUNT_ALIAS];
-    username_ = details[CONFIG_ACCOUNT_USERNAME];
-    hostname_ = details[CONFIG_ACCOUNT_HOSTNAME];
-    password_ = details[CONFIG_ACCOUNT_PASSWORD];
-    enabled_ = details[CONFIG_ACCOUNT_ENABLE] == "true";
-    mailBox_ = details[CONFIG_ACCOUNT_MAILBOX];
-    displayName_ = details[CONFIG_DISPLAY_NAME];
-    userAgent_ = details[CONFIG_ACCOUNT_USERAGENT];
+    parseString(details, CONFIG_ACCOUNT_ALIAS, alias_);
+    parseString(details, CONFIG_ACCOUNT_USERNAME, username_);
+    parseString(details, CONFIG_ACCOUNT_HOSTNAME, hostname_);
+    parseString(details, CONFIG_ACCOUNT_PASSWORD, password_);
+    parseBool(details, CONFIG_ACCOUNT_ENABLE, enabled_);
+    parseString(details, CONFIG_ACCOUNT_MAILBOX, mailBox_);
+    parseString(details, CONFIG_DISPLAY_NAME, displayName_);
+    parseString(details, CONFIG_ACCOUNT_USERAGENT, userAgent_);
 }
 
 std::map<std::string, std::string> IAXAccount::getAccountDetails() const
 {
     std::map<std::string, std::string> a;
 
-    a[CONFIG_ACCOUNT_ID] = accountID_;
     a[CONFIG_ACCOUNT_ALIAS] = alias_;
     a[CONFIG_ACCOUNT_ENABLE] = enabled_ ? "true" : "false";
     a[CONFIG_ACCOUNT_TYPE] = ACCOUNT_TYPE;
