@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Alexandre Bourget <alexandre.bourget@savoirfairelinux.com>
  *  Author: Yan Morin <yan.morin@savoirfairelinux.com>
  *
@@ -43,42 +43,36 @@
 #include "audio/audiolayer.h"
 #include "audio/samplerateconverter.h"
 #include "array_size.h"
-#include "scoped_lock.h"
 #include "map_utils.h"
 
 AccountMap IAXVoIPLink::iaxAccountMap_;
 IAXCallMap IAXVoIPLink::iaxCallMap_;
-// has same effect as pthread_mutex_init with default args, but for a statically
-// allocated mutex
-pthread_mutex_t IAXVoIPLink::iaxCallMapMutex_ = PTHREAD_MUTEX_INITIALIZER;
+std::mutex IAXVoIPLink::iaxCallMapMutex_;
 
 IAXVoIPLink::IAXVoIPLink(const std::string& accountID) :
     regSession_(NULL)
     , nextRefreshStamp_(0)
     , mutexIAX_()
-    , decData_()
-    , resampledData_()
+    , decData_(DEC_BUFFER_SIZE, 1, 8000)
+    , resampledData_(DEC_BUFFER_SIZE * 4, 1, 8000)
     , encodedData_()
     , converter_(44100)
     , initDone_(false)
     , accountID_(accountID)
     , evThread_(this)
 {
-    pthread_mutex_init(&mutexIAX_, NULL);
     srand(time(NULL));    // to get random number for RANDOM_PORT
 }
 
 
 IAXVoIPLink::~IAXVoIPLink()
 {
-    handlingEvents_ = false;
     regSession_ = NULL; // shall not delete it // XXX: but why?
     terminate();
 
     // This is our last account
     if (iaxAccountMap_.size() == 1)
         clearIaxCallMap();
-    pthread_mutex_destroy(&mutexIAX_);
 }
 
 void
@@ -103,19 +97,21 @@ IAXVoIPLink::terminate()
     if (!initDone_)
         return;
 
-    sfl::ScopedLock m(iaxCallMapMutex_);
+    std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
+    handlingEvents_ = false;
 
-    for (IAXCallMap::iterator iter = iaxCallMap_.begin(); iter != iaxCallMap_.end(); ++iter) {
-        IAXCall *call = static_cast<IAXCall*>(iter->second);
+    for (auto & item : iaxCallMap_) {
+        IAXCall *call = static_cast<IAXCall*>(item.second);
 
         if (call) {
-            sfl::ScopedLock lock(mutexIAX_);
+            std::lock_guard<std::mutex> lock(mutexIAX_);
             iax_hangup(call->session, const_cast<char*>("Dumped Call"));
             delete call;
         }
     }
 
     iaxCallMap_.clear();
+    evThread_.join();
 
     initDone_ = false;
 }
@@ -126,7 +122,7 @@ IAXVoIPLink::getEvent()
     iax_event *event = NULL;
 
     {
-        sfl::ScopedLock lock(mutexIAX_);
+        std::lock_guard<std::mutex> lock(mutexIAX_);
         event = iax_get_event(0);
     }
 
@@ -134,16 +130,16 @@ IAXVoIPLink::getEvent()
 
         // If we received an 'ACK', libiax2 tells apps to ignore them.
         if (event->etype == IAX_EVENT_NULL) {
-            sfl::ScopedLock lock(mutexIAX_);
+            std::lock_guard<std::mutex> lock(mutexIAX_);
             iax_event_free(event);
             event = iax_get_event(0);
             continue;
         }
 
-        IAXCall *call = iaxFindCallBySession(event->session);
+        const std::string id(iaxFindCallIDBySession(event->session));
 
-        if (call) {
-            iaxHandleCallEvent(event, call);
+        if (not id.empty()) {
+            iaxHandleCallEvent(event, id);
         } else if (event->session && event->session == regSession_) {
             // This is a registration session, deal with it
             iaxHandleRegReply(event);
@@ -153,13 +149,13 @@ IAXVoIPLink::getEvent()
         }
 
         {
-            sfl::ScopedLock lock(mutexIAX_);
+            std::lock_guard<std::mutex> lock(mutexIAX_);
             iax_event_free(event);
             event = iax_get_event(0);
         }
     }
 
-    if (nextRefreshStamp_ && nextRefreshStamp_ < time(NULL))
+    if (nextRefreshStamp_ and nextRefreshStamp_ < time(NULL))
         sendRegister(Manager::instance().getIaxAccount(accountID_));
 
     sendAudioFromMic();
@@ -173,7 +169,7 @@ std::vector<std::string>
 IAXVoIPLink::getCallIDs()
 {
     std::vector<std::string> v;
-    sfl::ScopedLock m(iaxCallMapMutex_);
+    std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
 
     map_utils::vectorFromMapKeys(iaxCallMap_, v);
     return v;
@@ -182,8 +178,8 @@ IAXVoIPLink::getCallIDs()
 void
 IAXVoIPLink::sendAudioFromMic()
 {
-    for (IAXCallMap::const_iterator iter = iaxCallMap_.begin(); iter != iaxCallMap_.end() ; ++iter) {
-        IAXCall *currentCall = static_cast<IAXCall*>(iter->second);
+    for (const auto & item : iaxCallMap_) {
+        IAXCall *currentCall = static_cast<IAXCall*>(item.second);
 
         if (!currentCall or currentCall->getState() != Call::ACTIVE)
             continue;
@@ -200,34 +196,35 @@ IAXVoIPLink::sendAudioFromMic()
 
         // we have to get 20ms of data from the mic *20/1000 = /50
         // rate/50 shall be lower than IAX__20S_48KHZ_MAX
-        const size_t bytesNeeded = mainBufferSampleRate * 20 / 1000 * sizeof(SFLDataFormat);
+        size_t samples = mainBufferSampleRate * 20 / 1000;
 
-        if (Manager::instance().getMainBuffer().availableForGet(currentCall->getCallId()) < bytesNeeded)
+        if (Manager::instance().getMainBuffer().availableForGet(currentCall->getCallId()) < samples)
             continue;
 
         // Get bytes from micRingBuffer to data_from_mic
-        int bytes = Manager::instance().getMainBuffer().getData(decData_, bytesNeeded, currentCall->getCallId());
-        int samples = bytes / sizeof(SFLDataFormat);
+        decData_.resize(samples);
+        samples = Manager::instance().getMainBuffer().getData(decData_, currentCall->getCallId());
 
         int compSize;
         unsigned int audioRate = audioCodec->getClockRate();
         int outSamples;
-        SFLDataFormat *in;
+        AudioBuffer *in;
 
         if (audioRate != mainBufferSampleRate) {
-            converter_.resample(decData_, resampledData_, ARRAYSIZE(resampledData_),
-                                audioRate, mainBufferSampleRate, samples);
-            in = resampledData_;
+            decData_.setSampleRate(audioRate);
+            resampledData_.setSampleRate(mainBufferSampleRate);
+            converter_.resample(decData_, resampledData_);
+            in = &resampledData_;
             outSamples = 0;
         } else {
             outSamples = samples;
-            in = decData_;
+            in = &decData_;
         }
 
-        compSize = audioCodec->encode(encodedData_, in, DEC_BUFFER_SIZE);
+        compSize = audioCodec->encode(encodedData_, in->getData(), DEC_BUFFER_SIZE);
 
-        if (currentCall->session and bytes > 0) {
-            sfl::ScopedLock m(mutexIAX_);
+        if (currentCall->session and samples > 0) {
+            std::lock_guard<std::mutex> lock(mutexIAX_);
 
             if (iax_send_voice(currentCall->session, currentCall->format, encodedData_, compSize, outSamples) == -1)
                 ERROR("IAX: Error sending voice data.");
@@ -235,7 +232,6 @@ IAXVoIPLink::sendAudioFromMic()
     }
 }
 
-
 IAXCall*
 IAXVoIPLink::getIAXCall(const std::string& id)
 {
@@ -247,13 +243,20 @@ IAXVoIPLink::sendRegister(Account *a)
 {
     IAXAccount *account = static_cast<IAXAccount*>(a);
 
+    if (!account)
+        throw VoipLinkException("Account is NULL");
+    else if (not account->isEnabled()) {
+        WARN("Account must be enabled to register, ignoring");
+        return;
+    }
+
     if (account->getHostname().empty())
         throw VoipLinkException("Account hostname is empty");
 
     if (account->getUsername().empty())
         throw VoipLinkException("Account username is empty");
 
-    sfl::ScopedLock m(mutexIAX_);
+    std::lock_guard<std::mutex> lock(mutexIAX_);
 
     if (regSession_)
         iax_destroy(regSession_);
@@ -271,7 +274,7 @@ void
 IAXVoIPLink::sendUnregister(Account *a)
 {
     if (regSession_) {
-        sfl::ScopedLock m(mutexIAX_);
+        std::lock_guard<std::mutex> lock(mutexIAX_);
         iax_destroy(regSession_);
         regSession_ = NULL;
     }
@@ -304,7 +307,7 @@ IAXVoIPLink::answer(Call *call)
     Manager::instance().addStream(call->getCallId());
 
     {
-        sfl::ScopedLock lock(mutexIAX_);
+        std::lock_guard<std::mutex> lock(mutexIAX_);
         call->answer();
     }
 
@@ -317,19 +320,20 @@ IAXVoIPLink::answer(Call *call)
 void
 IAXVoIPLink::hangup(const std::string& id, int reason UNUSED)
 {
-    IAXCall* call = getIAXCall(id);
-
-    if (call == NULL)
-        throw VoipLinkException("Could not find call");
-
-    Manager::instance().getMainBuffer().unBindAll(call->getCallId());
+    Manager::instance().getMainBuffer().unBindAll(id);
 
     {
-        sfl::ScopedLock lock(mutexIAX_);
-        iax_hangup(call->session, (char*) "Dumped Call");
-    }
+        std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
+        IAXCall* call = getIAXCall(id);
+        if (call == nullptr)
+            throw VoipLinkException("Could not find call");
 
-    call->session = NULL;
+        {
+            std::lock_guard<std::mutex> lock(mutexIAX_);
+            iax_hangup(call->session, (char*) "Dumped Call");
+        }
+        call->session = NULL;
+    }
 
     removeIaxCall(id);
 }
@@ -338,14 +342,16 @@ IAXVoIPLink::hangup(const std::string& id, int reason UNUSED)
 void
 IAXVoIPLink::peerHungup(const std::string& id)
 {
-    IAXCall* call = getIAXCall(id);
+    Manager::instance().getMainBuffer().unBindAll(id);
 
-    if (call == NULL)
-        throw VoipLinkException("Could not find call");
+    {
+        std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
+        IAXCall* call = getIAXCall(id);
+        if (call == NULL)
+            throw VoipLinkException("Could not find call");
 
-    Manager::instance().getMainBuffer().unBindAll(call->getCallId());
-
-    call->session = NULL;
+        call->session = NULL;
+    }
 
     removeIaxCall(id);
 }
@@ -355,54 +361,59 @@ IAXVoIPLink::peerHungup(const std::string& id)
 void
 IAXVoIPLink::onhold(const std::string& id)
 {
-    IAXCall* call = getIAXCall(id);
-
-    if (call == NULL)
-        throw VoipLinkException("Call does not exist");
-
-    Manager::instance().getMainBuffer().unBindAll(call->getCallId());
+    Manager::instance().getMainBuffer().unBindAll(id);
 
     {
-        sfl::ScopedLock lock(mutexIAX_);
-        iax_quelch_moh(call->session, true);
+        std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
+        IAXCall* call = getIAXCall(id);
+        if (call == NULL)
+            throw VoipLinkException("Could not find call");
+        {
+            std::lock_guard<std::mutex> lock(mutexIAX_);
+            iax_quelch_moh(call->session, true);
+        }
+        call->setState(Call::HOLD);
     }
 
-    call->setState(Call::HOLD);
 }
 
 void
 IAXVoIPLink::offhold(const std::string& id)
 {
-    IAXCall* call = getIAXCall(id);
-
-    if (call == NULL)
-        throw VoipLinkException("Call does not exist");
-
-    Manager::instance().addStream(call->getCallId());
+    Manager::instance().addStream(id);
 
     {
-        sfl::ScopedLock lock(mutexIAX_);
-        iax_unquelch(call->session);
+        std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
+        IAXCall* call = getIAXCall(id);
+        if (call == NULL)
+            throw VoipLinkException("Could not find call");
+
+        {
+            std::lock_guard<std::mutex> lock(mutexIAX_);
+            iax_unquelch(call->session);
+        }
+
+        call->setState(Call::ACTIVE);
     }
 
     Manager::instance().startAudioDriverStream();
-    call->setState(Call::ACTIVE);
 }
 
 void
 IAXVoIPLink::transfer(const std::string& id, const std::string& to)
 {
-    IAXCall* call = getIAXCall(id);
-
-    if (!call)
-        return;
-
-    char callto[to.length() +1];
+    char callto[to.length() + 1];
     strcpy(callto, to.c_str());
 
     {
-        sfl::ScopedLock lock(mutexIAX_);
-        iax_transfer(call->session, callto);
+        std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
+        IAXCall* call = getIAXCall(id);
+        if (call == NULL)
+            return;
+        {
+            std::lock_guard<std::mutex> lock(mutexIAX_);
+            iax_transfer(call->session, callto);
+        }
     }
 }
 
@@ -415,26 +426,30 @@ IAXVoIPLink::attendedTransfer(const std::string& /*transferID*/, const std::stri
 void
 IAXVoIPLink::refuse(const std::string& id)
 {
-    IAXCall* call = getIAXCall(id);
-
-    if (call) {
+    {
+        std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
+        IAXCall* call = getIAXCall(id);
+        if (call == NULL)
+            return;
         {
-            sfl::ScopedLock lock(mutexIAX_);
+            std::lock_guard<std::mutex> lock(mutexIAX_);
             iax_reject(call->session, (char*) "Call rejected manually.");
         }
-
-        removeIaxCall(id);
     }
+    removeIaxCall(id);
 }
 
 
 void
 IAXVoIPLink::carryingDTMFdigits(const std::string& id, char code)
 {
+    std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
     IAXCall* call = getIAXCall(id);
+    if (call == NULL)
+        return;
 
-    if (call) {
-        sfl::ScopedLock lock(mutexIAX_);
+    {
+        std::lock_guard<std::mutex> lock(mutexIAX_);
         iax_send_dtmf(call->session, code);
     }
 }
@@ -445,10 +460,13 @@ IAXVoIPLink::sendTextMessage(const std::string& callID,
                              const std::string& message,
                              const std::string& /*from*/)
 {
+    std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
     IAXCall* call = getIAXCall(callID);
+    if (call == nullptr)
+        return;
 
-    if (call) {
-        sfl::ScopedLock lock(mutexIAX_);
+    {
+        std::lock_guard<std::mutex> lock(mutexIAX_);
         sfl::InstantMessaging::send_iax_message(call->session, callID, message.c_str());
     }
 }
@@ -457,11 +475,10 @@ IAXVoIPLink::sendTextMessage(const std::string& callID,
 void
 IAXVoIPLink::clearIaxCallMap()
 {
-    sfl::ScopedLock m(iaxCallMapMutex_);
+    std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
 
-    for (IAXCallMap::const_iterator iter = iaxCallMap_.begin();
-            iter != iaxCallMap_.end(); ++iter)
-        delete iter->second;
+    for (const auto & item : iaxCallMap_)
+        delete item.second;
 
     iaxCallMap_.clear();
 
@@ -470,16 +487,18 @@ IAXVoIPLink::clearIaxCallMap()
 void
 IAXVoIPLink::addIaxCall(IAXCall* call)
 {
-    if (call and getIaxCall(call->getCallId()) == NULL) {
-        sfl::ScopedLock m(iaxCallMapMutex_);
+    if (call == nullptr)
+        return;
+
+    std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
+    if (getIaxCall(call->getCallId()) == NULL)
         iaxCallMap_[call->getCallId()] = call;
-    }
 }
 
 void
 IAXVoIPLink::removeIaxCall(const std::string& id)
 {
-    sfl::ScopedLock m(iaxCallMapMutex_);
+    std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
 
     DEBUG("Removing call %s from list", id.c_str());
 
@@ -487,18 +506,18 @@ IAXVoIPLink::removeIaxCall(const std::string& id)
     iaxCallMap_.erase(id);
 }
 
+
 IAXCall*
 IAXVoIPLink::getIaxCall(const std::string& id)
 {
     IAXCallMap::iterator iter = iaxCallMap_.find(id);
-
     if (iter != iaxCallMap_.end())
-
         return iter->second;
     else
         return NULL;
 }
 
+
 std::string
 IAXVoIPLink::getCurrentVideoCodecName(Call * /*call*/) const
 {
@@ -509,7 +528,11 @@ IAXVoIPLink::getCurrentVideoCodecName(Call * /*call*/) const
 std::string
 IAXVoIPLink::getCurrentAudioCodecNames(Call *c) const
 {
+    std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
     IAXCall *call = static_cast<IAXCall*>(c);
+    if (call == nullptr)
+        return "";
+
     sfl::AudioCodec *audioCodec = Manager::instance().audioCodecFactory.getCodec(call->getAudioCodec());
     return audioCodec ? audioCodec->getMimeSubtype() : "";
 }
@@ -517,7 +540,7 @@ IAXVoIPLink::getCurrentAudioCodecNames(Call *c) const
 void
 IAXVoIPLink::iaxOutgoingInvite(IAXCall* call)
 {
-    sfl::ScopedLock m(mutexIAX_);
+    std::lock_guard<std::mutex> lock(mutexIAX_);
 
     call->session = iax_session_new();
 
@@ -534,88 +557,151 @@ IAXVoIPLink::iaxOutgoingInvite(IAXCall* call)
 }
 
 
-IAXCall*
-IAXVoIPLink::iaxFindCallBySession(iax_session* session)
+std::string
+IAXVoIPLink::iaxFindCallIDBySession(iax_session* session)
 {
-    sfl::ScopedLock m(iaxCallMapMutex_);
+    std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
 
-    for (IAXCallMap::const_iterator iter = iaxCallMap_.begin(); iter != iaxCallMap_.end(); ++iter) {
-        IAXCall* call = static_cast<IAXCall*>(iter->second);
+    for (const auto & item : iaxCallMap_) {
+        IAXCall* call = static_cast<IAXCall*>(item.second);
 
         if (call and call->session == session)
-            return call;
+            return call->getCallId();
     }
 
-    return NULL;
+    return "";
 }
 
 void
-IAXVoIPLink::iaxHandleCallEvent(iax_event* event, IAXCall* call)
+IAXVoIPLink::handleReject(const std::string &id)
 {
-    std::string id = call->getCallId();
+    {
+        std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
+        IAXCall *call = getIAXCall(id);
+        if (call) {
+            call->setConnectionState(Call::CONNECTED);
+            call->setState(Call::ERROR);
+        }
+    }
+    Manager::instance().callFailure(id);
+    removeIaxCall(id);
+}
 
+void
+IAXVoIPLink::handleAccept(iax_event* event, const std::string &id)
+{
+    if (event->ies.format) {
+        std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
+        IAXCall *call = getIAXCall(id);
+        if (call)
+            call->format = event->ies.format;
+    }
+}
+
+void
+IAXVoIPLink::handleAnswerTransfer(iax_event* event, const std::string &id)
+{
+    {
+        std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
+        IAXCall *call = getIAXCall(id);
+        if (!call or call->getConnectionState() == Call::CONNECTED)
+            return;
+
+        call->setConnectionState(Call::CONNECTED);
+        call->setState(Call::ACTIVE);
+
+        if (event->ies.format)
+            call->format = event->ies.format;
+    }
+
+    Manager::instance().addStream(id);
+    Manager::instance().peerAnsweredCall(id);
+    Manager::instance().startAudioDriverStream();
+    Manager::instance().getMainBuffer().flushAllBuffers();
+}
+
+
+void
+IAXVoIPLink::handleBusy(const std::string &id)
+{
+    {
+        std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
+        IAXCall *call = getIAXCall(id);
+        if (call == nullptr)
+            return;
+        call->setConnectionState(Call::CONNECTED);
+        call->setState(Call::BUSY);
+    }
+    Manager::instance().callBusy(id);
+    removeIaxCall(id);
+}
+
+void
+IAXVoIPLink::handleMessage(iax_event* event, const std::string &id)
+{
+    std::string peerNumber;
+    {
+        std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
+        IAXCall *call = getIAXCall(id);
+        if (call == nullptr)
+            return;
+        peerNumber = call->getPeerNumber();
+    }
+
+    Manager::instance().incomingMessage(id, peerNumber, std::string((const char*) event->data));
+}
+
+void
+IAXVoIPLink::handleRinging(const std::string &id)
+{
+    {
+        std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
+        IAXCall *call = getIAXCall(id);
+        if (call == nullptr)
+            return;
+        call->setConnectionState(Call::RINGING);
+    }
+    Manager::instance().peerRingingCall(id);
+}
+
+void
+IAXVoIPLink::iaxHandleCallEvent(iax_event* event, const std::string &id)
+{
     switch (event->etype) {
         case IAX_EVENT_HANGUP:
             Manager::instance().peerHungupCall(id);
-
             removeIaxCall(id);
             break;
 
         case IAX_EVENT_REJECT:
-            call->setConnectionState(Call::CONNECTED);
-            call->setState(Call::ERROR);
-            Manager::instance().callFailure(id);
-            removeIaxCall(id);
+            handleReject(id);
             break;
 
         case IAX_EVENT_ACCEPT:
-
-            if (event->ies.format)
-                call->format = event->ies.format;
-
+            handleAccept(event, id);
             break;
 
         case IAX_EVENT_ANSWER:
         case IAX_EVENT_TRANSFER:
-
-            if (call->getConnectionState() == Call::CONNECTED)
-                break;
-
-            Manager::instance().addStream(call->getCallId());
-
-            call->setConnectionState(Call::CONNECTED);
-            call->setState(Call::ACTIVE);
-
-            if (event->ies.format)
-                call->format = event->ies.format;
-
-            Manager::instance().peerAnsweredCall(id);
-
-            Manager::instance().startAudioDriverStream();
-            Manager::instance().getMainBuffer().flushAllBuffers();
-
+            handleAnswerTransfer(event, id);
             break;
 
         case IAX_EVENT_BUSY:
-            call->setConnectionState(Call::CONNECTED);
-            call->setState(Call::BUSY);
-            Manager::instance().callBusy(id);
-            removeIaxCall(id);
+            handleBusy(id);
             break;
 
         case IAX_EVENT_VOICE:
-            iaxHandleVoiceEvent(event, call);
+            iaxHandleVoiceEvent(event, id);
             break;
 
         case IAX_EVENT_TEXT:
 #if HAVE_INSTANT_MESSAGING
-            Manager::instance().incomingMessage(call->getCallId(), call->getPeerNumber(), std::string((const char*) event->data));
+            handleMessage(event, id);
 #endif
             break;
 
         case IAX_EVENT_RINGA:
-            call->setConnectionState(Call::RINGING);
-            Manager::instance().peerRingingCall(call->getCallId());
+            handleRinging(id);
             break;
 
         case IAX_IE_MSGCOUNT:
@@ -635,44 +721,52 @@ IAXVoIPLink::iaxHandleCallEvent(iax_event* event, IAXCall* call)
 
 
 /* Handle audio event, VOICE packet received */
-void IAXVoIPLink::iaxHandleVoiceEvent(iax_event* event, IAXCall* call)
+void IAXVoIPLink::iaxHandleVoiceEvent(iax_event* event, const std::string &id)
 {
     // Skip this empty packet.
     if (!event->datalen)
         return;
 
-    sfl::AudioCodec *audioCodec = static_cast<sfl::AudioCodec *>(Manager::instance().audioCodecFactory.getCodec(call->getAudioCodec()));
+    AudioBuffer *out;
 
-    if (!audioCodec)
-        return;
+    {
+        std::lock_guard<std::mutex> lock(iaxCallMapMutex_);
+        IAXCall* call = getIAXCall(id);
+        if (call == nullptr)
+            throw VoipLinkException("Could not find call");
 
-    Manager::instance().getMainBuffer().setInternalSamplingRate(audioCodec->getClockRate());
-    unsigned int mainBufferSampleRate = Manager::instance().getMainBuffer().getInternalSamplingRate();
+        sfl::AudioCodec *audioCodec = static_cast<sfl::AudioCodec *>(Manager::instance().audioCodecFactory.getCodec(call->getAudioCodec()));
 
-    if (event->subclass)
-        call->format = event->subclass;
+        if (!audioCodec)
+            return;
 
-    unsigned char *data = (unsigned char*) event->data;
-    unsigned int size = event->datalen;
+        Manager::instance().getMainBuffer().setInternalSamplingRate(audioCodec->getClockRate());
+        unsigned int mainBufferSampleRate = Manager::instance().getMainBuffer().getInternalSamplingRate();
 
-    unsigned int max = audioCodec->getClockRate() * 20 / 1000;
+        if (event->subclass)
+            call->format = event->subclass;
 
-    if (size > max)
-        size = max;
+        unsigned char *data = (unsigned char*) event->data;
+        unsigned int size = event->datalen;
 
-    int samples = audioCodec->decode(decData_, data , size);
-    int outSize = samples * sizeof(SFLDataFormat);
-    SFLDataFormat *out = decData_;
-    unsigned int audioRate = audioCodec->getClockRate();
+        unsigned int max = audioCodec->getClockRate() * 20 / 1000;
 
-    if (audioRate != mainBufferSampleRate) {
-        outSize = (double)outSize * (mainBufferSampleRate / audioRate);
-        converter_.resample(decData_, resampledData_, ARRAYSIZE(resampledData_),
-                            mainBufferSampleRate, audioRate, samples);
-        out = resampledData_;
+        if (size > max)
+            size = max;
+
+        audioCodec->decode(decData_.getData(), data , size);
+        out = &decData_;
+        unsigned int audioRate = audioCodec->getClockRate();
+
+        if (audioRate != mainBufferSampleRate) {
+            decData_.setSampleRate(mainBufferSampleRate);
+            resampledData_.setSampleRate(audioRate);
+            converter_.resample(decData_, resampledData_);
+            out = &resampledData_;
+        }
     }
 
-    Manager::instance().getMainBuffer().putData(out, outSize, call->getCallId());
+    Manager::instance().getMainBuffer().putData(*out, id);
 }
 
 /**
@@ -686,7 +780,7 @@ void IAXVoIPLink::iaxHandleRegReply(iax_event* event)
         return;
 
     {
-        sfl::ScopedLock m(mutexIAX_);
+        std::lock_guard<std::mutex> lock(mutexIAX_);
         iax_destroy(regSession_);
         regSession_ = NULL;
     }
@@ -720,6 +814,7 @@ void IAXVoIPLink::iaxHandlePrecallEvent(iax_event* event)
             // if peerNumber exist append it to the name string
             if (event->ies.calling_number)
                 call->initRecFilename(std::string(event->ies.calling_number));
+
             Manager::instance().incomingCall(*call, accountID_);
 
             format = call->getFirstMatchingFormat(event->ies.format, accountID_);
@@ -735,13 +830,13 @@ void IAXVoIPLink::iaxHandlePrecallEvent(iax_event* event)
             break;
 
         case IAX_EVENT_HANGUP:
-            call = iaxFindCallBySession(event->session);
+            id = iaxFindCallIDBySession(event->session);
 
-            if (call) {
-                id = call->getCallId();
+            if (not id.empty()) {
                 Manager::instance().peerHungupCall(id);
                 removeIaxCall(id);
             }
+
             break;
 
         case IAX_EVENT_TIMEOUT: // timeout for an unknown session
@@ -749,6 +844,7 @@ void IAXVoIPLink::iaxHandlePrecallEvent(iax_event* event)
         case IAX_EVENT_REGACK:
         case IAX_EVENT_REGREJ:
         case IAX_EVENT_REGREQ:
+
             // Received when someone wants to register to us!?!
             // Asterisk receives and answers to that, not us, we're a phone.
         default:
@@ -759,6 +855,7 @@ void IAXVoIPLink::iaxHandlePrecallEvent(iax_event* event)
 void
 IAXVoIPLink::unloadAccountMap()
 {
-    std::for_each(iaxAccountMap_.begin(), iaxAccountMap_.end(), unloadAccount);
+    for (auto &a : iaxAccountMap_)
+        unloadAccount(a);
     iaxAccountMap_.clear();
 }
