@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Alexandre Bourget <alexandre.bourget@savoirfairelinux.com>
  *  Author: Yan Morin <yan.morin@savoirfairelinux.com>
@@ -36,9 +36,10 @@
 #include "config.h"
 #endif
 
-#include <pthread.h>
+#include <mutex>
 #include "account.h"
 #include "voiplink.h"
+#include "audio/audiobuffer.h"
 #include "audio/codecs/audiocodec.h" // for DEC_BUFFER_SIZE
 #include "sfl_types.h"
 #include "noncopyable.h"
@@ -191,6 +192,7 @@ class IAXVoIPLink : public VoIPLink {
 #endif
         static void clearIaxCallMap();
         static void addIaxCall(IAXCall* call);
+        // must be called while holding iaxCallMapMutex
         static IAXCall* getIaxCall(const std::string& id);
         static void removeIaxCall(const std::string &id);
 
@@ -204,12 +206,19 @@ class IAXVoIPLink : public VoIPLink {
     private:
         NON_COPYABLE(IAXVoIPLink);
 
+        void handleAccept(iax_event* event, const std::string &id);
+        void handleReject(const std::string &id);
+        void handleRinging(const std::string &id);
+        void handleAnswerTransfer(iax_event* event, const std::string &id);
+        void handleBusy(const std::string &id);
+        void handleMessage(iax_event* event, const std::string &id);
+
         /**
          * Contains a list of all IAX account
          */
         static AccountMap iaxAccountMap_;
 
-        static pthread_mutex_t iaxCallMapMutex_;
+        static std::mutex iaxCallMapMutex_;
         static IAXCallMap iaxCallMap_;
 
         /*
@@ -234,21 +243,21 @@ class IAXVoIPLink : public VoIPLink {
          * @param session an iax_session valid pointer
          * @return iaxcall or 0 if not found
          */
-        IAXCall* iaxFindCallBySession(struct iax_session* session);
+        std::string iaxFindCallIDBySession(struct iax_session* session);
 
         /**
          * Handle IAX Event for a call
          * @param event An iax_event pointer
          * @param call  An IAXCall pointer
          */
-        void iaxHandleCallEvent(iax_event* event, IAXCall* call);
+        void iaxHandleCallEvent(iax_event* event, const std::string &id);
 
         /**
          * Handle the VOICE events specifically
          * @param event The iax_event containing the IAX_EVENT_VOICE
          * @param call  The associated IAXCall
          */
-        void iaxHandleVoiceEvent(iax_event* event, IAXCall* call);
+        void iaxHandleVoiceEvent(iax_event* event, const std::string &id);
 
         /**
          * Handle IAX Registration Reply event
@@ -283,11 +292,13 @@ class IAXVoIPLink : public VoIPLink {
 
         /** Mutex for iax_ calls, since we're the only one dealing with the incorporated
          * iax_stuff inside this class. */
-        pthread_mutex_t mutexIAX_;
+        std::mutex mutexIAX_;
 
         /** encoder/decoder/resampler buffers */
-        SFLDataFormat decData_[DEC_BUFFER_SIZE];
-        SFLDataFormat resampledData_[DEC_BUFFER_SIZE * 4];
+        AudioBuffer decData_;
+        AudioBuffer resampledData_;
+        //SFLAudioSample decData_[DEC_BUFFER_SIZE];
+        //SFLAudioSample resampledData_[DEC_BUFFER_SIZE * 4];
         unsigned char encodedData_[DEC_BUFFER_SIZE];
 
         /** Sample rate converter object */
