@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Alexandre Bourget <alexandre.bourget@savoirfairelinux.com>
  *  Author: Yan Morin <yan.morin@savoirfairelinux.com>
  *  Author: Laurielle Lea <laurielle.lea@savoirfairelinux.com>
@@ -35,6 +35,7 @@
 #endif
 
 #include <iostream>
+#include <signal.h>
 #include <getopt.h>
 #include "fileutils.h"
 #include "logger.h"
@@ -44,7 +45,7 @@ namespace {
     void print_title()
     {
         std::cout << "SFLphone Daemon " << VERSION <<
-            ", by Savoir-Faire Linux 2004-2012" << std::endl <<
+            ", by Savoir-Faire Linux 2004-2014" << std::endl <<
             "http://www.sflphone.org/" << std::endl;
     }
 
@@ -124,8 +125,13 @@ namespace {
 namespace {
     void signal_handler(int code)
     {
+        // Unset signal handlers
+        signal(SIGHUP, SIG_DFL);
+        signal(SIGINT, SIG_DFL);
+        signal(SIGTERM, SIG_DFL);
+
         std::cerr << "Caught signal " << strsignal(code) << ", terminating..." << std::endl;
-        Manager::instance().finish();
+        Manager::instance().interrupt();
     }
 }
 
@@ -143,7 +149,7 @@ int main(int argc, char *argv [])
     std::vector<char> writable(programName.size() + 1);
     std::copy(programName.begin(), programName.end(), writable.begin());
 
-    fileutils::set_program_dir(&*writable.begin());
+    fileutils::set_program_dir(writable.data());
 
     print_title();
     if (parse_args(argc, argv))
@@ -167,12 +173,14 @@ int main(int argc, char *argv [])
         return 1;
     }
 
+    int ret = 0;
 #ifdef SFL_VIDEO
     WARN("Built with video support");
 #endif
+#if HAVE_DBUS
+    ret = Manager::instance().run();
+#endif
+    Manager::instance().finish();
 
-    Manager::instance().run();
-    Manager::instance().saveHistory();
-
-    return 0;
+    return ret;
 }
