@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Yan Morin <yan.morin@savoirfairelinux.com>
  *  Author: Laurielle Lea <laurielle.lea@savoirfairelinux.com>
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
@@ -43,10 +43,10 @@
 #include <vector>
 #include <set>
 #include <map>
-#include <tr1/memory>
+#include <memory>
+#include <mutex>
 
-#include <pthread.h>
-#include "dbus/dbusmanager.h"
+#include "client/client.h"
 
 #include "config/sfl_config.h"
 
@@ -129,10 +129,16 @@ class ManagerImpl {
          */
         void init(const std::string &config_file);
 
+        void setPath(const std::string &path);
+
+#ifdef HAVE_DBUS
         /**
          * Enter Dbus mainloop
          */
-        void run();
+        int run();
+        // Interrupt Dbus mainloop
+        int interrupt();
+#endif
 
         /*
          * Terminate all threads and exit DBus loop
@@ -144,9 +150,7 @@ class ManagerImpl {
          * it's multi-thread and use mutex internally
          * @return AudioLayer*  The audio layer object
          */
-        AudioLayer* getAudioDriver() {
-            return audiodriver_;
-        }
+        AudioLayer* getAudioDriver();
 
         void startAudioDriverStream();
 
@@ -558,24 +562,16 @@ class ManagerImpl {
          * Get the noise reduction engin state from
          * the current audio layer.
          */
-        std::string getNoiseSuppressState() const;
+        bool getNoiseSuppressState() const;
 
         /**
          * Set the noise reduction engin state in the current
          * audio layer.
          */
-        void setNoiseSuppressState(const std::string &state);
+        void setNoiseSuppressState(bool state);
 
-        /**
-         * Get the echo canceller engin state from
-         * the current audio layer
-         */
-        bool getEchoCancelState() const;
-
-        /**
-         * Set the echo canceller engin state
-         */
-        void setEchoCancelState(const std::string &state);
+        bool isAGCEnabled() const;
+        void setAGCState(bool enabled);
 
         /**
          * Ringtone option.
@@ -606,8 +602,9 @@ class ManagerImpl {
          * Set recording on / off
          * Start recording
          * @param id  The call identifier
+         * Returns true if the call was set to record
          */
-        void setRecordingCall(const std::string& id);
+        bool toggleRecordingCall(const std::string& id);
 
         /**
          * Return true if the call is currently recorded
@@ -705,13 +702,6 @@ class ManagerImpl {
          */
         std::string getConfigString(const std::string& section, const std::string& name) const;
 
-        /**
-         * Retrieve the soundcards index in the user config file and try to open audio devices
-         * with a specific alsa plugin.
-         * Set the audio layer sample rate
-         */
-        void selectAudioDriver();
-
         /**
          * Handle audio sounds heard by a caller while they wait for their
          * connection to a called party to be completed.
@@ -721,7 +711,7 @@ class ManagerImpl {
         /**
          * Handle played music when an incoming call occurs
          */
-        void ringtone(const std::string& accountID);
+        void playRingtone(const std::string& accountID);
 
         /**
          * Handle played music when a congestion occurs
@@ -754,7 +744,7 @@ class ManagerImpl {
          * @return true is there is one or many incoming call waiting
          * new call, not anwsered or refused
          */
-        bool incomingCallWaiting() const;
+        bool incomingCallsWaiting();
 
         /**
          * Return a new random callid that is not present in the list
@@ -787,18 +777,18 @@ class ManagerImpl {
         const AudioCodecFactory audioCodecFactory;
 
     private:
+        void removeAccounts();
+
+        bool parseConfiguration();
+
+        // Set the ringtone or recorded call to be played
+        void updateAudioFile(const std::string &file, int sampleRate);
 
         /**
          * Get the Call referred to by callID. If the Call does not exist, return NULL
          */
         Call *getCallFromCallID(const std::string &callID);
 
-        /**
-         * Play the dtmf-associated sound
-         * @param code  The pressed key
-         */
-        void playDtmf(char code);
-
         /**
          * Process remaining participant given a conference and the current call id.
          * Mainly called when a participant is detached or hagned up
@@ -810,7 +800,7 @@ class ManagerImpl {
         /**
          * Create config directory in home user and return configuration file path
          */
-        std::string createConfigFile() const;
+        std::string retrieveConfigPath() const;
 
         /*
          * Initialize zeroconf module and scanning
@@ -834,7 +824,7 @@ class ManagerImpl {
          */
         void playATone(Tone::TONEID toneId);
 
-        DBusManager dbus_;
+        Client client_;
 
         /** The configuration tree. It contains accounts parameters, general user settings ,audio settings, ... */
         Conf::ConfigTree config_;
@@ -843,20 +833,20 @@ class ManagerImpl {
         std::string currentCallId_;
 
         /** Protected current call access */
-        pthread_mutex_t currentCallMutex_;
+        std::mutex currentCallMutex_;
 
         /** Audio layer */
         AudioLayer* audiodriver_;
 
         // Main thread
-        std::tr1::shared_ptr<DTMF> dtmfKey_;
+        std::unique_ptr<DTMF> dtmfKey_;
 
         /////////////////////
         // Protected by Mutex
         /////////////////////
-        pthread_mutex_t toneMutex_;
-        std::tr1::shared_ptr<TelephoneTone> telephoneTone_;
-        std::tr1::shared_ptr<AudioFile> audiofile_;
+        std::mutex toneMutex_;
+        std::unique_ptr<TelephoneTone> telephoneTone_;
+        std::unique_ptr<AudioFile> audiofile_;
 
         // To handle volume control
         // short speakerVolume_;
@@ -866,22 +856,17 @@ class ManagerImpl {
         /**
          * Mutex used to protect audio layer
          */
-        pthread_mutex_t audioLayerMutex_;
+        std::mutex audioLayerMutex_;
 
         /**
          * Waiting Call Vectors
          */
-        CallIDSet waitingCall_;
+        CallIDSet waitingCalls_;
 
         /**
          * Protect waiting call list, access by many voip/audio threads
          */
-        pthread_mutex_t waitingCallMutex_;
-
-        /**
-         * Number of waiting call, synchronize with waitingcall callidvector
-         */
-        unsigned int nbIncomingWaitingCall_;
+        std::mutex waitingCallsMutex_;
 
         /**
          * Add incoming callid to the waiting list
@@ -895,12 +880,6 @@ class ManagerImpl {
          */
         void removeWaitingCall(const std::string& id);
 
-        /** Remove a CallID/std::string association
-         * Protected by mutex
-         * @param callID the CallID to remove
-         */
-        void removeCallAccount(const std::string& callID);
-
         /**
          * Path of the ConfigFile
          */
@@ -949,9 +928,7 @@ class ManagerImpl {
         /**
          * Return a pointer to the  instance of the mainbuffer
          */
-        MainBuffer &getMainBuffer() {
-            return mainBuffer_;
-        }
+        MainBuffer &getMainBuffer();
 
         /**
          * Tell if there is a current call processed
@@ -960,17 +937,12 @@ class ManagerImpl {
         bool hasCurrentCall() const;
 
         /**
-         * Return the current DBusManager
-         * @return A pointer to the DBusManager instance
+         * Return the current Client
+         * @return A pointer to the Client instance
          */
-        DBusManager * getDbusManager() {
-            return &dbus_;
-        }
-
+        Client* getClient();
 #ifdef SFL_VIDEO
-        VideoControls * getVideoControls() {
-            return dbus_.getVideoControls();
-        }
+        VideoControls * getVideoControls();
 #endif
 
         /**
@@ -1027,32 +999,39 @@ class ManagerImpl {
          */
         VoIPLink* getAccountLink(const std::string& accountID);
 
-        std::string getStunServer() const;
-        void setStunServer(const std::string &server);
-
-        int isStunEnabled();
-        void enableStun();
-
-        // Map containing conference pointers
-        ConferenceMap conferenceMap_;
-
         /**
          * Send registration to all enabled accounts
          */
         void registerAccounts();
         void saveHistory();
 
+        /**
+         * Suspends SFLphone's audio processing if no calls remain, allowing
+         * other applications to resume audio.
+         * See:
+         * https://projects.savoirfairelinux.com/issues/7037
+        */
+        void
+        checkAudio();
+
     private:
         NON_COPYABLE(ManagerImpl);
 
+        /**
+         * Play the dtmf-associated sound
+         * @param code  The pressed key
+         */
+        void playDtmf(char code);
+
+
+        // Map containing conference pointers
+        ConferenceMap conferenceMap_;
+
         /**
          * Get a map with all the current SIP and IAX accounts
          */
         AccountMap getAllAccounts() const;
 
-        void
-        checkAudio();
-
         /**
           * To handle the persistent history
           * TODO: move this to ConfigurationManager
