@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -34,15 +34,23 @@
 
 #include "preferences.h"
 #include "logger.h"
+#include "audio/audiolayer.h"
+#ifdef __ANDROID__
+#include "audio/opensl/opensllayer.h"
+#else
+#if HAVE_ALSA
 #include "audio/alsa/alsalayer.h"
+#endif
 #if HAVE_PULSE
 #include "audio/pulseaudio/pulselayer.h"
 #endif
+#endif
 #include "config/yamlemitter.h"
 #include "config/yamlnode.h"
 #include "hooks/urlhook.h"
 #include "sip/sip_utils.h"
 #include <sstream>
+#include <algorithm>
 #include "global.h"
 #include "fileutils.h"
 
@@ -77,7 +85,9 @@ static const char * const URL_COMMAND_KEY = "urlCommand";
 static const char * const URL_SIP_FIELD_KEY = "urlSipField";
 
 // audio preferences
+#if HAVE_ALSA
 static const char * const ALSAMAP_KEY = "alsa";
+#endif
 #if HAVE_PULSE
 static const char * const PULSEMAP_KEY = "pulse";
 #endif
@@ -94,7 +104,9 @@ static const char * const ALWAYS_RECORDING_KEY = "alwaysRecording";
 static const char * const VOLUMEMIC_KEY = "volumeMic";
 static const char * const VOLUMESPKR_KEY = "volumeSpkr";
 static const char * const NOISE_REDUCE_KEY = "noiseReduce";
-static const char * const ECHO_CANCEL_KEY = "echoCancel";
+static const char * const AGC_KEY = "automaticGainControl";
+static const char * const CAPTURE_MUTED_KEY = "captureMuted";
+static const char * const PLAYBACK_MUTED_KEY = "playbackMuted";
 
 // shortcut preferences
 static const char * const HANGUP_SHORT_KEY = "hangUp";
@@ -103,11 +115,9 @@ static const char * const POPUP_SHORT_KEY = "popupWindow";
 static const char * const TOGGLE_HOLD_SHORT_KEY = "toggleHold";
 static const char * const TOGGLE_PICKUP_HANGUP_SHORT_KEY = "togglePickupHangup";
 
-static const char * const DFT_PULSE_LENGTH_STR ="250";  /** Default DTMF lenght */
+static const char * const DFT_PULSE_LENGTH_STR = "250"; /** Default DTMF lenght */
 static const char * const ZRTP_ZIDFILE = "zidFile";     /** The filename used for storing ZIDs */
-static const char * const ALSA_DFT_CARD	= "0";          /** Default sound card index */
-static const char * const DFT_VOL_SPKR_STR = "100";     /** Default speaker volume */
-static const char * const DFT_VOL_MICRO_STR	= "100";    /** Default mic volume */
+static const char * const ALSA_DFT_CARD    = "0";          /** Default sound card index */
 } // end anonymous namespace
 
 Preferences::Preferences() :
@@ -122,6 +132,52 @@ Preferences::Preferences() :
     , md5Hash_(false)
 {}
 
+void Preferences::verifyAccountOrder(const std::vector<std::string> &accountIDs)
+{
+    using namespace std;
+
+    vector<string> tokens;
+    string token;
+    bool drop = false;
+
+    for (const auto c : accountOrder_) {
+        if (c != '/') {
+            token += c;
+        } else {
+            if (find(accountIDs.begin(), accountIDs.end(), token) != accountIDs.end())
+                tokens.push_back(token);
+            else {
+                DEBUG("Dropping nonexistent account %s", token.c_str());
+                drop = true;
+            }
+            token.clear();
+        }
+    }
+
+    if (drop) {
+        accountOrder_.clear();
+        for (const auto &t : tokens)
+            accountOrder_ += t + "/";
+    }
+}
+
+void Preferences::addAccount(const std::string &newAccountID)
+{
+    // Add the newly created account in the account order list
+    if (not accountOrder_.empty())
+        accountOrder_.insert(0, newAccountID + "/");
+    else
+        accountOrder_ = newAccountID + "/";
+}
+
+void Preferences::removeAccount(const std::string &oldAccountID)
+{
+    // include the slash since we don't want to remove a partial match
+    const size_t start = accountOrder_.find(oldAccountID + "/");
+    if (start != std::string::npos)
+        accountOrder_.erase(start, oldAccountID.length() + 1);
+}
+
 void Preferences::serialize(Conf::YamlEmitter &emiter)
 {
     Conf::MappingNode preferencemap(NULL);
@@ -290,13 +346,16 @@ AudioPreference::AudioPreference() :
     , pulseDeviceRingtone_("")
     , recordpath_("")
     , alwaysRecording_(false)
-    , volumemic_(atoi(DFT_VOL_SPKR_STR))
-    , volumespkr_(atoi(DFT_VOL_MICRO_STR))
-    , noisereduce_(false)
-    , echocancel_(false)
+    , volumemic_(1.0)
+    , volumespkr_(1.0)
+    , denoise_(false)
+    , agcEnabled_(false)
+    , captureMuted_(false)
+    , playbackMuted_(false)
 {}
 
 namespace {
+#if HAVE_ALSA
 void checkSoundCard(int &card, AudioLayer::PCMType stream)
 {
     if (not AlsaLayer::soundCardIndexExists(card, stream)) {
@@ -304,25 +363,37 @@ void checkSoundCard(int &card, AudioLayer::PCMType stream)
         card = ALSA_DFT_CARD_ID;
     }
 }
+#endif
 }
 
 AudioLayer* AudioPreference::createAudioLayer()
 {
+#ifdef __ANDROID__
+    return new OpenSLLayer(*this);
+#endif
+
 #if HAVE_PULSE
+
     if (audioApi_ == PULSEAUDIO_API_STR) {
         if (system("pactl info > /dev/null") == 0)
             return new PulseLayer(*this);
         else
             WARN("pulseaudio daemon not running, falling back to ALSA");
     }
+
 #endif
 
+#if HAVE_ALSA
+
     audioApi_ = ALSA_API_STR;
     checkSoundCard(alsaCardin_, AudioLayer::SFL_PCM_CAPTURE);
     checkSoundCard(alsaCardout_, AudioLayer::SFL_PCM_PLAYBACK);
     checkSoundCard(alsaCardring_, AudioLayer::SFL_PCM_RINGTONE);
 
     return new AlsaLayer(*this);
+#else
+    return NULL;
+#endif
 }
 
 AudioLayer* AudioPreference::switchAndCreateAudioLayer()
@@ -362,14 +433,16 @@ void AudioPreference::serialize(Conf::YamlEmitter &emitter)
     Conf::ScalarNode audioapi(audioApi_);
     Conf::ScalarNode recordpath(recordpath_); //: /home/msavard/Bureau
     Conf::ScalarNode alwaysRecording(alwaysRecording_);
-    std::stringstream micstr;
+    std::ostringstream micstr;
     micstr << volumemic_;
     Conf::ScalarNode volumemic(micstr.str()); //:  100
-    std::stringstream spkrstr;
+    std::ostringstream spkrstr;
     spkrstr << volumespkr_;
     Conf::ScalarNode volumespkr(spkrstr.str()); //: 100
-    Conf::ScalarNode noise(noisereduce_);
-    Conf::ScalarNode echo(echocancel_);
+    Conf::ScalarNode denoise(denoise_);
+    Conf::ScalarNode agc(agcEnabled_);
+    Conf::ScalarNode captureMuted(captureMuted_);
+    Conf::ScalarNode playbackMuted(playbackMuted_);
 
     Conf::MappingNode preferencemap(NULL);
     preferencemap.setKeyValue(AUDIO_API_KEY, &audioapi);
@@ -377,14 +450,18 @@ void AudioPreference::serialize(Conf::YamlEmitter &emitter)
     preferencemap.setKeyValue(ALWAYS_RECORDING_KEY, &alwaysRecording);
     preferencemap.setKeyValue(VOLUMEMIC_KEY, &volumemic);
     preferencemap.setKeyValue(VOLUMESPKR_KEY, &volumespkr);
+    preferencemap.setKeyValue(CAPTURE_MUTED_KEY, &captureMuted);
+    preferencemap.setKeyValue(PLAYBACK_MUTED_KEY, &playbackMuted);
 
     Conf::MappingNode alsapreferencemap(NULL);
+#if HAVE_ALSA
     preferencemap.setKeyValue(ALSAMAP_KEY, &alsapreferencemap);
     alsapreferencemap.setKeyValue(CARDIN_KEY, &cardin);
     alsapreferencemap.setKeyValue(CARDOUT_KEY, &cardout);
     alsapreferencemap.setKeyValue(CARDRING_KEY, &cardring);
     alsapreferencemap.setKeyValue(PLUGIN_KEY, &plugin);
     alsapreferencemap.setKeyValue(SMPLRATE_KEY, &alsaSmplrate);
+#endif
 
 #if HAVE_PULSE
     Conf::MappingNode pulsepreferencemap(NULL);
@@ -394,9 +471,8 @@ void AudioPreference::serialize(Conf::YamlEmitter &emitter)
     pulsepreferencemap.setKeyValue(DEVICE_RINGTONE_KEY, &pulseDeviceRingtone);
 #endif
 
-    preferencemap.setKeyValue(NOISE_REDUCE_KEY, &noise);
-    preferencemap.setKeyValue(ECHO_CANCEL_KEY, &echo);
-
+    preferencemap.setKeyValue(NOISE_REDUCE_KEY, &denoise);
+    preferencemap.setKeyValue(AGC_KEY, &agc);
     emitter.serializePreference(&preferencemap, "audio");
 }
 
@@ -417,16 +493,26 @@ void AudioPreference::unserialize(const Conf::YamlNode &map)
     map.getValue(AUDIO_API_KEY, &audioApi_);
     std::string tmpRecordPath;
     map.getValue(RECORDPATH_KEY, &tmpRecordPath);
+
     if (not setRecordPath(tmpRecordPath))
         setRecordPath(fileutils::get_home_dir());
 
     map.getValue(ALWAYS_RECORDING_KEY, &alwaysRecording_);
     map.getValue(VOLUMEMIC_KEY, &volumemic_);
-    map.getValue(VOLUMESPKR_KEY, &volumespkr_);
-    map.getValue(NOISE_REDUCE_KEY, &noisereduce_);
-    map.getValue(ECHO_CANCEL_KEY, &echocancel_);
 
-    Conf::MappingNode *alsamap =(Conf::MappingNode *) map.getValue("alsa");
+    const auto clamp = [] (double min, double max, double val) {
+        return std::min(max, std::max(min, val));
+    };
+
+    volumemic_ = clamp(-1.0, 1.0, volumemic_);
+    map.getValue(VOLUMESPKR_KEY, &volumespkr_);
+    volumespkr_ = clamp(-1.0, 1.0, volumespkr_);
+    map.getValue(NOISE_REDUCE_KEY, &denoise_);
+    map.getValue(AGC_KEY, &agcEnabled_);
+    map.getValue(CAPTURE_MUTED_KEY, &captureMuted_);
+    map.getValue(PLAYBACK_MUTED_KEY, &playbackMuted_);
+
+    Conf::MappingNode *alsamap = (Conf::MappingNode *) map.getValue("alsa");
 
     if (alsamap) {
         alsamap->getValue(CARDIN_KEY, &alsaCardin_);
@@ -437,13 +523,14 @@ void AudioPreference::unserialize(const Conf::YamlNode &map)
     }
 
 #if HAVE_PULSE
-    Conf::MappingNode *pulsemap =(Conf::MappingNode *)(map.getValue("pulse"));
+    Conf::MappingNode *pulsemap = (Conf::MappingNode *)(map.getValue("pulse"));
 
     if (pulsemap) {
         pulsemap->getValue(DEVICE_PLAYBACK_KEY, &pulseDevicePlayback_);
         pulsemap->getValue(DEVICE_RECORD_KEY, &pulseDeviceRecord_);
         pulsemap->getValue(DEVICE_RINGTONE_KEY, &pulseDeviceRingtone_);
     }
+
 #endif
 }
 
