@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -34,6 +34,7 @@
 #include "config/serializable.h"
 #include <string>
 #include <map>
+#include <vector>
 
 class AudioLayer;
 
@@ -51,6 +52,12 @@ class Preferences : public Serializable {
             return accountOrder_;
         }
 
+        // flush invalid accountIDs from account order
+        void verifyAccountOrder(const std::vector<std::string> &accounts);
+
+        void addAccount(const std::string &acc);
+        void removeAccount(const std::string &acc);
+
         void setAccountOrder(const std::string &ord) {
             accountOrder_ = ord;
         }
@@ -308,34 +315,50 @@ class AudioPreference : public Serializable {
             alwaysRecording_ = rec;
         }
 
-        int getVolumemic() const {
+        double getVolumemic() const {
             return volumemic_;
         }
-        void setVolumemic(int m) {
+        void setVolumemic(double m) {
             volumemic_ = m;
         }
 
-        int getVolumespkr() const {
+        double getVolumespkr() const {
             return volumespkr_;
         }
-        void setVolumespkr(int s) {
+        void setVolumespkr(double s) {
             volumespkr_ = s;
         }
 
+        bool isAGCEnabled() const {
+            return agcEnabled_;
+        }
+
+        void setAGCState(bool enabled) {
+            agcEnabled_ = enabled;
+        }
+
         bool getNoiseReduce() const {
-            return noisereduce_;
+            return denoise_;
         }
 
-        void setNoiseReduce(bool noise) {
-            noisereduce_ = noise;
+        void setNoiseReduce(bool enabled) {
+            denoise_ = enabled;
         }
 
-        bool getEchoCancel() const {
-            return echocancel_;
+        bool getCaptureMuted() const {
+            return captureMuted_;
         }
 
-        void setEchoCancel(bool echo) {
-            echocancel_ = echo;
+        void setCaptureMuted(bool muted) {
+            captureMuted_ = muted;
+        }
+
+        bool getPlaybackMuted() const {
+            return playbackMuted_;
+        }
+
+        void setPlaybackMuted(bool muted) {
+            playbackMuted_= muted;
         }
 
     private:
@@ -356,11 +379,13 @@ class AudioPreference : public Serializable {
         // general preference
         std::string recordpath_; //: /home/msavard/Bureau
         bool alwaysRecording_;
-        int volumemic_;
-        int volumespkr_;
+        double volumemic_;
+        double volumespkr_;
 
-        bool noisereduce_;
-        bool echocancel_;
+        bool denoise_;
+        bool agcEnabled_;
+        bool captureMuted_;
+        bool playbackMuted_;
 };
 
 class ShortcutPreferences : public Serializable {
