@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Pierre-Luc Bacon <pierre-luc.bacon@savoirfairelinux.com>
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *
@@ -32,7 +32,7 @@
 #include "pattern.h"
 
 #include <cstdio>
-#include <tr1/memory>
+#include <memory>
 #include <iostream>
 #include <sstream>
 #include <algorithm>
@@ -61,46 +61,44 @@ std::vector<CryptoAttribute *> SdesNegotiator::parse()
     // syntax :
     //a=crypto:tag 1*WSP crypto-suite 1*WSP key-params *(1*WSP session-param)
 
-    std::tr1::shared_ptr<Pattern> generalSyntaxPattern, tagPattern, cryptoSuitePattern,
+    std::unique_ptr<Pattern> generalSyntaxPattern, tagPattern, cryptoSuitePattern,
         keyParamsPattern;
 
     try {
         // used to match white space (which are used as separator)
-        generalSyntaxPattern.reset(new Pattern("[\x20\x09]+", "g"));
+        generalSyntaxPattern.reset(new Pattern("[\x20\x09]+", true));
 
-        tagPattern.reset(new Pattern("^a=crypto:(?P<tag>[0-9]{1,9})"));
+        tagPattern.reset(new Pattern("^a=crypto:(?P<tag>[0-9]{1,9})", false));
 
         cryptoSuitePattern.reset(new Pattern(
             "(?P<cryptoSuite>AES_CM_128_HMAC_SHA1_80|" \
             "AES_CM_128_HMAC_SHA1_32|" \
             "F8_128_HMAC_SHA1_80|" \
-            "[A-Za-z0-9_]+)")); // srtp-crypto-suite-ext
+            "[A-Za-z0-9_]+)", false)); // srtp-crypto-suite-ext
 
         keyParamsPattern.reset(new Pattern(
             "(?P<srtpKeyMethod>inline|[A-Za-z0-9_]+)\\:" \
             "(?P<srtpKeyInfo>[A-Za-z0-9\x2B\x2F\x3D]+)"	 \
             "(\\|2\\^(?P<lifetime>[0-9]+)\\|"		 \
             "(?P<mkiValue>[0-9]+)\\:"			 \
-            "(?P<mkiLength>[0-9]{1,3})\\;?)?", "g"));
+            "(?P<mkiLength>[0-9]{1,3})\\;?)?", true));
 
     } catch (const CompileError& exception) {
         throw ParseError("A compile exception occured on a pattern.");
     }
 
-
     // Take each line from the vector
     // and parse its content
 
     std::vector<CryptoAttribute *> cryptoAttributeVector;
 
-    for (std::vector<std::string>::iterator iter = remoteAttribute_.begin();
-            iter != remoteAttribute_.end(); ++iter) {
+    for (const auto &item : remoteAttribute_) {
 
         // Split the line into its component
         // that we will analyze further down.
         std::vector<std::string> sdesLine;
 
-        *generalSyntaxPattern << (*iter);
+        generalSyntaxPattern->updateSubject(item);
 
         try {
             sdesLine = generalSyntaxPattern->split();
@@ -113,7 +111,7 @@ std::vector<CryptoAttribute *> SdesNegotiator::parse()
 
         // Check if the attribute starts with a=crypto
         // and get the tag for this line
-        *tagPattern << sdesLine.at(0);
+        tagPattern->updateSubject(sdesLine.at(0));
 
         std::string tag;
 
@@ -128,7 +126,7 @@ std::vector<CryptoAttribute *> SdesNegotiator::parse()
 
         // Check if the crypto suite is valid and retreive
         // its value.
-        *cryptoSuitePattern << sdesLine.at(1);
+        cryptoSuitePattern->updateSubject(sdesLine.at(1));
 
         std::string cryptoSuite;
 
@@ -142,7 +140,7 @@ std::vector<CryptoAttribute *> SdesNegotiator::parse()
             return cryptoAttributeVector;
 
         // Parse one or more key-params field.
-        *keyParamsPattern << sdesLine.at(2);
+        keyParamsPattern->updateSubject(sdesLine.at(2));
 
         std::string srtpKeyInfo;
         std::string srtpKeyMethod;
