@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
@@ -39,6 +39,8 @@
 #include "manager.h"
 
 #include <algorithm>
+#include "sipaccount.h"
+
 
 #ifdef SFL_VIDEO
 #include "video/libav_utils.h"
@@ -60,7 +62,7 @@ Sdp::Sdp(pj_pool_t *pool)
     , video_codec_list_()
     , sessionAudioMedia_()
     , sessionVideoMedia_()
-    , localIpAddr_()
+    , publishedIpAddr_()
     , remoteIpAddr_()
     , localAudioDataPort_(0)
     , localAudioControlPort_(0)
@@ -73,11 +75,19 @@ Sdp::Sdp(pj_pool_t *pool)
     , telephoneEventPayload_(101) // same as asterisk
 {}
 
+Sdp::~Sdp()
+{
+    SIPAccount::releasePort(localAudioDataPort_);
+#ifdef SFL_VIDEO
+    SIPAccount::releasePort(localVideoDataPort_);
+#endif
+}
+
 namespace {
     bool hasPayload(const std::vector<sfl::AudioCodec*> &codecs, int pt)
     {
-        for (std::vector<sfl::AudioCodec*>::const_iterator i = codecs.begin(); i != codecs.end(); ++i)
-            if (*i and (*i)->getPayloadType() == pt)
+        for (const auto &i : codecs)
+            if (i and i->getPayloadType() == pt)
                 return true;
         return false;
     }
@@ -110,7 +120,7 @@ void Sdp::setActiveLocalSdpSession(const pjmedia_sdp_session *sdp)
 
             if (!pj_stricmp2(&current->desc.media, "audio")) {
                 const unsigned long pt = pj_strtoul(&current->desc.fmt[fmt]);
-                if (not hasPayload(sessionAudioMedia_, pt)) {
+                if (pt != telephoneEventPayload_ and not hasPayload(sessionAudioMedia_, pt)) {
                     sfl::AudioCodec *codec = Manager::instance().audioCodecFactory.getCodec(pt);
                     if (codec)
                         sessionAudioMedia_.push_back(codec);
@@ -194,9 +204,10 @@ string Sdp::getAudioCodecNames() const
     return result;
 }
 
-void Sdp::getSessionAudioMedia(std::vector<sfl::AudioCodec*> &codecs) const
+std::vector<sfl::AudioCodec*>
+Sdp::getSessionAudioMedia() const
 {
-    codecs = sessionAudioMedia_;
+    return sessionAudioMedia_;
 }
 
 
@@ -214,16 +225,19 @@ Sdp::setMediaDescriptorLines(bool audio)
     int dynamic_payload = 96;
 
     med->desc.fmt_count = audio ? audio_codec_list_.size() : video_codec_list_.size();
+
     for (unsigned i = 0; i < med->desc.fmt_count; ++i) {
         unsigned clock_rate;
         string enc_name;
         int payload;
+        const char *channels = "";
 
         if (audio) {
             sfl::AudioCodec *codec = audio_codec_list_[i];
             payload = codec->getPayloadType();
             enc_name = codec->getMimeSubtype();
-            clock_rate = codec->getClockRate();
+            clock_rate = codec->getSDPClockRate();
+            channels = codec->getSDPChannels();
             // G722 require G722/8000 media description even if it is 16000 codec
             if (codec->getPayloadType () == 9)
                 clock_rate = 8000;
@@ -247,13 +261,14 @@ Sdp::setMediaDescriptorLines(bool audio)
         rtpmap.pt = med->desc.fmt[i];
         rtpmap.enc_name = pj_str((char*) enc_name.c_str());
         rtpmap.clock_rate = clock_rate;
-        rtpmap.param.ptr = ((char* const)"");
-        rtpmap.param.slen = 0;
+        rtpmap.param.ptr = (char *) channels;
+        rtpmap.param.slen = strlen(channels); // don't include NULL terminator
 
         pjmedia_sdp_attr *attr;
         pjmedia_sdp_rtpmap_to_attr(memPool_, &rtpmap, &attr);
 
         med->attr[med->attr_count++] = attr;
+
 #ifdef SFL_VIDEO
         if (enc_name == "H264") {
             std::ostringstream os;
@@ -286,7 +301,7 @@ Sdp::setMediaDescriptorLines(bool audio)
 void Sdp::addRTCPAttribute(pjmedia_sdp_media *med)
 {
     std::ostringstream os;
-    os << localIpAddr_ << ":" << localAudioControlPort_;
+    os << publishedIpAddr_ << ":" << localAudioControlPort_;
     const std::string str(os.str());
     pj_str_t input_str = pj_str((char*) str.c_str());
     pj_sockaddr outputAddr;
@@ -300,6 +315,18 @@ void Sdp::addRTCPAttribute(pjmedia_sdp_media *med)
         pjmedia_sdp_attr_add(&med->attr_count, med->attr, attr);
 }
 
+void
+Sdp::setPublishedIP(const std::string &ip_addr)
+{
+    publishedIpAddr_ = ip_addr;
+    if (localSession_) {
+        localSession_->origin.addr = pj_str((char*) publishedIpAddr_.c_str());
+        localSession_->conn->addr = localSession_->origin.addr;
+        if (pjmedia_sdp_validate(localSession_) != PJ_SUCCESS)
+            ERROR("Could not validate SDP");
+    }
+}
+
 void
 Sdp::updatePorts(const std::vector<pj_sockaddr_in> &sockets)
 {
@@ -307,11 +334,30 @@ Sdp::updatePorts(const std::vector<pj_sockaddr_in> &sockets)
     localAudioControlPort_  = pj_ntohs(sockets[1].sin_port);
     localVideoDataPort_     = pj_ntohs(sockets[2].sin_port);
     localVideoControlPort_  = pj_ntohs(sockets[3].sin_port);
+
+    if (localSession_) {
+        if (localSession_->media[0]) {
+            localSession_->media[0]->desc.port = localAudioDataPort_;
+            // update RTCP attribute
+            if (pjmedia_sdp_media_remove_all_attr(localSession_->media[0], "rtcp"))
+                addRTCPAttribute(localSession_->media[0]);
+        }
+        if (localSession_->media[1])
+            localSession_->media[1]->desc.port = localVideoDataPort_;
+
+        if (not pjmedia_sdp_validate(localSession_))
+            ERROR("Could not validate SDP");
+    }
 }
 
 
 void Sdp::setTelephoneEventRtpmap(pjmedia_sdp_media *med)
 {
+    std::ostringstream s;
+    s << telephoneEventPayload_;
+    ++med->desc.fmt_count;
+    pj_strdup2(memPool_, &med->desc.fmt[med->desc.fmt_count - 1], s.str().c_str());
+
     pjmedia_sdp_attr *attr_rtpmap = static_cast<pjmedia_sdp_attr *>(pj_pool_zalloc(memPool_, sizeof(pjmedia_sdp_attr)));
     attr_rtpmap->name = pj_str((char *) "rtpmap");
     attr_rtpmap->value = pj_str((char *) "101 telephone-event/8000");
@@ -344,8 +390,8 @@ void Sdp::setLocalMediaAudioCapabilities(const vector<int> &selectedCodecs)
         WARN("No selected codec while building local SDP offer");
 
     audio_codec_list_.clear();
-    for (vector<int>::const_iterator i = selectedCodecs.begin(); i != selectedCodecs.end(); ++i) {
-        sfl::AudioCodec *codec = Manager::instance().audioCodecFactory.getCodec(*i);
+    for (const auto &i : selectedCodecs) {
+        sfl::AudioCodec *codec = Manager::instance().audioCodecFactory.getCodec(i);
 
         if (codec)
             audio_codec_list_.push_back(codec);
@@ -382,7 +428,7 @@ int Sdp::createLocalSession(const vector<int> &selectedAudioCodecs, const vector
     localSession_->origin.id = tv.sec + 2208988800UL;
     localSession_->origin.net_type = pj_str((char*) "IN");
     localSession_->origin.addr_type = pj_str((char*) "IP4");
-    localSession_->origin.addr = pj_str((char*) localIpAddr_.c_str());
+    localSession_->origin.addr = pj_str((char*) publishedIpAddr_.c_str());
 
     localSession_->name = pj_str((char*) PACKAGE);
 
@@ -449,8 +495,9 @@ void Sdp::receiveOffer(const pjmedia_sdp_session* remote,
 
     remoteSession_ = pjmedia_sdp_session_clone(memPool_, remote);
 
-    pjmedia_sdp_neg_create_w_remote_offer(memPool_, localSession_,
-                                          remoteSession_, &negotiator_);
+    if (pjmedia_sdp_neg_create_w_remote_offer(memPool_, localSession_,
+            remoteSession_, &negotiator_) != PJ_SUCCESS)
+        ERROR("Failed to initialize negotiator");
 }
 
 void Sdp::startNegotiation()
@@ -499,9 +546,9 @@ string Sdp::getLineFromSession(const pjmedia_sdp_session *sess, const string &ke
     int size = pjmedia_sdp_print(sess, buffer, sizeof buffer);
     string sdp(buffer, size);
     const vector<string> tokens(split(sdp, '\n'));
-    for (vector<string>::const_iterator iter = tokens.begin(); iter != tokens.end(); ++iter)
-        if ((*iter).find(keyword) != string::npos)
-            return *iter;
+    for (const auto &item : tokens)
+        if (item.find(keyword) != string::npos)
+            return item;
     return "";
 }
 
@@ -613,9 +660,8 @@ Sdp::getProfileLevelID(const pjmedia_sdp_session *session,
 
 void Sdp::addSdesAttribute(const vector<std::string>& crypto)
 {
-    for (vector<std::string>::const_iterator iter = crypto.begin();
-            iter != crypto.end(); ++iter) {
-        pj_str_t val = { (char*)(*iter).c_str(), static_cast<pj_ssize_t>((*iter).size()) };
+    for (const auto &item : crypto) {
+        pj_str_t val = { (char*) item.c_str(), static_cast<pj_ssize_t>(item.size()) };
         pjmedia_sdp_attr *attr = pjmedia_sdp_attr_create(memPool_, "crypto", &val);
 
         for (unsigned i = 0; i < localSession_->media_count; i++)
@@ -759,6 +805,8 @@ bool Sdp::getOutgoingVideoSettings(map<string, string> &args) const
         }
         return true;
     }
+#else
+    (void) args;
 #endif
     return false;
 }
