@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Alexandre Bourget <alexandre.bourget@savoirfairelinux.com>
@@ -35,13 +35,19 @@
 #ifndef SIPACCOUNT_H
 #define SIPACCOUNT_H
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <vector>
 #include <map>
-#include "account.h"
+
 #include "pjsip/sip_transport_tls.h"
 #include "pjsip/sip_types.h"
 #include "pjsip-ua/sip_regc.h"
+
 #include "noncopyable.h"
+#include "account.h"
 
 typedef std::vector<pj_ssl_cipher> CipherArray;
 
@@ -57,6 +63,11 @@ namespace Conf {
     const char *const DTMF_TYPE_KEY = "dtmfType";
     const char *const SERVICE_ROUTE_KEY = "serviceRoute";
     const char *const KEEP_ALIVE_ENABLED = "keepAlive";
+    const char *const PRESENCE_ENABLED_KEY = "presenceEnabled";
+    const char *const PRESENCE_PUBLISH_SUPPORTED_KEY = "presencePublishSupported";
+    const char *const PRESENCE_SUBSCRIBE_SUPPORTED_KEY = "presenceSubscribeSupported";
+    const char *const PRESENCE_STATUS_KEY = "presenceStatus";
+    const char *const PRESENCE_NOTE_KEY = "presenceNote";
 
     // TODO: write an object to store credential which implement serializable
     const char *const SRTP_KEY = "srtp";
@@ -90,14 +101,25 @@ namespace Conf {
     const char *const STUN_ENABLED_KEY = "stunEnabled";
     const char *const STUN_SERVER_KEY = "stunServer";
     const char *const CRED_KEY = "credential";
+    const char *const AUDIO_PORT_MIN_KEY = "audioPortMin";
+    const char *const AUDIO_PORT_MAX_KEY = "audioPortMax";
+#ifdef SFL_VIDEO
+    const char *const VIDEO_PORT_MIN_KEY = "videoPortMin";
+    const char *const VIDEO_PORT_MAX_KEY = "videoPortMax";
+#endif
 }
 
 class SIPVoIPLink;
+class SIPPresence;
+
 
 /**
  * @file sipaccount.h
  * @brief A SIP Account specify SIP specific functions and object = SIPCall/SIPVoIPLink)
  */
+enum {MAX_PORT = 65536};
+enum {HALF_MAX_PORT = MAX_PORT / 2};
+enum class MatchRank {NONE, PARTIAL, FULL};
 
 class SIPAccount : public Account {
     public:
@@ -110,20 +132,31 @@ class SIPAccount : public Account {
          * Constructor
          * @param accountID The account identifier
          */
-        SIPAccount(const std::string& accountID);
+        SIPAccount(const std::string& accountID, bool presenceEnabled);
+        ~SIPAccount();
 
         virtual VoIPLink* getVoIPLink();
 
+        pjsip_host_port getHostPortFromSTUN(pj_pool_t *pool);
+
         std::string getUserAgentName() const;
         void setRegistrationStateDetailed(const std::pair<int, std::string> &details) {
             registrationStateDetailed_ = details;
         }
 
+        void updateDialogViaSentBy(pjsip_dialog *dlg);
+
+        void resetAutoRegistration();
+        bool checkNATAddress(pjsip_regc_cbparam *param, pj_pool_t *pool);
+
         /**
          * Returns true if this is the IP2IP account
          */
         bool isIP2IP() const;
 
+        static void
+        releasePort(uint16_t port);
+
         /**
          * Serialize internal state of this account for configuration
          * @param YamlEmitter the configuration engine which generate the configuration file
@@ -136,12 +169,6 @@ class SIPAccount : public Account {
          */
         virtual void unserialize(const Conf::YamlNode &map);
 
-        /**
-         * Set the internal state for this account, mainly used to manage account details from the client application.
-         * @param The map containing the account information.
-         */
-        virtual void setAccountDetails(std::map<std::string, std::string> details);
-
         /**
          * Return an map containing the internal state of this account. Client application can use this method to manage
          * account info.
@@ -194,7 +221,7 @@ class SIPAccount : public Account {
 
 
         const pjsip_cred_info* getCredInfo() const {
-            return &(*cred_.begin());
+            return cred_.data();
         }
 
         /**
@@ -283,6 +310,8 @@ class SIPAccount : public Account {
             regc_ = regc;
         }
 
+        void destroyRegistrationInfo();
+
         /**
          * @return pjsip_tls_setting structure, filled from the configuration
          * file, that can be used directly by PJSIP to initialize
@@ -368,7 +397,7 @@ class SIPAccount : public Account {
          * Get the contact header for
          * @return pj_str_t The contact header based on account information
          */
-        std::string getContactHeader() const;
+        pj_str_t getContactHeader();
 
         /**
          * Get the local interface name on which this account is bound.
@@ -513,21 +542,65 @@ class SIPAccount : public Account {
         pjsip_transport* transport_;
 
         /* Returns true if the username and/or hostname match this account */
-        bool matches(const std::string &username, const std::string &hostname, pjsip_endpoint *endpt, pj_pool_t *pool) const;
+        MatchRank matches(const std::string &username, const std::string &hostname, pjsip_endpoint *endpt, pj_pool_t *pool) const;
+
+#ifdef SFL_PRESENCE
+        /**
+         * Presence management
+         */
+        SIPPresence * getPresence() const;
+
+        /**
+         * Activate the module.
+         * @param function Publish or subscribe to enable
+         * @param enable Flag
+         */
+        void enablePresence(const bool& enable);
+        /**
+         * Activate the publish/subscribe.
+         * @param enable Flag
+         */
+        void supportPresence(int function, bool enable);
+#endif
+
+//        unsigned generateAudioPort() const;
+        uint16_t generateAudioPort() const;
+#ifdef SFL_VIDEO
+        uint16_t generateVideoPort() const;
+#endif
+
+        void scheduleReregistration(pjsip_endpoint *endpt);
 
     private:
+        /**
+         * Set the internal state for this account, mainly used to manage account details from the client application.
+         * @param The map containing the account information.
+         */
+        void setAccountDetails(const std::map<std::string, std::string> &details);
+
         NON_COPYABLE(SIPAccount);
 
+        void usePublishedAddressPortInVIA();
         bool fullMatch(const std::string &username, const std::string &hostname, pjsip_endpoint *endpt, pj_pool_t *pool) const;
         bool userMatch(const std::string &username) const;
         bool hostnameMatch(const std::string &hostname, pjsip_endpoint *endpt, pj_pool_t *pool) const;
         bool proxyMatch(const std::string &hostname, pjsip_endpoint *endpt, pj_pool_t *pool) const;
 
+        struct {
+            pj_bool_t    active;    /**< Flag of reregister status. */
+            pj_timer_entry   timer;     /**< Timer for reregistration.  */
+            void        *reg_tp;    /**< Transport for registration.    */
+            unsigned     attempt_cnt; /**< Attempt counter.     */
+        } auto_rereg_;           /**< Reregister/reconnect data. */
+
+        static void autoReregTimerCb(pj_timer_heap_t *th, pj_timer_entry *te);
+
         /**
          * Map of credential for this account
          */
         std::vector< std::map<std::string, std::string > > credentials_;
 
+#if HAVE_TLS
         /**
          * Maps a string description of the SSL method
          * to the corresponding enum value in pjsip_ssl_method.
@@ -542,9 +615,11 @@ class SIPAccount : public Account {
         void initTlsConfiguration();
 
         /**
-         * Display the list of ciphers currently supported on the
+         * PJSIP aborts if the string length of our cipher list is too
+         * great, so this function forces our cipher list to fit this constraint.
          */
-        void displayCipherSuite();
+        void trimCiphers();
+#endif
 
         /**
          * Initializes STUN config from the config file
@@ -630,9 +705,9 @@ class SIPAccount : public Account {
         pjsip_tls_setting tlsSetting_;
 
         /**
-         * Allocate a static array to be used by pjsip to store the supported ciphers on this system.
+         * Allocate a vector to be used by pjsip to store the supported ciphers on this system.
          */
-        CipherArray ciphers;
+        CipherArray ciphers_;
 
         /**
          * The STUN server name (hostname)
@@ -653,7 +728,7 @@ class SIPAccount : public Account {
          * Determine if TLS is enabled for this account. TLS provides a secured channel for
          * SIP signalization. It is independant than the media encription provided by SRTP or ZRTP.
          */
-        std::string tlsEnable_;
+        bool tlsEnable_;
 
         /**
          * Certificate autority file
@@ -759,6 +834,36 @@ class SIPAccount : public Account {
          * Optional: via_addr construct from received parameters
          */
         pjsip_host_port via_addr_;
+
+        char contactBuffer_[PJSIP_MAX_URL_SIZE];
+        pj_str_t contact_;
+        int contactRewriteMethod_;
+        bool allowViaRewrite_;
+        /* Undocumented feature in pjsip, this can == 2 */
+        int allowContactRewrite_;
+        pjsip_transport *via_tp_;
+
+        /*
+         * Port range for audio RTP ports
+         */
+        std::pair<uint16_t, uint16_t> audioPortRange_;
+
+#ifdef SFL_VIDEO
+        /**
+         * Port range for video RTP ports
+         */
+        std::pair<uint16_t, uint16_t> videoPortRange_;
+#endif
+
+#ifdef SFL_PRESENCE
+        /**
+         * Presence data structure
+         */
+        SIPPresence * presence_;
+#endif
+
+        static bool portsInUse_[HALF_MAX_PORT];
+        static uint16_t getRandomEvenNumber(const std::pair<uint16_t, uint16_t> &range);
 };
 
 #endif
