@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Yun Liu <yun.liu@savoirfairelinux.com>
@@ -41,7 +41,7 @@
 #endif
 
 #include <map>
-#include <pthread.h>
+#include <mutex>
 
 #include "pjsip.h"
 #include "pjlib.h"
@@ -91,6 +91,10 @@ class SIPVoIPLink : public VoIPLink {
          */
         static void setSipLogLevel();
 
+#ifdef __ANDROID__
+        static void setSipLogger();
+#endif
+
         /**
          * Event listener. Each event send by the call manager is received and handled from here
          */
@@ -282,19 +286,22 @@ class SIPVoIPLink : public VoIPLink {
          */
         void createDefaultSipUdpTransport();
 
+    public:
         /**
          * Instance that maintain and manage transport (UDP, TLS)
          */
-        SipTransport sipTransport;
+        std::unique_ptr<SipTransport> sipTransport;
 
 #ifdef SFL_VIDEO
         static void enqueueKeyframeRequest(const std::string &callID);
 #endif
 
         std::string
-        getAccountIdFromNameAndServer(const std::string &userName,
-                                      const std::string &server) const;
-
+        guessAccountIdFromNameAndServer(const std::string &userName,
+                                        const std::string &server) const;
+        int getModId();
+        pjsip_endpoint * getEndpoint();
+        pjsip_module * getMod();
     private:
 
         NON_COPYABLE(SIPVoIPLink);
@@ -307,7 +314,7 @@ class SIPVoIPLink : public VoIPLink {
          */
         AccountMap sipAccountMap_;
 
-        pthread_mutex_t sipCallMapMutex_;
+        std::mutex sipCallMapMutex_;
         SipCallMap sipCallMap_;
 
         /**
@@ -325,11 +332,10 @@ class SIPVoIPLink : public VoIPLink {
 #ifdef SFL_VIDEO
         void dequeKeyframeRequests();
         void requestKeyframe(const std::string &callID);
-        pthread_mutex_t keyframeRequestsMutex_;
+        std::mutex keyframeRequestsMutex_;
         std::queue<std::string> keyframeRequests_;
 #endif
 
-        static bool destroyed_;
         static SIPVoIPLink *instance_;
 
         friend class SIPTest;
