@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Tristan Matthews <tristan.matthews@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -33,12 +33,8 @@
 
 #include "logger.h"
 
-// cast to void to avoid compiler warnings about unused return values
-#define set_false_atomic(x) static_cast<void>(__sync_fetch_and_and(x, false))
-#define atomic_increment(x) static_cast<void>(__sync_fetch_and_add(x, 1))
-#define atomic_decrement(x) static_cast<void>(__sync_fetch_and_sub(x, 1))
-
 // If condition A is false, print the error message in M and exit thread
-#define EXIT_IF_FAIL(A, M, ...) if (!(A)) { ERROR(M, ##__VA_ARGS__); set_false_atomic(&threadRunning_); pthread_exit(NULL); }
+#define EXIT_IF_FAIL(A, M, ...) if (!(A)) { \
+        ERROR(M, ##__VA_ARGS__); this->exit(); }
 
 #endif // CHECK_H_
