@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Tristan Matthews <tristan.matthews@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -35,20 +35,25 @@
 #include <map>
 #include <string>
 
+
 namespace libav_utils {
-    void
-    sfl_avcodec_init();
+    void sfl_avcodec_init();
 
-    std::map<std::string, std::string>
-    encodersMap();
+    int libav_pixel_format(int fmt);
+    int sfl_pixel_format(int fmt);
 
-    std::vector<std::string>
-    getVideoCodecList();
+    std::map<std::string, std::string> encodersMap();
+
+    std::vector<std::string> getVideoCodecList();
+
+    std::vector<std::map <std::string, std::string> > getDefaultCodecs();
 
-    std::vector<std::map<std::string, std::string> >
-    getDefaultCodecs();
     const char *const DEFAULT_H264_PROFILE_LEVEL_ID = "profile-level-id=428014";
     const char *const MAX_H264_PROFILE_LEVEL_ID = "profile-level-id=640034";
+
+    void sfl_url_split(const char *url,
+                      char *hostname, size_t hostname_size, int *port,
+                      char *path, size_t path_size);
 }
 
 #endif // __LIBAV_UTILS_H__
