@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2012-2013 Savoir-Faire Linux Inc.
  *  Author: Tristan Matthews <tristan.matthews@savoirfairelinux.com>
  *
  *  Portions derived from GStreamer:
@@ -37,10 +37,12 @@
 #include "config.h"
 #endif
 
+#include "video_scaler.h"
+
 #include "shm_sink.h"
 #include "shm_header.h"
 #include "logger.h"
-#include "video_provider.h"
+
 #include <sys/mman.h>
 #include <fcntl.h>
 #include <cstdio>
@@ -49,21 +51,22 @@
 #include <cerrno>
 #include <cstring>
 
+namespace sfl_video {
+
 SHMSink::SHMSink(const std::string &shm_name) :
-    shm_name_(shm_name),
-    fd_(-1),
-    shm_area_(static_cast<SHMHeader*>(MAP_FAILED)),
-    shm_area_len_(0),
-    opened_name_()
-    {}
+    shm_name_(shm_name)
+    , fd_(-1)
+    , shm_area_(static_cast<SHMHeader*>(MAP_FAILED))
+    , shm_area_len_(0)
+    , opened_name_()
+{}
 
 SHMSink::~SHMSink()
 {
     stop();
 }
 
-bool
-SHMSink::start()
+bool SHMSink::start()
 {
     if (fd_ != -1) {
         ERROR("fd must be -1");
@@ -76,8 +79,8 @@ SHMSink::start()
     if (not shm_name_.empty()) {
         fd_ = shm_open(shm_name_.c_str(), flags, perms);
         if (fd_ < 0) {
-            ERROR("could not open shm area \"%s\", shm_open failed:%s", shm_name_.c_str(), strerror(errno));
-            perror(strerror(errno));
+            ERROR("could not open shm area \"%s\"", shm_name_.c_str());
+            Logger::strErr();
             return false;
         }
     } else {
@@ -87,7 +90,7 @@ SHMSink::start()
             shm_name_ = name.str();
             fd_ = shm_open(shm_name_.c_str(), flags, perms);
             if (fd_ < 0 and errno != EEXIST) {
-                ERROR("%s", strerror(errno));
+                Logger::strErr();
                 return false;
             }
         }
@@ -100,7 +103,7 @@ SHMSink::start()
 
     if (ftruncate(fd_, shm_area_len_)) {
         ERROR("Could not make shm area large enough for header");
-        perror(strerror(errno));
+        Logger::strErr();
         return false;
     }
 
@@ -123,11 +126,11 @@ SHMSink::start()
     return true;
 }
 
-bool
-SHMSink::stop()
+bool SHMSink::stop()
 {
-    if (fd_ >= 0)
-        close(fd_);
+    if (fd_ >= 0 and close(fd_) == -1)
+        Logger::strErr();
+
     fd_ = -1;
 
     if (not opened_name_.empty()) {
@@ -143,23 +146,22 @@ SHMSink::stop()
     return true;
 }
 
-bool
-SHMSink::resize_area(size_t desired_length)
+bool SHMSink::resize_area(size_t desired_length)
 {
-    if (desired_length < shm_area_len_)
+    if (desired_length <= shm_area_len_)
         return true;
 
     shm_unlock();
 
     if (munmap(shm_area_, shm_area_len_)) {
         ERROR("Could not unmap shared area");
-        perror(strerror(errno));
+        Logger::strErr();
         return false;
     }
 
     if (ftruncate(fd_, desired_length)) {
         ERROR("Could not resize shared area");
-        perror(strerror(errno));
+        Logger::strErr();
         return false;
     }
 
@@ -183,14 +185,38 @@ void SHMSink::render(const std::vector<unsigned char> &data)
     if (!resize_area(sizeof(SHMHeader) + data.size()))
         return;
 
-    memcpy(shm_area_->data, &(*data.begin()), data.size());
+    memcpy(shm_area_->data, data.data(), data.size());
     shm_area_->buffer_size = data.size();
     shm_area_->buffer_gen++;
     sem_post(&shm_area_->notification);
     shm_unlock();
 }
 
-void SHMSink::render_callback(sfl_video::VideoProvider &provider, size_t bytes)
+void SHMSink::render_frame(VideoFrame& src)
+{
+    VideoFrame dst;
+    VideoScaler scaler;
+
+    dst.setGeometry(src.getWidth(), src.getHeight(), VIDEO_PIXFMT_BGRA);
+    size_t bytes = dst.getSize();
+
+    shm_lock();
+
+    if (!resize_area(sizeof(SHMHeader) + bytes)) {
+        ERROR("Could not resize area");
+        return;
+    }
+
+    dst.setDestination(shm_area_->data);
+    scaler.scale(src, dst);
+
+    shm_area_->buffer_size = bytes;
+    shm_area_->buffer_gen++;
+    sem_post(&shm_area_->notification);
+    shm_unlock();
+}
+
+void SHMSink::render_callback(VideoProvider &provider, size_t bytes)
 {
     shm_lock();
 
@@ -207,11 +233,12 @@ void SHMSink::render_callback(sfl_video::VideoProvider &provider, size_t bytes)
 }
 
 void SHMSink::shm_lock()
-{
-    sem_wait(&shm_area_->mutex);
-}
+{ sem_wait(&shm_area_->mutex); }
 
 void SHMSink::shm_unlock()
-{
-    sem_post(&shm_area_->mutex);
+{ sem_post(&shm_area_->mutex); }
+
+void SHMSink::update(Observable<std::shared_ptr<VideoFrame> >* /*obs*/, std::shared_ptr<VideoFrame> &frame_p)
+{ render_frame(*frame_p); }
+
 }
