@@ -1,6 +1,7 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Tristan Matthews <tristan.matthews@savoirfairelinux.com>
+ *  Author: Guillaume Roguez <Guillaume.Roguez@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,32 +29,35 @@
  *  as that of the covered work.
  */
 
+#include "client/video_controls.h"
 #include "video_rtp_session.h"
-#include <sstream>
-#include <map>
-#include <string>
-#include "video_send_thread.h"
+#include "video_sender.h"
 #include "video_receive_thread.h"
+#include "video_mixer.h"
 #include "socket_pair.h"
 #include "sip/sdp.h"
 #include "sip/sipvoiplink.h"
 #include "manager.h"
 #include "logger.h"
 
+#include <sstream>
+#include <map>
+#include <string>
+
 namespace sfl_video {
 
 using std::map;
 using std::string;
 
-VideoRtpSession::VideoRtpSession(const string &callID, const map<string, string> &txArgs) :
-    socketPair_(), sendThread_(), receiveThread_(), txArgs_(txArgs),
-    rxArgs_(), sending_(false), receiving_(false), callID_(callID)
+VideoRtpSession::VideoRtpSession(const string &callID,
+								 const map<string, string> &txArgs) :
+    socketPair_(), sender_(), receiveThread_(), txArgs_(txArgs),
+    rxArgs_(), sending_(false), receiving_(false), callID_(callID),
+    videoMixerSP_(), videoLocal_()
 {}
 
 VideoRtpSession::~VideoRtpSession()
-{
-    stop();
-}
+{ stop(); }
 
 void VideoRtpSession::updateSDP(const Sdp &sdp)
 {
@@ -109,7 +113,7 @@ void VideoRtpSession::updateDestination(const string &destination,
     tmp << "rtp://" << destination << ":" << port;
     // if destination has changed
     if (tmp.str() != txArgs_["destination"]) {
-        if (sendThread_.get() != 0) {
+        if (sender_) {
             ERROR("Video is already being sent");
             return;
         }
@@ -131,23 +135,42 @@ void VideoRtpSession::start(int localPort)
     try {
         socketPair_.reset(new SocketPair(txArgs_["destination"].c_str(), localPort));
     } catch (const std::runtime_error &e) {
-        ERROR("Socket creation failed: %s", e.what());
+        ERROR("Socket creation failed on port %d: %s", localPort, e.what());
         return;
     }
 
-    if (sending_) {
-        if (sendThread_.get())
+	if (sending_) {
+        // Local video startup if needed
+        auto videoCtrl = Manager::instance().getVideoControls();
+        const bool firstStart = not videoCtrl->hasCameraStarted();
+        videoCtrl->startCamera();
+        if (firstStart)
+            MYSLEEP(1);
+
+        videoLocal_ = videoCtrl->getVideoCamera();
+        if (sender_)
             WARN("Restarting video sender");
-        sendThread_.reset(new VideoSendThread("local", txArgs_));
-        sendThread_->addIOContext(*socketPair_);
-        sendThread_->start();
-    } else {
+
+        try {
+            sender_.reset(new VideoSender(callID_, txArgs_, *socketPair_));
+        } catch (const VideoSenderException &e) {
+            ERROR("%s", e.what());
+            sending_ = false;
+        }
+    }
+
+    /* sending may have been set to false in previous block */
+    if (not sending_) {
         DEBUG("Video sending disabled");
-        sendThread_.reset();
+        if (auto shared = videoLocal_.lock())
+            shared->detach(sender_.get());
+        if (videoMixerSP_)
+            videoMixerSP_->detach(sender_.get());
+        sender_.reset();
     }
 
     if (receiving_) {
-        if (receiveThread_.get())
+        if (receiveThread_)
             WARN("restarting video receiver");
         receiveThread_.reset(new VideoReceiveThread(callID_, rxArgs_));
         receiveThread_->setRequestKeyFrameCallback(&SIPVoIPLink::enqueueKeyframeRequest);
@@ -155,32 +178,94 @@ void VideoRtpSession::start(int localPort)
         receiveThread_->start();
     } else {
         DEBUG("Video receiving disabled");
+        if (receiveThread_)
+            receiveThread_->detach(videoMixerSP_.get());
         receiveThread_.reset();
     }
+
+    // Setup pipeline
+    if (videoMixerSP_) {
+        setupConferenceVideoPipeline();
+    } else if (auto shared = videoLocal_.lock()) {
+        if (sender_)
+            shared->attach(sender_.get());
+    }
 }
 
 void VideoRtpSession::stop()
 {
-    if (socketPair_.get())
+    if (auto shared = videoLocal_.lock())
+        shared->detach(sender_.get());
+
+    if (videoMixerSP_) {
+        videoMixerSP_->detach(sender_.get());
+        if (receiveThread_)
+            receiveThread_->detach(videoMixerSP_.get());
+    }
+
+    if (socketPair_)
         socketPair_->interrupt();
+
     receiveThread_.reset();
-    sendThread_.reset();
+    sender_.reset();
     socketPair_.reset();
+    auto videoCtrl = Manager::instance().getVideoControls();
+    videoCtrl->stopCamera();
 }
 
 void VideoRtpSession::forceKeyFrame()
 {
-    if (sendThread_.get())
-        sendThread_->forceKeyFrame();
-    else
-        ERROR("Video sending thread is NULL");
+    if (sender_)
+        sender_->forceKeyFrame();
 }
 
-void
-VideoRtpSession::addReceivingDetails(std::map<std::string, std::string> &details)
+void VideoRtpSession::addReceivingDetails(std::map<std::string, std::string> &details)
 {
-    if (receiveThread_.get())
-        receiveThread_->addDetails(details);
+    if (receiveThread_)
+        receiveThread_->addReceivingDetails(details);
+}
+
+void VideoRtpSession::setupConferenceVideoPipeline()
+{
+    if (sender_) {
+        videoMixerSP_->setDimensions(atol(txArgs_["width"].c_str()),
+                                     atol(txArgs_["height"].c_str()));
+        if (auto shared = videoLocal_.lock())
+            shared->detach(sender_.get());
+        videoMixerSP_->attach(sender_.get());
+
+        if (receiveThread_) {
+            receiveThread_->enterConference();
+            receiveThread_->attach(videoMixerSP_.get());
+        }
+    }
+}
+
+void VideoRtpSession::enterConference(Conference *conf)
+{
+    /* Detach from a possible previous conference */
+    exitConference();
+    videoMixerSP_ = std::move(conf->getVideoMixer());
+
+    setupConferenceVideoPipeline();
+}
+
+void VideoRtpSession::exitConference()
+{
+    if (videoMixerSP_) {
+        if (sender_)
+            videoMixerSP_->detach(sender_.get());
+
+        if (receiveThread_) {
+            receiveThread_->detach(videoMixerSP_.get());
+            receiveThread_->exitConference();
+        }
+
+        videoMixerSP_.reset();
+    }
+
+    if (auto shared = videoLocal_.lock())
+        shared->attach(sender_.get());
 }
 
 } // end namespace sfl_video
