@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2011-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2011-2013 Savoir-Faire Linux Inc.
  *  Copyright © 2009 Rémi Denis-Courmont
  *
  *  Author: Rafaël Carré <rafael.carre@savoirfairelinux.com>
@@ -34,9 +34,9 @@
 #include <stdexcept> // for std::runtime_error
 #include <sstream>
 #include <algorithm>
+#include <unistd.h>
 
 #include "logger.h"
-#include "scoped_lock.h"
 
 #include <libudev.h>
 #include <cstring>
@@ -51,13 +51,12 @@ extern "C" {
 
 #include "video_v4l2_list.h"
 #include "manager.h"
-#include "dbus/video_controls.h"
+#include "client/video_controls.h"
 
 namespace sfl_video {
 
 using std::vector;
 using std::string;
-using sfl::ScopedLock;
 
 static int is_v4l2(struct udev_device *dev)
 {
@@ -67,10 +66,9 @@ static int is_v4l2(struct udev_device *dev)
 }
 
 VideoV4l2ListThread::VideoV4l2ListThread() : devices_(),
-    thread_(0), mutex_(), udev_(0),
+    thread_(), mutex_(), udev_(0),
     udev_mon_(0), probing_(false)
 {
-    pthread_mutex_init(&mutex_, NULL);
     udev_list_entry *devlist;
     udev_enumerate *devenum;
 
@@ -149,18 +147,9 @@ udev_failed:
 void VideoV4l2ListThread::start()
 {
     probing_ = true;
-    pthread_create(&thread_, NULL, &runCallback, this);
+    thread_ = std::thread(&VideoV4l2ListThread::run, this);
 }
 
-
-void *VideoV4l2ListThread::runCallback(void *data)
-{
-    VideoV4l2ListThread *context = static_cast<VideoV4l2ListThread*>(data);
-    context->run();
-    return NULL;
-}
-
-
 namespace {
 
     typedef std::vector<VideoV4l2Device> Devices;
@@ -194,8 +183,8 @@ namespace {
     void giveUniqueName(VideoV4l2Device &dev, const vector<VideoV4l2Device> &devices)
     {
 start:
-        for (size_t i = 0; i < devices.size(); ++i) {
-            if (dev.name == devices[i].name) {
+        for (auto &item : devices) {
+            if (dev.name == item.name) {
                 size_t sharp;
                 int num = getNumber(dev.name, &sharp);
                 if (num < 0) // not numbered
@@ -214,21 +203,38 @@ start:
 VideoV4l2ListThread::~VideoV4l2ListThread()
 {
     probing_ = false;
-    if (thread_)
-        pthread_join(thread_, NULL);
+    if (thread_.joinable())
+        thread_.join();
     if (udev_mon_)
         udev_monitor_unref(udev_mon_);
     if (udev_)
         udev_unref(udev_);
+}
 
-    pthread_mutex_destroy(&mutex_);
+void VideoV4l2ListThread::updateDefault()
+{
+    const std::string &name = devices_.back().name;
+    auto controls = Manager::instance().getVideoControls();
+    controls->setActiveDevice(name);
+    const auto channel = devices_.back().getChannelList()[0];
+    controls->setActiveDeviceChannel(channel);
+    const auto size = devices_.back().getChannel(name).getSizeList()[0];
+    controls->setActiveDeviceSize(size);
+    const auto rateList(controls->getDeviceRateList(name, channel, size));
+    // compare by integer value
+    const auto highest = std::max_element(rateList.begin(), rateList.end(), []
+            (const std::string &l, const std::string &r) {
+                return atoi(l.c_str()) < atoi(r.c_str());
+            });
+
+    Manager::instance().getVideoControls()->setActiveDeviceRate(*highest);
 }
 
 void VideoV4l2ListThread::run()
 {
     if (!udev_mon_) {
         probing_ = false;
-        pthread_exit(NULL);
+        return;
     }
 
     const int udev_fd = udev_monitor_get_fd(udev_mon_);
@@ -255,8 +261,10 @@ void VideoV4l2ListThread::run()
                     if (!strcmp(action, "add")) {
                         DEBUG("udev: adding %s", node);
                         try {
-                            addDevice(node);
-                            Manager::instance().getVideoControls()->deviceEvent();
+                            if (addDevice(node)) {
+                                updateDefault();
+                                Manager::instance().getVideoControls()->deviceEvent();
+                            }
                         } catch (const std::runtime_error &e) {
                             ERROR("%s", e.what());
                         }
@@ -273,36 +281,39 @@ void VideoV4l2ListThread::run()
                     continue;
                 ERROR("udev monitoring thread: select failed (%m)");
                 probing_ = false;
-                pthread_exit(NULL);
+                return;
 
             default:
                 ERROR("select() returned %d (%m)", ret);
                 probing_ = false;
-                pthread_exit(NULL);
+                return;
         }
     }
 }
 
 void VideoV4l2ListThread::delDevice(const string &node)
 {
-    ScopedLock lock(mutex_);
+    std::lock_guard<std::mutex> lock(mutex_);
 
-    for (std::vector<VideoV4l2Device>::iterator itr = devices_.begin(); itr != devices_.end(); ++itr) {
-        if (itr->device == node) {
-            devices_.erase(itr);
-            Manager::instance().getVideoControls()->deviceEvent();
-            return;
-        }
+    const auto itr = std::find_if(devices_.begin(), devices_.end(),
+            [&] (const VideoV4l2Device &d) { return d.device == node; });
+
+    if (itr != devices_.end()) {
+        devices_.erase(itr);
+        Manager::instance().getVideoControls()->deviceEvent();
     }
 }
 
 bool VideoV4l2ListThread::addDevice(const string &dev)
 {
-    ScopedLock lock(mutex_);
+    std::lock_guard<std::mutex> lock(mutex_);
 
     int fd = open(dev.c_str(), O_RDWR);
-    if (fd == -1)
+    if (fd == -1) {
+        ERROR("Problem opening device");
+        Logger::strErr();
         return false;
+    }
 
     const string s(dev);
     VideoV4l2Device v(fd, s);
@@ -316,7 +327,7 @@ bool VideoV4l2ListThread::addDevice(const string &dev)
 vector<string>
 VideoV4l2ListThread::getChannelList(const string &dev)
 {
-    ScopedLock lock(mutex_);
+    std::lock_guard<std::mutex> lock(mutex_);
     Devices::const_iterator iter(findDevice(dev));
     if (iter != devices_.end())
         return iter->getChannelList();
@@ -327,7 +338,7 @@ VideoV4l2ListThread::getChannelList(const string &dev)
 vector<string>
 VideoV4l2ListThread::getSizeList(const string &dev, const string &channel)
 {
-    ScopedLock lock(mutex_);
+    std::lock_guard<std::mutex> lock(mutex_);
     Devices::const_iterator iter(findDevice(dev));
     if (iter != devices_.end())
         return iter->getChannel(channel).getSizeList();
@@ -338,7 +349,7 @@ VideoV4l2ListThread::getSizeList(const string &dev, const string &channel)
 vector<string>
 VideoV4l2ListThread::getRateList(const string &dev, const string &channel, const std::string &size)
 {
-    ScopedLock lock(mutex_);
+    std::lock_guard<std::mutex> lock(mutex_);
     Devices::const_iterator iter(findDevice(dev));
     if (iter != devices_.end())
         return iter->getChannel(channel).getSize(size).getRateList();
@@ -348,11 +359,11 @@ VideoV4l2ListThread::getRateList(const string &dev, const string &channel, const
 
 vector<string> VideoV4l2ListThread::getDeviceList()
 {
-    ScopedLock lock(mutex_);
+    std::lock_guard<std::mutex> lock(mutex_);
     vector<string> v;
 
-    for (std::vector<VideoV4l2Device>::iterator itr = devices_.begin(); itr != devices_.end(); ++itr)
-       v.push_back(itr->name.empty() ? itr->device : itr->name);
+    for (const auto &itr : devices_)
+       v.push_back(itr.name.empty() ? itr.device : itr.name);
 
     return v;
 }
@@ -368,7 +379,7 @@ VideoV4l2ListThread::findDevice(const string &name) const
 
 unsigned VideoV4l2ListThread::getChannelNum(const string &dev, const string &name)
 {
-    ScopedLock lock(mutex_);
+    std::lock_guard<std::mutex> lock(mutex_);
     Devices::const_iterator iter(findDevice(dev));
     if (iter != devices_.end())
         return iter->getChannel(name).idx;
@@ -378,7 +389,7 @@ unsigned VideoV4l2ListThread::getChannelNum(const string &dev, const string &nam
 
 string VideoV4l2ListThread::getDeviceNode(const string &name)
 {
-    ScopedLock lock(mutex_);
+    std::lock_guard<std::mutex> lock(mutex_);
     Devices::const_iterator iter(findDevice(name));
     if (iter != devices_.end())
         return iter->device;
