@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2011-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2011-2013 Savoir-Faire Linux Inc.
  *  Author: Rafaël Carré <rafael.carre@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -36,6 +36,8 @@
 #include <libudev.h>
 
 #include "video_v4l2.h"
+#include <mutex>
+#include <thread>
 #include "noncopyable.h"
 
 namespace sfl_video {
@@ -55,16 +57,16 @@ class VideoV4l2ListThread {
         unsigned getChannelNum(const std::string &dev, const std::string &name);
 
     private:
-        static void *runCallback(void *);
         void run();
+        void updateDefault();
 
         std::vector<VideoV4l2Device>::const_iterator findDevice(const std::string &name) const;
         NON_COPYABLE(VideoV4l2ListThread);
         void delDevice(const std::string &node);
         bool addDevice(const std::string &dev);
         std::vector<VideoV4l2Device> devices_;
-        pthread_t thread_;
-        pthread_mutex_t mutex_;
+        std::thread thread_;
+        std::mutex mutex_;
 
         udev *udev_;
         udev_monitor *udev_mon_;
