@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -36,7 +36,7 @@
 #include "resamplertest.h"
 
 ResamplerTest::ResamplerTest() :
-    CppUnit::TestCase("Resampler module test"), inputBuffer(), outputBuffer()
+    CppUnit::TestCase("Resampler module test"), inputBuffer(MAX_BUFFER_LENGTH, 1, 8000), outputBuffer(MAX_BUFFER_LENGTH, 1, 8000)
 {}
 
 void ResamplerTest::setUp()
@@ -53,9 +53,11 @@ namespace {
     template <typename T>
     void print_buffer(T &buffer)
     {
+#ifdef VERBOSE
         std::copy(buffer.begin(), buffer.end(),
-                std::ostream_iterator<SFLDataFormat>(std::cout, ", "));
+                std::ostream_iterator<SFLAudioSample>(std::cout, ", "));
         std::cout << std::endl;
+#endif
     }
 }
 
@@ -69,16 +71,16 @@ void ResamplerTest::testUpsamplingRamp()
 
     performUpsampling(converter);
 
-    LowSmplrBuffer tmpInputBuffer;
-    HighSmplrBuffer tmpOutputBuffer;
+    AudioBuffer tmpInputBuffer(TMP_LOWSMPLR_BUFFER_LENGTH, 1, 8000);
+    AudioBuffer tmpOutputBuffer(TMP_HIGHSMPLR_BUFFER_LENGTH, 1, 16000);
 
-    std::copy(inputBuffer.begin(), inputBuffer.begin() + tmpInputBuffer.size(), tmpInputBuffer.begin());
+    tmpInputBuffer.copy(inputBuffer);
     std::cout << "Input Buffer" << std::endl;
-    print_buffer(tmpInputBuffer);
+    print_buffer(*tmpInputBuffer.getChannel(0));
 
-    std::copy(outputBuffer.begin(), outputBuffer.begin() + tmpOutputBuffer.size(), tmpOutputBuffer.begin());
+    tmpOutputBuffer.copy(outputBuffer);
     std::cout << "Output Buffer" << std::endl;
-    print_buffer(tmpOutputBuffer);
+    print_buffer(*tmpOutputBuffer.getChannel(0));
 }
 
 void ResamplerTest::testDownsamplingRamp()
@@ -90,16 +92,16 @@ void ResamplerTest::testDownsamplingRamp()
 
     performDownsampling(converter);
 
-    HighSmplrBuffer tmpInputBuffer;
-    LowSmplrBuffer tmpOutputBuffer;
+    AudioBuffer tmpInputBuffer(TMP_HIGHSMPLR_BUFFER_LENGTH, 1, 16000);
+    AudioBuffer tmpOutputBuffer(TMP_LOWSMPLR_BUFFER_LENGTH, 1, 8000);
 
-    std::copy(inputBuffer.begin(), inputBuffer.begin() + tmpInputBuffer.size(), tmpInputBuffer.begin());
+    tmpInputBuffer.copy(inputBuffer);
     std::cout << "Input Buffer" << std::endl;
-    print_buffer(tmpInputBuffer);
+    print_buffer(*tmpInputBuffer.getChannel(0));
 
-    std::copy(outputBuffer.begin(), outputBuffer.begin() + tmpOutputBuffer.size(), tmpOutputBuffer.begin());
+    tmpOutputBuffer.copy(outputBuffer);
     std::cout << "Output Buffer" << std::endl;
-    print_buffer(tmpOutputBuffer);
+    print_buffer(*tmpOutputBuffer.getChannel(0));
 }
 
 void ResamplerTest::testUpsamplingTriangle()
@@ -111,16 +113,16 @@ void ResamplerTest::testUpsamplingTriangle()
 
     performUpsampling(converter);
 
-    LowSmplrBuffer tmpInputBuffer;
-    HighSmplrBuffer tmpOutputBuffer;
+    AudioBuffer tmpInputBuffer(TMP_LOWSMPLR_BUFFER_LENGTH, 1, 8000);
+    AudioBuffer tmpOutputBuffer(TMP_HIGHSMPLR_BUFFER_LENGTH, 1, 16000);
 
-    std::copy(inputBuffer.begin(), inputBuffer.begin() + tmpInputBuffer.size(), tmpInputBuffer.begin());
+    tmpInputBuffer.copy(inputBuffer);
     std::cout << "Input Buffer" << std::endl;
-    print_buffer(tmpInputBuffer);
+    print_buffer(*tmpInputBuffer.getChannel(0));
 
-    std::copy(outputBuffer.begin(), outputBuffer.begin() + tmpOutputBuffer.size(), tmpOutputBuffer.begin());
+    tmpOutputBuffer.copy(outputBuffer);
     std::cout << "Output Buffer" << std::endl;
-    print_buffer(tmpOutputBuffer);
+    print_buffer(*tmpOutputBuffer.getChannel(0));
 }
 
 void ResamplerTest::testDownsamplingTriangle()
@@ -132,16 +134,16 @@ void ResamplerTest::testDownsamplingTriangle()
 
     performDownsampling(converter);
 
-    HighSmplrBuffer tmpInputBuffer;
-    LowSmplrBuffer tmpOutputBuffer;
+    AudioBuffer tmpInputBuffer(TMP_HIGHSMPLR_BUFFER_LENGTH, 1, 16000);
+    AudioBuffer tmpOutputBuffer(TMP_LOWSMPLR_BUFFER_LENGTH, 1, 8000);
 
-    std::copy(inputBuffer.begin(), inputBuffer.begin() + tmpInputBuffer.size(), tmpInputBuffer.begin());
+    tmpInputBuffer.copy(inputBuffer);
     std::cout << "Input Buffer" << std::endl;
-    print_buffer(tmpInputBuffer);
+    print_buffer(*tmpInputBuffer.getChannel(0));
 
-    std::copy(outputBuffer.begin(), outputBuffer.begin() + tmpOutputBuffer.size(), tmpOutputBuffer.begin());
+    tmpOutputBuffer.copy(outputBuffer);
     std::cout << "Output Buffer" << std::endl;
-    print_buffer(tmpOutputBuffer);
+    print_buffer(*tmpOutputBuffer.getChannel(0));
 }
 void ResamplerTest::testUpsamplingSine()
 {
@@ -153,16 +155,16 @@ void ResamplerTest::testUpsamplingSine()
 
     performUpsampling(converter);
 
-    LowSmplrBuffer tmpInputBuffer;
-    HighSmplrBuffer tmpOutputBuffer;
+    AudioBuffer tmpInputBuffer(TMP_LOWSMPLR_BUFFER_LENGTH, 1, 8000);
+    AudioBuffer tmpOutputBuffer(TMP_HIGHSMPLR_BUFFER_LENGTH, 1, 16000);
 
-    std::copy(inputBuffer.begin(), inputBuffer.begin() + tmpInputBuffer.size(), tmpInputBuffer.begin());
+    tmpInputBuffer.copy(inputBuffer);
     std::cout << "Input Buffer" << std::endl;
-    print_buffer(tmpInputBuffer);
+    print_buffer(*tmpInputBuffer.getChannel(0));
 
-    std::copy(outputBuffer.begin(), outputBuffer.begin() + tmpOutputBuffer.size(), tmpOutputBuffer.begin());
+    tmpOutputBuffer.copy(outputBuffer);
     std::cout << "Output Buffer" << std::endl;
-    print_buffer(tmpOutputBuffer);
+    print_buffer(*tmpOutputBuffer.getChannel(0));
 }
 
 void ResamplerTest::testDownsamplingSine()
@@ -175,56 +177,59 @@ void ResamplerTest::testDownsamplingSine()
 
     performDownsampling(converter);
 
-    HighSmplrBuffer tmpInputBuffer;
-    LowSmplrBuffer tmpOutputBuffer;
+    AudioBuffer tmpInputBuffer(TMP_HIGHSMPLR_BUFFER_LENGTH, 1, 16000);
+    AudioBuffer tmpOutputBuffer(TMP_LOWSMPLR_BUFFER_LENGTH, 1, 8000);
 
-    std::copy(inputBuffer.begin(), inputBuffer.begin() + tmpInputBuffer.size(), tmpInputBuffer.begin());
+    tmpInputBuffer.copy(inputBuffer);
     std::cout << "Input Buffer" << std::endl;
-    print_buffer(tmpInputBuffer);
+    print_buffer(*tmpInputBuffer.getChannel(0));
 
-    std::copy(outputBuffer.begin(), outputBuffer.begin() + tmpOutputBuffer.size(), tmpOutputBuffer.begin());
+    tmpOutputBuffer.copy(outputBuffer);
     std::cout << "Output Buffer" << std::endl;
-    print_buffer(tmpOutputBuffer);
+    print_buffer(*tmpOutputBuffer.getChannel(0));
 }
 
 void ResamplerTest::generateRamp()
 {
-    for (size_t i = 0; i < inputBuffer.size(); ++i)
-        inputBuffer[i] = i;
+    std::vector<SFLAudioSample>* buf = inputBuffer.getChannel(0);
+    for (size_t i = 0; i < buf->size(); ++i)
+        (*buf)[i] = i;
 }
 
 void ResamplerTest::generateTriangularSignal()
 {
-    for (size_t i = 0; i < inputBuffer.size(); ++i)
-        inputBuffer[i] = i * 10;
+    std::vector<SFLAudioSample>* buf = inputBuffer.getChannel(0);
+    for (size_t i = 0; i < buf->size(); ++i)
+        (*buf)[i] = i * 10;
 }
 
 void ResamplerTest::generateSineSignal()
 {
-    for (size_t i = 0; i < inputBuffer.size(); ++i)
-        inputBuffer[i] = (SFLDataFormat) (1000.0 * sin(i));
+    std::vector<SFLAudioSample>* buf = inputBuffer.getChannel(0);
+    for (size_t i = 0; i < buf->size(); ++i)
+        (*buf)[i] = (SFLAudioSample) (1000.0 * sin(i));
 }
 
 void ResamplerTest::performUpsampling(SamplerateConverter &converter)
 {
-    LowSmplrBuffer tmpInputBuffer;
-    HighSmplrBuffer tmpOutputBuffer;
+    AudioBuffer tmpInputBuffer(TMP_LOWSMPLR_BUFFER_LENGTH, 1, 8000);
+    AudioBuffer tmpOutputBuffer(TMP_HIGHSMPLR_BUFFER_LENGTH, 1, 16000);
 
-    for (size_t i = 0, j = 0; i < (inputBuffer.size() / 2); i += tmpInputBuffer.size(), j += tmpOutputBuffer.size()) {
-        std::copy(inputBuffer.begin() + i, inputBuffer.begin() + tmpInputBuffer.size() + i, tmpInputBuffer.begin());
-        converter.resample(tmpInputBuffer.data(), tmpOutputBuffer.data(), tmpOutputBuffer.size(), 8000, 16000, tmpInputBuffer.size());
-        std::copy(tmpOutputBuffer.begin(), tmpOutputBuffer.end(), outputBuffer.begin() + j);
+    for (size_t i = 0, j = 0; i < (inputBuffer.frames() / 2); i += tmpInputBuffer.frames(), j += tmpOutputBuffer.frames()) {
+        tmpInputBuffer.copy(inputBuffer, i);
+        converter.resample(tmpInputBuffer, tmpOutputBuffer);
+        outputBuffer.copy(tmpOutputBuffer, -1, 0, j);
     }
 }
 
 void ResamplerTest::performDownsampling(SamplerateConverter &converter)
 {
-    HighSmplrBuffer tmpInputBuffer;
-    LowSmplrBuffer tmpOutputBuffer;
+    AudioBuffer tmpInputBuffer(TMP_HIGHSMPLR_BUFFER_LENGTH, 1, 16000);
+    AudioBuffer tmpOutputBuffer(TMP_LOWSMPLR_BUFFER_LENGTH, 1, 8000);
 
-    for (size_t i = 0, j = 0; i < inputBuffer.size(); i += tmpInputBuffer.size(), j += tmpOutputBuffer.size()) {
-        std::copy(inputBuffer.begin() + i, inputBuffer.begin() + tmpInputBuffer.size() + i, tmpInputBuffer.begin());
-        converter.resample(tmpInputBuffer.data(), tmpOutputBuffer.data(), tmpOutputBuffer.size(), 16000, 8000, tmpInputBuffer.size());
-        std::copy(tmpOutputBuffer.begin(), tmpOutputBuffer.end(), outputBuffer.begin() + j);
+    for (size_t i = 0, j = 0; i < inputBuffer.frames(); i += tmpInputBuffer.frames(), j += tmpOutputBuffer.frames()) {
+        tmpInputBuffer.copy(inputBuffer, i);
+        converter.resample(tmpInputBuffer, tmpOutputBuffer);
+        outputBuffer.copy(tmpOutputBuffer, -1, 0, j);
     }
 }
