@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2012-2013 Savoir-Faire Linux Inc.
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -36,8 +36,6 @@
 #include <cppunit/TestCase.h>
 #include <cppunit/TestSuite.h>
 
-#include <tr1/array>
-
 #include "audio/samplerateconverter.h"
 #include "noncopyable.h"
 
@@ -45,9 +43,6 @@
 #define TMP_LOWSMPLR_BUFFER_LENGTH 160
 #define TMP_HIGHSMPLR_BUFFER_LENGTH 320
 
-typedef std::tr1::array<SFLDataFormat, TMP_LOWSMPLR_BUFFER_LENGTH> LowSmplrBuffer;
-typedef std::tr1::array<SFLDataFormat, TMP_HIGHSMPLR_BUFFER_LENGTH> HighSmplrBuffer;
-
 class ResamplerTest : public CppUnit::TestCase {
 
     /**
@@ -138,12 +133,12 @@ private:
     /**
      * Used to store input samples
      */
-    std::tr1::array<SFLDataFormat, MAX_BUFFER_LENGTH> inputBuffer;
+    AudioBuffer inputBuffer;
 
     /**
      * Used to receive output samples
      */
-    std::tr1::array<SFLDataFormat, MAX_BUFFER_LENGTH> outputBuffer;
+    AudioBuffer outputBuffer;
 };
 
 /* Register the test module */
