@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -56,8 +56,8 @@ void SdesNegotiatorTest::testTagPattern()
     TITLE();
     std::string subject = "a=crypto:4";
 
-    sfl::Pattern pattern("^a=crypto:(?P<tag>[0-9]{1,9})");
-    pattern << subject;
+    sfl::Pattern pattern("^a=crypto:(?P<tag>[0-9]{1,9})", false);
+    pattern.updateSubject(subject);
 
     CPPUNIT_ASSERT(pattern.matches());
     CPPUNIT_ASSERT(pattern.group("tag").compare("4") == 0);
@@ -72,8 +72,8 @@ void SdesNegotiatorTest::testCryptoSuitePattern()
     sfl::Pattern pattern("(?P<cryptoSuite>AES_CM_128_HMAC_SHA1_80|" \
                                "AES_CM_128_HMAC_SHA1_32|"		\
                                "F8_128_HMAC_SHA1_80|"			\
-                               "[A-Za-z0-9_]+)");
-    pattern << subject;
+                               "[A-Za-z0-9_]+)", false);
+    pattern.updateSubject(subject);
 
     CPPUNIT_ASSERT(pattern.matches());
     CPPUNIT_ASSERT(pattern.group("cryptoSuite").compare("AES_CM_128_HMAC_SHA1_80") == 0);
@@ -90,9 +90,9 @@ void SdesNegotiatorTest::testKeyParamsPattern()
                                "(?P<srtpKeyInfo>[A-Za-z0-9\x2B\x2F\x3D]+)\\|" \
                                "(2\\^(?P<lifetime>[0-9]+)\\|"		\
                                "(?P<mkiValue>[0-9]+)\\:"		\
-                               "(?P<mkiLength>[0-9]{1,3})\\;?)?", "g");
+                               "(?P<mkiLength>[0-9]{1,3})\\;?)?", true);
 
-    pattern << subject;
+    pattern.updateSubject(subject);
 
     pattern.matches();
     CPPUNIT_ASSERT(pattern.group("srtpKeyMethod").compare("inline:"));
@@ -114,9 +114,9 @@ void SdesNegotiatorTest::testKeyParamsPatternWithoutMKI()
                                "(?P<srtpKeyInfo>[A-Za-z0-9\x2B\x2F\x3D]+)" \
                                "(\\|2\\^(?P<lifetime>[0-9]+)\\|"                \
                                "(?P<mkiValue>[0-9]+)\\:"                \
-                               "(?P<mkiLength>[0-9]{1,3})\\;?)?", "g");
+                               "(?P<mkiLength>[0-9]{1,3})\\;?)?", true);
 
-    pattern << subject;
+    pattern.updateSubject(subject);
     pattern.matches();
     CPPUNIT_ASSERT(pattern.group("srtpKeyMethod").compare("inline:"));
     CPPUNIT_ASSERT(pattern.group("srtpKeyInfo").compare("d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj")
