@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Pierre-Luc Beaudoin <pierre-luc.beaudoin@savoirfairelinux.com>
  *
@@ -130,12 +130,17 @@ status_bar_display_account()
     account_t *acc = account_list_get_current();
     status_tray_icon_online(acc != NULL);
 
+    static const char * const account_types[] = {
+        N_("(SIP)"),
+        N_("(IAX)")
+    };
+
     gchar* msg;
     if (acc) {
-        msg = g_markup_printf_escaped("%s %s (%s)" ,
-                                      _("Using account"),
+        const guint type_idx = account_is_IAX(acc);
+        msg = g_markup_printf_escaped(_("%s %s"),
                                       (gchar*) account_lookup(acc, CONFIG_ACCOUNT_ALIAS),
-                                      (gchar*) account_lookup(acc, CONFIG_ACCOUNT_TYPE));
+                                      _(account_types[type_idx]));
     } else {
         msg = g_markup_printf_escaped(_("No registered accounts"));
     }
@@ -341,8 +346,7 @@ sflphone_hang_up(SFLPhoneClient *client)
     statusbar_update_clock("");
 
     // Allow screen saver to start
-    guint nbcall = calllist_get_size(current_calls_tab);
-    if(nbcall == 1)
+    if (calllist_get_size(current_calls_tab) == 1)
         dbus_screensaver_uninhibit();
 }
 
@@ -352,8 +356,7 @@ sflphone_pick_up(SFLPhoneClient *client)
     callable_obj_t *selectedCall = calltab_get_selected_call(active_calltree_tab);
 
     // Disable screensaver if the list is empty call
-    guint nbcall = calllist_get_size(current_calls_tab);
-    if(nbcall == 0)
+    if (calllist_get_size(current_calls_tab) == 0)
         dbus_screensaver_inhibit();
 
     if (!selectedCall) {
@@ -506,7 +509,7 @@ sflphone_incoming_call(callable_obj_t * c, SFLPhoneClient *client)
     if (!account) {
         g_warning("Account is NULL");
     } else if (account_has_autoanswer_on(account)) {
-        calltab_set_selected_call(active_calltree_tab, c);
+        calltab_select_call(active_calltree_tab, c);
         sflphone_pick_up(client);
     }
 }
@@ -583,17 +586,18 @@ sflphone_new_call(SFLPhoneClient *client)
     if(nbcall == 0)
         dbus_screensaver_inhibit();
 
-    callable_obj_t *current_selected_call = calltab_get_selected_call(current_calls_tab);
+    callable_obj_t *selected_call = calllist_empty(current_calls_tab) ? NULL :
+        calltab_get_selected_call(current_calls_tab);
 
-    if (current_selected_call != NULL) {
-        gchar *confID = dbus_get_conference_id(current_selected_call->_callID);
+    if (selected_call != NULL) {
+        gchar *confID = dbus_get_conference_id(selected_call->_callID);
         if(g_strcmp0(confID, "") != 0) {
             sflphone_on_hold();
         }
     }
 
     // Play a tone when creating a new call
-    if (calllist_get_size(current_calls_tab) == 0)
+    if (calllist_empty(current_calls_tab))
         dbus_start_tone(TRUE , (current_account_has_new_message()  > 0) ? TONE_WITH_MESSAGE : TONE_WITHOUT_MESSAGE) ;
 
     callable_obj_t *c = create_new_call(CALL, CALL_STATE_DIALING, "", "", "", "");
@@ -611,7 +615,8 @@ sflphone_new_call(SFLPhoneClient *client)
 void
 sflphone_keypad(guint keyval, const gchar * key, SFLPhoneClient *client)
 {
-    callable_obj_t * c = calltab_get_selected_call(current_calls_tab);
+    callable_obj_t * c = calllist_empty(current_calls_tab) ? NULL :
+                         calltab_get_selected_call(current_calls_tab);
 
     const gboolean current_is_active_tab = calltab_has_name(active_calltree_tab, CURRENT_CALLS);
     if (!current_is_active_tab || (current_is_active_tab && !c)) {
@@ -744,7 +749,7 @@ sflphone_place_call(callable_obj_t * c, SFLPhoneClient *client)
 
     // Make sure the previously found account is registered, take first one registered elsewhere
     if (account) {
-        gpointer status = g_hash_table_lookup(account->properties, "Status");
+        const gchar *status = account_lookup(account, CONFIG_ACCOUNT_REGISTRATION_STATUS);
         if (!utf8_case_equal(status, "REGISTERED")) {
             // Place the call with the first registered account
             account = account_list_get_by_state(ACCOUNT_STATE_REGISTERED);
@@ -812,19 +817,19 @@ sflphone_add_main_participant(const conference_obj_t * c)
     dbus_add_main_participant(c->_confID);
 }
 
-void
+
+gboolean
 sflphone_rec_call(SFLPhoneClient *client)
 {
+    gboolean result = FALSE;
     callable_obj_t * selectedCall = calltab_get_selected_call(current_calls_tab);
     conference_obj_t * selectedConf = calltab_get_selected_conf(current_calls_tab);
 
     if (selectedCall) {
-        g_debug("Set record for selected call");
-        dbus_set_record(selectedCall->_callID);
+        result = dbus_toggle_recording(selectedCall->_callID);
         calltree_update_call(current_calls_tab, selectedCall, client, TRUE);
     } else if (selectedConf) {
-        g_debug("Set record for selected conf");
-        dbus_set_record(selectedConf->_confID);
+        result = dbus_toggle_recording(selectedConf->_confID);
 
         switch (selectedConf->_state) {
             case CONFERENCE_STATE_ACTIVE_ATTACHED:
@@ -851,14 +856,7 @@ sflphone_rec_call(SFLPhoneClient *client)
     } else {
         update_actions(client);
     }
-}
-
-void
-sflphone_mute_call()
-{
-    g_debug("%s", __PRETTY_FUNCTION__);
-
-    toggle_slider_mute_microphone();
+    return result;
 }
 
 static void
