@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Pierre-Luc Beaudoin <pierre-luc.beaudoin@savoirfairelinux.com>
  *  Author: Pierre-Luc Bacon <pierre-luc.bacon@savoirfairelinux.com>
@@ -56,6 +56,7 @@
 #include "tlsadvanceddialog.h"
 #include "dbus/dbus.h"
 #include "utils.h"
+#include "seekslider.h"
 
 /**
  * TODO: tidy this up
@@ -95,6 +96,17 @@ static GtkWidget *file_chooser;
 static GtkWidget *security_tab;
 static GtkWidget *advanced_tab;
 static GtkWidget *overrtp;
+static GtkWidget *ringtone_seekslider;
+static GtkWidget *audio_port_min_spin_box;
+static GtkWidget *audio_port_max_spin_box;
+#ifdef SFL_VIDEO
+static GtkWidget *video_port_min_spin_box;
+static GtkWidget *video_port_max_spin_box;
+#endif
+#ifdef SFL_PRESENCE
+static GtkWidget *presence_check_box;
+static gboolean is_account_new;
+#endif
 
 typedef struct OptionsData {
     account_t *account;
@@ -142,18 +154,32 @@ static void show_password_cb(G_GNUC_UNUSED GtkWidget *widget, gpointer data)
 }
 
 /* Signal to protocol_combo 'changed' */
-void change_protocol_cb()
+static void change_protocol_cb(G_GNUC_UNUSED GtkWidget *widget, gpointer data)
 {
     gchar *protocol = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(protocol_combo));
-
     // Only if tabs are not NULL
     if (security_tab && advanced_tab) {
         if (utf8_case_equal(protocol, "IAX")) {
             gtk_widget_hide(security_tab);
             gtk_widget_hide(advanced_tab);
+#ifdef SFL_PRESENCE
+            gtk_widget_set_sensitive(presence_check_box, FALSE);
+            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(presence_check_box),FALSE);
+#endif
         } else {
             gtk_widget_show(security_tab);
             gtk_widget_show(advanced_tab);
+#ifdef SFL_PRESENCE
+            if (data) {
+                account_t * account = data;
+                // the presence can be enabled when at least 1 presence feature is supported by the PBX
+                // OR when the account is new
+                gtk_widget_set_sensitive(presence_check_box,
+                        !g_strcmp0(account_lookup(account, CONFIG_PRESENCE_PUBLISH_SUPPORTED), "true") ||
+                        !g_strcmp0(account_lookup(account, CONFIG_PRESENCE_SUBSCRIBE_SUPPORTED), "true") ||
+                        is_account_new);
+            }
+#endif
         }
     }
 
@@ -171,7 +197,6 @@ select_dtmf_type(void)
 
 static GPtrArray* get_new_credential(void)
 {
-    gint row_count = 0;
     GPtrArray *credential_array = g_ptr_array_new();
 
     GtkTreeIter iter;
@@ -188,8 +213,6 @@ static GPtrArray* get_new_credential(void)
                            COLUMN_CREDENTIAL_PASSWORD, &password,
                            -1);
 
-        g_debug("Row %d: %s %s %s", row_count++, username, password, realm);
-
         GHashTable * new_table = g_hash_table_new_full(g_str_hash, g_str_equal,
                                                        g_free, g_free);
         g_hash_table_insert(new_table, g_strdup(CONFIG_ACCOUNT_REALM), realm);
@@ -213,16 +236,25 @@ static void update_credential_cb(GtkWidget *widget, G_GNUC_UNUSED gpointer data)
 }
 
 static GtkWidget*
-create_auto_answer_checkbox(account_t *account)
+create_auto_answer_checkbox(const account_t *account)
 {
     GtkWidget *auto_answer_checkbox = gtk_check_button_new_with_mnemonic(_("_Auto-answer calls"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(auto_answer_checkbox), account_has_autoanswer_on(account));
-    g_signal_connect(auto_answer_checkbox, "toggled", G_CALLBACK(auto_answer_cb), account);
+    g_signal_connect(auto_answer_checkbox, "toggled", G_CALLBACK(auto_answer_cb), (gpointer) account);
     return auto_answer_checkbox;
 }
 
+static void
+alias_changed_cb(GtkEditable *editable, gpointer data)
+{
+    const gchar *alias = gtk_entry_get_text(GTK_ENTRY(editable));
+    GtkDialog *dialog = GTK_DIALOG(data);
+    gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_APPLY, strlen(alias));
+}
+
+
 static GtkWidget*
-create_basic_tab(account_t *account, gboolean is_new)
+create_account_parameters(account_t *account, gboolean is_new, GtkWidget *dialog)
 {
     g_assert(account);
     gchar *password = NULL;
@@ -249,6 +281,10 @@ create_basic_tab(account_t *account, gboolean is_new)
     gtk_grid_attach(GTK_GRID(grid), label, 0, row, 1, 1);
     gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
     entry_alias = gtk_entry_new();
+    g_signal_connect(entry_alias, "changed", G_CALLBACK(alias_changed_cb), dialog);
+    /* make sure Apply is not sensitive while alias is empty */
+    g_signal_emit_by_name(entry_alias, "changed", NULL);
+
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry_alias);
     gchar *alias = account_lookup(account, CONFIG_ACCOUNT_ALIAS);
     gtk_entry_set_text(GTK_ENTRY(entry_alias), alias);
@@ -284,7 +320,7 @@ create_basic_tab(account_t *account, gboolean is_new)
 
     /* Link signal 'changed' */
     g_signal_connect(G_OBJECT(GTK_COMBO_BOX(protocol_combo)), "changed",
-                     G_CALLBACK(change_protocol_cb), NULL);
+                     G_CALLBACK(change_protocol_cb), account);
 
     row++;
     label = gtk_label_new_with_mnemonic(_("_Host name"));
@@ -376,12 +412,68 @@ create_basic_tab(account_t *account, gboolean is_new)
     GtkWidget *auto_answer_checkbox = create_auto_answer_checkbox(account);
     gtk_grid_attach(GTK_GRID(grid), auto_answer_checkbox, 0, row, 1, 1);
 
+
     gtk_widget_show_all(grid);
     gtk_container_set_border_width(GTK_CONTAINER(grid), 10);
 
     return frame;
 }
 
+#ifdef SFL_PRESENCE
+static GtkWidget*
+create_presence_checkbox(const account_t *account)
+{
+    g_assert(account);
+
+    GtkWidget *frame = gnome_main_section_new(_("Presence notifications"));
+    gtk_widget_show(frame);
+
+    GtkWidget *grid = gtk_grid_new();
+    gtk_grid_set_row_spacing(GTK_GRID(grid), 10);
+    gtk_grid_set_column_spacing(GTK_GRID(grid), 10);
+    gtk_widget_show(grid);
+    gtk_container_add(GTK_CONTAINER(frame), grid);
+
+    presence_check_box = gtk_check_button_new_with_mnemonic(_("_Enable"));
+
+    gboolean enabled = (g_strcmp0(account_lookup(account, CONFIG_PRESENCE_ENABLED), "true")==0);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(presence_check_box),enabled);
+    // sensitivity is set later in change_protocol_cb
+
+    gtk_grid_attach(GTK_GRID(grid), presence_check_box, 0, 0, 1, 1);
+
+    gtk_widget_show_all(grid);
+    gtk_container_set_border_width(GTK_CONTAINER(grid), 10);
+
+    return frame;
+}
+#endif
+
+static GtkWidget*
+create_basic_tab(account_t *account, gboolean is_new, GtkWidget *dialog)
+{
+    // Build the advanced tab, to appear on the account configuration panel
+    g_debug("Build basic tab");
+
+    GtkWidget *vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
+
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
+
+    GtkWidget *frame = create_account_parameters(account, is_new, dialog);
+    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
+
+#ifdef SFL_PRESENCE
+    frame = create_presence_checkbox(account);
+    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
+    is_account_new = is_new; // need a copy for a cb
+#endif
+
+    gtk_widget_show_all(vbox);
+
+    return vbox;
+}
+
+
 static void fill_treeview_with_credential(const account_t * account)
 {
     GtkTreeIter iter;
@@ -1030,6 +1122,40 @@ GtkWidget* create_published_address(const account_t *account)
     return frame;
 }
 
+static void
+add_port_spin_button(const account_t *account, GtkWidget *grid, GtkWidget **spin,
+                 const gchar *key, const gchar *label_text, int left, int top)
+{
+    gchar *value = NULL;
+
+    if (account)
+        value = account_lookup(account, key);
+
+    *spin = gtk_spin_button_new_with_range(1024, 65535, 1);
+    GtkWidget *label = gtk_label_new_with_mnemonic(label_text);
+    gtk_label_set_mnemonic_widget(GTK_LABEL(label), *spin);
+    gtk_grid_attach(GTK_GRID(grid), label, left, top, 1, 1);
+    if (value)
+        gtk_spin_button_set_value(GTK_SPIN_BUTTON(*spin), g_ascii_strtod(value, NULL));
+    gtk_grid_attach(GTK_GRID(grid), *spin, left + 1, top, 1, 1);
+}
+
+static GtkWidget*
+create_port_ranges(const account_t *account, const gchar * title,
+                   GtkWidget **min, GtkWidget **max,
+                   const gchar *key_min, const gchar *key_max, int top)
+{
+    GtkWidget *grid, *frame;
+    gnome_main_section_new_with_grid(title, &frame, &grid);
+    gtk_container_set_border_width(GTK_CONTAINER(grid), 10);
+    gtk_grid_set_row_spacing(GTK_GRID(grid), 5);
+
+    add_port_spin_button(account, grid, min, key_min, _("Min"), 0, top);
+    add_port_spin_button(account, grid, max, key_max, _("Max"), 2, top);
+
+    return frame;
+}
+
 GtkWidget* create_advanced_tab(const account_t *account)
 {
     // Build the advanced tab, to appear on the account configuration panel
@@ -1048,6 +1174,18 @@ GtkWidget* create_advanced_tab(const account_t *account)
     frame = create_published_address(account);
     gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
 
+    frame = create_port_ranges(account, _("Audio RTP Port Range"),
+            &audio_port_min_spin_box, &audio_port_max_spin_box,
+            CONFIG_ACCOUNT_AUDIO_PORT_MIN, CONFIG_ACCOUNT_AUDIO_PORT_MAX, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
+
+#ifdef SFL_VIDEO
+    frame = create_port_ranges(account, _("Video RTP Port Range"),
+            &video_port_min_spin_box, &video_port_max_spin_box,
+            CONFIG_ACCOUNT_VIDEO_PORT_MIN, CONFIG_ACCOUNT_VIDEO_PORT_MAX, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
+#endif
+
     gtk_widget_show_all(vbox);
 
     use_stun_cb(use_stun_check_box, NULL);
@@ -1063,6 +1201,18 @@ static void ringtone_enabled_cb(G_GNUC_UNUSED GtkWidget *widget, gpointer data,
     gtk_widget_set_sensitive(data, !gtk_widget_is_sensitive(data));
 }
 
+void update_ringtone_slider(guint position, guint size)
+{
+    if (ringtone_seekslider && GTK_IS_WIDGET(ringtone_seekslider))
+        sfl_seekslider_update_scale(SFL_SEEKSLIDER(ringtone_seekslider), position, size);
+}
+
+static void ringtone_changed_cb(GtkWidget *widget, gpointer data)
+{
+    GtkFileChooser *chooser = GTK_FILE_CHOOSER(widget);
+    SFLSeekSlider *seekslider = data;
+    g_object_set(seekslider, "file-path", gtk_file_chooser_get_filename(chooser), NULL);
+}
 
 static GtkWidget*
 create_audiocodecs_configuration(const account_t *account)
@@ -1112,24 +1262,36 @@ create_audiocodecs_configuration(const account_t *account)
     enable_tone = gtk_check_button_new_with_mnemonic(_("_Enable ringtones"));
     const gboolean ringtone_enabled = g_strcmp0(ptr, "true") == 0;
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enable_tone), ringtone_enabled);
-    g_signal_connect(G_OBJECT(enable_tone) , "clicked", G_CALLBACK(ringtone_enabled_cb), file_chooser);
+    g_signal_connect(G_OBJECT(enable_tone), "clicked", G_CALLBACK(ringtone_enabled_cb), file_chooser);
     gtk_grid_attach(GTK_GRID(grid), enable_tone, 0, 0, 1, 1);
 
     // file chooser button
-    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_chooser) , g_get_home_dir());
+    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_chooser), g_get_home_dir());
     ptr = account_lookup(account, CONFIG_RINGTONE_PATH);
-    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(file_chooser) , ptr);
+    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(file_chooser), ptr);
     gtk_widget_set_sensitive(file_chooser, ringtone_enabled);
 
+    // button to preview ringtones
+    ringtone_seekslider = GTK_WIDGET(sfl_seekslider_new());
+    g_object_set(G_OBJECT(ringtone_seekslider), "file-path", ptr, NULL);
+    g_signal_connect(G_OBJECT(file_chooser), "selection-changed", G_CALLBACK(ringtone_changed_cb), ringtone_seekslider);
+
     GtkFileFilter *filter = gtk_file_filter_new();
     gtk_file_filter_set_name(filter, _("Audio Files"));
     gtk_file_filter_add_pattern(filter, "*.wav");
     gtk_file_filter_add_pattern(filter, "*.ul");
     gtk_file_filter_add_pattern(filter, "*.au");
+    gtk_file_filter_add_pattern(filter, "*.flac");
+    gtk_file_filter_add_pattern(filter, "*.ogg");
 
     gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(file_chooser), filter);
     gtk_grid_attach(GTK_GRID(grid), file_chooser, 0, 1, 1, 1);
 
+    GtkWidget *alignment =  gtk_alignment_new(0.0, 0.0, 1.0, 1.0);
+    gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 0, 6, 6);
+    gtk_container_add(GTK_CONTAINER(alignment), ringtone_seekslider);
+    gtk_box_pack_start(GTK_BOX(vbox), alignment, FALSE, FALSE, 0);
+
     gtk_widget_show_all(vbox);
 
     return vbox;
@@ -1150,7 +1312,7 @@ create_videocodecs_configuration(const account_t *a)
     gtk_box_pack_start(GTK_BOX (vbox), videocodecs, FALSE, FALSE, 0);
     gtk_widget_set_size_request(GTK_WIDGET (videocodecs), -1, 200);
     gtk_widget_show(videocodecs);
-    gtk_container_add(GTK_CONTAINER (videocodecs) , box);
+    gtk_container_add(GTK_CONTAINER (videocodecs), box);
 
     gtk_widget_show_all(vbox);
 
@@ -1201,43 +1363,38 @@ static void update_account_from_basic_tab(account_t *account)
         proto = g_strdup("SIP");
 
     if (g_strcmp0(proto, "SIP") == 0) {
-        if (!account_is_IP2IP(account)) {
-            account_replace(account, CONFIG_ACCOUNT_REGISTRATION_EXPIRE,
-                            gtk_entry_get_text(GTK_ENTRY(expire_spin_box)));
+        account_replace(account, CONFIG_ACCOUNT_REGISTRATION_EXPIRE,
+                gtk_entry_get_text(GTK_ENTRY(expire_spin_box)));
 
-            account_replace(account, CONFIG_ACCOUNT_ROUTESET,
-                            gtk_entry_get_text(GTK_ENTRY(entry_route_set)));
+        account_replace(account, CONFIG_ACCOUNT_ROUTESET,
+                gtk_entry_get_text(GTK_ENTRY(entry_route_set)));
 
-            account_replace(account, CONFIG_ACCOUNT_USERAGENT,
-                            gtk_entry_get_text(GTK_ENTRY(entry_user_agent)));
+        gboolean v = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(use_stun_check_box));
+        account_replace(account, CONFIG_STUN_ENABLE,
+                bool_to_string(v));
 
-            gboolean v = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(use_stun_check_box));
-            account_replace(account, CONFIG_STUN_ENABLE,
-                            bool_to_string(v));
+        account_replace(account, CONFIG_STUN_SERVER,
+                gtk_entry_get_text(GTK_ENTRY(stun_server_entry)));
 
-            account_replace(account, CONFIG_STUN_SERVER,
-                            gtk_entry_get_text(GTK_ENTRY(stun_server_entry)));
+        v = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(same_as_local_radio_button));
+        account_replace(account, CONFIG_PUBLISHED_SAMEAS_LOCAL, bool_to_string(v));
 
-            v = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(same_as_local_radio_button));
-            account_replace(account, CONFIG_PUBLISHED_SAMEAS_LOCAL, bool_to_string(v));
+        account_replace(account, CONFIG_ACCOUNT_AUDIO_PORT_MIN,
+                gtk_entry_get_text(GTK_ENTRY(audio_port_min_spin_box)));
+        account_replace(account, CONFIG_ACCOUNT_AUDIO_PORT_MAX,
+                gtk_entry_get_text(GTK_ENTRY(audio_port_max_spin_box)));
+#ifdef SFL_VIDEO
+        account_replace(account, CONFIG_ACCOUNT_VIDEO_PORT_MIN,
+                gtk_entry_get_text(GTK_ENTRY(video_port_min_spin_box)));
+        account_replace(account, CONFIG_ACCOUNT_VIDEO_PORT_MAX,
+                gtk_entry_get_text(GTK_ENTRY(video_port_max_spin_box)));
+#endif
 
-            if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(same_as_local_radio_button))) {
-                account_replace(account, CONFIG_PUBLISHED_PORT,
-                                gtk_entry_get_text(GTK_ENTRY(published_port_spin_box)));
-
-                account_replace(account, CONFIG_PUBLISHED_ADDRESS,
-                                gtk_entry_get_text(GTK_ENTRY(published_address_entry)));
-            } else {
-                account_replace(account, CONFIG_PUBLISHED_PORT,
-                                gtk_entry_get_text(GTK_ENTRY(local_port_spin_box)));
-                gchar *local_interface = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(local_address_combo));
-
-                gchar *published_address = dbus_get_address_from_interface_name(local_interface);
-                g_free(local_interface);
-
-                account_replace(account, CONFIG_PUBLISHED_ADDRESS, published_address);
-            }
-        }
+#ifdef SFL_PRESENCE
+        v = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(presence_check_box));
+        account_replace(account, CONFIG_PRESENCE_ENABLED, bool_to_string(v));
+        // TODO enable/disable the presence window view
+#endif
 
         if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(overrtp))) {
             g_debug("Set dtmf over rtp");
@@ -1264,13 +1421,6 @@ static void update_account_from_basic_tab(account_t *account)
         const gboolean tls_enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(use_sip_tls_check_box));
         account_replace(account, CONFIG_TLS_ENABLE, bool_to_string(tls_enabled));
 
-        const gboolean tone_enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(enable_tone));
-        account_replace(account, CONFIG_RINGTONE_ENABLED, bool_to_string(tone_enabled));
-
-        gchar *ringtone_path = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));
-        account_replace(account, CONFIG_RINGTONE_PATH, ringtone_path);
-        g_free(ringtone_path);
-
         gchar *address_combo_text = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(local_address_combo));
         account_replace(account, CONFIG_LOCAL_INTERFACE, address_combo_text);
         g_free(address_combo_text);
@@ -1279,6 +1429,8 @@ static void update_account_from_basic_tab(account_t *account)
                         gtk_entry_get_text(GTK_ENTRY(local_port_spin_box)));
     }
 
+    account_replace(account, CONFIG_ACCOUNT_USERAGENT,
+                    gtk_entry_get_text(GTK_ENTRY(entry_user_agent)));
     account_replace(account, CONFIG_ACCOUNT_ALIAS, gtk_entry_get_text(GTK_ENTRY(entry_alias)));
     account_replace(account, CONFIG_ACCOUNT_TYPE, proto);
     account_replace(account, CONFIG_ACCOUNT_HOSTNAME, gtk_entry_get_text(GTK_ENTRY(entry_hostname)));
@@ -1288,15 +1440,48 @@ static void update_account_from_basic_tab(account_t *account)
     g_free(proto);
 }
 
-void update_account_from_dialog(GtkWidget *dialog, account_t *account)
+void update_account_from_dialog(GtkWidget *dialog, const gchar *accountID)
 {
     if (!dialog)
         return;
 
+    account_t *account = account_list_get_by_id(accountID);
+    if (!account)
+        return;
+
     const gboolean IS_IP2IP = account_is_IP2IP(account);
     if (!IS_IP2IP)
         update_account_from_basic_tab(account);
 
+    if (account_is_SIP(account)) {
+        if (IS_IP2IP ||
+            gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(same_as_local_radio_button))) {
+            account_replace(account, CONFIG_PUBLISHED_PORT,
+                    gtk_entry_get_text(GTK_ENTRY(local_port_spin_box)));
+            account_replace(account, CONFIG_LOCAL_PORT,
+                    gtk_entry_get_text(GTK_ENTRY(local_port_spin_box)));
+            gchar *local_interface = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(local_address_combo));
+
+            gchar *published_address = dbus_get_address_from_interface_name(local_interface);
+            g_free(local_interface);
+
+            account_replace(account, CONFIG_PUBLISHED_ADDRESS, published_address);
+        } else {
+            account_replace(account, CONFIG_PUBLISHED_PORT,
+                    gtk_entry_get_text(GTK_ENTRY(published_port_spin_box)));
+
+            account_replace(account, CONFIG_PUBLISHED_ADDRESS,
+                    gtk_entry_get_text(GTK_ENTRY(published_address_entry)));
+        }
+    }
+
+    const gboolean tone_enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(enable_tone));
+    account_replace(account, CONFIG_RINGTONE_ENABLED, bool_to_string(tone_enabled));
+
+    gchar *ringtone_path = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));
+    account_replace(account, CONFIG_RINGTONE_PATH, ringtone_path);
+    g_free(ringtone_path);
+
     // Get current protocol for this account
     gchar *current_protocol;
     if (protocol_combo)
@@ -1326,18 +1511,18 @@ void update_account_from_dialog(GtkWidget *dialog, account_t *account)
 }
 
 GtkWidget *
-show_account_window(account_t *account, SFLPhoneClient *client, gboolean is_new)
+show_account_window(const gchar *accountID, GtkDialog *parent, SFLPhoneClient *client, gboolean is_new)
 {
     // First we reset
     reset();
 
     GtkWidget *dialog = gtk_dialog_new_with_buttons(_("Account settings"),
-                        GTK_WINDOW(client->win),
+                        GTK_WINDOW(parent),
                         GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                         GTK_STOCK_CANCEL,
                         GTK_RESPONSE_CANCEL,
                         GTK_STOCK_APPLY,
-                        GTK_RESPONSE_ACCEPT,
+                        GTK_RESPONSE_APPLY,
                         NULL);
 
     gtk_container_set_border_width(GTK_CONTAINER(dialog), 0);
@@ -1346,26 +1531,30 @@ show_account_window(account_t *account, SFLPhoneClient *client, gboolean is_new)
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), notebook, TRUE, TRUE, 0);
     gtk_container_set_border_width(GTK_CONTAINER(notebook), 10);
     gtk_widget_show(notebook);
+
+    account_t *account = account_list_get_by_id(accountID);
+    if (!account) {
+        g_error("Invalid account %s", accountID);
+        return NULL;
+    }
+
     const gboolean IS_IP2IP = account_is_IP2IP(account);
 
     // We do not need the global settings for the IP2IP account
     if (!IS_IP2IP) {
         /* General Settings */
-        GtkWidget *basic_tab = create_basic_tab(account, is_new);
+        GtkWidget *basic_tab = create_basic_tab(account, is_new, dialog);
         gtk_notebook_append_page(GTK_NOTEBOOK(notebook), basic_tab, gtk_label_new(_("Basic")));
-        gtk_notebook_page_num(GTK_NOTEBOOK(notebook), basic_tab);
     }
 
     /* Audio Codecs */
     GtkWidget *audiocodecs_tab = create_audiocodecs_configuration(account);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), audiocodecs_tab, gtk_label_new(_("Audio")));
-    gtk_notebook_page_num(GTK_NOTEBOOK(notebook), audiocodecs_tab);
 
 #ifdef SFL_VIDEO
     /* Video Codecs */
     GtkWidget *videocodecs_tab = create_videocodecs_configuration(account);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), videocodecs_tab, gtk_label_new(_("Video")));
-    gtk_notebook_page_num(GTK_NOTEBOOK(notebook), videocodecs_tab);
 #endif
 
     // Do not need advanced or security one for the IP2IP account
@@ -1373,17 +1562,14 @@ show_account_window(account_t *account, SFLPhoneClient *client, gboolean is_new)
         /* Advanced */
         advanced_tab = create_advanced_tab(account);
         gtk_notebook_append_page(GTK_NOTEBOOK(notebook), advanced_tab, gtk_label_new(_("Advanced")));
-        gtk_notebook_page_num(GTK_NOTEBOOK(notebook), advanced_tab);
 
         /* Security */
         security_tab = create_security_tab(account, client);
         gtk_notebook_append_page(GTK_NOTEBOOK(notebook), security_tab, gtk_label_new(_("Security")));
-        gtk_notebook_page_num(GTK_NOTEBOOK(notebook), security_tab);
     } else {
         /* Custom tab for the IP to IP profile */
         GtkWidget *ip_tab = create_direct_ip_calls_tab(account);
         gtk_notebook_prepend_page(GTK_NOTEBOOK(notebook), ip_tab, gtk_label_new(_("Network")));
-        gtk_notebook_page_num(GTK_NOTEBOOK(notebook), ip_tab);
     }
 
     // Emit signal to hide advanced and security tabs in case of IAX
@@ -1395,12 +1581,11 @@ show_account_window(account_t *account, SFLPhoneClient *client, gboolean is_new)
     /* Run dialog, this blocks */
     gint response = gtk_dialog_run(GTK_DIALOG(dialog));
 
-    // If anything but "Apply" button is pressed
-    if (response != GTK_RESPONSE_ACCEPT) {
+    if (response == GTK_RESPONSE_APPLY) {
+        return dialog;
+    } else {
         gtk_widget_destroy(dialog);
         return NULL;
-    } else {
-        return dialog;
     }
 }
 
