@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Pierre-Luc Beaudoin <pierre-luc.beaudoin@savoirfairelinux.com>
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *
@@ -45,12 +45,18 @@
  * @return The dialog with the pertinent account information
  */
 GtkWidget *
-show_account_window(account_t *a, SFLPhoneClient *client, gboolean is_new);
+show_account_window(const gchar *accountID, GtkDialog *parent, SFLPhoneClient *client, gboolean is_new);
 
 /*
  * @param dialog The dialog the account will be update from
  * @param a The account you want to display
  */
-void update_account_from_dialog(GtkWidget *dialog, account_t *a);
+void update_account_from_dialog(GtkWidget *dialog, const gchar *accountID);
+
+/*
+ * @param position The position of the slider
+ * @param size The size of the slider
+ */
+void update_ringtone_slider(guint position, guint size);
 
 #endif
