@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -57,6 +57,7 @@ enum {
     COLUMN_CODEC_NAME,
     COLUMN_CODEC_FREQUENCY,
     COLUMN_CODEC_BITRATE,
+    COLUMN_CODEC_CHANNELS,
     CODEC_COLUMN_COUNT
 };
 
@@ -91,12 +92,14 @@ fill_codec_list(const account_t *account)
             gtk_list_store_append(codecStore, &iter);
             gchar *samplerate = g_strdup_printf("%d " KHZ, (gint) (c->sample_rate * 0.001));
             gchar *bitrate = g_strdup_printf("%s " KBPS, c->bitrate);
+            gchar *channels = g_strdup_printf("%d", c->channels);
 
             gtk_list_store_set(codecStore, &iter,
                                COLUMN_CODEC_ACTIVE, c->is_active,
                                COLUMN_CODEC_NAME, c->name,
                                COLUMN_CODEC_FREQUENCY, samplerate,
                                COLUMN_CODEC_BITRATE, bitrate,
+                               COLUMN_CODEC_CHANNELS, channels,
                                -1);
             g_free(samplerate);
             g_free(bitrate);
@@ -511,6 +514,7 @@ audiocodecs_box(const account_t *account)
             G_TYPE_STRING, /* Name */
             G_TYPE_STRING, /* Frequency */
             G_TYPE_STRING, /* Bitrate */
+            G_TYPE_STRING, /* Channels */
             G_TYPE_STRING  /* Bandwidth */);
 
     // Create codec tree view with list store
@@ -548,6 +552,11 @@ audiocodecs_box(const account_t *account)
     treeViewColumn = gtk_tree_view_column_new_with_attributes(_("Bitrate"), renderer, "text", COLUMN_CODEC_BITRATE, NULL);
     gtk_tree_view_append_column(GTK_TREE_VIEW(codecTreeView), treeViewColumn);
 
+    // Channels column
+    renderer = gtk_cell_renderer_text_new();
+    treeViewColumn = gtk_tree_view_column_new_with_attributes(_("Channels"), renderer, "text", COLUMN_CODEC_CHANNELS, NULL);
+    gtk_tree_view_append_column(GTK_TREE_VIEW(codecTreeView), treeViewColumn);
+
     gtk_container_add(GTK_CONTAINER(scrolledWindow), codecTreeView);
 
     // Create button box
@@ -582,7 +591,7 @@ static GtkWidget* alsa_box()
     gtk_box_pack_start(GTK_BOX(alsa_hbox), grid, TRUE, TRUE, 1);
     gtk_widget_show(grid);
 
-    gchar *message = "<small><i>default</i> plugin always uses internal sound card. Select <i>dmix/dsnoop</i> to use an alternate soundcard.</small>";
+    gchar *message = "<small>Select <i>dmix/dsnoop</i> to use non-default soundcard.</small>";
     GtkWidget *info_bar = gnome_info_bar(message, GTK_MESSAGE_INFO);
     /* Info bar gets a width of 2 cells */
     gtk_grid_attach(GTK_GRID(grid), info_bar, 1, 1, 2, 1);
@@ -768,7 +777,13 @@ select_audio_manager(GtkWidget *alsa_button, SFLPhoneClient *client)
         gtk_container_add(GTK_CONTAINER(alsa_conf), alsabox);
         gtk_widget_show(alsa_conf);
         gtk_widget_set_sensitive(alsa_conf, TRUE);
-        gtk_action_set_sensitive(volumeToggle_, TRUE);
+
+        if (gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(volumeToggle_))) {
+            main_window_volume_controls(FALSE);
+            g_settings_set_boolean(client->settings, "show-volume-controls", FALSE);
+            gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(volumeToggle_), FALSE);
+        }
+
     } else if (must_show_alsa_conf() && !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(alsa_button))) {
         dbus_set_audio_manager(PULSEAUDIO_API_STR);
         gtk_container_remove(GTK_CONTAINER(alsa_conf), alsabox);
@@ -778,7 +793,6 @@ select_audio_manager(GtkWidget *alsa_button, SFLPhoneClient *client)
         gtk_container_add(GTK_CONTAINER(pulse_conf), pulsebox);
         gtk_widget_show(pulse_conf);
         gtk_widget_set_sensitive(pulse_conf, TRUE);
-        gtk_action_set_sensitive(volumeToggle_, TRUE);
 
         if (gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(volumeToggle_))) {
             main_window_volume_controls(FALSE);
@@ -786,33 +800,20 @@ select_audio_manager(GtkWidget *alsa_button, SFLPhoneClient *client)
             gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(volumeToggle_), FALSE);
         }
 
-        gtk_action_set_sensitive(volumeToggle_, FALSE);
     } else
         g_warning("Unexpected audio API state");
 }
 
-static const gchar *reverse_state(const gchar *state)
-{
-    if (g_strcmp0(state, "enabled") == 0)
-        return "disabled";
-    else
-        return "enabled";
-}
-
 static void
 active_noise_suppress(void)
 {
-    gchar *state = dbus_get_noise_suppress_state();
-    dbus_set_noise_suppress_state(reverse_state(state));
-    g_free(state);
+    dbus_set_noise_suppress_state(!dbus_get_noise_suppress_state());
 }
 
 static void
-active_echo_cancel(void)
+toggle_agc(void)
 {
-    gchar *state = dbus_get_echo_cancel_state();
-    dbus_set_echo_cancel_state(reverse_state(state));
-    g_free(state);
+    dbus_set_agc_state(!dbus_get_agc_state());
 }
 
 static void
@@ -841,6 +842,17 @@ static void record_path_changed(GtkFileChooser *chooser, G_GNUC_UNUSED gpointer
     g_free(path);
 }
 
+static
+void
+mute_dtmf_toggled(GtkToggleButton *tb, G_GNUC_UNUSED gpointer data)
+{
+    if (gtk_toggle_button_get_active(tb))
+        dbus_mute_dtmf(TRUE);
+    else
+        dbus_mute_dtmf(FALSE);
+}
+
+
 GtkWidget* create_audio_configuration(SFLPhoneClient *client)
 {
     /* Main widget */
@@ -916,31 +928,27 @@ GtkWidget* create_audio_configuration(SFLPhoneClient *client)
     gtk_box_pack_start(GTK_BOX(audio_vbox), frame, FALSE, FALSE, 0);
 
     GtkWidget *enableNoiseReduction = gtk_check_button_new_with_mnemonic(_("_Noise Reduction"));
-    gchar *state = dbus_get_noise_suppress_state();
 
-    if (g_strcmp0(state, "enabled") == 0)
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enableNoiseReduction), TRUE);
-    else
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enableNoiseReduction), FALSE);
-
-    g_free(state);
-    state = NULL;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enableNoiseReduction), dbus_get_noise_suppress_state());
 
     g_signal_connect(G_OBJECT(enableNoiseReduction), "clicked", active_noise_suppress, NULL);
     gtk_grid_attach(GTK_GRID(grid), enableNoiseReduction, 0, 1, 1, 1);
 
-    GtkWidget *enableEchoCancel = gtk_check_button_new_with_mnemonic(_("_Echo Cancellation"));
-    state = dbus_get_echo_cancel_state();
+    GtkWidget *enableAGC = gtk_check_button_new_with_mnemonic(_("Automatic _Gain Control"));
 
-    if (g_strcmp0(state, "enabled") == 0)
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enableEchoCancel), TRUE);
-    else
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enableEchoCancel), FALSE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enableAGC), dbus_get_agc_state());
 
-    g_free(state);
+    g_signal_connect(G_OBJECT(enableAGC), "clicked", toggle_agc, NULL);
+    gtk_grid_attach(GTK_GRID(grid), enableAGC, 0, 2, 1, 1);
 
-    g_signal_connect(G_OBJECT(enableEchoCancel), "clicked", active_echo_cancel, NULL);
-    gtk_grid_attach(GTK_GRID(grid), enableEchoCancel, 0, 2, 1, 1);
+    gnome_main_section_new_with_grid(_("Tone settings"), &frame, &grid);
+    gtk_box_pack_start(GTK_BOX(audio_vbox), frame, FALSE, FALSE, 0);
+
+    GtkWidget *muteDtmf = gtk_check_button_new_with_mnemonic(_("_Mute DTMF"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(muteDtmf), dbus_is_dtmf_muted());
+
+    g_signal_connect(G_OBJECT(muteDtmf), "toggled", mute_dtmf_toggled, NULL);
+    gtk_grid_attach(GTK_GRID(grid), muteDtmf, 0, 1, 1, 1);
 
     gtk_widget_show_all(audio_vbox);
 
@@ -964,5 +972,5 @@ gboolean must_show_alsa_conf()
 gboolean
 must_show_volume(SFLPhoneClient *client)
 {
-    return g_settings_get_boolean(client->settings, "show-volume-controls") && must_show_alsa_conf();
+    return g_settings_get_boolean(client->settings, "show-volume-controls");
 }
