@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -34,7 +34,7 @@
 #include <gtk/gtk.h>
 
 GtkWidget *gnome_main_section_new(const gchar * const title);
-void gnome_main_section_new_with_grid(gchar *title, GtkWidget**, GtkWidget**);
+void gnome_main_section_new_with_grid(const gchar *title, GtkWidget**, GtkWidget**);
 GtkWidget *gnome_info_bar (gchar *message, GtkMessageType type);
 
 #endif // _UTILS_
