@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Pierre-Luc Beaudoin <pierre-luc.beaudoin@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -96,6 +96,13 @@ calllist_free_element(gpointer data, G_GNUC_UNUSED gpointer user_data)
     free_callable_obj_t(call);
 }
 
+gboolean
+calllist_empty(calltab_t *tab)
+{
+    g_return_val_if_fail(tab != NULL && tab->callQueue != NULL, FALSE);
+    return !calllist_get_size(tab);
+}
+
 void
 calllist_clean(calltab_t* tab)
 {
@@ -165,6 +172,10 @@ calllist_remove_call(calltab_t* tab, const gchar * callID, SFLPhoneClient *clien
         g_queue_remove(tab->callQueue, call);
     }
 
+    /* If removing a call from the history, don't add it back to the history! */
+    if (calltab_has_name(tab, HISTORY))
+        return;
+
     /* Don't save empty (i.e. started dialing, then deleted) calls */
     if (call->_peer_number && strlen(call->_peer_number) > 0) {
         calllist_add_call(history_tab, call);
