@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Pierre-Luc Beaudoin <pierre-luc.beaudoin@savoirfairelinux.com>
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
@@ -53,6 +53,11 @@
 #include "searchbar.h"
 #include "sflphone_client.h"
 
+#ifdef SFL_PRESENCE
+#include "presencewindow.h"
+#include "presence.h"
+#endif
+
 #if !GLIB_CHECK_VERSION(2, 30, 0)
 #define G_VALUE_INIT  { 0, { { 0 } } }
 #endif
@@ -101,6 +106,12 @@ is_conference(GtkTreeModel *model, GtkTreeIter *iter)
     return result;
 }
 
+static void
+update_ringtone_seekslider_path(callable_obj_t *call)
+{
+    main_window_update_seekslider(call->_recordfile);
+}
+
 /* Call back when the user click on a call in the list */
 static void
 call_selected_cb(GtkTreeSelection *sel, SFLPhoneClient *client)
@@ -132,8 +143,11 @@ call_selected_cb(GtkTreeSelection *sel, SFLPhoneClient *client)
         callable_obj_t *selected_call = calllist_get_call(active_calltree_tab, id);
         g_free(id);
 
-        if (selected_call)
+        if (selected_call) {
             calltab_select_call(active_calltree_tab, selected_call);
+            if (calltab_has_name(active_calltree_tab, HISTORY))
+                update_ringtone_seekslider_path(selected_call);
+        }
     }
 
     update_actions(client);
@@ -217,6 +231,9 @@ row_single_click(G_GNUC_UNUSED GtkTreeView *tree_view, SFLPhoneClient *client)
 {
     gchar * displaySasOnce = NULL;
 
+    if (calllist_empty(active_calltree_tab))
+        return;
+
     callable_obj_t *selectedCall = calltab_get_selected_call(active_calltree_tab);
     conference_obj_t *selectedConf = calltab_get_selected_conf(active_calltree_tab);
 
@@ -377,6 +394,33 @@ calltree_display_call_info(callable_obj_t * call, CallDisplayType display_type,
     return msg;
 }
 
+#ifdef SFL_PRESENCE
+void on_call_drag_data_get(GtkWidget *widget,
+        G_GNUC_UNUSED GdkDragContext *drag_context,
+        GtkSelectionData *sdata,
+        G_GNUC_UNUSED guint info,
+        G_GNUC_UNUSED guint time_,
+        G_GNUC_UNUSED gpointer user_data)
+{
+    GtkTreeSelection *selector = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
+    if(!gtk_tree_selection_get_selected(selector, NULL, NULL))
+        return;
+
+    callable_obj_t * c = calltab_get_selected_call(active_calltree_tab);
+    buddy_t * b = presence_buddy_create();
+
+    presence_callable_to_buddy(c, b);
+
+    g_debug("Drag src from calltree: b->uri : %s",b->uri);
+
+    gtk_selection_data_set(sdata,
+            gdk_atom_intern ("struct buddy_t pointer", FALSE),
+            8,           // Tell GTK how to pack the data (bytes)
+            (void *)&b,  // The actual pointer that we just made
+            sizeof (b)); // The size of the pointer
+}
+#endif
+
 void
 calltree_create(calltab_t* tab, gboolean has_searchbar, SFLPhoneClient *client)
 {
@@ -428,6 +472,12 @@ calltree_create(calltab_t* tab, gboolean has_searchbar, SFLPhoneClient *client)
                      G_CALLBACK(button_pressed),
                      client);
 
+#ifdef SFL_PRESENCE
+    gtk_drag_source_set(tab->view, GDK_BUTTON1_MASK,
+         &presence_drag_targets, 1, GDK_ACTION_COPY|GDK_ACTION_MOVE);
+    g_signal_connect(tab->view, "drag-data-get", G_CALLBACK(on_call_drag_data_get), NULL);
+#endif
+
     if (calltab_has_name(tab, CURRENT_CALLS)) {
 
         gtk_tree_view_enable_model_drag_source(GTK_TREE_VIEW(tab->view), GDK_BUTTON1_MASK, target_list, n_targets, GDK_ACTION_DEFAULT | GDK_ACTION_MOVE);
@@ -502,6 +552,7 @@ calltree_create(calltab_t* tab, gboolean has_searchbar, SFLPhoneClient *client)
         }
     }
 
+
     if (!calltab_has_name(tab, HISTORY))
         gtk_widget_show(tab->tree);
 }
@@ -530,9 +581,11 @@ calltree_remove_call(calltab_t* tab, const gchar *target_id)
     gtk_tree_model_foreach(model, remove_element_if_match, (gpointer) target_id);
 
     /* invalidate selected call if it was our target */
-    callable_obj_t *sel = calltab_get_selected_call(tab);
-    if (sel && g_strcmp0(sel->_callID, target_id) == 0)
-        calltab_select_call(tab, NULL);
+    if (!calllist_empty(tab)) {
+        callable_obj_t *sel = calltab_get_selected_call(tab);
+        if (sel && g_strcmp0(sel->_callID, target_id) == 0)
+            calltab_select_call(tab, NULL);
+    }
 
     statusbar_update_clock("");
 }
@@ -1152,6 +1205,9 @@ void calltree_display(calltab_t *tab, SFLPhoneClient *client)
 
 gboolean calltree_update_clock(G_GNUC_UNUSED gpointer data)
 {
+    if (calllist_empty(current_calls_tab))
+        return TRUE;
+
     char timestr[20];
     const gchar *msg = "";
     long duration;
