@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Pierre-Luc Beaudoin <pierre-luc.beaudoin@savoirfairelinux.com>
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Guillaume Carmel-Archambault <guillaume.carmel-archambault@savoirfairelinux.com>
@@ -50,12 +50,18 @@
 #include "assistant.h"
 #include "accountlist.h"
 #include "accountlistconfigdialog.h"
+#include "accountconfigdialog.h"
 #include "messaging/message_tab.h"
-
 #include "sflphone_client.h"
 #include "dbus.h"
 #include "actions.h"
 
+#ifdef SFL_PRESENCE
+#include "presence.h"
+#include "presencemanager-glue.h"
+#include "presencewindow.h"
+#endif
+
 #ifdef SFL_VIDEO
 #include "config/videoconf.h"
 #include "video/video_callbacks.h"
@@ -69,7 +75,9 @@ static DBusGProxy *video_proxy;
 static DBusGProxy *call_proxy;
 static DBusGProxy *config_proxy;
 static DBusGProxy *instance_proxy;
-// static DBusGProxy *session_manager_proxy;
+#ifdef SFL_PRESENCE
+static DBusGProxy *presence_proxy;
+#endif
 static GDBusProxy *session_manager_proxy;
 
 /* Returns TRUE if there was an error, FALSE otherwise */
@@ -412,9 +420,11 @@ record_playback_stopped_cb(G_GNUC_UNUSED DBusGProxy *proxy, const gchar *filepat
 }
 
 static void
-update_playback_scale_cb(G_GNUC_UNUSED DBusGProxy *proxy, guint position, guint size)
+update_playback_scale_cb(G_GNUC_UNUSED DBusGProxy *proxy,
+        const gchar *file_path, guint position, guint size)
 {
-    main_window_update_playback_scale(position, size);
+    if (!main_window_update_playback_scale(file_path, position, size))
+        update_ringtone_slider(position, size);
 }
 
 static void
@@ -434,10 +444,16 @@ registration_state_changed_cb(G_GNUC_UNUSED DBusGProxy *proxy, const gchar *acco
 static void
 accounts_changed_cb(G_GNUC_UNUSED DBusGProxy *proxy, G_GNUC_UNUSED void *foo)
 {
+    g_debug("Account details changed.");
     sflphone_fill_account_list();
     sflphone_fill_ip2ip_profile();
+
+    // ui updates
     status_bar_display_account();
     statusicon_set_tooltip();
+#ifdef SFL_PRESENCE
+    update_presence_statusbar();
+#endif
 }
 
 static void
@@ -600,6 +616,80 @@ screensaver_dbus_proxy_new_cb (G_GNUC_UNUSED GObject * source, GAsyncResult *res
         g_warning("could not initialize gnome session manager");
 }
 
+#ifdef SFL_PRESENCE
+static void
+sip_presence_subscription_state_changed_cb(G_GNUC_UNUSED DBusGProxy *proxy, const gchar *accID,
+                              const gchar *uri, gboolean state, G_GNUC_UNUSED void *foo)
+{
+    g_debug("DBus: Presence subscription state changed to %s for account %s, buddy:%s",
+          state? "active":"inactive", accID, uri);
+    account_t *acc = account_list_get_by_id(accID);
+    if (acc)
+    {
+        buddy_t * b = presence_buddy_list_buddy_get_by_string(accID, uri);
+        if(b)
+        {
+            b->subscribed = state;
+            if(!state) // not monitored means default value for status ( == Offline)
+            {
+                b->status = FALSE;
+                g_free(b->note);
+                b->note = g_strdup("Not found");
+            }
+            update_presence_view();
+        }
+    }
+}
+
+static void
+sip_presence_notification_cb(G_GNUC_UNUSED DBusGProxy *proxy, const gchar *accID, const gchar *uri,
+                  gboolean status, const gchar * note)
+{
+    g_debug("DBus: Presence notification (%s) for %s status=%s lineStatus=%s.",
+            accID, uri, status? PRESENCE_STATUS_ONLINE:PRESENCE_STATUS_OFFLINE, note);
+
+    account_t *acc = account_list_get_by_id(accID);
+    if (acc)
+    {
+        gchar *my_uri = g_strconcat("<sip:",
+                account_lookup(acc, CONFIG_ACCOUNT_USERNAME),
+                "@",
+                account_lookup(acc, CONFIG_ACCOUNT_HOSTNAME),
+                ">", NULL);
+
+        if ((g_strcmp0(uri, my_uri) == 0)) // self-subscription
+        {
+            account_replace(acc, CONFIG_PRESENCE_STATUS, status? PRESENCE_STATUS_ONLINE:PRESENCE_STATUS_OFFLINE);
+            account_replace(acc, CONFIG_PRESENCE_NOTE, note);
+            update_presence_statusbar();
+        }
+        else
+        {
+            buddy_t * b = presence_buddy_list_buddy_get_by_string(accID, uri);
+            if(b)
+            {
+                b->status = status;
+                g_free(b->note);
+                b->note = g_strdup(note);
+                update_presence_view();
+            }
+        }
+    }
+}
+
+static void
+sip_presence_server_error_cb(G_GNUC_UNUSED DBusGProxy *proxy, const gchar *accID, const gchar *err, const gchar *msg)
+{
+    g_debug("DBus: Presence error from server (%s) : %s / %s.",accID, err, msg);
+}
+
+static void
+sip_presence_new_subscription_request_cb(G_GNUC_UNUSED DBusGProxy *proxy, const gchar *uri)
+{
+    g_debug("DBus: Presence new subscription from %s.",uri);
+}
+#endif
+
 #define GS_SERVICE   "org.gnome.SessionManager"
 #define GS_PATH      "/org/gnome/SessionManager"
 #define GS_INTERFACE "org.gnome.SessionManager"
@@ -641,8 +731,10 @@ gboolean dbus_connect(GError **error, SFLPhoneClient *client)
     const char *callmanager_interface = "org.sflphone.SFLphone.CallManager";
     const char *configurationmanager_object_instance = "/org/sflphone/SFLphone/ConfigurationManager";
     const char *configurationmanager_interface = "org.sflphone.SFLphone.ConfigurationManager";
-
-    g_type_init();
+#ifdef SFL_PRESENCE
+    const char *presencemanager_object_instance = "/org/sflphone/SFLphone/PresenceManager";
+    const char *presencemanager_interface = "org.sflphone.SFLphone.PresenceManager";
+#endif
 
     DBusGConnection *connection = dbus_g_bus_get(DBUS_BUS_SESSION, error);
     if (connection == NULL) {
@@ -676,6 +768,14 @@ gboolean dbus_connect(GError **error, SFLPhoneClient *client)
         return FALSE;
     }
 
+#ifdef SFL_PRESENCE
+    presence_proxy = dbus_g_proxy_new_for_name(connection, dbus_message_bus_name, presencemanager_object_instance, presencemanager_interface);
+    if (presence_proxy == NULL) {
+        g_warning("Error: Failed to connect to %s", presencemanager_object_instance);
+        return FALSE;
+    }
+#endif
+
     /* Register INT Marshaller */
     dbus_g_object_register_marshaller(g_cclosure_user_marshal_VOID__INT,
                                       G_TYPE_NONE, G_TYPE_INT, G_TYPE_INVALID);
@@ -727,6 +827,11 @@ gboolean dbus_connect(GError **error, SFLPhoneClient *client)
         g_cclosure_user_marshal_VOID__STRING_STRING_INT_INT, G_TYPE_NONE,
         G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT, G_TYPE_INVALID);
 
+    /* Register STRING STRING BOOLEAN STRING Marshaller */
+    dbus_g_object_register_marshaller(
+        g_cclosure_user_marshal_VOID__STRING_STRING_BOOLEAN_STRING, G_TYPE_NONE,
+        G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_INVALID);
+
     g_debug("Adding callmanager Dbus signals");
 
     /* Incoming call */
@@ -765,9 +870,9 @@ gboolean dbus_connect(GError **error, SFLPhoneClient *client)
     dbus_g_proxy_connect_signal(call_proxy, "incomingMessage",
                                 G_CALLBACK(incoming_message_cb), client, NULL);
 
-    dbus_g_proxy_add_signal(call_proxy, "volumeChanged", G_TYPE_STRING,
+    dbus_g_proxy_add_signal(config_proxy, "volumeChanged", G_TYPE_STRING,
                             G_TYPE_DOUBLE, G_TYPE_INVALID);
-    dbus_g_proxy_connect_signal(call_proxy, "volumeChanged",
+    dbus_g_proxy_connect_signal(config_proxy, "volumeChanged",
                                 G_CALLBACK(volume_changed_cb), NULL, NULL);
 
     dbus_g_proxy_add_signal(call_proxy, "transferSucceeded", G_TYPE_INVALID);
@@ -804,7 +909,7 @@ gboolean dbus_connect(GError **error, SFLPhoneClient *client)
     dbus_g_proxy_connect_signal(call_proxy, "recordPlaybackStopped",
                                 G_CALLBACK(record_playback_stopped_cb), client, NULL);
 
-    dbus_g_proxy_add_signal(call_proxy, "updatePlaybackScale", G_TYPE_INT, G_TYPE_INT, G_TYPE_INVALID);
+    dbus_g_proxy_add_signal(call_proxy, "updatePlaybackScale", G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT, G_TYPE_INVALID);
     dbus_g_proxy_connect_signal(call_proxy, "updatePlaybackScale",
                                 G_CALLBACK(update_playback_scale_cb), NULL, NULL);
 
@@ -850,7 +955,6 @@ gboolean dbus_connect(GError **error, SFLPhoneClient *client)
     dbus_g_proxy_connect_signal(call_proxy, "sipCallStateChanged",
                                 G_CALLBACK(sip_call_state_cb), client, NULL);
 
-
     g_debug("Adding configurationmanager Dbus signals");
 
     dbus_g_proxy_add_signal(config_proxy, "accountsChanged", G_TYPE_INVALID);
@@ -869,6 +973,30 @@ gboolean dbus_connect(GError **error, SFLPhoneClient *client)
     dbus_g_proxy_connect_signal(config_proxy, "errorAlert",
                                 G_CALLBACK(error_alert), NULL, NULL);
 
+#ifdef SFL_PRESENCE
+    g_debug("Adding presencemanager Dbus signals");
+
+    /* Presence related callbacks */
+    dbus_g_proxy_add_signal(presence_proxy, "subscriptionStateChanged", G_TYPE_STRING,
+                            G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_INVALID);
+    dbus_g_proxy_connect_signal(presence_proxy, "subscriptionStateChanged",
+                                G_CALLBACK(sip_presence_subscription_state_changed_cb), client, NULL);
+
+    dbus_g_proxy_add_signal(presence_proxy, "newBuddyNotification", G_TYPE_STRING,
+                            G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_INVALID);
+    dbus_g_proxy_connect_signal(presence_proxy, "newBuddyNotification",
+                                G_CALLBACK(sip_presence_notification_cb), client, NULL);
+
+    dbus_g_proxy_add_signal(presence_proxy, "serverError", G_TYPE_STRING,G_TYPE_STRING,
+                            G_TYPE_STRING, G_TYPE_INVALID);
+    dbus_g_proxy_connect_signal(presence_proxy, "serverError",
+                                G_CALLBACK(sip_presence_server_error_cb), NULL, NULL);
+
+    dbus_g_proxy_add_signal(presence_proxy, "newServerSubscriptionRequest", G_TYPE_STRING, G_TYPE_INVALID);
+    dbus_g_proxy_connect_signal(presence_proxy, "newServerSubscriptionRequest",
+                                G_CALLBACK(sip_presence_new_subscription_request_cb), NULL, NULL);
+#endif
+
 #ifdef SFL_VIDEO
     const gchar *videocontrols_object_instance = "/org/sflphone/SFLphone/VideoControls";
     const gchar *videocontrols_interface = "org.sflphone.SFLphone.VideoControls";
@@ -897,14 +1025,15 @@ gboolean dbus_connect(GError **error, SFLPhoneClient *client)
 #endif
 
     /* Defines a default timeout for the proxies */
-#if HAVE_DBUS_G_PROXY_SET_DEFAULT_TIMEOUT
     static const int DEFAULT_DBUS_TIMEOUT = 30000;
     dbus_g_proxy_set_default_timeout(call_proxy, DEFAULT_DBUS_TIMEOUT);
     dbus_g_proxy_set_default_timeout(instance_proxy, DEFAULT_DBUS_TIMEOUT);
     dbus_g_proxy_set_default_timeout(config_proxy, DEFAULT_DBUS_TIMEOUT);
+#ifdef SFL_PRESENCE
+    dbus_g_proxy_set_default_timeout(presence_proxy, DEFAULT_DBUS_TIMEOUT);
+#endif
 #ifdef SFL_VIDEO
     dbus_g_proxy_set_default_timeout(video_proxy, DEFAULT_DBUS_TIMEOUT);
-#endif
 #endif
 
     gboolean status = dbus_connect_session_manager(connection);
@@ -924,6 +1053,9 @@ void dbus_clean()
     g_object_unref(call_proxy);
     g_object_unref(config_proxy);
     g_object_unref(instance_proxy);
+#ifdef SFL_PRESENCE
+    g_object_unref(presence_proxy);
+#endif
 }
 
 void dbus_hold(const callable_obj_t *c)
@@ -1064,6 +1196,18 @@ dbus_account_list()
     return array;
 }
 
+GHashTable *
+dbus_get_account_template()
+{
+    GError *error = NULL;
+    GHashTable *details = NULL;
+
+    org_sflphone_SFLphone_ConfigurationManager_get_account_template(config_proxy, &details, &error);
+    check_error(error);
+
+    return details;
+}
+
 GHashTable *
 dbus_get_account_details(const gchar *accountID)
 {
@@ -1153,16 +1297,70 @@ void
 dbus_set_volume(const gchar *device, gdouble value)
 {
     GError *error = NULL;
-    org_sflphone_SFLphone_CallManager_set_volume(call_proxy, device, value, &error);
+    org_sflphone_SFLphone_ConfigurationManager_set_volume(config_proxy, device, value, &error);
     check_error(error);
 }
 
+void
+dbus_mute_capture(gboolean muted)
+{
+    GError *error = NULL;
+    org_sflphone_SFLphone_ConfigurationManager_mute_capture(config_proxy, muted, &error);
+    check_error(error);
+}
+
+void
+dbus_mute_dtmf(gboolean muted)
+{
+    GError *error = NULL;
+    org_sflphone_SFLphone_ConfigurationManager_mute_dtmf(config_proxy, muted, &error);
+    check_error(error);
+}
+
+void
+dbus_mute_playback(gboolean muted)
+{
+    GError *error = NULL;
+    org_sflphone_SFLphone_ConfigurationManager_mute_playback(config_proxy, muted, &error);
+    check_error(error);
+}
+
+gboolean
+dbus_is_dtmf_muted()
+{
+    GError *error = NULL;
+    gboolean muted;
+    org_sflphone_SFLphone_ConfigurationManager_is_dtmf_muted(config_proxy, &muted, &error);
+    check_error(error);
+    return muted;
+}
+
+gboolean
+dbus_is_capture_muted()
+{
+    GError *error = NULL;
+    gboolean muted;
+    org_sflphone_SFLphone_ConfigurationManager_is_capture_muted(config_proxy, &muted, &error);
+    check_error(error);
+    return muted;
+}
+
+gboolean
+dbus_is_playback_muted()
+{
+    GError *error = NULL;
+    gboolean muted;
+    org_sflphone_SFLphone_ConfigurationManager_is_playback_muted(config_proxy, &muted, &error);
+    check_error(error);
+    return muted;
+}
+
 gdouble
 dbus_get_volume(const gchar *device)
 {
     gdouble value;
     GError *error = NULL;
-    org_sflphone_SFLphone_CallManager_get_volume(call_proxy, device, &value, &error);
+    org_sflphone_SFLphone_ConfigurationManager_get_volume(config_proxy, device, &value, &error);
     check_error(error);
     return value;
 }
@@ -1420,15 +1618,15 @@ dbus_get_current_audio_output_plugin()
 /**
  * Get noise reduction state
  */
-gchar *
+gboolean
 dbus_get_noise_suppress_state()
 {
-    gchar *state;
+    gboolean state;
     GError *error = NULL;
     org_sflphone_SFLphone_ConfigurationManager_get_noise_suppress_state(config_proxy, &state, &error);
 
     if (check_error(error))
-        state = g_strdup("");
+        state = FALSE;
 
     return state;
 }
@@ -1437,7 +1635,7 @@ dbus_get_noise_suppress_state()
  * Set noise reduction state
  */
 void
-dbus_set_noise_suppress_state(const gchar *state)
+dbus_set_noise_suppress_state(gboolean state)
 {
     GError *error = NULL;
     org_sflphone_SFLphone_ConfigurationManager_set_noise_suppress_state(config_proxy, state, &error);
@@ -1449,25 +1647,36 @@ dbus_set_noise_suppress_state(const gchar *state)
     }
 }
 
-gchar *
-dbus_get_echo_cancel_state(void)
+/**
+ * Get AGC state
+ */
+gboolean
+dbus_get_agc_state()
 {
+    gboolean state;
     GError *error = NULL;
-    gchar *state;
-    org_sflphone_SFLphone_ConfigurationManager_get_echo_cancel_state(config_proxy, &state, &error);
+    org_sflphone_SFLphone_ConfigurationManager_is_agc_enabled(config_proxy, &state, &error);
 
     if (check_error(error))
-        state = g_strdup("");
+        state = FALSE;
 
     return state;
 }
 
+/**
+ * Set AGC state
+ */
 void
-dbus_set_echo_cancel_state(const gchar *state)
+dbus_set_agc_state(gboolean state)
 {
     GError *error = NULL;
-    org_sflphone_SFLphone_ConfigurationManager_set_echo_cancel_state(config_proxy, state, &error);
-    check_error(error);
+    org_sflphone_SFLphone_ConfigurationManager_set_agc_state(config_proxy, state, &error);
+
+    if (error) {
+        g_warning("Failed to call set_agc_state() on "
+              "ConfigurationManager: %s", error->message);
+        g_error_free(error);
+    }
 }
 
 int
@@ -1533,12 +1742,14 @@ dbus_join_conference(const gchar *sel_confID, const gchar *drag_confID)
     check_error(error);
 }
 
-void
-dbus_set_record(const gchar *id)
+gboolean
+dbus_toggle_recording(const gchar *id)
 {
     GError *error = NULL;
-    org_sflphone_SFLphone_CallManager_set_recording(call_proxy, id, &error);
+    gboolean isRecording;
+    org_sflphone_SFLphone_CallManager_toggle_recording(call_proxy, id, &isRecording, &error);
     check_error(error);
+    return isRecording;
 }
 
 gboolean
@@ -2004,33 +2215,33 @@ dbus_send_text_message(const gchar *callID, const gchar *message)
 
 #ifdef SFL_VIDEO
 static void
-video_preview_async_cb(G_GNUC_UNUSED DBusGProxy *proxy, GError *error, G_GNUC_UNUSED gpointer userdata)
+video_camera_async_cb(G_GNUC_UNUSED DBusGProxy *proxy, GError *error, G_GNUC_UNUSED gpointer userdata)
 {
     check_error(error);
     // Reactivate it now that we're done, D-Bus wise
-    set_preview_button_sensitivity(TRUE);
+    set_camera_button_sensitivity(TRUE);
 }
 
 void
-dbus_start_video_preview()
+dbus_start_video_camera()
 {
-    set_preview_button_sensitivity(FALSE);
-    org_sflphone_SFLphone_VideoControls_start_preview_async(video_proxy, video_preview_async_cb, NULL);
+    set_camera_button_sensitivity(FALSE);
+    org_sflphone_SFLphone_VideoControls_start_camera_async(video_proxy, video_camera_async_cb, NULL);
 }
 
 void
-dbus_stop_video_preview()
+dbus_stop_video_camera()
 {
-    set_preview_button_sensitivity(FALSE);
-    org_sflphone_SFLphone_VideoControls_stop_preview_async(video_proxy, video_preview_async_cb, NULL);
+    set_camera_button_sensitivity(FALSE);
+    org_sflphone_SFLphone_VideoControls_stop_camera_async(video_proxy, video_camera_async_cb, NULL);
 }
 
 gboolean
-dbus_has_video_preview_started()
+dbus_has_video_camera_started()
 {
     GError *error = NULL;
     gboolean started = FALSE;
-    org_sflphone_SFLphone_VideoControls_has_preview_started(video_proxy, &started, &error);
+    org_sflphone_SFLphone_VideoControls_has_camera_started(video_proxy, &started, &error);
     check_error(error);
     return started;
 }
@@ -2079,6 +2290,8 @@ static void screensaver_uninhibit_cb(GObject * source_object,
     g_variant_unref(value);
 }
 
+
+
 void dbus_screensaver_inhibit(void)
 {
     const gchar *appname = g_get_application_name();
@@ -2120,3 +2333,23 @@ dbus_screensaver_uninhibit(void)
                       G_DBUS_CALL_FLAGS_NO_AUTO_START, -1, NULL,
                       screensaver_uninhibit_cb, NULL);
 }
+
+#ifdef SFL_PRESENCE
+void
+dbus_presence_publish(const gchar *accountID, gboolean status)
+{
+    GError *error = NULL;
+    g_debug("DBus: publish presence status.");
+    org_sflphone_SFLphone_PresenceManager_publish(presence_proxy, accountID,status, "Tout va bien.", NULL);
+    check_error(error);
+}
+
+void
+dbus_presence_subscribe(const gchar *accountID, const gchar *uri, gboolean flag)
+{
+    g_debug("DBus: subscrbe presence status %s:%s.", uri, flag? "true" : "false");
+    GError *error = NULL;
+    org_sflphone_SFLphone_PresenceManager_subscribe_buddy(presence_proxy, accountID, uri, flag, NULL);
+    check_error(error);
+}
+#endif
