@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Pierre-Luc Beaudoin <pierre-luc.beaudoin@savoirfairelinux.com>
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Guillaume Carmel-Archambault <guillaume.carmel-archambault@savoirfairelinux.com>
@@ -114,9 +114,15 @@ void dbus_place_call(const callable_obj_t *c);
 gchar **dbus_account_list();
 
 /**
- * ConfigurationManager - Get the details of a specific account
- * @param accountID The unique of the account
- * @return GHashTable* The details of the account
+ * configurationmanager - get a template for a new account
+ * @return ghashtable* the details of the account
+ */
+GHashTable *dbus_get_account_template();
+
+/**
+ * configurationmanager - get the details of a specific account
+ * @param accountid the unique of the account
+ * @return ghashtable* the details of the account
  */
 GHashTable *dbus_get_account_details(const gchar *accountID);
 
@@ -183,6 +189,14 @@ void dbus_set_volume(const gchar *device, gdouble value);
  */
 gdouble dbus_get_volume(const gchar *device);
 
+void dbus_mute_capture(gboolean mute);
+void dbus_mute_playback(gboolean mute);
+gboolean dbus_is_capture_muted();
+gboolean dbus_is_playback_muted();
+
+void dbus_mute_dtmf(gboolean mute);
+gboolean dbus_is_dtmf_muted();
+
 /**
  * ConfigurationManager - Play DTMF
  * @param key The DTMF to send
@@ -316,35 +330,27 @@ gchar *dbus_get_current_audio_output_plugin();
 
 /**
  * ConfigurationManager - Get the current noise suppressor state
- * @return gchar* The state (enabled/disabled)
+ * @return gboolean The state (enabled/disabled)
  */
-gchar *dbus_get_noise_suppress_state(void);
+gboolean dbus_get_noise_suppress_state(void);
 
 /**
  * ConfigurationManager - Set the current noise suppressor state
- * @param gchar* The state (enabled/disabled)
+ * @param gboolean The state (enabled/disabled)
  */
-void dbus_set_noise_suppress_state(const gchar *state);
+void dbus_set_noise_suppress_state(gboolean state);
 
 /**
- * ConfigurationManager - Get the current echo cancel state
- * @return gchar* The state (enabled/disabled)
+ * ConfigurationManager - Get the current AGC state
+ * @return gboolean The state (enabled/disabled)
  */
-gchar *dbus_get_echo_cancel_state(void);
+gboolean dbus_get_agc_state(void);
 
 /**
- * ConfigurationManager - Set the current echo cancel state
- * @param gchar* The state (enabled/disabled)
+ * ConfigurationManager - Set the current noise suppressor state
+ * @param gboolean The state (enabled/disabled)
  */
-void dbus_set_echo_cancel_state(const gchar *state);
-
-int dbus_get_echo_cancel_tail_length(void);
-
-void dbus_set_echo_cancel_tail_length(int length);
-
-int dbus_get_echo_cancel_delay(void);
-
-void dbus_set_echo_cancel_delay(int delay);
+void dbus_set_agc_state(gboolean state);
 
 /**
  * ConfigurationManager - Tells the GUI if IAX2 support is enabled
@@ -439,7 +445,7 @@ gchar *dbus_get_conference_id(const gchar *callID);
 /**
  * Toggle recording for this instance, may be call or conference
  */
-void dbus_set_record(const gchar *id);
+gboolean dbus_toggle_recording(const gchar *id);
 
 /**
  * Set the path where the recorded audio files will be stored
@@ -600,9 +606,9 @@ gboolean dbus_start_recorded_file_playback(const gchar *);
  */
 void dbus_stop_recorded_file_playback(const gchar *);
 
-void dbus_start_video_preview();
-void dbus_stop_video_preview();
-gboolean dbus_has_video_preview_started();
+void dbus_start_video_camera();
+void dbus_stop_video_camera();
+gboolean dbus_has_video_camera_started();
 
 /**
  * Prevent Gnome Session Manager from entering in screen-saver mode
@@ -614,4 +620,10 @@ void dbus_screensaver_inhibit(void);
  */
 void dbus_screensaver_uninhibit(void);
 
+
+/**
+ * Presence methods
+ */
+void dbus_presence_publish(const gchar *accountID, gboolean status);
+void dbus_presence_subscribe(const gchar *accountID, const gchar *uri, gboolean flag);
 #endif
