@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Pierre-Luc Beaudoin <pierre-luc.beaudoin@savoirfairelinux.com>
  *  Author: Pierre-Luc Bacon <pierre-luc.bacon@savoirfairelinux.com>
@@ -59,6 +59,7 @@
 
 #include <gdk/gdkkeysyms.h>
 
+
 /** Local variables */
 static GtkAccelGroup *accelGroup;
 static GtkWidget *subvbox;
@@ -217,6 +218,15 @@ main_window_bring_to_front(SFLPhoneClient *client, guint32 timestamp)
     gtk_window_present_with_time(GTK_WINDOW(client->win), timestamp);
 }
 
+
+GtkWidget*
+create_status_bar()
+{
+    GtkWidget *bar = gtk_statusbar_new();
+    return bar;
+}
+
+
 void
 create_main_window(SFLPhoneClient *client)
 {
@@ -282,11 +292,7 @@ create_main_window(SFLPhoneClient *client)
     pack_main_window_start(GTK_BOX(vbox), client->toolbar, FALSE, TRUE, 0);
 
     /* Setup call main widget*/
-#if GTK_MAJOR_VERSION == 2
-    GtkWidget *vpaned = gtk_vpaned_new();
-#else
     GtkWidget *vpaned = gtk_paned_new(GTK_ORIENTATION_VERTICAL);
-#endif
     current_calls_tab->mainwidget = vpaned;
 
     const gint messaging_height = g_settings_get_int(client->settings, "message-tab-height");
@@ -320,6 +326,12 @@ create_main_window(SFLPhoneClient *client)
     g_object_ref(speaker_control);
     g_object_ref(mic_control);
 
+    if (g_settings_get_boolean(client->settings, "show-dialpad")) {
+        dialpad = create_dialpad(client);
+        gtk_box_pack_end(GTK_BOX(subvbox), dialpad, FALSE, TRUE, 0);
+        gtk_widget_show_all(dialpad);
+    }
+
     if (must_show_volume(client)) {
         gtk_box_pack_end(GTK_BOX(subvbox), speaker_control, FALSE, TRUE, 0);
         gtk_box_pack_end(GTK_BOX(subvbox), mic_control, FALSE, TRUE, 0);
@@ -330,14 +342,9 @@ create_main_window(SFLPhoneClient *client)
         gtk_widget_hide(mic_control);
     }
 
-    if (g_settings_get_boolean(client->settings, "show-dialpad")) {
-        dialpad = create_dialpad(client);
-        gtk_box_pack_end(GTK_BOX(subvbox), dialpad, FALSE, TRUE, 0);
-        gtk_widget_show_all(dialpad);
-    }
 
     /* Status bar */
-    statusBar = gtk_statusbar_new();
+    statusBar = create_status_bar();
     pack_main_window_start(GTK_BOX(vbox), statusBar, FALSE, TRUE, 0);
 
     /* Add to main window */
@@ -546,10 +553,14 @@ main_window_confirm_go_clear(callable_obj_t * c, SFLPhoneClient *client)
     add_error_dialog(GTK_WIDGET(mini_dialog));
 }
 
-void
-main_window_update_playback_scale(guint current, guint size)
+gboolean
+main_window_update_playback_scale(const gchar *file_path, guint current, guint size)
 {
-     sfl_seekslider_update_scale(SFL_SEEKSLIDER(seekslider), current, size);
+    if (sfl_seekslider_has_path(SFL_SEEKSLIDER(seekslider), file_path)) {
+        sfl_seekslider_update_scale(SFL_SEEKSLIDER(seekslider), current, size);
+        return TRUE;
+    }
+    return FALSE;
 }
 
 void
@@ -589,3 +600,9 @@ main_window_pause_keygrabber(gboolean value)
 {
     pause_grabber = value;
 }
+
+void
+main_window_update_seekslider(const gchar *recordfile)
+{
+    g_object_set(G_OBJECT(seekslider), "file-path", recordfile, NULL);
+}
