@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2012-2013 Savoir-Faire Linux Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,7 +36,6 @@
 #include <string.h>
 #include "seekslider.h"
 #include "dbus.h"
-#include "calltab.h"
 
 /**
  * SECTION:sfl-seekslider
@@ -89,13 +88,18 @@ struct SFLSeekSliderPrivate
     gboolean is_dragging;
     gboolean can_update_scale;
     gboolean is_playing;
+    gchar *file_path;
 };
 
 enum
 {
     PROP_0,
+    PROP_FILE_PATH,
+    N_PROPERTIES
 };
 
+static GParamSpec *obj_properties[N_PROPERTIES] = { NULL, };
+
 G_DEFINE_TYPE(SFLSeekSlider, sfl_seekslider, GTK_TYPE_HBOX)
 
 static void
@@ -108,6 +112,10 @@ sfl_seekslider_class_init(SFLSeekSliderClass *klass)
     object_class->set_property = sfl_seekslider_set_property;
     object_class->get_property = sfl_seekslider_get_property;
 
+    obj_properties[PROP_FILE_PATH] = g_param_spec_string("file-path", "File path",
+            "Set file path for playback", "" /* default value */, G_PARAM_READWRITE);
+
+    g_object_class_install_properties(object_class, N_PROPERTIES, obj_properties);
     g_type_class_add_private(klass, sizeof(SFLSeekSliderPrivate));
 }
 
@@ -186,6 +194,7 @@ sfl_seekslider_init(SFLSeekSlider *seekslider)
     seekslider->priv->current = 0;
     seekslider->priv->size = 0;
     seekslider->priv->is_playing = FALSE;
+    seekslider->priv->file_path = NULL;
 }
 
 static void
@@ -199,6 +208,9 @@ sfl_seekslider_finalize(GObject *object)
     seekslider = SFL_SEEKSLIDER(object);
     g_return_if_fail(seekslider->priv != NULL);
 
+    /* Ensure that we've stopped playback */
+    sfl_seekslider_stop_playback_record_cb(NULL, seekslider);
+
     G_OBJECT_CLASS(sfl_seekslider_parent_class)->finalize(object);
 }
 
@@ -206,15 +218,54 @@ sfl_seekslider_finalize(GObject *object)
 static void
 sfl_seekslider_set_property (GObject *object, guint prop_id, const GValue *value G_GNUC_UNUSED, GParamSpec *pspec)
 {
-    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+    SFLSeekSlider *self = SFL_SEEKSLIDER(object);
+
+    switch (prop_id)
+    {
+        case PROP_FILE_PATH:
+            /* no change */
+            if (g_strcmp0(self->priv->file_path, g_value_get_string(value)) == 0)
+                break;
+
+            /* cache is_playing as it will be modified */
+            const gboolean resume_playing = self->priv->is_playing;
+            if (resume_playing)
+                sfl_seekslider_stop_playback_record_cb(NULL, self);
+
+            g_free(self->priv->file_path);
+            self->priv->file_path = g_value_dup_string(value);
+            g_debug("filepath: %s\n", self->priv->file_path);
+
+            if (resume_playing)
+                sfl_seekslider_play_playback_record_cb(NULL, self);
+            break;
+
+        default:
+            /* We don't have any other property... */
+            G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+            break;
+    }
 }
 
 static void
 sfl_seekslider_get_property (GObject *object, guint prop_id, GValue *value G_GNUC_UNUSED, GParamSpec *pspec)
 {
-    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+    SFLSeekSlider *self = SFL_SEEKSLIDER(object);
+
+    switch (prop_id)
+    {
+        case PROP_FILE_PATH:
+            g_value_set_string(value, self->priv->file_path);
+            break;
+
+        default:
+            /* We don't have any other property... */
+            G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+            break;
+    }
 }
 
+
 /**
  * sfl_seekslider_new:
  * @shell_player: the #RBShellPlayer instance
@@ -298,37 +349,32 @@ on_playback_scale_scrolled_cb(GtkWidget *widget G_GNUC_UNUSED, GdkEvent *event G
 
 static void sfl_seekslider_play_playback_record_cb (GtkButton *button G_GNUC_UNUSED, gpointer user_data)
 {
-    SFLSeekSlider *seekslider = SFL_SEEKSLIDER(user_data);
+    SFLSeekSlider *self = SFL_SEEKSLIDER(user_data);
 
-    callable_obj_t *selectedCall = calltab_get_selected_call(history_tab);
-    if (selectedCall == NULL)
+    if (self->priv->file_path == NULL || (*self->priv->file_path == 0))
         return;
 
-    g_debug("Start selected call file playback %s", selectedCall->_recordfile);
-    seekslider->priv->is_playing = selectedCall->_record_is_playing =
-        dbus_start_recorded_file_playback(selectedCall->_recordfile);
+    g_debug("Start file playback %s", self->priv->file_path);
+    self->priv->is_playing = dbus_start_recorded_file_playback(self->priv->file_path);
 
-    if (seekslider->priv->is_playing)
-        sfl_seekslider_set_display(seekslider, SFL_SEEKSLIDER_DISPLAY_PAUSE);
+    if (self->priv->is_playing)
+        sfl_seekslider_set_display(self, SFL_SEEKSLIDER_DISPLAY_PAUSE);
 }
 
-static void sfl_seekslider_stop_playback_record_cb (GtkButton *button G_GNUC_UNUSED, gpointer user_data)
+static void sfl_seekslider_stop_playback_record_cb(GtkButton *button G_GNUC_UNUSED, gpointer user_data)
 {
-    SFLSeekSlider *seekslider = SFL_SEEKSLIDER(user_data);
+    SFLSeekSlider *self = SFL_SEEKSLIDER(user_data);
 
-    callable_obj_t *selectedCall = calltab_get_selected_call(history_tab);
-    if (selectedCall == NULL)
+    if (self->priv->file_path == NULL || (*self->priv->file_path == 0))
         return;
 
-    if (selectedCall->_recordfile == NULL ||
-        strlen(selectedCall->_recordfile) == 0)
-        return;
+    if (self->priv->is_playing) {
+        dbus_stop_recorded_file_playback(self->priv->file_path);
+        g_debug("Stop file playback %s", self->priv->file_path);
+        self->priv->is_playing = FALSE;
+    }
 
-    dbus_stop_recorded_file_playback(selectedCall->_recordfile);
-    g_debug("Stop selected call file playback %s", selectedCall->_recordfile);
-    seekslider->priv->is_playing = selectedCall->_record_is_playing = FALSE;
-
-    sfl_seekslider_set_display(seekslider, SFL_SEEKSLIDER_DISPLAY_PLAY);
+    sfl_seekslider_set_display(self, SFL_SEEKSLIDER_DISPLAY_PLAY);
 }
 
 void sfl_seekslider_update_timelabel(SFLSeekSlider *seekslider, guint current, guint size)
@@ -382,7 +428,9 @@ void sfl_seekslider_update_scale(SFLSeekSlider *seekslider, guint current, guint
 
 void sfl_seekslider_set_display(SFLSeekSlider *seekslider, SFLSeekSliderDisplay display) {
 
-    if (seekslider == NULL)
+    if (seekslider == NULL || !seekslider->priv ||
+        !GTK_IS_WIDGET(seekslider->priv->playRecordWidget) ||
+        !GTK_IS_WIDGET(seekslider->priv->stopRecordWidget))
         return;
 
     switch (display) {
@@ -408,10 +456,17 @@ void sfl_seekslider_reset(SFLSeekSlider *seekslider)
     seekslider->priv->can_update_scale = FALSE;
     gtk_range_set_value(GTK_RANGE(seekslider->priv->hscale), 0.0);
     sfl_seekslider_set_display(seekslider, SFL_SEEKSLIDER_DISPLAY_PLAY);
-    sfl_seekslider_stop_playback_record_cb(NULL, seekslider);
+    if (seekslider->priv->is_playing)
+        sfl_seekslider_stop_playback_record_cb(NULL, seekslider);
     gtk_label_set_text(GTK_LABEL(seekslider->priv->timeLabel), "");
     seekslider->priv->current = 0;
     seekslider->priv->size = 0;
     seekslider->priv->is_playing = FALSE;
     seekslider->priv->can_update_scale = TRUE;
 }
+
+gboolean
+sfl_seekslider_has_path(SFLSeekSlider *seekslider, const gchar *file_path)
+{
+    return g_strcmp0(seekslider->priv->file_path, file_path) == 0;
+}
