@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Pierre-Luc Beaudoin <pierre-luc.beaudoin@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -65,11 +65,6 @@ void set_slider_value(const gchar *device, gdouble value);
  */
 void set_slider_no_update (const gchar * device, gdouble value);
 
-/**
- * Mute the audio device setting the sliders to 0
- */
-void toggle_slider_mute_microphone(void);
-
 /**
  * Returns the mute/unmute state
  */
