@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2012 Savoir-Faire Linux Inc.
+ *  Copyright (C) 2004-2013 Savoir-Faire Linux Inc.
  *  Author: Pierre-Luc Bacon <pierre-luc.bacon@savoirfairelinux.com
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -45,7 +45,6 @@
 
 #include "uimanager.h"
 #include "statusicon.h"
-
 #include "config/audioconf.h"
 #include "uimanager.h"
 #include "statusicon.h"
@@ -64,6 +63,11 @@
 
 #include "sliders.h"
 
+#ifdef SFL_PRESENCE
+#include "presencewindow.h"
+#include "presence.h"
+#endif
+
 typedef struct
 {
     callable_obj_t *call;
@@ -83,7 +87,6 @@ static GtkAction * hangUpAction_;
 static GtkAction * copyAction_;
 static GtkAction * pasteAction_;
 static GtkAction * recordAction_;
-static GtkAction * muteAction_;
 static GtkAction * imAction_;
 
 static GtkWidget * pickUpWidget_;
@@ -94,7 +97,6 @@ static GtkWidget * holdToolbar_;
 static GtkWidget * offHoldToolbar_;
 static GtkWidget * transferToolbar_;
 static GtkWidget * recordWidget_;
-static GtkWidget * muteWidget_;
 static GtkWidget * voicemailToolbar_;
 static GtkWidget * imToolbar_;
 
@@ -127,14 +129,6 @@ static void add_to_toolbar(GtkWidget *toolbar, GtkWidget *item, gint pos)
     gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(item), pos);
 }
 
-static void
-call_mute(void)
-{
-    g_debug("Mute call button pressed");
-    sflphone_mute_call();
-}
-
-
 static void
 update_toolbar_for_call(callable_obj_t *selectedCall, gboolean instant_messaging_enabled, SFLPhoneClient *client)
 {
@@ -161,14 +155,12 @@ update_toolbar_for_call(callable_obj_t *selectedCall, gboolean instant_messaging
                 // Make the button toolbar clickable
                 gtk_action_set_sensitive(pickUpAction_, TRUE);
                 gtk_action_set_sensitive(hangUpAction_, TRUE);
-                gtk_action_set_sensitive(muteAction_, TRUE);
                 // Replace the dial button with the hangup button
                 g_object_ref(newCallWidget_);
                 remove_from_toolbar(toolbar, newCallWidget_);
                 pos = 0;
                 add_to_toolbar(toolbar, pickUpWidget_, pos++);
                 add_to_toolbar(toolbar, hangUpWidget_, pos++);
-                add_to_toolbar(toolbar, muteWidget_, pos++);
                 break;
         }
         case CALL_STATE_HOLD:
@@ -198,7 +190,6 @@ update_toolbar_for_call(callable_obj_t *selectedCall, gboolean instant_messaging
                 gtk_action_set_sensitive(hangUpAction_, TRUE);
                 pos = 1;
                 add_to_toolbar(toolbar, hangUpWidget_, pos++);
-                add_to_toolbar(toolbar, muteWidget_, pos++);
                 break;
         }
         case CALL_STATE_DIALING:
@@ -236,11 +227,9 @@ update_toolbar_for_call(callable_obj_t *selectedCall, gboolean instant_messaging
 
                 gtk_action_set_sensitive(hangUpAction_, TRUE);
                 gtk_action_set_sensitive(recordAction_, TRUE);
-                gtk_action_set_sensitive(muteAction_, TRUE);
                 gtk_widget_set_sensitive(holdMenu_, TRUE);
                 gtk_widget_set_sensitive(holdToolbar_, TRUE);
                 gtk_widget_set_sensitive(transferToolbar_, TRUE);
-                gtk_widget_set_sensitive(muteWidget_, TRUE);
                 if (instant_messaging_enabled)
                     gtk_action_set_sensitive(imAction_, TRUE);
 
@@ -249,7 +238,6 @@ update_toolbar_for_call(callable_obj_t *selectedCall, gboolean instant_messaging
                 add_to_toolbar(toolbar, holdToolbar_, pos++);
                 add_to_toolbar(toolbar, transferToolbar_, pos++);
                 add_to_toolbar(toolbar, recordWidget_, pos++);
-                add_to_toolbar(toolbar, muteWidget_, pos++);
                 if (instant_messaging_enabled)
                     add_to_toolbar(toolbar, imToolbar_, pos++);
 
@@ -274,19 +262,16 @@ update_toolbar_for_call(callable_obj_t *selectedCall, gboolean instant_messaging
         {
                 pos = 1;
                 gtk_action_set_sensitive(hangUpAction_, TRUE);
-                gtk_action_set_sensitive(muteAction_, TRUE);
                 gtk_widget_set_sensitive(holdMenu_, TRUE);
                 gtk_widget_set_sensitive(holdToolbar_, TRUE);
                 gtk_widget_set_sensitive(transferToolbar_, TRUE);
                 gtk_widget_set_sensitive(transferToolbar_, TRUE);
-                gtk_widget_set_sensitive(muteWidget_, TRUE);
 
                 add_to_toolbar(toolbar, hangUpWidget_, pos++);
                 add_to_toolbar(toolbar, transferToolbar_, pos++);
                 g_signal_handler_block(transferToolbar_, transferButtonConnId_);
                 gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(transferToolbar_), TRUE);
                 g_signal_handler_unblock(transferToolbar_, transferButtonConnId_);
-                add_to_toolbar(toolbar, muteWidget_, pos++);
                 break;
         }
         default:
@@ -389,7 +374,6 @@ update_actions(SFLPhoneClient *client)
     gtk_action_set_sensitive(pickUpAction_, FALSE);
     gtk_action_set_sensitive(hangUpAction_, FALSE);
     gtk_action_set_sensitive(recordAction_, FALSE);
-    gtk_action_set_sensitive(muteAction_, FALSE);
     gtk_action_set_sensitive(copyAction_, FALSE);
     gtk_action_set_sensitive(imAction_, FALSE);
 
@@ -397,13 +381,11 @@ update_actions(SFLPhoneClient *client)
     gtk_widget_set_sensitive(holdToolbar_, FALSE);
     gtk_widget_set_sensitive(offHoldToolbar_, FALSE);
     gtk_widget_set_sensitive(recordWidget_, FALSE);
-    gtk_widget_set_sensitive(muteWidget_, FALSE);
     gtk_widget_set_sensitive(historyButton_, FALSE);
 
     // Increment the reference counter
     g_object_ref(hangUpWidget_);
     g_object_ref(recordWidget_);
-    g_object_ref(muteWidget_);
     g_object_ref(holdToolbar_);
     g_object_ref(offHoldToolbar_);
     g_object_ref(historyButton_);
@@ -420,7 +402,6 @@ update_actions(SFLPhoneClient *client)
     GtkWidget *toolbar = client->toolbar;
     remove_from_toolbar(toolbar, hangUpWidget_);
     remove_from_toolbar(toolbar, recordWidget_);
-    remove_from_toolbar(toolbar, muteWidget_);
     remove_from_toolbar(toolbar, transferToolbar_);
     remove_from_toolbar(toolbar, historyButton_);
     remove_from_toolbar(toolbar, voicemailToolbar_);
@@ -457,11 +438,16 @@ update_actions(SFLPhoneClient *client)
         }
     }
 
-    callable_obj_t * selectedCall = calltab_get_selected_call(active_calltree_tab);
-    conference_obj_t * selectedConf = calltab_get_selected_conf(active_calltree_tab);
+    callable_obj_t * selectedCall = NULL;
+    conference_obj_t * selectedConf = NULL;
 
     const gboolean instant_messaging_enabled = g_settings_get_boolean(client->settings, "instant-messaging-enabled");
 
+    if (!calllist_empty(active_calltree_tab)) {
+        selectedCall = calltab_get_selected_call(active_calltree_tab);
+        selectedConf = calltab_get_selected_conf(active_calltree_tab);
+    }
+
     if (selectedCall) {
         update_toolbar_for_call(selectedCall, instant_messaging_enabled, client);
     } else if (selectedConf) {
@@ -519,6 +505,18 @@ dialpad_bar_cb(GtkToggleAction *togglemenuitem, SFLPhoneClient *client)
         g_settings_set_boolean(client->settings, "show-dialpad", toggled);
 }
 
+#ifdef SFL_PRESENCE
+static void
+toggle_presence_window_cb(GtkToggleAction *togglemenuitem, SFLPhoneClient *client)
+{
+    const gboolean toggled = gtk_toggle_action_get_active(togglemenuitem);
+    if (toggled)
+        create_presence_window(client, togglemenuitem);
+    else
+        destroy_presence_window();
+}
+#endif
+
 static void
 help_contents_cb(G_GNUC_UNUSED GtkAction *action, G_GNUC_UNUSED gpointer data)
 {
@@ -550,6 +548,7 @@ help_about(G_GNUC_UNUSED GtkAction *action, SFLPhoneClient *client)
         "Rafaël Carré <rafael.carre@savoirfairelinux.com>",
         "Vivien Didelot <vivien.didelot@savoirfairelinux.com>",
         "Emmanuel Lepage <emmanuel.lepage@savoirfairelinux.com>",
+        "Partick Keroulas <patrick.keroulas@savoirfairelinux.com>",
         NULL
     };
     static const gchar *artists[] = {
@@ -561,7 +560,7 @@ help_about(G_GNUC_UNUSED GtkAction *action, SFLPhoneClient *client)
             "artists", artists,
             "authors", authors,
             "comments", _("SFLphone is a VoIP client compatible with SIP and IAX2 protocols."),
-            "copyright", "Copyright © 2004-2012 Savoir-faire Linux Inc.",
+            "copyright", "Copyright © 2004-2014 Savoir-faire Linux Inc.",
             "name", PACKAGE_NAME,
             "title", _("About SFLphone"),
             "version", PACKAGE_VERSION,
@@ -748,7 +747,10 @@ static void
 call_record(G_GNUC_UNUSED GtkAction *action, SFLPhoneClient *client)
 {
     g_debug("Record button pressed");
-    sflphone_rec_call(client);
+    /* Ensure that button is set to correct state, but suppress signal */
+    g_signal_handler_block(recordWidget_, recordButtonConnId_);
+    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(recordWidget_), sflphone_rec_call(client));
+    g_signal_handler_unblock(recordWidget_, recordButtonConnId_);
 }
 
 static void
@@ -757,20 +759,21 @@ call_configuration_assistant(G_GNUC_UNUSED GtkAction *action, G_GNUC_UNUSED gpoi
     build_wizard();
 }
 
-static void
-remove_from_history(G_GNUC_UNUSED GtkAction *action, SFLPhoneClient *client)
+typedef struct
 {
-    callable_obj_t* call = calltab_get_selected_call(history_tab);
+    callable_obj_t *call;
+    SFLPhoneClient *client;
+} EditNumberData;
 
-    g_debug("Remove the call from the history");
-
-    if (call == NULL) {
+static void
+remove_from_history(G_GNUC_UNUSED GtkAction *action, EditNumberData *data)
+{
+    if (data->call == NULL) {
         g_warning("Call is NULL");
         return;
     }
 
-    calllist_remove_from_history(call, client);
-    update_actions(client);
+    calllist_remove_from_history(data->call, data->client);
 }
 
 static void
@@ -1057,81 +1060,25 @@ static const GtkActionEntry menu_entries[] = {
       N_("About this application"), G_CALLBACK(help_about) }
 };
 
-static void register_custom_stock_icon(void) {
-
-    static gboolean registered = FALSE;
-
-    if (!registered) {
-        GdkPixbuf *pixbuf;
-        GtkIconFactory *factory;
-
-        static GtkStockItem items[] = {
-            { "SFLPHONE_MUTE_CALL",
-              "_GTK!",
-              0, 0, NULL }
-        };
-
-        registered = TRUE;
-
-        /* Register our stock items */
-        gtk_stock_add (items, G_N_ELEMENTS (items));
-
-        /* Add our custom icon factory to the list of defaults */
-        factory = gtk_icon_factory_new ();
-        gtk_icon_factory_add_default (factory);
-
-        /* demo_find_file() looks in the current directory first,
-         * so you can run gtk-demo without installing GTK, then looks
-         * in the location where the file is installed.
-         */
-        pixbuf = NULL;
-        pixbuf = gdk_pixbuf_new_from_file (ICONS_DIR "/mic.svg", NULL);
-        if (pixbuf == NULL) {
-            g_debug("Error could not create mic.svg pixbuf");
-        }
-
-        /* Register icon to accompany stock item */
-        if (pixbuf != NULL) {
-            GtkIconSet *icon_set;
-            GdkPixbuf *transparent;
-
-            /* The gtk-logo-rgb icon has a white background, make it transparent */
-            transparent = gdk_pixbuf_add_alpha (pixbuf, TRUE, 0xff, 0xff, 0xff);
-
-            icon_set = gtk_icon_set_new_from_pixbuf (transparent);
-            gtk_icon_factory_add (factory, "SFLPHONE_MUTE_CALL", icon_set);
-            gtk_icon_set_unref (icon_set);
-            g_object_unref (pixbuf);
-            g_object_unref (transparent);
-        }
-        else
-            g_warning ("failed to load GTK logo for toolbar");
-
-        /* Drop our reference to the factory, GTK will hold a reference. */
-        g_object_unref (factory);
-    }
-}
-
 static const GtkToggleActionEntry toggle_menu_entries[] = {
     { "Transfer", GTK_STOCK_TRANSFER, N_("_Transfer"), "<control>T", N_("Transfer the call"), NULL, TRUE },
     { "Record", GTK_STOCK_MEDIA_RECORD, N_("_Record"), "<control>R", N_("Record the current conversation"), NULL, TRUE },
-    { "Mute", "SFLPHONE_MUTE_CALL", N_("_Mute"), "<control>M", N_("Mute microphone for this call"), G_CALLBACK(call_mute), FALSE },
     { "Toolbar", NULL, N_("_Show toolbar"), "<control>T", N_("Show the toolbar"), NULL, TRUE },
     { "Dialpad", NULL, N_("_Dialpad"), "<control>D", N_("Show the dialpad"), G_CALLBACK(dialpad_bar_cb), TRUE },
     { "VolumeControls", NULL, N_("_Volume controls"), "<control>V", N_("Show the volume controls"), G_CALLBACK(volume_bar_cb), TRUE },
     { "History", "appointment-soon", N_("_History"), NULL, N_("Calls history"), G_CALLBACK(toggle_history_cb), FALSE },
     { "Addressbook", GTK_STOCK_ADDRESSBOOK, N_("_Address book"), NULL, N_("Address book"), G_CALLBACK(toggle_addressbook_cb), FALSE },
+#ifdef SFL_PRESENCE
+    { "Buddies", NULL, N_("_Buddy list"), NULL, N_("Display the buddy list"), G_CALLBACK(toggle_presence_window_cb), FALSE},
+#endif
 };
 
 GtkUIManager *uimanager_new(SFLPhoneClient *client)
 {
-    const gint nb_entries = addrbook ? 8 : 7;
+    const gint nb_entries = sizeof(toggle_menu_entries) / sizeof(toggle_menu_entries[0]);
 
     GtkUIManager *ui = gtk_ui_manager_new();
 
-    /* Register new icons as GTK_STOCK_ITEMS */
-    register_custom_stock_icon();
-
     /* Create an accel group for window's shortcuts */
     gchar *path = g_build_filename(SFLPHONE_UIDIR_UNINSTALLED, "./ui.xml", NULL);
     guint manager_id;
@@ -1184,12 +1131,6 @@ fail:
     return NULL;
 }
 
-typedef struct
-{
-    callable_obj_t *call;
-    SFLPhoneClient *client;
-} EditNumberData;
-
 static void
 edit_number_cb(G_GNUC_UNUSED GtkWidget *widget, EditNumberData *data)
 {
@@ -1197,6 +1138,28 @@ edit_number_cb(G_GNUC_UNUSED GtkWidget *widget, EditNumberData *data)
     g_free(data);
 }
 
+#ifdef SFL_PRESENCE
+void
+add_presence_subscription_cb(G_GNUC_UNUSED GtkWidget * widget, G_GNUC_UNUSED calltab_t * tab)
+{
+    callable_obj_t * c = calltab_get_selected_call(tab);
+    buddy_t * b = presence_buddy_create();
+
+    presence_callable_to_buddy(c, b);
+
+    g_debug("Presence : trying to create a new subscription (%s,%s)", b->uri, b->acc);
+
+    // popup
+    if(show_buddy_info_dialog(_("Add new buddy"), b))
+    {
+        presence_buddy_list_add_buddy(b);
+        update_presence_view();
+    }
+    else
+        presence_buddy_delete(b);
+}
+#endif
+
 void
 add_registered_accounts_to_menu(GtkWidget *menu)
 {
@@ -1249,7 +1212,7 @@ void
 show_popup_menu(GtkWidget *my_widget, GdkEventButton *event, SFLPhoneClient *client)
 {
     // TODO update the selection to make sure the call under the mouse is the call selected
-    gboolean pickup = FALSE, hangup = FALSE, hold = FALSE, copy = FALSE, record = FALSE, im = FALSE, mute = FALSE;
+    gboolean pickup = FALSE, hangup = FALSE, hold = FALSE, copy = FALSE, record = FALSE, im = FALSE;
     gboolean accounts = FALSE;
 
     // conference type boolean
@@ -1287,7 +1250,6 @@ show_popup_menu(GtkWidget *my_widget, GdkEventButton *event, SFLPhoneClient *cli
                     hold = TRUE;
                     record = TRUE;
                     im = TRUE;
-                    mute = TRUE;
                     break;
                 case CALL_STATE_BUSY:
                 case CALL_STATE_FAILURE:
@@ -1396,18 +1358,6 @@ show_popup_menu(GtkWidget *my_widget, GdkEventButton *event, SFLPhoneClient *cli
             gtk_widget_show(menu_items);
         }
 
-        if (mute) {
-            GtkWidget *menu_items = gtk_image_menu_item_new_with_mnemonic(_("_Mute"));
-            GtkWidget *image = gtk_image_new_from_stock(GTK_STOCK_MEDIA_RECORD,
-                               GTK_ICON_SIZE_MENU);
-            gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_items), image);
-            gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_items);
-            g_signal_connect(G_OBJECT(menu_items), "activate",
-                             G_CALLBACK(call_mute),
-                             NULL);
-            gtk_widget_show(menu_items);
-        }
-
         if (im) {
             // do not display message if instant messaging is disabled
             const gboolean instant_messaging_enabled = g_settings_get_boolean(client->settings, "instant-messaging-enabled");
@@ -1423,11 +1373,10 @@ show_popup_menu(GtkWidget *my_widget, GdkEventButton *event, SFLPhoneClient *cli
                 gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_items);
                 g_signal_connect(G_OBJECT(menu_items), "activate",
                                  G_CALLBACK(call_im),
-                                 client);
+                                 selectedCall);
                 gtk_widget_show(menu_items);
             }
         }
-
     } else {
         g_debug("Build conf menus");
 
@@ -1486,6 +1435,17 @@ show_popup_menu_history(GtkWidget *my_widget, GdkEventButton *event, SFLPhoneCli
         gtk_widget_show(menu_items);
     }
 
+#ifdef SFL_PRESENCE
+    if (selectedCall) {
+        GtkWidget *menu_items = gtk_image_menu_item_new_with_mnemonic(_("Follow status"));
+        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_items);
+        if(!presence_buddy_list_get())
+            gtk_widget_set_sensitive(menu_items, FALSE);
+        g_signal_connect(G_OBJECT(menu_items), "activate", G_CALLBACK(add_presence_subscription_cb), history_tab);
+        gtk_widget_show(menu_items);
+    }
+#endif
+
     GtkWidget *separator = gtk_separator_menu_item_new();
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), separator);
     gtk_widget_show(separator);
@@ -1535,6 +1495,17 @@ show_popup_menu_contacts(GtkWidget *my_widget, GdkEventButton *event, SFLPhoneCl
         g_signal_connect(new_call, "activate", G_CALLBACK(call_back), client);
         gtk_widget_show(new_call);
 
+#ifdef SFL_PRESENCE
+        GtkWidget *presence = gtk_image_menu_item_new_with_mnemonic(_("Follow status"));
+        gtk_menu_shell_append(GTK_MENU_SHELL(menu), presence);
+
+        if(!presence_buddy_list_get())
+            gtk_widget_set_sensitive(presence, FALSE);
+
+        g_signal_connect(G_OBJECT(presence), "activate", G_CALLBACK(add_presence_subscription_cb), contacts_tab);
+        gtk_widget_show(presence);
+#endif
+
         GtkWidget *edit = gtk_image_menu_item_new_from_stock(GTK_STOCK_EDIT,
                           get_accel_group());
         gtk_menu_shell_append(GTK_MENU_SHELL(menu), edit);
@@ -1667,7 +1638,6 @@ create_menus(GtkUIManager *ui, SFLPhoneClient *client)
     hangUpAction_ = get_action(ui, "/MenuBar/CallMenu/HangUp");
     holdMenu_ = get_widget(ui, "/MenuBar/CallMenu/OnHoldMenu");
     recordAction_ = get_action(ui, "/MenuBar/CallMenu/Record");
-    muteAction_ = get_action(ui, "/MenuBar/CallMenu/Mute");
     imAction_ = get_action(ui, "/MenuBar/CallMenu/InstantMessaging");
     copyAction_ = get_action(ui, "/MenuBar/EditMenu/Copy");
     pasteAction_ = get_action(ui, "/MenuBar/EditMenu/Paste");
@@ -1675,9 +1645,13 @@ create_menus(GtkUIManager *ui, SFLPhoneClient *client)
     // Set the toggle buttons
     gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(get_action(ui, "/MenuBar/ViewMenu/Dialpad")), g_settings_get_boolean(client->settings, "show-dialpad"));
     gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(volumeToggle_), must_show_volume(client));
-    gtk_action_set_sensitive(volumeToggle_, must_show_alsa_conf());
+    gtk_action_set_sensitive(volumeToggle_, TRUE);
     gtk_action_set_sensitive(get_action(ui, "/MenuBar/ViewMenu/Toolbar"), FALSE);
 
+#ifdef SFL_PRESENCE
+    gtk_action_set_sensitive(get_action(ui, "/MenuBar/ViewMenu/Buddies"), TRUE);
+#endif
+
     /* Add the loading icon at the right of the toolbar. It is used for addressbook searches. */
     waitingLayer = create_waiting_icon();
     gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), waitingLayer);
@@ -1697,7 +1671,6 @@ create_toolbar_actions(GtkUIManager *ui, SFLPhoneClient *client)
     pickUpWidget_ = get_widget(ui, "/ToolbarActions/PickUpToolbar");
     hangUpWidget_ = get_widget(ui, "/ToolbarActions/HangUpToolbar");
     recordWidget_ = get_widget(ui, "/ToolbarActions/RecordToolbar");
-    muteWidget_ = get_widget(ui, "/ToolbarActions/MuteToolbar");
     imToolbar_ = get_widget(ui, "/ToolbarActions/InstantMessagingToolbar");
     historyButton_ = get_widget(ui, "/ToolbarActions/HistoryToolbar");
     if (addrbook)
