@@ -60,21 +60,12 @@ typedef struct _Hit {
 /**
  * Book structure for "outside world"
  */
-#if EDS_CHECK_VERSION(3,5,3)
-typedef struct {
-    gchar *uid;
-    gchar *name;
-    gboolean active;
-} book_data_t;
-#else
 typedef struct {
     gchar *uid;
     gchar *uri;
     gchar *name;
     gboolean active;
-    gboolean isdefault;
 } book_data_t;
-#endif
 
 typedef struct _AddressBook_Config {
     // gint64: a signed integer guaranteed to be 64 bits on all platforms
@@ -90,7 +81,7 @@ typedef struct _AddressBook_Config {
 /**
  * Initialize addressbook
  */
-void addressbook_init(gchar **book_list);
+void addressbook_init();
 
 /**
  * Return addressbook state
@@ -116,11 +107,11 @@ void addressbook_search(void (*search_cb)(GList *, gpointer), GtkEntry *, Addres
 /**
  * Get a list of addressbook book
  */
-GSList *addressbook_get_books_data(gchar **book_list);
+GSList *addressbook_get_books_data();
 
 book_data_t *addressbook_get_book_data_by_uid(gchar *);
 
-void addressbook_set_current_book(gchar *);
+void addressbook_set_current_book(const gchar *);
 
 void addressbook_set_search_type(AddrbookSearchType);
 
